
/* ---------------------------------------------------------------------------*
 *  File: Debug.java                                                  *
 *                                                                            *
 *  Copyright Information:                                                    *
 *    Licensed Materials - Property of IBM                                    *
 *    5648-A06                                                                *
 *    (c) Copyright IBM Corporation, 1996-2000. All rights reserved.          *
 *    US Government Users Restricted Rights - Use, duplication                *
 *    or disclosure restricted by GSA ADP Schedule Contract                   *
 *    with IBM                                                                *
 *                                                                            *
 * -------------------------------------------------------------------------- */


import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.util.StringTokenizer;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.ifs.servlets.IfsHttpServlet;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
/**
 * Debug servlet.
 * @author      Sterling Smith*/

public class Debug extends IfsHttpServlet {
	public static boolean  debugOn = false;

	public void doGet (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {

		String[] servlets = null, goldObjects = null, objects = null, selectedObjects = null;
		int i = 0, j;
		String debugReq = null, debugMedia = null, debugLogName = null;

		//PrintWriter pw = res.getWriter();

		setDefaultHeader(res);
		IfsSystem.initIfs();
		IfsSession ifsSession = new IfsSession(req,res);
		try {
			servlets = getServlets();
			goldObjects = getIfsObjects();
			objects = new String[servlets.length + goldObjects.length];
			while(i < objects.length) {
				for(j = 0; j < servlets.length; i++, j++)
					objects[i] = servlets[j];
				for(j = 0; j < goldObjects.length; i++, j++)
					objects[i] = goldObjects[j];
			}
			sort(objects);
			selectedObjects = req.getParameterValues("Objects");
			debugReq = (String) getRequestParameter(req, "Debug");
			debugMedia = (String) getRequestParameter(req, "DebugMedia");
			debugLogName = (String) getRequestParameter(req, "LogFileName");
			debug(res, "Debug selection is " + debugReq);
			debug(res, "Debug output option is " + debugMedia);
			debug(res, "Debug LogName is " + debugLogName);
			IfsSystem.debugInfoToHtml = false;
			IfsSystem.debugInfoToFile = false;


         DebugViewBean debugBean = new DebugViewBean();
         debugBean.setDebugReq(debugReq);
			debugBean.setDebugOn(IfsSystem.DEBUG_ON);
			debugBean.setDebugMode(IfsSystem.debugMode);
			debugBean.setDebugOff(IfsSystem.DEBUG_OFF);
         debugBean.setDebugInfoToHtml(IfsSystem.debugInfoToHtml);
         debugBean.setDbgLogFileName(IfsSystem.getDebugLogFileName());
         debugBean.setServerPath(IfsSystem.ServerPath);



			if(null != debugMedia && debugMedia.equals("HTML"))
				IfsSystem.debugInfoToHtml = true;

			if(null != debugMedia && debugMedia.equals("File")) {
				IfsSystem.debugInfoToFile = true;
				if(null != debugLogName && !debugLogName.equals(IfsSystem.getDebugLogFileName()))
					IfsSystem.setDebugLogFileName(debugLogName);
			}
			if(null != debugReq && debugReq.equals("DebugOn"))
				IfsSystem.debugMode = IfsSystem.DEBUG_ON;

			if(null != debugReq && debugReq.equals("DebugOff"))
				IfsSystem.debugMode = IfsSystem.DEBUG_OFF;

			if(null != debugReq && debugReq.equals("DebugSome"))
				IfsSystem.debugMode = IfsSystem.DEBUG_SOME_CLASSES;

			if(null != debugReq && null != selectedObjects && debugReq.equals("DebugSome"))
				setObjectsDebugFlags(selectedObjects, debugReq, res);

			if(null != debugReq && debugReq.equals("DebugReset"))
				setObjectsDebugFlags(objects, debugReq, res);
         

			setDefaultHeader(res);
         req.setAttribute("DEBUG_BEAN", debugBean);
			//req.getRequestDispatcher("/debug.jsp").forward(req, res);
         getServletContext().getContext("/").getRequestDispatcher("/debug.jsp").forward(req, res);
		}
		catch(Exception e) {
			debug(res, "Exception ocurred, value is " + e.toString());
		}
	}

   /*
   *
   */

	public String[] getServlets() {
		String path = getServerPath();
		if(null != path) {
			File file = new File(path + "cws/docs/servlets");
			String[] files = file.list();
			return getJavaFiles(files);
		}
		return new String[0];
	}

   /*
   *
   */

	public String[] getIfsObjects() {
		String[] ifsObjects = null, ifsGold = null, ifsAuto = null;
		int i = 0, j;
		String path = getServerPath();

		if(null != path) {
			File file = new File(path + "cws/docs/jcwsapi/com/ibm/ifs/gold");
			String[] files = file.list();
			ifsGold = getJavaFiles(files);
			file = new File(path + "cws/docs/jcwsapi/com/ibm/ifs/gold/auto");
			files = file.list();
			ifsAuto = getJavaFiles(files);
			ifsObjects = new String[ifsGold.length + ifsAuto.length];
			while(i < ifsObjects.length) {
				for(j = 0; j < ifsGold.length; j++,i++) {
					ifsObjects[i] = ifsGold[j];
				}
				for(j = 0; j < ifsAuto.length; j++,i++) {
					ifsObjects[i] = ifsAuto[j];
				}
			}
         return ifsObjects;	
		}
		return new String[0];	
	}

   /*
   *
   */

	public String[] getJavaFiles(String[] files) {
		String[] javaFiles;
		int javaCount = 0;
		StringTokenizer tokens;
		String token, fileName;

		for(int i = 0; i < files.length; i++) {
			tokens = new StringTokenizer(files[i], ".");
			token = null;
			while(tokens.hasMoreTokens())
				token = tokens.nextToken();

			if("java".equals(token))
				javaCount++;

		}
		javaFiles = new String[javaCount];
		javaCount = 0;
		for(int i = 0; i < files.length; i++) {
			tokens = new StringTokenizer(files[i], ".");
			token = null;
			fileName = tokens.nextToken();
			while(tokens.hasMoreTokens())
				token = tokens.nextToken();

			if("java".equals(token))
				javaFiles[javaCount++] = fileName;
		}
		return javaFiles;
	}

   /*
   *
   */

   public static String getServerPath() {
		int idx;

      String path = System.getProperty("user.dir");
      if(path.endsWith("config") && ((idx = path.lastIndexOf("config")) != -1))
	 	return path.substring(0, idx);
                        
      return null;
   }
	
   /*
   *
   */

   public void setObjectsDebugFlags(String[] objects, String debugReq, HttpServletResponse res) {
		String packagedName = null;
		Class theClass = null;
		Field field;

		for(int i = 0; i < objects.length; i++) {
			packagedName = objects[i];
			while(null != packagedName) {
				try {
					theClass = Class.forName(packagedName);
					packagedName = null;
				}
				catch(ClassNotFoundException e) {
					if(packagedName.equals(objects[i]))
						packagedName = "com.ibm.ifs.gold." + objects[i];

					else if(packagedName.startsWith("com.ibm.ifs.gold.auto."))
						packagedName = null;

					else
						packagedName = "com.ibm.ifs.gold.auto." + objects[i];
				}
			}
			try {
				field = theClass.getField("debugOn");
				field.setBoolean(null, debugReq.equals("DebugSome"));
			}
			catch(Exception e) {
				debug(res, "Exception ocurred, value is " + e.toString());
			}
		}
	}

   /*
   *
   */

	public void sort(String[] objects) {
		int i, j, min;
		String temp;

		for(i = 0; i < objects.length; i++) {
			min = i;
			for(j = i; j < objects.length; j++) {
				if(objects[j].compareTo(objects[min]) < 0)
					min = j;
			}
			temp = objects[i];
			objects[i] = objects[min];
			objects[min] = temp;
		}
		return;
	}
}

