import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.ifs.servlets.IfsHttpServlet;
import com.ibm.ifs.gold.auto.GtUserProfile;
import com.ibm.ifs.gold.GtLogon;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsTxn;
import java.io.*; // remove before file check in


/**
 * Logon servlet.
 * @author      Sterling Smith*/

public class ProcessLogon extends IfsHttpServlet {
	//WARNING: This servlet should not attempt to write any data out to the response.  Writing any type of data to the response
	//object within this servlet, whether directly or indirectly(via a debug statement by one of the objects used in this servlet)
	//will break the logic within this servlet.  Normal process flow within this servlet is to perform a logon based on request
	//parameters passed to this servlet, retrieve the user's profile data and store them in the Http Session cache, retrieve the 
	//Locale information from the IfsSystem object and store it's contents in the Http Session cache and to then redirect processing
	//to the first displayable page.  The first displayable page is a changable parameter within the IfsSystem object.  If any errors
	//occur during processing of this servlet, this servlet will redirect to the appropriate error, info or redo page.  These pages
	//also are configurable within the IfsSystem object.  Redirection to first displayable page, error page, info page or redo page
	//is accomplished using the sendRedirect command of the HttpServletResponse object.  A restriction in using that method is that
	//it must be the only thing written to the response, hence the restriction of writing data to the response within this servlet.
	//If you do write data to the response before the sendRedirect command is invoked, the command simply appears to be ignored, thus
	//no redirect will take place.
	public void doPost (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
	   try {
		boolean ret_Status = true;
		IfsSystem.initIfs();
		setDefaultHeader(res);
		IfsSession ifsSession = new IfsSession(req, res);
		String[] browserInfo = initBrowserInfo(req);
		
		GtLogon logon = (GtLogon) createInstance(ifsSession, "com.ibm.ifs.gold.GtLogon", "logon");
		if(null!=logon) {
			logon.setUserID((String) getRequestParameter(req,"UserID"));
			logon.setPinEncryptionType("0");
			logon.setPin((String) getRequestParameter(req,"Pin"));
			logon.setPasswordEncryptionType("0");
			logon.setPassword((String) getRequestParameter(req,"Password"));
			logon.setProductName(browserInfo[0]);
			logon.setReleaseVersion(browserInfo[1]);
			logon.setClientOS(browserInfo[2]);
		} else
			res.sendRedirect(IfsSystem.ErrorPage + "?Exception=ProcessLogon Servlet could not create a new GtLogon object");

		logon.execute();
		ret_Status = handleErrors(logon, IfsSystem.ErrorPage, IfsSystem.InfoPage, IfsSystem.RedoPage);


		//Everything was successful, redirect to first displayable page.

		if (ret_Status) {
			 res.sendRedirect(IfsSystem.PageDisplayedFollowingLogon);
			}
	   }
	   catch(Exception e) {
		res.sendRedirect(IfsSystem.ErrorPage + "?Exception=" + e.toString());
	   }
	}
	private String getBrowserItem(String browserInfo) {
		int i;
		String item;

		if(null==browserInfo)
			return "";

		i = browserInfo.indexOf(';');
		if (i > -1) // found separator
			return browserInfo.substring(0,i);  // got element

		else //no separator found
			return browserInfo;   // return entire string
		
	} //end of getBrowserItem
	private String[] initBrowserInfo(HttpServletRequest req) {
		String browserUserAgent, browser, token, browserVersion = null, browserPlatform = null, browserName = null;
		String[] browserInfo = new String[3];
		int i;

		// get browser's 'User-agent' string from request header
		browser = req.getHeader("User-agent");
		browserUserAgent = browser;

		i = browserUserAgent.indexOf("Mozilla");

		if (i > -1) {
			// remove "Mozilla/" from UserAgent string
			browser = browserUserAgent.substring(browserUserAgent.indexOf("/")+1);

			// locate blank following version number
			i = browser.indexOf(" ");
			// get browser version number
			browserVersion = browser.substring(0, i);
			browserVersion = browserVersion.trim();  // remove any whitespace around version

			// get the browser info between the parens
			browser = browser.substring( (browser.indexOf("(")+1),
									  (browser.indexOf(")"))   );

			token = getBrowserItem(browser).trim();
			browser = updateBrowserItem(browser);

			if ( token == "compatible") {
				browserVersion += " compatible";   // add compatible to version

				// internet explorer

				browserName = getBrowserItem(browser).trim();
				browser = updateBrowserItem(browser);
				token = getBrowserItem(browser).trim();
				browser = updateBrowserItem(browser);

				if (token.indexOf("Update") > -1) {
					// should be platform or update
					browserName +=  " " + token;       // add update string to name
					token = getBrowserItem(browser).trim();
				}
			} else {
				// netscape
				browserName = "Netscape";
			}
			browserPlatform = token;                // operating system of browser
		}
		browserInfo[0] = new String(browserName);
		browserInfo[1] = new String(browserVersion);
		browserInfo[2] = new String(browserPlatform);

		return browserInfo;
	}   // end of initBrowserInfo()
		private String updateBrowserItem(String browserInfo) {
				int i;
				
				if(null==browserInfo)
						return "";

				i = browserInfo.indexOf(';');
				if (i > -1) // found separator
						return browserInfo.substring(i+1);  // remove element and separator

				else //no separator found
						return null;   // remove the list
				
		} //end of getBrowserItem
}

