/* ---------------------------------------------------------------------------*
 *  File: AccountDetail.java                                                  *
 *                                                                            *
 *  Copyright Information:                                                    *
 *    Licensed Materials - Property of IBM                                    *
 *    5648-A06                                                                *
 *    (c) Copyright IBM Corporation, 1996-2000. All rights reserved.          *
 *    US Government Users Restricted Rights - Use, duplication                *
 *    or disclosure restricted by GSA ADP Schedule Contract                   *
 *    with IBM                                                                *
 *                                                                            *
 * Description:                                                               *
 * ------------                                                               *
 * This servlet implements account detail of non-deposit account(MLA) by      *
 * providing:                                                                 *
 *    a. Payment and Balance information                                      *
 *    b. Interest and APR information                                         *
 *    c. Transaction Authority information                                    *
 *    d. Loan Origination information                                         *
 *    e. Escrow information                                                   *
 *    f. Signatures on Account information                                    *
 * This servlet is accessed when the user clicks on one of the account's      *
 * product name of the non-deposit accounts on the account summary page.      *
 * After all the necessary data is populated for generating the HTML page     *
 * it forwards control to the corresponding jsp.                              *
 * -------------------------------------------------------------------------- */

import java.io.*;
import java.util.*;
import javax.servlet.*;
import com.ibm.ifs.gold.*;
import java.net.URLEncoder;
import javax.servlet.http.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.IfsHttpServlet;

public class AccountDetail extends IfsHttpServlet {

/* ---------------------------- Class Variables ----------------------------- */

  /**  Set to true to turn on debugging. */
  private static final boolean debugOn = false;

/* ---------------------------- Instance Methods ---------------------------- */

  /**
   *
  */
  public void doGet (HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
    
    try {

    /*
     *
     * Declare and initialize local variables
     *
     */
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = (GtUserProfile)getObject(ifsSession, "__UserProfile");
      GtAccountDetail acctDtlTxn = (GtAccountDetail)createInstance(ifsSession,
                                                                   "com.ibm.ifs.gold.auto.GtAccountDetail",
                                                                   "acctDtlTxn");
      GsAcctDtl       accountDtl    = up.getAccountDetail()[0];
      GsAccount       account       = accountDtl.getAccount();

    /*
     *
     * Execute GtAccountDetail transaction to display the account details
     *
     */
		setFields(ifsSession,"acctDtlTxn",acctDtlTxn);
		acctDtlTxn.setExecutionMode(acctDtlTxn.EXECUTE_SMART);
		acctDtlTxn.execute();

      // Populate data for HTML generation using JavaBean
      AccountDetailViewBean viewBean = new AccountDetailViewBean();
      viewBean.setType(account.getType());
      viewBean.setNickName(account.getNickname());
      viewBean.setMortgageAccount(acctDtlTxn.getMortgageDetail().length > 0);

      GsMortgageAcctDtl mortgageDetails = acctDtlTxn.getMortgageDetail()[0];

      if (viewBean.isMortgageAccount()) {
        viewBean.setNumber(account.getNumber());
        try {
          viewBean.setStatus(accountDtl.getAccountStatusType_text());
        }
        catch (IfsException e) {
          viewBean.setStatus("");
        }
        viewBean.setInformationDate(accountDtl.getInformationDate().toString());
        viewBean.setPayoffAmount(mortgageDetails.getPayoffAmount().toString());
        viewBean.setPrincipalBalance(mortgageDetails.getPrincipalBalanceAmount().toString());
        viewBean.setNextPayment(mortgageDetails.getNextPaymentAmount().toString());
        viewBean.setNextPaymentDueDate(mortgageDetails.getNextPaymentDueDate().toString());
        try {
          viewBean.setLastBalance(acctDtlTxn.getLastStatementBalanceAmount().toString());
        }
        catch (Exception e) {
          viewBean.setLastBalance("");
        }
        viewBean.setLastStatementDate(acctDtlTxn.getLastStatementDate().toString());
        try {
          viewBean.setLastPaymentAmount(mortgageDetails.getPreviousPaymentAmount().toString());
        }
        catch (Exception e) {
          viewBean.setLastPaymentAmount("");
        }
        if ( mortgageDetails.getPreviousPaymentDate() != null           &&
             mortgageDetails.getPreviousPaymentDate().getDate() != null &&
            !mortgageDetails.getPreviousPaymentDate().getDate().equals(""))
          viewBean.setLastPaymentDate(mortgageDetails.getPreviousPaymentDate().toString());
        else
          viewBean.setLastPaymentDate("&nbsp;");

        viewBean.setNrPaymentsComplete(mortgageDetails.getNumberPaymentsComplete() + "");
        viewBean.setPrepaymentPenaltyAmount(mortgageDetails.getPrepaymentPenaltyAmount().toString());
        viewBean.setYTDInterestAmount(acctDtlTxn.getYTDInterestAmount().toString());
        viewBean.setLastYearInterestAmount(acctDtlTxn.getLastYearInterestAmount().toString());
        viewBean.setInterestRate(acctDtlTxn.getInterestRate().toString());
        int authority = Integer.parseInt(acctDtlTxn.getAuthority());
        String transferTo = "", transferFrom = "", billPayment = "";
        if ( (authority == 1) || (authority == 8) ) {
          transferTo = "No";
          transferFrom = "No";
        }
        else if ( (authority == 2) || (authority == 5) ) {
          transferTo = "No";
          transferFrom = "Yes";
        }
        else if ( (authority == 3) || (authority == 6) ) {
          transferTo= "Yes";
          transferFrom= "No";
        }
        else if ( (authority == 4) || (authority == 7) ) {
          transferTo= "Yes";
          transferFrom= "Yes";
        }
        if (authority > 4) {
          billPayment= "Yes";
        }
        else {
          billPayment= "No";
        }
        viewBean.setTransferTo(transferTo);
        viewBean.setTransferFrom(transferFrom);
        viewBean.setBillPayment(billPayment);

        try {
          viewBean.setOriginalLoanAmount(mortgageDetails.getOriginalLoanAmount().toString());

        }
        catch (Exception e) {
          viewBean.setOriginalLoanAmount("");

        }
        if ( mortgageDetails.getOriginalLoanDate() != null &&
             mortgageDetails.getOriginalLoanDate().getDate() != null &&
             !mortgageDetails.getOriginalLoanDate().getDate().equals(""))
          viewBean.setOriginalLoanDate(mortgageDetails.getOriginalLoanDate().toString());
        else
          viewBean.setOriginalLoanDate("&nbsp;");

        viewBean.setTermValue(mortgageDetails.getTermValue() + "");

        //viewBean.setTermPeriodType(mortgageDetails.getTermPeriodType_text());
        //remove when resource file has been fixed
        viewBean.setTermPeriodType("");
        viewBean.setEscrowBalance(mortgageDetails.getEscrowBalanceAmount().toString());
        viewBean.setTaxPaidAmount(mortgageDetails.getTaxPaidAmount().toString());
        viewBean.setLastYearTaxPaidDate(mortgageDetails.getLastYearTaxPaidDate().toString());
        viewBean.setTaxPaidDate(mortgageDetails.getTaxPaidDate().toString());
        viewBean.setInsurancePaidAmount(mortgageDetails.getInsurancePaidAmount().toString());
        viewBean.setLastYearInsurancePaidAmount(mortgageDetails.getLastYearInsurancePaidAmount().toString());
        viewBean.setInsurancePaidDate(mortgageDetails.getInsurancePaidDate().toString());
        viewBean.setNrSignaturesRequired( (acctDtlTxn.getCosignatureDetail().length + 1) + "" );

        String priority = "&nbsp;";
        if (acctDtlTxn.getAccountPriority().equals("P"))
          priority = "Primary";
        else if (acctDtlTxn.getAccountPriority().equals("R"))
          priority = "Related";
        viewBean.setAccountPriority(priority);

      }

      if (handleErrors(acctDtlTxn, IfsSystem.ErrorPage, "AccountSummary", "AccountSummary")) {
        setDefaultHeader(response);
        request.setAttribute("VIEW_BEAN", viewBean);
        request.getRequestDispatcher("/accountDetail.jsp").forward(request, response);
      }

    }
    catch (Exception e) {
      StringBuffer strBuf = new StringBuffer("ErrorPage?Exception=");
       strBuf.append(URLEncoder.encode("Exception caught in doGet method of AccountDetail Servlet:"));
       strBuf.append(URLEncoder.encode(e.toString()));
       callPage(response,strBuf.toString());
    }
  }

  /**
   *
  */
  public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {

   
     doGet(request, response);

  }

}  // End AccountDetail class
