
/*
 * AccountHistory.java
 *
 * Licensed Materials - Property of IBM
 * 5648-A06
 * (c) Copyright IBM Corporation, 1996-2000. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication
 *  or disclosure restricted by GSA ADP Schedule Contract
 *  with IBM
 *
 */

/*
 * Description:
 * -----------
 * This servlet implements deposit accounts history.
 * The servlet is accessed when the user clicks on one of the account's
 * product name of the non-deposit accounts on the account summary page.
 *
 * Http Request Parameters:
 * -----------------------
 * Following request parameter names are used in this servlet:
 *   bi    - oid of GtBalanceInquiry
 *   ah    - oid of GtAccountHistory
 *   bi.reqAccount - account
 *   ah.reqAccount - account
 */

import java.io.IOException;
import java.io.PrintWriter;
import java.net.URLEncoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import com.ibm.ifs.servlets.IfsHttpServlet;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.auto.GtUserProfile;
import com.ibm.ifs.gold.auto.GtBalanceInquiry;
import com.ibm.ifs.gold.auto.GtAccountHistory;
import com.ibm.ifs.gold.GsAccount;
import com.ibm.ifs.gold.IfsTxn;
import com.ibm.ifs.gold.IfsException;

public class AccountHistory extends IfsHttpServlet
{
   /*
    *  Use the flag to debug this servlet
    */
   public static boolean debugOn = false;

   
	public void doGet (HttpServletRequest request, HttpServletResponse response)
   throws ServletException, IOException
   {
      /*
       *
       * Declare and initialize local variables
       *
       */
      setDefaultHeader(response);   // Need to set character set before getWriter()
      IfsSession       ifsSession    = new IfsSession(request,
                                                      response);
		GtUserProfile    up            = (GtUserProfile)getObject(ifsSession,
																					 "__UserProfile");
		GtBalanceInquiry bi            = (GtBalanceInquiry)createInstance(ifsSession,
																								"com.ibm.ifs.gold.auto.GtBalanceInquiry",
																								"bi");
		GtAccountHistory ah            = (GtAccountHistory)createInstance(ifsSession,
																								"com.ibm.ifs.gold.auto.GtAccountHistory",
																								"ah");
      GsAccount        account       = null;



      try
      {
         /*
          *
          * Execute GtBalanceInquiry transaction to display the account balance details
          *
          */
         setFields(ifsSession,"bi",bi);
         bi.setExecutionMode(bi.EXECUTE_SMART);
         bi.execute();
         handleErrors(bi, "ErrorPage", "AccountSummary", "AccountSummary");

         /*
          *
          * Execute GtAccountHistory transaction to display the account details
          *
          */
         setFields(ifsSession,"ah",ah);
         ah.setReqTransactionType("1"); // get all types (credit, debit, check, deposit, withdrawls )
         ah.setReqMaximumNumberOfRecords(15); // Maximum Records
         ah.setReqLastNumberOfRecords(0);
         ah.setExecutionMode(ah.EXECUTE_SMART);
         ah.execute();
         handleErrors(ah, "ErrorPage", "AccountSummary", "AccountSummary");
			
			AccountBean viewBean= new AccountBean();
         account = ah.getReqAccount();
			viewBean.setNickName(account.getNickname());
			viewBean.setNumber(account.getNumber());
			viewBean.setType(account.getType());
			viewBean.setLedgerBalAmount(bi.getLedgerBalanceAmount().toString());
			viewBean.setInformationDate(ah.getInfoDate().toString());

			viewBean.setUnknownString("is unknown.");
			viewBean.setAccountHistory(ah);

			setDefaultHeader(response);
			request.setAttribute("VIEW_BEAN", viewBean);
			request.getRequestDispatcher("/accountHistory.jsp").forward(request, response);


      }
      catch (Exception e)
      {
         StringBuffer strBuf = new StringBuffer("ErrorPage?Exception=");
         strBuf.append(URLEncoder.encode("Exception caught in doGet method of AccountHistory Servlet:"));
         strBuf.append(URLEncoder.encode(e.toString()));
         callPage(response,strBuf.toString());
      }

           
   }//end of doGet

    /*
    *
    * We perform the same operations for POST request as done for GET requests,
    * so this method just sends the request to the doGet method.
    *
    */
   public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
   {
         doGet(request,response);
   }
}

