/* ----------------------------------------------------------------------*
 * File: AccountSummary.java                                               *
 *                                                                       *
 * Description:                                                          *
 *                                                                       *
 * Copyright Information:                                                *
 *                                                                       *
 * Licensed Materials - Property of IBM                                  *
 * 5648-A06                                                              *
 * (c) Copyright IBM Corporation, 1996-2000. All rights                  *
 * reserved. US Government Users Restricted Rights - Use,                *
 * duplication or disclosure restricted by GSA ADP Schedule              *
 * Contract with IBM.                                                    *
 *                                                                       *
 * ----------------------------------------------------------------------*
 * This servlet implements account summary by providing                  *
 * information of deposit and non-deposit (MLA only) accounts.           *
 * The initial call to this servlet is after the user enters his         *
 * name, password, pin then clicksthe enter button on the Logon page     *
 * and thereafter can be called repeatedly during page navigation to     *
 * repaint the account information when:                                 *
 *    a. the user clicks on the footer link 'ACCOUNTS' on any            *
 *       page.                                                           *
 *    b. the user clicks on one of the table header links of             *
 *       the accounts viz.                                               *
 *           [Product Name],                                             *
 *           [Account Type]/[Account Number],                            *
 *           [Ledger Balance] ,                                          *
 *           [Principal Balance],                                        *
 *         [Date]).                                                      *
 *                                                                       *
 * The account summary page pre-sorts by default with account            *
 * nickname when sortBy is null.                                         *
 * The sortBy variable can have the value 'ProductName',                 *
 * 'AccountNumber', 'LedgerBalance','PrincipalBalance', 'Date'           *
 * or null. This value is used to call the correct sort method           *
 * to organize accounts before they are displayed to the user.           *
 *                                                                       *
 * Http Request Parameters:                                              *
 * -----------------------                                               *
 * Following request parameter names are used in this servlet:           *
 *   sortBy - flag used to call the correct sort method call             *
 *                                                                       *
 *-----------------------------------------------------------------------*/

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.ifs.servlets.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.gold.*;
import java.text.Collator; 

public class AccountSummary extends IfsHttpServlet {

  public void doPost (HttpServletRequest req, HttpServletResponse res) 
    throws ServletException, IOException {
	   
    try {
         IfsSession ifsSession = new IfsSession(req, res);
      
             Collator      collator    = Collator.getInstance();
             GtUserProfile up          = null;
             String        sortBy      = (String)getRequestParameter(req,"sortBy");

             try {
                  if (sortBy == null) sortBy = "";

                  up = (GtUserProfile)getObject(ifsSession,"__UserProfile");

                    // up may be null, if null then execute it.
                  if (up == null) {
                     up = new GtUserProfile(ifsSession);
                     try {
                           up.execute();
                           if (!handleErrors(up, "ErrorPage", "ErrorPage", "ErrorPage"))
                           return;
                           putObject(ifsSession,"__UserProfile",up); // put in cache for later use
                          }
                     catch (Exception e) {
                        // Forward to error page
                         req.getRequestDispatcher(IfsSystem.ErrorPage).forward(req, res);          
                      }
                 }

          /*
           *
           * Sort the accounts before displaying to the user
           *
          */
          if (sortBy.equals("AccountNumber")) {
            try {
              debug(res,"DEBUG:Sorting accounts by account number in ascending order");
              sort(collator,up.getAccountDetail(),"getAccount.getNumber",true);
            } 
            catch (Exception e) {
              debug(res,"DEBUG:Exception caught when sorting by account number:"+e.toString());
            }
          }
          else if (sortBy.equals("LegderBalance")) {
            try {
              debug(res,"DEBUG:Sorting accounts by ledger balance in ascending order");
              sort(up.getAccountDetail(),"getLedgerBalanceAmount",new GsCurrency("$1.00",ifsSession),true);
            }
            catch (Exception e) {
              debug(res,"DEBUG:Exception caught when sorting by ledger balance :"+e.toString());
            }
          }
          else if (sortBy.equals("PrincipalBalance")) {
            try {
              debug(res,"DEBUG:Sorting accounts by principal balance in ascending order");
              sort(up.getAccountDetail(),"getAvailableBalanceAmount",new GsCurrency("$1.00",ifsSession),true);
            }
            catch (Exception e) {
              debug(res,"DEBUG:Exception caught when sorting by ledger balance:"+e.toString());
            }
          }
          else if (sortBy.equals("Date")) {
            try {
              debug(res,"DEBUG:Sorting accounts by account nickname in ascending order");
            }
            catch (Exception e) {
              debug(res,"DEBUG:Exception caught when sorting by nickname:"+e.toString());
            }
          }
          else {
            try {
              debug(res,"DEBUG:Sorting accounts by account nickname in ascending order");
              sort(collator,up.getAccountDetail(),"getAccount.getNickname",true);
            }
            catch (Exception e) {
              debug(res,"DEBUG:Exception caught when sorting by account nickname:"+e.toString());
            }
          }
		   }

        catch (Exception e) {
          StringBuffer strBuf = new StringBuffer("ErrorPage?Exception=");
          strBuf.append(URLEncoder.encode("Exception caught in doPost method of AccountSummary Servlet:"));
          strBuf.append(URLEncoder.encode(e.toString()));
          req.getRequestDispatcher(IfsSystem.ErrorPage).forward(req, res);
        }

        // Create and populate view bean
        UserProfileViewBean viewBean = new UserProfileViewBean();
        viewBean.setName(up.getConsumerType().equalsIgnoreCase("S") ? up.getLegalName() :
                         up.getFirstName() + " " + up.getMiddleName() + " " + up.getLastName());
        Vector depositAccountsVec = new Vector();
        Vector mortgageAccountsVec = new Vector();
        GsAcctDtl[] acctDtl = up.getAccountDetail();
        for (int i = 0; i < acctDtl.length; ++i) {
          GsAcctDtl accountDetail = acctDtl[i];
          GsAccount account = accountDetail.getAccount();
          AccountBean accountBean = new AccountBean();
          accountBean.setType(account.getType());
          accountBean.setNumber(account.getNumber());
          accountBean.setInformationDate(accountDetail.getInformationDate().toString());

          String type = account.getType();
          String acctNick = account.getNickname();
          StringBuffer strBuf = new StringBuffer();

          if ((type.equals("CD"))||(type.equals("DDA"))||(type.equals("IRA"))||(type.equals("SDA"))) {
            strBuf.append("AccountHistory");
            strBuf.append("?bi=");
            strBuf.append(getNewOid(ifsSession,"GtBalanceInquiry"));
            strBuf.append("&ah=");
            strBuf.append(getNewOid(ifsSession,"GtAccountHistory"));
            strBuf.append("&bi.reqAccountID=__UserProfile.accountDetail.");
            strBuf.append(Integer.toString(i));
            strBuf.append(".account&ah.reqAccount=__UserProfile.accountDetail.");
            strBuf.append(Integer.toString(i));
            strBuf.append(".account");
            
            depositAccountsVec.add(accountBean);
          }
          else if (type.equals("MLA")) {
            strBuf.append("AccountDetail");
            strBuf.append("?index=");
            strBuf.append(Integer.toString(i));
            strBuf.append("&acctDtlTxn=");
            strBuf.append(getNewOid(ifsSession,"GtAccountDetail"));
            strBuf.append("&acctDtlTxn.reqAccountID=__UserProfile.accountDetail.");
            strBuf.append(Integer.toString(i));
            strBuf.append(".account");

            mortgageAccountsVec.add(accountBean);
          }

         // Account name displayed, link to more information if it is available
         if ( acctNick != null ) {
           // As long as the account status is not "Unavailable"
           if (!accountDetail.getAccountStatusType().equals("0")) {
             accountBean.setNickName("<A href =\" " +strBuf.toString() + "\">" + acctNick + "</A>");
           }
           else {
             accountBean.setNickName(acctNick);
           }
         }
         else {
           if (!accountDetail.getAccountStatusType().equals("0")) {
             try {
               accountBean.setNickName("<A href =\"" + strBuf.toString() + "\">" +
                                       accountDetail.getAccount().getType_text() + "</A>");
             }
             catch (IfsException e) {
             }
           }
           else {
             try {
               accountBean.setNickName(accountDetail.getAccount().getType_text());
             }
             catch (IfsException e) {
             }
           }
         }

         // Show account's balance unless account status indicates unavailable
         if (!accountDetail.getAccountStatusType().equals("0")) {
           if ((type.equals("CD"))||(type.equals("DDA"))||(type.equals("IRA"))||(type.equals("SDA"))) {
             accountBean.setAmount(accountDetail.getLedgerBalanceAmount().toString());
           }
           else {
             String s = "";
             try {
               s = accountDetail.getAvailableBalanceAmount().toString();
             }
             catch (Exception e) {
               debug(res,
                     "ERROR:Exception occured in acct.getAvailableBalanceAmount().toString() method:"+
                     e.toString());
             }
             accountBean.setAmount(s);
           }
         }
         else {
           // Account is not available at this time, display this status to user
           try { 
             accountBean.setAmount(accountDetail.getAccountStatusType_text());
           }
           catch (IfsException e) { 
           }
         }
       }

       AccountBean[] depositAccounts = new AccountBean[depositAccountsVec.size()];
       for (int i = 0; i < depositAccounts.length; ++i)
         depositAccounts[i] = (AccountBean)depositAccountsVec.elementAt(i);
       AccountBean[] mortgageAccounts = new AccountBean[mortgageAccountsVec.size()];
       for (int i = 0; i < mortgageAccounts.length; ++i) 
         mortgageAccounts[i] = (AccountBean)mortgageAccountsVec.elementAt(i);

       viewBean.setDepositAccounts(depositAccounts);
       viewBean.setMortgageAccounts(mortgageAccounts);

       req.setAttribute("VIEW_BEAN", viewBean);
       setDefaultHeader(res);
       // req.getRequestDispatcher("/accountSummary.jsp").forward(req, res);
       getServletContext().getContext("/").getRequestDispatcher("/accountSummary.jsp").forward(req, res);
    }
    catch(Exception e) {
      // Log error message here
      e.printStackTrace();
      System.err.println("ERROR: " + e.getMessage());  
      req.getRequestDispatcher(IfsSystem.ErrorPage).forward(req, res);
    }

  }

  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
     {
		 try
		 {
			 doPost(request,response);
		 }
		 catch (Exception e)
		 {
			 StringBuffer  errMsg      = null;
			 errMsg = new StringBuffer("ErrorPage?Exception=");
			 errMsg.append(URLEncoder.encode("Exception caught in doPost method when calling doGet method:"));
			 errMsg.append(URLEncoder.encode(e.toString()));
			 callPage(response,errMsg.toString());
		 }
     }



}

