
/*
 *  AddOneTimeTransfer.java
 *
 * Licensed Materials - Property of IBM 
 * 5648-A06              
 * (c) Copyright IBM Corporation, 1996-2000. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication 
 *  or disclosure restricted by GSA ADP Schedule Contract   
 *  with IBM                                               
 *
 */

 /*
  * Description:
  * -----------
  * This servlet implements add variable transfer confirmation by
  * displaying the following fields:
  *    1. Transfer From Account - selected source account 
  *    2. Transfer To Account - selected target account 
  *    3. Amount - amount entered
  *    4. Transfer Frequency - shows as One-Time since the servlet is
  *                            designed to add variable transfer only.
  *    5. Transfer Date - date entered by the user (usually is the current
  *                       date). Adjusted date will be displayed if the 
  *                       Gold message returns one.
  *    6. Memo - memo description entered.
  *    7. Confirmation - confirmation number 
  * 
  * The page is intended to show confirmation of the transfer
  * after successful execution of the GtAddVariableTransfer transaction.
  * The user is confirmed of this transfer by displaying a confirmation
  * number. 
  *
  * Http Request Parameters:
  * -----------------------
  * Following request parameter names are used in this servlet
  *   a. transfer - an oid of the cached GtAddVariablePayment object 
  *   c. reqMemo - memo description entered on the user input form.
  *
  */

import java.io.PrintWriter;
import java.io.IOException;
import java.net.URLEncoder;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.ibm.ifs.gold.IfsTxn;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.GsAccount;
import com.ibm.ifs.gold.GsCurrency;
import com.ibm.ifs.gold.GsDate;
import com.ibm.ifs.gold.auto.GtAddVariableTransfer;
import com.ibm.ifs.servlets.IfsHttpServlet;

/*
 *
 * Servlet Class to display the user input and confirm the transfer
 * 
 */
 public class AddOneTimeTransfer extends IfsHttpServlet 
 { 
  /*
   *  Use the flag to debug this servlet 
   */
   public static boolean debugOn = false;

  /*
   *  doPost method
   */
   public void doPost (HttpServletRequest request, HttpServletResponse response) 
		       throws ServletException, IOException 
   { 
    /*
     *
     * Declare and initialize local variables
     *
     */
     setDefaultHeader(response);
     PrintWriter           out          = response.getWriter();
     IfsSession            ifsSession   = new IfsSession(request,response);
     GtAddVariableTransfer transfer     = null; 
     GsDate                transferDate = null;
      
    try
    {
    /*
     * 
     * Execute GtAddVariableTransfer Transaction
     *
     */
     transfer = (GtAddVariableTransfer)createInstance(ifsSession, 
						      "com.ibm.ifs.gold.auto.GtAddVariableTransfer", 
						      "transfer");

     transfer.setExecutionMode(transfer.EXECUTE_SMART);
     transfer.execute();
     handleErrors(transfer,"ErrorPage", "AddOneTimeTransferForm", "AddOneTimeTransferForm");
    }
    catch (Exception e)
    {
      StringBuffer strBuf = new StringBuffer("ErrorPage?Exception=");
      strBuf.append(URLEncoder.encode("Exception caught in doPost method of AddOneTimeTransfer Servlet:"));
      strBuf.append(URLEncoder.encode(e.toString()));
      callPage(response,strBuf.toString());
    }
      
     out.println("<HTML>");
     out.println("<HEAD>");

    /*
     * 
     * Adds HTML comment of the copyright statement for this servlet 
     *
     */
     addCopyright(response);

    /*
     * 
     * Display page title
     *
     */
     out.println("<TITLE>");
     out.println("WebBank Interactive Banking: TRANSFER FUNDS");
     out.println("</TITLE>");
     out.println("<META NAME=\"Author\" VALUE=\"IBM Corporation\">");
     out.println("<META NAME=\"Application\" VALUE=\"Customizable Web Server Reference Pages\">");
     out.println("</HEAD>");
      
    /*
     * 
     * Background color, webank logo, banner....
     *
     */
     out.println("<BODY BGCOLOR=\"#FFFFFF\" BACKGROUND=\"\">");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
     out.println("<TR><TD><IMG SRC=/images/3dlogo.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
     out.println("<BR>");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
     out.println("<TR><TD><IMG SRC=/images/3dbanner.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
     out.println("<BR>");
      
    /*
     * 
     * Display standard title and instructions
     *
     */
     out.println("<P><TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0><TR>");
     out.println("<TD VALIGN=BASELINE ALIGN=CENTER><FONT SIZE=+2 COLOR=#000000>TRANSFER WITHIN MY ACCOUNTS - ");
     out.println("</FONT><FONT SIZE=+2 COLOR=#000080>ONE-TIME TRANSFER SCHEDULED</FONT></TD>");
     out.println("</TR></TABLE>");
     out.println("<IMG SRC=/images/hr_2.gif WIDTH=100% HEIGHT=3 ALIGN=CENTER><BR>");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=3 CELLPADDING=2>");
     out.println("<TR>");
     out.println("<TD WIDTH=90% BGCOLOR=#8EB5F0>&nbsp;</TD>");
     out.println("</TR></TABLE>");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=2 CELLPADDING=0>");
     out.println("<TR><TD WIDTH=100% VALIGN=TOP><FONT>Your one-time transfer has been scheduled. ");
     out.println("Your confirmation number regarding this transfer is as follows:</FONT></TD>");
     out.println("</TABLE>");
      
    /*
     * 
     * Display Adjusted Transfer Date Note
     *
     */
     if((transferDate = transfer.getAdjustedTransferDate()) != null)
     {
       if(!(transferDate.getDate().equals("")))
       {
	 out.println("<BR><B>NOTE: The Transmit Date has been changed to "+transferDate.toString() +" as ");
	 out.println("the requested date was on a non-business day or you have entered ");
         out.println("the transfer for today and it is past today's cutoff time.</B><BR><P>");
       }
       else
       {
	 transferDate = transfer.getReqEffectiveDate();
       }
     }
     out.println("<BR>");
     out.println("<TABLE WIDTH=100% ALIGN=CENTER BGCOLOR=#FFFFFF BORDER=0 CELLPADDING=2 CELLSPACING=1>");

     // Confirmation Number
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Confirmation Number: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+transfer.getReferenceNumber()+"</B></TD>");
     out.println("</TR>");

     // Source Account 
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Transfer From Account: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+transfer.getReqSource().getNickname() +" "+transfer.getReqSource().getNumber()+"</B></TD>");
     out.println("</TR>");

     // Target Account
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Transfer To Account: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+transfer.getReqTarget().getNickname()+" "+transfer.getReqTarget().getNumber()+"</B></TD>");
     out.println("</TR>");

     // Transfer Amount
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Amount: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+transfer.getReqTransactionAmount()+"</B></TD>");
     out.println("</TR>");

     // Transfer Frequency
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Transfer Frequency: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>One-Time</B></TD>");
     out.println("</TR>");

     // Transfer Date
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Transmit Date: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+transferDate+"</B></TD>");
     out.println("</TR>");

     // Transfer Memo
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Memo: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+(String)getRequestParameter(request,"reqMemo")+"</B></TD>");
     out.println("</TR>");
     out.println("</TABLE>");
     out.println("</P>");
      
     // Begin FormatButtonBarFooter 
     out.println("<P><IMG SRC=/images/hr_2.gif WIDTH=100% HEIGHT=3 ALIGN=BOTTOM>");
     out.println("<P ALIGN=CENTER><FONT SIZE=-2>");
     out.println("<A HREF=AccountSummary>ACCOUNTS</A> | ");
     out.println("<A HREF=AddOneTimeTransferForm>TRANSFERS</A> | ");
     out.println("<A HREF=Logoff>EXIT</A></FONT></P>");

     out.println("</BODY>");
     out.println("</HTML>");
  }         

 /*
  *
  * We perform the same operations for GET request as done for POST requests, 
  * so this method just sends the request to the doPost method.
  *
  */
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
  {
    StringBuffer  errMsg      = null;
    try
    {
      doPost(request,response);
    }
    catch (Exception e)
    {
      errMsg = new StringBuffer("ErrorPage?Exception=");
      errMsg.append(URLEncoder.encode("Exception caught in doGet method when calling doPost method:"));
      errMsg.append(e.toString());
      callPage(response,errMsg.toString());
    }
  }
}

