
/*
 *  AddOneTimeTransferConfirmation.java
 *
 * Licensed Materials - Property of IBM 
 * 5648-A06              
 * (c) Copyright IBM Corporation, 1996-2000. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication 
 *  or disclosure restricted by GSA ADP Schedule Contract   
 *  with IBM                                               
 *
 */

 /*
  * Description:
  * -----------
  * This servlet implements confirm add variable transfer by
  * displaying the following fields:
  *    1. Transfer From Account - selected source account 
  *    2. Transfer To Account - selected target account 
  *    3. Amount - amount entered
  *    4. Transfer Frequency - shows as One-Time since the servlet is
  *                            designed to add variable transfer only.
  *    5. Transfer Date - date entered by the user (usually is the current
  *                       date).
  *    6. Memo - memo description entered.
  * 
  * The page is intended for the user to confirm before issuing
  * the Gold transaction. A confirmation page will be displayed once the user has
  * clicked the 'OK' button at the bottom. If the user clicks on the 
  * 'Cancel' button, displays back the user input form to correct
  * the information which caused the cancellation.
  *
  * Http Request Parameters:
  * -----------------------
  * Following request parameter names/values are used in this servlet:
  *   transfer - an oid of GtAddVariableTransfer
  *   transfer.reqTransferImmediateFlag - value of "I"
  *   transfer.reqTransferDuplChkOvrdFlag - value of "N"
  *   transfer.reqSource - account selected
  *   transfer.reqTarget - account selected
  *   transfer.reqTransactionAmount - amount entered
  *   transfer.reqEffectiveDate - date entered
  *   reqMemo  - memo description 
  *
  */


import java.io.PrintWriter;
import java.io.IOException;
import java.net.URLEncoder;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import com.ibm.ifs.gold.auto.GtAddVariableTransfer;
import com.ibm.ifs.servlets.IfsHttpServlet;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsSecurity;
import com.ibm.ifs.gold.GsAccount;
import com.ibm.ifs.gold.GsCurrency;
import com.ibm.ifs.gold.GsDate;

/*
 *
 * Servlet Class to display the user input to confirm the transfer
 * 
 */
 public class AddOneTimeTransferConfirmation extends IfsHttpServlet 
 {
  /*
   *  Use the flag to debug this servlet 
   */
   public static boolean debugOn = false;

  /*
   *  doPost method
   */
   public void doPost (HttpServletRequest request, HttpServletResponse response)
	  throws ServletException, IOException
   {
    /*
     *
     * Declare and initialize local variables
     *
     */
     setDefaultHeader(response);
     PrintWriter           out         = response.getWriter();
     IfsSession            ifsSession  = new IfsSession(request, response);
     String                validateMsg = "OK";
     String                oid         = "";
     String                tfrMemo     = (String)getRequestParameter(request,"reqMemo");
     GtAddVariableTransfer transfer    = (GtAddVariableTransfer)createInstance(ifsSession,
									       "com.ibm.ifs.gold.auto.GtAddVariableTransfer",
									       "transfer");
     StringBuffer          strBuf      = null;

    try
    {
    /*
     * 
     * Set the fields of the GtAddVariableTransfer txn instance 
     *
     */
     setFields(ifsSession,"transfer",transfer);

    /*
     * 
     * Get the object ID for the cached GtAddVariableTransfer instance object - transfer
     *
     */
     oid = getOid(ifsSession,transfer);

    /*
     * 
     * Register a custom method to validate the fields before executing the GtAddVariableTransfer txn.
     *
     */
     if(!transfer.hasRegisteredMethod("validate"))
     {
       try 
       {
         transfer.registerMethod("validate", CustomMethods.class.getMethod("validate", new Class[]{GtAddVariableTransfer.class}));
       }
       catch(Exception e) 
       {
	 debug(response,"Exception occured while registering the validate method");
       }
     }

    /*
     * 
     * Validate the input fields. If found any errors go back to the form page to correct the errors. 
     *
     */
     validateMsg = transfer.validate();
     if(validateMsg != null && !validateMsg.equals("OK"))
     {
       strBuf = new StringBuffer();
       strBuf.append("AddOneTimeTransferForm?phase=REDISPLAY&transfer=");
       strBuf.append(oid);
       strBuf.append("&errMsg=");
       strBuf.append(URLEncoder.encode(validateMsg));
       strBuf.append("&memo=");
       strBuf.append(URLEncoder.encode(tfrMemo));
       callPage(response,strBuf.toString());
     }
    }
    catch (Exception e)
    {
      strBuf = new StringBuffer("ErrorPage?Exception=");
      strBuf.append(URLEncoder.encode("Exception caught in doPost method of AddOneTimeTransferConfirmation Servlet:"));
      strBuf.append(URLEncoder.encode(e.toString()));
      callPage(response,strBuf.toString());
    }

     out.println("<HTML>");
     out.println("<HEAD>");

    /*
     * 
     * Adds HTML comment of the copyright statement for this servlet 
     *
     */
     addCopyright(response);

    /*
     * 
     * Display page title
     *
     */
     out.println("<TITLE>");
     out.println("WebBank Interactive Banking: TRANSFER FUNDS"); 
     out.println("</TITLE>");
     out.println("<META NAME=\"Author\" VALUE=\"IBM Corporation\">");
     out.println("<META NAME=\"Application\" VALUE=\"Customizable Web Server Reference Pages\">");
     out.println("</HEAD>");
      
    /*
     * 
     * Background color, webank logo, banner....
     *
     */
     out.println("<BODY BGCOLOR=\"#FFFFFF\" BACKGROUND=\"\">");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
     out.println("<TR><TD><IMG SRC=/images/3dlogo.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
     out.println("<BR>");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
     out.println("<TR><TD><IMG SRC=/images/3dbanner.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
     out.println("<BR>");

    /*
     * 
     * Display standard title and instructions
     *
     */
     out.println("<P><TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0><TR>");
     out.println("<TD VALIGN=BASELINE ALIGN=CENTER><FONT SIZE=+2 COLOR=#000000>TRANSFER WITHIN MY ACCOUNTS - ");
     out.println("</FONT><FONT SIZE=+2 COLOR=#000080>CONFIRM TRANSFER</FONT></TD>");
     out.println("</TR></TABLE>");
     out.println("<IMG SRC=/images/hr_2.gif WIDTH=100% HEIGHT=3 ALIGN=CENTER><BR>");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=3 CELLPADDING=2>");
     out.println("<TR>");
     out.println("<TD WIDTH=90% BGCOLOR=#8EB5F0>&nbsp;</TD>");
     out.println("</TR></TABLE>");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=2 CELLPADDING=0>");
     out.println("<TR><TD WIDTH=100% VALIGN=TOP><FONT>The following transfer will be scheduled. To confirm, ");
     out.println("click on <B>OK</B>. Otherwise, click on <B>Cancel</B>.</FONT></TD>");
     out.println("</TABLE>");
      
    /*
     * 
     * Display user input
     *
     */
     out.println("<P>");
     out.println("<TABLE WIDTH=100% ALIGN=CENTER BGCOLOR=#FFFFFF BORDER=0 CELLPADDING=2 CELLSPACING=1>");
     // Source account selected 
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Transfer From Account: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+transfer.getReqSource().getNickname()+" "+transfer.getReqSource().getNumber()+"</B></TD>");
     out.println("</TR>");

     // Target account selected
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Transfer To Account: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+transfer.getReqTarget().getNickname()+" "+transfer.getReqTarget().getNumber()+"</B></TD>");
     out.println("</TR>");

     // Transfer Amount entered
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Amount: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+transfer.getReqTransactionAmount()+"</B></TD>");
     out.println("</TR>");

     // Transfer frequency:One-Time
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Transfer Frequency: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>One-Time</B></TD>");
     out.println("</TR>");

     // Transfer Date:Today's date or future date
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Transmit Date: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+transfer.getReqEffectiveDate()+"</B></TD>");
     out.println("</TR>");

     // Memo
     out.println("<TR>");
     out.println("<TD ALIGN=RIGHT WIDTH=50% NOWRAP>");
     out.println("Memo: </TD>");
     out.println("<TD ALIGN=LEFT WIDTH=50% NOWRAP>");
     out.println("<B>"+tfrMemo+"</B></TD>");
     out.println("</TR>");
     out.println("</TABLE>");
     out.println("</P>");
      
     out.println("<CENTER><FORM METHOD=post ACTION=\"AddOneTimeTransfer\">"); 

    /*
     *
     * Send the cached GtAddVariableTransfer txn object 
     *
     */
     out.println("<INPUT TYPE=HIDDEN NAME=transfer VALUE=\""+oid+"\">");

    /*
     *
     * Save memo and transfer frequency to display on the next page
     *
     */
     out.println("<INPUT TYPE=HIDDEN NAME=reqMemo VALUE=\""+tfrMemo+"\">");
     out.println("<P><CENTER>");
     out.println("<TABLE Width=30% BORDER=0 CELLSPACING=2 CELLPADDING=0>");

    /*
     *
     * Display OK button and if clicked display the confirmation page informing the 
     * scheduled transfer 
     *
     */
     out.println("<TR>");
     out.println("<TD WIDTH=50% ALIGN=CENTER>");
     out.println("<INPUT TYPE=image BORDER=0 NAME=ok SRC=/images/ok.gif WIDTH=85 HEIGHT=25>");
     out.println("</TD>");
     out.println("</FORM>");

    /*
     *
     * Display CANCEL button and if clicked display the user input form
     *
     */
     out.println("<FORM METHOD=post ACTION=\"AddOneTimeTransferForm\">"); 
     out.println("<TD WIDTH=50% ALIGN=CENTER>");
     out.println("<INPUT TYPE=HIDDEN NAME=transfer VALUE=\""+oid+"\">");
     out.println("<INPUT TYPE=HIDDEN NAME=phase VALUE=\"REDISPLAY\">");
     out.println("<INPUT TYPE=HIDDEN NAME=memo VALUE=\""+tfrMemo+"\">");
     out.println("<INPUT TYPE=image BORDER=0 NAME=cancel SRC=/images/cancel.gif WIDTH=85 HEIGHT=25>");
     out.println("</TD>");
     out.println("</TR>");
     out.println("</TABLE>");
     out.println("</CENTER>");
     out.println("</FORM>");
     out.println("<P>");
     out.println("</CENTER><P><IMG SRC=/images/blkblue.gif WIDTH=100% HEIGHT=7 ALIGN=BOTTOM>");
      
     out.println("</BODY>");
     out.println("</HTML>");
  }         
   
 /*
  *
  * We perform the same operations for GET request as done for POST requests, 
  * so this method just sends the request to the doPost method.
  *
  */
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
  {
    IfsSession    ifsSession  = new IfsSession(request, response);
    StringBuffer  errMsg      = null;
    try
    {
      doPost(request,response);
    }
    catch (Exception e)
    {
      errMsg = new StringBuffer("ErrorPage?Exception=");
      errMsg.append(URLEncoder.encode("Exception caught in doGet method when calling doPost method:"));
      errMsg.append(URLEncoder.encode(e.toString()));
      callPage(response,errMsg.toString());
    }
  }
}


