
/*
 *  AddOneTimeTransferForm.java
 *
 * Licensed Materials - Property of IBM 
 * 5648-A06              
 * (c) Copyright IBM Corporation, 1996-2000. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication 
 *  or disclosure restricted by GSA ADP Schedule Contract   
 *  with IBM                                               
 *
 *
 */

 /*
  * Description:
  * -----------
  * This servlet implements add variable transfer user input form by
  * displaying following fields:
  *    1. Transfer From Account - lists a drop down of source accounts
  *                               to allow the user to select one.
  *    2. Transfer To Account - lists a drop down of target accounts
  *                             to allow the user to select one.
  *    3. Amount - a text box for user to enter the amount in dollars.
  *    4. Transfer Frequency - shows as One-Time since the servlet is
  *                            designed to add variable transfer only.
  *    5. Transfer Date - a text box for user to enter the date in 
  *                       mm/dd/yyyy format. By default the text box
  *                       is pre-filed with the current date.
  *    6. Memo - an optional field for the user to enter memo description.
  *
  * A confirm page will be displayed once the user has entered all the information 
  * correctly and clicked the 'Enter' button at the bottom. If the confirm page 
  * servlet detects any validation errors, it will redirect back to this page 
  *
  *
  * Http Request Parameters:
  * -----------------------
  * No request parameter names/values are used on an initial call to this servlet.
  * If an error occurs due to the field validation, the following parameter names 
  * are posted to this page by AddOneTimeTransferConfirmation servlet:
  *    1. phase - flag indicationg the redisplay of the input form to correct the
  *               the errors. This has a value - "REDISPLAY".
  *    2. memo - memo description entered on the input form page
  *    3. errMsg - error message describing the cause of the error
  *    4. transfer - oid of the GtAddVariableTransfer instance that was cached
  *                  on the confirm page
  *
  * If the user clicks the 'CANCEL' button on the confirm page (AddOneTimeTransferConfirmation), 
  * the following parameter names are posted on this page: 
  *    1. phase
  *    2. memo
  *    3. transfer
  */

import java.io.PrintWriter;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.Calendar;
import java.util.Date;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsTxn;
import com.ibm.ifs.gold.GsAccount;
import com.ibm.ifs.gold.auto.GtAddVariableTransfer;
import com.ibm.ifs.gold.auto.GtUserProfile;
import com.ibm.ifs.gold.auto.GsAcctDtl;
import com.ibm.ifs.servlets.IfsHttpServlet;

/*
 *
 * Servlet Class to display one-time transfer form information
 * 
 */
 public class AddOneTimeTransferForm extends IfsHttpServlet 
 {
  /*
   *  Use the flag to debug this servlet 
   */
   public static boolean debugOn = false;

  /*
   *  Define contants 
   */
   public static final int SOURCE_ACCOUNT = 0;
   public static final int TARGET_ACCOUNT = 1;

  /*
   *  doGet method
   */
   public void doGet (HttpServletRequest request, HttpServletResponse response)
	  throws ServletException, IOException
   {
    /*
     *
     * Declare and initialize local variables
     *
     */
     setDefaultHeader(response);
     PrintWriter           out        = response.getWriter();
     IfsSession            ifsSession = new IfsSession(request, response);
     GtUserProfile         up         = (GtUserProfile)getObject(ifsSession, "__UserProfile");
     IfsTxn                txn        = (IfsTxn)getRequestParameter(request,"txn"); 
     String                phase      = (String)getRequestParameter(request,"phase"); 
     Calendar              cal        = Calendar.getInstance();
     StringBuffer          date       = new StringBuffer();
     GtAddVariableTransfer transfer   = null;
     String                errMsg     = null;

     GsAccount             tfrToAcct  = null;
     GsAccount             tfrFromAcct= null;
     String                tfrAmount  = null;
     String                tfrDate    = null;
     String                tfrMemo    = null;

    try
    {
    /*
     * 
     * Construct today's date in USA format
     *
     */
     cal.setTime(new Date());
     date.append(Integer.toString(cal.get(Calendar.MONTH)+1));
     date.append("/");
     date.append(Integer.toString(cal.get(Calendar.DAY_OF_MONTH)));
     date.append("/");
     date.append(Integer.toString(cal.get(Calendar.YEAR)));

    /*
     *
     * Enter this condition if found field validation errors or 
     * clicked 'CANCEL' button on the confirm page.
     *
     */
     if(phase != null && phase.equals("REDISPLAY"))
     {
       transfer = (GtAddVariableTransfer)createInstance(ifsSession, 
							"com.ibm.ifs.gold.auto.GtAddVariableTransfer", 
							"transfer");
      /*
       *
       * Extract the data from the object to redisplay the user input. 
       *
       */
       tfrToAcct = transfer.getReqSource();
       tfrToAcct = transfer.getReqTarget();
       if(transfer.getReqTransactionAmount() != null)
       {
	  transfer.getReqTransactionAmount().getFormat().setFormatStyle(IfsSystem.CURRENCY_TEXT_STYLE);
	  transfer.getReqTransactionAmount().setDigits(transfer.getReqTransactionAmount().getDigits());
          tfrAmount = transfer.getReqTransactionAmount().toString();
       }
       else 
          tfrAmount = "";
       if(transfer.getReqEffectiveDate() != null)
          tfrDate   = transfer.getReqEffectiveDate().toString();
       else
          tfrDate = date.toString();
       tfrMemo  = (String)getRequestParameter(request,"memo");
       errMsg   = (String)getRequestParameter(request,"errMsg");
     }
     else
     {
       tfrDate   = date.toString();
       tfrAmount = "";
       tfrMemo   = "";
     }

    /*
     *
     * Do we have a message to display to the user
     *
     */
     if(txn != null && txn.getMessage() != null)
       errMsg = txn.getMessage();
     else if(errMsg == null)
       errMsg = "";
    }
    catch (Exception e)
    {
      StringBuffer strBuf = new StringBuffer("ErrorPage?Exception=");
      strBuf.append(URLEncoder.encode("Exception caught in doGet method of AddOneTimeTransferForm Servlet:"));
      strBuf.append(URLEncoder.encode(e.toString()));
      callPage(response,strBuf.toString());
    }
       
     out.println("<HTML>");
     out.println("<HEAD>");

    /*
     * 
     * Adds HTML comment of the copyright statement for this servlet 
     *
     */
     addCopyright(response);

    /*
     * 
     * Display page title
     *
     */
     out.println("<TITLE>");
     out.println("WebBank Interactive Banking: TRANSFER FUNDS");
     out.println("</TITLE>");

     out.println("<META NAME=\"Author\" VALUE=\"IBM Corporation\">");
     out.println("<META NAME=\"Application\" VALUE=\"Customizable Web Server Reference Pages\">");
     out.println("</HEAD>");
     out.println("<BODY BGCOLOR=\"#FFFFFF\" BACKGROUND=\"\">");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
     out.println("<TR><TD><IMG SRC=/images/3dlogo.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
     out.println("<BR>");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
     out.println("<TR><TD><IMG SRC=/images/3dbanner.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
     out.println("<BR>");

    /*
     * 
     * Display standard title and instructions
     *
     */
     out.println("<P><TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0><TR>");
     out.println("<TD VALIGN=BASELINE ALIGN=CENTER><FONT SIZE=+2 COLOR=#000000>TRANSFER WITHIN MY ACCOUNTS - ");
     out.println("</FONT><FONT SIZE=+2 COLOR=#000080>SCHEDULE A TRANSFER</FONT></TD>"); 
     out.println("</TR></TABLE>"); 
     out.println("<IMG SRC=/images/hr_2.gif WIDTH=100% HEIGHT=3 ALIGN=CENTER><BR>");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=3 CELLPADDING=2>");
     out.println("<TR>");
     out.println("<TD WIDTH=90% BGCOLOR=#8EB5F0>");

    /*
     * 
     * Display message if found any errors
     *
     */
     out.println(errMsg);

     out.println("&nbsp;</TD>");
     out.println("</TR></TABLE>");
     out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=2 CELLPADDING=0>");
     out.println("<TR><TD WIDTH=100% VALIGN=TOP><FONT>To schedule a transfer between accounts listed in your Account Summary, "); 
     out.println("please provide the following information and click on <B>Enter</B>.");
     out.println("</FONT></TD>");
     out.println("</TABLE>");
      
    /*
     * 
     * Create form for user input
     *
     */
     out.println("<FORM METHOD=POST ACTION=\"AddOneTimeTransferConfirmation\">");
     out.println("<INPUT TYPE=HIDDEN NAME=transfer VALUE=\""+getNewOid(ifsSession,"GtAddVariableTransfer")+"\">");
     out.println("<INPUT TYPE=HIDDEN NAME=transfer.reqTransferImmediateFlag VALUE=\"I\">");
     out.println("<INPUT TYPE=HIDDEN NAME=transfer.reqTransferDuplChkOvrdFlag VALUE=\"N\">");
     out.println("<TABLE BORDER=0 CELLSPACING=1 CELLPADDING=2 ALIGN=CENTER>");

     // Transfer From Account drop-down
     out.println("<TR ALIGN=CENTER>");
     out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Transfer From Account: </B></TD>");
     out.println("<TD ALIGN=LEFT>");
     out.println("<SELECT NAME=transfer.reqSource>");
     displayAcccounts(response,up,SOURCE_ACCOUNT,tfrFromAcct);
     out.println("</SELECT></TD></TR>");

     // Transfer To Account drop-down
     out.println("<TR ALIGN=CENTER>");
     out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Transfer To Account: </B></TD>");
     out.println("<TD ALIGN=LEFT>");
     out.println("<SELECT NAME=transfer.reqTarget>");
     displayAcccounts(response,up,TARGET_ACCOUNT,tfrToAcct);
     out.println("</SELECT></TD></TR>");

     // Transfer Amount
     out.println("<TR ALIGN=CENTER>");
     out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Amount: </B></TD>");
     out.println("<TD ALIGN=LEFT><INPUT ALIGN=right TYPE=text NAME=transfer.reqTransactionAmount ");
     out.println("SIZE=22 MAXLENGTH=21 VALUE=\""+tfrAmount+"\"></TD></TR>");

     // Transfer Frequency:One-Time
     out.println("<TR ALIGN=CENTER>");
     out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Transfer Frequency: </B></TD>");
     out.println("<TD ALIGN=LEFT><B>One-Time</B></TD></TR>");

     // Transfer Date
     out.println("<TR ALIGN=CENTER>");
     out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Transfer Date: </B></TD>");
     out.println("<TD ALIGN=LEFT><INPUT ALIGN=left TYPE=text NAME=transfer.reqEffectiveDate ");
     out.println("SIZE=11 MAXLENGTH=10 VALUE=\""+tfrDate+"\"> mm/dd/yyyy</TD>");
     out.println("</TR>");
      
     // Memo
     out.println("<TR ALIGN=CENTER>");
     out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Memo: </B></TD>");
     out.println("<TD ALIGN=LEFT><INPUT ALIGN=left TYPE=text NAME=reqMemo SIZE=41 MAXLENGTH=40 "+ "VALUE=\""+tfrMemo+"\" > Optional</TD>");
     out.println("</TR>");
     out.println("</TABLE>");
     
     out.println("<P>");
     out.println("<CENTER><TABLE><TR><TD>");
     out.println("<INPUT TYPE=Image NAME=enter SRC=/images/enter.gif ALIGN=center ALT=Enter BORDER=0 WIDTH=85 HEIGHT=25>");
     out.println("</TD></FORM>");
     out.println("</TR>");
     out.println("</TABLE></CENTER>");
      
    /*
     * 
     * Display Footer
     *
     */
     out.println("<P><IMG SRC=/images/hr_2.gif WIDTH=100% HEIGHT=3 ALIGN=BOTTOM>");
     out.println("<P ALIGN=CENTER><FONT SIZE=-2>");
     out.println("<A HREF=AccountSummary>ACCOUNTS</A> |   ");
     out.println("<A HREF=AddOneTimeTransferForm>TRANSFERS</A> | ");
     out.println("<A HREF=Logoff>EXIT</A></FONT></P>");
      
    /*
     * 
     * Done with the HTML
     *
     */
     out.println("</BODY>");
     out.println("</HTML>");
   }   

  /*
   *
   * We perform the same operations for POST request as done for GET requests, 
   * so this method just sends the request to the doGet method.
   *
   */
   public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
   {
      StringBuffer  errMsg      = null;
      try
      {
         doGet(request,response);
      }
      catch (Exception e)
      {
	 errMsg = new StringBuffer("ErrorPage?Exception=");
	 errMsg.append(URLEncoder.encode("Exception caught in doPost method when calling doGet method:"));
         errMsg.append(URLEncoder.encode(e.toString()));
         callPage(response,errMsg.toString());
      }
   }

  /*
   *
   * Display Transfer Source Accounts or Transfer Target Accounts
   *
   * type    - 0 for source accounts; 1 for target accounts
   * account - account to be selected in the drop-down list
   *           if account is null, then the first element in the list 
   *           is selected
   *
   */
   public void displayAcccounts(HttpServletResponse response,
			        GtUserProfile up,
				int type,
				GsAccount account) 
	       throws IOException
   {
     setDefaultHeader(response);
     PrintWriter out            = null;
     GsAcctDtl   acctDtl        = null;
     GsAccount   acct           = null;
     String      txnAuthType    = ""; 
     String      acctType       = "";
     String      acctNumber     = "";
     String      acctNickname   = "";
     boolean     isValidSource  = false;
     boolean     isValidTarget  = false;
     StringBuffer acctStr       = new StringBuffer();
     try {
       out = response.getWriter();
     }
     catch (IOException e){
       throw e;
     }

     for(int i = 0 ; i < up.getAccountDetail().length ; i++)
     {
       acctDtl      = up.getAccountDetail()[i];
       acct         = acctDtl.getAccount();
       acctType     = acct.getType();
       acctNumber   = acct.getNumber();
       acctNickname = acct.getNickname();
       txnAuthType  = acctDtl.getTransactionAuthorizationType();

       if(type == SOURCE_ACCOUNT) // source accounts
         isValidSource = (txnAuthType.equals("2") || // Transfer From Only
			  txnAuthType.equals("4") || // Transfers Only
			  txnAuthType.equals("5") || // Transfer From & Bill Pay
			  txnAuthType.equals("7"));  // Transfers & Bill Pay
       else if(type == TARGET_ACCOUNT) // target accounts
         isValidTarget = (txnAuthType.equals("3") || // Transfer To Only
		          txnAuthType.equals("4") || // Transfers Only
			  txnAuthType.equals("6") || // Transfer To & Bill Pay
			  txnAuthType.equals("7"));  // Transfers & Bill Pay

       if((type == SOURCE_ACCOUNT && isValidSource) || (type == TARGET_ACCOUNT && isValidTarget))
       {
	 acctStr = new StringBuffer();
	 acctStr.append("??type=");
	 acctStr.append(acctType);
	 acctStr.append("&number=");
	 acctStr.append(acctNumber);
	 acctStr.append("&nickname=");
	 acctStr.append(acctNickname);
	 if(account != null && 
	    account.getNickname().equals(acctNickname) && 
	    account.getType().equals(acctType) && 
	    account.getNumber().equals(acctNumber))
	 {
	   out.println("<OPTION SELECTED VALUE=\""+acctStr.toString()+"\">");
	 }
	 else
	 {
	   out.println("<OPTION VALUE=\""+acctStr.toString()+"\">");
	 }
         if(acctNickname != null || !acctNickname.equals(""))
	 {
	     out.println(acctNickname);
	     out.println(" ");
	     out.println(acctNumber);
	 }
         else
	 {
	     out.println(acctType);
	     out.println(" ");
	     out.println(acctNumber);
	 }
       }
     } // end for loop
   } // end displayAcccounts method
 }

