
/*
 * CustomMethods.java        
 *
 * Licensed Materials - Property of IBM 
 * 5648-A06              
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication 
 *  or disclosure restricted by GSA ADP Schedule Contract   
 *  with IBM                                               
 *
 */

 /* Description:
  * -----------
  * The class defines static methods that are registered by 
  * IfsObjects to perform special functionality.
  *
  */

import com.ibm.ifs.gold.auto.GtAddVariableTransfer;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.GsAccount;

public class CustomMethods
{
 /* 
  * Use this flag to debug the class 
  */
  public static boolean debugOn=false;
  
 /* 
  * 
  *  This is a validation method for GtAddVariableTransfer transaction.
  *  To use this method register using an IfsObject object with
  *  the GtAddVariableTransfer class by the name "validate".
  *  Then invoke the validate method using the GtAddVariableTransfer 
  *  instance object. 
  *  This methods validates following fields:
  *      a. reqSource and reqTarget - verifies whether the accounts are different
  *      b. reqTransactionAmount - verifies whether the amount is not null
  *      c. reqSource and reqTarget - verifies whether the date is not null
  */
  public static String validate(GtAddVariableTransfer avf)
  {
    StringBuffer strBuf = new StringBuffer("");
    GsAccount sourceAccount = avf.getReqSource();
    GsAccount targetAccount = avf.getReqTarget();
       
    try
    {
    /* 
     * 
     * Validate reqSource and reqTarget fields
     *
     */
     if ((sourceAccount.getNickname().equals(targetAccount.getNickname())) &&
	 (sourceAccount.getType().equals(targetAccount.getType())) &&
	 (sourceAccount.getNumber().equals(targetAccount.getNumber()))) 
     {
       strBuf.append("* The source and target accounts are equal");
     }

    /* 
     * 
     * Validate reqTransactionAmount field
     *
     */
     if(avf.getReqTransactionAmount() == null)
     {
       strBuf.append("<BR>* The transfer amount is empty");
     }

    /* 
     * 
     * Validate reqEffectiveDate field
     *
     */
     if(avf.getReqEffectiveDate() == null)
     {
       strBuf.append("<BR>* The transfer date is empty");
     }
    }
    catch(Exception e)
    {
      IfsSystem.debug("Exception occurred in validate(GtAddVariableTransfer) method of CustomMethods class");
    }

    if(strBuf.toString().equals("")) 
    {
      strBuf.append("OK");
    }
    return strBuf.toString();
   }
}

