
/*
 * ErrorPage.java
 *
 * Licensed Materials - Property of IBM
 * 5648-A06
 * (c) Copyright IBM Corporation, 1996-2000. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication
 *  or disclosure restricted by GSA ADP Schedule Contract
 *  with IBM
 *
 */

/*
 * Description:
 * -----------
 * This servlet implements general error display.
 * This servlet will be called whenever an
 * error occured during page navigation.
 *
 * Http Request Parameters:
 * -----------------------
 * Following request parameter names are used in this servlet:
 *   txn       - value posted from any page when there is an error executing the txn
 *   Exception - value posted from any page when an exeption occured.
 *
 */

import java.io.PrintWriter;
import java.io.IOException;
import java.net.URLEncoder;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsTxn;
import com.ibm.ifs.servlets.IfsHttpServlet;

/*
 *
 * Servlet Class to display errors
 *
 */
public class ErrorPage extends IfsHttpServlet
{
   /*
    *  Use the flag to debug this servlet
    */
   public static boolean debugOn = false;

   public void doGet (HttpServletRequest req, HttpServletResponse res)
   throws ServletException, IOException
   {
      /*
       *
       * Declare and initialize local variables
       *
       */
      setDefaultHeader(res);   // Need to set character set before getWriter()
      PrintWriter out        = res.getWriter();
      IfsSession  ifsSession = new IfsSession(req,res);
      IfsTxn      txn        = null;
      String      message    = null;

      IfsSystem.initIfs();

      out.println("<HTML>");
      out.println("<HEAD>");

      /*
       *
       * Adds HTML comment of the copyright statement for this servlet
       *
       */
      addCopyright(res);

      /*
       *
       * Display page title
       *
       */
      out.println("<TITLE>");
      out.println("WebBank Interactive Banking: ERROR APPLICATION");
      out.println("</TITLE>");
      out.println("</HEAD>");

      out.println("<BODY BGCOLOR=\"#ffffff\">");
      out.println("<BODY BGCOLOR=\"#FFFFFF\" BACKGROUND=\"\">");
      out.println("<TABLE WIDTH=\"100%\" BORDER=\"0\" CELLSPACING=\"0\" CELLPADDING=\"0\">");
      out.println("<TR>");
      out.println("<TD><IMG SRC=\"/images/3dlogo.gif\" ALIGN=\"TOP\"></TD>");
      out.println("</TR>");
      out.println("</TABLE>");
      out.println("<BR>");
      out.println("<TABLE WIDTH=\"100%\" BORDER=\"0\" CELLSPACING=\"0\" CELLPADDING=\"0\">");
      out.println("<TR>");
      out.println("<TD><IMG SRC=\"/images/3dbanner.gif\" ALIGN=\"TOP\"></TD>");
      out.println("</TR>");
      out.println("</TABLE>");
      out.println("<BR>");

      /*
       *
       * Display standard title and instructions
       *
       */
      out.println("<P><TABLE WIDTH='100%' BORDER='0' CELLSPACING='0' CELLPADDING='0'>");
      out.println("<TR>");
      out.println("<TD VALIGN='BASELINE' ALIGN='CENTER'><FONT SIZE=+2 COLOR='#000000'>Error - </FONT><FONT SIZE=+2 COLOR=#000080>Application</FONT></TD>");
      out.println("</TR>");
      out.println("</TABLE>");
      out.println("<IMG SRC='/images/hr_2.gif' WIDTH='100%' HEIGHT='3' ALIGN='CENTER'><BR>");
      out.println("<TABLE WIDTH='100%' BORDER='0' CELLSPACING='3' CELLPADDING='2'>");
      out.println("<TR>");
      out.println("<TD WIDTH='90%' BGCOLOR='#8EB5F0'>&nbsp;</TD>");
      out.println("</TR>");
      out.println("</TABLE>");
      out.println("<TABLE WIDTH='100%' BORDER='0' CELLSPACING='2' CELLPADDING='0'>");
      out.println("<TR>");
      out.println("<TD WIDTH='100%' VALIGN='TOP'><FONT>An error has occurred. The following information was provided:</FONT></TD>");
      out.println("</TR>");
      out.println("</TABLE>");
      out.println("<BR>");
      out.println("<HR>");
      out.println("<BR>");

      /*
       *
       * Process transaction data if available (otherwise we have exeception data).
       *
       */
      debug(res, "Inside ErrorPage");
      try
      {
         txn = (IfsTxn) getRequestParameter(req, "txn");
         if (txn != null)
         {
            out.println("<B>Transaction Error in:</B>" + txn.getClass().getName()   + "<BR>");
            out.println("<B>ErrorCode:</B>"            + txn.getErrorCode()         + "<BR>");
            out.println("<B>BRC:</B>"                  + txn.getBrc()               + "<BR>");
            out.println("<B>Status:</B>"               + txn.getStatus()            + "<BR>");
            out.println("<B>CompCode:</B>"             + txn.getCompCode()          + "<BR>");
            out.println("<B>ReasonCode:</B>"           + txn.getReasonCode()        + "<BR>");
            out.println("<B>ErrorFunctionName:</B>"    + txn.getErrorFunctionName() + "<BR>");
            out.println("<B>ErrorModuleName:</B>"      + txn.getErrorModuleName()   + "<BR>");
            out.println("<B>Message:</B>"              + txn.getMessage()           + "<BR>");
            out.println("<B>GmMessage:</B>"            + txn.getGmMessage()         + "<BR>");
            out.println("<B>SenseData:</B>"            + txn.getSenseData()         + "<BR>");
            out.println("<B>SenseDataLength:</B>"      + txn.getSenseDataLength()   + "<BR>");
         }
         else
         {
            message = (String) getRequestParameter(ifsSession, "Exception");
            if (message != null)
               out.println("Exception occurred, message received :-<BR>" + message);
         }
      }
      catch (Exception e)
      {
         out.println("Exception occurred in error application, message received :-" + e.toString());
      }
      out.println("<BR>");
      out.println("<HR>");
      out.println("<BR>");
      out.println("<P>If this problem persists, or if you are in doubt ");
      out.println("as to the status of your last transaction, you may wish to contact your ");
      out.println("customer service representative.</P>");
      out.println("<BR><BR>Click <A HREF=\"Logoff\">here</A> to exit.");

      out.println("</BODY>");
      out.println("</HTML>");
   }

   /*
    *
    * We perform the same operations for POST request as done for GET requests,
    * so this method just sends the request to the doGet method.
    *
    */
   public void doPost (HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
   {
      try
      {
         doGet(request,response);
      }
      catch (Exception e)
      {
         StringBuffer errMsg = new StringBuffer();
         errMsg.append("ErrorPage?Exception=");
         errMsg.append(URLEncoder.encode("Exception caught in doPost method when calling doGet method in ErrorPage Servlet, explanation:"));
         errMsg.append(URLEncoder.encode(e.toString()));
         callPage(response,errMsg.toString());
      }
   }
}
