/*
 * ProcessIfsLogon.java
 *
 * Licensed Materials - Property of IBM
 * 5648-A06
 * (c) Copyright IBM Corporation, 1996-2000. All rights reserved.
 * US Government Users Restricted Rights - Use, duplication
 * or disclosure restricted by GSA ADP Schedule Contract
 * with IBM
 *
 */

import java.io.IOException;
import java.net.URLEncoder;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.ifs.servlets.IfsHttpServlet;
import com.ibm.ifs.gold.auto.GtUserProfile;
import com.ibm.ifs.gold.GtOBOLogon;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsTxn;

public class ProcessIfsLogon extends IfsHttpServlet
{

   public static boolean debugOn = false;

   public void doPost (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
   {
      IfsSystem.initIfs();
      setDefaultHeader(res);
      IfsSession ifsSession = new IfsSession(req, res);
      StringBuffer strBuf = null;

      GtOBOLogon obologon = (GtOBOLogon) createInstance(ifsSession, "com.ibm.ifs.gold.GtOBOLogon", "obologon");

      // Set the CSR user id and password from what was passed to this page from the
      // CSR logon page.
      try
      {
         obologon.setReqUserID((String)getRequestParameter(req,"reqUserID"));
         String password = (String)getRequestParameter(req,"reqPassword");
         obologon.setReqPassword(password);
         obologon.setReqPasswordEncryptionType(IfsSystem.NO_ENCRYPTION);

         // Since there is no separate pin for CSR, use the password value for the
         // security pin.
         obologon.setReqSecurity(password);
         obologon.setReqSecurityEncryptionType(IfsSystem.NO_ENCRYPTION);
      }
      catch (Throwable e)
      {
         debug(res, "An Exception occured when setting the CSR user ID and password. Exception=" + e.toString());

         strBuf = new StringBuffer(IfsSystem.ErrorPage + "ErrorPage?Exception=");
         strBuf.append(URLEncoder.encode("An Exception occured when setting the CSR user ID and password. Exception: "));
         strBuf.append(URLEncoder.encode(e.toString()));
         callPage(res, strBuf.toString());
      }

      // Execute the GtOBOLogon transaction.
      try
      {
         obologon.execute();
      }
      catch (Exception e)
      {
         handleErrors(obologon,
                      IfsSystem.ErrorPage,
                      IfsSystem.ServerPath + "IfsLogon",   // Info page
                      IfsSystem.ServerPath + "IfsLogon");  // Redo page
      }

      // Check for successful CSR logon.
      if ( obologon.getBrc().equals(IfsSystem.SUCCESS) == false)
      {
         strBuf = new StringBuffer(IfsSystem.ServerPath + "IfsLogon?errmsg=");
         strBuf.append(URLEncoder.encode("Unable to logon, business response code is "));
         strBuf.append(URLEncoder.encode(obologon.getBrc()));
         strBuf.append(URLEncoder.encode(". Enter valid User ID, password, and try again."));
         callPage(res, strBuf.toString());
      }

      //Everything was successful, redirect to first displayable page.
      res.sendRedirect(IfsSystem.ServerPath + "IfsCustCare");
   }

}
