/*
 *  QIFDownLoadForm.java
 *
 * Licensed Materials - Property of IBM
 * 5648-A06
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication
 *  or disclosure restricted by GSA ADP Schedule Contract
 *  with IBM
 *
 *
 */


//--------------------------------------------------------------------------
//
// Function:
//
//    To receive data from the user for down loading of account information
//    in the Quicken (tm) Interchange format (QIF).
//
// Page Flow.
//    1. Fill in the information required on the form;
//          account - which account for the information
//          transaction type - credit, debit, all, etc
//          tranaction location - ATM, check, etc
//          specific date or range of dates
//          specific amount or range of amounts
//          specific check number of check number range
//    2. Hit enter to begin download.
//          If details are ok then the data will be downloaded else an error
//          message will be displayed and the form redisplayed for
//          another go
//
// Http Request Parameters:
//
//    No request parameter names/values are used on an initial call to this servlet.
//    If an error occurs due to the field validation, the following parameter names
//    are posted to this page by QIFConfirmation servlet:
//       1. phase - flag indicationg the redisplay of the input form to correct the
//                  the errors. This has a value - "REDISPLAY".
//       2. errMsg - error message describing the cause of the error
//       3. ah - object id of the GtAccountHistory instance that was cached
//                     on this confirm page.
//
//    If the user clicks the 'CANCEL' button on the confirm page (QIFConfirmation),
//    the following parameter names are posted on this page:
//       1. phase - flag indicationg the redisplay of the input form for changing of
//                  the input data. This has a value of "REDISPLAY".
//       2. ah - object id of the GtAccountHistory instance that was cached
//                     on the confirm page.
//--------------------------------------------------------------------------


import java.io.PrintWriter;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.Calendar;
import java.util.Date;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsTxn;
import com.ibm.ifs.gold.GsAccount;
import com.ibm.ifs.gold.GsDateFormat;
import com.ibm.ifs.gold.auto.GtAccountHistory;
import com.ibm.ifs.gold.auto.GtUserProfile;
import com.ibm.ifs.gold.auto.GsAcctDtl;
import com.ibm.ifs.servlets.IfsHttpServlet;


//#################################################
//             QIFDownLoadForm
//
// Servlet Class which displays the input form for downloading QIF information.
//#################################################
public class QIFDownLoadForm extends IfsHttpServlet
{
   // Set this flag to true for debug information to be outputted.
   public static boolean debugOn = false;

   //#################################################
   //             doGet
   //
   // Processes the HTTP GET method request.
   //#################################################
   public void doGet (HttpServletRequest request, HttpServletResponse response)
   throws ServletException, IOException
   {
      //----------------
      // Declare and initialize local variables
      //
      PrintWriter      out         = null;
      StringBuffer     strBuf      = null;
      IfsSession       ifsSession  = null;
      GtUserProfile    up          = null;
      GtAccountHistory ah          = null;

      String           errMsg      = null;
      String           ifsTxnMsg   = null;
      String           phase       = null;
      GsAccount        account     = null;

      String           transactionType     = "1";     //default
      String           transactionSource   = "";      //default
      String           transactionPeriod   = "";      //default

      String           reqLowAmount        = "";
      String           reqHighAmount       = "";
      String           reqPostingStartDate = "";
      String           reqPostingEndDate   = "";
      String           reqHighCheckNumber  = "";
      String           reqLowCheckNumber   = "";

      setDefaultHeader(response);

      try
      {
         out        = response.getWriter();
         ifsSession = new IfsSession(request, response);

         //---------------
         // Get a reference to the user profile object which was cached at logon.
         //
         up = (GtUserProfile) getObject(ifsSession, "__UserProfile");
         if ( up == null )
         {
            // Have no cached user profile.  Most likely cause is that the
            // servlet is being run prior to logon.
            strBuf = new StringBuffer("ErrorPage?Exception=");
            strBuf.append(URLEncoder.encode("-- User profile does not exist. Need to logon before running QIF download transaction."));
            callPage(response,strBuf.toString());
         }

         //---------------
         // Check if we are entering due to field validation errors or
         // clicking of 'CANCEL' button on the confirm page.
         //
         phase = (String)getRequestParameter(request,"phase");
         if (phase != null && phase.equals("REDISPLAY"))
         {
            //---------------
            // Need to redisplay the originally entered information.
            // Get a reference to the GtAccountHistory cached object.
            // The "ah" request parameter passes the object identifier (OID).
            //
            ah = (GtAccountHistory)
                 createInstance(ifsSession,
                                "com.ibm.ifs.gold.auto.GtAccountHistory",
                                "ah");

            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            account = ah.getReqAccount();
            transactionType =ah.getReqTransactionType();
            transactionSource =ah.getReqTransactionSource();
            transactionPeriod =ah.getReqProductType();

            // low amount
            if (ah.getReqLowAmount() != null)
            {
               ah.getReqLowAmount().getFormat().setFormatStyle(IfsSystem.CURRENCY_TEXT_STYLE);
               ah.getReqLowAmount().setDigits(ah.getReqLowAmount().getDigits());
               reqLowAmount = ah.getReqLowAmount().toString();

               if (reqLowAmount == null)
                  reqLowAmount = "";
            }

            // high amount
            if (ah.getReqHighAmount() != null)
            {
               ah.getReqHighAmount().getFormat().setFormatStyle(IfsSystem.CURRENCY_TEXT_STYLE);
               ah.getReqHighAmount().setDigits(ah.getReqHighAmount().getDigits());
               reqHighAmount = ah.getReqHighAmount().toString();

               if (reqHighAmount == null)
                  reqHighAmount = "";
            }

            // start date
            if (ah.getReqPostingStartDate() != null)
            {
               try
               {
                  // Need to specify toString pattern because the USA default is MM/dd/yy
                  // and a date of mm/dd/00 for year 2000 causes a number format exception.
                  reqPostingStartDate = ah.getReqPostingStartDate().toString(new GsDateFormat("MM/dd/yyyy"));
                  if (reqPostingStartDate == null)
                     reqPostingStartDate = "";
               }
               catch (Exception e)
               {
                  debug(ifsSession, "Exception getting Start Date - " + e.toString());
                  reqPostingStartDate = "";
               }
            }

            // end date
            if (ah.getReqPostingEndDate() != null)
            {
               try
               {
                  // Need to specify toString pattern because the USA default is MM/dd/yy
                  // and a date of mm/dd/00 for year 2000 causes a number format exception.
                  reqPostingEndDate = ah.getReqPostingEndDate().toString(new GsDateFormat("MM/dd/yyyy"));
                  if (reqPostingEndDate == null)
                     reqPostingEndDate = "";
               }
               catch (Exception e)
               {
                  debug(ifsSession, "Exception getting End Date - " + e.toString());
                  reqPostingEndDate = "";
               }
            }

            // low check number
            if (ah.getReqLowCheckNumber() != 0 )
               reqLowCheckNumber = Integer.toString (ah.getReqLowCheckNumber());

            // high check number
            if (ah.getReqHighCheckNumber() != 0 )
               reqHighCheckNumber = Integer.toString (ah.getReqHighCheckNumber());

            // message
            ifsTxnMsg = ah.getMessage();

            // validation error message
            errMsg = (String)getRequestParameter(request,"errMsg");

         } // end-if, for phase equals REDISPLAY

      }
      catch (Throwable e)
      {
         strBuf = new StringBuffer("ErrorPage?Exception=");
         strBuf.append(URLEncoder.encode("Exception caught in doGet method of QIFDownLoadForm Servlet:"));
         strBuf.append(URLEncoder.encode(e.toString()));
         callPage(response,strBuf.toString());
      }

      //---------------
      // Start creating the HTML data for this servlet and add a copyright statement.
      //
      out.println("<HTML> <HEAD>");
      addCopyright(response);

      //---------------
      // Create HTML page title data.
      //
      out.println("<TITLE>");
      out.println("WebBank Interactive Banking: FILE DOWNLOAD - SELECT INFORMATION FOR QIF DOWNLOAD");
      out.println("</TITLE>");
      out.println("<META NAME=\"Author\" VALUE=\"IBM Corporation\">");
      out.println("<META NAME=\"Application\" VALUE=\"Customizable Web Server Reference Pages\">");
      out.println("</HEAD>");

      //---------------
      // Start the page body.
      //
      out.println("<BODY BGCOLOR=\"#FFFFFF\" BACKGROUND=\"\">");
      out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
      out.println("<TR><TD><IMG SRC=/images/3dlogo.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
      out.println("<BR>");
      out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
      out.println("<TR><TD><IMG SRC=/images/3dbanner.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
      out.println("<BR>");

      //---------------
      // Display standard title and instructions
      //
      out.println("<P><TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
      out.println("<TR> <TD VALIGN=BASELINE ALIGN=CENTER><FONT SIZE=+2 COLOR=#000000>FILE DOWNLOAD - ");
      out.println("</FONT><FONT SIZE=+2 COLOR=#000080>SELECT INFORMATION FOR QIF DOWNLOAD</FONT></TD>");
      out.println("</TR></TABLE>");

      // When error messages exist, display them
      out.println("<IMG SRC=/images/hr_2.gif WIDTH=100% HEIGHT=3 ALIGN=CENTER><BR>");
      out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=3 CELLPADDING=2>");
      out.println("<TR>");
      out.println("<TD WIDTH=90% BGCOLOR=#8EB5F0>");
      if (ifsTxnMsg != null)
         out.println(ifsTxnMsg + "<BR><BR>");

      if (errMsg != null)
         out.println(errMsg);

      out.println("&nbsp;</TD> </TR></TABLE>");

      out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=2 CELLPADDING=0>");
      out.println("<TR> <TD WIDTH=100% VALIGN=TOP><FONT>To download account information in the Quicken Interchange Format (QIF), provide the Account and one or more of the following criteria, then click on <B>Enter</B>.");
      out.println("</FONT> </TD> </TR> </TABLE>");

      //---------------
      // Create form for user input
      //
      out.println("<FORM METHOD=POST ACTION=\"QIFConfirmation\">");
      out.println("<INPUT TYPE=HIDDEN NAME=ah VALUE=\"__GtAccountHistory\">");
      out.println("<TABLE BORDER=0 CELLSPACING=1 CELLPADDING=2 ALIGN=CENTER>");

      //  Account drop-down
      out.println("<TR ALIGN=CENTER>");
      out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Account: </B></TD>");
      out.println("<TD ALIGN=LEFT>");
      out.println("<SELECT NAME=\"ah.reqAccount\">");
      displayAcccounts(response, up, account, out);
      out.println("</SELECT></TD></TR>");

      //  transaction type drop-down
      out.println("<TR ALIGN=CENTER>");
      out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Transaction Type: </B></TD>");
      out.println("<TD ALIGN=LEFT>");
      out.println("<SELECT NAME=\"ah.reqTransactionType\">");
      displayQIFTransactionTypeSelectList( response, transactionType, out );
      out.println("</SELECT> </TD> </TR>");

      //  transaction source drop-down
      out.println("<TR ALIGN=CENTER>");
      out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Where Transaction Occurred: </B></TD>");
      out.println("<TD ALIGN=LEFT>");
      out.println("<SELECT NAME=\"ah.reqTransactionSource\">");
      displayQIFTransactionSourceSelectList( response, transactionSource, out );
      out.println("</SELECT> </TD> </TR>");

      //  transaction period drop-down
      out.println("<TR ALIGN=CENTER>");
      out.println("<TD HEIGHT=39 ALIGN=RIGHT NOWRAP><B> Time Frame: </B></TD>");
      out.println("<TD ALIGN=LEFT>");
      out.println("<SELECT NAME=\"ah.reqProductType\">");
      displayQIFTransactionPeriodSelectList( response, transactionPeriod, out );
      out.println("</SELECT> </TD> </TR>");

      //---------------
      // Create fields for non-select-list fields
      //
      out.println("<TR> <TD WIDTH='40%' ALIGN='RIGHT' NOWRAP>");
      out.println("<B>Date or Range:</B></TD>");
      out.println("<TD WIDTH='20%' ALIGN='LEFT' NOWRAP COLSPAN='1'>");
      out.println("<INPUT NAME='reqPostingStartDate' TYPE='text' MAXLENGTH='10' SIZE='11' VALUE=\""+reqPostingStartDate+"\" >");
      out.println("</TD>");
      out.println("<TD WIDTH='20%' ALIGN='RIGHT' NOWRAP COLSPAN='1'>");
      out.println("<B>To</B></TD>");
      out.println("<TD WIDTH='20%' ALIGN='LEFT' NOWRAP COLSPAN='1'>");
      out.println("<INPUT NAME='reqPostingEndDate' TYPE='text' MAXLENGTH='10' SIZE='11' VALUE=\""+reqPostingEndDate+"\" >");
      out.println("</TD> </TR>");

      out.println("<TR>");
      out.println("<TD WIDTH='40%' ALIGN='RIGHT' NOWRAP> &nbsp;</TD>");
      out.println("<TD WIDTH='20%' ALIGN='LEFT' NOWRAP COLSPAN='1'>mm/dd/yyyy</TD>");
      out.println("<TD WIDTH='20%' ALIGN='RIGHT' NOWRAP COLSPAN='1'></TD>");
      out.println("<TD WIDTH='20%' ALIGN='LEFT' NOWRAP COLSPAN='1'>mm/dd/yyyy</TD>");
      out.println("</TR>");

      out.println("<TR>");
      out.println("<TD WIDTH='40%' ALIGN='RIGHT' NOWRAP> <B>Amount or Range:</B></TD>");
      out.println("<TD WIDTH='20%' ALIGN='LEFT' NOWRAP COLSPAN='1'>");
      out.println("<INPUT NAME='reqLowAmount' TYPE='text' MAXLENGTH='10' SIZE='11' VALUE=\"" +
                  reqLowAmount + "\" > </TD>");
      out.println("<TD WIDTH='20%' ALIGN='RIGHT' NOWRAP COLSPAN='1'> <B>To</B></TD>");
      out.println("<TD WIDTH='20%' ALIGN='LEFT' NOWRAP COLSPAN='1'>");
      out.println("<INPUT NAME='reqHighAmount' TYPE='text' MAXLENGTH='10' SIZE='11' VALUE=\"" +
                  reqHighAmount + "\" > </TD>");
      out.println("</TR>");

      out.println("<TR>");
      out.println("<TD WIDTH='40%' ALIGN='RIGHT' NOWRAP> <B>Check Number or Range:</B></TD>");
      out.println("<TD WIDTH='20%' ALIGN='LEFT' NOWRAP COLSPAN='1'>");
      out.println("<INPUT NAME='reqLowCheckNumber' TYPE='text' MAXLENGTH='10' SIZE='11' VALUE=\"" +
                  reqLowCheckNumber + "\" ></TD>");
      out.println("<TD WIDTH='20%' ALIGN='RIGHT' NOWRAP> <B>To</B></TD>");
      out.println("<TD WIDTH='20%' ALIGN='LEFT' NOWRAP>");
      out.println("<INPUT NAME='reqHighCheckNumber' TYPE='text' MAXLENGTH='10' SIZE='11' VALUE=\"" +
                  reqHighCheckNumber + "\" ></TD>");
      out.println("</TR>");
      out.println("</TABLE>");

      out.println("<P> <CENTER> <TABLE> <TR> <TD>");
      out.println("<INPUT TYPE=Image NAME=enter SRC=/images/enter.gif ALIGN=center ALT=Enter BORDER=0 WIDTH=85 HEIGHT=25>");
      out.println("</TD> </TR> </TABLE> </CENTER>");

      out.println("</FORM>");

      //---------------
      // Display Footer
      //
      out.println("<P><IMG SRC=/images/hr_2.gif WIDTH=100% HEIGHT=3 ALIGN=BOTTOM>");
      out.println("<P ALIGN=CENTER><FONT SIZE=-2>");
      out.println("<A HREF=Logoff>EXIT</A></FONT></P>");

      //---------------
      // Done with the HTML
      //
      out.println("</BODY> </HTML>");

   } // end-of, doGet method


   //#################################################
   //             doPost
   //
   // We perform the same operations for POST request as for GET requests,
   // so this method just sends the request to the doGet method.
   //#################################################
   public void doPost(HttpServletRequest request, HttpServletResponse response)
   throws ServletException, IOException
   {
      StringBuffer  errMsg      = null;
      try
      {
         doGet(request,response);
      }
      catch (Exception e)
      {
         errMsg = new StringBuffer("ErrorPage?Exception=");
         errMsg.append(URLEncoder.encode("Exception caught in doPost method when calling doGet method:"));
         errMsg.append(URLEncoder.encode(e.toString()));
         callPage(response,errMsg.toString());
      }

   }  // end-of, doPost method


   //#################################################
   //             displayAcccounts
   //
   // Displays the list of available accounts within the user profile
   // in a pull-down select list.
   //
   // The account parameter contains the account to be initially selected.
   // If account is null, then the first element in the list is selected.
   //#################################################
   private void displayAcccounts(HttpServletResponse response,
                                 GtUserProfile up,
                                 GsAccount account,
                                 PrintWriter out)
   {
      GsAcctDtl   acctDtl        = null;
      GsAccount   acct           = null;
      String      acctType       = "";
      String      acctNumber     = "";
      String      acctNickname   = "";
      StringBuffer acctStr       = new StringBuffer();

      for (int i = 0 ; i < up.getAccountDetail().length ; i++)
      {
         acctDtl      = up.getAccountDetail()[i];
         acct         = acctDtl.getAccount();
         acctType     = acct.getType();
         acctNumber   = acct.getNumber();
         acctNickname = acct.getNickname();

         acctStr = new StringBuffer();
         acctStr.append("??type=");
         acctStr.append(acctType);
         acctStr.append("&number=");
         acctStr.append(acctNumber);
         acctStr.append("&nickname=");
         acctStr.append(acctNickname);

         if ( (account != null) &&
              (account.getNickname().equals(acctNickname)) &&
              (account.getType().equals(acctType)) &&
              (account.getNumber().equals(acctNumber)) )
            out.println("<OPTION SELECTED VALUE=\""+ acctStr.toString() + "\">");
         else
            out.println("<OPTION VALUE=\""+ acctStr.toString() + "\">");

         if (acctNickname != null || !acctNickname.equals(""))
         {
            out.println(acctNickname);
            out.println(" -- ");
            out.println(acctNumber);
         }
         else
         {
            out.println(acctType);
            out.println(" -- ");
            out.println(acctNumber);
         }
      } // end for loop

   } // end-of, displayAcccounts method


   //#################################################
   //             displayQIFTransactionTypeSelectList
   //
   // Displays the list of transaction types
   // in a pull-down select list.
   //
   // The transactionType parameter contains the type to be initially selected.
   // If transactionType is null, then the first element in the list is selected.
   //#################################################
   private void displayQIFTransactionTypeSelectList(HttpServletResponse response,
                                                    String transactionType,
                                                    PrintWriter out )
   {
      // Possible transaction Types
      String[] transactionTypeText = {"All types", "Debits only", "Credits only",
                                      "Withdrawals only", "Checks only", "Deposits only"};
      String[] transactionTypeCodes = { "1", "2", "3", "4", "5", "6"};

      for (int i = 0; i < transactionTypeText.length; i ++)
      {
         if ( (transactionType != null) &&
              (transactionType.equals(transactionTypeCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\"" + transactionTypeCodes[i] + "\">");
         else
            out.println("<OPTION VALUE=\"" + transactionTypeCodes[i] + "\">");

         out.println( transactionTypeText[i]);
      }

   }  // end-of, displayQIFTransactionTypeSelectList method



   //#################################################
   //             displayQIFTransactionTypeSelectList
   //
   // Displays the list of transaction sources
   // in a pull-down select list.
   //
   // The transactionSource parameter contains the source to be initially selected.
   // If transactionSource is null, then the first element in the list is selected.
   //#################################################
   private void displayQIFTransactionSourceSelectList(HttpServletResponse response,
                                                      String transactionSource,
                                                      PrintWriter out )
   {
      // Possible transaction sources
      String[] transactionSourcesText = {"All", "ATM", "Teller", "POS",
                                         "VRU", "Home Banking", "ACH"};
      String defaultValue = "";
      String[] transactionSourcesCodes = { defaultValue, "1", "2", "3", "4", "5", "6"};

      for (int i = 0; i < transactionSourcesText.length; i ++)
      {
         if ( (transactionSource != null) &&
              (transactionSource.equals(transactionSourcesCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\""+transactionSourcesCodes[i]+"\">");
         else
            out.println("<OPTION VALUE=\""+transactionSourcesCodes[i]+"\">");

         out.println( transactionSourcesText[i]);
      }

   } // end-of, displayQIFTransactionSourceSelectList method


   //#################################################
   //             displayQIFTransactionPeriodSelectList
   //
   // Displays the list of transaction periods
   // in a pull-down select list.
   //
   // The transactionPeriod parameter contains the period to be initially selected.
   // If transactionPeriod is null, then the first element in the list is selected.
   //#################################################
   private void displayQIFTransactionPeriodSelectList(HttpServletResponse response,
                                                      String transactionPeriod,
                                                      PrintWriter out )
   {
      // Possible transaction periods
      String[] transactionPeriodsText = {"Select a Time Frame or Date Range",
                                         "Since Last Statement",
                                         "Since Previous Statement",
                                         "Since Last Inquiry"};
      String defaultValue = "";
      String[] transactionPeriodsCodes = { defaultValue, "S", "I", "P"};

      for (int i = 0; i < transactionPeriodsText.length; i ++)
      {
         if ( (transactionPeriod != null) &&
              (transactionPeriod.equals(transactionPeriodsCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\"" + transactionPeriodsCodes[i] + "\">");
         else
            out.println("<OPTION VALUE=\"" + transactionPeriodsCodes[i] + "\">");

         out.println( transactionPeriodsText[i]);
      }

   } // end-of, displayQIFTransactionPeriodSelectList method

} // end-of, QIFDownLoadForm class

