/*
 * QIFDownLoadSuccess.java
 *
 * Licensed Materials - Property of IBM
 * 5648-A06
 * (c) Copyright IBM Corporation, 1996-2000. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication
 *  or disclosure restricted by GSA ADP Schedule Contract
 *  with IBM
 *
 */

// --------------------------------------------------------------------------
//
// Function:
//    Gets the account history information from the host system and formats
//    the data into the Quicken Interchange Format.
//
// Http Request Parameters:
//   The following request parameter name is posted to this servlet:
//      ah    - object ID of the cached GtAccountHistory object
// --------------------------------------------------------------------------

import java.io.IOException;
import java.io.PrintWriter;
import java.net.URLEncoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import com.ibm.ifs.servlets.IfsHttpServlet;
import com.ibm.ifs.gold.auto.GtAccountHistory;
import com.ibm.ifs.gold.auto.GsTxnRespDtl;
import com.ibm.ifs.gold.GsAccount;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;

//#################################################
//             QIFDownLoadSuccess
//
// This servlet class gets the account history information and formats
// the data into the Quicken Interchange Format.
//#################################################
public class QIFDownLoadSuccess extends IfsHttpServlet
{
   // Set this flag to true for debug information to be outputted.
   public static boolean debugOn = false;

   //#################################################
   //             doPost
   //
   // Processes the HTTP POST method request.
   //#################################################
   public void doPost (HttpServletRequest request, HttpServletResponse response)
   throws ServletException, IOException
   {
      //----------------
      // Set response to the QIF format.
      //
      // Need to set character set before getWriter()
      response.setContentType("application/qif; charset=" + IfsSystem.cwsBrowserCharSet);
      response.setHeader("Pragma","No-cache");
      response.setHeader("Cache-Control", "no-cache");
      response.setDateHeader("Expires",0);

      //----------------
      // Declare and initialize local variables
      //
      PrintWriter  out        = response.getWriter();
      int          i          = 0;
      IfsSession   ifsSession = new IfsSession(request, response);

      GtAccountHistory ah  = (GtAccountHistory)
                             createInstance(ifsSession,
                                            "com.ibm.ifs.gold.auto.GtAccountHistory",
                                            "ah");
      GsAccount        account    = null;

      GsTxnRespDtl[]  myTransactions = null;

      try
      {
         //------------
         // Execute GtAccountHistory transaction to obtain the account details
         //
         ah.setReqMaximumNumberOfRecords(100);       // Maximum Records
         ah.execute();
         handleErrors(ah, "ErrorPage", "QIFConfirmation", "QIFConfirmation");
      }
      catch (Exception e)
      {
         StringBuffer strBuf = new StringBuffer("ErrorPage?Exception=");
         strBuf.append(URLEncoder.encode("Exception caught in doPost method of QIFDownLoadSuccess Servlet:"));
         strBuf.append(URLEncoder.encode(e.toString()));
         callPage(response,strBuf.toString());
      }

      //------------
      // Build the Quicken Interchange Format (QIF) data
      //
      myTransactions = (GsTxnRespDtl[])ah.getTransaction();

      StringBuffer textOut = new StringBuffer();

      // Create the QIF header.
      String accountType = ah.getAccountIDDetail().getType();
      debug(response, "AccountType is " + accountType);
      if ((accountType.equals("DDA")) ||
          (accountType.equals("SDA")) ||
          (accountType.equals("CD"))  ||
          (accountType.equals("MMA")) ||
          (accountType.equals("IRA")) )
         textOut.append("!Type:Bank\n");

      else if ((accountType.equals("LOC")) ||
               (accountType.equals("ILA")) ||
               (accountType.equals("CLA")) ||
               (accountType.equals("MLA")) )
         textOut.append("!Type:Oth L\n");

      else if ( accountType.equals("CCA"))
         textOut.append("!Type:CCard\n");

      // Create the QIF information for each transaction in the returned history.
      for ( i = 0; i < myTransactions.length; i++ )
      {
         // date
         textOut.append("D");
         textOut.append(myTransactions[i].getOriginalDate().toString());
         textOut.append("\n");

         // transaction amount
         textOut.append("T");
         myTransactions[i].getTransactionAmount().getFormat().setFormatStyle(IfsSystem.CURRENCY_TEXT_STYLE);
         textOut.append(myTransactions[i].getTransactionAmount().toString());
         textOut.append("\n");

         // write the num field for quicken only if there are check numbers
         if ( myTransactions[i].getCheckID().length > 0 )
         {
            if ( myTransactions[i].getCheckID()[0].getNumber() > 0 )
            {
               textOut.append("N");
               textOut.append(Integer.toString(myTransactions[i].getCheckID()[0].getNumber()));
               textOut.append("\n");
            }
         }

         // transaction payee
         textOut.append("P");
         textOut.append(myTransactions[i].getDescription());
         textOut.append("\n");

         // end of entry
         textOut.append("^\n");

      } // end-for, for creation transaction QIF data.

      // Delete last new line from file as toString method adds a new line character to the
      // end of the string.  We don't need two of them.
      if (textOut.length() > 0) // DOB
         textOut.setLength(textOut.length() - 1);

      out.println(textOut.toString());

   } // end-of, doPost method


   //#################################################
   //             doGet
   //
   // We perform the same operations for Get request as for POST requests,
   // so this method just sends the request to the doPost method.
   //#################################################
   public void doGet(HttpServletRequest request, HttpServletResponse response)
   throws ServletException, IOException
   {
      try
      {
         doPost(request,response);
      }
      catch (Exception e)
      {
         StringBuffer strBuf = new StringBuffer("ErrorPage?Exception=");
         strBuf.append(URLEncoder.encode("Exception caught in doGet method of QIFDownLoadSuccess Servlet:"));
         strBuf.append(URLEncoder.encode(e.toString()));
         callPage(response,strBuf.toString());
      }
   } // end-of, doGet method

} // end-of, QIFDownLoadSuccess class

