//-*************************************
//
// FILE NAME:  IfsWpActDetails.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/14/2002     replaced 'getValue' call with 'getAttribute'
 *                        1/18/2002     added try-catch block
 *                                      added if statement for HandleError
 */                                     


import java.io.*;
import java.net.URLEncoder;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/14/02
//import com.ibm.ifs.resource.*;

/**
* This class displays the details of a specific account.
*/
public class IfsWpActDetails extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   //*******************************
   // Displays details of a specific account or a list of all of the user's accounts.
   //
   // The following Http Request Parameters are posted to this page:
   //     ReqTag       - Specifies whether a list of all accounts or the details
   //                    for a specific account is to be displayed.
   //                    It can have values of "AllAccounts" or "Details".
   //     AccountIndex - When ReqTag equals "Details", it specifies the index
   //                    of the account to be displayed.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);

      GtUserProfile up = null;

      try
      {
         // get user profile
         //Changed from getValue to getAttribute 1/14/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         String reqTag = (String)getRequestParameter(request,"ReqTag");

         if (reqTag.equals("AllAccounts"))
            showAllAccounts(up, request, response);

         else if (reqTag.equals("Details"))
            showDetails(up, ifsSession, request, response);

         else
            callPage_RedoPage(response, "IfsWpActDetails",
                   "Unable to get account details.  Unexpected ReqTag value. ReqTag = " + reqTag);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to get account details. Exception in doGet method of IfsWpActDetails Servlet, explanation: ");
      }
   }


   /**
   * Display details of certificate of deposit account
   */
   private void handleCDADetails(GtAccountDetail txn, GsCdaAcctDtl[] acctDtl,
                                 PrintWriter out)
                throws IOException
   {
      String strValue = "";

      // Balance and Fee Information
      out.println("<HR><B><DIV ALIGN=CENTER>Balance and Fee Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Ledger Balance:", acctDtl[0].getLedgerBalanceAmount());
      show2DivBlocks(out, "Available Balance:", acctDtl[0].getAvailableBalanceAmount());

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Date of Last Interest Payment:",
                     acctDtl[0].getLastInterestDate());

      show2DivBlocks(out, "Date of Next Interest Payment:",
                     acctDtl[0].getNextInterestDate());

      show2DivBlocks(out, "Interest Accrued, But Not Yet Paid:",
                     acctDtl[0].getDepositInterestAccruedNotPaid());

      // Interest Distribution Method
      try { strValue = acctDtl[0].getInterestDstMth_text(); }
      catch (Exception e) { strValue = "Unknown distribution method: " + acctDtl[0].getInterestDstMth(); }
      show2DivBlocks(out, "Interest Distribution Method:", strValue);

      show2DivBlocks(out, "Interest Distribution Amount:",
                     acctDtl[0].getInterestDstAmount());

      // Interest Distribution Frequency
      try { strValue = acctDtl[0].getFrequency_text(); }
      catch (Exception e) { strValue = "Unknown frequency: " + acctDtl[0].getFrequency(); }
      show2DivBlocks(out, "Interest Distribution Frequency:", strValue);

      show2DivBlocks(out, "Date of Next Interest Distribution:",
                     acctDtl[0].getNextInterestDstDate());

      show2DivBlocks(out, "Interest Distribution FI:", acctDtl[0].getLine1Name());

      show2DivBlocks(out, "Interest Distribution FI Routing/Transit:",
                     acctDtl[0].getDestFIiRoutingNumber());

      show2DivBlocks(out, "Interest Distribution Payee Name:",
                     acctDtl[0].getPayeeName());

      // output common Interest information
      handleCommonInterestAndAPY(txn, out);

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Transaction, Deposit and Maturity
      out.println("<HR><B><DIV ALIGN=CENTER>Transaction, Deposit and Maturity</DIV></B><HR><BR>");

      // Term
      try
      {
         strValue = "" + acctDtl[0].getTermValue();
         try { strValue += " " + acctDtl[0].getPeriodType_text(); }
         catch (IfsException e) { strValue += " Unknown period: " + acctDtl[0].getPeriodType(); }

         show2DivBlocks (out, "Term:", strValue);
      }
      catch (Exception e) {}

      show2DivBlocks(out, "Maturity Date:", acctDtl[0].getDepMatDate());

      show2DivBlocks(out, "Deposit Redemption Amount:",
                     acctDtl[0].getDepRedemptionAmount());

      show2DivBlocks(out, "Grace Period End Date:",
                     acctDtl[0].getGracePeriodEndDate());

      show2DivBlocks(out, "Original Deposit Amount:",
                     acctDtl[0].getOriginalDepositAmount());

      show2DivBlocks(out, "Original Deposit Date:",
                     acctDtl[0].getOriginalDepositDate());

      show2DivBlocks(out, "YTD Withholding Amount:",
                     acctDtl[0].getYTDWithholdingAmount());

      show2DivBlocks(out, "Last Year Withholding Amount:",
                     acctDtl[0].getLastYearWithholdingAmount());

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }


   /**
   * Display details of checking account
   */
   private void handleCheckingDetails(GtAccountDetail txn, GsChkAcctDtl[] acctDtl,
                                      PrintWriter out)
                throws IOException
   {
      String strValue = "";

      // Balance and Fee Information
      out.println("<HR><B><DIV ALIGN=CENTER>Balance and Fee Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Ledger Balance:", acctDtl[0].getLedgerBalanceAmount());
      show2DivBlocks(out, "Collected Balance:", acctDtl[0].getCollBalanceAmount());
      show2DivBlocks(out, "Available Balance:", acctDtl[0].getAvailableBalanceAmount());

      show2DivBlocks(out, "One Day Float:", acctDtl[0].getOneDayFloatAmount());

      show2DivBlocks(out, "Two or More Day Float:",
                     acctDtl[0].getTwoDayFloatAmount());

      show2DivBlocks(out, "Average Collected Balance for Cycle:",
                     acctDtl[0].getAvgCollBalanceAmount());

      show2DivBlocks(out, "Overdraft Balance:",
                     acctDtl[0].getOverDraftUsedBalanceAmount());

      show2DivBlocks(out, "Overdraft Limit Expiry Date:",
                     acctDtl[0].getOverDraftLimitExpDate());

      show2DivBlocks(out, "Minimum Balance to Avoid Fees:",
                     acctDtl[0].getMinBalanceToAvoidFeeAmount());

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Interest Accrued but Not Yet Paid:",
                     acctDtl[0].getDepInterestAccruedNotPaid());

      // output common Interest information
      handleCommonInterestAndAPY(txn, out);

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Transaction, Deposit and Maturity
      out.println("<HR><B><DIV ALIGN=CENTER>Transaction, Deposit and Maturity</DIV></B><HR><BR>");

      show2DivBlocks(out, "Number of Holds:", acctDtl[0].getHoldValue());
      show2DivBlocks(out, "Number of Stop Payments:", acctDtl[0].getStopPayment());

      show2DivBlocks(out, "Last Deposit Amount:",  acctDtl[0].getLastDepositAmount());
      show2DivBlocks(out, "Last Deposit Date:", acctDtl[0].getLastDepositDate());

      show2DivBlocks(out, "Original Deposit Amount:", acctDtl[0].getOriginalDepositAmount());
      show2DivBlocks(out, "Original Deposit Date:", acctDtl[0].getOriginalDepositDate());

      // output common transaction information
      handleCommonTransaction(txn, out);

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }


   /**
   * Display common balance and fee information
   */
   private void handleCommonBalanceAndFee(GtAccountDetail txn, PrintWriter out)
   {
      show2DivBlocks(out, "Last Statement Balance:", txn.getLastStatementBalanceAmount());
      show2DivBlocks(out, "Last Statement Date:", txn.getLastStatementDate());
   }


   /**
   * Display common interest and APY information
   */
   private void handleCommonInterestAndAPY(GtAccountDetail txn, PrintWriter out)
   {
      show2DivBlocks(out, "Interest Paid Year To Date:", txn.getYTDInterestAmount());
      show2DivBlocks(out, "Interest Paid Last Year:", txn.getLastYearInterestAmount());
      show2DivBlocks (out, "Interest Rate:", txn.getInterestRate());
   }

   /**
   * Display common signatures on account information
   */
   private void handleCommonSignatureAcct(GtAccountDetail txn, PrintWriter out)
                throws IOException
   {
      String strCode = "";
      String strValue = "";
      int intValue = 0;
      int len = -1;

      // Signatures on Account
      out.println("<HR><B><DIV ALIGN=CENTER>Signatures on Account</DIV></B><HR><BR>");

      // Number of Signatures Required:
      try { intValue = txn.getCosignatureDetail().length + 1; }
      catch (Exception e) { intValue = 0; }
      show2DivBlocks (out, "Number of Signatures Required:", intValue);

      // Account Priority
      try
      {
         strCode = txn.getAccountPriority();
         if (strCode.length() > 0)
         {
            strValue = (strCode.trim().equals("P")) ? "Primary" : "Related";
            show2DivBlocks (out, "Account Priority:", strValue);
         }
         else
         {
            len = txn.getCosignatureDetail().length;

            strValue = (len == 0) ? "Primary" : "Related";
            show2DivBlocks (out, "Account Priority:", strValue);
         }
      }
      catch (Exception e) {}
   }


   /**
   * Display common transaction information
   */
   private void handleCommonTransaction(GtAccountDetail txn, PrintWriter out)
   {
      show2DivBlocks(out, "Last Transaction Amount:", txn.getLastTransactionAmount());
      show2DivBlocks(out, "Last Transaction Date:", txn.getLastTransactionDate());
   }


   /**
   * Display common transaction authority information
   */
   private void handleCommonTxnAuthority(GtAccountDetail txn, PrintWriter out)
                throws IOException
   {
      String strCode = "";
      String strValue = "";

      // get transaction authority
      try  { strCode = txn.getAuthority(); }
      catch (Exception e)
      {
         return;
      }

      // Following transaction authority values are defined in
      // IfsTxnAuthClResource property file.
      //
      //   1 = Inquiry Only
      //   2 = Transfer From Only
      //   3 = Transfer To Only
      //   4 = Transfer Only
      //   5 = Transfer From & Bill Pay
      //   6 = Transfer To & Bill Pay
      //   7 = Transfer & Bill Pay
      //   8 = Bill Pay Only

      // Transaction Authority title
      out.println("<HR><B><DIV ALIGN=CENTER>Transaction Authority</DIV></B><HR><BR>");

      // Inquiry
      strValue = "No";
      if (strCode.trim().equals("1")) strValue = "Yes";

      show2DivBlocks (out, "Inquiry:", strValue);

      // Transfer To
      strValue = "No";
      if ((strCode.trim().equals("3")) || (strCode.trim().equals("4")) ||
          (strCode.trim().equals("6")) || (strCode.trim().equals("7")))
         strValue = "Yes";

      show2DivBlocks (out, "Transfer To:", strValue);

      // Transfer From
      strValue = "No";
      if ((strCode.trim().equals("2")) || (strCode.trim().equals("4")) ||
          (strCode.trim().equals("5")) || (strCode.trim().equals("7")))
         strValue = "Yes";

      show2DivBlocks (out, "Transfer From:", strValue);

      // Bill Payment
      strValue = "No";
      if ((strCode.trim().equals("5")) || (strCode.trim().equals("6")) ||
          (strCode.trim().equals("7")) || (strCode.trim().equals("8")))
         strValue = "Yes";

      show2DivBlocks (out, "Bill Payment:", strValue);
   }

   /**
   * Display details of credit card information
   */
   private void handleCreditCardDetails(GtAccountDetail txn, GsCcAcctDtl[] acctDtl,
                                        PrintWriter out)
                throws IOException
   {
      String strValue = "";

      // Payment and Balance Information
      out.println("<HR><B><DIV ALIGN=CENTER>Payment and Balance Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Payoff Amount:", acctDtl[0].getPayoffAmount());

      show2DivBlocks(out, "Payoff Expiration Date:",
                     acctDtl[0].getPayoffExpirationDate());

      show2DivBlocks(out, "Outstanding Balance:",
                     acctDtl[0].getOutstandingBalanceAmount());

      show2DivBlocks(out, "Minimum Payment Due Amount:",
                     acctDtl[0].getMinimumPaymentAmount());

      show2DivBlocks(out, "Last Payment Amount:", acctDtl[0].getLastPaymentAmount());

      show2DivBlocks(out, "Last Payment Date:", acctDtl[0].getLastPaymentDate());

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      // output common Interest and APR information
      handleCommonInterestAndAPY(txn, out);

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Past Due Information
      out.println("<HR><B><DIV ALIGN=CENTER>Past Due Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Amount Past Due:", acctDtl[0].getPastDueAmount());

      show2DivBlocks(out, "Number of Payments Past Due:",
                     acctDtl[0].getNumberPaymentsPastDue());

      show2DivBlocks(out, "Past Due Finance Charge:",
                     acctDtl[0].getPastDueChargeAmount());

      // Credit Line Information
      out.println("<HR><B><DIV ALIGN=CENTER>Credit Line Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Original Line of Credit Amount:",
                     acctDtl[0].getOriginalLineOfCreditAmount());

      show2DivBlocks(out, "Original Line of Credit Date:",
                     acctDtl[0].getOriginalLineOfCreditDate());

      show2DivBlocks(out, "Available Credit:", acctDtl[0].getAvailableBalanceAmount());

      show2DivBlocks(out, "Credit Limit Amount:", acctDtl[0].getCreditLimitAmount());
      show2DivBlocks(out, "Overdraft Limit Amount:", acctDtl[0].getOverdraftLimitAmount());

      show2DivBlocks(out, "Payment Due Date:", acctDtl[0].getPaymentDueDate());
      show2DivBlocks(out, "Cash Advance Limit:",  acctDtl[0].getCashAdvanceLimit());

      show2DivBlocks(out, "Available Cash Advance:",
                     acctDtl[0].getCashAdvanceAvailable());

      show2DivBlocks(out, "Outstanding Cash Advance Amount:",
                     acctDtl[0].getOutstandingCashAdvanceAmount());

      show2DivBlocks(out, "Expiration Date:",  acctDtl[0].getExpirationDate());

      // Account Information
      out.println("<HR><B><DIV ALIGN=CENTER>Account Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "YTD Finance Charge:", acctDtl[0].getYTDFinanceChargeAmount());

      show2DivBlocks(out, "Last Year Finance Charge:",
                     acctDtl[0].getLastYearInterestAmount());

      show2DivBlocks(out, "Number of Days in Billing Cycle:",
                     acctDtl[0].getNumDaysBillingCycle());

      show2DivBlocks(out, "Bonus Redemption:", acctDtl[0].getBonusRedemption());

      show2DivBlocks(out, "Bonus Credit:", acctDtl[0].getBonusCredits());

      // output common transaction information
      handleCommonTransaction(txn, out);

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }

   /**
   * Display details of equity line accounts
   */
   private void handleEquityLineDetails(GtAccountDetail txn, GsElcAcctDtl[] acctDtl,
                                        PrintWriter out)
                throws IOException
   {
      String strValue = "";

      // Payment and Balance Information
      out.println("<HR><B><DIV ALIGN=CENTER>Payment and Balance Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Payoff Amount:", acctDtl[0].getPayoffAmount());

      show2DivBlocks(out, "Payoff Expiration Date:",
                     acctDtl[0].getPayoffExpirationDate());

      show2DivBlocks(out, "Outstanding Balance:",
                     acctDtl[0].getOutstandingBalanceAmount());

      show2DivBlocks(out, "Outstanding Balance as of Date:",
                     acctDtl[0].getOutstandingBalanceDate());

      show2DivBlocks(out, "Current Balance:", acctDtl[0].getCurrentBalanceAmount());

      show2DivBlocks(out, "Minimum Payment Due Amount:",
                     acctDtl[0].getMinimumPaymentAmount());

      show2DivBlocks(out, "Minimum Payment Due Date:",
                     acctDtl[0].getPaymentDueDate());

      show2DivBlocks(out, "Last Payment Amount:", acctDtl[0].getLastPaymentAmount());
      show2DivBlocks(out, "Last Payment Date:", acctDtl[0].getLastPaymentDate());

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      // output common Interest and APR information
      handleCommonInterestAndAPY(txn, out);

      show2DivBlocks(out, "Interest Paid Included in Last Payment:",
                     acctDtl[0].getInterestPaidLastPaymentAmount());

      show2DivBlocks(out, "Index:", acctDtl[0].getIndex());

      show2DivBlocks(out, "Spread:", acctDtl[0].getSpread());

      show2DivBlocks(out, "Last Rate Change Date:", acctDtl[0].getLastRateChangeDate());
      show2DivBlocks(out, "Next Rate Change Date:", acctDtl[0].getNextRateChangeDate());

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Past Due Information
      out.println("<HR><B><DIV ALIGN=CENTER>Past Due Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Amount Past Due:",  acctDtl[0].getPastDueAmount());

      show2DivBlocks(out, "Number of Payments Past Due:",
                     acctDtl[0].getNumberPaymentsPastDue());

      show2DivBlocks(out, "Past Due Finance Charge:",
                     acctDtl[0].getPastDueFinanceChargeAmount());

      // Credit Line Information
      out.println("<HR><B><DIV ALIGN=CENTER>Credit Line Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Original Line of Credit Amount:",
                     acctDtl[0].getOriginalLOCAmount());

      show2DivBlocks(out, "Original Line of Credit Date:",
                     acctDtl[0].getOriginalLOCDate());

      show2DivBlocks(out, "Amount Over Limit:", acctDtl[0].getOverLimitAmount());

      show2DivBlocks(out, "Credit Line Expires:", acctDtl[0].getCreditLineExpDate());

      show2DivBlocks(out, "ELC Available Credit:",
                     acctDtl[0].getCreditAvailableAmount());

      show2DivBlocks(out, "Credit Line Term Date:",
                     acctDtl[0].getCreditLineTrmDate());

      // Account Information
      out.println("<HR><B><DIV ALIGN=CENTER>Account Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Finance Charge:",
                     acctDtl[0].getFinanceChargeAmount());

      // output common transaction information
      handleCommonTransaction(txn, out);

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }

   /**
   * Display details of individual retirement account
   */
   private void handleIraDetails(GtAccountDetail txn, GsIraAcctDtl[] acctDtl,
                                 PrintWriter out)
                throws IOException
   {
      String strValue = "";

      // Balance and Fee Information
      out.println("<HR><B><DIV ALIGN=CENTER>Balance and Fee Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Ledger Balance:", acctDtl[0].getLedgerBalanceAmount());

      show2DivBlocks(out, "Available Balance:",
                     acctDtl[0].getAvailableBalanceAmount());

      show2DivBlocks(out, "Principal Distribution Amount:", acctDtl[0].getDistAmount());

      // Principal Distribution Frequency
      try { strValue = acctDtl[0].getFrequency_text(); }
      catch (Exception e) { strValue = "Unknown frequency: " + acctDtl[0].getFrequency(); }
      show2DivBlocks (out, "Principal Distribution Frequency:", strValue);

      show2DivBlocks(out, "Date of Next Principal Distribution:",
                     acctDtl[0].getNextDistDate());

      show2DivBlocks(out, "Principal Distribution FI:", acctDtl[0].getFINumberID());

      // Principal Distribution Account Type
      try { strValue = acctDtl[0].getAccountID().getType_text(); }
      catch (Exception e) { strValue = "Unknown account type: " + acctDtl[0].getAccountID().getType(); }
      show2DivBlocks(out, "Principal Distribution Account Type:", strValue);

      show2DivBlocks(out, "Principal Distribution Account Number:",
                     acctDtl[0].getAccountID());

      show2DivBlocks(out, "Principal Distribution Payee Name:",
                     acctDtl[0].getPayeeName());

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Interest Accrued but Not Yet Paid:",
                     acctDtl[0].getDepositInterestNotPaid());

      // Interest Distribution Method text
      try { strValue = acctDtl[0].getIntDistMethod_text(); }
      catch (Exception e) { strValue = "Unknown Distribution Method: " + acctDtl[0].getIntDistMethod(); }
      show2DivBlocks(out, "Interest Distribution Method:", strValue);

      // output common Interest information
      handleCommonInterestAndAPY(txn, out);

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Transaction, Deposit and Maturity
      out.println("<HR><B><DIV ALIGN=CENTER>Transaction, Deposit and Maturity</DIV></B><HR><BR>");

      show2DivBlocks(out, "Last Deposit Amount:",  acctDtl[0].getLastDepositAmount());
      show2DivBlocks(out, "Last Deposit Date:",  acctDtl[0].getLastDepositDate());

      show2DivBlocks(out, "Original Deposit Amount:",
                     acctDtl[0].getOriginalDepositAmount());

      show2DivBlocks(out, "Original Deposit Date:",
                     acctDtl[0].getOriginalDepositDate());

      // Term
      try
      {
         strValue = "" + acctDtl[0].getTermValue();
         try { strValue += " " + acctDtl[0].getTermPeriodType_text(); }
         catch (IfsException e) { strValue += " Unknown period: " + acctDtl[0].getTermPeriodType(); }

         show2DivBlocks (out, "Term:", strValue);
      }
      catch (Exception e) {}

      show2DivBlocks(out, "Redemption Amount:",  acctDtl[0].getRedemptionAmount());

      show2DivBlocks(out, "Maturity Date:", acctDtl[0].getDepositMaturityDate());

      show2DivBlocks(out, "Consumer First Maturity Date:",
                     acctDtl[0].getConsumerFirstMatDate());

      show2DivBlocks(out, "Consumer Second Maturity Date:",
                     acctDtl[0].getConsumerSecMatDate());

      show2DivBlocks(out, "YTD Contribution Amount:",
                     acctDtl[0].getCurrentYearContribution());

      show2DivBlocks(out, "Last Year Contribution Amount:",
                     acctDtl[0].getLastYearContributionAmount());

      show2DivBlocks(out, "YTD Withholding Amount:",
                     acctDtl[0].getYTDWithholdingAmount());

      show2DivBlocks(out, "Last Year Withholding Amount:",
                     acctDtl[0].getLastYearWithholdingAmount());

      // Life Expectancy Calculation Method
      try
      {
         strValue = (acctDtl[0].getLifeExpectCalcMethod() == 1)
                    ? "Straight line" : "Recalculation";
         show2DivBlocks (out, "Life Expectancy Calculation Method:", strValue);
      }
      catch (Exception e) {}

      show2DivBlocks(out, "Life Expectancy Distribution Amount:",
                     acctDtl[0].getLifeExpectDistAmount());

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }

   /**
   * Display details of line of credit account
   */
   private void handleLineOfCreditDetails(GtAccountDetail txn, GsLocAcctDtl[] acctDtl,
                                          PrintWriter out)
                throws IOException
   {
      String strValue = "";

      // Payment and Balance Information
      out.println("<HR><B><DIV ALIGN=CENTER>Payment and Balance Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Payoff Amount:", acctDtl[0].getPayoffAmount());

      show2DivBlocks(out, "Payoff Expiration Date:",
                     acctDtl[0].getPayoffExpirationDate());

      show2DivBlocks(out, "Payment Due Date:",  acctDtl[0].getPaymentDueDate());

      show2DivBlocks(out, "Principal Balance:",
                     acctDtl[0].getPrincipalBalanceAmount());

      show2DivBlocks(out, "Available Balance:",
                     acctDtl[0].getCreditAvailableAmount());

      show2DivBlocks(out, "Minimum Payment Due Amount:",
                     acctDtl[0].getMinimumPaymentAmount());

      show2DivBlocks(out, "Last Payment Amount:",
                     acctDtl[0].getLastPaymentAmount());

      show2DivBlocks(out, "Last Payment Date:",
                     acctDtl[0].getLastPaymentDate());

      show2DivBlocks(out, "Last Advance Payment Amount:",
                     acctDtl[0].getLastAdvancePaymentAmount());

      show2DivBlocks(out, "Last Advance Payment Date:",
                     acctDtl[0].getLastAdvancePaymentDate());

      show2DivBlocks(out, "Number of Payments Completed:",
                     acctDtl[0].getNumberPaymentsDone());

      // Payment Frequency
      try { strValue = acctDtl[0].getFrequency_text(); }
      catch (Exception e) { strValue = "Unknown frequency: " + acctDtl[0].getFrequency(); }
      show2DivBlocks (out, "Payment Frequency:", strValue);

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      // output common Interest and APR information
      handleCommonInterestAndAPY(txn, out);

      show2DivBlocks(out, "Index:", acctDtl[0].getIndex());

      show2DivBlocks(out, "Spread:", acctDtl[0].getSpread());

      show2DivBlocks(out, "Last Rate Change Date:",
                     acctDtl[0].getLastRateChangeDate());

      show2DivBlocks(out, "Next Rate Change Date:",
                     acctDtl[0].getNextRateChangeDate());

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Past Due Information
      out.println("<HR><B><DIV ALIGN=CENTER>Past Due Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Amount Past Due:", acctDtl[0].getPastDueAmount());

      show2DivBlocks(out, "Number of Payments Past Due:",
                     acctDtl[0].getNumberPaymentsPastDue());

      show2DivBlocks(out, "Past Due Finance Charge:",
                     acctDtl[0].getPastDueChargeAmount());

      // Credit Line Information
      out.println("<HR><B><DIV ALIGN=CENTER>Credit Line Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Purpose Description:",
                     acctDtl[0].getLineOfCreditPurposeDescription());

      show2DivBlocks(out, "Collateral Description:",
                     acctDtl[0].getLineOfCreditCollateralDescription());

      show2DivBlocks(out, "Original Line of Credit Amount:",
                     acctDtl[0].getOriginalLineOfCreditAmount());

      show2DivBlocks(out, "Original Line of Credit Date:",
                     acctDtl[0].getOriginalLineOfCreditDate());

      show2DivBlocks(out, "Amount Over Limit:", acctDtl[0].getOverLimitAmount());

      // Line Term
      try
      {
         strValue = "" + acctDtl[0].getTermValue();
         try { strValue += " " + acctDtl[0].getTermPeriodType_text(); }
         catch (Exception e) { strValue += " Unknown period: " + acctDtl[0].getTermPeriodType(); }

         show2DivBlocks (out, "Line Term:", strValue);
      }
      catch (Exception e) {}

      show2DivBlocks(out, "Credit Line Expires:", acctDtl[0].getExpirationDate());

      // Account Information
      out.println("<HR><B><DIV ALIGN=CENTER>Account Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "YTD Finance Charge:",
                     acctDtl[0].getYTDFinanceChargeAmount());

      show2DivBlocks(out, "Last Year Finance Charge:",
                     acctDtl[0].getLastYearFinanceChargeAmount());

      // output common transaction information
      handleCommonTransaction(txn, out);

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }

   /**
   * Display details of loan account
   */
   private void handleLoanDetails(GtAccountDetail txn, GsLoanAcctDtl[] acctDtl,
                                  PrintWriter out)
                throws ServletException, IOException
   {
      String strValue = "";

      // Payment and Balance Information
      out.println("<HR><B><DIV ALIGN=CENTER>Payment and Balance Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Payoff Amount:", acctDtl[0].getPayoffAmount());

      show2DivBlocks(out, "Payoff Expiration Date:",
                     acctDtl[0].getPayoffExpirationDate());

      show2DivBlocks(out, "Principal Balance:", acctDtl[0].getPrincipalBalanceAmount());

      show2DivBlocks(out, "Amount of Next Payment:",
                     acctDtl[0].getNextPaymentAmount());

      show2DivBlocks(out, "Next Payment Due Date:",
                     acctDtl[0].getPaymentDueDate());

      show2DivBlocks(out, "Last Payment Amount:", acctDtl[0].getLastPaymentAmount());
      show2DivBlocks(out, "Last Payment Date:", acctDtl[0].getLastPaymentDate());

      show2DivBlocks(out, "Number of Payments Completed:",
                     acctDtl[0].getNumberPaymentsDone());

      show2DivBlocks(out, "Number of Payments Remaining:",
                     acctDtl[0].getNumberPaymentsRemaining());

      show2DivBlocks(out, "Maturity / Final Payment Date:",
                     acctDtl[0].getMaturityDate());

      show2DivBlocks(out, "Prepayment Penalty Amount:",
                     acctDtl[0].getPrepaymentPenaltyAmount());

      // Payment Frequency
      try { strValue = acctDtl[0].getFrequency_text(); }
      catch (Exception e) { strValue = "Unknown frequency: " + acctDtl[0].getFrequency(); }
      show2DivBlocks (out, "Payment Frequency:", strValue);

      show2DivBlocks (out, "Outstanding Cash Advance Balance:",
                     acctDtl[0].getOutstandingCashAdvanceAmount());

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      // output common Interest information
      handleCommonInterestAndAPY(txn, out);

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Past Due Information
      out.println("<HR><B><DIV ALIGN=CENTER>Past Due Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Amount Past Due:", acctDtl[0].getPastDueAmount());

      show2DivBlocks(out, "Number of Payments Past Due:",
                     acctDtl[0].getNumberPaymentsPastDue());

      show2DivBlocks(out, "Past Due Finance Charge:",
                     acctDtl[0].getPastDueChargeAmount());

      // Loan Origination Information
      out.println("<HR><B><DIV ALIGN=CENTER>Loan Origination Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Loan Purpose Description:", acctDtl[0].getLoanPurpose());
      show2DivBlocks(out, "Collateral Description:", acctDtl[0].getLoanCollateral());

      show2DivBlocks(out, "Original Loan Amount:", acctDtl[0].getOriginalLoanAmount());
      show2DivBlocks(out, "Original Loan Date:",  acctDtl[0].getOriginalLoanDate());

      // Term
      try
      {
         strValue = "" + acctDtl[0].getTermValue();
         try { strValue += " " + acctDtl[0].getTermPeriodType_text(); }
         catch (IfsException e) { strValue += " Unknown period: " + acctDtl[0].getTermPeriodType(); }

         show2DivBlocks (out, "Term:", strValue);
      }
      catch (Exception e) {}

      // Account Information
      out.println("<HR><B><DIV ALIGN=CENTER>Account Information</DIV></B><HR><BR>");

      show2DivBlocks (out, "YTD Finance Charge:", acctDtl[0].getYTDFinanceChargeAmount());

      show2DivBlocks (out, "Last Year Finance Charge:",
                     acctDtl[0].getLastYearFinanceCharge());

      // output common transaction information
      handleCommonTransaction(txn, out);

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }

   /**
   * Display details of money market account
   */
   private void handleMoneyMarketDetails(GtAccountDetail txn, GsMmaAcctDtl[] acctDtl,
                                         PrintWriter out)
                throws ServletException, IOException
   {
      String strValue = "";

      // Balance and Fee Information
      out.println("<HR><B><DIV ALIGN=CENTER>Balance and Fee Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Ledger Balance:", acctDtl[0].getLedgerBalanceAmount());

      show2DivBlocks(out, "Available Balance:",
                     acctDtl[0].getAvailableBalanceAmount());

      show2DivBlocks(out, "Average Collected Balance for Cycle:",
                     acctDtl[0].getAverageCollectedBalanceAmount());

      show2DivBlocks(out, "Minimum Balance to Avoid Fees:",
                     acctDtl[0].getMinimumBalanceToAvoidFeeAmount());

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Interest Accrued but Not Yet Paid:",
                     acctDtl[0].getDepInterestAccruedNotPaid());

      // output common Interest information
      handleCommonInterestAndAPY(txn, out);

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Transaction, Deposit and Maturity
      out.println("<HR><B><DIV ALIGN=CENTER>Transaction, Deposit and Maturity</DIV></B><HR><BR>");

      show2DivBlocks(out, "Number of Checks Written:",
                     acctDtl[0].getNumberChecksWritten());

      show2DivBlocks (out, "Number of Checks Written As of Date:",
                     acctDtl[0].getNumberChecksWrittenDate());

      show2DivBlocks (out, "Minimum Value of Checks:",
                     acctDtl[0].getMinimumCheckValueAmount());

      show2DivBlocks (out, "Last Deposit Amount:", acctDtl[0].getLastDepositAmount());
      show2DivBlocks (out, "Last Deposit Date:", acctDtl[0].getLastDepositDate());

      show2DivBlocks (out, "Original Deposit Amount:",
                     acctDtl[0].getOriginalDepositAmount());

      show2DivBlocks (out, "Original Deposit Date:",
                      acctDtl[0].getOriginalDepositDate());

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }

   /**
   * Display details of mortgage account
   */
   private void handleMortgageDetails(GtAccountDetail txn, GsMortgageAcctDtl[] acctDtl,
                                      PrintWriter out)
                throws ServletException, IOException
   {
      String strValue = "";

      // Payment and Balance Information
      out.println("<HR><B><DIV ALIGN=CENTER>Payment and Balance Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Payoff Amount:", acctDtl[0].getPayoffAmount());

      show2DivBlocks(out, "Principal Balance:", acctDtl[0].getPrincipalBalanceAmount());

      show2DivBlocks(out, "Amount of Next Payment:",
                     acctDtl[0].getNextPaymentAmount());

      show2DivBlocks(out, "Next Payment Due Date:",
                     acctDtl[0].getNextPaymentDueDate());

      show2DivBlocks(out, "Last Payment Amount:",
                     acctDtl[0].getPreviousPaymentAmount());

      show2DivBlocks(out, "Last Payment Date:",
                     acctDtl[0].getPreviousPaymentDate());

      show2DivBlocks(out, "Number of Payments Completed:",
                     acctDtl[0].getNumberPaymentsComplete());

      show2DivBlocks(out, "Number of Payments Remaining:",
                     acctDtl[0].getNumberPaymentsRemaining());

      show2DivBlocks(out, "Prepayment Penalty Amount:",
                     acctDtl[0].getPrepaymentPenaltyAmount());

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      // output common Interest information
      handleCommonInterestAndAPY(txn, out);

      show2DivBlocks(out, "Index:", acctDtl[0].getIndex());

      show2DivBlocks(out, "Spread:", acctDtl[0].getSpread());

      show2DivBlocks(out, "Last Rate Change Date:",
                     acctDtl[0].getLastRateChangeDate());

      show2DivBlocks(out, "Next Rate Change Date:",
                     acctDtl[0].getNextRateChangeDate());

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Past Due Information
      out.println("<HR><B><DIV ALIGN=CENTER>Past Due Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Amount Past Due:", acctDtl[0].getPastDueAmount());

      show2DivBlocks(out, "Number of Payments Past Due:",
                     acctDtl[0].getNumberPaymentsPastDue());

      show2DivBlocks(out, "Past Due Finance Charge:",
                     acctDtl[0].getPastDueChargeAmount());

      // Loan Origination Information
      out.println("<HR><B><DIV ALIGN=CENTER>Loan Origination Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Original Loan Amount:", acctDtl[0].getOriginalLoanAmount());
      show2DivBlocks(out, "Original Loan Date:", acctDtl[0].getOriginalLoanDate());

      // Term
      try
      {
         strValue = "" + acctDtl[0].getTermValue();
         try { strValue += " " + acctDtl[0].getTermPeriodType_text(); }
         catch (IfsException e) { strValue += " Unknown period: " + acctDtl[0].getTermPeriodType(); }

         show2DivBlocks (out, "Term:", strValue);
      }
      catch (Exception e) {}

      // Escrow Information
      out.println("<HR><B><DIV ALIGN=CENTER>Escrow Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Escrow Balance:", acctDtl[0].getEscrowBalanceAmount());

      show2DivBlocks(out, "YTD Tax Paid Amount:",  acctDtl[0].getTaxPaidAmount());
      show2DivBlocks(out, "YTD Tax Paid Date:", acctDtl[0].getTaxPaidDate());

      show2DivBlocks(out, "Last Year Tax Paid Amount:",
                     acctDtl[0].getLastYearTaxPaidAmount());

      show2DivBlocks(out, "Last Year Tax Paid Date:",
                     acctDtl[0].getLastYearTaxPaidDate());

      show2DivBlocks(out, "YTD Insurance Paid Amount:", acctDtl[0].getInsurancePaidAmount());
      show2DivBlocks(out, "YTD Insurance Paid Date:", acctDtl[0].getInsurancePaidDate());

      show2DivBlocks(out, "Last Year Insurance Paid Amount:",
                     acctDtl[0].getLastYearInsurancePaidAmount());

      show2DivBlocks(out, "Last Year Insurance Paid Date:",
                     acctDtl[0].getLastYearInsurancePaidDate());

      show2DivBlocks(out, "Balloon Amount:", acctDtl[0].getBalloonAmount());
      show2DivBlocks(out, "Balloon Date:", acctDtl[0].getBalloonDate());

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }

   /**
   * Display details of saving account
   */
   private void handleSavingDetails(GtAccountDetail txn, GsDepAcctDtl[] acctDtl,
                                    PrintWriter out)
                throws ServletException, IOException
   {
      String strValue = "";

      // Balance and Fee Information
      out.println("<HR><B><DIV ALIGN=CENTER>Balance and Fee Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Ledger Balance:", acctDtl[0].getLedgerBalanceAmount());
      show2DivBlocks(out, "Available Balance:", acctDtl[0].getAvailableBalanceAmount());

      // output common balance and fee information
      handleCommonBalanceAndFee(txn, out);

      // Interest Information
      out.println("<HR><B><DIV ALIGN=CENTER>Interest Information</DIV></B><HR><BR>");

      show2DivBlocks(out, "Interest Accrued but Not Yet Paid:",
                     acctDtl[0].getInterestAccruedNotPaid());

      // output common Interest information
      handleCommonInterestAndAPY(txn, out);

      // output common transaction authority
      handleCommonTxnAuthority(txn, out);

      // Transaction, Deposit and Maturity
      out.println("<HR><B><DIV ALIGN=CENTER>Transaction, Deposit and Maturity</DIV></B><HR><BR>");

      show2DivBlocks(out, "Number of Holds:", acctDtl[0].getHoldValue());
      show2DivBlocks(out, "Number of Stop Payments:", acctDtl[0].getStopPaymentValue());

      show2DivBlocks(out, "Last Deposit Amount:", acctDtl[0].getLastDepositAmount());
      show2DivBlocks(out, "Last Deposit Date:", acctDtl[0].getLastDepositDate());

      show2DivBlocks(out, "Original Deposit Amount:", acctDtl[0].getOriginalDepositAmount());
      show2DivBlocks(out, "Original Deposit Date:", acctDtl[0].getOriginalDepositDate());

      // Term
      try
      {
         strValue = "" + acctDtl[0].getTermValue();
         try { strValue += " " + acctDtl[0].getTermPeriodType_text(); }
         catch (IfsException e) { strValue += " Unknown period: " + acctDtl[0].getTermPeriodType(); }

         show2DivBlocks (out, "Term:", strValue);
      }
      catch (Exception e) {}

      show2DivBlocks(out, "Redemption Amount:", acctDtl[0].getRedemptionAmount());

      show2DivBlocks(out, "Withholding Amount Year to Date:",
                     acctDtl[0].getYTDWithholdingAmount());

      show2DivBlocks(out, "Withholding Amount Last Year:",
                     acctDtl[0].getLastYearWithholdingAmount());

      show2DivBlocks(out, "Maturity Date:", acctDtl[0].getMaturityDate());

      // output common transaction information
      handleCommonTransaction(txn, out);

      // output common signatures on account
      handleCommonSignatureAcct(txn, out);
   }

   /**
   * Display a list of all accounts
   */
   private void showAllAccounts(GtUserProfile up,
                                HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GsAcctDtl acctDtl;
      String acctNbr;
      String acctNick;

      PrintWriter out = startHtmlOutput(response, "ACCOUNT DETAILS");

      out.println("<B>Please select an account:</B><HR>");

      // Display all the accounts
      for (int i = 0; i < up.getAccountDetail().length; i++)
      {
         try
         {
            acctDtl = up.getAccountDetail()[i];
            acctNbr = acctDtl.getAccount().getNumber();
            acctNick = acctDtl.getAccount().getNickname();
         }
         catch (Exception e) { continue; }

         // link detail
         out.println("<A href=\"IfsWpActDetails?AccountIndex=" +
                     String.valueOf(i) + "&ReqTag=Details\" >" +
                     acctNbr + " -- " + acctNick + "</A><BR>");
         }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Display details of selected account
   */
   private void showDetails(GtUserProfile up, IfsSession ifsSession,
                            HttpServletRequest request, HttpServletResponse response)
                throws Exception
   {
      String acctNbr = null;
      String type = null;
      String acctNick = null;
      GtAccountDetail txn = null;

      GsAcctDtl acct = null;
      int AccountIndex = 0;
      String strIndex = null;
      String strValue = "";
      boolean accountDetailsReturned = true;

      try
      {
         // get account index
         strIndex = (String)getRequestParameter(request,"AccountIndex");
         AccountIndex = Integer.parseInt(strIndex);

         // get account information
         acct = up.getAccountDetail()[AccountIndex];
         type = acct.getAccount().getType();
         acctNbr = acct.getAccount().getNumber();
         acctNick = acct.getAccount().getNickname();

         // create account
         GsAccount anAccount = new GsAccount(type, acctNbr, acctNick);
         anAccount.setIfsSession(ifsSession);

         // create details txn
         txn = new GtAccountDetail(ifsSession);
         txn.setReqAccountID(anAccount);

         // execute transaction
        //added try-catch block 1/18/2002
         try
        {
        
         txn.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         
         if(! handleErrors(txn, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpActDetails",
                      "IfsWpRedoPage?nextpage=IfsWpActDetails")){
             return;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to get account details. Exception in showDetails method of IfsWpActDetails Servlet, explanation: ");
      }

      // start html output
      PrintWriter out = startHtmlOutput(response, "ACCOUNT DETAILS");

      // account title
      out.println("<DIV ALIGN=CENTER><B>" + acctNick + "</B></DIV>");

      // status
      try { strValue = txn.getAccountStatus_text(); }
      catch (Exception e)
      {
         strValue = (txn.getAccountStatus() == null)
                    ? "Account status code is null"
                    : "Invalid Account status code of " + txn.getAccountStatus();
      }
      show2DivBlocks (out, "Status:", strValue);

      show2DivBlocks(out, "Information as of:", txn.getInfoDate());

      if (type.equals("DDA"))
      {
         // Checking Account
         if (txn.getCheckDetail().length == 0)
            accountDetailsReturned = false;
         else
            handleCheckingDetails (txn, txn.getCheckDetail(), out);
      }

      else if (type.equals("SDA"))
      {
         // Saving Account
         if (txn.getDepositDetail().length == 0)
            accountDetailsReturned = false;
         else
            handleSavingDetails (txn, txn.getDepositDetail(), out);
      }

      else if (type.equals("MLA"))
      {
         // Mortgage Loan Account
         if (txn.getMortgageDetail().length == 0)
            accountDetailsReturned = false;
         else
            handleMortgageDetails (txn, txn.getMortgageDetail(), out);
      }

      else if ( (type.equals("ILA")) || (type.equals("CLA")) )
      {
         // Loan Account, Installment Loan, Commercial Loan
         if (txn.getLoanDetail().length == 0)
            accountDetailsReturned = false;
         else
            handleLoanDetails (txn, txn.getLoanDetail(), out);
      }

      else if (type.equals("IRA"))
      {
         // Individual Retirement Account
         if (txn.getIraDetail().length == 0)
            accountDetailsReturned = false;
         else
            handleIraDetails (txn, txn.getIraDetail(), out);
      }

      else if (type.equals("LOC"))
      {
         // Line of Credit
         if (txn.getLineOfCreditDetail().length == 0)
            accountDetailsReturned = false;
         else
            handleLineOfCreditDetails (txn, txn.getLineOfCreditDetail(), out);
      }

      else if (type.equals("CCA"))
      {
         // Credit Card
         if (txn.getCreditCardDetail().length == 0)
            accountDetailsReturned = false;
         else
            handleCreditCardDetails (txn, txn.getCreditCardDetail(), out);
      }

      else if (type.equals("MMA"))
      {
         // Money Market Account
         if (txn.getMMADetail().length == 0)
            accountDetailsReturned = false;
         else
            handleMoneyMarketDetails (txn, txn.getMMADetail(), out);
      }

      else if (type.equals("CDA"))
      {
         // Certificate of Deposit Account
         if (txn.getCDADetail().length == 0)
            accountDetailsReturned = false;
         else
            handleCDADetails (txn, txn.getCDADetail(), out);
      }

      else if (type.equals("ELC"))
      {
         // Equity Line account
         if (txn.getELCDetail().length == 0)
            accountDetailsReturned = false;
         else
            handleEquityLineDetails (txn, txn.getELCDetail(), out);
      }

      else
      {
         out.println("<HR> Unknown account type: " + type);
      }

      if (accountDetailsReturned == false)
         out.println("<HR> Details for this account are not available. ");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

}
