//-*************************************
//
// FILE NAME:  IfsWpActSummary.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//-*************************************

             
/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/14/2002     replaced 'getValue' call with 'getAttribute'
 *
 */


import java.io.*;
import java.net.URLEncoder;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

/**
* This class displays a summary of all accounts including
* deposit accounts, credit accounts and loan accounts.
*/
public class IfsWpActSummary extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;

   private GsAcctDtl acct;
   private String acctNbr;
   private String type;
   private String type_text;

   //*******************************
   // Displays a summary of all the accounts for the user including
   // deposit accounts, credit accounts and loan accounts.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);

      GtUserProfile up = null;

      try
      {
         // get user profile
         //replaced getString with getAttribute 1/14/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         // start html output
         PrintWriter out = startHtmlOutput(response, "ACCOUNT SUMMARY");
         out.println ("<HR>");

         // Display the summary of the accounts.
         handleDepositAccounts (up, out);
         handleCreditAccounts (up, out);
         handleLoanAccounts (up, out);

         // link to other pages and close the print writer.
         endHtmlOutput(out);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
             "Account summary failed. Exception caught in doGet method of IfsWpActSummary Servlet, explanation: ");
      }
   }

   /**
   * Display account summary of credit accounts
   */
   private void handleCreditAccounts(GtUserProfile up, PrintWriter out)
                throws ServletException, IOException
   {
      int acctIndex = 0;
      boolean flagAccount = false;

      // credit accounts
      for (int i = 0; i < up.getAccountDetail().length; i++)
      {
         // set account variables
         try { setAccountInfo(up.getAccountDetail()[i]); }
         catch (Exception e)
         {
            acctIndex++;
            continue;
         }

         // credit accounts
         if (type.equals("CCA") || type.equals("LOC") || type.equals("ELC"))
         {
            if (!flagAccount)
               out.println("<DIV ALIGN=CENTER><B>CREDIT ACCOUNTS</B></DIV><HR><BR>");
            flagAccount = true;

            displayAccountInfo("Outstanding Balance:", acctIndex, out);
         }
         acctIndex++;
      }
   }

   /**
   * Display account summary of deposit account
   */
   private void handleDepositAccounts(GtUserProfile up, PrintWriter out)
                throws ServletException, IOException
   {
      String strValue = "";
      int acctIndex = 0;
      boolean flagAccount = false;

      // credit accounts
      for (int i = 0; i < up.getAccountDetail().length; i++)
      {
         // set account variables
         try { setAccountInfo(up.getAccountDetail()[i]); }
         catch (Exception e)
         {
            acctIndex++;
            continue;
         }

         // deposit accounts
         if (type.equals("CDA") || type.equals("DDA") || type.equals("IRA") ||
             type.equals("SDA") || type.equals("MMA"))
         {
            if (!flagAccount)
               out.println("<DIV ALIGN=CENTER><B>DEPOSIT ACCOUNTS</B></DIV><HR><BR>");
            flagAccount = true;

            displayAccountInfo("Ledger Balance:", acctIndex, out);
         }
         acctIndex++;
      }
   }

   /**
   * Display account summary of loan accounts
   */
   private void handleLoanAccounts(GtUserProfile up, PrintWriter out)
                throws ServletException, IOException
   {
      int acctIndex = 0;
      boolean flagAccount = false;

      // credit accounts
      for (int i = 0; i < up.getAccountDetail().length; i++)
      {
         // set account variables
         try { setAccountInfo(up.getAccountDetail()[i]); }
         catch (Exception e)
         {
            acctIndex++;
            continue;
         }

         // loan accounts
         if (type.equals("CLA") || type.equals("ILA") ||type.equals("MLA"))
         {
            if (!flagAccount)
               out.println("<DIV ALIGN=CENTER><B>LOAN ACCOUNTS</B></DIV><HR><BR>");
            flagAccount = true;

            displayAccountInfo("Principal Balance:", acctIndex, out);
         }
         acctIndex++;
      }
   }


   /**
   * Sets the acct, acctNbr, type, and type_text variables.
   */
   private void setAccountInfo(GsAcctDtl accountDetail) throws Exception
   {
      // set account detail
      acct = accountDetail;

      // account number
      try { acctNbr = acct.getAccount().getNumber(); }
      catch (Exception e) { acctNbr = ""; }

      // account type and type description
      try
      {
         type = acct.getAccount().getType();
         try { type_text = acct.getAccount().getType_text(); }
         catch (IfsException e) { type_text = "Unknown account type: " + type ; }
      }
      catch (Exception e) { throw e; }

   }


   /**
   * Displays the account information.
   */
   private void displayAccountInfo(String balanceTitle, int acctIndex, PrintWriter out)
   {
      String strValue = "";

      // Account number
      out.println("<DIV ALIGN=LEFT>Account Number:</DIV>");
      out.println("<DIV ALIGN=RIGHT><B>");
      out.println("<A href =\"IfsWpActDetails?AccountIndex=" +
                  acctIndex + "&ReqTag=Details\">" +
                  acctNbr + "</A></B></DIV>");

      // Account type, Principal Balance, date
      show2DivBlocks(out, "Account Type:", type_text);
      show2DivBlocks(out, balanceTitle, acct.getPrincipalBalanceAmount());
      show2DivBlocks(out, "Date:", acct.getInformationDate());

      // Product name
      try { strValue = acct.getAccount().getNickname(); }
      catch (Exception e) { strValue = " "; }
      show2DivBlocks(out, "Account Nick Name:", strValue);

      out.println ("<HR>");
   }
}
