//-*************************************
//
// FILE NAME:  IfsWpAddFixedTransfer.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/14/2002     replaced 'getValue' call with 'getAttribute'
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statements for handleError
 */

import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import javax.servlet.http.*;
import javax.servlet.*;
import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

/**
* This class is used to handle the transaction of add fixed transfer.
*/
public class IfsWpAddFixedTransfer extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;

   /**
   * This method displays values to let user confirm..
   */
   private void confirmAddFix(GtUserProfile up, IfsSession ifsSession,
                              HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      String msg = "";
      String reqTransactionAmount = "";
      String oid = "";
      String strValue = "";

      GtAddFixTransfer transfer = null;

      try
      {
         //---------------
         // Get a reference to the GtAddFixTransfer object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transfer = (GtAddFixTransfer) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtAddFixTransfer",
                             "transfer");


// clearing cache data for the transfer object.
transfer.setReqTotalPayment(0);

try {                                                               
      GsCurrency nullCurrency = new GsCurrency();
      nullCurrency = GsCurrency.fromString(" ");                                                                 
      transfer.setReqTransactionAmount(nullCurrency);
     }
catch(Exception e)
    { debug("GsCurrency creation and initialization" + e.toString());
      throw e;
     }


         try
         {
            //---------------
            // Set the fields of the GtAddFixTransfer object instance from the
            // corresponding request parameters values in the HTTP request.
            //
            setFields(ifsSession,"transfer",transfer);


            // insert the first 2 digits of the year with 80 as the cut-off value.
            GsDate date = new GsDate();
            date = yearWindow(transfer.getReqEffectiveDate());
            transfer.setReqEffectiveDate(date);

            if ( transfer.getReqLastTransferDate() != null)
            {
              date = yearWindow(transfer.getReqLastTransferDate());
              transfer.setReqLastTransferDate(date);
            } 
 
         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         // Get the object ID of the cached GtAddFixTransfer instance object.
         oid = getOid(ifsSession,transfer);

         // Validate the data that was entered.
         msg = validateAddFix(transfer, request);
         if ( msg.equals("OK") == false)
            callPage_RedoPage(response, "IfsWpAddFixedTransfer", msg);

         // format the transaction amount.
         transfer.getReqTransactionAmount().getFormat().setFormatStyle(IfsSystem.CURRENCY_TEXT_STYLE);
         transfer.getReqTransactionAmount().setDigits(transfer.getReqTransactionAmount().getDigits());
         reqTransactionAmount = transfer.getReqTransactionAmount().toString();

         if (reqTransactionAmount == null)
             reqTransactionAmount = "";

         if (transfer.getReqLastTransferAmount() == null)
            transfer.setReqLastTransferAmount(new GsCurrency(strCurrencyPrefix +
                                                              "0", ifsSession));

         if (transfer.getReqTransferNickname() == null)
            transfer.setReqTransferNickname("");

         if (transfer.getReqTfrSrceAcctRefr() == null)
            transfer.setReqTfrSrceAcctRefr("");

         if (transfer.getReqTfrTrgtAcctRefr() == null)
            transfer.setReqTfrTrgtAcctRefr("");

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to do add fixed transfer. Exception caught in confirmAddFix method of IfsWpAddFixedTransfer Servlet, explanation: ");
      }

      //-----------------
      // Start outputting add fixed transfer confirmation information.
      //
      PrintWriter out = startHtmlOutput(response, " Confirm Add Fixed Transfer");

      out.println("<B>Please click OK button to confirm your transfer.</B><HR>");

      show2DivBlocks (out, "Transfer From Account:", transfer.getReqSource());
      show2DivBlocks (out, "Transfer To Account:", transfer.getReqTarget());

      try { strValue = getTextString(transfer.getReqFrequency(),
                                     "IfsFreqClResources", ifsSession, out); }
      catch (Exception e) { strValue = ""; }
      show2DivBlocks(out, "Transfer Frequency:", strValue);

      show2DivBlocks(out, "Amount:", reqTransactionAmount);
      show2DivBlocks(out, "Effective Date:", transfer.getReqEffectiveDate());
      show2DivBlocks(out, "Last Transfer Date:", transfer.getReqLastTransferDate());
      show2DivBlocks(out, "Total Number of Transfers:", transfer.getReqTotalPayment());
      show2DivBlocks(out, "Transfer Nickname:", transfer.getReqTransferNickname());

      out.println("<CENTER> <P><TABLE>");
      out.println("<TR><TD>");
      out.println("<FORM METHOD=post NAME=TransferConfirm ACTION=IfsWpAddFixedTransfer >");
      out.println("<INPUT TYPE=HIDDEN NAME=whereFlag  VALUE=\"fromConfirm\" >");
      out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + oid + "\" >");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"OK\" ></FORM></TD>");

      out.println("<TD><FORM METHOD=get NAME=TransferCancel ACTION=IfsWpAddFixedTransfer >");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Cancel\" >");
      out.println("</FORM> </TD></TR>");

      out.println(" </TABLE>");
   }

   //*******************************
   // Displays the input form for generating a Add Fixed Transfer transaction.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);

      GtUserProfile    up  = null;
      GtAddFixTransfer aft = null;

      GsAccount  source            = null;
      GsAccount  target            = null;
      String     frequency         = null;

      String     transactionAmount = "";
      String     effectiveDate     = "";
      String     lastTransferDate  = "";
      String     totalPayment      = "";
      String     transferNickname  = "";


      //Get account data
      //replaced getValue with getAttribute 1/14/02
      up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
      if (up == null)
      {
         // Since no user profile exists, there is no active session for this consumer.
         callPage_NoSession(response);
      }

      try
      {
         //---------------
         // Check for redisplay of originally entered information.
         // Get a reference to the GtAddFixTransfer cached object.
         //
         aft = (GtAddFixTransfer) getObject(ifsSession, ADD_FIXED_TRANSFER);
         if (aft != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            source = aft.getReqSource();
            target = aft.getReqTarget();
            frequency = aft.getReqFrequency();

            transactionAmount = getAmountString(aft.getReqTransactionAmount());

            effectiveDate = getDateString(aft.getReqEffectiveDate());
            lastTransferDate = getDateString(aft.getReqLastTransferDate());

            if (aft.getReqTotalPayment() != 0)
               totalPayment = String.valueOf(aft.getReqTotalPayment());

            if (aft.getReqTransferNickname() != null)
               transferNickname = aft.getReqTransferNickname();
         }
      }
      catch ( Exception e) { }

      PrintWriter out = startHtmlOutput(response, "Add Fixed Transfer");

      out.println("<FORM METHOD=POST  ACTION=IfsWpAddFixedTransfer >");
      out.println("<INPUT TYPE=HIDDEN NAME=whereFlag VALUE=\"fromForm\" ></TR>");
      out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + ADD_FIXED_TRANSFER + "\" >");

      out.println("<TABLE BORDER=0   WIDTH=100%>");

      // Transfer from Account drop-down
      out.println("<TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT ><B> Transfer From Account: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD ALIGN=LEFT>");
      out.println("<SELECT NAME=reqSource >");
      displayAccounts(up, source, out);
      out.println("</SELECT></TD></TR>");

      // Transfer to Account drop-down
      out.println("<TR ALIGN=CENTER>");
      out.println("<TD  ALIGN=LEFT ><B> Transfer To Account: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<SELECT NAME=reqTarget >");
      displayAccounts(up, target, out);
      out.println("</SELECT></TD></TR>");

      // Transfer frequency drop-down
      out.println("<TR ALIGN=LEFT>");
      out.println("<TD ALIGN=LEFT NOWRAP><B> Transfer Frequency: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<SELECT NAME=reqFrequency >");
      displayFrequencySelectList(frequency, out );
      out.println("</SELECT></TD></TR>");

      // Amount to transfer
      out.println("<TR ALIGN=LEFT>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Amount: </B>" + strCurrencyPrefix );
      out.println(" <INPUT NAME=reqTransactionAmount TYPE=text SIZE=13 MAXLENGTH=20 VALUE=\"" +
                  transactionAmount + "\" >");
      out.println("</TD></TR>");

      // Effective date to start transfer
      out.println("<TR ALIGN=LEFT>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Effective Date: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<INPUT NAME=reqEffectiveDate TYPE=text SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  effectiveDate + "\" > (" + strDateHint + ")");
      out.println("</TD></TR>");

      // Last date to do a transfer
      out.println("<TR ALIGN=LEFT>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Final Transfer Date: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<INPUT NAME=reqLastTransferDate TYPE=text SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  lastTransferDate + "\" > (" + strDateHint + ")");
      out.println("</TD></TR>");

      out.println("<TR ALIGN=LEFT>");
      out.println("<TD ALIGN=LEFT><B> OR</B>");
      out.println("</TD></TR>");

      // Number of transfers to do.
      out.println("<TR ALIGN=LEFT>");
      out.println("<TD ALIGN=LEFT NOWRAP><B> Total Number of Transfers: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<INPUT NAME=reqTotalPayment TYPE=text SIZE=6 MAXLENGTH=6 VALUE=\"" +
                  totalPayment + "\" >");
      out.println("</TD></TR>");

      // Optional transfer nickname.
      out.println("<TR ALIGN=LEFT>");
      out.println("<TD ALIGN=LEFT NOWRAP><B>Transfer Nickname: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<INPUT NAME=reqTransferNickname TYPE=text SIZE=25 MAXLENGTH=30 VALUE=\"" +
                  transferNickname + "\">");
      out.println("</TD></TR>");

      // Add pushbutton.
      out.println("<TR ALIGN=Center>");
      out.println("<TD><BR><INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Add\" >");
      out.println("</TD></TR>");
      out.println("</TABLE> </FORM>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   //*******************************
   // Either displays the confirmation page for a Add Fixed Transfer
   // transaction or executes it.
   //
   // The following Http Request Parameters are posted to this page:
   //     whereFlag - Indicates whether to display confirmation or execute.
   //                 It can have the values of "fromForm" or "fromConfirm".
   //     transfer - Specifies the object identifier (oid) of a
   //                GtAddFixTransfer object.
   //     req????  - The input values from the Add Fix Transfer form.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {


      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;
      String strReq = "";

      try
      {
          //replaced getValue with getAttribute 1/14/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         strReq = (String)getRequestParameter(request,"whereFlag");

         if (strReq.equals("fromForm"))
            confirmAddFix(up, ifsSession, request, response);
         else
            processAddFix(ifsSession, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to do add fix transfer. Exception caught in dopost method of IfsWpAddFixedTransfer Servlet, explanation: ");
      }
   }


   /**
   * This method process the transaction..
   */
   private void processAddFix(IfsSession ifsSession, HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GtAddFixTransfer transfer = null;
      String strValue = "";

      try
      {
         transfer = (GtAddFixTransfer) createInstance(ifsSession,
                         "com.ibm.ifs.gold.auto.GtAddFixTransfer",
                         "transfer");

         if (transfer == null)
            callPage_RedoPage(response, "IfsWpAddFixedTransfer",
                 "There is no GtAddFixTransfer object in the session cache! ");
       //added try-catch block
        try
        {
        
         transfer.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");

        
        }
                      
         if (! handleErrors(transfer, "IfsWpErrorPage",
                       "IfsWpRedoPage?nextpage=IfsWpAddFixedTransfer",
                      "IfsWpRedoPage?nextpage=IfsWpAddFixedTransfer")) {
           return;
                       
          }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to do add fix transfer. Exception caught in processAddFix method of IfsWpAddFixedTransfer Servlet, explanation: ");
      }

      //-------------------
      // Start outputting the add fixed transfer result information.
      //
      PrintWriter out = startHtmlOutput(response, "Add Fixed Transfer results");

      out.println("<B>Add Fixed Transfer transaction was successful.</B>");
      out.println("<HR>");

      show2DivBlocks(out, "Confirmation Number:", transfer.getTransferNumber());
      show2DivBlocks(out, "Adjusted Transfer Date:", transfer.getAdjustedTransferDate());
      show2DivBlocks(out, "Adjusted Final Transfer Date:",
                     transfer.getAdjustedFinalTransferDate());

      out.println("<P>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Validate the data that was entered.
   */
   private String validateAddFix(GtAddFixTransfer af, HttpServletRequest request)
   {
      StringBuffer strBuf = new StringBuffer("");
      String strTemp = "";
      double dAmount = 0;
      String reqEffectiveDate = "";
      String reqLastTransferDate = "";

      // Get the transaction amount information.
       try {
               dAmount = getAndCheckAmount(af.getReqTransactionAmount());
            }
       catch (Exception e)
       {
          af.debug("Exception getting amount in validateAddFix - " + e.toString());
          dAmount = 0;
       }

      // Get the effective Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
          reqEffectiveDate = af.getReqEffectiveDate().toString(new GsDateFormat("yyyy/MM/dd"));

         if (reqEffectiveDate == null)
             reqEffectiveDate = "";

      }
      catch (Exception e)
      {
         af.debug("Exception getting effective date in validateAddFix - " + e.toString());
         reqEffectiveDate = "";
      }

      // Get the last transfer Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         reqLastTransferDate = af.getReqLastTransferDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (reqLastTransferDate == null)
            reqLastTransferDate = "";
      }
      catch (Exception e)
      {
         af.debug("Exception getting last transfer date in validateAddFix - " + e.toString());
         reqLastTransferDate = "";
      }

      // Validate the data that was entered.
       if (dAmount <= 0)
          strBuf.append("<BR>Must specify a valid positive non-zero amount within the allowed Currency range.");

      if (reqEffectiveDate.equals(""))
         strBuf.append("<BR>Must specify a valid effective date.");

      strTemp = (String)getRequestParameter(request,"reqLastTransferDate");
      if ( ((strTemp == null) || (strTemp.length() == 0)) &&
           (af.getReqTotalPayment() <= 0) )
         strBuf.append("<BR>Must specify a valid final transfer date or a positive number of transfers.");

      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (af.getReqTotalPayment() != 0) )
         strBuf.append("<BR>Cannot specify both a final transfer date and a number of transfers.");

      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (reqLastTransferDate.equals("")) )
         strBuf.append("<BR>Invalid final transfer date!");

      if ( (reqLastTransferDate.equals("") == false) &&
           (reqEffectiveDate.equals("") == false) &&
           (reqEffectiveDate.compareTo(reqLastTransferDate) >= 0))
         strBuf.append("<BR>The final transfer date must be later than the effective date!");

      if ( af.getReqSource().getNumber().equals(af.getReqTarget().getNumber()) )
         strBuf.append("<BR>The source and target accounts cannot be the same account!");

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         af.debug("Validation failed in validateAddFix method of IfsWpAddFixedTransfer class");
         af.debug(strBuf.toString());
      }

      return strBuf.toString();
   }
}
