//-*************************************
//
// FILE NAME:  IfsWpAddVariableTransfer.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/14/2002     replaced 'getValue' call with 'getAttribute'
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */

import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import javax.servlet.http.*;
import javax.servlet.*;
import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

/**
* This class handles the Add Variable Transfer transaction.
*/
public class IfsWpAddVariableTransfer extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   /**
   * Checks user input and displays the confirmation information to user.
   */
   private void confirmAddSingle(GtUserProfile up, IfsSession ifsSession,
                                 HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      String msg = "";
      String reqTransactionAmount = "";
      String oid = "";
      String strValue = "";

      String tranImediatelyFlag = "";
      String checkDuplicateFlag = "";

      GtAddVariableTransfer transfer = null;

      try
      {
         //---------------
         // Get a reference to the GtAddVariableTransfer object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transfer = (GtAddVariableTransfer) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtAddVariableTransfer",
                             "transfer");

//clearing the cached data.
try {
      GsCurrency nullCurrency = new GsCurrency();
      nullCurrency = GsCurrency.fromString(" ");
      transfer.setReqTransactionAmount(nullCurrency);
     }
catch(Exception e)
    { debug("GsCurrency creation and initialization" + e.toString());
      throw e;
     }


         try
         {
            // Set the fields of the GtAddVariableTransfer object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transfer",transfer);



            // insert the first 2 digits of the year with 80 as the cut-off value.

            GsDate date = new GsDate();
            date = yearWindow(transfer.getReqEffectiveDate());
            transfer.setReqEffectiveDate(date);

         }
         catch (Exception e)
         {
            debug(response, "setFields in IfsWpAddVariableTransfer has thrown an exception: " + e.toString());
            throw e;
         }

         // Get the object ID of the cached GtAddVariableTransfer instance object.
         oid = getOid(ifsSession,transfer);

         // Validate the data that was entered.
         msg = validateAddVariable(transfer);
         if ( msg.equals("OK") == false)
            callPage_RedoPage(response, "IfsWpAddVariableTransfer", msg);

         // format the transaction amount.
         transfer.getReqTransactionAmount().getFormat().setFormatStyle(IfsSystem.CURRENCY_TEXT_STYLE);
         transfer.getReqTransactionAmount().setDigits(transfer.getReqTransactionAmount().getDigits());
         reqTransactionAmount = transfer.getReqTransactionAmount().toString();

         if (reqTransactionAmount == null)
             reqTransactionAmount = "";

         if (transfer.getReqTransferNickname() == null)
            transfer.setReqTransferNickname("");

         if (transfer.getReqTfrSrceAcctRefr() == null)
            transfer.setReqTfrSrceAcctRefr("");

         if (transfer.getReqTfrTrgtAcctRefr() == null)
            transfer.setReqTfrTrgtAcctRefr("");

         if (transfer.getReqFIReferenceNumber() == null)
            transfer.setReqFIReferenceNumber("");

         // Transfer imediately or not.
         // I & D values from IfsTfrImmedFResource property file.
         if ( transfer.getReqTransferImmediateFlag().equals("I"))
            tranImediatelyFlag = "Yes";
         else
            tranImediatelyFlag = "No";

         // Check Duplicate Transfers or not.
         // Y & N values from IfsTfrDuplChkOvrdResource property file.
         if ( transfer.getReqTransferDuplChkOvrdFlag().equals("Y"))
            checkDuplicateFlag = "Yes";
         else
            checkDuplicateFlag = "No";

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to do add variable transfer. Exception caught in confirmAddSingle method of IfsWpAddVariableTransfer Servlet, explanation: ");
      }

      //------------------
      // Start outputting Add Variable Transfer confirmation information.
      //
      PrintWriter out = startHtmlOutput(response, "Confirm Add Variable Transfer");

      out.println("<B>Please click OK button to confirm your Transfer</B></FONT>");
      out.println("<HR>");

      try { strValue = transfer.getReqSource().getNumber(); }
      catch (Exception e) { strValue = ""; }
      show2DivBlocks(out, "Transfer From Account:", strValue);

      try { strValue = transfer.getReqTarget().getNumber(); }
      catch (Exception e) { strValue = ""; }
      show2DivBlocks(out, "Transfer To Account:", strValue);

      show2DivBlocks(out, "Transfer Immediately:", tranImediatelyFlag);
      show2DivBlocks(out, "Duplicate Transfers Override", checkDuplicateFlag);
      show2DivBlocks(out, "Amount:", reqTransactionAmount);
      show2DivBlocks(out, "Effective Date:", transfer.getReqEffectiveDate());
      show2DivBlocks(out, "Transfer Nickname:", transfer.getReqTransferNickname());

      out.println("<CENTER> <P><TABLE>");
      out.println("<TR><TD>");
      out.println("<FORM METHOD=post NAME=SingleTransfer ACTION=IfsWpAddVariableTransfer >");
      out.println("<INPUT TYPE=HIDDEN NAME=whereFlag  VALUE=\"fromConfirm\" >");
      out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + oid + "\" >");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Ok\" ></FORM></TD>");

      out.println("<TD><FORM METHOD=get NAME=TransferCancel ACTION=IfsWpAddVariableTransfer >");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Cancel\" >");
      out.println("</FORM> </TD></TR>");

      out.println("</TABLE>");
   }

   //*******************************
   // Displays the input form for generating a Add Variable Transfer transaction.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);

      GtUserProfile up = null;
      GtAddVariableTransfer avt = null;

      GsAccount  source            = null;
      GsAccount  target            = null;

      String     immediateFlag     = "I";
      String     duplChkOvrdFlag   = "Y";

      String     frequency         = "";
      String     transactionAmount = "";
      String     effectiveDate     = "";
      String     lastTransferDate  = "";
      String     totalPayment      = "";
      String     transferNickname  = "";

      //replaced getValue with getAttribute 1/14/02
      up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
      if (up == null)
      {
         // Since no user profile exists, there is no active session for this consumer.
         callPage_NoSession(response);
      }

      try
      {
         //---------------
         // Check for redisplay of originally entered information.
         // Get a reference to the GtAddVariableTransfer cached object.
         //
         avt = (GtAddVariableTransfer) getObject(ifsSession, ADD_VARIABLE_TRANSFER);
         if (avt != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            source = avt.getReqSource();
            target = avt.getReqTarget();

            if (avt.getReqTransferImmediateFlag() != null)
               immediateFlag = avt.getReqTransferImmediateFlag();

            if (avt.getReqTransferDuplChkOvrdFlag() != null)
               duplChkOvrdFlag = avt.getReqTransferDuplChkOvrdFlag();

            transactionAmount = getAmountString(avt.getReqTransactionAmount());

            effectiveDate = getDateString(avt.getReqEffectiveDate());

            if (avt.getReqTransferNickname() != null)
               transferNickname = avt.getReqTransferNickname();
         }
      }
      catch ( Exception e) { }

      PrintWriter out = startHtmlOutput(response, "Add Variable Transfer");

      out.println("<FORM METHOD=POST NAME=AddVariableTransfer ACTION=IfsWpAddVariableTransfer >");
      out.println("<INPUT TYPE=HIDDEN NAME=whereFlag  VALUE=\"fromForm\" ></TR>");
      out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + ADD_VARIABLE_TRANSFER + "\" >");

      out.println("<TABLE BORDER=0   WIDTH=100%>");

      // Transfer from Account drop-down
      out.println("<TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Transfer From Account: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<SELECT NAME=reqSource >");
      displayAccounts(up, source, out);
      out.println("</SELECT></TD></TR>");

      // Transfer to Account drop-down
      out.println("<TR ALIGN=CENTER>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Transfer To Account: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<SELECT NAME=reqTarget >");
      displayAccounts(up, target, out);
      out.println("</SELECT></TD></TR>");

      out.println("<TR ALIGN=LEFT>");
      out.println("<TD ALIGN=LEFT NOWRAP><B> Transfer Immediately: </B>");
      out.println("<SELECT NAME=reqTransferImmediateFlag >");
      if (immediateFlag.equals("I")) // Values from IfsTfrImmedFResource property file.
         out.println("<OPTION SELECTED VALUE=\"I\" >Yes <OPTION VALUE=\"D\" >No ");
      else
         out.println("<OPTION VALUE=\"I\" >Yes <OPTION SELECTED VALUE=\"D\" >No ");

      out.println("</SELECT></TD></TR>");

      out.println("<TR ALIGN=LEFT>");
      out.println("<TD ALIGN=LEFT NOWRAP><B> Duplicate Override: </B>");
      out.println("<SELECT NAME=reqTransferDuplChkOvrdFlag >");
      if (duplChkOvrdFlag.equals("Y")) // Values from IfsTfrDuplChkOvrdResource property file.
         out.println("<OPTION SELECTED VALUE=\"Y\" >Yes <OPTION VALUE=\"N\" >No ");
      else
         out.println("<OPTION VALUE=\"Y\" >Yes <OPTION SELECTED VALUE=\"N\" >No ");

      out.println("</SELECT></TD></TR>");

      out.println("<TR ALIGN=LEFT>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Amount:</B> " + strCurrencyPrefix );
      out.println("<INPUT NAME=reqTransactionAmount TYPE=text SIZE=13 MAXLENGTH=20 VALUE=\"" +
                  transactionAmount + "\"> ");
      out.println("</TD></TR>");

      out.println("<TR ALIGN=LEFT>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Effective Date: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<INPUT NAME=reqEffectiveDate TYPE=text SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  effectiveDate + "\"> (" + strDateHint + ") ");
      out.println("</TD></TR>");

      out.println("<TR ALIGN=LEFT>");
      out.println("<TD ALIGN=LEFT NOWRAP><B>Transfer Nickname: </B>(Optional)");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<INPUT NAME=reqTransferNickname TYPE=text SIZE=25 MAXLENGTH=30 VALUE=\"" +
                  transferNickname + "\"> ");
                  out.println("</TD></TR>");

      out.println("<TR ALIGN=CENTER>");
      out.println("<TD><BR><INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Add\" >");
      out.println("</TD></TR>");

      out.println("</TABLE> </FORM>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   //*******************************
   // Either displays the confirmation page for a Add Variable Transfer
   // transaction or executes it.
   //
   // The following Http Request Parameters are posted to this page:
   //     whereFlag - Indicates whether to display confirmation or execute.
   //                 It can have the values of "fromForm" or "fromConfirm".
   //     transfer - Specifies the object identifier (oid) of a
   //                GtAddVariableTransfer object.
   //     req????  - The input values from the Add Variable Transfer form.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;
      String strReq = null;

      try
      {
         //replaced getValue with getAttribute 1/14/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         strReq = (String)getRequestParameter(request,"whereFlag");

         if (strReq.equals("fromForm"))
            confirmAddSingle(up, ifsSession, request, response);
         else
            processAddSingle(up, ifsSession, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to add variable transfer. Exception in doPost method of IfsWpAddVariableTransfer Servlet, explanation: ");
      }
   }

   /**
   * This method process the transaction..
   */
   private void processAddSingle(GtUserProfile up, IfsSession ifsSession,
                                 HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GtAddVariableTransfer transfer = null;

      try
      {
         transfer = (GtAddVariableTransfer) createInstance(ifsSession,
                         "com.ibm.ifs.gold.auto.GtAddVariableTransfer",
                         "transfer");

         if (transfer == null)
            callPage_RedoPage(response, "IfsWpAddVariableTransfer",
                 "There is no GtAddVariableTransfer object in the session cache! ");

        //added try-catch block 1/18/2002
         try
       {

        transfer.execute();
       } catch (Exception e )  {
           callPage_ErrorPage(response, e,
                "Exception in execute method: ");
       }
         if (! handleErrors(transfer, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpAddVariableTransfer",
                      "IfsWpRedoPage?nextpage=IfsWpAddVariableTransfer")){
          return;
          }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to do add variable transfer. Exception caught in processAddSingle method of IfsWpAddVariableTransfer Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Add Variable Transfer results");

      out.println("<B>Add Variable Transfer transaction was successful.</B>");
      out.println("<HR>");

      show2DivBlocks (out, "Confirmation Number:", transfer.getTransferNumber());

      try
      {
         GsDateFormat df = new GsDateFormat(strDatePattern);
         GsDate adjDate = transfer.getAdjustedTransferDate();

         if ( (adjDate != null) || (adjDate.toString().length() != 0) )
            show2DivBlocks (out, "Adjusted Transfer Date:", adjDate.toString(df));
      }
      catch (Exception e) {}

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   /**
   * Validate the data that was entered.
   */
   private String validateAddVariable(GtAddVariableTransfer av)
   {
      StringBuffer strBuf = new StringBuffer("");

      String reqEffectiveDate = "";

      // Get the effective Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         reqEffectiveDate = av.getReqEffectiveDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (reqEffectiveDate == null)
            reqEffectiveDate = "";
      }
      catch (Exception e)
      {
         av.debug("Exception getting effective date in validateAddVariable - " + e.toString());
         reqEffectiveDate = "";
      }

      // Validate the data that was entered.
      if (isValidAmount(av.getReqTransactionAmount()) == false )
         strBuf.append("<BR>Must specify a valid positive non-zero amount within the allowed Currency range.");

      if (reqEffectiveDate.equals(""))
         strBuf.append("<BR>Must specify a valid effective date.");

      if ( av.getReqSource().getNumber().equals(av.getReqTarget().getNumber()) )
         strBuf.append("<BR>The source and target accounts cannot be the same account!");

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         av.debug("Validation failed in validateAddVariable method of IfsWpAddVariableTransfer class");
         av.debug(strBuf.toString());
      }

      return strBuf.toString();
   }

}
