//-*************************************
//
// FILE NAME:  IfsWpApplication.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson           01/14/2002  commented out 'addCopyright' call
 *
 */

import java.io.*;
import java.net.URLEncoder;
import java.util.*;


import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

/**
* This is the super class for the pervasive computing (palm pilot) application.
* It contains common methods for the application.
*/
public class IfsWpApplication extends IfsHttpServlet
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;

   // date hint
   protected static String strDateHint = IfsSystem.DATE_HINT;
   protected static String strDatePattern = strDateHint.replace('m','M');
   protected static String strDateFormat = IfsSystem.GOLD_DATE_FORMAT;

   // currency prefix
   protected static String strCurrencyPrefix = IfsSystem.gsCurrencyFormat.getPositivePrefix();

   // Session cache object identifiers
   protected static String USER_PROFILE        = "__UserProfile";
   protected static String TRANSACTION_SEARCH  = "__TransactionSearch";
   protected static String TRANSACTION_HISTORY = "__TransactionHistory";
   protected static String CHECK_COPY          = "__CheckCopy";
   protected static String ORDER_CHECK_BOOK    = "__OrderCheckBook";
   protected static String STOP_CHECK_PAYMENT  = "__StopCheckPayment";
   protected static String SEARCH_TRANSFER     = "__SearchTransfer";
   protected static String TRANSFER_INQUIRY    = "__TransferInquiry";

   protected static String ADD_VARIABLE_TRANSFER    = "__AddVariableTransfer";
   protected static String CHANGE_VARIABLE_TRANSFER = "__ChangeVariableTransfer";

   protected static String ADD_FIXED_TRANSFER       = "__AddFixedTransfer";
   protected static String CHANGE_FIXED_TRANSFER    = "__ChangeFixedTransfer";

   /**
   * Passes the message to the error page.
   * <P>
   * NOTE: Servlet page flow continues with the error page.
   */
   public void callPage_ErrorPage(HttpServletResponse response,
                                  Exception e, String message)
               throws IOException
   {
      StringBuffer errMsg = new StringBuffer();
      errMsg.append("IfsWpErrorPage?Exception=");
      errMsg.append(URLEncoder.encode(message));
      errMsg.append(URLEncoder.encode(e.toString()));
      callPage(response,errMsg.toString());
   }

   /**
   * Passes the no session message to the redo page.
   * <P>
   * NOTE: Servlet page flow continues with the redo page.
   */
   public void callPage_NoSession(HttpServletResponse response)
               throws IOException
   {
      StringBuffer message = new StringBuffer();
      message.append("IfsWpRedoPage?nextpage=IfsWpLogon&redomsg=");
      message.append(URLEncoder.encode("No session is active. Session may have timed out. You need to logon again."));
      callPage(response,message.toString());
   }

   /**
   * Passes the message to the redo page.
   * <P>
   * NOTE: Servlet page flow continues with the redo page.
   */
   public void callPage_RedoPage(HttpServletResponse response,
                                 String nextPage, String redoMsg)
               throws IOException
   {
      StringBuffer message = new StringBuffer();
      message.append("IfsWpRedoPage?nextpage=" + nextPage + "&redomsg=");
      message.append(URLEncoder.encode(redoMsg));
      callPage(response,message.toString());
   }


   /**
   * Displays the list of available accounts within the user profile
   * in a pull-down select list.
   *
   * The account parameter contains the account to be initially selected.
   * If account is null, then the first element in the list is selected.
   */
   public void displayAccounts(GtUserProfile up,
                               GsAccount account,
                               PrintWriter out)
   {
      GsAcctDtl   acctDtl        = null;
      GsAccount   acct           = null;
      String      acctType       = "";
      String      acctNumber     = "";
      String      acctNickname   = "";
      StringBuffer acctStr       = new StringBuffer();

      for (int i = 0 ; i < up.getAccountDetail().length ; i++)
      {
         acctDtl      = up.getAccountDetail()[i];
         acct         = acctDtl.getAccount();
         acctType     = acct.getType();
         acctNumber   = acct.getNumber();
         acctNickname = acct.getNickname();

         acctStr = new StringBuffer();
         acctStr.append("??type=");
         acctStr.append(acctType);
         acctStr.append("&number=");
         acctStr.append(acctNumber);
         acctStr.append("&nickname=");
         acctStr.append(acctNickname);

         if ( (account != null) &&
              (account.getType().equals(acctType)) &&
              (account.getNumber().equals(acctNumber)) )
            out.println("<OPTION SELECTED VALUE=\""+ acctStr.toString() + "\">");
         else
            out.println("<OPTION VALUE=\""+ acctStr.toString() + "\">");

         out.println(acctType);
         out.println(" -- ");
         out.println(acctNumber);

      } // end for loop

   } // end-of, displayAccounts method


   /**
   * Displays the list of available account numbers within the user profile
   * in a pull-down select list.
   *
   * The account parameter contains the account to be initially selected.
   * If account is null, then the first element in the list is selected.
   */
   public void displayAccountNum(GtUserProfile up,
                                 GsAccount account,
                                 PrintWriter out)
   {
      GsAcctDtl   acctDetail     = null;
      String      acctType       = "";
      String      acctNumber     = "";
      StringBuffer acctStr       = new StringBuffer();

      for (int i = 0 ; i < up.getAccountDetail().length ; i++)
      {
         try
         {
            acctDetail = up.getAccountDetail()[i];
            acctType   = acctDetail.getAccount().getType();
            acctNumber = acctDetail.getAccount().getNumber();

            if ( (account != null) &&
                 (account.getType().equals(acctType)) &&
                 (account.getNumber().equals(acctNumber)) )
               out.println("<OPTION SELECTED VALUE=\"" + String.valueOf(i) + "\" >" + acctNumber);
            else
               out.println("<OPTION VALUE=\"" + String.valueOf(i) + "\" >" + acctNumber);

         }
         catch (Exception e) { continue; }
      }

   } // end-of, displayAcccountNum method


   /**
   * Displays the list of check book style options
   * in a pull-down select list.
   *
   * The checkBookStyle parameter contains the element to be initially selected.
   * If checkBookStyle is null, then the first element in the list is selected.
   */
   public void displayCheckBookStyleList(String checkBookStyle, PrintWriter out )
   {
      // Possible check book style options from
      // IfsChkBkStlClResource property file.
      String[] checkBookStyleText  = { "Style A", "Style B", "Style C", "Style D"};
      String[] checkBookStyleCodes = { "A", "B", "C", "D"};

      for (int i = 0; i < checkBookStyleText.length; i++)
      {
         if ( (checkBookStyle != null) &&
              (checkBookStyle.equals(checkBookStyleCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\"" + checkBookStyleCodes[i] + "\" >");
         else
            out.println("<OPTION VALUE=\"" + checkBookStyleCodes[i] + "\" >");

         out.println( checkBookStyleText[i]);
      }

   }  // end-of, displayCheckBookStyleList method


   /**
   * Displays the list of frequency types
   * in a pull-down select list.
   *
   * The frequencyType parameter contains the type to be initially selected.
   * If frequencyType is null, then the first element in the list is selected.
   *
   * Use following Frquency Type and Frequency Code if supported by your backend.
   * frequencyTypeText = { "Twice Monthly", "Weekly", "Bi-weekly",
   *                                 "Bi_monthly", "Monthly", "Quarterly",
   *                                 "Semi-annually", "Annually", "Daily",
   *                                 "Monthly (at EOM)", "Every Four Weeks" };
   *
   * frequencyTypeCodes = { "0", "1", "2", "3", "4", "5", "6", "7", "8",
   *                                  "9", "A" };  
   *
   */
   public void displayFrequencySelectList(String frequencyType,
                                          PrintWriter out )
   {
      // Possible frequency types from
      // IfsFreqClResource property file.
      String[] frequencyTypeText = { "Weekly", "Bi-weekly",
                                     "Bi_monthly", "Monthly", "Quarterly",
                                     "Semi-annually", "Annually", "Daily" };
                                     
      String[] frequencyTypeCodes = { "1", "2", "3", "4", "5", "6", "7", "8" };
                                      

      for (int i = 0; i < frequencyTypeText.length; i++)
      {
         if ( (frequencyType != null) &&
              (frequencyType.equals(frequencyTypeCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\"" + frequencyTypeCodes[i] + "\" >");
         else
            out.println("<OPTION VALUE=\"" + frequencyTypeCodes[i] + "\" >");

         out.println( frequencyTypeText[i]);
      }

   }  // end-of, displayFrequencySelectList method


   /**
   * Displays the list of postal delivery options
   * in a pull-down select list.
   *
   * The postalCode parameter contains the element to be initially selected.
   * If postalCode is null, then the first element in the list is selected.
   */
   public void displayPostalSelectList(String postalCode, PrintWriter out )
   {
      // Possible postal delivery options from
      // IfsDlvyMethIndResource property file.
      String[] postalDeliveryText = { "Postal Delivery", "Courier Delivery",
                                      "Email Delivery", "Online Delivery",
                                      "Hold At Branch"};
      String[] postalDeliveryCodes = { "1", "2", "3", "4", "5"};

      for (int i = 0; i < postalDeliveryText.length; i++)
      {
         if ( (postalCode != null) &&
              (postalCode.equals(postalDeliveryCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\"" + postalDeliveryCodes[i] + "\" >");
         else
            out.println("<OPTION VALUE=\"" + postalDeliveryCodes[i] + "\" >");

         out.println( postalDeliveryText[i]);
      }

   }  // end-of, displayPostalSelectList method


   /**
   * Displays the list of stop check payment options
   * in a pull-down select list.
   *
   * The stopCode parameter contains the element to be initially selected.
   * If stopCode is null, then the first element in the list is selected.
   */
   public void displayStopPaymentList(String stopCode, PrintWriter out )
   {
      // Possible stop check payment options from
      // IfsStopPymtTpClResource property file.
      String[] stopPaymentText = { "by Check Number", "by Check Number and Amount",
                                   "by Check Number Range", "by Amount",
                                   "by Amount Range", "by Date",
                                   "by Date and Amount", "by Date Range",
                                   "All Checks" };
      String[] stopPaymentCodes = { "0", "1", "2", "3", "4", "5", "6", "7", "8"};

      for (int i = 0; i < stopPaymentText.length; i++)
      {
         if ( (stopCode != null) &&
              (stopCode.equals(stopPaymentCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\"" + stopPaymentCodes[i] + "\" >");
         else
            out.println("<OPTION VALUE=\"" + stopPaymentCodes[i] + "\" >");

         out.println( stopPaymentText[i]);
      }

   }  // end-of, displayStopPaymentList method


   /**
   * Displays the list of transaction types
   * in a pull-down select list.
   *
   * The transactionType parameter contains the type to be initially selected.
   * If transactionType is null, then the first element in the list is selected.
   */
   public void displayTransactionTypeList(String transactionType,
                                          PrintWriter out )
   {
      // Possible transaction types from
      // IfsTxnTpIndResource property file.
      String[] transactionTypeText = { "All types", "Debit only", "Credit only",
                                       "Withdrawal only", "Check only", "Deposit only",
                                       "Fee only", "Charge only", "Tax only",
                                       "Rebate only", "Interest only", "Credit Interest only",
                                       "Debit Interest only", "Manual only", "Adjustment only",
                                       "Non-Financial only"};

      String[] transactionTypeCodes = { "1", "2", "3", "4", "5", "6", "7", "8", "9", "10",
                                        "11", "12", "13", "14", "15", "16"};

      for (int i = 0; i < transactionTypeText.length; i++)
      {
         if ( (transactionType != null) &&
              (transactionType.equals(transactionTypeCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\"" + transactionTypeCodes[i] + "\" >");
         else
            out.println("<OPTION VALUE=\"" + transactionTypeCodes[i] + "\" >");

         out.println( transactionTypeText[i]);
      }

   }  // end-of, displayTransactionTypeList method

   /**
   * Displays the list of transaction sources
   * in a pull-down select list.
   *
   * The transactionSource parameter contains the source to be initially selected.
   * If transactionSource is null, then the first element in the list is selected.
   */
   public void displayTransactionSourceList(String transactionSource,
                                            PrintWriter out )
   {
      // Possible transaction sources from
      // IfsTxnSrceValResource property file.
      String[] transactionSourcesText = {"All", "ATM", "Teller", "Point Of Sale",
                                         "Audio Response Unit", "Home Banking",
                                         "Automated Clearing House",
                                         "Check", "KIOSK"};

      String[] transactionSourcesCodes = { "0", "1", "2", "3", "4", "5",
                                           "6", "7", "8" };

      for (int i = 0; i < transactionSourcesText.length; i++)
      {
         if ( (transactionSource != null) &&
              (transactionSource.equals(transactionSourcesCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\"" + transactionSourcesCodes[i] + "\" >");
         else
            out.println("<OPTION VALUE=\"" + transactionSourcesCodes[i] + "\">");

         out.println( transactionSourcesText[i]);
      }

   } // end-of, displayTransactionSourceList method


   /**
   * Displays the list of transaction periods
   * in a pull-down select list.
   *
   * The transactionPeriod parameter contains the period to be initially selected.
   * If transactionPeriod is null, then the first element in the list is selected.
   */
   public void displayTransactionPeriodList(String transactionPeriod,
                                            PrintWriter out )
   {
      // Possible transaction periods from
      // IfsTxnPrdTpClResource property file.
      String[] transactionPeriodsText = {"Select a Time Frame or Date Range",
                                         "Since last statement",
                                         "Since last inquiry",
                                         "Since previous statement"};
      String[] transactionPeriodsCodes = { "-", "S", "I", "P"};

      for (int i = 0; i < transactionPeriodsText.length; i++)
      {
         if ( (transactionPeriod != null) &&
              (transactionPeriod.equals(transactionPeriodsCodes[i])) )
            out.println("<OPTION SELECTED VALUE=\"" + transactionPeriodsCodes[i] + "\">");
         else
            out.println("<OPTION VALUE=\"" + transactionPeriodsCodes[i] + "\">");

         out.println( transactionPeriodsText[i]);
      }

   } // end-of, displayTransactionPeriodList method


   /**
   * Put page links to print writer and closes it.
   */
   public void endHtmlOutput(PrintWriter out)
   {
      out.println("<P><DIV ALIGN=CENTER><A href=IfsWpMainDisplay >IFS </A>");
      out.println("|<A href=IfsWpAccounts >Accounts </A>");
      out.println("|<A href=IfsWpTransfer >Transfer </A>");
      out.println("|<A href=IfsWpServices >Services </A>");
      out.println("|<A href=IfsWpLogoff   >Logoff   </A></DIV>");

      out.println("</BODY> </HTML>");
      out.close();
   }


   /**
   * Returns the currency amount in string format.
   * An empty string is returned when the input is null or an exception occurs.
   */
   public String getAmountString(GsCurrency amt)
   {
      String temp = "";

      try
      {
         if (amt != null)
         {
            amt.getFormat().setFormatStyle(IfsSystem.CURRENCY_TEXT_STYLE);
            amt.setDigits(amt.getDigits());
            temp = amt.toString();

            if (temp == null)
               temp = "";
         }
      }
      catch (Exception e) {}

      return(temp);
   }


   /**
   * If the currency amount exists, it is checked for validity.
   * A zero is returned when the input is null or an exception occurs.
   */
   public double getAndCheckAmount(GsCurrency amount)
   {
      double dAmount = 0;
      String msg = "";
      try
      {
         if ( (amount == null) ||
              (amount.getDigits() == null) ||
              (amount.getDigits().length() == 0) )
            return(dAmount);   // currency amount does not exist.

         try { msg = amount.validate();
               if (msg != "OK") return(dAmount); 
             } 
         catch (Exception e) {msg = ""; return(0);}

         try { dAmount = amount.getNumeric(); }
         catch (NumberFormatException ee) {return(0);}
      }
      catch (Exception e) {return(0);}


      return(dAmount);
   }


   /**
   * Accepts a GsDate object and inserts a window in the year.
   * The first two digits will be "19" if the year is < 80 and "20" if > 80.
   * Returns the adjusted year as GsDate object.
   */
   public GsDate yearWindow(GsDate date)
   {
    String daystr = "";
    String monthstr = "";
    String yearstr = "";
    String pattern = "";

    String datestr = "";
    String newdate = "";
    String temp = "";
    int len = 0;
    int newint = 0;

    GsDate retdate = new GsDate();

    try {

         datestr = date.toString(new GsDateFormat(strDatePattern));
         daystr = date.getDay();
         monthstr = date.getMonth();

         yearstr = IfsApi.chop(date.getYear(),'0');  
// pattern = datestr.substring(2,3); // change according to gold std.
        
         len = yearstr.length(); 

         if ( len > 4 || len < 1 || len == 3 )
            yearstr = "";

         if (len == 1)
            { temp = "0"; 
              yearstr = temp.concat(yearstr);
             }
            
         if ( len == 4 )
            yearstr = yearstr.substring(2,4);

           try { Integer newInt = new Integer(yearstr);
                       newint = newInt.intValue();
                  if ( newint < 80 )
                      {
                       temp = "20";
                       yearstr = temp.concat(yearstr);  
                       }
                  else 
                      { 
                       temp = "19";
                       yearstr = temp.concat(yearstr);
                       }

                }
         catch(NumberFormatException n) { newint = 0; yearstr = ""; }

if (strDateFormat.compareTo("USA") == 0)
{ pattern = "/";
  newdate = newdate.concat(monthstr);
  newdate = newdate.concat(pattern);
  newdate = newdate.concat(daystr);
  newdate = newdate.concat(pattern);
  newdate = newdate.concat(yearstr);  
}

if (strDateFormat.compareTo("EUR") == 0)
{ pattern = ".";
  newdate = newdate.concat(daystr);
  newdate = newdate.concat(pattern);
  newdate = newdate.concat(monthstr);
  newdate = newdate.concat(pattern);
  newdate = newdate.concat(yearstr);
}

if (strDateFormat.compareTo("ISO") == 0)
{ pattern = "-";
  newdate = newdate.concat(yearstr);
  newdate = newdate.concat(pattern);
  newdate = newdate.concat(monthstr);
  newdate = newdate.concat(pattern);
  newdate = newdate.concat(daystr);
}

   retdate = GsDate.fromString(newdate);


     }
     catch(Exception e) { retdate = null; }

   return retdate;
}


   /**
   * Returns the date in string format.
   * An empty string is returned when the input is null or an exception occurs.
   */
   public String getDateString(GsDate date)
   {
      String temp = "";

      if (date != null)
      {
         try
         {
            temp = date.toString(new GsDateFormat(strDatePattern));
            if (temp == null)
               temp = "";
         }
         catch (Exception e) {}
      }

      return(temp);
   }


   /**
   * Returns the text corresponding to the integer "key" in the properities
   * "resourceFile" file.
   */
   public String getTextString(int key, String resourceFile,
                               IfsSession ifsSession, PrintWriter out)
                 throws IfsException
   {
      Locale locale;
      try { locale = ifsSession.getLocale(); }
      catch(Exception e) { throw new IfsException(e,"Locale not set"); }

      String s = IfsNLS.getText(resourceFile, locale, String.valueOf(key));
      return s;
   }


   /**
   * Returns the text corresponding to the string "key" in the properities
   * "resourceFile" file.
   */
   public String getTextString(String key, String resourceFile,
                               IfsSession ifsSession, PrintWriter out)
                 throws IfsException
   {
      Locale locale;
      try { locale = ifsSession.getLocale(); }
      catch(Exception e) { throw new IfsException(e,"Locale not set"); }

      String s = IfsNLS.getText(resourceFile, locale, key);
      return s;
   }


   /**
   * Verifies that the currency amount exists and is valid.
   *
   * @param amount The currency amount to be validated.
   * @return Returns true when valid, false otherwise.
   */
   public boolean isValidAmount(GsCurrency amount)
   {
      String msg="";
      try
      {
         if ( (amount == null) ||
              (amount.getDigits() == null) ||
              (amount.getDigits().length() == 0) )
            return false;

         try { double dAmount = amount.getNumeric(); }
         catch (NumberFormatException ee) { return false; }
      }
      catch (Exception e) { return false; }


      try { msg = amount.validate();
            if (msg != "OK") return false;
          }
      catch (Exception e) {msg = ""; return false; }

   return true;
   }

   /**
   * Checks if the date is valid.
   *
   * @param strDate The date to be validated.
   * @return Returns true when valid, false otherwise.
   */
   public boolean isValidDate(GsDate gsDate)
   {
      try
      {
         if (gsDate == null)
            return false;

         String temp = gsDate.toString(new GsDateFormat(strDatePattern));
         if ( (temp == null) || (temp.length() == 0) )
            return false;
      }
      catch (Exception e) { return false; }

      return true;
   }

   /**
   *  Checks if the number is valid.
   *
   * @param strNumber java.lang.String
   * @return Returns true when valid, false otherwise.
   */
   public boolean isValidNumber(String strNumber)
   {
      try
      {
         if ( (strNumber == null) ||
              (strNumber.trim().length() == 0) ||
              (Integer.parseInt(strNumber) <= 0) )
            return false;  // empty string
      }
      catch (Exception e) { return false; }

      return true;
   }

   /**
   *  Checks if the integer number is greater than zero.
   *
   * @param intNumber Integer value to be evaluated.
   * @return boolean
   */
   public boolean isValidNumber(int intNumber)
   {
      try { if (intNumber > 0) return true; }
      catch (Exception e) {}

      return false;
   }


   /**
   * Display item title and its string value
   */
   public void show2DivBlocks(PrintWriter out, String strTitle, String strValue)
   {
      if ( (strValue != null) && (strValue.length() > 0) )
      {
         out.println("<DIV ALIGN=LEFT>" + strTitle + "</DIV>");
         out.println("<DIV ALIGN=RIGHT><B>" + strValue + "</B></DIV>");
      }
   }


   /**
   * Display item title and its integer value
   */
   public void show2DivBlocks(PrintWriter out, String strTitle, int intValue)
   {
      String strValue = "";

      try { strValue = String.valueOf(intValue); }
      catch (Exception e) { strValue = "invalid integer value "; }

      show2DivBlocks(out, strTitle, strValue);
   }


   /**
   * Display item title and its account number
   */
   public void show2DivBlocks(PrintWriter out, String strTitle, GsAccount acct)
   {
      String strValue = "";

      if (acct != null)
      {
         try { strValue = acct.getNumber(); }
         catch (Exception e) { strValue = ""; }
      }

      show2DivBlocks(out, strTitle, strValue);
   }


   /**
   * Display item title and its currency value
   */
   public void show2DivBlocks(PrintWriter out, String strTitle, GsCurrency amtValue)
   {
      String strValue = "";

      if (amtValue != null)
      {
         try
         {
            amtValue.getFormat().setFormatStyle(IfsSystem.CURRENCY_TEXT_STYLE);
            amtValue.setDigits(amtValue.getDigits());
            strValue = amtValue.toString();
         }
         catch (Exception e) { strValue = ""; }
      }

      show2DivBlocks(out, strTitle, strValue);
   }


   /**
   * Display item title and its date value
   */
   public void show2DivBlocks(PrintWriter out, String strTitle, GsDate dateValue)
   {
      String strValue = "";

      if (dateValue != null)
      {
         try { strValue = dateValue.toString(new GsDateFormat(strDatePattern)); }
         catch (Exception e) { strValue = ""; }
      }

      show2DivBlocks(out, strTitle, strValue);
   }


   /**
   * Display item title and its decimal value
   */
   public void show2DivBlocks(PrintWriter out, String strTitle, GsDecimal decValue)
   {
      String strValue = "";

      if (decValue != null)
      {
         try { strValue = decValue.toString(IfsSystem.gsDecimalFormat); }
         catch (Exception e) { strValue = ""; }
      }

      show2DivBlocks(out, strTitle, strValue);
   }


   /**
   * Setup for outputting HTML information.
   */
   public PrintWriter startHtmlOutput(HttpServletResponse response, String title)
          throws IOException
   {
      // Need to have setDefaultHeader set charset before getting the writer.
      setDefaultHeader(response);
      PrintWriter out = response.getWriter();

      out.println("<HTML><HEAD>");
      // **commented by Sonya Johnson 1/14/2002
      // addCopyright(response);
      out.println("<META NAME=HandheldFriendly CONTENT=True >");
      out.println("<TITLE>" + title);
      out.println("</TITLE> </HEAD> <BODY BGCOLOR=\"#FFFFFF\" >");

      return(out);
   }

}

