//-*************************************
//
// FILE NAME:  IfsWpBalanceInquiry.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//-*************************************
 
/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/14/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resources.*;
 *                        1/18/2002    added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */

import java.io.*;
import java.net.URLEncoder;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This class is used for the consumer to request various balances associated
* with a specific consumer account.
*/
public class IfsWpBalanceInquiry extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   //*******************************
   // Displays balance of a specific account or a list of all of the user's accounts.
   //
   // The following Http Request Parameters are posted to this page:
   //     ReqTag       - Specifies whether a list of all accounts or the balance
   //                    for a specific account is to be displayed.
   //                    It can have values of "AllAccounts" or "Details".
   //     AccountIndex - When ReqTag equals "Details", it specifies the index
   //                    of the account to be displayed.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);

      GtUserProfile up;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/14/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         String reqTag = (String)getRequestParameter(request,"ReqTag");

         if (reqTag.equals("AllAccounts"))
            showAllAccounts(up, request, response);

         else if (reqTag.equals("Details"))
            showBalanceInfo(up, ifsSession, request, response);

         else
            callPage_RedoPage(response, "IfsWpBalanceInquiry",
                   "Unable to do balance inquiry.  Unexpected ReqTag value. ReqTag = " + reqTag);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
             "Balance Inquiry failed. Exception in doGet method of IfsWpBalanceInquiry Servlet, explanation: ");
      }
   }

   /**
   * Display a list of all accounts
   */
   private void showAllAccounts(GtUserProfile up,
                                HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GsAcctDtl acct;
      String acctNbr;
      String acctNick;

      PrintWriter out = startHtmlOutput(response, "BALANCE INQUIRY");

      out.println("<B>Please select an account:</B><HR>");

      for (int i = 0; i < up.getAccountDetail().length; ++i)
      {
         try
         {
            acct = up.getAccountDetail()[i];
            acctNbr = acct.getAccount().getNumber();
            acctNick = acct.getAccount().getNickname();
         }
         catch (Exception e) { continue; }

         out.println ("<A href=\"IfsWpBalanceInquiry?AccountIndex=" +
                      String.valueOf(i) + "&ReqTag=Details\">");
         out.println (acctNbr + " -- " + acctNick + "</A><BR>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Display balance information of selected account
   */
   private void showBalanceInfo(GtUserProfile up, IfsSession ifsSession,
                                HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      String acctNbr = null;
      String type = null;
      String acctNick = null;
      GtBalanceInquiry txn = null;

      GsAcctDtl acct = null;
      int AccountIndex = 0;
      String strIndex = null;
      String strValue = "";

      try
      {
         
         // get account index
         strIndex = (String)getRequestParameter(request,"AccountIndex");
         AccountIndex = Integer.parseInt(strIndex);
         acct = up.getAccountDetail()[AccountIndex];

         // get values
         type = acct.getAccount().getType();
         acctNbr = acct.getAccount().getNumber();
         acctNick = acct.getAccount().getNickname();

         // create account
         GsAccount anAccount = new GsAccount(type, acctNbr, acctNick);
         anAccount.setIfsSession(ifsSession);

         // create details txn
         txn = new GtBalanceInquiry(ifsSession);

         // execute
         txn.setReqAccountID(anAccount);

         //added try-catch block
         try
       {

        txn.execute();
       } catch (Exception e )  {
           callPage_ErrorPage(response, e,
                "Exception in execute method: ");
       }
        if (! handleErrors(txn, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpBalanceInquiry",
                      "IfsWpRedoPage?nextpage=IfsWpBalanceInquiry")){
            return;
        }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
             "Balance Inquiry failed. Exception in showBalanceInfo method of IfsWpBalanceInquiry Servlet, explanation: ");
      }

      //-----------
      // Start outputting the account balance information.
      //
      PrintWriter out = startHtmlOutput(response, "ACCOUNT DETAILS");

      // Account title
      out.println("<DIV ALIGN=CENTER><B>" + acctNick + "</B></DIV><P><HR>");

      show2DivBlocks(out, "Information as of:", txn.getInformationDate());
      show2DivBlocks(out, "Ledger Balance:", txn.getLedgerBalanceAmount());
      show2DivBlocks(out, "Available Balance:", txn.getAvailableBalanceAmount());
      show2DivBlocks(out, "Uncleared Balance:", txn.getUnclearedFundAmount());
      show2DivBlocks(out, "Total Holding Balance:", txn.getTotalHoldAmount());
      show2DivBlocks(out, "Total Credit Line Amount:", txn.getTotalCreditLineAmount());
      show2DivBlocks(out, "Principal Balance:", txn.getPrincipalBalanceAmount());
      show2DivBlocks(out, "Outstanding Balance:", txn.getOutstandingBalanceAmount());
      show2DivBlocks(out, "Payoff Amount:", txn.getPayoffAmount());

      out.println("<HR>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }
}
