//-*************************************
//
// FILE NAME:  IfsWpChangeVariableTransfer.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 *   Author                  Date        Changes
 * ----------------------  ----------  -----------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */ 

import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import javax.servlet.http.*;
import javax.servlet.*;
import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

/**
* This class is used to handle the transaction of change variable transfer.
*/
public class IfsWpChangeVariableTransfer extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;

   private String transferHistoryInquiryOid = "";
   private int transferHistoryInquiryIndex = 0;

   /**
   * This method displays a message page..
   */
   private void confirmChangeSingle(GtChangeVariableTransfer transfer, IfsSession ifsSession,
                                    HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      String oid = "";
      String msg = "";
      String reqTransactionAmount = "";
      String strValue = "";
      try
      {
         // Get the object ID of the cached GtChangeVariableTransfer instance object.
         oid = getOid(ifsSession,transfer);

         // Validate the data that was entered.
         msg = validateChangeVariable(transfer);

         if ( msg.equals("OK") == false)
         {
            callPage_RedoPage(response, "IfsWpChangeVariableTransfer&transfer=" +
                  transferHistoryInquiryOid +
                  "&i=" + String.valueOf(transferHistoryInquiryIndex), msg);
         }

         // format the transaction amount.
         transfer.getReqTransactionAmount().getFormat().setFormatStyle(IfsSystem.CURRENCY_TEXT_STYLE);
         transfer.getReqTransactionAmount().setDigits(transfer.getReqTransactionAmount().getDigits());
         reqTransactionAmount = transfer.getReqTransactionAmount().toString();

         if (reqTransactionAmount == null)
             reqTransactionAmount = "";

         if (transfer.getReqTransferNickname() == null)
            transfer.setReqTransferNickname("");

         if (transfer.getReqTfrSrceAcctRefr() == null)
            transfer.setReqTfrSrceAcctRefr("");

         if (transfer.getReqTfrTrgtAcctRefr() == null)
            transfer.setReqTfrTrgtAcctRefr("");

         if (transfer.getReqFIReferenceNumber() == null)
            transfer.setReqFIReferenceNumber("");

         if (transfer.getReqTmstpVal() == null)
            transfer.setReqTmstpVal("");

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to do change variable transfer. Exception caught in confirmAddSingle method of IfsWpChangeVariableTransfer Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Confirm Change Variable Transfer");

      out.println("<B>Please click OK button to confirm your transfer change </B>");
      out.println("<HR>");

      show2DivBlocks (out, "Transfer From Account:", transfer.getReqSource());
      show2DivBlocks (out, "Transfer To Account:", transfer.getReqTarget());

      show2DivBlocks (out, "Amount:", reqTransactionAmount);
      show2DivBlocks (out, "Transfer Number:", transfer.getReqTransferNumber());
      show2DivBlocks (out, "Transfer Nickname:", transfer.getReqTransferNickname());

      out.println("<CENTER><P><TABLE>");
      out.println("<TR><TD>");
      out.println("<FORM METHOD=POST NAME=ChangeVariableTransfer  ACTION=IfsWpChangeVariableTransfer >");
      out.println("<INPUT TYPE=HIDDEN NAME=whereFlag  VALUE=\"fromConfirm\" >");
      out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + oid + "\" >");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Ok\" ></FORM></TD>");

      out.println("<TD><FORM METHOD=get NAME=TransferCancel ACTION=IfsWpChangeVariableTransfer >");
      out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + transferHistoryInquiryOid + "\" >");
      out.println("<INPUT TYPE=hidden NAME=i VALUE=\"" + String.valueOf(transferHistoryInquiryIndex) + "\" >");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Cancel\" ></FORM></TD>");
      out.println("</TR>");
      out.println("</TABLE><P>");
   }


   //*******************************
   // Displays the input form for generating a Change Variable Transfer transaction.
   //
   // The following Http Request Parameters are posted to this page:
   //     transfer - Specifies the object identifier (oid) of a
   //                GtTransferHistoryInquiry object in the session cache.
   //     i        - The transfer history detail index.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;

      GtTransferHistoryInquiry transfer = null;
      GsTfrDtl detail = null;
      GtChangeVariableTransfer cvt = null;

      int ipointer = 0;
      GsAccount source = null;
      GsAccount target = null;
      String    amount = "";
      String    transfer_identifier = "";
      String    nickname = "";
      String    effectiveDateStr = "";

      try
      {
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtTransferHistoryInquiry object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transfer = (GtTransferHistoryInquiry) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtTransferHistoryInquiry",
                             "transfer");
         try
         {
            // Set fields of GtTransferHistoryInquiry object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transfer",transfer);
         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         ipointer = Integer.parseInt( (String)getRequestParameter(request,"i"));

         // Save for a possible REDO request.
         transferHistoryInquiryOid = getOid(ifsSession,transfer);
         transferHistoryInquiryIndex = ipointer;

         detail = transfer.getTransferHistoryDetail()[ipointer];

         source = detail.getSource();
         target = detail.getTarget();

         amount = getAmountString(detail.getTransactionAmount());
         transfer_identifier = detail.getTransferNumber();

         effectiveDateStr = getDateString(detail.getEffectiveDate());

         nickname = detail.getTransferNickname();
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to change variable transfer. Exception in doGet method of IfsWpChangeVariableTransfer Servlet, explanation: ");
      }

      try
      {
         //---------------
         // Check for redisplay of originally entered information.
         // Get a reference to the GtChangeVariableTransfer cached object.
         //
         cvt = (GtChangeVariableTransfer) getObject(ifsSession, CHANGE_VARIABLE_TRANSFER);
         if (cvt != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            source = cvt.getReqSource();
            target = cvt.getReqTarget();

            amount = getAmountString(cvt.getReqTransactionAmount());

            effectiveDateStr = getDateString(cvt.getReqEffectiveDate());

            nickname = "";
            if (cvt.getReqTransferNickname() != null)
               nickname = cvt.getReqTransferNickname();

            // Finished with cached object, remove it.
            removeObject(ifsSession, CHANGE_VARIABLE_TRANSFER);
         }
      }
      catch ( Exception e) {removeObject(ifsSession, CHANGE_VARIABLE_TRANSFER); }

      PrintWriter out = startHtmlOutput(response, "Change Variable Transfer");

      out.println("<FORM METHOD=POST NAME=ChangeVariableTransfer ACTION=IfsWpChangeVariableTransfer >");
      out.println("<INPUT TYPE=HIDDEN NAME=reqTransferNumber  VALUE=\"" + transfer_identifier + "\">");
      out.println("<INPUT TYPE=HIDDEN NAME=whereFlag  VALUE=\"fromForm\" >");
      out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + CHANGE_VARIABLE_TRANSFER + "\" >");
      out.println("<INPUT TYPE=HIDDEN NAME=i  VALUE=\"" + String.valueOf(ipointer) + "\" >");

      out.println("<TABLE BORDER=0   >");

      // account data
      out.println("<TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Transfer From Account: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<SELECT NAME=reqSource >");
      displayAccounts(up, source, out);
      out.println("</SELECT></TD></TR>");

      out.println("<TR ALIGN=CENTER>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Transfer To Account: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<SELECT NAME=reqTarget >");
      displayAccounts(up, target, out);
      out.println("</SELECT></TD></TR>");

      out.println("<TR>");
      out.println("<TD  ALIGN=LEFT ><B> Amount:</B> " + strCurrencyPrefix + "</TD>");
      out.println("</TR><TR>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<INPUT NAME=reqTransactionAmount TYPE=text SIZE=13 MAXLENGTH=20 VALUE=\"" +
                   amount + "\" >");
      out.println("</TD></TR>");

      out.println("<TR ALIGN=LEFT>");
      out.println("<TD  ALIGN=LEFT NOWRAP><B> Effective Date: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<INPUT NAME=reqEffectiveDate TYPE=text SIZE=10 MAXLENGTH=10  VALUE=\"" +
                  effectiveDateStr + "\" > (" + strDateHint + ")");
      out.println("</TD></TR>");

      out.println("<TR ALIGN=LEFT>");
      out.println("<TD ALIGN=LEFT NOWRAP><B>Transfer Nickname: </B></TD>");
      out.println("</TR><TR ALIGN=Left>");
      out.println("<TD  ALIGN=LEFT>");
      out.println("<INPUT NAME=reqTransferNickname TYPE=text SIZE=25 MAXLENGTH=30 VALUE=\"" +
                  nickname + "\">");
      out.println("</TD></TR>");

      out.println("<TR ALIGN=CENTER>");
      out.println("<TD><BR><INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Change\" > </FORM></TD>");
      out.println("</TR>");
      out.println("</TABLE>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   //*******************************
   // Either displays the confirmation page for a Change Variable Transfer
   // transaction or executes it.
   //
   // The following Http Request Parameters are posted to this page:
   //     whereFlag - Indicates whether to display confirmation or execute.
   //                 It can have the values of "fromForm" or "fromConfirm".
   //     transfer  - Specifies the object identifier (oid) of a
   //                 GtChangeVariableTransfer object.
   //     req?????  - The input values from the Change Variable Transfer form.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;
      GtChangeVariableTransfer transfer = null;

      try
      {
        //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtChangeVariableTransfer object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transfer = (GtChangeVariableTransfer) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtChangeVariableTransfer",
                             "transfer");
         try
         {
            //---------------
            // Set the fields of the GtChangeVariableTransfer object instance from the
            // corresponding request parameters values in the HTTP request.
            //
            setFields(ifsSession,"transfer",transfer);

            // insert the first 2 digits of the year with 80 as the cut-off value.
            GsDate date =  new GsDate();
            date = yearWindow(transfer.getReqEffectiveDate());
            transfer.setReqEffectiveDate(date);

         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         String strReq = (String)getRequestParameter(request,"whereFlag");

         if (strReq.equals("fromForm"))
            confirmChangeSingle(transfer, ifsSession, request, response);
         else
            processChangeSingle(transfer, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to change variable transfer. Exception in doPost method of IfsWpChangeVariableTransfer Servlet, explanation: ");
      }
   }

   /**
   * This method processes the transaction..
   */
   private void processChangeSingle(GtChangeVariableTransfer transfer,
                                    HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      String strValue = "";
      try
      {
       //added try-catch block 1/18/2002
         try
        {
        
         transfer.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(transfer, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpTransfer",
                      "IfsWpRedoPage?nextpage=IfsWpTransfer")){
             return;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to change variable transfer. Exception in processChangeSingle method of IfsWpChangeVariableTransfer Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Change Variable Transfer results");

      out.println("<B>Change Variable Transfer transaction was successful.</B>");
      out.println("<HR>");

      show2DivBlocks (out, "Confirmation Reference Number:", transfer.getReferenceNumber());

      GsDate adjDate = transfer.getAdjustedTransferDate();
      try
      {
         if ( (adjDate != null) && (adjDate.toString().length() != 0) )
            show2DivBlocks (out, "Adjusted Transfer Date:", adjDate);
      }
      catch (Exception e) {}

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   /**
   * Validate the data that was entered.
   */
   private String validateChangeVariable(GtChangeVariableTransfer cv)
   {
      StringBuffer strBuf = new StringBuffer("");

      double dAmount = 0;
      String reqEffectiveDate = "";

      // Get the transaction amount information.
      try { dAmount = getAndCheckAmount(cv.getReqTransactionAmount()); }
      catch (Exception e)
      {
         cv.debug("Exception getting amount in validateChangeVariable - " + e.toString());
         dAmount = 0;
      }

      // Get the effective Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         reqEffectiveDate = cv.getReqEffectiveDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (reqEffectiveDate == null)
            reqEffectiveDate = "";
      }
      catch (Exception e)
      {
         cv.debug("Exception getting effective date in validateChangeVariable - " + e.toString());
         reqEffectiveDate = "";
      }

      // Validate the data that was entered.
      if (dAmount <= 0)
         strBuf.append("<BR>Must specify a valid positive non-zero amount within the allowed Currency range.");

      if (reqEffectiveDate.equals(""))
         strBuf.append("<BR>Must specify a valid effective date.");

      if ( cv.getReqSource().getNumber().equals(cv.getReqTarget().getNumber()) )
         strBuf.append("<BR>The source and target accounts cannot be the same account!");

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         cv.debug("Validation failed in validateChangeVariable method of IfsWpChangeVariableTransfer class");
         cv.debug(strBuf.toString());
      }

      return strBuf.toString();
   }

}
