//-*************************************
//
// FILE NAME:  IfsWpCheckBook.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resource.*;
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */


import java.io.*;
import java.net.URLEncoder;
import javax.servlet.http.*;
import javax.servlet.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This class is used to handle the transaction of check book reorder.
*/
public class IfsWpCheckBook extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   /**
   * Checks user input and displays the confirm information to user.
   */
   private void displayConfirmInfo(GtUserProfile up, IfsSession ifsSession,
                                   GtOrderCheckBook checkBook,
                                   HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      String oid = null;
      String msg = null;
      String strValue = null;

      try
      {
         // Get the object ID of the cached GtOrderCheckBook instance object.
         oid = getOid(ifsSession,checkBook);

         // Validate the data that was entered.
         msg = validateCheckBook(checkBook);
         if ( msg.equals("OK") == false)
            callPage_RedoPage(response, "IfsWpCheckBook", msg);

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in displayConfirmInfo method of IfsWpCheckBook Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Confirm Request");

      out.println("Your check book reorder will be submitted. To confirm, click on <B>OK</B>, otherwise, click on <B>Cancel.</B>");
      out.println("<P><B>Note: Your check book will be mailed to the address on the account selected within 10 working days. There will be a $5.00 fee charged to your account.</B>");

      out.println("<HR>");

      show2DivBlocks(out, "Account:", checkBook.getReqAccount());
      show2DivBlocks(out, "Check book style:", checkBook.getReqCheckBookStyle());
      show2DivBlocks(out, "Number of checks:", checkBook.getReqNumberCheck());

      // Check Book Delivery Method
      try { strValue = getTextString(checkBook.getReqDeliveryMethod(),
                                     "IfsDlvyMethIndResources", ifsSession, out); }
      catch (Exception e) { strValue = "Invalid check delivery method:"; }
      show2DivBlocks(out, "Check book Delivery Method:", strValue);


      show2DivBlocks(out, "Consumer Nickname:", checkBook.getReqConsumerNickname() );
      show2DivBlocks(out, "Consumer Account Name:", checkBook.getReqConsumerAccountName() );
      show2DivBlocks(out, "Address Line 1:", checkBook.getReqAddressLine1() );
      show2DivBlocks(out, "Address Line 2:", checkBook.getReqAddressLine2() );
      show2DivBlocks(out, "Address Line 3:", checkBook.getReqAddressLine3() );

      show2DivBlocks(out, "City:", checkBook.getReqCity() );
      show2DivBlocks(out, "Province:", checkBook.getReqProvince() );
      show2DivBlocks(out, "State:", checkBook.getReqState() );
      show2DivBlocks(out, "Country:", checkBook.getReqCountry() );
      show2DivBlocks(out, "Zip code:", checkBook.getReqZipCode() );
      show2DivBlocks(out, "Postal code:", checkBook.getReqPostalCode() );

      out.println("<P> <CENTER>");
      out.println("<TABLE WIDTH=\"100%\" BORDER=0 >");
      out.println("<TR><TD ALIGN=RIGHT >");
      out.println("<FORM METHOD=POST ACTION=IfsWpCheckBook >");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"Result\" >");
      out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + oid + "\">");
      out.println("<INPUT TYPE=submit NAME=OKButton  VALUE=\"OK\" >");
      out.println("</FORM></TD>");

      out.println("<TD ALIGN=LEFT>");
      out.println("<FORM METHOD=get ACTION=IfsWpCheckBook >");
      out.println("<INPUT TYPE=submit NAME=CancelButton  VALUE=\"Cancel\" >");
      out.println("</FORM></TD>");
      out.println("</TR></TABLE>");

      out.println("</CENTER></BODY></HTML>");
      out.close();
   }

   /**
   * Displays the input form for user.
   */
   private void displayInputForm(GtUserProfile up, IfsSession ifsSession,
                                 HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GtOrderCheckBook  cb      = null;
      GsAccount         account = null;
      String       deliveryMethod         = "";
      String       checkBookStyle         = "";
      String       reqNumberCheck         = "";
      String       reqConsumerNickname    = "";
      String       reqConsumerAccountName = "";
      String       reqAddressLine1        = "";
      String       reqAddressLine2        = "";
      String       reqAddressLine3        = "";
      String       reqCity                = "";
      String       reqProvince            = "";
      String       reqState               = "";
      String       reqCountry             = "";
      String       reqZipCode             = "";
      String       reqPostalCode          = "";

      try
      {
         //---------------
         // Need to redisplay the originally entered information.
         // Get a reference to the GtCheckCopy cached object.
         //
         cb = (GtOrderCheckBook) getObject(ifsSession, ORDER_CHECK_BOOK);
         if (cb != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            account = cb.getReqAccount();
            deliveryMethod = cb.getReqDeliveryMethod();
            checkBookStyle = cb.getReqCheckBookStyle();

            if (cb.getReqNumberCheck() != 0 )
               reqNumberCheck = String.valueOf(cb.getReqNumberCheck());

            reqConsumerNickname = cb.getReqConsumerNickname();
            if (reqConsumerNickname == null) reqConsumerNickname = "";

            reqConsumerAccountName = cb.getReqConsumerAccountName();
            if (reqConsumerAccountName == null) reqConsumerAccountName = "";

            reqAddressLine1 = cb.getReqAddressLine1();
            if (reqAddressLine1 == null) reqAddressLine1 = "";

            reqAddressLine2 = cb.getReqAddressLine2();
            if (reqAddressLine2 == null) reqAddressLine2 = "";

            reqAddressLine3 = cb.getReqAddressLine3();
            if (reqAddressLine3 == null) reqAddressLine3 = "";

            reqCity = cb.getReqCity();
            if (reqCity == null) reqCity = "";

            reqProvince = cb.getReqProvince();
            if (reqProvince == null) reqProvince = "";

            reqState = cb.getReqState();
            if (reqState == null) reqState = "";

            reqCountry = cb.getReqCountry();
            if (reqCountry == null) reqCountry = "";

            reqZipCode = cb.getReqZipCode();
            if (reqZipCode == null) reqZipCode = "";

            reqPostalCode = cb.getReqPostalCode();
            if (reqPostalCode == null) reqPostalCode = "";
         }
      }
      catch ( Exception e) { }

      PrintWriter out = startHtmlOutput(response, "Check Book Reorder");

      out.println("<FORM METHOD=POST ACTION=IfsWpCheckBook >");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"Confirm\" >");
      out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + ORDER_CHECK_BOOK + "\" >");

      out.println("<DIV><B>Select an Account:</B></DIV>");
      out.println("<DIV><SELECT NAME=reqAccount >");
      displayAccounts(up, account, out);
      out.println("</SELECT></DIV>");

      // Check Book Style list box
      out.println("<DIV><B>Select Check Book Style:</B></DIV>");
      out.println("<DIV><SELECT NAME=reqCheckBookStyle >");
      displayCheckBookStyleList(checkBookStyle, out);
      out.println("</SELECT></DIV>");

      // Number of Checks input field
      out.println("<DIV><B>Number of Checks:</B>");
      out.println("<INPUT NAME=reqNumberCheck TYPE=text SIZE=6 MAXLENGTH=8 VALUE=\"" +
                  reqNumberCheck + "\" > </DIV>");

      // Check Copy Delivery Method list box
      out.println("<DIV><B>Check Book Delivery Method:</B></DIV>");
      out.println("<DIV><SELECT NAME=reqDeliveryMethod >");
      displayPostalSelectList(deliveryMethod, out);
      out.println("</SELECT></DIV>");

      // Consumer Nickname input field
      out.println("<DIV><B>Consumer Nickname:</B></DIV>");
      out.println("<DIV><INPUT NAME=reqConsumerNickname TYPE=text SIZE=25 MAXLENGTH=30 VALUE=\"" +
                  reqConsumerNickname + "\"></DIV>");

      // Consumer Account Name input field
      out.println("<DIV><B>Consumer Account Name:</B></DIV>");
      out.println("<DIV><INPUT NAME=reqConsumerAccountName TYPE=text SIZE=25 MAXLENGTH=40 VALUE=\"" +
                  reqConsumerAccountName + "\"></DIV>");

      // Consumer Address Information input field
      out.println("<P><DIV ALIGN=CENTER>Consumer Address (optional)</DIV><HR>");
      out.println("<DIV><B>Address Line 1:</B></DIV>");
      out.println("<DIV><INPUT NAME=reqAddressLine1 TYPE=text SIZE=25 MAXLENGTH=40 VALUE=\"" +
                  reqAddressLine1 + "\"></DIV>");
      out.println("<DIV><B>Address Line 2:</B></DIV>");
      out.println("<DIV><INPUT NAME=reqAddressLine2 TYPE=text SIZE=25 MAXLENGTH=40 VALUE=\"" +
                  reqAddressLine2 + "\"></DIV>");
      out.println("<DIV><B>Address Line 3:</B></DIV>");
      out.println("<DIV><INPUT NAME=reqAddressLine3 TYPE=text SIZE=25 MAXLENGTH=40 VALUE=\"" +
                  reqAddressLine3 + "\"></DIV>");

      out.println("<TABLE WIDTH=\"100%\" BORDER=0 >");
      out.println("<TR><TD ALIGN=LEFT><B>City:</B></TD>");
      out.println("<TD ALIGN=LEFT><INPUT TYPE=text NAME=reqCity SIZE=10 MAXLENGTH=30 VALUE=\"" +
                  reqCity + "\"></TD></TR>");
      out.println("<TR><TD ALIGN=LEFT><B>Province:</B></TD>");
      out.println("<TD ALIGN=LEFT><INPUT TYPE=text NAME=reqProvince SIZE=10 MAXLENGTH=30 VALUE=\"" +
                  reqProvince + "\"></TD></TR>");
      out.println("<TR><TD ALIGN=LEFT><B>State:</B></TD>");
      out.println("<TD ALIGN=LEFT><INPUT TYPE=text NAME=reqState SIZE=10 MAXLENGTH=20 VALUE=\"" +
                  reqState + "\"></TD></TR>");
      out.println("<TR><TD ALIGN=LEFT><B>Country:</B></TD>");
      out.println("<TD ALIGN=LEFT><INPUT TYPE=text NAME=reqCountry SIZE=10 MAXLENGTH=20 VALUE=\"" +
                  reqCountry + "\"></TD></TR>");
      out.println("<TR><TD ALIGN=LEFT><B>Zip Code:</B></TD>");
      out.println("<TD ALIGN=LEFT><INPUT TYPE=text NAME=reqZipCode SIZE=10 MAXLENGTH=9 VALUE=\"" +
                  reqZipCode + "\"></TD></TR>");
      out.println("<TR><TD ALIGN=LEFT><B>Postal Code:</B></TD>");
      out.println("<TD ALIGN=LEFT><INPUT TYPE=text NAME=reqPostalCode SIZE=10 MAXLENGTH=6 VALUE=\"" +
                  reqPostalCode + "\"></TD></TR>");
      out.println("</TABLE>");

      out.println("<P><CENTER><INPUT NAME=SUBMIT TYPE=submit VALUE=\"Enter\" ></P>");
      out.println("</CENTER></FORM>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   //*******************************
   // Displays the input form for user to generate a Order Check Book transaction.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         displayInputForm(up, ifsSession, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in doGet method of IfsWpCheckBook Servlet, explanation: ");
      }
   }

   //*******************************
   // Either displays the confirmation page for a Order Check Book
   // transaction or executes it.
   //
   // The following Http Request Parameters are posted to this page:
   //     ReqTag   - Indicates whether to display confirmation or execute.
   //                It can have the values of "Confirm" or "Result".
   //     transfer - Specifies the object identifier (oid) of a
   //                GtOrderCheckBook object.
   //     req????  - The input values from the Order Check Book form.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;
      GtOrderCheckBook transact = null;
      String strReqTag = null;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtOrderCheckBook object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transact = (GtOrderCheckBook) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtOrderCheckBook",
                             "transact");

         // clearing cached data.
         transact.setReqNumberCheck(0);


         try
         {
            // Set fields of GtOrderCheckBook object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transact",transact);
         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         strReqTag = (String)getRequestParameter(request,"ReqTag");

         if (strReqTag.equals("Confirm"))
            displayConfirmInfo(up, ifsSession, transact, request, response);
         else if (strReqTag.equals("Result"))
            showTxnResults(ifsSession, transact, request, response);

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in doPost method of IfsWpCheckBook Servlet, explanation: ");
      }

   }

   /**
   * Displays the transaction results.
   */
   private void showTxnResults(IfsSession ifsSession, GtOrderCheckBook transact,
                               HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      try
      {
         //added try-catch block 1/18/2002
         try
        {
        
         transact.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(transact, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpCheckBook",
                      "IfsWpRedoPage?nextpage=IfsWpCheckBook")){
             return;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to order check book. Exception caught in showTxnResults method of IfsWpCheckBook Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Check Book Requested");

      out.println("<P>Your check book has been requested successfully!<HR>");

      show2DivBlocks(out, "FI Reference Number:", transact.getReferenceNumber() );
      show2DivBlocks(out, "Service Provider Reference Id:",
                     transact.getServiceProviderReference() );

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Validate the data that was entered.
   */
   private String validateCheckBook(GtOrderCheckBook ocb)
   {
      StringBuffer strBuf = new StringBuffer("");

      // Validate the data that was entered.

      if ( isValidNumber(ocb.getReqNumberCheck()) == false )
         strBuf.append("<BR>Must specify a valid positive non-zero number of checks.");

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         ocb.debug("Validation failed in validateCheckBook method of IfsWpCheckBook class");
         ocb.debug(strBuf.toString());
      }

      return strBuf.toString();
   }

}
