//-*************************************
//
// FILE NAME:  IfsWpCheckCopy.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resource.*;
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */


import java.io.*;
import java.net.URLEncoder;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This class is used to handle the transaction of check copy order.
*/
public class IfsWpCheckCopy extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   /**
   * Checks user input and displays the confirm information to user.
   */
   private void displayConfirmInfo(GtUserProfile up, IfsSession ifsSession,
                                   GtCheckCopy checkCopy,
                                   HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      String oid = null;
      String msg = null;
      String strValue = null;

      try
      {
         // Get the object ID of the cached GtCheckCopy instance object.
         oid = getOid(ifsSession,checkCopy);

         // Validate the data that was entered.
         msg = validateCheckCopy(checkCopy, request);
         if ( msg.equals("OK") == false)
            callPage_RedoPage(response, "IfsWpCheckCopy", msg);

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in displayConfirmInfo method of IfsWpCheckCopy Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Confirm Request");

      out.println("Your check copy request will be submitted. To confirm, click on <B>OK</B>, otherwise, click on <B>Cancel.</B>");
      out.println("<P></P><B>Note: Your check copy will be mailed to the address on the account selected within 10 working days. There will be a $3.00 fee charged to your account for each check copy.</B>");

      out.println("<HR>");

      show2DivBlocks(out, "Account:", checkCopy.getReqAccountID());
      show2DivBlocks(out, "Check Number:", checkCopy.getReqCheckSerialNumber());
      show2DivBlocks(out, "Check Amount:", checkCopy.getReqCheckAmount());
      show2DivBlocks(out, "Posting Date:", checkCopy.getReqCheckPostingDate());
      show2DivBlocks(out, "Reference Number:", checkCopy.getReqReferenceNumber() );

      // Check Book Delivery Method
      try { strValue = getTextString(checkCopy.getReqDeliveryMethodIndicator(),
                                     "IfsDlvyMethIndResources", ifsSession, out); }
      catch (Exception e) { strValue = "Invalid check delivery method:"; }
      show2DivBlocks(out, "Check Copy Delivery Method:", strValue);

      show2DivBlocks(out, "Check Description:", checkCopy.getReqCheckDescription() );
      show2DivBlocks(out, "Home Phone Number:", checkCopy.getReqHomePhone() );
      show2DivBlocks(out, "Work Phone Number:", checkCopy.getReqWorkPhone() );
      show2DivBlocks(out, "Fax Phone Number:", checkCopy.getReqFaxNumber() );
      show2DivBlocks(out, "E-Mail Address:", checkCopy.getReqEmailAddress() );

      out.println("<P> <CENTER>");
      out.println("<TABLE WIDTH=\"100%\" BORDER=0 >");
      out.println("<TR><TD ALIGN=RIGHT >");
      out.println("<FORM METHOD=POST ACTION=IfsWpCheckCopy >");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"Result\" >");
      out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + oid + "\">");
      out.println("<INPUT TYPE=submit NAME=OKButton  VALUE=\"OK\" >");
      out.println("</FORM></TD>");

      out.println("<TD ALIGN=LEFT>");
      out.println("<FORM METHOD=get ACTION=IfsWpCheckCopy>");
      out.println("<INPUT TYPE=submit NAME=CancelButton  VALUE=\"Cancel\" >");
      out.println("</FORM></TD>");
      out.println("</TR></TABLE>");

      out.println("</CENTER></BODY></HTML>");
      out.close();

   }

   /**
   * Displays the input form for user.
   */
   private void displayInputForm(GtUserProfile up, IfsSession ifsSession,
                                 HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GtCheckCopy  cp      = null;
      GsAccount    account = null;
      String       deliveryMethod       = "";
      String       reqCheckSerialNumber = "";
      String       reqCheckAmount       = "";
      String       reqCheckPostingDate  = "";
      String       reqReferenceNumber   = "";
      String       reqCheckDescription  = "";
      String       reqHomePhone         = "";
      String       reqWorkPhone         = "";
      String       reqFaxNumber         = "";
      String       reqEmailAddress      = "";

      try
      {
         //---------------
         // Need to redisplay the originally entered information.
         // Get a reference to the GtCheckCopy cached object.
         //
         cp = (GtCheckCopy) getObject(ifsSession, CHECK_COPY);
         if (cp != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            account = cp.getReqAccountID();
            deliveryMethod = cp.getReqDeliveryMethodIndicator();

            reqCheckAmount = getAmountString(cp.getReqCheckAmount());
            reqCheckPostingDate = getDateString(cp.getReqCheckPostingDate());

            if (cp.getReqCheckSerialNumber() != 0 )
               reqCheckSerialNumber = String.valueOf(cp.getReqCheckSerialNumber());

            reqReferenceNumber = cp.getReqReferenceNumber();
            if (reqReferenceNumber == null) reqReferenceNumber = "";

            reqCheckDescription = cp.getReqCheckDescription();
            if (reqCheckDescription == null) reqCheckDescription = "";

            reqHomePhone = cp.getReqHomePhone();
            if (reqHomePhone == null) reqHomePhone = "";

            reqWorkPhone = cp.getReqWorkPhone();
            if (reqWorkPhone == null) reqWorkPhone = "";

            reqFaxNumber = cp.getReqFaxNumber();
            if (reqFaxNumber == null) reqFaxNumber = "";

            reqEmailAddress = cp.getReqEmailAddress();
            if (reqEmailAddress == null) reqEmailAddress = "";
         }
      }
      catch ( Exception e) { }

      PrintWriter out = startHtmlOutput(response, "Check Copy Order");

      out.println("<FORM METHOD=POST ACTION=IfsWpCheckCopy >");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"Confirm\" >");
      out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + CHECK_COPY + "\" >");

      out.println("<DIV><B>Select an Account:</B></DIV>");
      out.println("<DIV><SELECT NAME=reqAccountID >");
      displayAccounts(up, account, out);
      out.println("</SELECT></DIV>");

      // Check Number input field
      out.println("<BR> <B>Check Number: </B>");
      out.println("<INPUT NAME=reqCheckSerialNumber TYPE=text SIZE=10 MAXLENGTH=9 VALUE=\"" +
                  reqCheckSerialNumber + "\"> ");

      // Amount input field
      out.println("<BR> <B>Amount:    </B>");
      out.println(strCurrencyPrefix +
                  " <INPUT NAME=reqCheckAmount TYPE=text SIZE=13 MAXLENGTH=20 VALUE=\"" +
                  reqCheckAmount + "\" >");

      // Posting Date input field
      out.println("<BR> <B>Posting Date:</B> (optional)");
      out.println("<BR> <INPUT NAME=reqCheckPostingDate TYPE=text SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  reqCheckPostingDate + "\" > " + strDateHint);

      // FI Reference Number input field
      out.println("<BR> <B>Reference Number:</B> (optional)");
      out.println("<BR> <INPUT NAME=reqReferenceNumber TYPE=text SIZE=10 MAXLENGTH=12 VALUE=\"" +
                  reqReferenceNumber + "\"> ");

      // Check Copy Delivery Method list box
      out.println("<BR> <B>Check Copy Delivery Method:</B>");
      out.println("<BR> <SELECT NAME=reqDeliveryMethodIndicator >");
      displayPostalSelectList(deliveryMethod, out);
      out.println("</SELECT>");

      // Check Description input field
      out.println("<BR> <B>Check Description:</B> (optional)");
      out.println("<BR> <INPUT NAME=reqCheckDescription TYPE=text SIZE=25 MAXLENGTH=50 VALUE=\"" +
                  reqCheckDescription + "\"> ");

      out.println("<P><CENTER>Additional Information (optional)<BR><HR>");
      out.println("<TABLE WIDTH=\"100%\" BORDER=0 >");

      // Home Phone Number input field
      out.println("<TR><TD ALIGN=LEFT >Home Phone:</TD>");
      out.println("<TD ALIGN=LEFT >");
      out.println("<INPUT NAME=reqHomePhone TYPE=text SIZE=12 MAXLENGTH=15 VALUE=\"" +
                  reqHomePhone + "\"> ");
      out.println("</TD></TR>");

      // Work Phone Number input field
      out.println("<TR><TD ALIGN=LEFT >Work Phone:</TD>");
      out.println("<TD ALIGN=LEFT >");
      out.println("<INPUT NAME=reqWorkPhone TYPE=text SIZE=12 MAXLENGTH=15 VALUE=\"" +
                  reqWorkPhone + "\"> ");
      out.println("</TD></TR>");

      // Fax Number input field
      out.println("<TR><TD ALIGN=LEFT >Fax Number:</TD>");
      out.println("<TD ALIGN=LEFT >");
      out.println("<INPUT NAME=reqFaxNumber TYPE=text SIZE=12 MAXLENGTH=15 VALUE=\"" +
                  reqFaxNumber + "\"> ");
      out.println("</TD></TR>");
      out.println("</TABLE></CENTER>");

      // E-Mail Address input field
      out.println("<DIV>E-Mail Address:</DIV>");
      out.println("<DIV ALIGN=LEFT>");
      out.println("<INPUT TYPE=text NAME=reqEmailAddress SIZE=25 MAXLENGTH=30 VALUE=\"" +
                  reqEmailAddress + "\"></DIV>");

      out.println("<P><CENTER><INPUT NAME=SUBMIT TYPE=submit VALUE=\"Enter\" ></P>");
      out.println("</CENTER></FORM>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   //*******************************
   // Displays the input form for user to generate a Check Copy transaction.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;

      try
      {
         // get user profile
         //replaced getValue with getAttribute
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         displayInputForm(up, ifsSession, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in doGet method of IfsWpCheckCopy Servlet, explanation: ");
      }
   }


   //*******************************
   // Either displays the confirmation page for a Check Copy
   // transaction or executes it.
   //
   // The following Http Request Parameters are posted to this page:
   //     ReqTag   - Indicates whether to display confirmation or execute.
   //                It can have the values of "Confirm" or "Result".
   //     transfer - Specifies the object identifier (oid) of a
   //                GtCheckCopy object.
   //     req????  - The input values from the Check Copy form.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;
      GtCheckCopy transact = null;
      String strReqTag = null;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtCheckCopy object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transact = (GtCheckCopy) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtCheckCopy",
                             "transact");

         transact.setReqCheckSerialNumber(0);

         try {                                                               
               GsCurrency nullCurrency = new GsCurrency();
               nullCurrency = GsCurrency.fromString(" ");                                                                 
               transact.setReqCheckAmount(nullCurrency);
             }
         catch(Exception e)
             { debug("GsCurrency creation and initialization" + e.toString());
               throw e;
             }
 

         try
         {
            // Set fields of GtCheckCopy object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transact",transact);

            // insert the first 2 digits of the year with 80 as the cut-off value.
            GsDate date = new GsDate();
            date = yearWindow(transact.getReqCheckPostingDate());
            transact.setReqCheckPostingDate(date);

         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         strReqTag = (String)getRequestParameter(request,"ReqTag");

         if (strReqTag.equals("Confirm"))
            displayConfirmInfo(up, ifsSession, transact, request, response);
         else if (strReqTag.equals("Result"))
            showTxnResults(ifsSession, transact, request, response);

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in doPost method of IfsWpCheckCopy Servlet, explanation: ");
      }

   }

   /**
   * Displays the transaction results.
   */
   private void showTxnResults(IfsSession ifsSession, GtCheckCopy transact,
                               HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      try
      { 
         //added try-catch block 1/18/2002
        try
        {
        
         transact.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(transact, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpCheckCopy",
                      "IfsWpRedoPage?nextpage=IfsWpCheckCopy")){
             return;
         }

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to get check copy. Exception caught in showTxnResults method of IfsWpCheckCopy Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Check Copy Requested");

      out.println("<P>Your check copy has been requested successfully!<HR>");

      show2DivBlocks(out, "FI Reference Number:", transact.getReferenceNumber() );
      show2DivBlocks(out, "Service Provider Reference Id:",
                     transact.getServiceProviderReference() );

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   /**
   * Validate the data that was entered.
   */
   private String validateCheckCopy(GtCheckCopy cc, HttpServletRequest request)
   {
      StringBuffer strBuf = new StringBuffer("");
      String       strTemp = "";

      // Validate the data that was entered.

      if ( isValidNumber(cc.getReqCheckSerialNumber()) == false )
         strBuf.append("<BR>Must specify a valid positive non-zero check number.");

      if ( isValidAmount(cc.getReqCheckAmount()) == false )
         strBuf.append("<BR>Check Amount is invalid!");

      strTemp = (String)getRequestParameter(request,"reqCheckPostingDate");
      if ( (strTemp != null) && (strTemp.length() != 0) &&
           (isValidDate(cc.getReqCheckPostingDate()) == false) )
         strBuf.append("<BR>Posting date is invalid!");

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         cc.debug("Validation failed in validateCheckCopy method of IfsWpCheckCopy class");
         cc.debug(strBuf.toString());
      }

      return strBuf.toString();
   }


}
