//-*************************************
//
// FILE NAME:  IfsWpDeleteVariableTransfer.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************

/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */


import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import javax.servlet.http.*;
import javax.servlet.*;
import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

/**
* This class handles the transaction for Delete Variable Transfer.
*/
public class IfsWpDeleteVariableTransfer extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;

   //*******************************
   // Displays the confirmation page for a Delete Variable Transfer transaction.
   //
   // The following Http Request Parameters are posted to this page:
   //     transfer - Specifies the object identifier (oid) of a
   //                GtTransferHistoryInquiry object in the session cache.
   //     i        - The transfer history detail index.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);
      GtTransferHistoryInquiry transfer = null;

      GtUserProfile up = null;

      String strValue = "";
      String transfer_number = new String();
      GsTfrDtl detail = null;

      try
      {
        //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtTransferHistoryInquiry object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transfer = (GtTransferHistoryInquiry) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtTransferHistoryInquiry",
                             "transfer");
         try
         {
            // Set fields of GtTransferHistoryInquiry object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transfer",transfer);
         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         int i = Integer.parseInt( (String)getRequestParameter(request,"i"));
         detail = transfer.getTransferHistoryDetail()[i];

         transfer_number = detail.getTransferNumber();
         if (transfer_number == null)
            transfer_number = "";
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to delete variable transfer. Exception in doGet method of IfsWpDeleteVariableTransfer Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Confirm Delete Variable Transfer");

      out.println("<B>Do you really want to delete the following Transfer? </B>");
      out.println("<HR>");

      show2DivBlocks(out, "Transfer From Account:", detail.getSource());
      show2DivBlocks(out, "Transfer To Account:", detail.getTarget());

      try { strValue = detail.getFrequency_text(); }
      catch (Exception e) { strValue = "Unknown frequency: " + detail.getFrequency(); }
      show2DivBlocks(out, "Transfer Frequency:", strValue);

      show2DivBlocks(out, "Amount:", detail.getTransactionAmount());
      show2DivBlocks(out, "Effective Date:", detail.getEffectiveDate());
      show2DivBlocks(out, "Transfer Nickname:", detail.getTransferNickname());

      out.println("<CENTER><P><TABLE>");
      out.println("<TR><TD>");
      out.println("<FORM METHOD=POST NAME=DeleteVariableTransfer ACTION=IfsWpDeleteVariableTransfer >");
      out.println("<INPUT TYPE=hidden NAME=reqTransferNumber VALUE=\"" + transfer_number + "\">");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Yes\" ></FORM></TD>");

      out.println("<TD><FORM METHOD=get NAME=TransferCancel ACTION=IfsWpTransfer >");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"No\" >");
      out.println("</FORM></TD>");
      out.println("</TR>");
      out.println("</TABLE>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   //*******************************
   // Executes a Delete Variable Transfer transaction.
   //
   // The following Http Request Parameters are posted to this page:
   //     transfer  - Specifies the object identifier (oid) of a
   //                 GtDeleteVariableTransfer object.
   //     req?????  - The input values from the Delete Variable Transfer confirmation.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;
      GtDeleteVariableTransfer transfer = null;

      try
      {
        //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtDeleteVariableTransfer object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transfer = (GtDeleteVariableTransfer) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtDeleteVariableTransfer",
                             "transfer");
         try
         {
            // Set fields of GtDeleteVariableTransfer object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transfer",transfer);
         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }
        
         if (transfer.getReqTransferNickname() == null)
            transfer.setReqTransferNickname("");

         if (transfer.getReqTmstpVal() == null)
            transfer.setReqTmstpVal("");

         //added try-catch block 1/18/2002
         try
        {
        
         transfer.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(transfer, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpTransfer",
                      "IfsWpRedoPage?nextpage=IfsWpTransfer")){
             return;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to delete variable transfer. Exception in doPost method of IfsWpDeleteVariableTransfer Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Delete Variable Transfer results");

      out.println("<B>Delete variable transfer was successful</B>");
      out.println("<HR>");

      show2DivBlocks(out, "Confirmation Reference Number:",
                     transfer.getReferenceNumber());

      show2DivBlocks(out, "Service Provider Reference:",
                     transfer.getServiceProviderReference());

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

}
