//-*************************************
//
// FILE NAME:  IfsWpErrorPage.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .  Had to add Next button
//                              because WorkPad was defaulting to doPost
//                              instead of doGet.
//
//-*************************************

import java.io.PrintWriter;
import java.io.IOException;
import java.net.URLEncoder;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsTxn;
import com.ibm.ifs.servlets.IfsHttpServlet;

/**
* This servlet provides a general error display.
* Whenever an error occurs, the page flow is redirected to this servlet.
*/
public class IfsWpErrorPage extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   //*******************************
   // Displays the information assoicated with the error.
   //
   // The following Http Request Parameters are posted to this page:
   //   txn       - Value posted from any page when there is an error executing the
   //               transaction.
   //   Exception - Value posted from any page when an exception occured.
   //*******************************
   public void doGet (HttpServletRequest req, HttpServletResponse res)
               throws ServletException, IOException
   {
      IfsSession  ifsSession = new IfsSession(req,res);
      IfsTxn      txn        = null;
      String      message    = null;

      IfsSystem.initIfs();

      PrintWriter out = startHtmlOutput(res, "Ifs Error Information Servlet");

      out.println("<P>An error has occurred. The following information was provided:");
      out.println("<HR>");

      // Process transaction data if available (otherwise we have exeception data).
      debug(res, "Inside IfsWpErrorPage");
      try
      {
         txn = (IfsTxn) getRequestParameter(req, "txn");
         if (txn != null)
         {
            // out.println("<BR><B>Transaction Error in:</B>" + txn.getClass().getName()  );
            // out.println("<BR><B>ErrorCode:</B>"            + txn.getErrorCode()        );
            // out.println("<BR><B>BRC:</B>"                  + txn.getBrc()              );
            // out.println("<BR><B>Status:</B>"               + txn.getStatus()           );
            // out.println("<BR><B>CompCode:</B>"             + txn.getCompCode()         );
            // out.println("<BR><B>ReasonCode:</B>"           + txn.getReasonCode()       );
            // out.println("<BR><B>ErrorFunctionName:</B>"    + txn.getErrorFunctionName());
            // out.println("<BR><B>ErrorModuleName:</B>"      + txn.getErrorModuleName()  );

            out.println("<BR><B>Message:</B>"              + txn.getMessage()          );

            // out.println("<BR><B>GmMessage:</B>"            + txn.getGmMessage()        );
            // out.println("<BR><B>SenseData:</B>"            + txn.getSenseData()        );
            // out.println("<BR><B>SenseDataLength:</B>"      + txn.getSenseDataLength()  );
         }
         else
         {
            message = (String) getRequestParameter(ifsSession, "Exception");
            if (message != null)
               out.println("<BR>Exception occurred, message received: " + message);
         }
      }
      catch (Exception e)
      {
         out.println("<BR>Exception occurred in Ifs Error Information Servlet, message received: " +
                     e.toString());
      }

      out.println("<HR>If this problem persists, or if you are in doubt as to the ");
      out.println("status of your last transaction, you may wish to contact your ");
      out.println("customer service representative.");
      out.println("<BR><BR>Click button to logoff. <BR>");

      out.println("<FORM METHOD=get NAME=NextPage ACTION=\"IfsWpLogoff\" >");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Logoff\" ></FORM> ");

      out.println("</BODY> </HTML>");
   }

   /**
   * We perform the same operations for POST request as done for GET requests,
   * so this method just sends the request to the doGet method.
   */
   public void doPost (HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      try
      {
         doGet(request,response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
             "Exception caught in doPost method when calling doGet method in IfsWpErrorPage Servlet, explanation: ");
      }
   }
}

