//-*************************************
//
// FILE NAME:  IfsWpInterestRates.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resource.*;
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */

import java.io.*;
import java.net.URLEncoder;
import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This class displays a list of the current interest rates.
*/
public class IfsWpInterestRates extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   //*******************************
   // Gets and displays the interest rates.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);

      GtInterestRates txn = null;
      GtUserProfile up;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         // create transaction and execute.
         txn = new GtInterestRates(ifsSession);


         //added try-catch block 1/18/2002
         try
        {
        
         txn.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(txn, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpInterestRates",
                      "IfsWpRedoPage?nextpage=IfsWpInterestRates")){
             return;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Exception caught in doGet method of IfsWpInterestRates Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "INTEREST RATES");

      // Display the interest rate list on the servlet page.
      try
      {
         String strTitle = null;
         String strValue = null;
         GsIntRateDtl[] interestRateList = txn.getInterestRateList();

         if (interestRateList.length == 0)
            out.println("<BR>No interest rate list was returned.  <BR><BR>");

         for (int i = 0; i < interestRateList.length; i++)
         {
            try { strTitle = interestRateList[i].getInterestRateCode_text(); }
            catch (Exception e)
            {
               strTitle = "Interest rate code is " +
                          String.valueOf(interestRateList[i].getInterestRateCode());
            }

            try { strValue = interestRateList[i].getInterestRate().toString(IfsSystem.gsDecimalFormat) + " %"; }
            catch (Exception e) { strValue = "Unprintable rate "; }

            show2DivBlocks (out, strTitle, strValue);
         }
      }
      catch (Exception e)
      {
         out.println("<BR>Unable to use interest rate list.  <BR>Exception is: " +
                     e.toString() + "<BR><BR>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }
}
