//-*************************************
//
// FILE NAME:  IfsWpMain.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resources.*;
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */

import java.io.*;
import java.net.URLEncoder;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This class is used to logon to IFS system and display the main functions screen.
*
*/
public class IfsWpMain extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   //*******************************
   // Executes the logon transaction and displays the main function screen.
   //
   // The following Http Request Parameters are posted to this page by
   // IfsWpLogon servlet:
   //     userID   - The ID of the user being logged on.
   //     pin      - The user's pin value.
   //     password - The user's password.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {

      // create session
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;
      GtLogon gtLogon = null;

      String strUser = "";
      String strPIN = "";
      String strPwd = "";

      try
      {
        
         // Check if there is still a active session.
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // No session exists, logon the user.

            IfsSystem.initIfs();
            gtLogon = new GtLogon(ifsSession);

            // Get Customer ID, PIN and Password
            strUser = (String)getRequestParameter(request,"userID");
            strPIN = (String)getRequestParameter(request,"pin");
            strPwd = (String)getRequestParameter(request,"password");

            // Check customer id and password
            if ( (strUser.length() == 0) ||
                 (strPIN.length() == 0) || (strPwd.length() == 0))
            {
               callPage_RedoPage(response, "IfsWpLogon",
                   "Your Customer ID, PIN or Password is missing, please re-enter.");
            }

            // extract browser information
            //BrowserInfo info = new BrowserInfo();
            //info.initBrowserInfo(request);

            // set transaction parameters
            gtLogon.setUserID(strUser);
            gtLogon.setPin(strPIN);
            gtLogon.setPinEncryptionType(IfsSystem.NO_ENCRYPTION);
            gtLogon.setPassword(strPwd);
            gtLogon.setPasswordEncryptionType(IfsSystem.NO_ENCRYPTION);
            //gtLogon.setClientOS(info.browserPlatform);

            // Execute transaction

            //added try-catch block 1/18/2002
         try
        {
        
         gtLogon.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
        if (! handleErrors(gtLogon, "IfsWpErrorPage","IfsWpRedoPage","IfsWpRedoPage")){
            return;
            }

            // Check for availability of user profile.
            //replaced getValue with getAttribute 1/15/02

                up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
                if (up == null)
                {
                    callPage_RedoPage(response, "IfsWpLogon",
                         "Logon failed. The user profile is not available.");
                } 
           
         }

         PrintWriter out = startHtmlOutput(response, "IFS");

         // output main functions
         out.println("<CENTER><TABLE>");

         // accounts
         out.println("<TR><TD COLSPAN=3 ALIGN=CENTER>");
         out.println("<A href=IfsWpAccounts>");
         out.println("<IMG SRC=\"/ifsgif/accounts.gif\" ALT=Accounts BORDER=0 ></A>");

         // transfer
         out.println("<TR><TD ALIGN=CNETER>");
         out.println("<P><A href=IfsWpTransfer>");
         out.println("<IMG SRC=\"/ifsgif/transfer.gif\" ALT=Transfer BORDER=0 ></A>");

         // ifs logo
         out.println("<TD ALIGN=CENTER>");
         out.println("<IMG SRC=\"/ifsgif/ifslogo.gif\" BORDER=0 >");

         // services
         out.println("<TD ALIGN=CENTER>");
         out.println("<P><A href=IfsWpServices>");
         out.println("<IMG SRC=\"/ifsgif/services.gif\" ALT=Services BORDER=0 ></A>");

         // pay bills (not implemented yet)
         //out.println("<TR><TD COLSPAN=3 ALIGN=CENTER>");
         //out.println("<P>");
         //out.println("<IMG SRC=\"/ifsgif/paybills.gif\" ALT=\"Pay bills\" BORDER=0 >");
         out.println("</TABLE>");

         // logoff
            out.println("<A href=IfsWpLogoff>Logoff</A>");

         // end html output
         out.println("</CENTER></BODY></HTML>");
         out.close();
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
             "Logon failed. Please check your Customer ID, PIN and Password. Exception caught in doGet method of IfsWpLogon Servlet, explanation: ");
      }
   }

}

/**
* Class to get the client browser information.
*/
class BrowserInfo
{
   public String browserType;
   public String browserVersion;
   public String browserName;
   public String browserPlatform;
   public String browserUserAgent;
   public boolean browserJavaScriptEnabled;

   public BrowserInfo()
   {
   }

   public void initBrowserInfo(HttpServletRequest httpReq)
   {
      String browser, bInfo, token;
      int i;
      long l;

      // get browser's 'User-agent' string from request header
      browser = httpReq.getHeader("User-agent");
      browserUserAgent = browser;

      i = browserUserAgent.indexOf("Mozilla");

      if (i > -1)
      {
         // remove "Mozilla/" from UserAgent string
         browser = browserUserAgent.substring(browserUserAgent.indexOf("/")+1);

         // locate blank following version number
         i = browser.indexOf(" ");
         // get browser version number
         browserVersion = browser.substring(0, i);
         browserVersion = browserVersion.trim();  // remove any whitespace around version

         try
         {
            Double d;
            d = Double.valueOf(browserVersion);  // convert string to Double
            browserJavaScriptEnabled = (d.doubleValue() >= 2.0) ? true : false;
         }
         catch (NumberFormatException e)
         {
         }

         // get the browser info between the parens
         browser = browser.substring( (browser.indexOf("(")+1), (browser.indexOf(")"))   );

         StringTokenizer list = new StringTokenizer(browser, ";");

         token = list.nextToken().trim();

         if ( token.equals("compatible"))
         {
            browserType = "explorer";
            browserVersion += " compatible";   // add compatible to version

            // internet explorer

            browserName = list.nextToken().trim();

            token = list.nextToken().trim();

            if (token.indexOf("Update") > -1)
            {
               // should be platform or update
               browserName +=  " " + token;       // add update string to name
               token = list.nextToken().trim();
            }
         }
         else
         {
            // netscape
            browserType = "netscape";
            browserName = "Netscape";
         }

         browserPlatform = token;   // operating system of browser
      }
   }

}
