//-*************************************
//
// FILE NAME:  IfsWpRedoPage.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .  Had to add Next button
//                              because WorkPad was defaulting to doPost
//                              instead of doGet.
//-*************************************

import java.io.PrintWriter;
import java.io.IOException;
import java.net.URLEncoder;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsTxn;
import com.ibm.ifs.servlets.IfsHttpServlet;

/**
* This servlet provides a general error display for REDO and INFO errors.
* Whenever a REDO or INFO error occurs, the page flow is redirected to this servlet.
*/
public class IfsWpRedoPage extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   //*******************************
   // Displays the information associated with the error.
   //
   // The following Http Request Parameters may be posted to this page:
   //
   //   txn       - Value posted from any page when there is an error executing the
   //               transaction.
   //   Exception - Value posted from any page when an exception occured.
   //   redomsg   - The REDO message which is to be displayed.
   //   nextpage  - Controls page flow. It specifies the next page to be displayed.
   //
   //   op        - Search transfer operation (change or delete).
   //   transfer  - Change transfer object identifier (oid) for GtTransferHistoryInquiry
   //               object.
   //   i         - Change transfer index GtTransferHistoryInquiry object.
   //*******************************
   public void doGet (HttpServletRequest req, HttpServletResponse res)
               throws ServletException, IOException
   {
      IfsSession  ifsSession = new IfsSession(req,res);

      IfsTxn      txn        = null;
      String      message    = null;
      String      redoMsg    = null;
      String      nextPage   = null;
      String      nextPageMsg = null;
      String[]    transferParm = null;
      String      iParm = null;
      String      opParm = null;

      IfsSystem.initIfs();

      PrintWriter out = startHtmlOutput(res, "Ifs Redo Information Servlet");

      out.println("<P>Unable to complete the last request. The following information was provided:");
      out.println("<HR>");

      debug(res, "Inside IfsWpRedoPage");
      try
      {
         //---------------------
         // Get the Http request parameters
         //
         nextPage = (String)getRequestParameter(req, "nextpage");
         if (nextPage == null)
            nextPage = "IfsWpLogoff";

         redoMsg = (String)getRequestParameter(req, "redomsg");
         txn = (IfsTxn) getRequestParameter(req, "txn");

         iParm = (String)getRequestParameter(req, "i");
         opParm = (String)getRequestParameter(req, "op");

         // Have to use getParameterValues because of the oid value.
         transferParm = IfsHttpServlet.getParameterValues( "transfer",
                                                  IfsSystem.cwsGMJCharSet, req);

         //---------------------
         // Display the corresponding message to the user.
         //
         if (redoMsg != null)
         {
            // User entered invalid data.
            out.println(redoMsg );
         }
         else if (txn != null)
         {
            // Transaction completed with a REDO or INFO completion code.
            // out.println("    <B>Transaction:</B>"          + txn.getClass().getName()  );
            // out.println("<BR>   completed with a REDO or INFO completion code type."   );
            // out.println("<BR><B>ErrorCode:</B>"            + txn.getErrorCode()        );
            // out.println("<BR><B>BRC:</B>"                  + txn.getBrc()              );
            // out.println("<BR><B>Status:</B>"               + txn.getStatus()           );
            // out.println("<BR><B>CompCode:</B>"             + txn.getCompCode()         );
            // out.println("<BR><B>ReasonCode:</B>"           + txn.getReasonCode()       );
            out.println("<BR><B>Message:</B>"              + txn.getMessage()          );
            // out.println("<BR><B>GmMessage:</B>"            + txn.getGmMessage()        );
         }
         else
         {
            // Exception occurred.
            message = (String) getRequestParameter(req, "Exception");
            if (message != null)
               out.println("Exception occurred, message received: " + message);
         }

         out.println("<HR>If this problem persists, or if you are in doubt as to the ");
         out.println("status of your last transaction, you may wish to contact your ");
         out.println("customer service representative.");
         out.println("<BR><BR>Click button to continue. <BR>");

         //------------------
         // Output the corresponding next page data and parameters.
         //
         if ( (transferParm != null) && (transferParm.length > 0) &&
              (iParm != null) )
         {
            // This is a REDO for a change transfer transaction.
            out.println("<FORM METHOD=get NAME=NextPage ACTION=\"" + nextPage + "\" >");
            out.println("<INPUT TYPE=hidden NAME=transfer  VALUE=\"" + transferParm[0] + "\" >");
            out.println("<INPUT TYPE=hidden NAME=i  VALUE=\"" + iParm + "\" >");
            out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Edit\" ></FORM> ");
         }

         else if (opParm != null)
         {
            // This is a REDO for a search transfer transaction.
            out.println("<FORM METHOD=get NAME=NextPage ACTION=\"" + nextPage + "\" >");
            out.println("<INPUT TYPE=hidden NAME=op  VALUE=\"" + opParm + "\" >");
            out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Edit\" ></FORM> ");
         }
         else
         {
            out.println("<FORM METHOD=get NAME=NextPage ACTION=\"" + nextPage + "\" >");
            out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Edit\" ></FORM> ");
         }

         out.println("</BODY> </HTML>");
      }
      catch (Exception e)
      {
         callPage_ErrorPage(res, e,
               "Exception caught in doGet method of IfsWpRedoPage Servlet, explanation: ");
      }
   }

   /**
   * We perform the same operations for POST request as what is done for GET requests,
   * so this method just sends the request to the doGet method.
   */
   public void doPost (HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
   {
      try
      {
         doGet(request,response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
             "Exception caught in doPost method when calling doGet method in IfsWpRedoPage Servlet, explanation: ");
      }
   }
}

