//-*************************************
//
// FILE NAME:  IfsWpSearchTransfer.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resource.*;
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */

import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import javax.servlet.http.*;
import javax.servlet.*;
import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This servlet handles the Search Transfer transaction and
* prepares the data for Change or Delete Transfer transactions.
*/
public class IfsWpSearchTransfer extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;

   // the number of displayed records in one page
   final static int TOTAL_PAGE_DISPAY = 5;

   // default value of maxmium received record number
   final static int MAX_REC_NUM =  2 * TOTAL_PAGE_DISPAY;

   // when total matched number larger than maxmium record number,
   // MAX_REC_NUM = MAX_REC_NUM + MAX_REC_NUM_STEP
   final static int MAX_REC_NUM_STEP = 2 * TOTAL_PAGE_DISPAY;


   //*******************************
   // Displays the input form page for a Search Transfer transaction.
   //
   // The following Http Request Parameter is posted to this page:
   //     op - Indicates the transfer change type.
   //          It can have the values of "search" or "delete".
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);

      GtUserProfile up = null;
      String        op = null;

      GtTransferHistoryInquiry th      = null;
      GsAccount                account = null;
      String         transferStartDate = "";
      String         transferEndDate   = "";

      //replaced getValue with getAttribute 1/15/02
      up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
      if (up == null)
      {
         // Since no user profile exists, there is no active session for this consumer.
         callPage_NoSession(response);
      }

      try { op = (String)getRequestParameter(request,"op"); }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in doGet method of IfsWpSearchTransfer Servlet, explanation: ");
      }

      if ( (op == null) || (op.length() == 0) )
      {
         callPage_RedoPage(response, "IfsWpTransfer",
           "Cannot do a search transfer because of missing operation parameter.");
      }

      try
      {
         //---------------
         // Check for redisplay of originally entered information.
         // Get a reference to the GtTransferHistoryInquiry cached object.
         //
         th = (GtTransferHistoryInquiry) getObject(ifsSession, SEARCH_TRANSFER);
         if (th != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            account = th.getReqFundingAccount();

            // start and end dates
            transferStartDate = getDateString(th.getReqTransferDateRangeStartDate());
            transferEndDate = getDateString(th.getReqTransferDateRangeEndDate());
         }
      }
      catch ( Exception e) { }

      PrintWriter out = startHtmlOutput(response, "Search Transfer");

      out.println("<FORM METHOD=POST NAME=IfsWpSearchTransfer ACTION=IfsWpSearchTransfer >");
      out.println("<INPUT TYPE=hidden NAME=op  VALUE=\"" + op + "\" >");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispFirst\" >");
      out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + SEARCH_TRANSFER + "\" >");

      // Generate SELECT with account numbers
      out.println("<B>Please select an account:</B> <BR>");
      out.println("<SELECT NAME=SourceAccount >");
      out.println("<OPTION VALUE=\"-1\" > All Funding Accounts");
      displayAccountNum(up, account, out);
      out.println("</SELECT><BR>");

      out.println("<B> Date Range: </B>(Optional)<BR>");
      out.println("<TABLE BORDER=0   WIDTH=95%>");
      out.println("<TR>");
      out.println("<TD  ALIGN=LEFT > From: </TD>");
      out.println("<TD> <INPUT NAME=reqTransferDateRangeStartDate TYPE=text SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  transferStartDate + "\"></TD>");
      out.println("</TR><TR>");
      out.println("<TD ALIGN=LEFT > To: </TD>");
      out.println("<TD> <INPUT NAME=reqTransferDateRangeEndDate TYPE=text SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  transferEndDate + "\"></TD>");
      out.println("</TR><TR>");
      out.println("<TD></TD> <TD >" + strDateHint + "</TD>");
      out.println("</TR>");
      out.println("</TABLE>");

      //submit
      out.println("<P><DIV ALIGN=CENTER>");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Search\" >");
      out.println("</DIV>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   //*******************************
   // Displays the results of a Search Transfer transaction.
   //
   // The following Http Request Parameters are posted to this page:
   //     ReqTag   - Indicates which group of results is to be displayed.
   //                It can have the values of "DispFirst" or "DispMore".
   //     transfer - Specifies the object identifier (oid) of a
   //                GtTransferHistoryInquiry object.
   //     req????  - The input values from the Search Transfer Transaction form.
   //
   //     SourceAccount - The index of the selected account to be searched.
   //                     It is posted when ReqTag=DispFirst.
   //
   //     NUM     - The starting index of the next transfer to display.
   //               It is posted when ReqTag=DispMore.
   //     LEFTNUM - The number of transfers left to display.
   //               It is posted when ReqTag=DispMore.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);
      GtTransferHistoryInquiry transfer = null;

      GtUserProfile up = null;
      String strReq = null;

      try
      {
        //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtTransferHistoryInquiry object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transfer = (GtTransferHistoryInquiry) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtTransferHistoryInquiry",
                             "transfer");
         try
         {
            // Set fields of GtTransferHistoryInquiry object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transfer",transfer);

            // insert the first 2 digits of the year with 80 as the cut-off value
            GsDate date = new GsDate();
            date = yearWindow(transfer.getReqTransferDateRangeStartDate());
            transfer.setReqTransferDateRangeStartDate(date);

            date = yearWindow(transfer.getReqTransferDateRangeEndDate());
            transfer.setReqTransferDateRangeEndDate(date);

         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         strReq = (String)getRequestParameter(request,"ReqTag");

         if (strReq.compareTo("DispFirst") == 0)
            showFirstHistoryPage(up, ifsSession, transfer, request, response);
         else
            showMoreHistoryPages(up, ifsSession, transfer, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to do search transfer. Exception caught in doPost method of IfsWpSearchTransfer Servlet, explanation: ");
      }
   }

   /**
   * Display the first transfer history page.
   */
   private void showFirstHistoryPage(GtUserProfile up, IfsSession ifsSession,
                                     GtTransferHistoryInquiry transfer,
                                     HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      int i;
      GsTfrDtl detail = null;

      GsAcctDtl acct = null;
      String type = "";
      String acctNbr = "";
      String acctNick = "";

      String op = null;
      String oid = null;
      String msg = null;

      //Check input information

      try
      {
         // Get type of search request (change, delete).
         op = ((String)getRequestParameter(request,"op")).trim();

         //set Amount Range from $0.00 to $max .
         try
         {
            transfer.setReqTransferAmountRangeLowAmount(new GsCurrency(strCurrencyPrefix + "0", ifsSession));
            transfer.setReqTransferAmountRangeHighAmount(new GsCurrency(strCurrencyPrefix +
                                                                        IfsSystem.MAX_CURRENCY_VALUE, ifsSession));
         }
         catch (Exception e) {}

         //set funding account
         i = Integer.parseInt( (String)getRequestParameter(request,"SourceAccount"));
         if (i >= 0)
         {
            acct = up.getAccountDetail()[i];
            type = acct.getAccount().getType();
            acctNbr = acct.getAccount().getNumber();
            acctNick = acct.getAccount().getNickname();

            // create account
            GsAccount anAccount = new GsAccount(type, acctNbr, acctNick);
            transfer.setReqFundingAccount(anAccount);
         }

         transfer.setReqMaximumRecordNumber(MAX_REC_NUM);
         transfer.setReqTransferStateIndicator("1");       // Pending transfers
         transfer.setReqTransferType("3");                 // All transfer types

         // Get the object ID of the cached GtTransferHistoryInquiry instance object.
         oid = getOid(ifsSession,transfer);

         // Validate the data that was entered.
         msg = validateSearchTransfer(transfer, request);
         if ( msg.equals("OK") == false)
            callPage_RedoPage(response, "IfsWpSearchTransfer&op=" + op, msg);

         //added try-catch block 1/18/2002
         try
        {
        
         transfer.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(transfer, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpSearchTransfer",
                      "IfsWpRedoPage?nextpage=IfsWpSearchTransfer")){
             return;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in showFirstHistoryPage method of IfsWpSearchTransfer Servlet, explanation: ");
      }

      //----------------
      // Start outputting the search transfer information.
      //
      PrintWriter out = startHtmlOutput(response, "Search Transfer");

      int totalMatched = 0;
      int totalSent = 0;
      int leftNum = 0;

      // get totalMatched and totalSent
      totalMatched = transfer.getTotalTransactionsMatched();
      totalSent = transfer.getTotalTransactionsSent();
      leftNum = totalSent;
      if (totalMatched == 0)
      {
         out.println("<P>There is no transfers history.");

         // link to other pages and close the print writer.
         endHtmlOutput(out);
         return;
      }

      int endPageNum = 0;
      // set end page display number
      if (totalSent < TOTAL_PAGE_DISPAY)
         endPageNum = totalSent;
      else
         endPageNum = TOTAL_PAGE_DISPAY;

      // display each Transfer
      out.println("<HR>");
      for (i = 0; i < endPageNum; ++i)
      {
         if ( (i >= transfer.getTransferHistoryDetail().length) ||
              (transfer.getTransferHistoryDetail()[i] == null) )
            continue;

         detail = transfer.getTransferHistoryDetail()[i];
         showHistoryPage(detail, oid, op, i, out);
      }


      // link to more Transfers
      if (totalSent > 0)
         out.println("<P><DIV ALIGN=CENTER>Transfer 1 - " + endPageNum + " (Total: " + totalMatched + ")</DIV>");
      else
         out.println("<P>There is no transfers history. <BR>");

      if (endPageNum < totalMatched)
      {
         leftNum -= endPageNum;
         out.println("<DIV ALIGN=CENTER>");
         out.println("<FORM METHOD=POST ACTION=IfsWpSearchTransfer >");
         out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispMore\"  >");
         out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + oid + "\" >");
         out.println("<INPUT TYPE=hidden NAME=NUM VALUE=\"" + endPageNum + "\" >");
         out.println("<INPUT TYPE=hidden NAME=LEFTNUM VALUE=\"" + leftNum + "\" >");
         out.println("<INPUT TYPE=hidden NAME=op  VALUE=\"" + op + "\" >");
         out.println("<INPUT TYPE=submit NAME=BUTTON VALUE=\"More\"  >");
         out.println("</FORM></DIV>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Displays the actual transfer history page.
   */
   private void showHistoryPage(GsTfrDtl detail, String oid, String op, int i, PrintWriter out)
   {
      String transferType = "";
      String strValue = "";

      //transfer type
      try { strValue = detail.getTransferType_text(); }
      catch (Exception e) { strValue = "Unknown transfer type: " + detail.getTransferType(); }
      show2DivBlocks(out, "Type:", strValue);

      // Frequency
      transferType = detail.getTransferType();

      if ( (transferType != null) && (transferType.equals("1")) )
      {
         try { strValue = detail.getFrequency_text(); }
         catch (Exception e) { strValue = "Unknown frequency: " + detail.getFrequency(); }
         show2DivBlocks(out, "Frequency:", strValue);
      }

      show2DivBlocks(out, "Source:", detail.getSource());
      show2DivBlocks(out, "Target:", detail.getTarget());

      out.println("<DIV ALIGN=LEFT> Number: </DIV>");
      out.println("<DIV ALIGN=RIGHT> <B>");

      if (op.equals("change"))
      {
         if (transferType.equals("1"))   // Is it a recurring transfer?
            out.println("<A href =\"IfsWpChangeFixedTransfer?transfer=" + oid + "&i=" + i + "\">");
         else
            out.println("<A href =\"IfsWpChangeVariableTransfer?transfer=" + oid + "&i=" + i + "\">");
      }
      else
      {
         if (transferType.equals("1"))  // Is it a recurring transfer?
            out.println("<A href =\"IfsWpDeleteFixedTransfer?transfer=" + oid + "&i=" + i + "\">");
         else
            out.println("<A href =\"IfsWpDeleteVariableTransfer?transfer=" + oid + "&i=" + i + "\">");
      }

      try { out.println(detail.getTransferNumber() + "</A> "); }
      catch (Exception e) { out.println("transfer_identifier numberERROR </A> "); }

      out.println("</B></DIV>");

      out.println("<HR>");

   }


   /**
   * Display the continued transfer history screens
   */
   private void showMoreHistoryPages(GtUserProfile up, IfsSession ifsSession,
                                     GtTransferHistoryInquiry transfer,
                                     HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GsTfrDtl detail = null;

      int maxRecNum = 0;
      int totalMatched = 0;
      int totalSent = 0;

      String op = "";
      String oid = "";

      int startPageNum = 0;
      int leftNum = 0;

      try
      {
         op = ((String)getRequestParameter(request,"op")).trim();

         // Get the object ID of the cached GtTransferHistoryInquiry instance object.
         oid = getOid(ifsSession,transfer);

         maxRecNum = transfer.getReqMaximumRecordNumber();
         totalMatched = transfer.getTotalTransactionsMatched();
         totalSent = transfer.getTotalTransactionsSent();

         // Page numbers
         startPageNum = Integer.parseInt( (String)getRequestParameter(request,"NUM"));
         leftNum = Integer.parseInt( (String)getRequestParameter(request,"LEFTNUM"));

         // need execute Transfer again
         if ((startPageNum + leftNum < totalMatched) && // not all matched has been sent
             (leftNum < TOTAL_PAGE_DISPAY))
         {
            // not enought to fill all page

            // set maximum number
            maxRecNum += MAX_REC_NUM_STEP;
            transfer.setReqMaximumRecordNumber(maxRecNum);

         // execute

         //added try-catch block 1/18/2002
         try
        {
        
         transfer.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
        if (! handleErrors(transfer, "IfsWpErrorPage",
                         "IfsWpRedoPage?nextpage=IfsWpSearchTransfer",
                         "IfsWpRedoPage?nextpage=IfsWpSearchTransfer")) {
            return;
        }

            // get totalSent
            totalSent = transfer.getTotalTransactionsSent();
            leftNum = totalSent - startPageNum;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to do transfer inquiry. Exception in showMoreHistoryPages method of IfsWpSearchTransfer Servlet, explanation: ");
      }

      //-------------------
      // Start outputting search transfer information.
      //
      PrintWriter out = startHtmlOutput(response, "Search Transfer");

      int endPageNum = 0;
      // set end page display number
      if (leftNum < TOTAL_PAGE_DISPAY)
         endPageNum = leftNum + startPageNum;
      else
         endPageNum = TOTAL_PAGE_DISPAY + startPageNum;

      for (int i = startPageNum; i < endPageNum; ++i)
      {
         if ( (i >= transfer.getTransferHistoryDetail().length) ||
              (transfer.getTransferHistoryDetail()[i] == null) )
            continue;

         detail = transfer.getTransferHistoryDetail()[i];
         showHistoryPage(detail, oid, op, i, out);
      }

      // link to more Transfers
      out.println("<P><DIV ALIGN=CENTER>Transfer " + (startPageNum + 1) + " - " + endPageNum + " (Total: " + totalMatched + ")</DIV>");
      if (endPageNum < totalMatched)
      {
         leftNum = leftNum - (endPageNum - startPageNum);
         out.println("<DIV ALIGN=CENTER>");
         out.println("<FORM METHOD=POST ACTION=IfsWpSearchTransfer >");
         out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispMore\"  >");
         out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + oid + "\" >");
         out.println("<INPUT TYPE=hidden NAME=NUM VALUE=\"" + endPageNum + "\" >");
         out.println("<INPUT TYPE=hidden NAME=LEFTNUM VALUE=\"" + leftNum + "\" >");
         out.println("<INPUT TYPE=hidden NAME=op  VALUE=\"" + op + "\" >");
         out.println("<INPUT TYPE=submit NAME=BUTTON VALUE=\"More\"  >");
         out.println("</FORM></DIV>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Validate the data that was entered.
   */
   private String validateSearchTransfer(GtTransferHistoryInquiry th, HttpServletRequest request)
   {
      StringBuffer strBuf = new StringBuffer("");
      String strTemp = "";
      String transferStartDate = "";
      String transferEndDate = "";

      // Get the start Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         transferStartDate = th.getReqTransferDateRangeStartDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (transferStartDate == null)
            transferStartDate = "";
      }
      catch (Exception e)
      {
         th.debug("Exception getting starting date in validateSearchTransfer - " + e.toString());
         transferStartDate = "";
      }

      // Get the last transfer Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         transferEndDate = th.getReqTransferDateRangeEndDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (transferEndDate == null)
            transferEndDate = "";
      }
      catch (Exception e)
      {
         th.debug("Exception getting ending date in validateSearchTransfer - " + e.toString());
         transferEndDate = "";
      }

      // Validate the data that was entered.

      strTemp = (String)getRequestParameter(request,"reqTransferDateRangeStartDate");
      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (transferStartDate.equals("")) )
         strBuf.append("<BR>Invalid From Date!");

      strTemp = (String)getRequestParameter(request,"reqTransferDateRangeEndDate");
      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (transferEndDate.equals("")) )
         strBuf.append("<BR>Invalid To Date!");

      if ( (transferStartDate.equals("") == false) &&
           (transferEndDate.equals("") == false) &&
           (transferStartDate.compareTo(transferEndDate) >= 0))
         strBuf.append("<BR>The To Date must be later than the From Date!");

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         th.debug("Validation failed in validateSearchTransfer method of IfsWpSearchTransfer class");
         th.debug(strBuf.toString());
      }

      return strBuf.toString();
   }
}
