//-*************************************
//
// FILE NAME:  IfsWpStopPayment.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resource.*;
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 */

import java.io.*;
import java.net.URLEncoder;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This class is used to handle the transaction of stop check payment.
*/
public class IfsWpStopPayment extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;


   /**
   * Displays the confirm information to user.
   */
   private void displayConfirmInfo(GtUserProfile up, IfsSession ifsSession,
                                   GtStopCheckPayment stopPayment,
                                   HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      String oid = null;
      String msg = null;
      String strValue = null;
      int index = 0;

      try
      {
         // Get the object ID of the cached GtStopCheckPayment instance object.
         oid = getOid(ifsSession,stopPayment);

         // Validate the data that was entered.
         msg = validateStopPayment(stopPayment);
         if ( msg.equals("OK") == false)
            callPage_RedoPage(response, "IfsWpStopPayment", msg);

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in displayConfirmInfo method of IfsWpStopPayment Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Confirm Request");

      out.println("Your check stop payment will be submitted. To confirm, click on <B>OK</B>, otherwise, click on <B>Cancel.</B>");
      out.println("<P><B>Note: There will be a $5.00 fee charged to your account for each stop payment.</B>");

      out.println("<HR>");

      show2DivBlocks(out, "Account:", stopPayment.getReqAccountID());

      try { index = Integer.parseInt(stopPayment.getReqStopPaymentType()); }
      catch (NumberFormatException e) { index = 20; }   // Invalid stop payment type.

      switch (index)
      {
         case 0:  // display check number
            show2DivBlocks(out, "Stop Payment Type:", "by Check Number");
            show2DivBlocks(out, "Check Number:",
                               stopPayment.getReqCheckNumberLow());
            break;

         case 1:  // display check number and amount
            show2DivBlocks(out, "Stop Payment Type:", "by Check Number and Amount");
            show2DivBlocks(out, "Check Number:", stopPayment.getReqCheckNumberLow());
            show2DivBlocks(out, "Amount:", stopPayment.getReqCheckRangeLowAmount());
            break;

         case 2:  // display check number range
            show2DivBlocks(out, "Stop Payment Type:", "by Check Number Range");
            show2DivBlocks(out, "From:", stopPayment.getReqCheckNumberLow());
            show2DivBlocks(out, "To:", stopPayment.getReqCheckNumberHigh());
            break;

         case 3:  // display amount
            show2DivBlocks(out, "Stop Payment Type:", "by Amount");
            show2DivBlocks(out, "Amount:", stopPayment.getReqCheckRangeLowAmount());
            break;

         case 4:  // display amount range
            show2DivBlocks(out, "Stop Payment Type:", "by Amount Range");
            show2DivBlocks(out, "From:", stopPayment.getReqCheckRangeLowAmount());
            show2DivBlocks(out, "To:", stopPayment.getReqCheckRangeHighAmount());
            break;

         case 5:  // display posting date
            show2DivBlocks(out, "Stop Payment Type:", "by Date");
            show2DivBlocks(out, "Posting Date:", stopPayment.getReqCheckIssueStartDate());
            break;

         case 6:  // display date and amount
            show2DivBlocks(out, "Stop Payment Type:", "by Date and Amount");
            show2DivBlocks(out, "Posting Date:", stopPayment.getReqCheckIssueStartDate());
            show2DivBlocks(out, "Amount:", stopPayment.getReqCheckRangeLowAmount());
            break;

         case 7:  // display date range
            show2DivBlocks(out, "Stop Payment Type:", "by Date Range");
            show2DivBlocks(out, "From:", stopPayment.getReqCheckIssueStartDate());
            show2DivBlocks(out, "To:", stopPayment.getReqCheckIssueEndDate());
            break;

         case 8:
            show2DivBlocks(out, "Stop Payment Type:", "All Checks");
            break;

         default:
            show2DivBlocks(out, "Stop Payment Type:", "Invalid Stop Payment Type !!!");
            break;
      }

      show2DivBlocks(out, "Stop Message:",
                         stopPayment.getReqMessage()[0].getMessagetext() );
      show2DivBlocks(out, "Home Phone Number:",
                         stopPayment.getReqContactHomePhoneNumber() );
      show2DivBlocks(out, "Work Phone Number:",
                         stopPayment.getReqContactWorkPhone() );
      show2DivBlocks(out, "Fax Number:",
                         stopPayment.getReqContactFaxNumber() );
      show2DivBlocks(out, "E-Mail Address:",
                         stopPayment.getReqContactEmailAddress() );

      out.println("<P> <CENTER>");
      out.println("<TABLE WIDTH=\"100%\" BORDER=0 >");
      out.println("<TR><TD ALIGN=RIGHT >");
      out.println("<FORM METHOD=POST ACTION=IfsWpStopPayment >");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"Result\" >");
      out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + oid + "\">");
      out.println("<INPUT TYPE=submit NAME=OKButton  VALUE=\"OK\" >");
      out.println("</FORM></TD>");

      out.println("<TD ALIGN=LEFT>");
      out.println("<FORM METHOD=get ACTION=IfsWpStopPayment >");
      out.println("<INPUT TYPE=submit NAME=CancelButton  VALUE=\"Cancel\" >");
      out.println("</FORM></TD>");
      out.println("</TR></TABLE>");
      out.println("</CENTER></BODY></HTML>");
      out.close();

   }

   /**
   * Displays the input form for user.
   */
   private void displayInputForm(GtUserProfile up, IfsSession ifsSession,
                                 HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GtStopCheckPayment sp      = null;
      GsAccount          account = null;
      String           stopPayment               = "";
      String           reqCheckNumberLow         = "";
      String           reqCheckNumberHigh        = "";
      String           reqCheckRangeLowAmount    = "";
      String           reqCheckRangeHighAmount   = "";
      String           reqCheckIssueStartDate    = "";
      String           reqCheckIssueEndDate      = "";
      String           reqContactHomePhoneNumber = "";
      String           reqContactWorkPhone       = "";
      String           reqContactFaxNumber       = "";
      String           reqContactEmailAddress    = "";

      GsMsgTxt[] reqMessage = null;
      String     messagetext = "";

      try
      {
         //---------------
         // Need to redisplay the originally entered information.
         // Get a reference to the GtStopCheckPayment cached object.
         //
         sp = (GtStopCheckPayment) getObject(ifsSession, STOP_CHECK_PAYMENT);
         if (sp != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            account = sp.getReqAccountID();
            stopPayment = sp.getReqStopPaymentType();

            // low and high amounts
            reqCheckRangeLowAmount = getAmountString(sp.getReqCheckRangeLowAmount());
            reqCheckRangeHighAmount = getAmountString(sp.getReqCheckRangeHighAmount());

            // start and end dates
            reqCheckIssueStartDate = getDateString(sp.getReqCheckIssueStartDate());
            reqCheckIssueEndDate = getDateString(sp.getReqCheckIssueEndDate());

            // low check number
            if (sp.getReqCheckNumberLow() != 0 )
               reqCheckNumberLow = String.valueOf(sp.getReqCheckNumberLow());

            // high check number
            if (sp.getReqCheckNumberHigh() != 0 )
               reqCheckNumberHigh = String.valueOf(sp.getReqCheckNumberHigh());

            reqContactHomePhoneNumber = sp.getReqContactHomePhoneNumber();
            if (reqContactHomePhoneNumber == null) reqContactHomePhoneNumber = "";

            reqContactWorkPhone = sp.getReqContactWorkPhone();
            if (reqContactWorkPhone == null) reqContactWorkPhone = "";

            reqContactFaxNumber = sp.getReqContactFaxNumber();
            if (reqContactFaxNumber == null) reqContactFaxNumber = "";

            reqContactEmailAddress = sp.getReqContactEmailAddress();
            if (reqContactEmailAddress == null) reqContactEmailAddress = "";

            reqMessage = sp.getReqMessage();
            if ( (reqMessage != null) && (reqMessage.length > 0) )
            {
               messagetext = reqMessage[0].getMessagetext();

               if (messagetext == null)
                  messagetext = "";
            }
         }
      }
      catch ( Exception e) { }

      PrintWriter out = startHtmlOutput(response, "Stop Check Payment");

      out.println("<FORM METHOD=POST ACTION=IfsWpStopPayment >");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"Confirm\" >");
      out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + STOP_CHECK_PAYMENT + "\" >");

      out.println("<DIV><B>Select an Account:</B></DIV>");
      out.println("<DIV><SELECT NAME=reqAccountID >");
      displayAccounts(up, account, out);
      out.println("</SELECT></DIV>");

      // Stop Payment Type choice
      out.println("<DIV><B>Select Stop Payment Type:</B></DIV>");
      out.println("<DIV><SELECT NAME=reqStopPaymentType >");
      displayStopPaymentList(stopPayment, out);
      out.println("</SELECT></DIV>");

      // Check Number input field
      out.println("<DIV><B>Check Number or Range:</B></DIV>");
      out.println("<TABLE WIDTH=\"100%\" BORDER=0 >");
      out.println("<TR><TD>From:</TD>");
      out.println("<TD><INPUT TYPE=text NAME=reqCheckNumberLow SIZE=10 MAXLENGTH=9 VALUE=\"" +
                  reqCheckNumberLow + "\" ></TD></TR>");
      out.println("<TR><TD>To:</TD>");
      out.println("<TD><INPUT TYPE=text NAME=reqCheckNumberHigh SIZE=10 MAXLENGTH=9 VALUE=\"" +
                  reqCheckNumberHigh + "\" ></TD></TR>");
      out.println("</TABLE>");

      // Check Amount input field
      out.println("<DIV><B>Amount or Range:</B></DIV>");
      out.println("<TABLE WIDTH=\"100%\" BORDER=0 >");
      out.println("<TR><TD>From:</TD>");
      out.println("<TD>" + strCurrencyPrefix +
                  " <INPUT TYPE=text NAME=reqCheckRangeLowAmount SIZE=13 MAXLENGTH=20 VALUE=\"" +
                  reqCheckRangeLowAmount + "\" ></TD></TR>");
      out.println("<TR><TD>To:</TD>");
      out.println("<TD>" + strCurrencyPrefix +
                  " <INPUT TYPE=text NAME=reqCheckRangeHighAmount SIZE=13 MAXLENGTH=20 VALUE=\"" +
                  reqCheckRangeHighAmount + "\" ></TD></TR>");
      out.println("</TABLE>");

      // Posting Date input field
      out.println("<DIV><B>Posting Date or Range:</B></DIV>");
      out.println("<TABLE WIDTH=\"100%\" BORDER=0 >");
      out.println("<TR><TD>From:</TD>");
      out.println("<TD><INPUT TYPE=text NAME=reqCheckIssueStartDate SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  reqCheckIssueStartDate + "\" ></TD></TR>");
      out.println("<TR><TD>To:</TD>");
      out.println("<TD><INPUT TYPE=text NAME=reqCheckIssueEndDate SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  reqCheckIssueEndDate + "\" ></TD></TR>");
      out.println("<TR><TD></TD>");
      out.println("<TD>" + strDateHint + "</TD></TR>");
      out.println("</TABLE>");

      // Contact Information input field
      out.println("<P><DIV><B>Contact Information:</B> (optional)</DIV><HR>");
      out.println("<TABLE WIDTH=\"100%\" BORDER=0 >");

      // Home Phone Number input field
      out.println("<TR><TD ALIGN=LEFT >Home Phone:</TD>");
      out.println("<TD><INPUT NAME=reqContactHomePhoneNumber TYPE=text SIZE=12 MAXLENGTH=15 VALUE=\"" +
                  reqContactHomePhoneNumber + "\"> </TD></TR>");

      // Work Phone Number input field
      out.println("<TR><TD ALIGN=LEFT >Work Phone:</TD>");
      out.println("<TD><INPUT NAME=reqContactWorkPhone TYPE=text SIZE=12 MAXLENGTH=15 VALUE=\"" +
                  reqContactWorkPhone + "\"> </TD></TR>");

      // Fax Number input field
      out.println("<TR><TD ALIGN=LEFT >Fax Number:</TD>");
      out.println("<TD><INPUT NAME=reqContactFaxNumber TYPE=text SIZE=12 MAXLENGTH=15 VALUE=\"" +
                  reqContactFaxNumber + "\"> </TD></TR>");
      out.println("</TABLE>");

      // E-Mail Address input field
      out.println("<DIV>E-Mail Address:</DIV>");
      out.println("<DIV ALIGN=LEFT><INPUT TYPE=text NAME=reqContactEmailAddress SIZE=25 MAXLENGTH=30 VALUE=\"" +
                  reqContactEmailAddress + "\"></DIV>");

      // Stop Message input field
      out.println("<P><DIV>Additional Information: (optional)</DIV>");
      out.println("<DIV><TEXTAREA NAME=transact.reqMessage.0.messagetext ROWS=2 COLS=17 WRAP=virtual>" +
                  messagetext + "</TEXTAREA></DIV>");

      out.println("<P><CENTER><INPUT NAME=SUBMIT TYPE=submit VALUE=\"Enter\" ></CENTER>");
      out.println("</FORM>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   //*******************************
   // Displays the input form for generating a Stop Check Payment transaction.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         displayInputForm(up, ifsSession, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in doGet method of IfsWpStopPayment Servlet, explanation: ");
      }
   }


   //*******************************
   // Either displays the confirmation page for a Stop Check Payment
   // transaction or executes it.
   //
   // The following Http Request Parameters are posted to this page:
   //     ReqTag   - Indicates whether to display confirmation or execute.
   //                It can have the values of "Confirm" or "Result".
   //     transfer - Specifies the object identifier (oid) of a
   //                GtStopCheckPayment object.
   //     req????  - The input values from the Stop Check Payment form.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;
      GtStopCheckPayment transact = null;
      String strReqTag = null;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtStopCheckPayment object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transact = (GtStopCheckPayment) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtStopCheckPayment",
                             "transact");

         transact.setReqCheckNumberLow(0);
         transact.setReqCheckNumberHigh(0);

         try {                                                               
              GsCurrency nullCurrency = new GsCurrency();
              nullCurrency = GsCurrency.fromString(" ");                                                                 
              transact.setReqCheckRangeLowAmount(nullCurrency);
              transact.setReqCheckRangeHighAmount(nullCurrency);
             }
         catch(Exception e)
            { debug("GsCurrency creation and initialization" + e.toString());
              throw e;
             }
 

         try
         {
            // Since setFields requires arrays to be initialized, assure the
            // reqMessage array exists.
            GsMsgTxt[] stopMsg = transact.getReqMessage();
            if ( stopMsg == null )
            {
               stopMsg = new GsMsgTxt[1];
               stopMsg[0] = new GsMsgTxt();

               transact.setReqMessage(stopMsg);
            }


            // Set fields of GtStopCheckPayment object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transact",transact);

            // insert the first 2 digits of the year with 80 as the cut-off value.
            GsDate date = new GsDate();
            date = yearWindow(transact.getReqCheckIssueStartDate());
            transact.setReqCheckIssueStartDate(date);

            date = yearWindow(transact.getReqCheckIssueEndDate());
            transact.setReqCheckIssueEndDate(date);

         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         strReqTag = (String)getRequestParameter(request,"ReqTag");

         if (strReqTag.equals("Confirm"))
            displayConfirmInfo(up, ifsSession, transact, request, response);
         else if (strReqTag.equals("Result"))
            showTxnResults(ifsSession, transact, request, response);

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in doPost method of IfsWpStopPayment Servlet, explanation: ");
      }
   }

   /**
   * Displays the transaction results.
   */
   private void showTxnResults(IfsSession ifsSession,
                               GtStopCheckPayment transact,
                               HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      try
     {
         //added try-catch block 1/18/2002
         try
        {
        
         transact.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(transact, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpStopPayment",
                      "IfsWpRedoPage?nextpage=IfsWpStopPayment")){
             return;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Unable to do stop check payment. Exception caught in showTxnResults method of IfsWpStopPayment Servlet, explanation: ");
      }

      PrintWriter out = startHtmlOutput(response, "Stop Payment Requested");

      out.println("<P>Your stop check payment has been requested successfully!<HR>");

      show2DivBlocks(out, "FI Reference Number:", transact.getReferenceNumber() );
      show2DivBlocks(out, "Service Provider Reference Id:",
                     transact.getServiceProviderReference() );

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Validate the data that was entered.
   */
   private String validateStopPayment(GtStopCheckPayment scp)
   {
      StringBuffer strBuf = new StringBuffer("");
      int index = 0;
      String sStart = "";
      String sEnd = "";

      try { index = Integer.parseInt(scp.getReqStopPaymentType()); }
      catch (NumberFormatException e) { index = 20; }   // Invalid stop payment type.

      switch (index)
      {
         case 0:  // stop payment by check number
            if ( isValidNumber(scp.getReqCheckNumberLow()) == false )
               strBuf.append("<BR>From Check Number is invalid! It must be a positive non-zero value.");
            break;

         case 1:  // stop payment by check number and amount
            if ( isValidNumber(scp.getReqCheckNumberLow()) == false )
               strBuf.append("<BR>From Check number is invalid! It must be a positive non-zero value.");

            if ( isValidAmount(scp.getReqCheckRangeLowAmount()) == false )
               strBuf.append("<BR>From Check amount is invalid!");

            break;

         case 2:  // stop payment by check number range
            if (isValidNumber(scp.getReqCheckNumberLow()) == false)
               strBuf.append("<BR>From Check number is invalid! It must be a positive non-zero value.");

            if (isValidNumber(scp.getReqCheckNumberHigh()) == false)
               strBuf.append("<BR>To Check number is invalid! It must be a positive non-zero value.");

            if (scp.getReqCheckNumberLow() > scp.getReqCheckNumberHigh())
               strBuf.append("<BR>From Check number must be less than or equal to To Check number!");

            break;

         case 3:  // stop payment by amount
            if ( isValidAmount(scp.getReqCheckRangeLowAmount()) == false )
               strBuf.append("<BR>From Check amount is invalid!");

            break;

         case 4:  // stop payment by amount range
            try
            {
               if (isValidAmount(scp.getReqCheckRangeLowAmount()) == false )
                  strBuf.append("<BR>From Check amount is invalid!");

               if (isValidAmount(scp.getReqCheckRangeHighAmount()) == false )
                  strBuf.append("<BR>To Check amount is invalid!");

               if ( scp.getReqCheckRangeLowAmount().getNumeric() >
                    scp.getReqCheckRangeHighAmount().getNumeric() )
                  strBuf.append("<BR>The From Check amount must be greater than or equal to the To Check amount!");
            }
            catch (Exception e)
            {
               strBuf.append("<BR>The From Check amount or the To Check amount is invalid!");
            }
            break;

         case 5:  // stop payment by date
            if ( isValidDate(scp.getReqCheckIssueStartDate()) == false)
               strBuf.append("<BR>Posting From Date is invalid!");

            break;

         case 6:  // stop payment by date and amount
            if ( isValidDate(scp.getReqCheckIssueStartDate()) == false)
               strBuf.append("<BR>Posting From Date is invalid!");

            if ( isValidAmount(scp.getReqCheckRangeLowAmount()) == false )
               strBuf.append("<BR>From check amount is invalid!");

            break;

         case 7:  // stop payment by date range
            try
            {
               if (isValidDate(scp.getReqCheckIssueStartDate()) == false)
                  strBuf.append("<BR>Posting From Date is invalid!");

               if (isValidDate(scp.getReqCheckIssueEndDate()) == false)
                  strBuf.append("<BR>Posting To Date is invalid!");

               sStart = scp.getReqCheckIssueStartDate().toString(new GsDateFormat("yyyy/MM/dd"));
               sEnd = scp.getReqCheckIssueEndDate().toString(new GsDateFormat("yyyy/MM/dd"));

               if (sStart.compareTo(sEnd) >= 0)
                  strBuf.append("<BR>The From Date must be later than the To Date!");
            }
            catch (Exception e)
            {
               strBuf.append("<BR>The From Date or the To Date is invalid!");
            }

            break;

         case 8:  // stop all check payment
            break;

         default:
            strBuf.append("<BR>Invalid stop payment type");
            break;
      }

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         scp.debug("Validation failed in validateStopPayment method of IfsWpStopPayment class");
         scp.debug(strBuf.toString());
      }

      return strBuf.toString();
   }

}
