//-*************************************
//
// FILE NAME:  IfsWpTransferInquiry.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resource.*;
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statement for HandleErrors
 *
 */

import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import javax.servlet.http.*;
import javax.servlet.*;
import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This class handles the transaction for Transfer Inquiry
* which allows user to request information about Transfers.
*/
public class IfsWpTransferInquiry extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;

   // the number of displayed records in one page
   final static int TOTAL_PAGE_DISPAY = 5;

   // default value of maxmium received record number
   final static int MAX_REC_NUM =  2 * TOTAL_PAGE_DISPAY;
   final static int MAX_REC_NUM_STEP = 2 * TOTAL_PAGE_DISPAY;


   //*******************************
   // Displays the input form page for a Transfer Inquiry transaction.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      String[] transferStatusText = { "Scheduled - All", "Scheduled - Recurring",
                                      "Scheduled - One-Time", "Historical" };
      int[] transferStatusCodes = { 1, 2, 3, 4 };

      IfsSession ifsSession = new IfsSession(request, response);

      GtUserProfile up = null;

      GtTransferHistoryInquiry th      = null;
      GsAccount                account = null;
      int           transferStatus     = 1;

      String        transferLowAmount  = "";
      String        transferHighAmount = "";
      String        transferStartDate  = "";
      String        transferEndDate    = "";

      //replaced getValue with getAttribute 1/15/02
      up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
      if (up == null)
      {
         // Since no user profile exists, there is no active session for this consumer.
         callPage_NoSession(response);
      }

      try
      {
         //---------------
         // Check for redisplay of originally entered information.
         // Get a reference to the GtTransferHistoryInquiry cached object.
         //
         th = (GtTransferHistoryInquiry) getObject(ifsSession, TRANSFER_INQUIRY);
         if (th != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            account = th.getReqFundingAccount();

            // Status of transfers to inquire.
            if ( (th.getReqTransferType() != null) &&
                 (th.getReqTransferStateIndicator() != null))
            {
               if (th.getReqTransferType().equals("1"))
                  transferStatus = 2;

               else if (th.getReqTransferType().equals("2"))
                  transferStatus = 3;

               else if ( (th.getReqTransferType().equals("3")) &&
                         (th.getReqTransferStateIndicator().equals("2")) )
                  transferStatus = 4;
               else
                  transferStatus = 1;
            }

            // start and end dates
            transferStartDate = getDateString(th.getReqTransferDateRangeStartDate());
            transferEndDate = getDateString(th.getReqTransferDateRangeEndDate());

            // low and high amounts
            transferLowAmount = getAmountString(th.getReqTransferAmountRangeLowAmount());
            transferHighAmount = getAmountString(th.getReqTransferAmountRangeHighAmount());
         }
      }
      catch ( Exception e) { }

      PrintWriter out = startHtmlOutput(response, "Transfer Inquiry");

      out.println("<FORM METHOD=POST NAME=\"TransferInquiry\" ACTION=\"IfsWpTransferInquiry\">");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispFirst\" >");
      out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + TRANSFER_INQUIRY + "\" >");

      // Generate SELECT with account numbers
      out.println("<B>Please select an account:</B><BR>");
      out.println("<SELECT NAME=SourceAccount >");
      out.println("<OPTION VALUE=\"-1\"> All Funding Accounts");
      displayAccountNum(up, account, out);
      out.println("</SELECT><BR>");

      out.println("<B> Date Range: </B>(Optional)<BR>");
      out.println("<TABLE BORDER=0   WIDTH=95%>");
      out.println("<TR>");
      out.println("<TD  ALIGN=LEFT > From: </TD>");
      out.println("<TD ><INPUT NAME=reqTransferDateRangeStartDate TYPE=text SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  transferStartDate + "\" ></TD>");
      out.println("</TR><TR>");
      out.println("<TD  ALIGN=LEFT > To: </TD>");
      out.println("<TD  ><INPUT NAME=reqTransferDateRangeEndDate TYPE=text SIZE=10 MAXLENGTH=10 VALUE=\"" +
                  transferEndDate + "\" ></TD>");
      out.println("</TR><TR>");
      out.println("<TD></TD> <TD >"+strDateHint+"</TD>");
      out.println("</TR>");
      out.println("</TABLE>");

      out.println("<B>Amount Range: </B>(Optional)<BR>");
      out.println("<TABLE BORDER=0   WIDTH=95%>");
      out.println("<TR>");
      out.println("<TD  ALIGN=LEFT > From:  </TD>");
      out.println("<TD >" + strCurrencyPrefix +
                  " <INPUT NAME=reqTransferAmountRangeLowAmount TYPE=text SIZE=13 MAXLENGTH=20 VALUE=\"" +
                  transferLowAmount + "\" ></TD>");
      out.println("</TR><TR>");
      out.println("<TD  ALIGN=LEFT > To:  </TD>");
      out.println("<TD>" + strCurrencyPrefix +
                  " <INPUT NAME=reqTransferAmountRangeHighAmount TYPE=text SIZE=13 MAXLENGTH=20 VALUE=\"" +
                  transferHighAmount + "\" ></TD>");
      out.println("</TR>");
      out.println("</TABLE>");

      out.println("<B>Transfer Type and Status: </B><BR>");
      out.println("<SELECT NAME=transfer_status >");
      for (int i = 0; i < transferStatusText.length; i++)
      {
         if (transferStatus == transferStatusCodes[i])
            out.println("<OPTION SELECTED VALUE=\"" +
                        String.valueOf(transferStatusCodes[i]) + "\" >");
         else
            out.println("<OPTION VALUE=\"" + String.valueOf(transferStatusCodes[i]) + "\" >");

         out.println( transferStatusText[i]);
      }
      out.println("</SELECT><BR>");

      //submit
      out.println("<P><DIV ALIGN=CENTER>");
      out.println("<INPUT TYPE=SUBMIT NAME=SUBMIT VALUE=\"Search\" ></FORM>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   //*******************************
   // Displays the results of a Transfer Inquiry transaction.
   //
   // The following Http Request Parameters are posted to this page:
   //     ReqTag   - Indicates which group of results is to be displayed.
   //                It can have the values of "DispFirst" or "DispMore".
   //     transfer - Specifies the object identifier (oid) of a
   //                GtTransferHistoryInquiry object.
   //     req????  - The input values from the Transfer Inquiry Transaction form.
   //
   //     SourceAccount - The selected account to be searched.
   //                     It is posted when ReqTag=DispFirst.
   //
   //     NUM     - The starting index of the next transfer to display.
   //               It is posted when ReqTag=DispMore.
   //     LEFTNUM - The number of transfers left to display.
   //               It is posted when ReqTag=DispMore.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      IfsSession ifsSession = new IfsSession(request, response);
      GtTransferHistoryInquiry transfer = null;

      GtUserProfile up;
      String msg = null;
      String strReq = null;

      try
      {
        //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtTransferHistoryInquiry object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transfer = (GtTransferHistoryInquiry) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtTransferHistoryInquiry",
                             "transfer");


// clearing cached data.
try {
      GsCurrency nullCurrency = new GsCurrency();
      nullCurrency = GsCurrency.fromString(" ");
      transfer.setReqTransferAmountRangeLowAmount(nullCurrency);
      transfer.setReqTransferAmountRangeHighAmount(nullCurrency);
     }
catch(Exception e)
    { debug("GsCurrency creation and initialization" + e.toString());
      throw e;
     }


         try
         {
            // Set fields of GtTransferHistoryInquiry object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transfer",transfer);


            // insert the first 2 digits of the year with 80 as the cut-off value.
            GsDate date = new GsDate();
            date = yearWindow(transfer.getReqTransferDateRangeStartDate());
            transfer.setReqTransferDateRangeStartDate(date);

            date = yearWindow(transfer.getReqTransferDateRangeEndDate());
            transfer.setReqTransferDateRangeEndDate(date);

         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         strReq = (String)getRequestParameter(request,"ReqTag");

         if (strReq.equals("DispFirst"))
            showFirstHistoryPage(up, ifsSession, transfer, request, response);
         else
            showMoreHistoryPages(up, ifsSession, transfer, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to get transfer inquiry results. Exception in doPost method of IfsWpTransferInquiry Servlet, explanation: ");
      }
   }

   /**
   * Display the first transfer history screen
   */
   private void showFirstHistoryPage(GtUserProfile up, IfsSession ifsSession,
                                     GtTransferHistoryInquiry transfer,
                                     HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GsTfrDtl detail;
      String msg = "";
      String transfer_status = "";
      String oid = "";

      GsAcctDtl acct = null;
      String type = "";
      String acctNbr = "";
      String acctNick = "";

      //Check input information
      try
      {
         transfer_status = (String)getRequestParameter(request,"transfer_status");
         int intCode = Integer.parseInt(transfer_status);

         // Transfer type values are from IfsTfrTpIndResource property file.
         // Transfer state values are from IfsTfrStIndResource property file.
         switch (intCode)
         {
            case 1 :  // All scheduled transfers.
               transfer.setReqTransferType("3");             // All
               transfer.setReqTransferStateIndicator("1");   // Pending
               break;
            case 2 :  // All scheduled recurring transfers.
               transfer.setReqTransferType("1");             // Recurring
               transfer.setReqTransferStateIndicator("1");   // Pending
               break;
            case 3 :  // All scheduled single transfers.
               transfer.setReqTransferType("2");             // Single
               transfer.setReqTransferStateIndicator("1");   // Pending
               break;
            case 4 :  // All completed transfers.
               transfer.setReqTransferType("3");             // All
               transfer.setReqTransferStateIndicator("2");   // Completed
               break;
            default :
               callPage_RedoPage(response, "IfsWpTransferInquiry",
                      "Transfer status is not valid. Valid values are 1, 2, 3, 4. transfer_status = " +
                      String.valueOf(transfer_status));
         }

         //set funding account
         int ii = Integer.parseInt( (String)getRequestParameter(request,"SourceAccount"));
         if (ii >= 0)
         {
            // get values
            acct = up.getAccountDetail()[ii];
            type = acct.getAccount().getType();
            acctNbr = acct.getAccount().getNumber();
            acctNick = acct.getAccount().getNumber();

            // create account
            GsAccount anAccount = new GsAccount(type, acctNbr, acctNick);
            transfer.setReqFundingAccount(anAccount);
         }
        
         transfer.setReqMaximumRecordNumber(MAX_REC_NUM);

         // Get the object ID of the cached GtTransferHistoryInquiry instance object.
         oid = getOid(ifsSession,transfer);

         // Validate the data that was entered.
         msg = validateTransferInquiry(transfer, request);
         if ( msg.equals("OK") == false)
            callPage_RedoPage(response, "IfsWpTransferInquiry", msg);


         //added try-catch block 1/18/2002
         try
        {
        
         transfer.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(transfer, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpTransferInquiry",
                      "IfsWpRedoPage?nextpage=IfsWpTransferInquiry")){
             return;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to do transfer inquiry. Exception in showFirstHistoryPage method of IfsWpTransferInquiry Servlet, explanation: ");
      }
   

      //--------------------
      // Start outputting the transfer inquiry information.
      //
      PrintWriter out = startHtmlOutput(response, "Transfer Inquiry");

      // get totalMatched and totalSent
      int totalMatched = transfer.getTotalTransactionsMatched();
      int totalSent = transfer.getTotalTransactionsSent();
      int leftNum = totalSent;
      if (totalMatched == 0)
      {
         out.println("There is no Transfers History.");

         // link to other pages and close the print writer.
         endHtmlOutput(out);
         return;
      }

      int endPageNum = 0;
      // set end page display number
      if (totalSent < TOTAL_PAGE_DISPAY)
         endPageNum = totalSent;
      else
         endPageNum = TOTAL_PAGE_DISPAY;

      // display each Transfer
      out.println("<HR>");
      for (int i = 0; i < endPageNum; i++)
      {
         if ( (i >= transfer.getTransferHistoryDetail().length) ||
              (transfer.getTransferHistoryDetail()[i] == null) )
            continue;

         detail = transfer.getTransferHistoryDetail()[i];
         showHistoryPage(detail, out);
      }

      // link to more Transfers
      if (totalSent > 0)
         out.println("<P><DIV ALIGN=CENTER>Transfer 1 - " + endPageNum + " (Total: " + totalMatched + ")</DIV>");
      else
         out.println("<P> There is no transfer history. <BR>");

      if (endPageNum < totalMatched)
      {
         leftNum -= endPageNum;
         out.println("<DIV ALIGN=CENTER>");
         out.println("<FORM METHOD=POST ACTION=IfsWpTransferInquiry >");
         out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispMore\" >");
         out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + oid + "\" >");
         out.println("<INPUT TYPE=hidden NAME=NUM VALUE=\"" + endPageNum + "\" >");
         out.println("<INPUT TYPE=hidden NAME=LEFTNUM VALUE=\"" + leftNum + "\" >");
         out.println("<INPUT TYPE=submit NAME=BUTTON VALUE=\"More\" >");
         out.println("</FORM></DIV>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Show first page of transfer inquiry history.
   */
   private void showHistoryPage(GsTfrDtl detail, PrintWriter out)
   {
      String transferType = "";
      String strValue = "";

      //transfer type
      try { strValue = detail.getTransferType_text(); }
      catch (Exception e) { strValue = "Unknown transfer type: " + detail.getTransferType(); }
      show2DivBlocks (out, "Type:", strValue);

      // Frequency
      transferType = detail.getTransferType();

      if ( (transferType != null) && (transferType.equals("1")) )
      {
         try { strValue = detail.getFrequency_text(); }
         catch (Exception e) { strValue = "Unknown frequency: " + detail.getFrequency(); }
         show2DivBlocks (out, "Frequency:", strValue);
      }

      show2DivBlocks (out, "Source:", detail.getSource());
      show2DivBlocks (out, "Target:", detail.getTarget());
      show2DivBlocks (out, "Amount:", detail.getTransactionAmount());
      show2DivBlocks (out, "Effective Date:", detail.getEffectiveDate());
      show2DivBlocks (out, "Nickname:", detail.getTransferNickname());

      out.println("<HR>");
   }

   /**
   * Display the continued transfer history screens
   */
   private void showMoreHistoryPages(GtUserProfile up, IfsSession ifsSession,
                                     GtTransferHistoryInquiry transfer,
                                     HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      String oid = "";

      GsTfrDtl detail = null;
      GsAcctDtl acct = null;

      int maxRecNum = 0;
      int totalMatched = 0;
      int totalSent = 0;
      int startPageNum  = 0;
      int leftNum = 0;
      int endPageNum = 0;

      try
      {
         // Get the object ID of the cached GtTransferHistoryInquiry instance object.
         oid = getOid(ifsSession,transfer);

         maxRecNum = transfer.getReqMaximumRecordNumber();
         totalMatched = transfer.getTotalTransactionsMatched();
         totalSent = transfer.getTotalTransactionsSent();

         // get startPageNum
         startPageNum = Integer.parseInt( (String)getRequestParameter(request,"NUM"));

         // get leftNum
         leftNum = Integer.parseInt( (String)getRequestParameter(request,"LEFTNUM"));

         // need execute Transfer again
         if ((startPageNum + leftNum < totalMatched) &&  // not all matched has been sent
             (leftNum < TOTAL_PAGE_DISPAY))            // not enought to fill all page
         {

            // set maximum number
            maxRecNum += MAX_REC_NUM_STEP;
            transfer.setReqMaximumRecordNumber(maxRecNum);

            // execute

            //added try-catch block 1/18/2002
         try
        {
        
         transfer.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
           if (! handleErrors(transfer, "IfsWpErrorPage",
                         "IfsWpRedoPage?nextpage=IfsWpTransferInquiry",
                         "IfsWpRedoPage?nextpage=IfsWpTransferInquiry")){
               return;
           }

            // get totalSent
            totalSent = transfer.getTotalTransactionsSent();
            leftNum = totalSent - startPageNum;
         }
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to do transfer inquiry. Exception in showMoreHistoryPages method of IfsWpTransferInquiry Servlet, explanation: ");
      }

      //--------------------
      // Start outputting transfer inquiry data,
      //
      PrintWriter out = startHtmlOutput(response, "Transfer Inquiry>");

      // set end page display number
      if (leftNum < TOTAL_PAGE_DISPAY)
         endPageNum = leftNum + startPageNum;
      else
         endPageNum = TOTAL_PAGE_DISPAY + startPageNum;

      for (int i = startPageNum ; i < endPageNum; i++)
      {
         if ( (i >= transfer.getTransferHistoryDetail().length) ||
              (transfer.getTransferHistoryDetail()[i] == null) )
            continue;

         detail = transfer.getTransferHistoryDetail()[i];
         showHistoryPage(detail, out);
      }

      // link to more Transfers
      out.println ("<P><DIV ALIGN=CENTER>Transfer " + (startPageNum + 1) + " - "
                   + endPageNum + " (Total: " + totalMatched + ")</DIV>");

      if (endPageNum < totalMatched)
      {
         leftNum = leftNum - (endPageNum - startPageNum);

         out.println("<DIV ALIGN=CENTER>");
         out.println("<FORM METHOD=POST ACTION=IfsWpTransferInquiry >");
         out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispMore\" >");
         out.println("<INPUT TYPE=hidden NAME=transfer VALUE=\"" + oid + "\" >");
         out.println("<INPUT TYPE=hidden NAME=NUM VALUE=\"" + endPageNum + "\" >");
         out.println("<INPUT TYPE=hidden NAME=LEFTNUM VALUE=\"" + leftNum + "\" >");
         out.println("<INPUT TYPE=submit NAME=BUTTON VALUE=\"More\" >");
         out.println("</FORM></DIV>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   /**
   * Validate the data that was entered.
   */
   private String validateTransferInquiry(GtTransferHistoryInquiry th, HttpServletRequest request)
   {
      StringBuffer strBuf = new StringBuffer("");
      String strTemp = "";
      String reqTransferDateRangeStartDate = "";
      String reqTransferDateRangeEndDate = "";

      double dLowAmount = 0;
      double dHighAmount = 0;

      // Get the low amount information.
      try { dLowAmount = getAndCheckAmount(th.getReqTransferAmountRangeLowAmount()); }
      catch (Exception e)
      {
         th.debug("Exception getting low amount in validateTransferInquiry - " + e.toString());
         dLowAmount = 0;
      }

      // Get the high amount information.
      try { dHighAmount = getAndCheckAmount(th.getReqTransferAmountRangeHighAmount()); }
      catch (Exception e)
      {
         th.debug("Exception getting high amount in validateTransferInquiry - " + e.toString());
         dHighAmount = 0;
      }

      // Get the start Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         reqTransferDateRangeStartDate = th.getReqTransferDateRangeStartDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (reqTransferDateRangeStartDate == null)
            reqTransferDateRangeStartDate = "";
      }
      catch (Exception e)
      {
         th.debug("Exception getting starting date in validateSearchTransfer - " + e.toString());
         reqTransferDateRangeStartDate = "";
      }

      // Get the last transfer Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         reqTransferDateRangeEndDate = th.getReqTransferDateRangeEndDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (reqTransferDateRangeEndDate == null)
            reqTransferDateRangeEndDate = "";
      }
      catch (Exception e)
      {
         th.debug("Exception getting ending date in validateSearchTransfer - " + e.toString());
         reqTransferDateRangeEndDate = "";
      }

      // Validate the data that was entered.
      strTemp = (String)getRequestParameter(request,"reqTransferAmountRangeLowAmount");
      if ( (strTemp != null) && (strTemp.length() > 0) && (dLowAmount == 0) )
         strBuf.append("<BR>The From Amount value is invalid.");

      strTemp = (String)getRequestParameter(request,"reqTransferAmountRangeHighAmount");
      if ( (strTemp != null) && (strTemp.length() > 0) && (dHighAmount == 0) )
         strBuf.append("<BR>The To Amount value is invalid.");

      if (dLowAmount > dHighAmount)
         strBuf.append("<BR>The From Amount must be equal to or less than the To Amount.");

      strTemp = (String)getRequestParameter(request,"reqTransferDateRangeStartDate");
      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (reqTransferDateRangeStartDate.equals("")) )
         strBuf.append("<BR>Invalid From Date!");

      strTemp = (String)getRequestParameter(request,"reqTransferDateRangeEndDate");
      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (reqTransferDateRangeEndDate.equals("")) )
         strBuf.append("<BR>Invalid To Date!");

      if ( (reqTransferDateRangeStartDate.equals("") == false) &&
           (reqTransferDateRangeEndDate.equals("") == false) &&
           (reqTransferDateRangeStartDate.compareTo(reqTransferDateRangeEndDate) >= 0))
         strBuf.append("<BR>The From Date must be later than or equal to the To Date!");

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         th.debug("Validation failed in validateSearchTransfer method of IfsWpSearchTransfer class");
         th.debug(strBuf.toString());
      }

      return strBuf.toString();
   }

}
