//-*************************************
//
// FILE NAME:  IfsWpTxnHistory.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resource.*;
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statements for handleErrors                                       
 *
 */

import java.io.*;
import java.net.URLEncoder;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This class displays the posted transactions for a specific account
* by transaction date range selection.
*/
public class IfsWpTxnHistory extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;

   // the number of displayed records in one page
   final static int TOTAL_PAGE_DISPAY = 4;

   // default value of maxmium received record number
   final static int MAX_REC_NUM =  10 * TOTAL_PAGE_DISPAY;
   // when total matched number larger than maxmium record number,
   // MAX_REC_NUM = MAX_REC_NUM + MAX_REC_NUM_STEP
   final static int MAX_REC_NUM_STEP = 10 * TOTAL_PAGE_DISPAY;


   //*******************************
   // Displays the input form for generating a History transaction.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);

      GtUserProfile up;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         showAllAccounts(up, ifsSession, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to get transaction accounts. Exception in doGet method of IfsWpTxnHistory Servlet, explanation: ");
      }
   }


   //*******************************
   // Either displays the confirmation page for a History
   // transaction or executes it.
   //
   // The following Http Request Parameters are posted to this page:
   //     ReqTag   - Indicates whether to display confirmation or execute.
   //                It can have the values of "DispFirst" or "DispMore".
   //     transfer - Specifies the object identifier (oid) of a
   //                GtAccountHistory object.
   //     req????  - The input values from the History Transaction form.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);
      GtAccountHistory transact = null;
      GtUserProfile up = null;
      String strReq = null;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtTransferHistoryInquiry object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transact = (GtAccountHistory) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtAccountHistory",
                             "transact");
         try
         {
            // Set fields of GtAccountHistory object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transact",transact);

            // insert the first 2 digits of the year with 80 as the cut-off value.
            GsDate date = new GsDate();
            date = yearWindow(transact.getReqPostingStartDate());
            transact.setReqPostingStartDate(date);

            date = yearWindow(transact.getReqPostingEndDate());
            transact.setReqPostingEndDate(date);

         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         strReq = (String)getRequestParameter(request,"ReqTag");

         if (strReq.equals("DispFirst"))
            showFirstHistoryPage(up, ifsSession, transact, request, response);
         else
            showMoreHistoryPages(ifsSession, transact, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to show transaction history. Exception in doPost method of IfsWpTxnHistory Servlet, explanation: ");
      }
   }


   /**
   * Display transaction history of checking account
   */
   private void handleCheckingDisplay(GsTxnRespDtl txnRespDtl, PrintWriter out)
                throws IOException
   {
      String strValue = "";

     show2DivBlocks (out, "Date:", txnRespDtl.getPostingDate());

      // The common part of the accounts.
      handleCommonPartDisplay(txnRespDtl, out);
   }


   /**
   * Display the common part of the accounts.
   */
   private void handleCommonPartDisplay(GsTxnRespDtl txnRespDtl, PrintWriter out)
                throws IOException
   {
      String strValue = "";

      // Transaction Type
      try { strValue = txnRespDtl.getType_text(); }
      catch (Exception e) { strValue = ""; }
      show2DivBlocks (out, "Transaction Type:", strValue);

      // Transaction Source
      try { strValue = txnRespDtl.getSource_text(); }
      catch (Exception e) { strValue = ""; }
      show2DivBlocks (out, "Transaction Source:", strValue);

      show2DivBlocks (out, "Transaction Amount:", txnRespDtl.getTransactionAmount());

      // Check Number
      try { strValue = "" + txnRespDtl.getCheckID()[0].getNumber(); }
      catch (Exception e) { strValue = ""; }
      show2DivBlocks (out, "Check Number:", strValue);

      show2DivBlocks (out, "Description:", txnRespDtl.getDescription());

      out.println ("<HR>");

   }


   /**
   * Display transaction history of credit card account
   */
   private void handleCreditcardDisplay(GsTxnRespDtl txnRespDtl, PrintWriter out)
                throws IOException
   {
      show2DivBlocks (out, "Posting Date:", txnRespDtl.getPostingDate());
      show2DivBlocks (out, "Original Date:", txnRespDtl.getOriginalDate());
      show2DivBlocks (out, "Reference Number:", txnRespDtl.getReferenceNumber());

      // The common part of the accounts.
      handleCommonPartDisplay(txnRespDtl, out);
   }

   /**
   * Display transaction history of loan account
   */
   private void handleLoanDisplay(GsTxnRespDtl txnRespDtl, PrintWriter out)
                throws IOException
   {
      show2DivBlocks (out, "Date:", txnRespDtl.getPostingDate());
      show2DivBlocks (out, "Amount to Principal:", txnRespDtl.getToPrincipalAmount());
      show2DivBlocks (out, "Amount to Interest:", txnRespDtl.getToInterestAmount());
      show2DivBlocks (out, "Amount to Escrow:", txnRespDtl.getToEscrowAmount());

      // The common part of the accounts.
      handleCommonPartDisplay(txnRespDtl, out);
   }


   /**
   * Display a list of all accounts
   */
   private void showAllAccounts(GtUserProfile up, IfsSession ifsSession,
                                HttpServletResponse response)
                throws ServletException, IOException
   {
      GtAccountHistory ah      = null;
      GsAccount        account = null;

      String           reqPostingStartDate = "";
      String           reqPostingEndDate   = "";

      try
      {
         //---------------
         // Need to redisplay the originally entered information.
         // Get a reference to the GtAccountHistory cached object.
         //
         ah = (GtAccountHistory) getObject(ifsSession, TRANSACTION_HISTORY);
         if (ah != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            account = ah.getReqAccount();

            // start and end dates
            reqPostingStartDate = getDateString(ah.getReqPostingStartDate());
            reqPostingEndDate = getDateString(ah.getReqPostingEndDate());
         }
      }
      catch ( Exception e) { }

      PrintWriter out = startHtmlOutput(response, "TRANSACTION HISTORY");

      // output form
      out.println("<FORM METHOD=POST ACTION=IfsWpTxnHistory >");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispFirst\" >");
      out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + TRANSACTION_HISTORY + "\" >");

      // account
      out.println("<B>Please select an account:</B><BR>");
      out.println("<SELECT NAME=reqAccount >");
      displayAccounts(up, account, out);
      out.println("</SELECT><BR>");

      // start date item
      out.println("<B>From Date: </B>(optional)<BR>");
      out.println("<INPUT TYPE=text NAME=reqPostingStartDate SIZE=10 " +
                  "MAXLENGTH=10 VALUE=\"" + reqPostingStartDate + "\" > (" +
                  strDateHint + ")<BR>");

      // end date item
      out.println("<B>To Date: </B>(optional)<BR>");
      out.println("<INPUT TYPE=text NAME=reqPostingEndDate SIZE=10 " +
                  "MAXLENGTH=10 VALUE=\"" + reqPostingEndDate + "\" > (" +
                  strDateHint + ")<BR>");

      // submit
      out.println("<P><DIV ALIGN=CENTER>");
      out.println("<INPUT TYPE=submit NAME=SUBMIT VALUE=\"Enter\" >");
      out.println("</DIV>");
      out.println("</FORM>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Display the first transaction history screen
   */
   private void showFirstHistoryPage(GtUserProfile up, IfsSession ifsSession,
                                     GtAccountHistory txn,
                                     HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GsTxnRespDtl[] txnRespDtl = null;

      int endPageNum = 0;
      int maxRecNum = 0;
      int totalMatched = 0;
      int totalSent = 0;
      int leftNum = 0;

      String oid = null;
      String msg = null;
      String type = null;

      try
      {
         // Get the object ID of the cached GtAccountHistory instance object.
         oid = getOid(ifsSession,txn); 

         txn.setReqTransactionType("1");    // ALL TYPES
         txn.setReqTransactionSource("0");  // ALL accounts

         txn.setReqProductType("");
         txn.setReqReferenceNumber("");

         txn.setReqMaximumNumberOfRecords(MAX_REC_NUM);

         // Validate the data that was entered.
         msg = validateTransactionHistory(txn, request);
         if ( msg.equals("OK") == false)
            callPage_RedoPage(response, "IfsWpTxnHistory", msg);

         //added try-catch block 1/18/2002
         try
        {
        
         txn.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(txn, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpTxnHistory",
                      "IfsWpRedoPage?nextpage=IfsWpTxnHistory")){
             return;
         }

         // get totalMatched and totalSent
         totalMatched = txn.getTotalTransactionsMatched();
         totalSent = txn.getTotalTransactionsSent();
         leftNum = totalSent;

         txnRespDtl = txn.getTransaction();

         // set end page display number
         if (totalSent < TOTAL_PAGE_DISPAY)
            endPageNum = totalSent;
         else
            endPageNum = TOTAL_PAGE_DISPAY;

         type = txn.getReqAccount().getType();
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to show transaction history. Exception in showFirstHistoryPage method of IfsWpTxnHistory Servlet, explanation: ");
      }

      //--------------
      // Start outputting transaction history data.
      //
      PrintWriter out = startHtmlOutput(response, "TRANSACTION HISTORY");

      // display each transaction
      out.println ("<HR>");
      for (int i = 0; i < endPageNum; i++ )
      {
         if ( (i >= txnRespDtl.length) ||
              (txnRespDtl[i] == null) )
            continue;

         // deposit accounts
         if (type.equals("CDA") || type.equals("DDA") || type.equals("IRA") ||
             type.equals("SDA") || type.equals("MMA"))
         {
            // deposit accounts
            handleCheckingDisplay (txnRespDtl[i], out);
         }
         else if (type.equals("CCA") || type.equals("LOC") ||type.equals("ELC"))
         {
            // credit accounts
            handleCreditcardDisplay (txnRespDtl[i], out);
         }
         else if (type.equals("CLA") || type.equals("ILA") ||type.equals("MLA"))
         {
            // loan accounts
            handleLoanDisplay (txnRespDtl[i], out);
         }
         // invalid account type
         else
            out.println ("invalid account type<HR>");
      }

      // link to more transactions
      if (totalSent > 0)
         out.println ("<P><DIV ALIGN=CENTER>Transaction 1 - " +
                      endPageNum + " (Total: " + totalMatched + ")</DIV>");
      else
         out.println ("No transactions match your criteria.<BR>");

      if (endPageNum < totalMatched)
      {
         leftNum -= endPageNum;
         out.println("<DIV ALIGN=CENTER>");
         out.println("<FORM METHOD=POST ACTION=IfsWpTxnHistory >");
         out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispMore\" >");
         out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + oid + "\" >");
         out.println("<INPUT TYPE=hidden NAME=ENDPAGENUM VALUE=\"" + endPageNum + "\" >");
         out.println("<INPUT TYPE=hidden NAME=LEFTNUM VALUE=\"" + leftNum + "\" >");
         out.println("<INPUT TYPE=submit NAME=BUTTON VALUE=\"More\" >");
         out.println("</FORM></DIV>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Display the continued transaction history screens
   */
   private void showMoreHistoryPages(IfsSession ifsSession,
                                     GtAccountHistory txn,
                                     HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GsTxnRespDtl[] txnRespDtl = null;

      int startPageNum = 0;
      int endPageNum = 0;

      String type = "";
      String oid = "";

      int maxRecNum = 0;
      int totalMatched = 0;
      int totalSent = 0;
      int leftNum = 0;

      try
      {
         // get transaction response detail object ID.
         txnRespDtl = txn.getTransaction();

         maxRecNum = txn.getReqMaximumNumberOfRecords();
         totalMatched = txn.getTotalTransactionsMatched();
         totalSent = txn.getTotalTransactionsSent();
         type = txn.getReqAccount().getType();

         // Get the object ID of the cached GtAccountHistory instance object.
         oid = getOid(ifsSession,txn);

         // get startPageNum
         startPageNum = Integer.parseInt( (String)getRequestParameter(request,"ENDPAGENUM"));

         // get leftNum
         leftNum = Integer.parseInt( (String)getRequestParameter(request,"LEFTNUM"));

         // need execute transaction again
         if ((startPageNum + leftNum < totalMatched) &&  // not all matched has been sent
             (leftNum < TOTAL_PAGE_DISPAY))            // not enought to fill all page
         {
            // set maximum number
            maxRecNum += MAX_REC_NUM_STEP;
            txn.setReqMaximumNumberOfRecords(maxRecNum);

            // execute
            //added try-catch block 1/18/2002
         try
        {
        
         txn.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
           if (! handleErrors(txn, "IfsWpErrorPage",
                         "IfsWpRedoPage?nextpage=IfsWpTxnHistory",
                         "IfsWpRedoPage?nextpage=IfsWpTxnHistory")){
               return;
           }

            // get totalSent
            totalSent = txn.getTotalTransactionsSent();
            leftNum = totalSent - startPageNum;

            // get txnRespDtl
            txnRespDtl = txn.getTransaction();
         }

         // set end page display number
         if (leftNum < TOTAL_PAGE_DISPAY)
            endPageNum = leftNum + startPageNum;
         else
            endPageNum = TOTAL_PAGE_DISPAY + startPageNum;
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to show transaction history. Exception in showMoreHistoryPages method of IfsWpTxnHistory Servlet, explanation: ");
      }

      //----------------
      // Start outputting transaction information.
      //
      PrintWriter out = startHtmlOutput(response, "TRANSACTION HISTORY");

      // display each transaction
      out.println ("<HR>");
      for (int i = startPageNum; i < endPageNum; i++)
      {
         if ( (i >= txnRespDtl.length) ||
              (txnRespDtl[i] == null) )
            continue;

         if (type.equals("CDA") || type.equals("DDA") || type.equals("IRA") ||
             type.equals("SDA") || type.equals("MMA"))
         {
            // deposit accounts
            handleCheckingDisplay (txnRespDtl[i], out);
         }
         else if (type.equals("CCA") || type.equals("LOC") ||type.equals("ELC"))
         {
            // credit accounts
            handleCreditcardDisplay (txnRespDtl[i], out);
         }
         else if (type.equals("CLA") || type.equals("ILA") ||type.equals("MLA"))
         {
            // loan accounts
            handleLoanDisplay (txnRespDtl[i], out);
         }
         // invalid account type
         else
            out.println ("invalid account type<HR>");
      }

      // link to more transactions
      out.println ("<P><DIV ALIGN=CENTER>Transaction " + (startPageNum+1) + " - "
                   + endPageNum + " (Total: " + totalMatched + ")</DIV>");

      if (endPageNum < totalMatched)
      {
         leftNum = leftNum - (endPageNum - startPageNum);

         out.println("<DIV ALIGN=CENTER>");
         out.println("<FORM METHOD=POST ACTION=IfsWpTxnHistory >");
         out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispMore\" >");
         out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + oid + "\" >");
         out.println("<INPUT TYPE=hidden NAME=ENDPAGENUM VALUE=\"" + endPageNum + "\" >");
         out.println("<INPUT TYPE=hidden NAME=LEFTNUM VALUE=\"" + leftNum + "\" >");
         out.println("<INPUT TYPE=submit NAME=BUTTON VALUE=\"More\" >");
         out.println("</FORM></DIV>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }


   /**
   * Validate the data that was entered.
   */
   private String validateTransactionHistory(GtAccountHistory ah, HttpServletRequest request)
   {
      StringBuffer strBuf = new StringBuffer("");
      String strTemp = "";
      String reqPostingStartDate = "";
      String reqPostingEndDate = "";

      // Get the start Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         reqPostingStartDate = ah.getReqPostingStartDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (reqPostingStartDate == null)
            reqPostingStartDate = "";
      }
      catch (Exception e)
      {
         ah.debug("Exception getting From Date in validateTransactionHistory- " + e.toString());
         reqPostingStartDate = "";
      }

      // Get the last transfer Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         reqPostingEndDate = ah.getReqPostingEndDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (reqPostingEndDate == null)
            reqPostingEndDate = "";
      }
      catch (Exception e)
      {
         ah.debug("Exception getting To Date in validateTransactionHistory - " + e.toString());
         reqPostingEndDate = "";
      }

      // Validate the data that was entered.

      strTemp = (String)getRequestParameter(request,"reqPostingStartDate");
      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (reqPostingStartDate.equals("") == true) )
         strBuf.append("<BR>Invalid From Date!");

      strTemp = (String)getRequestParameter(request,"reqPostingEndDate");
      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (reqPostingEndDate.equals("") == true) )
         strBuf.append("<BR>Invalid To Date!");

      if ( (reqPostingStartDate.equals("") == false) &&
           (reqPostingEndDate.equals("") == false) &&
           (reqPostingStartDate.compareTo(reqPostingEndDate) >= 0))
         strBuf.append("<BR>The To Date must be later than the From Date!");

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         ah.debug("Validation failed in validateTransactionHistory method of IfsWpTxnHistory class");
         ah.debug(strBuf.toString());
      }

      return strBuf.toString();
   }

}
