//-*************************************
//
// FILE NAME:  IfsWpTxnSearch.java
//
// COPYRIGHT:
//    Licensed Materials - Property of IBM
//    5648-A06
//    (c) Copyright IBM Corporation, 1999, 2000. All rights reserved.
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract with IBM
//
//  CHANGES:
//    07/27/2000  rhs   d7065   WorkPad requires VALUE=xxxx to be
//                              VALUE="xxxx" .
//
//-*************************************


/* Change History:
 *
 * Author                  Date        Changes
 * ----------------------  ----------  ---------------------------------------------
 * Sonya Johnson          1/15/2002     replaced 'getValue' call with 'getAttribute'
 *                                      commented out import com.ibm.ifs.resource.*;
 *                        1/18/2002     added try-catch block
 *                        1/21/2002     added if statements for handleErrors                
 *
 */


import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import java.text.ParsePosition;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.*;

//Commented out 1/15/02
//import com.ibm.ifs.resource.*;

/**
* This class displays the posted transactions for a specific account based
* on selection criteria inputted by the consumer, such as the transaction amount.
*/
public class IfsWpTxnSearch extends IfsWpApplication
{
   // Controls the generation of debugging output for this class when
   // IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   public static boolean debugOn = false;

   // the number of displayed records in one page
   final static int TOTAL_PAGE_DISPAY = 4;

   // default value of maxmium received record number
   final static int MAX_REC_NUM =  10 * TOTAL_PAGE_DISPAY;

   // when total matched number larger than maxmium record number,
   // MAX_REC_NUM = MAX_REC_NUM + MAX_REC_NUM_STEP
   final static int MAX_REC_NUM_STEP = 10 * TOTAL_PAGE_DISPAY;


   //*******************************
   // Displays the input form for generating a Search transaction.
   //
   // No Http Request Parameters are expected.
   //*******************************
   public void doGet(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         showAllAccounts(up, ifsSession, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in doGet method of IfsWpTxnSearch Servlet, explanation: ");
      }
   }


   //*******************************
   // Either displays the confirmation page for a Search
   // transaction or executes it.
   //
   // The following Http Request Parameters are posted to this page:
   //     ReqTag   - Indicates whether to display confirmation or execute.
   //                It can have the values of "DispFirst" or "DispMore".
   //     transfer - Specifies the object identifier (oid) of a
   //                GtAccountHistory object.
   //     req????  - The input values from the Search Transaction form.
   //*******************************
   public void doPost(HttpServletRequest request, HttpServletResponse response)
               throws ServletException, IOException
   {
      // get session
      IfsSession ifsSession = new IfsSession(request, response);
      GtUserProfile up = null;
      GtAccountHistory transact = null;
      String strReq = null;

      try
      {
         // get user profile
         //replaced getValue with getAttribute 1/15/02
         up = (GtUserProfile) ifsSession.getHttpSession().getAttribute(USER_PROFILE);
         if (up == null)
         {
            // Since no user profile exists, there is no active session for this consumer.
            callPage_NoSession(response);
         }

         //---------------
         // Get a reference to the GtTransferHistoryInquiry object using the object identifier
         // passed via the transfer request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         transact = (GtAccountHistory) createInstance(ifsSession,
                             "com.ibm.ifs.gold.auto.GtAccountHistory",
                             "transact");

         // clearing cache data for the next transaction as entered by the user.
         transact.setReqLowCheckNumber(0);
         transact.setReqHighCheckNumber(0);

         try {
         GsCurrency nullCurrency = new GsCurrency();
         nullCurrency = GsCurrency.fromString(" ");
         transact.setReqLowAmount(nullCurrency);
         transact.setReqHighAmount(nullCurrency);
             }
         catch(Exception e)
            {
              debug(response, "GsCurrency object creation and initialization" + e.toString());
              throw e; 
             }
 

         try
         {
            // Set fields of GtAccountHistory object instance from the
            // corresponding request parameters values in the HTTP request.
            setFields(ifsSession,"transact",transact);


            // insert the first 2 digits of the year with 80 as the cut-off value.
            GsDate date = new GsDate();
            date = yearWindow(transact.getReqPostingStartDate());
            transact.setReqPostingStartDate(date);

            date = yearWindow(transact.getReqPostingEndDate());
            transact.setReqPostingEndDate(date);

         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         strReq = (String)getRequestParameter(request,"ReqTag");

         if (strReq.equals("DispFirst"))
            showFirstResultPage(up, ifsSession, transact, request, response);
         else
            showMoreResultPages(ifsSession, transact, request, response);
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
              "Exception caught in doPost method of IfsWpTxnSearch Servlet, explanation: ");
      }
   }


   /**
   * Display transaction history of checking account
   */
   private void handleCheckingDisplay(GsTxnRespDtl txnRespDtl, PrintWriter out)
                throws IOException
   {
      show2DivBlocks (out, "Date:", txnRespDtl.getPostingDate());

      // The common part of the accounts.
      handleCommonPartDisplay(txnRespDtl, out);
   }



   /**
   * Display the common part of the accounts.
   */
   private void handleCommonPartDisplay(GsTxnRespDtl txnRespDtl, PrintWriter out)
                throws IOException
   {
      String strValue = "";

      // Transaction Type
      try { strValue = txnRespDtl.getType_text(); }
      catch (Exception e) { strValue = ""; }
      show2DivBlocks (out, "Transaction Type:", strValue);

      // Transaction Source
      try { strValue = txnRespDtl.getSource_text(); }
      catch (Exception e) { strValue = ""; }
      show2DivBlocks (out, "Transaction Source:", strValue);

      show2DivBlocks (out, "Transaction Amount:", txnRespDtl.getTransactionAmount());

      // Check Number
      try { strValue = "" + txnRespDtl.getCheckID()[0].getNumber(); }
      catch (Exception e) { strValue = ""; }
      show2DivBlocks (out, "Check Number:", strValue);

      show2DivBlocks (out, "Description:",txnRespDtl.getDescription());

      out.println ("<HR>");

   }


   /**
   * Display transaction history of credit card account
   */
   private void handleCreditcardDisplay(GsTxnRespDtl txnRespDtl, PrintWriter out)
                throws IOException
   {
      show2DivBlocks (out, "Posting Date:", txnRespDtl.getPostingDate());
      show2DivBlocks (out, "Original Date:", txnRespDtl.getOriginalDate());
      show2DivBlocks (out, "Reference Number:", txnRespDtl.getReferenceNumber());

      // The common part of the accounts.
      handleCommonPartDisplay(txnRespDtl, out);
   }

   /**
   * Display transaction history of loan account
   */
   private void handleLoanDisplay(GsTxnRespDtl txnRespDtl, PrintWriter out)
                throws IOException
   {
      show2DivBlocks (out, "Date:", txnRespDtl.getPostingDate());
      show2DivBlocks (out, "Amount to Principal:", txnRespDtl.getToPrincipalAmount());
      show2DivBlocks (out, "Amount to Interest:", txnRespDtl.getToInterestAmount());
      show2DivBlocks (out, "Amount to Escrow:", txnRespDtl.getToEscrowAmount());

      // The common part of the accounts.
      handleCommonPartDisplay(txnRespDtl, out);
   }


   /**
   * Display a list of all accounts
   */
   private void showAllAccounts(GtUserProfile up, IfsSession ifsSession,
                                HttpServletResponse response)
                throws ServletException, IOException
   {
      GtAccountHistory ah      = null;
      GsAccount        account = null;

      String           transactionType     = "1";     //default
      String           transactionSource   = "0";      //default
      String           transactionPeriod   = "-";      //default

      String           reqLowAmount        = "";
      String           reqHighAmount       = "";
      String           reqPostingStartDate = "";
      String           reqPostingEndDate   = "";
      String           reqHighCheckNumber  = "";
      String           reqLowCheckNumber   = "";

      try
      {
         //---------------
         // Need to redisplay the originally entered information.
         // Get a reference to the GtAccountHistory cached object.
         //
         ah = (GtAccountHistory) getObject(ifsSession, TRANSACTION_SEARCH);
         if (ah != null)
         {
            //----------------
            // Extract the data from the cached object to redisplay the user input.
            //
            account = ah.getReqAccount();
            transactionType =ah.getReqTransactionType();
            transactionSource =ah.getReqTransactionSource();
            transactionPeriod =ah.getReqProductType();

            // low and high amounts
            reqLowAmount = getAmountString(ah.getReqLowAmount());
            reqHighAmount = getAmountString(ah.getReqHighAmount());

            // start and end dates
            reqPostingStartDate = getDateString(ah.getReqPostingStartDate());
            reqPostingEndDate = getDateString(ah.getReqPostingEndDate());

            // low check number
            if (ah.getReqLowCheckNumber() != 0 )
               reqLowCheckNumber = String.valueOf(ah.getReqLowCheckNumber());

            // high check number
            if (ah.getReqHighCheckNumber() != 0 )
               reqHighCheckNumber = String.valueOf(ah.getReqHighCheckNumber());
         }
      }
      catch ( Exception e) { }

      PrintWriter out = startHtmlOutput(response, "TRANSACTION SEARCH");

      // output form
      out.println("<FORM METHOD=POST ACTION=IfsWpTxnSearch >");
      out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispFirst\" >");
      out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + TRANSACTION_SEARCH + "\" >");

      // account
      out.println("<B>Please select an account:</B><BR>");
      out.println("<SELECT NAME=reqAccount>");
      displayAccounts(up, account, out);
      out.println("</SELECT><BR>");

      // Transaction type
      out.println("<B>Transaction type:</B><BR>");
      out.println("<SELECT NAME=reqTransactionType>");
      displayTransactionTypeList(transactionType, out);
      out.println("</SELECT><BR>");

      // Transaction source
      out.println("<B>Where transaction occurred:</B><BR>");
      out.println("<SELECT NAME=reqTransactionSource>");
      displayTransactionSourceList(transactionSource, out);
      out.println("</SELECT><BR>");

      // Product type
      out.println("<B>When transaction occurred:</B><BR>");
      out.println("<SELECT NAME=reqProductType VALUE=\"A\" >");
      displayTransactionPeriodList(transactionPeriod, out);
      out.println("</SELECT><BR>");

      // start date
      out.println("<B>OR</B><BR>");
      out.println("<B>Date or Range (" + strDateHint + "):</B><BR>");
      out.println("<TABLE BORDER=0>");
      out.println("<TR><TD>From (optional):</TD><TD>" +
                  "<INPUT TYPE=text NAME=reqPostingStartDate SIZE=10 " +
                  "MAXLENGTH=10 VALUE=\"" + reqPostingStartDate + "\"></TD></TR>");

      // end date
      out.println("<TR><TD>To (optional):</TD>" +
                  "<TD><INPUT TYPE=text NAME=reqPostingEndDate SIZE=10 " +
                  "MAXLENGTH=10 VALUE=\"" + reqPostingEndDate + "\"></TD></TR>");
      out.println("</TABLE>");

      // start amount
      out.println("<B>Amount or Range:</B><BR>");
      out.println("<TABLE BORDER=0>");
      out.println("<TR><TD>From (optional):</TD><TD>" + strCurrencyPrefix +
                  " <INPUT TYPE=text NAME=reqLowAmount SIZE=13 MAXLENGTH=20 " +
                  "VALUE=\"" + reqLowAmount + "\"></TD></TR>");

      // end amount
      out.println("<TR><TD>To (optional):</TD><TD>" + strCurrencyPrefix +
                  " <INPUT TYPE=text NAME=reqHighAmount SIZE=13 MAXLENGTH=20 " +
                  "VALUE=\"" + reqHighAmount + "\"></TD></TR>");
      out.println("</TABLE>");

      // start check number
      out.println("<B>Check number or Range:</B><BR>");
      out.println("<TABLE BORDER=0>");
      out.println("<TR><TD>From (optional):</TD>" +
                  "<TD><INPUT TYPE=text NAME=reqLowCheckNumber SIZE=9 " +
                  "MAXLENGTH=9 VALUE=\"" + reqLowCheckNumber + "\"></TD></TR>");
      // end check number
      out.println("<TR><TD>To (optional):</TD>" +
                  "<TD><INPUT TYPE=text NAME=reqHighCheckNumber SIZE=9 " +
                  "MAXLENGTH=9 VALUE=\"" + reqHighCheckNumber + "\"></TD></TR>");
      out.println("</TABLE>");

      // submit
      out.println("<P><DIV ALIGN=CENTER>");
      out.println("<INPUT TYPE=submit NAME=SUBMIT VALUE=\"Enter\" >");
      out.println("</DIV>");
      out.println("</FORM>");

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Display the first transaction history screen
   */
   private void showFirstResultPage(GtUserProfile up, IfsSession ifsSession,
                                    GtAccountHistory txn,
                                    HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GsTxnRespDtl[] txnRespDtl = null;

      int endPageNum = 0;
      int totalMatched = 0;
      int totalSent = 0;
      int leftNum = 0;

      String oid = null;
      String msg = null;
      String type = null;

      try
      {
         // set maximum number
         txn.setReqMaximumNumberOfRecords(MAX_REC_NUM);

         // Get the object ID of the cached GtAccountHistory instance object.
         oid = getOid(ifsSession,txn);

         // Validate the data that was entered.
         msg = validateTransactionSearch(txn, request);

         if ( msg.equals("OK") == false)
            callPage_RedoPage(response, "IfsWpTxnSearch", msg);

         // execute

         //added try-catch block 1/18/2002
         try
        {
        
         txn.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
         if (! handleErrors(txn, "IfsWpErrorPage",
                      "IfsWpRedoPage?nextpage=IfsWpTxnSearch",
                      "IfsWpRedoPage?nextpage=IfsWpTxnSearch")){
             return;
         }


         // get totalMatched and totalSent
         totalMatched = txn.getTotalTransactionsMatched();
         totalSent = txn.getTotalTransactionsSent();
         leftNum = totalSent;

         // get txnRespDtl
         txnRespDtl = txn.getTransaction();

         // set end page display number
         if (totalSent < TOTAL_PAGE_DISPAY)
            endPageNum = totalSent;
         else
            endPageNum = TOTAL_PAGE_DISPAY;

         type = txn.getReqAccount().getType();
      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to show transaction search information. Exception in showFirstResultsPage method of IfsWpTxnSearch Servlet, explanation: ");
      }

      //--------------
      // Start outputting transaction search information.
      //
      PrintWriter out = startHtmlOutput(response, "TRANSACTION SEARCH");

      // display each transaction
      out.println ("<HR>");
      for (int i = 0; i < endPageNum; i++ )
      {
         if ( (i >= txnRespDtl.length) ||
              (txnRespDtl[i] == null) )
            continue;

         // deposit accounts
         if (type.equals("CDA") || type.equals("DDA") || type.equals("IRA") ||
             type.equals("SDA") || type.equals("MMA"))
            handleCheckingDisplay (txnRespDtl[i], out);

         // credit accounts
         else if (type.equals("CCA") || type.equals("LOC") ||type.equals("ELC"))
            handleCreditcardDisplay (txnRespDtl[i], out);

         // loan accounts
         else if (type.equals("CLA") || type.equals("ILA") ||type.equals("MLA"))
            handleLoanDisplay (txnRespDtl[i], out);

         // invalid account type
         else
            out.println ("invalid account type<HR>");
      }

      // link to more transactions
      if (totalSent > 0)
         out.println ("<P><DIV ALIGN=CENTER>Transaction 1 - " +
                      endPageNum + " (Total: " + totalMatched + ")</DIV>");
      else
         out.println ("No transactions match your criteria.<BR>");

      if (endPageNum < totalMatched)
      {
         leftNum -= endPageNum;
         out.println("<DIV ALIGN=CENTER>");
         out.println("<FORM METHOD=POST ACTION=IfsWpTxnSearch >");
         out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispMore\" >");
         out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + oid + "\">");
         out.println("<INPUT TYPE=hidden NAME=ENDPAGENUM VALUE=\"" + endPageNum + "\">");
         out.println("<INPUT TYPE=hidden NAME=LEFTNUM VALUE=\"" + leftNum + "\">");
         out.println("<INPUT TYPE=submit NAME=BUTTON VALUE=\"More\" >");
         out.println("</FORM></DIV>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Display the continued transaction history screens
   */
   private void showMoreResultPages(IfsSession ifsSession,
                                    GtAccountHistory txn,
                                    HttpServletRequest request, HttpServletResponse response)
                throws ServletException, IOException
   {
      GsTxnRespDtl[] txnRespDtl = null;

      int startPageNum = 0;
      int endPageNum = 0;

      String type = "";
      String oid = "";

      int maxRecNum = 0;
      int totalMatched = 0;
      int totalSent = 0;
      int leftNum = 0;

      try
      {
      
         // get transaction response detail object ID.
         txnRespDtl = txn.getTransaction();

         maxRecNum = txn.getReqMaximumNumberOfRecords();
         totalMatched = txn.getTotalTransactionsMatched();
         totalSent = txn.getTotalTransactionsSent();
         type = txn.getReqAccount().getType();

         // Get the object ID of the cached GtAccountHistory instance object.
         oid = getOid(ifsSession,txn);

         // get startPageNum
         startPageNum = Integer.parseInt( (String)getRequestParameter(request,"ENDPAGENUM"));

         // get leftNum
         leftNum = Integer.parseInt( (String)getRequestParameter(request,"LEFTNUM"));

         // need execute transaction again
         if ((startPageNum + leftNum < totalMatched) &&  // not all matched has been sent
             (leftNum < TOTAL_PAGE_DISPAY))            // not enought to fill all page
         {
            // set maximum number
            maxRecNum += MAX_REC_NUM_STEP;
            txn.setReqMaximumNumberOfRecords(maxRecNum);

            // execute
            //added try-catch block 1/18/2002
         try
        {
        
         txn.execute();
        } catch (Exception e )  {
            callPage_ErrorPage(response, e,
                 "Exception in execute method: ");
        }
          if (! handleErrors(txn, "IfsWpErrorPage",
                         "IfsWpRedoPage?nextpage=IfsWpTxnSearch",
                         "IfsWpRedoPage?nextpage=IfsWpTxnSearch")){
              return;
          }


            // get totalSent
            totalSent = txn.getTotalTransactionsSent();
            leftNum = totalSent - startPageNum;

            txnRespDtl = txn.getTransaction();
         }

         // set end page display number
         if (leftNum < TOTAL_PAGE_DISPAY) endPageNum = leftNum + startPageNum;
         else endPageNum = TOTAL_PAGE_DISPAY + startPageNum;

      }
      catch (Exception e)
      {
         callPage_ErrorPage(response, e,
                "Unable to show transaction search information. Exception in showMoreResultPages method of IfsWpTxnSearch Servlet, explanation: ");
      }

      //--------------
      // Start outputting transaction search information.
      //
      PrintWriter out = startHtmlOutput(response, "TRANSACTION SEARCH");

      // display each transaction
      out.println ("<HR>");
      for (int i = startPageNum; i < endPageNum; i++)
      {
         if ( (i >= txnRespDtl.length) ||
              (txnRespDtl[i] == null) )
            continue;

         if (type.equals("CDA") || type.equals("DDA") || type.equals("IRA") ||
            type.equals("SDA") || type.equals("MMA"))
         {
            // deposit accounts
            handleCheckingDisplay (txnRespDtl[i], out);
         }
         else if (type.equals("CCA") || type.equals("LOC") ||type.equals("ELC"))
         {
            // credit accounts
            handleCreditcardDisplay (txnRespDtl[i], out);
         }
         else if (type.equals("CLA") || type.equals("ILA") ||type.equals("MLA"))
         {
            // loan accounts
            handleLoanDisplay (txnRespDtl[i], out);
         }
         else
            out.println ("invalid account type<HR>");
      }

      // link to more transactions
      out.println ("<P><DIV ALIGN=CENTER>Transaction " + (startPageNum + 1) + " - "
                   + endPageNum + " (Total: " + totalMatched + ")</DIV>");

      if (endPageNum < totalMatched)
      {
         leftNum = leftNum - (endPageNum - startPageNum);

         out.println("<DIV ALIGN=CENTER>");
         out.println("<FORM METHOD=POST ACTION=IfsWpTxnSearch >");
         out.println("<INPUT TYPE=hidden NAME=ReqTag VALUE=\"DispMore\" >");
         out.println("<INPUT TYPE=hidden NAME=transact VALUE=\"" + oid + "\">");
         out.println("<INPUT TYPE=hidden NAME=ENDPAGENUM VALUE=\"" + endPageNum + "\">");
         out.println("<INPUT TYPE=hidden NAME=LEFTNUM VALUE=\"" + leftNum + "\">");
         out.println("<INPUT TYPE=submit NAME=BUTTON VALUE=\"More\" >");
         out.println("</FORM></DIV>");
      }

      // link to other pages and close the print writer.
      endHtmlOutput(out);
   }

   /**
   * Validate the data that was entered.
   */
   private String validateTransactionSearch(GtAccountHistory ah, HttpServletRequest request)
   {
      StringBuffer strBuf = new StringBuffer("");
      String strTemp = "";
      String reqPostingStartDate = "";
      String reqPostingEndDate = "";

      double dLowAmount = 0;
      double dHighAmount = 0;

      // Get the low amount information.
      try { dLowAmount = getAndCheckAmount(ah.getReqLowAmount()); }
      catch (Exception e)
      {
         ah.debug("Exception getting From Amount in validateTransactionSearch - " + e.toString());
         dLowAmount = 0;
      }

      // Get the high amount information.
      try { dHighAmount = getAndCheckAmount(ah.getReqHighAmount()); }
      catch (Exception e)
      {
         ah.debug("Exception getting To Amount in validateTransactionSearch - " + e.toString());
         dHighAmount = 0;
      }

      // Get the start Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         reqPostingStartDate = ah.getReqPostingStartDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (reqPostingStartDate == null)
            reqPostingStartDate = "";
      }
      catch (Exception e)
      {
         ah.debug("Exception getting From Date in validateTransactionSearch- " + e.toString());
         reqPostingStartDate = "";
      }

      // Get the last transfer Date
      try
      {
         // Using toString pattern of yyyy/MM/dd so we can do date compares.
         reqPostingEndDate = ah.getReqPostingEndDate().toString(new GsDateFormat("yyyy/MM/dd"));
         if (reqPostingEndDate == null)
            reqPostingEndDate = "";
      }
      catch (Exception e)
      {
         ah.debug("Exception getting To Date in validateTransactionSearch - " + e.toString());
         reqPostingEndDate = "";
      }

      // Validate the data that was entered.
      strTemp = (String)getRequestParameter(request,"reqLowAmount");
      if ( (strTemp != null) && (strTemp.length() > 0) && (dLowAmount == 0) )
         strBuf.append("<BR>The From Amount value is invalid.");

      strTemp = (String)getRequestParameter(request,"reqHighAmount");
      if ( (strTemp != null) && (strTemp.length() > 0) && (dHighAmount == 0) )
         strBuf.append("<BR>The To Amount value is invalid.");

      if (dLowAmount > dHighAmount)
         strBuf.append("<BR>The From Amount range must be equal to or less than the To Amount range.");

      if (ah.getReqLowCheckNumber() > ah.getReqHighCheckNumber())
         strBuf.append("<BR>The From Check number must be equal to or less than the To Check number.");

      strTemp = (String)getRequestParameter(request,"reqPostingStartDate");
      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (reqPostingStartDate.equals("")) )
         strBuf.append("<BR>Invalid From Date!");

      strTemp = (String)getRequestParameter(request,"reqPostingEndDate");
      if ( (strTemp != null) && (strTemp.length() > 0) &&
           (reqPostingEndDate.equals("")) )
         strBuf.append("<BR>Invalid To Date!");

      if ( (reqPostingStartDate.equals("") == false) &&
           (reqPostingEndDate.equals("") == false) &&
           (reqPostingStartDate.compareTo(reqPostingEndDate) >= 0))
         strBuf.append("<BR>The To Date must be later than the From Date!");

      // Check if all product type and date range are be seleceted
      if ( (ah.getReqProductType().equals("-") == false) &&
           (reqPostingStartDate.equals("") == false) &&
           (reqPostingEndDate.equals("") == false) )
         strBuf.append("You must select either a transaction period or a date range, but not both.");

      // When no validation error messages are generated, return an "OK" string.
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
      {
         ah.debug("Validation failed in validateTransactionSearch method of IfsWpTxnSearch class");
         ah.debug(strBuf.toString());
      }

      return strBuf.toString();
   }

}
