package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   ClientFetch.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
// ***************************************************************************

//-----------------------------------------------------------------------
//
// ClientFetch:  Class provides a transaction which returns a character
//    string previously stored in the client's (user) session cache under
//    a character string name (name).
//
//----------------------------------------------------------------------

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class ClientFetch extends IfsTxn implements Serializable
{
public static boolean  debugOn = false;
// request message fields

   private String reqName = null;
// response message fields

   private String data = null;
// constructor

   public ClientFetch(){}         
// ===========================================================
// addGoldToGMJI method
// ===========================================================

   public void addGoldToGMJI(int argFlags, String cacheTxnName )
   {
	  addArgToGMJI( 10001 );
	  addArgToGMJI( reqName );
   }         
// ===========================================================
// executeTxn method
// ===========================================================

   public void execute(int argFlags, String cacheTxnName)
   {
	  // build the GMJI request byte buffer

	  initGMJI(getIfsSession());

	  // add the gold data to the GMJI

	  addGoldToGMJI(argFlags, cacheTxnName);

	  // call Gold Manager via JNI, the GMJI is input, a GMJO is returned.

	  debug("Calling gold manager from ClientFetch.executTxn.");
	  invokeGoldManager();

	  getGoldFromGMJO();

	  if (data.substring(0,8).equals("GMOB0000")) // if good header data from gold manager
	  {
		 data = data.substring(8);    // remove header data
		 status = OK;
	  }
	  else
	  {	
		 status = FATAL_ERROR;
		 errorCode = TXN_CWS;
		 debug("ClientFetch error="+data+".");
	  }

	  debug("ClientFetch exit, name="+reqName+", data="+data+".");

	  return;
   }                        
   public String getData() {return(data);}         
// ===========================================================
// getGoldFromGMJO method
// ===========================================================

   public void getGoldFromGMJO()
   {
	  data = getNextNullCharFromGMJO("data");
   }         
   public String getReqName() {return(reqName);}         
   public void setData(String s) {data=s;}         
   public void setReqName(String s) {reqName=s;}         
}

