package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   GsAccount.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    03/29/2000  ftc   dxxxx   Added NLS support
//    05/03/2000  rhs   d6662   Readded _text fields to toString or showObject.
//    05/25/2000  rhs   d6837   Added constructor which takes an IfsSession object.
//
// ***************************************************************************

public class GsAccount extends GsAccountS
{
   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean  debugOn = false;

   // getter/setter methods are contained in super class


   /**
   * Default constructor
   */
   public GsAccount() {}

   /**
   * Constructor with an associated IfsSession object.
   */
   public GsAccount(IfsSession sess)
   {
      setIfsSession(sess);
   }

   /**
   * GMJO/GMOB string constructor.
   * @param alias -- This parameter is not currently used,
   * it is there for future consideration.  Auto-gen requires the parameter.
   */
   public GsAccount(IfsTxn txn, String alias)
   {
     super(txn);
     setIfsSession(txn.getIfsSession());
   }

   /**
   * Explicit gold data constructor
   */
   public GsAccount(String type, String number, String nickname, int accountSubType, String accountSubType_text)
   {
     this.type = type;
     this.number = number;
     this.nickname = nickname;
     this.accountSubType = accountSubType;

   }

   /**
   * Explicit gold data constructor
   */
   public GsAccount(String type, String number, String nickname)
   {
       this(type,number,nickname,0,"");
   }

   /**
   * Converts the GsAccount object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix) {
        if(hasRegisteredMethod("toStringWithString"))
                return super.toString(prefix);

        StringBuffer results = new StringBuffer(200);
        results.append(prefix + "| type = " + getType());
        try { results.append(prefix + "| type_text = " + getType_text()); }
        catch (IfsException e)
        {
           results.append(prefix + "| type_text throws IfsException: " + e.getMessage());
        }

        results.append(prefix + "| number = " + getNumber());
        results.append(prefix + "| nickname = " + getNickname());
        results.append(prefix + "| accountSubType = " + Integer.toString(getAccountSubType()));
        try { results.append(prefix + "| accountSubType_text = " + getAccountSubType_text()); }
        catch (IfsException e)
        {
           results.append(prefix + "| accountSubType_text throws IfsException: " + e.getMessage());
        }

        return results.toString();
   }

   /**
   * Converts the GsAccount object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString() {
        if(hasRegisteredMethod("toString"))
                return super.toString();

        return toString(" ");
   }
}

