package com.ibm.ifs.gold;

//***************************************************************************
//
// FILE NAME:   GsAccountS.java
//
// COPYRIGHT:
//
//   Licensed Materials - Property of IBM
//   5648-A06
//   (C) Copyright IBM Corp. 1996, 2000 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//  CHANGES:
// 27-Jan-2000 larockc     d6259, d6260 - pack() throws an
//             exception when passed a null object.
// 04/11/2000  rhs   d6590   NLS changes.
// 05/03/2000  rhs   d6662   Readded _text fields to toString or showObject.
// 06/06/2000  rhs   d6865   Added use of IfsTxn.packNlsChar in the pack routine
//                           for nickname field.
// 11/28/2000  tlt   d7203   call getText with default locale if exception occurs
//***************************************************************************

import java.io.PrintWriter;
import java.util.Hashtable;
import javax.servlet.http.HttpServletResponse;

public class GsAccountS extends IfsObject
{
   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean  debugOn = false;
   public static final int SD_OWNER = 0;
   private static Hashtable registry = new Hashtable();

   protected String  type = null;
   protected String  number = null;
   protected String  nickname = null;
   protected String  clearinghouseFIID = null;
   protected int     accountSubType = 0;

   /**
   * Default constructor.
   */
   public GsAccountS() {}

   /**
   * GsAccountS constructor for an account in a transaction context.
   * @param txn -- the associated transaction
   */
   public GsAccountS(IfsTxn txn)
   {
      int i, repeatCounter;

      setIfsSession(txn.getIfsSession());
      type                 = txn.getNextCharFromGMJO( 3,"type" );
      number               = txn.getNextCharFromGMJO( 22,"number" );
      nickname             = txn.getNextNlsCharFromGMJO( 30,"nickname" );
      clearinghouseFIID    = txn.getNextCharFromGMJO( 28,"clearinghouseFIID" );
      accountSubType       = txn.getNextLongFromGMJO( "accountSubType" );

   }

   /**
   * addPropertiesToGMJI takes the values in the gsAccountInstance parameter
   * and asks the txn to add the values to the Gold Message.
   * @param gsAccountInstance -- the account instance
   * @param txn -- the associated transaction
   */
   public static final void addPropertiesToGMJI( GsAccountS gsAccountInstance, IfsTxn txn )
   {
      if (gsAccountInstance != null)
      {
         txn.addArgToGMJI( gsAccountInstance.type);
         txn.addArgToGMJI( gsAccountInstance.number);
         txn.addArgToGMJI( gsAccountInstance.nickname);
         txn.addArgToGMJI( gsAccountInstance.clearinghouseFIID);
         txn.addArgToGMJI( gsAccountInstance.accountSubType);
      }
      else
      {
         txn.addArgToGMJI();
         txn.addArgToGMJI();
         txn.addArgToGMJI();
         txn.addArgToGMJI();
         txn.addArgToGMJI();
      }
   }
   public  String  getClearinghouseFIID() {return(clearinghouseFIID);}
   public  int  getAccountSubType() {return(accountSubType);}
   public  String  getNickname() {return(nickname);}
   public  String  getNumber() {return(number);}
   public static Hashtable getRegistry() {return registry;}
   public int getSD_OWNER() { return( SD_OWNER );}
   public  String  getType() {return(type);}
   public  String  getType_text() throws IfsException
   {
      /* d7203 - added try/catch clause */
      try {
         return IfsNLS.getText("IfsAcctTpClResources",getIfsSession().getLocale(),getType());
      }
      catch ( Exception e ) {
         return IfsNLS.getText("IfsAcctTpClResources",IfsSystem.SERVER_LOCALE,getType());
      }
   }
   public  String  getAccountSubType_text() throws IfsException
   {
      /* d7203 - added try/catch clause */
      try {
         return IfsNLS.getText("IfsAcctSubTpClResources",getIfsSession().getLocale(),String.valueOf(getAccountSubType()));
      }
      catch ( Exception e ) {
         return IfsNLS.getText("IfsAcctSubTpClResources",IfsSystem.SERVER_LOCALE,String.valueOf(getAccountSubType()));
      }
   }

   public static String pack(GsAccountS gsAccountInstance)
   {
      String   str = "";


      if (gsAccountInstance == null)
      {
         //      123
         str += "   ";                     // type
         //      1234567890123456789012
         str += "                      ";  // number

         // NLS variable length format (kept it to 30 bytes overall).
         //          12345678901234567890123456
         str += "001A                          "; // nickname

         //      1234567890123456789012345678
         str += "                            ";   // clearinghouseFIID
         //      123456789012
         str += "000000000000";                   // accountSubType
      }
      else
      {
         str += IfsApi.left(gsAccountInstance.type,3);
         str += IfsApi.left(gsAccountInstance.number,22);
         str += IfsTxn.packNlsChar(gsAccountInstance.nickname,30);
         str += IfsApi.left(gsAccountInstance.clearinghouseFIID,28);
         str += IfsApi.right(gsAccountInstance.accountSubType,12,"0");
      }

      return str;
   }

   public  void    setClearinghouseFIID(String o) {clearinghouseFIID = o;}
   public  void    setAccountSubType(int o) {accountSubType = o;}
   public  void    setNickname(String o) {nickname = o;}
   public  void    setNumber(String o) {number = o;}
   public  void    setType(String o) {type = o;}

   /**
   * showObject writes out the values of this GsAccountS
   * object with the supplied PrintWriter and prefix String
   * @param res -- unused
   * @param pageWriter -- the PrintWriter to use
   * @param prefix -- the String prefix to put in front of each value
   */
   public void showObject( HttpServletResponse res, PrintWriter pageWriter, String prefix )
   {
      final String newPrefix = prefix + "|  ";
      pageWriter.println( prefix + "type = " + getType() );
      try { pageWriter.println(prefix + "type_text = " + getType_text()); }
      catch (IfsException e)
      {
         pageWriter.println(prefix + "type_text throws IfsException: " + e.getMessage());
      }
      pageWriter.println( prefix + "number = " + getNumber() );
      pageWriter.println( prefix + "nickname = " + getNickname() );
      pageWriter.println( prefix + "clearinghouseFIID = " + getClearinghouseFIID() );
      pageWriter.println( prefix + "accountSubType = " + Integer.toString(getAccountSubType()) );
      try { pageWriter.println(prefix + "accountSubType_text = " + getAccountSubType_text()); }
      catch (IfsException e)
      {
         pageWriter.println(prefix + "accountSubType_text throws IfsException: " + e.getMessage());
      }
   }
}   // End of Class

