package com.ibm.ifs.gold;

//***************************************************************************
//
// FILE NAME:   GsCurrencyS.java
//
// COPYRIGHT:
//
//   Licensed Materials - Property of IBM
//   5648-A06
//   (C) Copyright IBM Corp. 1996, 2000 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//  CHANGES:
//    03/29/2000  ftc   dxxxx   Added NLS support
//    05/03/2000  rhs   d6662   Readded _text fields to toString or showObject.
//
//***************************************************************************

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class GsCurrencyS extends IfsObject
{
   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean  debugOn = false;
   public static final int SD_OWNER = 0;
   private static Hashtable registry = new Hashtable();

   protected String  digits = null;
   protected int     precision = 0;
   protected int     debitCredit = 0;
   protected String  isoCode = null;

   /**
   * Default constructor.
   */
   public GsCurrencyS() {}

   /**
   * GsCurrencyS constructor for a currency in a transaction context.
   * @param txn -- the associated transaction
   */
   public GsCurrencyS(IfsTxn txn)
   {
     int i, repeatCounter;

     setIfsSession(txn.getIfsSession());

     digits                             = txn.getNextCharFromGMJO( 15,"digits" );
     precision                          = txn.getNextLongFromGMJO( "precision" );
     debitCredit                        = txn.getNextLongFromGMJO( "debitCredit" );
     isoCode                            = txn.getNextCharFromGMJO( 3,"isoCode" );
   }

   /**
   * addPropertiesToGMJI takes the values in the gsCurrencyInstance parameter
   * and asks the txn to add the values to the Gold Message.
   * @param gsCurrencyInstance -- the currency instance
   * @param txn -- the associated transaction
   */
   public static final void addPropertiesToGMJI( GsCurrencyS gsCurrencyInstance, IfsTxn txn )
   {
     if (gsCurrencyInstance != null)
     {
       txn.addArgToGMJI(IfsApi.chop(gsCurrencyInstance.digits, '0'));
       txn.addArgToGMJI(gsCurrencyInstance.precision);
       txn.addArgToGMJI(gsCurrencyInstance.debitCredit);
       txn.addArgToGMJI(gsCurrencyInstance.isoCode);
     }
     else
     {
       txn.addArgToGMJI();
       txn.addArgToGMJI();
       txn.addArgToGMJI();
       txn.addArgToGMJI();
     }
   }
   public  int     getDebitCredit() {return(debitCredit);}
   public String getDebitCredit_text() throws IfsException
   {
     return IfsNLS.getText("IfsDrCrIndResources",
                      getLocale(),
                     String.valueOf(getDebitCredit()));
   }
   public  String  getDigits() {if(null==digits) digits="0"; return(digits);}
   public  String  getIsoCode() {return(isoCode);}
   public  int     getPrecision() {return(precision);}
   public static Hashtable getRegistry() {return registry;}
   public int getSD_OWNER() { return( SD_OWNER ); }
   public static String pack(GsCurrencyS gsCurrencyInstance)
   {
     String str;
     if(null!=gsCurrencyInstance)

      str = IfsApi.right(gsCurrencyInstance.digits,15,"0") +
         IfsApi.right(gsCurrencyInstance.precision,12,"0") +
         IfsApi.right(gsCurrencyInstance.debitCredit,12,"0") +
         IfsApi.right(gsCurrencyInstance.isoCode,3,"0") +
         "";

     else
      str = IfsApi.right("0",15) +
         IfsApi.right(0,12,"0") +
         IfsApi.right(0,12,"0") +
         IfsApi.right("0",3) +
         "";

     return(str);
   }
   public  void    setDebitCredit(int o) {debitCredit = o;}
   public  void    setDigits(String o) {digits = o;}
   public  void    setIsoCode(String o) {isoCode = o;}
   public  void    setPrecision(int o) {precision = o;}

   /**
   * showObject writes out the values of this GsCurrencyS
   * object with the supplied PrintWriter and prefix String
   * @param res -- unused
   * @param pageWriter -- the PrintWriter to use
   * @param prefix -- the String prefix to put in front of each value
   */
   public void showObject( HttpServletResponse res, PrintWriter pageWriter, String prefix )
   {
      final String newPrefix = prefix + "|  ";
     pageWriter.println( prefix + "digits = " + getDigits() );
     pageWriter.println( prefix + "precision = " + getPrecision() );
     pageWriter.println( prefix + "debitCredit = " + getDebitCredit() );
     try { pageWriter.println( prefix + "debitCredit_text = " + getDebitCredit_text() ); }
     catch (IfsException e)
     {
        pageWriter.println(prefix + "debitCredit_text throws IfsException: " + e.getMessage());
     }
     pageWriter.println( prefix + "isoCode = " + getIsoCode() );
   }
}   // End of Class
