package com.ibm.ifs.gold;

//***************************************************************************
//
// FILE NAME:   GsDecimalS.java
//
// COPYRIGHT:
//
//   Licensed Materials - Property of IBM
//   5648-A06
//   (C) Copyright IBM Corp. 1996, 2000 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//  CHANGES:
//    03/29/2000  ftc   dxxxx   Added NLS support
//    05/03/2000  rhs   d6662   Readded _text fields to toString or showObject.
//
//***************************************************************************


import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
* GsDecimalS class extends IfsObject and is the super class of GsDecimal.
* It takes care of the JNI interface for the Gold message.  It keeps the
* decimal value in a form compatible with that of the GOLD_DCML_DATA type.
*/
public class GsDecimalS extends IfsObject
{
   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean  debugOn = false;
   public static final int SD_OWNER = 0;
   private static Hashtable registry = new Hashtable();

   protected String  digits = null;
   protected int     precision = 0;
   protected int     debitCredit = 0;

   /**
   * GsDecimalS default constructor
   */
   public GsDecimalS() {}
   /**
   * GsDecimalS constructor for a decimal in a transaction context.
   * @param txn -- the associated transaction
   */
   public GsDecimalS(IfsTxn txn)
   {
     int i, repeatCounter;

     setIfsSession(txn.getIfsSession());

     digits                             = txn.getNextCharFromGMJO( 15,"digits" );
     precision                          = txn.getNextLongFromGMJO( "precision" );
     debitCredit                        = txn.getNextLongFromGMJO( "debitCredit" );
   }
   /**
   * addPropertiesToGMJI takes the values in the gsDecimalDataInstance parameter
   * and asks the txn to add the values to the Gold Message.
   * @param gsDecimalDataInstance -- the Decimal value
   * @param txn -- the associated transaction
   */
   public static final void addPropertiesToGMJI( GsDecimalS gsDecimalDataInstance, IfsTxn txn )
   {
     if (gsDecimalDataInstance != null)
     {
       txn.addArgToGMJI(IfsApi.chop(gsDecimalDataInstance.digits, '0'));
       txn.addArgToGMJI(gsDecimalDataInstance.precision);
       txn.addArgToGMJI(gsDecimalDataInstance.debitCredit);
     }
     else
     {
       txn.addArgToGMJI();
       txn.addArgToGMJI();
       txn.addArgToGMJI();
     }
   }
   /**
   * getDebitCredit returns an integer DebitCredit Indicator. The valid
   * values are: Debit  = NEGATIVE = 0
   *             Credit = POSITIVE = 1
   */
   public  int     getDebitCredit() {return(debitCredit);}
   /**
   * getDebitCredit_text returns a locale dependent String that indicates
   * the debit/credit status
   */
   public String getDebitCredit_text() throws IfsException
   {
     return IfsNLS.getText("IfsDrCrIndResources",
                      getLocale(),
                     String.valueOf(getDebitCredit()));
   }
   /**
   * getDigits returns a String representing the digits of the decimal
   * without any punctuation
   */
   public  String  getDigits() {return(digits);}
   /**
   * getPrecision returns an int representing the number of digits to
   * the right of the decimal point
   */
   public  int     getPrecision() {return(precision);}
   /**
   * getRegistry returns a Hastable registry used for custom method
   * registration
   */
   public static Hashtable getRegistry() {return registry;}
   /**
   * getSD_OWNER returns an integer identifier for the software
   * development owner.
   */
   public int getSD_OWNER() { return( SD_OWNER ); }
   /**
   * pack returns a String of the decimal values right justified
   * and padded on the left with "0" to fill the fields.
   */
   public static String pack(GsDecimalS gsDecimalDataInstance)
   {
     String str;
        if(null!=gsDecimalDataInstance)
      str = IfsApi.right(gsDecimalDataInstance.digits,15,"0") +
         IfsApi.right(gsDecimalDataInstance.precision,12,"0") +
         IfsApi.right(gsDecimalDataInstance.debitCredit,12,"0") +
         "";

     else
      str = IfsApi.right("0",15) +
         IfsApi.right(0,12,"0") +
         IfsApi.right(0,12,"0") +
         "";

     return(str);
   }
   /**
   * setDebitCredit sets the debitCredit field.
   * values are: Debit  = NEGATIVE = 0
   *             Credit = POSITIVE = 1
   * @param o -- the integer value to set DebitCredit with
   */
   public  void    setDebitCredit(int o) {debitCredit = o;}
   /**
   * setDigits sets the digits field.
   * @param o -- the String value to set digits with
   */
   public  void    setDigits(String o) {digits = o;}
   /**
   * setPrecision sets the precision field.
   * @param o -- the integer value to set precision with
   */
   public  void    setPrecision(int o) {precision = o;}
   /**
   * showObject writes out the values of the decimal with the supplied
   * PrintWriter and prefix String
   * @param res -- unused
   * @param pageWriter -- the PrintWriter to use
   * @param prefix -- the String prefix to put in front of each value
   */
   public void showObject( HttpServletResponse res, PrintWriter pageWriter, String prefix )
   {
      final String newPrefix = prefix + "|  ";
     pageWriter.println( prefix + "digits = " + getDigits() );
     pageWriter.println( prefix + "precision = " + getPrecision() );
     pageWriter.println( prefix + "debitCredit = " + getDebitCredit() );
     try { pageWriter.println( prefix + "debitCredit_text = " + getDebitCredit_text() ); }
     catch (IfsException e)
     {
        pageWriter.println(prefix + "debitCredit_text throws IfsException: " + e.getMessage());
     }
   }
}   // End of Class
