package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   GtChangePinPassword.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    02/16/2000  rhs   d6361   Made JavaDoc correction/addtions.
//    03/15/2000  rhs   d6545   Added NLS support
//    05/03/2000  rhs   d6662   Readded _text fields to toString or showObject.
//    06/27/2001  tlt  T0987315 Corrected pin/pswd/userid len checks
//
// ***************************************************************************

import java.io.UnsupportedEncodingException;
import java.util.Hashtable;
import java.util.ResourceBundle;
import com.ibm.ifs.gold.IfsNLS;

public class GtChangePinPassword extends IfsTxn
{
   private static String GMRequestId             = "pinpswd_java";

   public static final int SD_OWNER = 0;
   public static final int SDThis = 0;
   public static final String NO_ENCRYPTION = "0";
   public static final String RSA_ENCRYPTION = "1";

   // request fields
   private String  userId                 = null;
   private String  newUserId              = null;

   private byte[]  oldPin                    = null;
   private String  oldPinEncryptionType      = IfsSystem.RSA_ENCRYPTION;
   private boolean encryptOldPin             = false;

   private byte[]  newPin                    = null;
   private String  newPinEncryptionType      = IfsSystem.RSA_ENCRYPTION;
   private boolean encryptNewPin             = false;

   private byte[]  oldPassword               = null;
   private String  oldPasswordEncryptionType = IfsSystem.RSA_ENCRYPTION;
   private boolean encryptOldPassword        = false;

   private byte[]  newPassword               = null;
   private String  newPasswordEncryptionType = IfsSystem.RSA_ENCRYPTION;
   private boolean encryptNewPassword        = false;

   // response fields
   private String  responseCode           = null;
   private String  fiReferenceNumber      = null;
   private String  spReferenceNumber      = null;
   private String  fiProcessIndicator     = null;

   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean  debugOn = false;

   // Get the registry defined this class to register customized
   // methods applicable to all IfsObjects.
   private static Hashtable registry = new Hashtable();

   /**
   * Default constructor
   */
   public GtChangePinPassword(){}

   /**
   * Constructor with an associated IfsSession object.
   */
   public GtChangePinPassword(IfsSession sess)
   {
      setIfsSession(sess);
   }

   /**
   * Add ChangePinPassword gold message to Gold Message Java Input buffer.
   */
   public void addGoldToGMJI(int argFlags, String cacheTxnName )
   {
      addArgToGMJI( getOldPassword(IfsTxn.ENCRYPT));
      addArgToGMJI( getOldPin(IfsTxn.ENCRYPT));
      addArgToGMJI( oldPasswordEncryptionType );
      addArgToGMJI( oldPinEncryptionType );
      addArgToGMJI( getNewPassword(IfsTxn.ENCRYPT));
      addArgToGMJI( getNewPin(IfsTxn.ENCRYPT));
      addArgToGMJI( newPasswordEncryptionType );
      addArgToGMJI( newPinEncryptionType );
      addArgToGMJI( newUserId );

      reqSelfDef_packed = GsGmSelfDefiningFieldDef.pack( reqSelfDef);
      addArgToGMJI( reqSelfDef_packed );

   }

   /**
   * Executes the GtChangePinPassword transaction by sending it to the
   * Financial Instition (FI) and the response is not saved in the Gold Manager
   * transaction cache.
   * The transaction execution mode is controlled by the IfsTxn object executionMode
   * attribute.
   *
   * @exception IfsException When execution fails
   */
   public void execute() throws IfsException {

      setContainsRespCode(IfsTxn.RETURNS_RESPONSE_CODE);
      changePinPasswordTxn = true;
      super.execute();

   }
   public  String  getFiProcessIndicator() {return fiProcessIndicator;}
   public  String  getFiProcessIndicator_text() throws IfsException
   {
      return IfsNLS.getText("IfsFiProcIndResources",
                            getIfsSession().getLocale(),
                            getFiProcessIndicator());
   }
   public  String  getFiReferenceNumber() {return fiReferenceNumber;}
   public  String getGMRequestId()                 {return(GMRequestId);}

   /**
   * Get ChangePinPassword gold message from Gold Message Java Output buffer.
   */
   public void getGoldFromGMJO()
   {
      int i, repeatCounter;

      responseCode                       = getNextCharFromGMJO( 3,"responseCode" );
      fiReferenceNumber                  = getNextCharFromGMJO( 12,"fiReferenceNumber" );
      spReferenceNumber                  = getNextCharFromGMJO( 36,"spReferenceNumbe" );
      fiProcessIndicator                 = getNextCharFromGMJO( 1,"fiProcessIndicator" );

      addSelfDefProperties();
   }
   public byte[] getNewPassword(){return newPassword;}
   private byte[]  getNewPassword(boolean format) {
      if (true==format)
      {
         if (encryptNewPassword && (IfsSystem.RSA_ENCRYPTION.equals(newPasswordEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(newPassword, IfsSystem.RSA_ENCRYPTION);
            if (!(security.isValid()))
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;
            }
            return security.getEncodedValue();
         }
      }
      return newPassword;
   }
   public String getNewPasswordEncryptionType(){return newPasswordEncryptionType;}
   public byte[] getNewPin() {return newPin;}
   private byte[]  getNewPin(boolean format) {
      if (true==format)
      {
         if (encryptNewPin && (IfsSystem.RSA_ENCRYPTION.equals(newPinEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(newPin, IfsSystem.RSA_ENCRYPTION);
            if (!(security.isValid()))
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;
            }
            return security.getEncodedValue();
         }
      }
      return newPin;
   }
   public String getNewPinEncryptionType(){return newPinEncryptionType;}
   public String getNewUserId() {return newUserId;}
   public byte[] getOldPassword(){return oldPassword;}
   private byte[]  getOldPassword(boolean format) {
      if (true==format)
      {
         if (encryptOldPassword && (IfsSystem.RSA_ENCRYPTION.equals(oldPasswordEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(oldPassword, IfsSystem.RSA_ENCRYPTION);
            if (!(security.isValid()))
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;
            }
            return security.getEncodedValue();
         }
      }
      return oldPassword;
   }
   public String getOldPasswordEncryptionType(){return oldPasswordEncryptionType;}
   public byte[] getOldPin() {return oldPin;}
   private byte[]  getOldPin(boolean format) {
      if (true==format)
      {
         if (encryptOldPin && (IfsSystem.RSA_ENCRYPTION.equals(oldPinEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(oldPin, IfsSystem.RSA_ENCRYPTION);
            if (!(security.isValid()))
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;
            }
            return security.getEncodedValue();
         }
      }
      return oldPin;
   }
   public String getOldPinEncryptionType(){return oldPinEncryptionType;}

   /**
   * Returns a Hashtable object which contains the list of registered methods.
   */
   public static Hashtable getRegistry()
   {
      return registry;
   }
   public String getResponseCode() {return responseCode;}
   public String getResponseCode_text() throws IfsException
   {
      return IfsNLS.getText("IfsRespCodeClResources",
                            getIfsSession().getLocale(),
                            getResponseCode());
   }

   public int getSD_OWNER() { return( SD_OWNER );}
   public  String  getSpReferenceNumber() {return spReferenceNumber;}
   public String getUserId() {return userId;}
   protected byte[] invokeGmJavaReq_UnUsed(byte[] gmji)
   {
      return gmJavaChangePinPasswordReq(gmji);
   }
   public void setNewPassword(byte[] b)
   {
      newPassword = b;
      encryptNewPassword = false;
      return;
   }
   public void setNewPassword(String s)
   {
      try
      {
         newPassword = s.getBytes(IfsSystem.cwsGMJCharSet);
         encryptNewPassword = true;
      }
      catch (UnsupportedEncodingException e)
      {
         newPassword = null;
         encryptNewPassword = false;
         debug("setNewPassword failure in GtPinPassword, exception is -- " + e.toString() );
      }
   }
   public void setNewPasswordEncryptionType(String s){newPasswordEncryptionType = s;}
   public void setNewPin(byte[] b)
   {
      newPin = b;
      encryptNewPin = false;
      return;
   }
   public void setNewPin(String s)
   {
      try
      {
         newPin = s.getBytes(IfsSystem.cwsGMJCharSet);
         encryptNewPin = true;
      }
      catch (UnsupportedEncodingException e)
      {
         newPin = null;
         encryptNewPin = false;
         debug("setNewPin failure in GtPinPassword, exception is -- " + e.toString() );
      }
   }
   public void setNewPinEncryptionType(String s){newPinEncryptionType = s;}
   public void setNewUserId(String s) {newUserId = s;}
   public void setOldPassword(byte[] b)
   {
      oldPassword = b;
      encryptOldPassword = false;
      return;
   }
   public void setOldPassword(String s)
   {
      try
      {
         oldPassword = s.getBytes(IfsSystem.cwsGMJCharSet);
         encryptOldPassword = true;
      }
      catch (UnsupportedEncodingException e)
      {
         oldPassword = null;
         encryptOldPassword = false;
         debug("setOldPassword failure in GtPinPassword, exception is -- " + e.toString() );
      }
   }
   public void setOldPasswordEncryptionType(String s){oldPasswordEncryptionType = s;}
   public void setOldPin(byte[] b)
   {
      oldPin = b;
      encryptOldPin = false;
      return;
   }
   public void setOldPin(String s)
   {
      try
      {
         oldPin = s.getBytes(IfsSystem.cwsGMJCharSet);
         encryptOldPin = true;
      }
      catch (UnsupportedEncodingException e)
      {
         oldPin = null;
         encryptOldPin = false;
         debug("setOldPin failure in GtPinPassword, exception is -- " + e.toString() );
      }
   }
   public void setOldPinEncryptionType(String s){oldPinEncryptionType = s;}
   public void setResponseCode(String code) {responseCode=code;}
   public void setUserId(String s) {userId = s;}

   /**
   * Converts the GtChangePinPassword object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      if (hasRegisteredMethod("toString"))
      {
         return super.toString();
      }
      else
         return toString("\n");
   }

   /**
   * Converts the GtChangePinPassword object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix)
   {
      if (hasRegisteredMethod("toStringWithString"))
      {
         return super.toString(prefix);
      }
      else
      {
         StringBuffer buffer = new StringBuffer(256);
         buffer.append(super.toString(prefix));
         buffer.append(prefix);
         buffer.append("userId = ");
         buffer.append(userId);
         buffer.append(prefix);
         buffer.append("newUserId = ");
         buffer.append(newUserId);
         buffer.append(prefix);
         buffer.append("oldPin = ");
         buffer.append(new String(oldPin));
         buffer.append(prefix);
         buffer.append("oldPinEncryptionType = ");
         buffer.append(oldPinEncryptionType);
         buffer.append(prefix);
         buffer.append("newPin = ");
         buffer.append(new String(newPin));
         buffer.append(prefix);
         buffer.append("newPinEncryptionType = ");
         buffer.append(newPinEncryptionType);
         buffer.append(prefix);
         buffer.append("oldPassword = ");
         buffer.append(new String(oldPassword));
         buffer.append(prefix);
         buffer.append("oldPasswordEncryptionType = ");
         buffer.append(oldPasswordEncryptionType);
         buffer.append(prefix);
         buffer.append("newPassword = ");
         buffer.append(new String(newPassword));
         buffer.append(prefix);
         buffer.append("newPasswordEncryptionType = ");
         buffer.append(newPasswordEncryptionType);
         buffer.append(prefix);
         buffer.append("responseCode = ");
         buffer.append(responseCode);
         try { buffer.append( prefix + "responseCode_text = " + getResponseCode_text() ); }
         catch (IfsException e)
         {
            buffer.append(prefix + "responseCode_text throws IfsException: " + e.getMessage());
         }
         buffer.append(prefix);
         buffer.append("fiReferenceNumber = ");
         buffer.append(fiReferenceNumber);
         buffer.append(prefix);
         buffer.append("fiProcessIndicator = ");
         buffer.append(fiProcessIndicator);
         try { buffer.append( prefix + "fiProcessIndicator_text = " + getFiProcessIndicator_text() ); }
         catch (IfsException e)
         {
            buffer.append(prefix + "fiProcessIndicator_text throws IfsException: " + e.getMessage());
         }
         buffer.append(prefix);
         buffer.append("spReferenceNumber = ");
         buffer.append(spReferenceNumber);
         buffer.append("\n");
         return buffer.toString();
      }
   }

   /**
   * Validates the GtChangePinPassword object.
   * When a method is registered to <EM>validate</EM> for this class,
   * it is used.  Otherwise, a default validation is done.
   *
   * @return
   *    Returns "OK" if the GtChangePinPassword is valid.  Otherwise, it returns a message
   *    string indicating the error.
   */
   public String validate()
   {
      ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",
                                                       getLocale());
      String str = super.validate();
      if (str == null)
      {
         boolean fUserId=true, fPin=true, fPassword=true;

         if (userId != null || newUserId != null)
         {
            if (userId == null || newUserId == null)
               return bundle.getString("GtChangePinPassword.validation.invalidUserID");
            if (userId.length() > IfsSystem.MAX_USER_ID_LEN ||
                newUserId.length() > IfsSystem.MAX_USER_ID_LEN)
               return bundle.getString("GtChangePinPassword.validation.invalidUserIDLength");
         }
         else
            fUserId = false;

         boolean fOldPin = false;
         boolean fNewPin = false;
         int     oldPinLen = 0;
         int     newPinLen = 0;

         if (oldPin != null)
         {
            fOldPin = true;
            oldPinLen = oldPin.length;
         }

         if (newPin != null)
         {
            fNewPin = true;
            newPinLen = newPin.length;
         }

         if (fOldPin || fNewPin)
         {
            if (!fOldPin || !fNewPin)
               return bundle.getString("GtChangePinPassword.validation.invalidPin");
            if (oldPinLen > IfsSystem.MAX_PIN_LENGTH ||
                newPinLen > IfsSystem.MAX_PIN_LENGTH)
               return bundle.getString("GtChangePinPassword.validation.invalidPinLength");
         }
         else
            fPin = false;

         boolean fOldPassword = false;
         boolean fNewPassword = false;
         int     oldPasswordLen = 0;
         int     newPasswordLen = 0;

         if (oldPassword != null)
         {
            fOldPassword = true;
            oldPasswordLen = oldPassword.length;
         }

         if (newPassword != null)
         {
            fNewPassword = true;
            newPasswordLen = newPassword.length;
         }

         if (fOldPassword || fNewPassword)
         {
            if (!fOldPassword || !fNewPassword)
               return bundle.getString("GtChangePinPassword.validation.invalidPassword");
            if (oldPasswordLen > IfsSystem.MAX_PASSWORD_LENGTH ||
                newPasswordLen > IfsSystem.MAX_PASSWORD_LENGTH)
               return bundle.getString("GtChangePinPassword.validation.invalidPasswordLength");
         }
         else
            fPassword = false;

         if (fUserId || fPin || fPassword)
            str = "OK";
         else
            str = bundle.getString("GtChangePinPassword.validation.allFieldsUndefined");
      }
      return str;
   }
}

