//***************************************************************************
//
// FILE NAME:   GtGetUndeliveredMessage.java
//
// COPYRIGHT:
//
//   Licensed Materials - Property of IBM
//   5648-A06
//   (C) Copyright IBM Corp. 1999, 2000 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//  CHANGES:
//    04/28/2000  rhs   d6679   Corrected execute method, added use of
//                              IfsTxn.UNDELIVERED.FLAG, added toString method,
//                              and added JavaDoc comments.
//
//***************************************************************************
//
//
package com.ibm.ifs.gold;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.ifs.gold.*;

public class GtGetUndeliveredMessage extends IfsTxn
{
   public static boolean debugOn=false;

   // constructor
   public GtGetUndeliveredMessage(){}

   public GtGetUndeliveredMessage(IfsSession sess) {
      setIfsSession(sess);
   }

   private static final String BuildLevelSignature = null;
   public  String getBuildLevelSignature()         {return(BuildLevelSignature);}

   private static final String GMRequestId             = "genundlv_java";
   public  String getGMRequestId()                 {return(GMRequestId);}

   private static final String MTILevel                = "00.01.03";
   public  String getMTILevel()                    {return(MTILevel);}

   public static final int SD_OWNER = 0;
   public int getSD_OWNER() { return( SD_OWNER );}
   private static Hashtable registry = new Hashtable();
   public static Hashtable getRegistry() {return registry;}

   public static final int SDThis = 0;
   // request message fields/setters/getters

   // End of Header -- Start of Declarations

   private String responseCode = null;
   public void setResponseCode(String o) {responseCode = o;}
   public String getResponseCode() {return(responseCode);}
   public String getResponseCode_text() throws IfsException {
      return IfsNLS.getText("IfsRespCodeClResources",
                            getIfsSession().getLocale(),
                            getResponseCode());
   }

   private String referenceNumber = null;
   public void setReferenceNumber(String o) {referenceNumber = o;}
   public String getReferenceNumber() {return(referenceNumber);}

   private String serviceProviderReference = null;
   public void setServiceProviderReference(String o) {serviceProviderReference = o;}
   public String getServiceProviderReference() {return(serviceProviderReference);}

   private String fIProcInd = null;
   public void setFIProcInd(String o) {fIProcInd = o;}
   public String getFIProcInd() {return(fIProcInd);}
   public String getFIProcInd_text() throws IfsException {
      return IfsNLS.getText("IfsFiProcIndResources",
                            getIfsSession().getLocale(),
                            getFIProcInd());
   }

   /**
   * Adds GtGetUndeliveredMessage gold message to Gold Message Java Input buffer.
   */
   public void addGoldToGMJI( int argFlags, String cacheTxnName )
   {
      addArgToGMJI( com.ibm.ifs.gold.auto.GMDGlobals.GtGetUndeliveredMessage );   // Add Transaction identifier.
      addArgToGMJI( argFlags );
      addArgToGMJI( cacheTxnName );
      addArgToGMJI( BuildLevelSignature );

      reqSelfDef_packed = GsGmSelfDefiningFieldDef.pack( reqSelfDef);
      addArgToGMJI( reqSelfDef_packed );
   }

   /**
   * Executes the GtGetUndeliveredMessage transaction by sending it to the
   * Financial Instition (FI) and the response is not saved in the Gold Manager
   * transaction cache.
   * The MTI bit stream message is to be obtained from the undelivered message
   * transaction.
   * The transaction execution mode is controlled by the IfsTxn object executionMode
   * attribute.
   *
   * @exception IfsException When execution fails
   */
   public void execute() throws IfsException
   {
      super.execute(IfsTxn.UNDELIVERED_FLAG, null);
   }

   /**
   * Get GtGetUndeliveredMessage gold message from Gold Message Java Output buffer.
   */
   public void getGoldFromGMJO()
   {
      responseCode            = getNextCharFromGMJO( 3,"responseCode" );
      referenceNumber         = getNextCharFromGMJO( 12,"referenceNumber" );
      serviceProviderReference = getNextCharFromGMJO( 36,"serviceProviderReference" );
      fIProcInd            = getNextCharFromGMJO( 1,"fIProcInd" );
      addSelfDefProperties();
   }

   /**
   * Converts the GtGetUndeliveredMessage object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix)
   {
      if (hasRegisteredMethod("toStringWithString"))
      {
         return super.toString(prefix);
      }
      else
      {
         Cookie cookie = getIfsSession().getSessionKey();
         StringBuffer buffer = new StringBuffer(256);
         buffer.append(super.toString(prefix));

         buffer.append(prefix + "| SessionKey.Name = " + cookie.getName() );
         buffer.append(prefix + "| SessionKey.Value = " + cookie.getValue() );
         buffer.append(prefix + "| BRC = " + getBrc() );
         buffer.append(prefix + "| Status = " + getStatus() );
         buffer.append(prefix + "| Message = " + getMessage());

         buffer.append(prefix + "| responseCode = " + getResponseCode() );
         try { buffer.append(prefix + "| responseCode_text = " + getResponseCode_text() ); }
         catch ( IfsException e)
         {
            buffer.append( prefix + "| responseCode_text throws IfsException: " + e.getMessage());
         }

         buffer.append(prefix + "| serviceProviderReference = " + getServiceProviderReference() );
         buffer.append(prefix + "| fIProcInd = " + getFIProcInd() );
         try { buffer.append(prefix + "| fIProcInd_text = " + getFIProcInd_text() ); }
         catch ( IfsException e)
         {
            buffer.append( prefix + "| fIProcInd_text throws IfsException: " + e.getMessage());
         }

         return buffer.toString();
      }
   }

}

