package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   GtLogoff.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    02/16/2000  rhs   d6361   Made JavaDoc correction/addtions.
//    05/03/2000  rhs   d6662   Readded _text fields to toString or showObject.
//
// ***************************************************************************
import java.util.Enumeration;
import java.util.Hashtable;
import javax.servlet.http.Cookie;

public class GtLogoff extends IfsTxn
{
   private static String GMRequestId             = "logoff_java";
   public String getGMRequestId() {return GMRequestId;}
   public static final int SD_OWNER              = 0;
   public static final int SDThis                = 0;

   private String  fIProcInd = null;
   private String  responseCode = null;
   public static Hashtable registry = new Hashtable();

   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean debugOn = false;

   /*
   * Default constructor
   */
   public GtLogoff(){}

   /**
   * Constructor with an associated IfsSession object.
   */
   public GtLogoff(IfsSession ifsSession) {setIfsSession(ifsSession);}

   /**
   * Add GtLogoff gold message to Gold Message Java Input buffer.
   */
   public void addGoldToGMJI(int argFlags, String cacheTxnName )
   {
      // Select the gmJavaLogoffReq by turning this flag on
      logoffTxn = true;

   }
   public  String  getFIProcInd() {return(fIProcInd);}
   public String getResponseCode() {return responseCode;}
   public  String  getFIProcInd_text() throws IfsException
   {
     return IfsNLS.getText("IfsFiProcIndResources",getIfsSession().getLocale(),getFIProcInd());
   }
   public String getResponseCode_text() throws IfsException
   {
     return IfsNLS.getText("IfsRespCodeClResources",getIfsSession().getLocale(),getResponseCode());
   }

   /**
   * Get GtLogoff gold message from Gold Message Java Output buffer.
   */
   public void getGoldFromGMJO()
   {
      // ===========================================================
      // Note: this is a dummy method
      // ===========================================================
      responseCode           = getNextCharFromGMJO( 3,"responseCode" );
      fIProcInd              = getNextCharFromGMJO( 1,"fIProcInd" );
   }
   public static Hashtable getRegistry() {
      return registry;
   }
   public  void    setFIProcInd(String o) {fIProcInd = o;}
   public void setResponseCode(String o) {responseCode=o;}

   /**
   * Executes the GtLogoff transaction by sending it to the
   * Financial Instition (FI) and the response is not saved in the Gold Manager
   * transaction cache.
   * The transaction execution mode is controlled by the IfsTxn object executionMode
   * attribute.
   *
   * @exception IfsException When execution fails
   */
   public void execute() throws IfsException {
      setContainsRespCode(IfsTxn.RETURNS_RESPONSE_CODE);
      super.execute();
      getIfsSession().getHttpSession().invalidate();
   }

   /**
   * Converts the GtLogoff object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString() {
      if (hasRegisteredMethod("toString"))
         return super.toString();
      else
         return toString("\n");

   }

   /**
   * Converts the GtLogoff object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix) {
      if (hasRegisteredMethod("toStringWithString"))
         return super.toString(prefix);

      Cookie cookie = getIfsSession().getSessionKey();
      StringBuffer buff = new StringBuffer(250);
      buff.append(super.toString(prefix));

      buff.append(prefix);
      buff.append("| BRC = ");
      buff.append(getBrc());

      buff.append(prefix);
      buff.append("| Status = ");
      buff.append(getStatus());

      buff.append(prefix);
      buff.append("| Message = ");
      buff.append(getMessage());

      buff.append(prefix);
      buff.append("| responseCode = ");
      buff.append(responseCode);

      try { buff.append( prefix + "responseCode_text = " + getResponseCode_text() ); }
      catch (IfsException e)
      {
         buff.append(prefix + "responseCode_text throws IfsException: " + e.getMessage());
      }

      buff.append(prefix);
      buff.append("| fIProcInd = ");
      buff.append(fIProcInd);

      try { buff.append( prefix + "fIProcInd_text = " + getFIProcInd_text() ); }
      catch (IfsException e)
      {
         buff.append(prefix + "fIProcInd_text throws IfsException: " + e.getMessage());
      }

      buff.append(prefix);
      if (registry.isEmpty())
      {

         buff.append("| Registry is empty");

      }
      else
      {
         Enumeration keys = registry.keys();
         while (keys.hasMoreElements())
         {
            String key = (String) keys.nextElement();

            buff.append(prefix);
            buff.append("| Registry contains: ");

            buff.append("| ");
            buff.append(key);
            buff.append(" = ");
            buff.append(registry.get(key).toString());
         }
      }
      return buff.toString();

   }
}

