package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   GtLogon.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    02/16/2000  rhs   d6361   Made JavaDoc correction/addtions.
//    03/14/2000  rhs   d6545   Added NLS support
//    04/20/2000  rhs   d6631   Removed Language and country codes from GtLogon GMJI
//    04/28/2000  rhs   d6662   Corrected getters for secAddressLineX. Added secCountry
//                              setter/getter. Added secCity getter. Removed secCountryCode
//                              variable. Corrected toString() and toString(prefix).
//    06/27/2001  tlt  T0987315 Corrected pin/pswd/userid len checks
//    02/05/2002  pdl  d7430    Fixed calls to deprecated APIs
//
// ***************************************************************************
import java.io.UnsupportedEncodingException;
import java.util.Locale;
import javax.servlet.http.Cookie;
import java.util.ResourceBundle;
import java.util.Hashtable;
import java.util.Enumeration;
import com.ibm.ifs.gold.IfsNLS;
import com.ibm.ifs.gold.IfsException;
import com.ibm.ifs.gold.auto.GsAcctDtl;
import com.ibm.ifs.gold.auto.GsInvAcctDtl;
import com.ibm.ifs.gold.auto.GsCnsmrInfo;
import com.ibm.ifs.gold.auto.GtUserProfile;
import com.ibm.ifs.gold.auto.GsCnsmrSrvcProvdRules;
import com.ibm.ifs.servlets.IfsHttpServlet;

public class GtLogon extends IfsTxn
{

   public static final boolean WSP_BAD_POST_DATA = true;
   public static final boolean WSP_NO_TRUNCATE   = true;

   private static final String GMRequestId    = "logon_java";

   public static final int SD_OWNER = 0;

   public static final int SDThis = 0;
   public static final int SDAccountDetail = 2622;
   public static final String IFSCC_FAILURE = "1";
   public static final String IFSRC_INVALID_CNSMR_ID = "2068";
   public static final String IFSRC_INVALID_CH_CNSMR_ID = "2069";
   public static final String INVALID_LOGON = "064";

   // request fields

   private String  userID    = null;

   private byte[] pin = null;
   private boolean encryptPin = false;
   private String pinEncryptionType = IfsSystem.RSA_ENCRYPTION;

   private byte[]  passwordID    = null;
   private boolean encryptPassword = false;
   private String passwordEncryptionType = IfsSystem.RSA_ENCRYPTION;

   private Locale  locale           = IfsSystem.SERVER_LOCALE;

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   private String  countryCode      = null;

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   private String  languageCode     = null;

   private String  productName      = IfsSystem.DefaultValueMissing;  //Product Name (e.g. Netscape, Quicken)
   private String  releaseVersion   = IfsSystem.DefaultValueMissing;  //Product Release Version
   private String  clientOS         = IfsSystem.DefaultValueMissing;  //Client Operating System

   // response fields

   private Cookie  sessionKey                = null;
   private GsGmSelfDefiningFieldDef[]    respSelfDefData = null;
   private String  responseCode = null;
   private String  fIProcInd = null;
   private String  consumerID = null;
   private String  changeableConsumerID = null;
   private String  primaryConsumerID = null;
   private String  permUserID = null;
   private String  lastName = null;
   private String  firstName = null;
   private String  middleName = null;
   private String  signupSource = null;
   private String  faxNumber = null;
   private String  suffix = null;
   private String  fIEmploymentType = null;
   private String  addressLine1 = null;
   private String  addressLine2 = null;
   private String  addressLine3 = null;
   private String  city = null;
   private String  province = null;
   private String  state = null;
   private String  country = null;
   private String  zipCode = null;
   private String  postalCode = null;
   private String  secAddressLine1 = null;
   private String  secAddressLine2 = null;
   private String  secAddressLine3 = null;
   private String  secCity = null;
   private String  secProvince = null;
   private String  secState = null;
   private String  secCountry = null;
   private String  secZipCode = null;
   private String  secPostalCode = null;
   private String  homePhone = null;
   private String  workPhone = null;
   private GsDate effectiveDate = null;
   private GsDate endDate = null;
   private String  newMessageFlag = null;
   private GsAcctDtl[]    accountDetail = null;
   private String  serviceChargePlan = null;
   private String  consumerType = null;
   private String  lastSessionTimestamp = null;
   private int     gMTOffsetMinutes = 0;
   private int     marketSegment = 0;
   private int     division = 0;
   private String  legalName = null;
   private String  socialSecurityNumber = null;
   private String  chargeBasisCode = null;
   private GsCurrency dailyLimitAmount = null;
   private String  queueQualifier = null;
   private String  sLASampleFlag = null;
   private String  tokenInformation = null;
   private String  emailAddress = null;
   private int     taxType = 0;
   private String  brokerageName = null;
   private String  brokerageLastName = null;
   private String  brokerageFirstName = null;
   private String  brokerageMiddleName = null;
   private String  brokerageOfficeID = null;
   private int     brokerageInvCat = 0;
   private String  brokerageEmail = null;
   private GsInvAcctDtl[] ifsInvAcctDtl = null;
   private GsCnsmrInfo[] ifsCnsmrInfo = null;

   private GsCnsmrSrvcProvdRules[] ifsCnsmrSvcProviderRules = null;
   private int     secCnsmrMntAuth = 0;
   private String  lastPinTimestamp = null;
   private int     lastPinGMTOffset = 0;
   private String  lastLogTimestamp = null;
   private int     lastLogGMTOffset = 0;
   private int     fICnsmrStCode = 0;
   private String  businessName = null;
   private int     busnIdCode = 0;
   private String  businessId = null;
   private String  altCnsmrLastName = null;
   private String  altCnsmrFirstName = null;
   private String  altCnsmrMiddleName = null;
   private String  altCnsmrSuffix = null;
   private String  houseHoldID = null;
   private int  disclosureIndicator = 0;

   public static Hashtable registry = new Hashtable();

   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean debugOn = false;

   /**
   * Default constructor
   */
   public GtLogon(){}

   /**
   * Constructor with an associated IfsSession object.
   */
   public GtLogon(IfsSession sess)
   {
      setIfsSession(sess);
   }

   /**
   * Add Logon gold message to Gold Message Java Input buffer.
   */
   public void addGoldToGMJI(int argFlags, String cacheTxnName) {
      addArgToGMJI(userID);
      addArgToGMJI(getPin(IfsTxn.ENCRYPT));
      addArgToGMJI(pinEncryptionType);
      addArgToGMJI(getPassword(IfsTxn.ENCRYPT));
      addArgToGMJI(passwordEncryptionType);
      addArgToGMJI(productName);
      addArgToGMJI(releaseVersion);
      addArgToGMJI(clientOS);
      addArgToGMJI(disclosureIndicator);
      addArgToGMJI(houseHoldID);

      reqSelfDef_packed = GsGmSelfDefiningFieldDef.pack( reqSelfDef);
      addArgToGMJI( reqSelfDef_packed );
      return;
   }

   /**
   * Executes the GtLogon transaction by sending it to the
   * Financial Instition (FI) and the response is not saved in the Gold Manager
   * transaction cache.
   * The transaction execution mode is controlled by the IfsTxn object executionMode
   * attribute.
   *
   * @exception IfsException When execution fails
   */
   public void execute() throws IfsException
   {
      setContainsRespCode(IfsTxn.RETURNS_RESPONSE_CODE);
      IfsSession sess = getIfsSession();

      if ( IfsSystem.browserLocaleUsage == IfsSystem.USE_SERVER_LOCALE )
      {
         sess.setLocale(IfsSystem.SERVER_LOCALE );
      }
      else
         IfsHttpServlet.checkBrowserLocale(sess.getHttpServletRequest() );

      logonTxn = true;
      super.execute();

      if (status==OK)
      {
         sess.setSessionKey(getSessionKey());

         //get profile data from Gold Mgr cache
         GtUserProfile userProfile = new GtUserProfile(sess);
         userProfile.execute(CACHE);
         if (userProfile.getStatus() == userProfile.OK)
            sess.getHttpSession().setAttribute("__UserProfile", userProfile);

         else
         {
            errorCode = userProfile.getErrorCode();
            brc = userProfile.getBrc();
            status = userProfile.getStatus();
            compCode = userProfile.getCompCode();
            reasonCode = userProfile.getReasonCode();
            errorFunctionName = userProfile.getErrorFunctionName();
            errorModuleName = userProfile.getErrorModuleName();
            message = "Logon transaction successful, error executing UserProfile transaction, all of this error information relates to the GtUserProfile transaction and not the GtLogon transaction - " + userProfile.getMessage();
            gmMessage = userProfile.getGmMessage();
            senseData = userProfile.getSenseData();
            senseDataLength = userProfile.getSenseDataLength();
         }
      }
   }
   /**
   * This method overrides the IfsTxn version of processCCErrors to check specifically for a Core Controller failure
   * with either a reason code of invalid change consumer ID or invalid consumer ID.  If encountered, a BRC value and the
   * appropriate text message will be set for the invalid logon identification, and the transaction status will be changed
   * to reflect a fatal error.  The BRC value can be retrieved using the getBrc method, the message text can be retrieved
   * using the getMessage method and the transaction status can be retrieved with the getStatus method.
   * <P>All other types of CoreController errors, or reason codes that do not signify an invalid change consumer ID or
   * and invalid consumer ID, are handled by the IfsTxn version of processCCErrors.
   */
   public void processCCErrors() {
        String tempStatus = null;

        if (compCode.equals(IFSCC_FAILURE) && (reasonCode.equals(IFSRC_INVALID_CH_CNSMR_ID) || reasonCode.equals(IFSRC_INVALID_CNSMR_ID))) {
                brc = INVALID_LOGON;
                errorCode = TXN_3_TOKEN;
                message = getMessageText("com.ibm.ifs.resource.IfsBrcErrorTextBundle", brc);
                debug("<BR>BRC Error Message = " + message + "<BR>");
                tempStatus = (String)IfsSystem.allBrcTypesDictionary.get(brc);
                if (tempStatus == null) {
                        debug("<BR>Developer, this BRC error not in hashtable, setting status to FATAL_ERROR!<BR>");
                        status = FATAL_ERROR;
                } else
                        setStatus(tempStatus);
        } else
                super.processCCErrors();
   }
   public String getResponseCode() {return responseCode;}
   public String getResponseCode_text() throws IfsException
   {
      return IfsNLS.getText("IfsRespCodeClResources",
                            getIfsSession().getLocale(),
                            getResponseCode());
   }

   public void setResponseCode(String o) {responseCode=o;}
   public GsAcctDtl[]    getAccountDetail() {return(accountDetail);}
   public  String  getAddressLine1() {return(addressLine1);}
   public  String  getAddressLine2() {return(addressLine2);}
   public  String  getAddressLine3() {return(addressLine3);}
   public  String  getChangeableConsumerID() {return(changeableConsumerID);}
   public  String  getPrimaryConsumerID() {return(primaryConsumerID);}
   public  String  getChargeBasisCode() {return(chargeBasisCode);}
   public  String  getChargeBasisCode_text() throws IfsException
   {
      return IfsNLS.getText("IfsChrgBasisTpClResources",
                            getIfsSession().getLocale(),
                            getChargeBasisCode());
   }
   public  String  getCity() {return(city);}
   public  String  getClientOS() {return(clientOS);}
   public  String  getCnsmrEmailAddr() {return(emailAddress);}
   public  String  getConsumerID() {return(consumerID);}
   public  String  getConsumerType() {return(consumerType);}
   public  String  getConsumerType_text()  throws IfsException
   {
      return IfsNLS.getText("IfsCnsmrTpClResources",
                            getIfsSession().getLocale(),
                            getConsumerType());
   }
   public  String  getCountry() {return(country);}

   public  String  getSecAddressLine1() {return(secAddressLine1);}
   public  String  getSecAddressLine2() {return(secAddressLine2);}
   public  String  getSecAddressLine3() {return(secAddressLine3);}
   public  String  getSecCity() {return(secCity);}
   public  String  getSecCountry() {return(secCountry);}
   public  String  getSecProvince() {return(secProvince);}
   public  String  getSecState() {return(secState);}
   public  String  getSecZipCode() {return(secZipCode);}
   public  String  getSecPostalCode() {return(secPostalCode);}

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   public  String  getCountryCode() {return(countryCode);}
   public  GsCurrency getDailyLimitAmount() {return(dailyLimitAmount);}
   public  int     getDivision() {return(division);}
   public  String  getDivision_text() throws IfsException
   {
      return IfsNLS.getText("IfsCnsmrDivisionResources",
                            getIfsSession().getLocale(),
                            String.valueOf(getDivision()) );
   }
   public  GsDate getEffectiveDate() {return(effectiveDate);}
   public  GsDate getEndDate() {return(endDate);}
   public  String  getFaxNumber() {return(faxNumber);}
   public  String  getFIEmploymentType() {return(fIEmploymentType);}
   public  String  getFIEmploymentType_text() throws IfsException
   {
      return IfsNLS.getText("IfsFiEmpTpIndResources",
                            getIfsSession().getLocale(),
                            getFIEmploymentType());
   }
   public  String  getFIProcInd() {return(fIProcInd);}
   public  String  getFIProcInd_text() throws IfsException
   {
      return IfsNLS.getText("IfsFiProcIndResources",
                             getIfsSession().getLocale(),
                             getFIProcInd());
   }
   public  String  getFirstName() {return(firstName);}
   public  String getGMRequestId()                 {return(GMRequestId);}
   public  int     getGMTOffsetMinutes() {return(gMTOffsetMinutes);}
   public  String  getEmailAddress() {return emailAddress;}
   public  int     getTaxType() {return taxType;}
   public  String  getBrokerageName() {return brokerageName;}
   public  String  getBrokerageLastName() {return brokerageLastName;}
   public  String  getBrokerageFirstName() {return brokerageFirstName;}
   public  String  getBrokerageMiddleName() {return brokerageMiddleName;}
   public  String  getBrokerageOfficeID() {return brokerageOfficeID;}
   public  int     getBrokerageInvCat() {return brokerageInvCat;}
   public  String  getBrokerageEmail() {return brokerageEmail;}
   public  GsInvAcctDtl[] getIfsInvAcctDtl() {return ifsInvAcctDtl;}
   public  GsCnsmrInfo[] getIfsCnsmrInfo() {return ifsCnsmrInfo;}
   public GsCnsmrSrvcProvdRules[] getIfsCnsmrSvcProviderRules() {return ifsCnsmrSvcProviderRules;}
   public int     getSecCnsmrMntAuth() {return secCnsmrMntAuth;}
   public String  getSecCnsmrMntAuth_text() throws IfsException
   {
      return IfsNLS.getText("IfsSecCnsmrMntAuthFResources",
                            getIfsSession().getLocale(),
                            String.valueOf(getSecCnsmrMntAuth()) );
   }
   public String  getLastPinTimestamp() {return lastPinTimestamp;}
   public int     getLastPinGMTOffset() {return lastPinGMTOffset;}
   public String  getLastLogTimestamp() {return lastLogTimestamp;}
   public int     getLastLogGMTOffset() {return lastLogGMTOffset;}
   public int     getFICnsmrStCode() {return fICnsmrStCode;}
   public String  getBusinessName() {return businessName;}
   public int     getBusnIdCode() {return busnIdCode;}
   public String  getFICnsmrStCode_text() throws IfsException
   {
      return IfsNLS.getText("IfsFICnsmrStTpClResources",
                            getIfsSession().getLocale(),
                            String.valueOf(getFICnsmrStCode()) );
   }
   public String  getBusnIdCode_text()  throws IfsException
   {
      return IfsNLS.getText("IfsBusnIdTpClResources",
                             getIfsSession().getLocale(),
                             String.valueOf(getBusnIdCode()) );
   }
   public String  getBusinessId() {return businessId;}
   public String  getAltCnsmrLastName() {return altCnsmrLastName;}
   public String  getAltCnsmrFirstName() {return altCnsmrFirstName;}
   public String  getAltCnsmrMiddleName() {return altCnsmrMiddleName;}
   public String  getAltCnsmrSuffix() {return altCnsmrSuffix;}
   public String getHouseHoldID() {return houseHoldID;}
   public int getDisclosureIndicator() {return disclosureIndicator;}

   /**
   * Get Logon gold message from Gold Message Java Output buffer.
   */
   public void getGoldFromGMJO()
   {
      int i, repeatCounter;

      responseCode                       = getNextCharFromGMJO( 3,"responseCode" );
      fIProcInd                          = getNextCharFromGMJO( 1,"fIProcInd" );
      consumerID                         = getNextCharFromGMJO( 30,"consumerID" );
      changeableConsumerID               = getNextCharFromGMJO( 30,"changeableConsumerID" );
      permUserID                         = getNextCharFromGMJO( 57,"permUserID" );
      primaryConsumerID                  = getNextCharFromGMJO( 30,"primaryConsumerID" );
      lastName                           = getNextNlsCharFromGMJO( 30,"lastName" );
      firstName                          = getNextNlsCharFromGMJO( 30,"firstName" );
      middleName                         = getNextNlsCharFromGMJO( 20,"middleName" );
      signupSource                       = getNextCharFromGMJO( 3,"signupSource" );
      faxNumber                          = getNextCharFromGMJO( 15,"faxNumber" );
      suffix                             = getNextNlsCharFromGMJO( 8,"suffix" );
      fIEmploymentType                   = getNextCharFromGMJO( 3,"fIEmploymentType" );
      addressLine1                       = getNextNlsCharFromGMJO( 40,"addressLine1" );
      addressLine2                       = getNextNlsCharFromGMJO( 40,"addressLine2" );
      addressLine3                       = getNextNlsCharFromGMJO( 40,"addressLine3" );
      city                               = getNextNlsCharFromGMJO( 30,"city" );
      province                           = getNextNlsCharFromGMJO( 30,"province" );
      state                              = getNextCharFromGMJO( 2,"state" );
      country                            = getNextCharFromGMJO( 3,"country" );
      zipCode                            = getNextCharFromGMJO( 9,"zipCode" );
      postalCode                         = getNextCharFromGMJO( 6,"postalCode" );
      secAddressLine1                    = getNextNlsCharFromGMJO( 40,"secAddressLine1" );
      secAddressLine2                    = getNextNlsCharFromGMJO( 40,"secAddressLine2" );
      secAddressLine3                    = getNextNlsCharFromGMJO( 40,"secAddressLine3" );
      secCity                            = getNextNlsCharFromGMJO( 30,"secCity" );
      secProvince                        = getNextNlsCharFromGMJO( 30,"secProvince" );
      secState                           = getNextCharFromGMJO( 2,"secState" );
      secCountry                         = getNextCharFromGMJO( 3,"secCountry" );
      secZipCode                         = getNextCharFromGMJO( 9,"zipCode" );
      secPostalCode                      = getNextCharFromGMJO( 6,"secPostalCode" );
      homePhone                          = getNextCharFromGMJO( 15,"homePhone" );
      workPhone                          = getNextCharFromGMJO( 15,"workPhone" );
      effectiveDate                      = new GsDate(this,"effectiveDate");
      endDate                            = new GsDate(this,"endDate");
      newMessageFlag                     = getNextCharFromGMJO( 1,"newMessageFlag" );

      repeatCounter = getNextLongFromGMJO( "repeatCounter" );
      accountDetail = new GsAcctDtl[repeatCounter];
      for (i = 0; i <    repeatCounter; i++)
      {
         accountDetail[i]                         = new GsAcctDtl(this);
      }

      serviceChargePlan                  = getNextCharFromGMJO( 1,"serviceChargePlan" );
      consumerType                       = getNextCharFromGMJO( 1,"consumerType" );
      lastSessionTimestamp               = getNextCharFromGMJO( 26,"lastSessionTimestamp" );
      gMTOffsetMinutes                   = getNextLongFromGMJO( "gMTOffsetMinutes" );
      marketSegment                      = getNextLongFromGMJO( "marketSegment" );
      division                           = getNextLongFromGMJO( "division" );
      legalName                          = getNextNlsCharFromGMJO( 64,"legalName" );
      socialSecurityNumber               = getNextCharFromGMJO( 9,"socialSecurityNumber" );
      chargeBasisCode                    = getNextCharFromGMJO( 1,"chargeBasisCode" );
      dailyLimitAmount                   = new GsCurrency(this,"dailyLimitAmount");
      emailAddress                       = getNextCharFromGMJO( 128,"emailAddress" );
      taxType                            = getNextLongFromGMJO( "taxType" );
      brokerageName                      = getNextNlsCharFromGMJO( 35,"brokerageName" );
      brokerageLastName                  = getNextNlsCharFromGMJO( 30,"brokerageLastName" );
      brokerageFirstName                 = getNextNlsCharFromGMJO( 30,"brokerageFirstName" );
      brokerageMiddleName                = getNextNlsCharFromGMJO( 30,"brokerageMiddleName" );
      brokerageOfficeID                  = getNextNlsCharFromGMJO( 10,"brokerageOfficeID" );
      brokerageInvCat                    = getNextLongFromGMJO( "brokerageInvCat" );
      brokerageEmail                     = getNextCharFromGMJO( 128,"brokerageEmail" );

      repeatCounter = getNextLongFromGMJO( "repeatCounter" );
      ifsInvAcctDtl = new GsInvAcctDtl[repeatCounter];
      for (i = 0; i < repeatCounter; i++)
      {
         ifsInvAcctDtl[i]                   = new GsInvAcctDtl(this);
      }

      repeatCounter = getNextLongFromGMJO( "repeatCounter" );
      ifsCnsmrInfo = new GsCnsmrInfo[repeatCounter];
      for (i = 0; i < repeatCounter; i++)
      {
         ifsCnsmrInfo[i]                    = new GsCnsmrInfo(this);
      }

      repeatCounter = getNextLongFromGMJO( "repeatCounter" );
      ifsCnsmrSvcProviderRules = new GsCnsmrSrvcProvdRules[repeatCounter];
      for (i = 0; i < repeatCounter; i++)
      {
         ifsCnsmrSvcProviderRules[i]        = new GsCnsmrSrvcProvdRules(this);
      }
      secCnsmrMntAuth                    = getNextLongFromGMJO( "secCnsmrMntAuth" );
      lastPinTimestamp                   = getNextCharFromGMJO( 26,"lastPinTimestamp" );
      lastPinGMTOffset                   = getNextLongFromGMJO( "lastPinGMTOffset" );
      lastLogTimestamp                   = getNextCharFromGMJO( 26,"lastLogTimestamp" );
      lastLogGMTOffset                   = getNextLongFromGMJO( "lastLogGMTOffset" );

      fICnsmrStCode                      = getNextLongFromGMJO( "fICnsmrStCode" );
      businessName                       = getNextCharItemFromGMJO( "businessName" );
      busnIdCode                         = getNextLongFromGMJO( "busnIdCode" );

      businessId                         = getNextNlsCharFromGMJO( 20,"businessID" );
      altCnsmrLastName                   = getNextNlsCharFromGMJO( 30,"altCnsmrLastName" );
      altCnsmrFirstName                  = getNextNlsCharFromGMJO( 30,"altCnsmrFirstName" );
      altCnsmrMiddleName                 = getNextNlsCharFromGMJO( 20,"altCnsmrMiddleName" );
      altCnsmrSuffix                     = getNextNlsCharFromGMJO( 8,"altCnsmrSuffix" );
      queueQualifier                     = getNextCharFromGMJO( 8,"queueQualifier" );
      sLASampleFlag                      = getNextCharFromGMJO( 1,"sLASampleFlag" );
      tokenInformation                   = getNextCharFromGMJO( 80,"tokenInformation" );
      sessionKey                         = getNextCookieFromGMJO("cookieName", "cookieValue");

      addSelfDefProperties();

   }
   public  String  getHomePhone() {return(homePhone);}

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   public  String  getLanguageCode() {return(languageCode);}
   public  String  getLastName() {return(lastName);}
   public  String  getLastSessionTimestamp() {return(lastSessionTimestamp);}
   public  String  getLegalName() {return(legalName);}
   public  int     getMarketSegment() {return(marketSegment);}
   public  String  getMarketSegment_text() throws IfsException
   {
      return IfsNLS.getText("IfsCnsmrMktSegResources",
                            getIfsSession().getLocale(),
                            String.valueOf(getMarketSegment()) );
   }
   public  String  getMiddleName() {return(middleName);}
   public  String  getNewMessageFlag() {return(newMessageFlag);}
   public  String  getNewMessageFlag_text() throws IfsException
   {
      return IfsNLS.getText("IfsNewMsgFResources",
                            getIfsSession().getLocale(),
                            getNewMessageFlag());
   }
   public  byte[]  getPassword() {return(passwordID);}
   private byte[]  getPassword(boolean format) {
      if (true==format)
      {
         if (encryptPassword && (IfsSystem.RSA_ENCRYPTION.equals(passwordEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(passwordID, IfsSystem.RSA_ENCRYPTION);
            if (!security.isValid())
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;

            }
            return security.getEncodedValue();
         }
      }
      return passwordID;
   }
   public  String  getPasswordEncryptionType() {return(passwordEncryptionType);}
   public  String  getPermUserID() {return(permUserID);}
   public  byte[]  getPin() {return(pin);}
   private byte[]  getPin(boolean format) {
      if (true==format)
      {
         if (encryptPin && (IfsSystem.RSA_ENCRYPTION.equals(pinEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(pin, IfsSystem.RSA_ENCRYPTION);
            debug("IfsSecurity.getReturnCode() value is , " + security.getReturnCode());
            if (security.isValid())
               return security.getEncodedValue();

            else
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;
               return security.getEncodedValue();
            }
         }
      }
      return pin;
   }
   public  String  getPinEncryptionType() {return(pinEncryptionType);}
   public  String  getPostalCode() {return(postalCode);}
   public  String  getProductName() {return(productName);}
   public  String  getProvince() {return(province);}
   public  String  getQueueQualifier() {return(queueQualifier);}

   /**
   * Returns a Hashtable object which contains the list of registered methods.
   */
   public static Hashtable getRegistry() {
      return registry;
   }
   public  String  getReleaseVersion() {return(releaseVersion);}
   public GsGmSelfDefiningFieldDef[]    getRespSelfDefData() {return(respSelfDefData);}

   public int getSD_OWNER() { return( SD_OWNER );}
   public  String  getServiceChargePlan() {return(serviceChargePlan);}
   public  String  getServiceChargePlan_text() throws IfsException
   {
      return IfsNLS.getText("IfsSrvcChrgPlnResources",
                            getIfsSession().getLocale(),
                            getServiceChargePlan());
   }
   public Cookie getSessionKey(){return sessionKey;}
   public  String  getSignupSource() {return(signupSource);}
   public  String  getSignupSource_text()  throws IfsException
   {
      return IfsNLS.getText("IfsCnsmrSignupSrcResources",
                            getIfsSession().getLocale(),
                            getSignupSource());
   }
   public  String  getSLASampleFlag() {return(sLASampleFlag);}
   public  String  getSLASampleFlag_text()  throws IfsException
   {
      return IfsNLS.getText("IfsSLASampleFResources",
                            getIfsSession().getLocale(),
                            getSLASampleFlag());
   }
   public  String  getSocialSecurityNumber() {return(socialSecurityNumber);}
   public  String  getState() {return(state);}
   public  String  getSuffix() {return(suffix);}
   public  String  getTokenInformation() {return(tokenInformation);}
   public String getUserID() {return(userID);}
   public  String  getWorkPhone() {return(workPhone);}
   public  String  getZipCode() {return(zipCode);}
   public  void      setAccountDetail(GsAcctDtl[] o) {accountDetail = o;}
   public  void    setAddressLine1(String o) {addressLine1 = o;}
   public  void    setAddressLine2(String o) {addressLine2 = o;}
   public  void    setAddressLine3(String o) {addressLine3 = o;}
   public  void    setSecAddressLine1(String o) {secAddressLine1 = o;}
   public  void    setSecAddressLine2(String o) {secAddressLine2 = o;}
   public  void    setSecAddressLine3(String o) {secAddressLine3 = o;}
   public  void    setChangeableConsumerID(String o) {changeableConsumerID = o;}
   public  void    setPrimaryConsumerID(String o) {primaryConsumerID = o;}
   public  void    setChargeBasisCode(String o) {chargeBasisCode = o;}
   public  void    setCity(String o) {city = o;}
   public  void    setSecCity(String o) {secCity = o;}
   public void setClientOS (String o) {clientOS = o;}
   public  void    setCnsmrEmailAddr(String o) {emailAddress = o;}
   public  void    setConsumerID(String o) {consumerID = o;}
   public  void    setConsumerType(String o) {consumerType = o;}
   public  void    setCountry(String o) {country = o;}
   public void     setSecCountry(String o) {secCountry = o;}

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   public void setCountryCode (String o) {countryCode = o;}
   public  void         setDailyLimitAmount(GsCurrency o) {dailyLimitAmount = o;}
   public  void    setDivision(int o) {division = o;}
   public  void         setEffectiveDate(GsDate o) {effectiveDate = o;}
   public  void         setEndDate(GsDate o) {endDate = o;}
   public  void    setFaxNumber(String o) {faxNumber = o;}
   public  void    setFIEmploymentType(String o) {fIEmploymentType = o;}
   public  void    setFIProcInd(String o) {fIProcInd = o;}
   public  void    setFirstName(String o) {firstName = o;}
   public  void    setGMTOffsetMinutes(int o) {gMTOffsetMinutes = o;}
   public  void    setHomePhone(String o) {homePhone = o;}

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   public  void    setLanguageCode (String o) {languageCode = o;}
   public  void    setLastName(String o) {lastName = o;}
   public  void    setLastSessionTimestamp(String o) {lastSessionTimestamp = o;}
   public  void    setLegalName(String o) {legalName = o;}
   public  void    setMarketSegment(int o) {marketSegment = o;}
   public  void    setMiddleName(String o) {middleName = o;}
   public  void    setNewMessageFlag(String o) {newMessageFlag = o;}

   public void setPassword(byte[] o){passwordID = o;encryptPassword=false;}

   public void setPassword(String o)
   {
      try
      {
         passwordID = o.getBytes(IfsSystem.cwsGMJCharSet);
         encryptPassword = true;
      }
      catch (UnsupportedEncodingException e)
      {
         passwordID = null;
         encryptPassword = false;
         debug("setPasswordID failure in GtLogon, exception is -- " + e.getMessage() );
      }
   }
   public void setPasswordEncryptionType(String o){passwordEncryptionType = o;}
   public  void    setPermUserID(String o) {permUserID = o;}

   public void setPin(byte[] o) {pin = o;encryptPin=false;}

   public void setPin(String o)
   {
      try
      {
         pin = o.getBytes(IfsSystem.cwsGMJCharSet);
         encryptPin=true;
      }
      catch (UnsupportedEncodingException e)
      {
         pin = null;
         encryptPin = false;
         debug("setPin failure in GtLogon, exception is -- " + e.getMessage() );
      }
   }
   public void setPinEncryptionType(String o){pinEncryptionType = o;}
   public  void    setPostalCode(String o) {postalCode = o;}
   public  void    setSecPostalCode(String o) {secPostalCode = o;}
   public void setProductName (String o) {productName = o;}
   public  void    setProvince(String o) {province = o;}
   public  void    setSecProvince(String o) {secProvince = o;}
   public  void    setQueueQualifier(String o) {queueQualifier = o;}
   public void setReleaseVersion (String o) {releaseVersion = o;}
   public  void      setRespSelfDefData(GsGmSelfDefiningFieldDef[] o) {respSelfDefData = o;}
   public  void    setServiceChargePlan(String o) {serviceChargePlan = o;}
   public  void    setSignupSource(String o) {signupSource = o;}
   public  void    setSLASampleFlag(String o) {sLASampleFlag = o;}
   public  void    setSocialSecurityNumber(String o) {socialSecurityNumber = o;}
   public  void    setState(String o) {state = o;}
   public  void    setSecState(String o) {secState = o;}
   public  void    setSuffix(String o) {suffix = o;}
   public  void    setTokenInformation(String o) {tokenInformation = o;}
   public void setUserID(String o) {userID = o;}
   public  void    setWorkPhone(String o) {workPhone = o;}
   public  void    setZipCode(String o) {zipCode = o;}
   public  void    setSecZipCode(String o) {secZipCode = o;}
   public  void    setEmailAddress(String o) {emailAddress = o;}
   public  void    setTaxType(int o) {taxType = o;}
   public  void    setBrokerageName(String o) {brokerageName = o;}
   public  void    setBrokerageLastName(String o) {brokerageLastName = o;}
   public  void    setBrokerageFirstName(String o) {brokerageFirstName = o;}
   public  void    setBrokerageMiddleName(String o) {brokerageMiddleName = o;}
   public  void    setBrokerageOfficeID(String o) {brokerageOfficeID = o;}
   public  void    setBrokerageInvCat(int o) {brokerageInvCat = o;}
   public  void    setBrokerageEmail(String o) {brokerageEmail = o;}
   public  void    setIfsInvAcctDtl(GsInvAcctDtl[] o) {ifsInvAcctDtl = o;}
   public  void    setIfsCnsmrInfo(GsCnsmrInfo[] o) {ifsCnsmrInfo = o;}
   public  void    setIfsCnsmrSvcProviderRules(GsCnsmrSrvcProvdRules[] o) {ifsCnsmrSvcProviderRules = o;}
   public  void        setSecCnsmrMntAuth(int o) {secCnsmrMntAuth = o;}
   public  void     setLastPinTimestamp(String o) {lastPinTimestamp = o;}
   public  void        setLastPinGMTOffset(int o) {lastPinGMTOffset = o;}
   public  void     setLastLogTimestamp(String o) {lastLogTimestamp = o;}
   public  void        setLastLogGMTOffset(int o) {lastLogGMTOffset = o;}
   public  void        setFICnsmrStCode(int o) {fICnsmrStCode = o;}
   public  void        setBusnIdCode(int o) {busnIdCode = o;}
   public  void     setBusinessName(String o) {businessName = o;}
   public  void     setBusinessId(String o) {businessId = o;}
   public  void     setAltCnsmrLastName(String o) {altCnsmrLastName = o;}
   public  void     setAltCnsmrFirstName(String o) {altCnsmrFirstName = o;}
   public  void     setAltCnsmrMiddleName(String o) {altCnsmrMiddleName = o;}
   public  void     setAltCnsmrSuffix(String o) {altCnsmrSuffix = o;}
   public void setHouseHoldID(String o) {houseHoldID=o;}
   public void setDisclosureIndicator(int o) {disclosureIndicator=o;}

   /**
   * Converts the GtLogon object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      if (hasRegisteredMethod("toString"))
         return super.toString();
      else
         return toString("\n");  // Return string from toString with
                                 // carriage return, line feed prefix.
   }

   /**
   * Converts the GtLogon object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix) {
      if (hasRegisteredMethod("toStringWithString"))
         return super.toString(prefix);

      Cookie cookie = getIfsSession().getSessionKey();
      StringBuffer buff = new StringBuffer(250);
      buff.append(super.toString(prefix));

      buff.append(prefix + "| userID = " + (new String(userID)) );
      buff.append(prefix + "| Pin = " + (new String(pin)) );
      buff.append(prefix + "| PinEncryptionType = " + pinEncryptionType);
      buff.append(prefix + "| Password = " + (new String(passwordID)) );
      buff.append(prefix + "| PasswordEncryptionType = " + passwordEncryptionType);
      buff.append(prefix + "| SessionKey.Name = " + cookie.getName());
      buff.append(prefix + "| SessionKey.Value = " + sessionKey.getValue());
      buff.append(prefix + "| BRC = " + getBrc());
      buff.append(prefix + "| Status = " + getStatus());
      buff.append(prefix + "| Message = " + getMessage());

      buff.append(prefix + "| responseCode = " + responseCode);
      try { buff.append(prefix + "| responseCode_text = " + getResponseCode_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| responseCode_text throws IfsException: " + e.getMessage());
      }

      buff.append(prefix + "| fIProcInd = " + fIProcInd);
      try { buff.append(prefix + "| fIProcInd_text = " + getFIProcInd_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| fIProcInd_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| consumerID = " + consumerID);
      buff.append( prefix + "| changeableConsumerID = " + changeableConsumerID);
      buff.append( prefix + "| permUserID = " + permUserID);
      buff.append( prefix + "| primaryConsumerID = " + primaryConsumerID);
      buff.append( prefix + "| lastName = " + lastName);
      buff.append( prefix + "| firstName = " + firstName);
      buff.append( prefix + "| middleName = " + middleName);
      buff.append( prefix + "| signupSource = " + signupSource);
      try { buff.append( prefix + "| signupSource_text = " + getSignupSource_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| signupSource_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| faxNumber  = " + faxNumber);
      buff.append( prefix + "| suffix  = " + suffix);
      buff.append( prefix + "| fIEmploymentType = " + fIEmploymentType);
      try { buff.append( prefix + "| fIEmploymentType_text = " + getFIEmploymentType_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| fIEmploymentType_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| addressLine1 = " + addressLine1);
      buff.append( prefix + "| addressLine2 = " + addressLine2);
      buff.append( prefix + "| addressLine3 = " + addressLine3);
      buff.append( prefix + "| city = " + city);
      buff.append( prefix + "| province = " + province);
      buff.append( prefix + "| state = " + state);
      buff.append( prefix + "| country = " + country);
      buff.append( prefix + "| zipCode = " + zipCode);
      buff.append( prefix + "| postalCode = " + postalCode);
      buff.append( prefix + "| secAddressLine1 = " + secAddressLine1);
      buff.append( prefix + "| secAddressLine2 = " + secAddressLine2);
      buff.append( prefix + "| secAddressLine3 = " + secAddressLine3);
      buff.append( prefix + "| secCity = " + secCity);
      buff.append( prefix + "| secProvince = " + secProvince);
      buff.append( prefix + "| secState = " + secState);
      buff.append( prefix + "| secCountry = " + secCountry);
      buff.append( prefix + "| secZipCode = " + zipCode);
      buff.append( prefix + "| secPostalCode = " + secPostalCode );
      buff.append( prefix + "| homePhone = " + homePhone);
      buff.append( prefix + "| workPhone = " + workPhone);
      buff.append( prefix + "| effectiveDate = " + effectiveDate.toString());
      buff.append( prefix + "| endDate = " + endDate.toString());
      buff.append( prefix + "| newMessageFlag = " + newMessageFlag);
      try { buff.append( prefix + "| newMessageFlag_text = " + getNewMessageFlag_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| newMessageFlag_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| accountDetail[i] = ");             // Not displaying.

      buff.append( prefix + "| serviceChargePlan = " + serviceChargePlan);
      try { buff.append( prefix + "| serviceChargePlan_text = " + getServiceChargePlan_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| serviceChargePlan_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| consumerType = " + consumerType);
      try { buff.append( prefix + "| consumerType_text = " + getConsumerType_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| consumerType_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| lastSessionTimestamp = " + lastSessionTimestamp);
      buff.append( prefix + "| gMTOffsetMinutes = " + Integer.toString(gMTOffsetMinutes));
      try { buff.append( prefix + "| marketSegment_text = " + getMarketSegment_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| marketSegment_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| division = " + Integer.toString(division));
      try { buff.append( prefix + "| division_text = " + getDivision_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| division_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| legalName  = " + legalName);
      buff.append( prefix + "| socialSecurityNumber = " + socialSecurityNumber);
      buff.append( prefix + "| chargeBasisCode = " + chargeBasisCode);
      try { buff.append( prefix + "| chargeBasisCode_text = " + getChargeBasisCode_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| chargeBasisCode_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| dailyLimitAmount = " + dailyLimitAmount.toString());
      buff.append( prefix + "| emailAddress = " + emailAddress);
      buff.append( prefix + "| taxType = " + Integer.toString(taxType));
      buff.append( prefix + "| brokerageName = " + brokerageName);
      buff.append( prefix + "| brokerageLastName = " + brokerageLastName);
      buff.append( prefix + "| brokerageFirstName = " + brokerageFirstName);
      buff.append( prefix + "| brokerageMiddleName = " + brokerageMiddleName);
      buff.append( prefix + "| brokerageOfficeID = " + brokerageOfficeID);
      buff.append( prefix + "| brokerageInvCat = " + Integer.toString(brokerageInvCat));
      buff.append( prefix + "| brokerageEmail = " + brokerageEmail);

      buff.append( prefix + "| ifsInvAcctDtl[i] = ");             // Not displaying.
      buff.append( prefix + "| ifsCnsmrInfo[i] = ");              // Not displaying.
      buff.append( prefix + "| ifsCnsmrSvcProviderRules[i] = ");  // Not displaying.

      buff.append( prefix + "| secCnsmrMntAuth = " + Integer.toString(secCnsmrMntAuth));
      try { buff.append( prefix + "| secCnsmrMntAuth_text = " + getSecCnsmrMntAuth_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| secCnsmrMntAuth_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| lastPinTimestamp = " + lastPinTimestamp);
      buff.append( prefix + "| lastPinGMTOffset = " + Integer.toString(lastPinGMTOffset));
      buff.append( prefix + "| lastLogTimestamp = " + lastLogTimestamp);
      buff.append( prefix + "| lastLogGMTOffset = " + Integer.toString(lastLogGMTOffset));

      buff.append( prefix + "| fICnsmrStCode = " + Integer.toString(fICnsmrStCode));
      try { buff.append( prefix + "| fICnsmrStCode_text = " + getFICnsmrStCode_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| fICnsmrStCode_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| businessName = " + businessName);
      buff.append( prefix + "| busnIdCode = " + Integer.toString(busnIdCode));
      try { buff.append( prefix + "| busnIdCode_text = " + getBusnIdCode_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| busnIdCode_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| businessId = " + businessId);
      buff.append( prefix + "| altCnsmrLastName = " + altCnsmrLastName);
      buff.append( prefix + "| altCnsmrFirstName = " + altCnsmrFirstName);
      buff.append( prefix + "| altCnsmrMiddleName = " + altCnsmrMiddleName);
      buff.append( prefix + "| altCnsmrSuffix = " + altCnsmrSuffix);
      buff.append( prefix + "| queueQualifier = " + queueQualifier);
      buff.append( prefix + "| sLASampleFlag = " + sLASampleFlag);
      try { buff.append( prefix + "| sLASampleFlag_text = " + getSLASampleFlag_text()); }
      catch (IfsException e)
      {
         buff.append( prefix + "| sLASampleFlag_text throws IfsException: " + e.getMessage());
      }

      buff.append( prefix + "| tokenInformation = " + tokenInformation);

      if (registry.isEmpty())
      {
         buff.append(prefix + "| Registry is empty");
      }
      else
      {
         Enumeration keys = registry.keys();
         while (keys.hasMoreElements())
         {
            String key = (String) keys.nextElement();
            buff.append(prefix + "| Registry contains: ");
            buff.append(prefix + "| " + key + " = ");
            buff.append(registry.get(key).toString());
         }
      }
      return buff.toString();

   }

   /**
   * Validates the GtLogon object.
   * When a method is registered to <EM>validate</EM> for this class,
   * it is used.  Otherwise, a default validation is done.
   *
   * @return
   *    Returns "OK" if the GtLogon is valid.  Otherwise, it returns a message
   *    string indicating the error.
   */
   public String validate() {
      ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",getLocale());
      String str = super.validate();
      if (str == null)
      {
         if (userID == null || userID.length() > IfsSystem.MAX_USER_ID_LEN || userID.length() == 0 ||
             pin == null || pin.length > IfsSystem.MAX_PIN_LENGTH || pin.length == 0 ||
             passwordID == null || passwordID.length > IfsSystem.MAX_PASSWORD_LENGTH || passwordID.length == 0)
         {
            str = bundle.getString("GtLogon.validation.invalidInput");
         }
         else
            str = "OK";
      }
      return str;
   }
}
