//***************************************************************************
//
// FILE NAME:   GtOBOLogon.java
//
// COPYRIGHT:
//
//   Licensed Materials - Property of IBM
//   5648-A06
//   (C) Copyright IBM Corp. 2000 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//
//  CHANGES:
//    05/01/2000  rhs   d6673   Added changes for NLS.  Commented out the
//                              setters/getters for fields that were not being
//                              used. Added Javadoc comments.
//                              Corrected toString() and toString(prefix).
//    05/02/2000  rhs   d6704   Removed reqCountryCode and reqLanguageCode
//                              from addGoldToGMJI.
//    05/15/2000  rhs   d6759   To remove confusion, block comments were added at
//                      d6736   beginning of unused getters and setters statements.
//                              Added the obtaining of all the GMJO data, just so
//                              we can get at the cookie information at the end of
//                              GMJO.
//    05/18/2000  rhs   d6734   Corrected getReqPassword() method. It was encrypting
//                              the wrong value.
//    06/27/2001  tlt  T0987315 Corrected pin/pswd/userid len checks
//
//***************************************************************************

package com.ibm.ifs.gold;

import java.io.UnsupportedEncodingException;
import java.util.Locale;
import javax.servlet.http.Cookie;
import java.util.ResourceBundle;
import java.util.Hashtable;
import java.util.Enumeration;

import com.ibm.ifs.gold.auto.GsSrvcRqst;
import com.ibm.ifs.servlets.IfsHttpServlet;

/**
* GtOBOLogon class enables a Customer Service Representative (CSR) to logon
* On Behalf Of (OBO) a bank customer by establishing the web session and
* giving the CSR the ability to do a Session Start.
* The CSR has to execute the IFS Consumer Session Start transaction to actually
* do something on behalf of a customer.
*/
public class GtOBOLogon extends IfsTxn
{
   public static boolean debugOn=false;

   private static final String GMRequestId  = "obologon_java";
   private static final String MTILevel     = "99.01.02";
   public static final int SD_OWNER = 0;
   private static Hashtable registry = new Hashtable();
   public static final int SDThis = 0;

   // request message fields/setters/getters

   private byte[]  reqPassword = null;
   private boolean encryptPassword = false;
   private byte[]  reqSecurity = null;
   private boolean encryptSecurity = false;
   private String  reqPasswordEncryptionType = IfsSystem.NO_ENCRYPTION;
   private String  reqSecurityEncryptionType = IfsSystem.NO_ENCRYPTION;

   private Locale  locale             = IfsSystem.SERVER_LOCALE;

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   private String  reqCountryCode     = locale.getCountry();

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   private String  reqLanguageCode    = locale.getLanguage();

   /** Customer Service Representative (CSR) user id */
   private String  reqUserID          = null;

   // response message fields/setters/getters

   private Cookie  sessionKey         = null;
   private GsGmSelfDefiningFieldDef[]    respSelfDefData = null;

   private String  responseCode = null;
   private String  fIProcInd = null;
   private String  queueQualifier = null;
   private String  sLASampleFlag = null;
   private String  staffLastName = null;
   private String  staffFirstName = null;
   private String  staffMiddleName = null;
   private String  staffSuffix = null;
   private String  staffIGNID = null;
   private String  toolkitAccessType = null;
   private String  staffDepartmentName = null;
   private String  staffTitleCode = null;
   private String  staffSecurityType = null;
   private String  staffID = null;
   private String  companyID = null;
   private GsSrvcRqst[]    serviceRequest = null;
   private String  employeeStatus = null;
   private int     passwordAttempts = 0;
   private int     passwordRetryLimit = 0;

   /**
   * Default constructor.
   */
   public GtOBOLogon(){}

   /**
   * Constructor with an associated IfsSession object.
   */
   public GtOBOLogon(IfsSession sess)
   {
      setIfsSession(sess);
   }

   /**
   * Add OBOLogon gold message to Gold Message Java Input buffer.
   */
   public void addGoldToGMJI(int argFlags, String cacheTxnName )
   {
      // order in gmJavaOBOLogonReq

      addArgToGMJI(reqUserID);
      addArgToGMJI(getReqSecurity(IfsTxn.ENCRYPT));
      addArgToGMJI( reqSecurityEncryptionType );
      addArgToGMJI(getReqPassword(IfsTxn.ENCRYPT));
      addArgToGMJI( reqPasswordEncryptionType );

      reqSelfDef_packed = GsGmSelfDefiningFieldDef.pack( reqSelfDef);
      addArgToGMJI( reqSelfDef_packed );
   }

   /**
   * Executes the GtOBOLogon transaction by sending it to the
   * Financial Instition (FI) and the response is not saved in the Gold Manager
   * transaction cache.
   * The transaction execution mode is controlled by the IfsTxn object executionMode
   * attribute.
   *
   * @exception IfsException When execution fails
   */
   public void execute() throws IfsException
   {
      setContainsRespCode(IfsTxn.RETURNS_RESPONSE_CODE);
      IfsSession sess = getIfsSession();

      if ( IfsSystem.browserLocaleUsage == IfsSystem.USE_SERVER_LOCALE )
      {
         try {sess.setLocale(IfsSystem.SERVER_LOCALE );}
         catch (IfsException e) { }
      }
      else
         IfsHttpServlet.checkBrowserLocale(sess.getHttpServletRequest() );

      oboLogonTxn = true;
      super.execute();

      if (status==OK)
      {
         sess.setSessionKey(getSessionKey());
      }
   }

   /**
   * Get OBOLogon gold message from Gold Message Java Output buffer.
   */
   public void getGoldFromGMJO()
   {
      int repeatCounter;

      // Even though the response data from the GtOBOLogon message is not to be used,
      // it still has to be read in because the session key (cookie) at the end
      // of the GMJO is required.  Since the response data is not expected to be used
      // the getters/setters for these fields have been commented out.
      //
      responseCode             = getNextCharFromGMJO( 3,"responseCode" );
      fIProcInd                = getNextCharFromGMJO( 1,"fIProcInd" );
      queueQualifier           = getNextCharFromGMJO( 8,"queueQualifier" );
      sLASampleFlag            = getNextCharFromGMJO( 1,"sLASampleFlag" );
      staffLastName            = getNextNlsCharFromGMJO( 30,"staffLastName" );
      staffFirstName           = getNextNlsCharFromGMJO( 30,"staffFirstName" );
      staffMiddleName          = getNextNlsCharFromGMJO( 20,"staffMiddleName" );
      staffSuffix              = getNextNlsCharFromGMJO( 8,"staffSuffix" );
      staffIGNID               = getNextCharFromGMJO( 8,"staffIGNID" );
      toolkitAccessType        = getNextCharFromGMJO( 3,"toolkitAccessType" );
      staffDepartmentName      = getNextNlsCharFromGMJO( 40,"staffDepartmentName" );
      staffTitleCode           = getNextNlsCharFromGMJO( 3,"staffTitleCode" );
      staffSecurityType        = getNextCharFromGMJO( 8,"staffSecurityType" );
      staffID                  = getNextCharFromGMJO( 30,"staffID" );
      companyID                = getNextCharFromGMJO( 10,"companyID" );

      repeatCounter = getNextLongFromGMJO( "repeatCounter" );
      serviceRequest = new GsSrvcRqst[repeatCounter];
      for (int i = 0; i < repeatCounter; i++)
      {
        serviceRequest[i]                        = new GsSrvcRqst(this);
      }

      employeeStatus            = getNextCharFromGMJO( 1,"employeeStatus" );
      passwordAttempts          = getNextLongFromGMJO( "passwordAttempts" );
      passwordRetryLimit        = getNextLongFromGMJO( "passwordRetryLimit" );

      addSelfDefProperties();
      sessionKey                = getNextCookieFromGMJO("cookieName", "cookieValue");

   }

   //------------------
   // Start of getters
   //------------------

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   public  String  getReqLanguageCode() {return(reqLanguageCode);}

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   public  String  getReqCountryCode() {return(reqCountryCode);}
   public  String  getGMRequestId()    {return(GMRequestId);}
   public  String  getMTILevel()       {return(MTILevel);}
   public  int     getSD_OWNER() { return( SD_OWNER );}
   public  Cookie  getSessionKey() {return sessionKey;}
   public static Hashtable getRegistry() {return registry;}
   public  byte[]  getReqPassword() {return(reqPassword);}
   private byte[]  getReqPassword(boolean format)
   {
      if (true==format)
      {
         if (encryptPassword && (IfsSystem.RSA_ENCRYPTION.equals(reqPasswordEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(reqPassword, IfsSystem.RSA_ENCRYPTION);
            debug("IfsSecurity.getReturnCode() value is , " + security.getReturnCode());
            if (security.isValid())
               return security.getEncodedValue();

            else
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;
               return security.getEncodedValue();
            }
         }
      }
      return reqPassword;
   }
   public  String  getReqPasswordEncryptionType() {return(reqPasswordEncryptionType);}
   public  byte[]  getReqSecurity() {return(reqSecurity);}
   private byte[]  getReqSecurity(boolean format)
   {
      if (true==format)
      {
         if (encryptSecurity && (IfsSystem.RSA_ENCRYPTION.equals(reqSecurityEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(reqSecurity, IfsSystem.RSA_ENCRYPTION);
            debug("IfsSecurity.getReturnCode() value is , " + security.getReturnCode());
            if (security.isValid())
               return security.getEncodedValue();

            else
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;
               return security.getEncodedValue();
            }
         }
      }
      return reqSecurity;
   }
   public  String  getReqSecurityEncryptionType() {return(reqSecurityEncryptionType);}

   /**
   * Getter for reqUserID field.
   * It contains the Customer Service Represenative (CSR) user identifier.
   */
   public  String  getReqUserID() {return(reqUserID);}
   public GsGmSelfDefiningFieldDef[]    getRespSelfDefData() {return(respSelfDefData);}


   //-------------------------
   // The profile data returned by the oboLogon txn is currently not used by the
   // CWS for any reason. The OBO_LOGON_TXN returns a cstruct called IFS_STAFF_PRFL_ITEM.
   // This contains profile data specific to the CSR representative that is  logging on.
   // Transactions involving the CSR profile are done via the CTW workstation application
   // (e.g. GET_FISTAFF_PRFL_TXN - Staff Profile Request Transaction), not through the CWS.
   //
   // When the CSR representative is logged on, he/she can issue an IFS Consumer Session
   // Start Transaction to log on as a particular user.
   // Once this is done, an  FI Consumer Profile Inquiry Transaction (USER_PROFILE_TXN)
   // can be issued to obtain the customer's profile.
   //
   // For the above reasons, the following getters are not needed and have been commented
   // out.  The have not been removed them because, we might need them for debug purposes.
   //
   // If the _text getters are ever required, they need to be updated for NLS.  They are
   // left here as a reminder that these fields do have code-to-text values.
   //
   //public  String  getQueueQualifier() {return(queueQualifier);}
   //public  String  getCompanyID() {return(companyID);}
   //public  String  getCompanyID_text() {return(companyID_text);}
   //public  String  getEmployeeStatus() {return(employeeStatus);}
   //public  String  getEmployeeStatus_text() {return(employeeStatus_text);}

   //public  String  getFIProcInd() {return(fIProcInd);}
   //public  String  getFIProcInd_text() {return(fIProcInd_text);}
   //public  String  getResponseCode() {return(responseCode);}
   //public  String  getResponseCode_text() {return(responseCode_text);}

   //public GsSrvcRqst[]    getServiceRequest() {return(serviceRequest);}
   //public  String  getSLASampleFlag() {return(sLASampleFlag);}
   //public  String  getSLASampleFlag_text() {return(sLASampleFlag_text);}
   //public  String  getStaffDepartmentName() {return(staffDepartmentName);}
   //public  String  getStaffFirstName() {return(staffFirstName);}
   //public  String  getStaffID() {return(staffID);}
   //public  String  getStaffIGNID() {return(staffIGNID);}
   //public  String  getStaffLastName() {return(staffLastName);}
   //public  String  getStaffMiddleName() {return(staffMiddleName);}
   //public  String  getStaffSecurityType() {return(staffSecurityType);}
   //public  String  getStaffSecurityType_text() {return(staffSecurityType_text);}
   //public  String  getStaffSuffix() {return(staffSuffix);}
   //public  String  getStaffTitleCode() {return(staffTitleCode);}
   //public  String  getStaffTitleCode_text() {return(staffTitleCode_text);}
   //public  String  getToolkitAccessType() {return(toolkitAccessType);}
   //public  String  getToolkitAccessType_text() {return(toolkitAccessType_text);}

   //public  int     getPasswordAttempts() {return(passwordAttempts);}
   //public  int     getPasswordRetryLimit() {return(passwordRetryLimit);}

   //------------------
   // Start of setters
   //------------------

   public  void      setRespSelfDefData(GsGmSelfDefiningFieldDef[] o) {respSelfDefData = o;}

   /**
   * Setter for reqUserID field.
   * It contains the Customer Service Represenative (CSR) user identifier.
   */
   public  void      setReqUserID(String o) {reqUserID = o;}

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   public  void      setReqCountryCode(String o) {reqCountryCode = o;}

   /** @deprecated  as of V1R6. Country Code and language Code
   * are now set dynamically according to the current locale
   * object contained by this transaction. This method is not
   * replaced by any new method.
   */
   public  void      setReqLanguageCode(String o) {reqLanguageCode = o;}

   public void     setReqPassword(byte[] o){reqPassword = o;encryptPassword=false;}
   public void setReqPassword(String o)
   {
      try
      {
         reqPassword = o.getBytes(IfsSystem.cwsGMJCharSet);
         encryptPassword = true;
      }
      catch (UnsupportedEncodingException e)
      {
         reqPassword = null;
         encryptPassword = false;
         debug("setReqPassword failure in GtOBOLogon, exception is -- " + e.toString() );
      }
   }
   public  void    setReqPasswordEncryptionType(String o) {reqPasswordEncryptionType = o;}

   public void setReqSecurity(byte[] o){reqSecurity = o;encryptSecurity=false;}
   public void setReqSecurity(String o)
   {
      try
      {
         reqSecurity = o.getBytes(IfsSystem.cwsGMJCharSet);
         encryptSecurity = true;
      }
      catch (UnsupportedEncodingException e)
      {
         reqSecurity = null;
         encryptSecurity = false;
         debug("setReqSecurity failure in GtOBOLogon, exception is -- " + e.toString() );
      }
   }
   public  void    setReqSecurityEncryptionType(String o) {reqSecurityEncryptionType = o;}

   //-------------------------
   // The profile data returned by the oboLogon txn is currently not used by the
   // CWS for any reason. The OBO_LOGON_TXN returns a cstruct called IFS_STAFF_PRFL_ITEM.
   // This contains profile data specific to the CSR representative that is  logging on.
   // Transactions involving the CSR profile are done via the CTW workstation application
   // (e.g. GET_FISTAFF_PRFL_TXN - Staff Profile Request Transaction), not through the CWS.
   //
   // When the CSR representative is logged on, he/she can issue an IFS Consumer Session
   // Start Transaction to log on as a particular user.
   // Once this is done, an  FI Consumer Profile Inquiry Transaction (USER_PROFILE_TXN)
   // can be issued to obtain the customer's profile.
   //
   // For the above reasons, the following setters are not needed and have been commented
   // out.  The have not been removed them because, we might need them for debug purposes.
   //
   //public  void    setResponseCode(String o) {responseCode = o;}
   //public  void    setFIProcInd(String o) {fIProcInd = o;}
   //public  void    setQueueQualifier(String o) {queueQualifier = o;}
   //public  void    setServiceRequest(GsSrvcRqst[] o) {serviceRequest = o;}
   //public  void    setSLASampleFlag(String o) {sLASampleFlag = o;}
   //public  void    setStaffDepartmentName(String o) {staffDepartmentName = o;}
   //public  void    setStaffFirstName(String o) {staffFirstName = o;}
   //public  void    setStaffID(String o) {staffID = o;}
   //public  void    setStaffIGNID(String o) {staffIGNID = o;}
   //public  void    setStaffLastName(String o) {staffLastName = o;}
   //public  void    setStaffMiddleName(String o) {staffMiddleName = o;}
   //public  void    setStaffSecurityType(String o) {staffSecurityType = o;}
   //public  void    setStaffSuffix(String o) {staffSuffix = o;}
   //public  void    setStaffTitleCode(String o) {staffTitleCode = o;}
   //public  void    setToolkitAccessType(String o) {toolkitAccessType = o;}

   //public  void    setCompanyID(String o) {companyID = o;}
   //public  void    setEmployeeStatus(String o) {employeeStatus = o;}
   //public  void    setPasswordAttempts(int o) {passwordAttempts = o;}
   //public  void    setPasswordRetryLimit(int o) {passwordRetryLimit = o;}


   /**
   * Converts the GtOBOLogon object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      if (hasRegisteredMethod("toString"))
         return super.toString();
      else
         return toString("\n");
   }

   /**
   * Converts the GtOBOLogon object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix)
   {
      if (hasRegisteredMethod("toStringWithString"))
         return super.toString(prefix);

      Cookie cookie = getIfsSession().getSessionKey();
      StringBuffer buff = new StringBuffer(250);
      buff.append(prefix);
      buff.append(super.toString(prefix));

      buff.append(prefix + "| reqUserID = " + (new String(reqUserID)));

      try { buff.append(prefix + "| reqSecurity = " + (new String(reqSecurity)));}
      catch (Throwable e)
      {
         buff.append(prefix + "| reqSecurity causes exception: " + e.toString() );
      }
      buff.append(prefix + "| reqSecurityEncryptionType = " + reqSecurityEncryptionType);

      try { buff.append(prefix + "| reqPassword = " + (new String(reqPassword))); }
      catch (Throwable e)
      {
         buff.append(prefix + "| reqPassword causes exception: " + e.toString() );
      }
      buff.append(prefix + "| reqPasswordEncryptionType = " + reqPasswordEncryptionType);

      buff.append(prefix + "| SessionKey.Name = " + cookie.getName());
      buff.append(prefix + "| SessionKey.Value = " + cookie.getValue());

      buff.append(prefix + "| BRC = " + getBrc());
      buff.append(prefix + "| Status = " + getStatus());
      buff.append(prefix + "| Message = " + getMessage());

      buff.append(prefix);
      if (registry.isEmpty())
      {
         buff.append("| Registry is empty");
      }
      else
      {
         Enumeration keys = registry.keys();
         while (keys.hasMoreElements())
         {
            String key = (String) keys.nextElement();

            buff.append(prefix);
            buff.append("| Registry contains: ");

            buff.append("| ");
            buff.append(key);
            buff.append(" = ");
            buff.append(registry.get(key).toString());
         }
      }
      return buff.toString();

   }

   /**
   * Validates the GtOBOLogon object.
   * When a method is registered to <EM>validate</EM> for this class,
   * it is used.  Otherwise, a default validation is done.
   *
   * @return
   *    Returns "OK" if the GtOBOLogon is valid.  Otherwise, it returns a message
   *    string indicating the error.
   */
   public String validate()
   {
      ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",getLocale());
      String str = super.validate();
      if (str == null)
      {
         if (reqUserID == null || reqUserID.length() > IfsSystem.MAX_USER_ID_LEN || reqUserID.length() == 0 ||
             reqPassword == null || reqPassword.length > IfsSystem.MAX_PASSWORD_LENGTH || reqPassword.length == 0)
         {
            str = "Validation failed: invalid input";
         }
         else
            str = "OK";
      }
      return str;
   }

}
