package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   GtReauthentication.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    02/16/2000  rhs   d6361   Made JavaDoc correction/addtions.
//    03/15/2000  rhs   d6545   Added NLS support
//    04/28/2000  rhs   d6688   Add addSelfDefProperties to getGoldFromGMJO.
//
// ***************************************************************************

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.Cookie;
import com.ibm.ifs.gold.IfsNLS;

public class GtReauthentication extends IfsTxn
{
   private static String GMRequestId = "reauth_java";
   public static final int SD_OWNER = 0;
   public static final int SDThis = 0;
   private static Hashtable registry = new Hashtable();

   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean debugOn = false;

   // request fields
   private byte[] pin = null;
   private byte[]  getPin(boolean format)
   {
      if (true==format)
      {
         if (encryptPin && (IfsSystem.RSA_ENCRYPTION.equals(pinEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(pin, IfsSystem.RSA_ENCRYPTION);
            if (!(security.isValid()))
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;
            }
            return security.getEncodedValue();
         }
      }
      return pin;
   }
   private boolean encryptPin = false;
   private String pinEncryptionType = IfsSystem.RSA_ENCRYPTION;
   private byte[] password = null;
   private byte[]  getPassword(boolean format) {
      if (true==format)
      {
         if (encryptPassword && (IfsSystem.RSA_ENCRYPTION.equals(passwordEncryptionType)))
         {
            IfsSecurity security = IfsSecurity.encryptData(password, IfsSystem.RSA_ENCRYPTION);
            if (!(security.isValid()))
            {
               message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
               status = ENCRYPTION_FAILURE_ERROR;
            }
            return security.getEncodedValue();
         }
      }
      return password;
   }
   private boolean encryptPassword = false;
   private String passwordEncryptionType = IfsSystem.RSA_ENCRYPTION;

   // response fields
   private String responseCode = null;
   private String fIProcInd = null;

   /**
   * Default Constructor
   */
   public GtReauthentication(){}

   /**
   * Constructor with an associated IfsSession object.
   */
   public GtReauthentication(IfsSession session) {
      setIfsSession(session);
   }

   /**
   * Add Reauthentication gold message to Gold Message Java Input buffer.
   */
   public void addGoldToGMJI(int argFlags, String cacheTxnName) {
      addArgToGMJI(getPassword(IfsTxn.ENCRYPT));
      addArgToGMJI(getPin(IfsTxn.ENCRYPT));
      addArgToGMJI(passwordEncryptionType);
      addArgToGMJI(pinEncryptionType);
      reqSelfDef_packed = GsGmSelfDefiningFieldDef.pack(reqSelfDef);
      addArgToGMJI(reqSelfDef_packed);

      return;
   }

   /**
   * Executes the GtReauthentication transaction by sending it to the
   * Financial Instition (FI) and the response is not saved in the Gold Manager
   * transaction cache.
   * The transaction execution mode is controlled by the IfsTxn object executionMode
   * attribute.
   *
   * @exception IfsException When execution fails
   */
   public void execute() throws IfsException {

      setContainsRespCode(IfsTxn.RETURNS_RESPONSE_CODE);
      reauthTxn = true;
      super.execute();

   }

   public String getFIProcInd() {
      return fIProcInd;
   }

   public String getFIProcIndText() throws IfsException
   {
      return IfsNLS.getText("IfsFiProcIndResources",
                            getIfsSession().getLocale(),
                            getFIProcInd());
   }

   public String getGMRequestId() {
      return(GMRequestId);
   }

   /**
   * Get Reauthentication gold message from Gold Message Java Output buffer.
   */
   public void getGoldFromGMJO() {
      responseCode = getNextCharFromGMJO(3,"responseCode");
      fIProcInd = getNextCharFromGMJO(1,"fIProcInd");

      addSelfDefProperties();

      return;
   }

   /**
   * Returns a Hashtable object which contains the list of registered methods.
   */
   public static Hashtable getRegistry() {
      return registry;
   }
   public String getResponseCode() {return responseCode;}
   public String getResponseCode_text() throws IfsException
   {
      return IfsNLS.getText("IfsRespCodeClResources",
                            getIfsSession().getLocale(),
                            getResponseCode());
   }

   public String getResponseCodeText() throws IfsException
   {
      return IfsNLS.getText("IfsRespCodeClResources",
                            getIfsSession().getLocale(),
                            getResponseCode());
   }

   public int getSD_OWNER() {
      return( SD_OWNER );
   }

   public void setFIProcInd(String fIProcInd) {
      this.fIProcInd = fIProcInd;
      return;
   }

   public void setPassword(byte[] password) {
      this.password = password;
      encryptPassword = false;
      return;
   }
   public void setPassword(String password)
   {
      try
      {
         this.password = password.getBytes(IfsSystem.cwsGMJCharSet);
         encryptPassword = true;
      }
      catch (UnsupportedEncodingException e)
      {
         this.password = null;
         encryptPassword = false;
         debug("setPassword failure in GtReauthentication, exception is -- " + e.toString() );
      }
   }
   public void setPasswordEncryptionType(String passwordType) {
      passwordEncryptionType = passwordType;
      return;
   }
   public void setPin(byte[] pin) {
      this.pin = pin;
      encryptPin = false;
      return;
   }
   public void setPin(String pin)
   {
      try
      {
         this.pin = pin.getBytes(IfsSystem.cwsGMJCharSet);
         encryptPin = true;
      }
      catch (UnsupportedEncodingException e)
      {
         this.pin = null;
         encryptPin = false;
         debug("setPin failure in GtReauthentication, exception is -- " + e.toString() );
      }
   }
   public void setPinEncryptionType(String pinType) {
      pinEncryptionType = pinType;
      return;
   }
   public void setResponseCode(String code) {responseCode=code;}

   /**
   * Converts the GtReauthentication object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      if (hasRegisteredMethod("toString"))
         return super.toString();

      return toString("\n");
   }

   /**
   * Converts the GtReauthentication object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix)
   {
      if (hasRegisteredMethod("toStringWithString"))
         return super.toString(prefix);

      Cookie cookie = getIfsSession().getSessionKey();
      StringBuffer buff = new StringBuffer(250);
      buff.append(super.toString(prefix));
      buff.append(prefix);
      buff.append("| Pin = ");
      buff.append(new String(pin));
      buff.append(prefix);
      buff.append("| PinEncryptionType = ");
      buff.append(pinEncryptionType);
      buff.append(prefix);
      buff.append("| Password = ");
      buff.append(new String(password));
      buff.append(prefix);
      buff.append("| PasswordEncryptionType = ");
      buff.append(passwordEncryptionType);
      buff.append(prefix);
      buff.append("| SessionKey.Name = ");
      buff.append(cookie.getName());
      buff.append(prefix);
      buff.append("| SessionKey.Value = ");
      buff.append(cookie.getValue());
      buff.append(prefix);
      buff.append("| BRC = ");
      buff.append(getBrc());
      buff.append(prefix);
      buff.append("| Status = ");
      buff.append(getStatus());
      buff.append(prefix);
      buff.append("| Message = ");
      buff.append(getMessage());

      buff.append(prefix + "| responseCode = " + responseCode);
      try { buff.append(prefix + "| responseCode_text = " + getResponseCode_text()); }
      catch ( IfsException e)
      {
         buff.append( prefix + "| responseCode_text throws IfsException: " + e.getMessage());
      }

      buff.append(prefix + "| fIProcInd = " + fIProcInd);
      try { buff.append(prefix + "| fIProcInd_text = " + getFIProcIndText() ); }
      catch ( IfsException e)
      {
         buff.append( prefix + "| fIProcInd_text throws IfsException: " + e.getMessage());
      }

      if (registry.isEmpty())
      {
         buff.append(prefix);
         buff.append("| Registry is empty");

      }
      else
      {
         Enumeration keys = registry.keys();
         while (keys.hasMoreElements())
         {
            String key = (String) keys.nextElement();
            buff.append(prefix);
            buff.append("| Registry contains: ");
            buff.append(prefix);
            buff.append("| ");
            buff.append(key);
            buff.append(" = ");
            buff.append(registry.get(key).toString());
         }
      }
      return buff.toString();

   }

   /**
   * Validates the GtReauthentication object.
   * When a method is registered to <EM>validate</EM> for this class,
   * it is used.  Otherwise, a default validation is done.
   *
   * @return
   *    Returns "OK" if the GtReauthentication is valid.  Otherwise, it returns a message
   *    string indicating the error.
   */
   public String validate()
   {
      ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",
                                                       getLocale());
      String result = super.validate();
      if (null==result)
      {
         if (null!=pin && !pin.equals(""))
         {
            if (pin.length > IfsSystem.MAX_PIN_LENGTH)
               return bundle.getString("GtReauthorization.validation.PinTooBig");

            else if (null!=password && !password.equals(""))
            {
               if (password.length <= IfsSystem.MAX_PASSWORD_LENGTH)
                  return "OK"; //needs to be OK and not translatable.

               else
                  return bundle.getString("GtReauthorization.validation.PassTooBig");
            }
            else
               return bundle.getString("GtReauthorization.validation.noPass");
         }
         return bundle.getString("GtReauthorization.validation.noPin");

      }
      else
         return result;
   }

}

