//***************************************************************************
//
// FILE NAME:   GtSmallBusWireTransferInit.java
//
// COPYRIGHT:
//
//   Licensed Materials - Property of IBM
//   5648-A06
//   (C) Copyright IBM Corp. 1996, 2000 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//
//  CHANGES:
//    05/03/2000  rhs   d6662   Readded _text fields to toString or showObject.
//    05/25/2000  rhs   d6837   Added use of constructor which takes an IfsSession
//                              when new Gs* objects are created.
//
//***************************************************************************

package com.ibm.ifs.gold;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.ifs.gold.auto.*;

public class GtSmallBusWireTransferInit extends IfsTxn
{
   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean debugOn=false;

   /**
   * Default constructor.
   */
   public GtSmallBusWireTransferInit(){}

   /**
   * Constructor with an associated IfsSession object.
   */
   public GtSmallBusWireTransferInit(IfsSession sess) {
      setIfsSession(sess);
   }
   private static final String GMRequestId             = "wtfrinit_java";
   public  String getGMRequestId()                 {return(GMRequestId);}

   private static final String MTILevel                = "99.01.02";
   public  String getMTILevel()                    {return(MTILevel);}

   public static final int SD_OWNER = 0;
   public int getSD_OWNER() { return( SD_OWNER );}
   private static Hashtable registry = new Hashtable();
   public static Hashtable getRegistry() {return registry;}

   public static final int SDThis = 0;
   int i = 0;

// request message fields/setters/getters

   private boolean encryptConsumerIDData = false;

   private String consumerIDDataEncryptionType = IfsSystem.RSA_ENCRYPTION;
   public String getConsumerIDDataEncryptionType() {return consumerIDDataEncryptionType;}
   public void setConsumerIDDataEncryptionType(String o) {consumerIDDataEncryptionType = o;}

   private GsCnsmrDataId[] reqConsumerIDData = null;
   public  void      setReqConsumerIDData(GsCnsmrDataId[] o) {reqConsumerIDData = o;encryptConsumerIDData = false;}
   public  void      setReqConsumerIDData(String[] o) {
      reqConsumerIDData = new GsCnsmrDataId[o.length];
      for (i=0; i<o.length; i++)
      {
         reqConsumerIDData[i] = new GsCnsmrDataId(getIfsSession());
         try
         {
            reqConsumerIDData[i].setConsumerDataIDItem(o[i].getBytes(IfsSystem.cwsGMJCharSet));
         }
         catch (UnsupportedEncodingException e)
         {
            debug("Exception caught in the statement \"reqConsumerIDData[i].setConsumerDataIDItem(o[i].getBytes(IfsSystem.cwsGMJCharSet))\", exception is: "+
                  e.toString());
         }
         reqConsumerIDData[i].setConsumerDataEncryptionType(consumerIDDataEncryptionType);
      }
      encryptConsumerIDData = true;
   }
   public GsCnsmrDataId[]    getReqConsumerIDData() {return(reqConsumerIDData);}
   private GsCnsmrDataId[]   getReqConsumerIDData(boolean format) {
      if (true==format && null!=reqConsumerIDData)
      {
         if (encryptConsumerIDData && (IfsSystem.RSA_ENCRYPTION.equals(consumerIDDataEncryptionType)))
         {
            for (i=0; i<reqConsumerIDData.length; i++)
            {
               IfsSecurity security = IfsSecurity.encryptData(new String(reqConsumerIDData[i].getConsumerDataIDItem()), IfsSystem.RSA_ENCRYPTION);
               if (!security.isValid())
               {
                  message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
                  status = ENCRYPTION_FAILURE_ERROR;
                  return reqConsumerIDData;

               }
               reqConsumerIDData[i].setConsumerDataIDItem(security.getEncodedValue());
               reqConsumerIDData[i].setConsumerDataEncryptionType(IfsSystem.RSA_ENCRYPTION);
            }
            encryptConsumerIDData = false;
         }
      }
      return reqConsumerIDData;
   }

   private String        reqConsumerIDData_packed = null;
   public  void          setReqConsumerIDData_packed( String o ) {reqConsumerIDData_packed = o;}
   public  String        getReqConsumerIDData_packed() {return(reqConsumerIDData_packed);}

   private String  reqWireTransferReferenceNumber = null;
   public  void    setReqWireTransferReferenceNumber(String o) {reqWireTransferReferenceNumber = o;}
   public  String  getReqWireTransferReferenceNumber() {return(reqWireTransferReferenceNumber);}

   private String  reqWireTransferNickname = null;
   public  void    setReqWireTransferNickname(String o) {reqWireTransferNickname = o;}
   public  String  getReqWireTransferNickname() {return(reqWireTransferNickname);}

   private GsCurrency reqWireTransferAmount = null;
   public  void         setReqWireTransferAmount(GsCurrency o) {reqWireTransferAmount = o;}
   public  GsCurrency getReqWireTransferAmount() {return(reqWireTransferAmount);}

   private GsDate reqWireTransferEffectiveDate = null;
   public  void         setReqWireTransferEffectiveDate(GsDate o) {reqWireTransferEffectiveDate = o;}
   public  GsDate getReqWireTransferEffectiveDate() {return(reqWireTransferEffectiveDate);}

   private String  reqConsumerInstructionText = null;
   public  void    setReqConsumerInstructionText(String o) {reqConsumerInstructionText = o;}
   public  String  getReqConsumerInstructionText() {return(reqConsumerInstructionText);}

   private String  reqHomePhone = null;
   public  void    setReqHomePhone(String o) {reqHomePhone = o;}
   public  String  getReqHomePhone() {return(reqHomePhone);}

   private String  reqWorkPhone = null;
   public  void    setReqWorkPhone(String o) {reqWorkPhone = o;}
   public  String  getReqWorkPhone() {return(reqWorkPhone);}

   private String  reqEmailAddress = null;
   public  void    setReqEmailAddress(String o) {reqEmailAddress = o;}
   public  String  getReqEmailAddress() {return(reqEmailAddress);}

   private String  reqFaxNumber = null;
   public  void    setReqFaxNumber(String o) {reqFaxNumber = o;}
   public  String  getReqFaxNumber() {return(reqFaxNumber);}

   private int     reqResponsePath = 0;
   public  void    setReqResponsePath(int o) {reqResponsePath = o;}
   public  int     getReqResponsePath() {return(reqResponsePath);}

   private int     reqContactRequired = 0;
   public  void    setReqContactRequired(int o) {reqContactRequired = o;}
   public  int     getReqContactRequired() {return(reqContactRequired);}

   private String  reqContactOptional = null;
   public  void    setReqContactOptional(String o) {reqContactOptional = o;}
   public  String  getReqContactOptional() {return(reqContactOptional);}

   private String  reqStartTimeValue = null;
   public  void    setReqStartTimeValue(String o) {reqStartTimeValue = o;}
   public  String  getReqStartTimeValue() {return(reqStartTimeValue);}

   private String  reqStartTimeFormat = null;
   public  void    setReqStartTimeFormat(String o) {reqStartTimeFormat = o;}
   public  String  getReqStartTimeFormat() {return(reqStartTimeFormat);}

   private int     reqStartTimeGMTOffsetMinutes = 0;
   public  void    setReqStartTimeGMTOffsetMinutes(int o) {reqStartTimeGMTOffsetMinutes = o;}
   public  int     getReqStartTimeGMTOffsetMinutes() {return(reqStartTimeGMTOffsetMinutes);}

   private String  reqEndTimeValue = null;
   public  void    setReqEndTimeValue(String o) {reqEndTimeValue = o;}
   public  String  getReqEndTimeValue() {return(reqEndTimeValue);}

   private String  reqEndTimeFormat = null;
   public  void    setReqEndTimeFormat(String o) {reqEndTimeFormat = o;}
   public  String  getReqEndTimeFormat() {return(reqEndTimeFormat);}

   private int     reqEndTimeGMTOffsetMinutes = 0;
   public  void    setReqEndTimeGMTOffsetMinutes(int o) {reqEndTimeGMTOffsetMinutes = o;}
   public  int     getReqEndTimeGMTOffsetMinutes() {return(reqEndTimeGMTOffsetMinutes);}

   private String  reqFreeFormText = null;
   public  void    setReqFreeFormText(String o) {reqFreeFormText = o;}
   public  String  getReqFreeFormText() {return(reqFreeFormText);}

   private String  reqAddressLine1 = null;
   public  void    setReqAddressLine1(String o) {reqAddressLine1 = o;}
   public  String  getReqAddressLine1() {return(reqAddressLine1);}

   private String  reqAddressLine2 = null;
   public  void    setReqAddressLine2(String o) {reqAddressLine2 = o;}
   public  String  getReqAddressLine2() {return(reqAddressLine2);}

   private String  reqAddressLine3 = null;
   public  void    setReqAddressLine3(String o) {reqAddressLine3 = o;}
   public  String  getReqAddressLine3() {return(reqAddressLine3);}

   private String  reqCity = null;
   public  void    setReqCity(String o) {reqCity = o;}
   public  String  getReqCity() {return(reqCity);}

   private String  reqProvince = null;
   public  void    setReqProvince(String o) {reqProvince = o;}
   public  String  getReqProvince() {return(reqProvince);}

   private String  reqStateCode = null;
   public  void    setReqStateCode(String o) {reqStateCode = o;}
   public  String  getReqStateCode() {return(reqStateCode);}

   private String  reqCountryCode = null;
   public  void    setReqCountryCode(String o) {reqCountryCode = o;}
   public  String  getReqCountryCode() {return(reqCountryCode);}

   private String  reqZipCode = null;
   public  void    setReqZipCode(String o) {reqZipCode = o;}
   public  String  getReqZipCode() {return(reqZipCode);}

   private String  reqPostalCode = null;
   public  void    setReqPostalCode(String o) {reqPostalCode = o;}
   public  String  getReqPostalCode() {return(reqPostalCode);}



// response message fields/setters/getters

   private String  responseCode = null;
   public  void    setResponseCode(String o) {responseCode = o;}
   public  String  getResponseCode() {return(responseCode);}
   public  String  getResponseCode_text() throws IfsException
   {
      return IfsNLS.getText("IfsRespCodeClResources",getIfsSession().getLocale(),getResponseCode());
   }

   private String  referenceNumber = null;
   public  void    setReferenceNumber(String o) {referenceNumber = o;}
   public  String  getReferenceNumber() {return(referenceNumber);}

   private String  serviceProviderReference = null;
   public  void    setServiceProviderReference(String o) {serviceProviderReference = o;}
   public  String  getServiceProviderReference() {return(serviceProviderReference);}

   private String  fIProcInd = null;
   public  void    setFIProcInd(String o) {fIProcInd = o;}
   public  String  getFIProcInd() {return(fIProcInd);}
   public  String  getFIProcInd_text() throws IfsException
   {
      return IfsNLS.getText("IfsFiProcIndResources",getIfsSession().getLocale(),getFIProcInd());
   }

   /**
   * Add GtSmallBusWireTransferInit gold message to Gold Message Java Input buffer.
   */
   public void addGoldToGMJI(int argFlags, String cacheTxnName )
   {
      addArgToGMJI( com.ibm.ifs.gold.auto.GMDGlobals.GtSmallBusWireTransferInit );
      addArgToGMJI( argFlags );
      addArgToGMJI( cacheTxnName );
      addArgToGMJI();


      if ( reqConsumerIDData != null)
      {
         reqConsumerIDData_packed = GsCnsmrDataId.pack( getReqConsumerIDData(IfsTxn.ENCRYPT));
         addArgToGMJI( reqConsumerIDData_packed );
      }
      else
      {
         addArgToGMJI();
      }
      addArgToGMJI( reqWireTransferReferenceNumber );
      addArgToGMJI( reqWireTransferNickname );
      GsCurrency.addPropertiesToGMJI( reqWireTransferAmount , this );
      GsDate.addPropertiesToGMJI( reqWireTransferEffectiveDate , this );
      addArgToGMJI( reqConsumerInstructionText );
      addArgToGMJI( reqHomePhone );
      addArgToGMJI( reqWorkPhone );
      addArgToGMJI( reqEmailAddress );
      addArgToGMJI( reqFaxNumber );
      addArgToGMJI( reqResponsePath );
      addArgToGMJI( reqContactRequired );
      addArgToGMJI( reqContactOptional );
      addArgToGMJI( reqStartTimeValue );
      addArgToGMJI( reqStartTimeFormat );
      addArgToGMJI( reqStartTimeGMTOffsetMinutes );
      addArgToGMJI( reqEndTimeValue );
      addArgToGMJI( reqEndTimeFormat );
      addArgToGMJI( reqEndTimeGMTOffsetMinutes );
      addArgToGMJI( reqFreeFormText );
      addArgToGMJI( reqAddressLine1 );
      addArgToGMJI( reqAddressLine2 );
      addArgToGMJI( reqAddressLine3 );
      addArgToGMJI( reqCity );
      addArgToGMJI( reqProvince );
      addArgToGMJI( reqStateCode );
      addArgToGMJI( reqCountryCode );
      addArgToGMJI( reqZipCode );
      addArgToGMJI( reqPostalCode );

      reqSelfDef_packed = GsGmSelfDefiningFieldDef.pack( reqSelfDef);
      addArgToGMJI( reqSelfDef_packed );

   }

   /**
   * Get GtSmallBusWireTransferInit gold message from Gold Message
   * Java Output buffer.
   */
   public void getGoldFromGMJO()
   {
      int repeatCounter;

      responseCode                       = getNextCharFromGMJO( 3,"responseCode" );
      referenceNumber                    = getNextCharFromGMJO( 12,"referenceNumber" );
      serviceProviderReference           = getNextCharFromGMJO( 36,"serviceProviderReference" );
      fIProcInd                          = getNextCharFromGMJO( 1,"fIProcInd" );

      addSelfDefProperties();

   }

   /**
   * showObject writes out the values of this GtSmallBusWireTransferInit
   * object with the supplied PrintWriter and prefix String
   * @param res -- unused
   * @param pageWriter -- the PrintWriter to use
   * @param prefix -- the String prefix to put in front of each value
   */
   public void showObject( HttpServletResponse res, PrintWriter pageWriter, String prefix )
   {
      final String newPrefix = prefix + "|  ";
      pageWriter.println( prefix + "reqConsumerIDData =" );
      if (reqConsumerIDData == null)
      {
         pageWriter.println(" (null)");
      }
      else
      {
         for (int j = 0; j < reqConsumerIDData.length; j++)
         {
            reqConsumerIDData[j].showObject( res, pageWriter, newPrefix );
         }
         pageWriter.println("<br>");
      }
      pageWriter.println( prefix + "reqConsumerIDData_packed = " + getReqConsumerIDData_packed() );
      pageWriter.println( prefix + "reqWireTransferReferenceNumber = " + getReqWireTransferReferenceNumber() );
      pageWriter.println( prefix + "reqWireTransferNickname = " + getReqWireTransferNickname() );
      pageWriter.println( prefix + "reqWireTransferAmount =" );
      if (reqWireTransferAmount == null)
      {
         pageWriter.println(" (null)");
      }
      else
      {
         reqWireTransferAmount.showObject(res,pageWriter,newPrefix);
         pageWriter.println("<br>");
      }
      pageWriter.println( prefix + "reqWireTransferEffectiveDate =" );
      if (reqWireTransferEffectiveDate == null)
      {
         pageWriter.println(" (null)");
      }
      else
      {
         reqWireTransferEffectiveDate.showObject(res,pageWriter,newPrefix);
         pageWriter.println("<br>");
      }
      pageWriter.println( prefix + "reqConsumerInstructionText = " + getReqConsumerInstructionText() );
      pageWriter.println( prefix + "reqHomePhone = " + getReqHomePhone() );
      pageWriter.println( prefix + "reqWorkPhone = " + getReqWorkPhone() );
      pageWriter.println( prefix + "reqEmailAddress = " + getReqEmailAddress() );
      pageWriter.println( prefix + "reqFaxNumber = " + getReqFaxNumber() );
      pageWriter.println( prefix + "reqResponsePath = " + getReqResponsePath() );
      pageWriter.println( prefix + "reqContactRequired = " + getReqContactRequired() );
      pageWriter.println( prefix + "reqContactOptional = " + getReqContactOptional() );
      pageWriter.println( prefix + "reqStartTimeValue = " + getReqStartTimeValue() );
      pageWriter.println( prefix + "reqStartTimeFormat = " + getReqStartTimeFormat() );
      pageWriter.println( prefix + "reqStartTimeGMTOffsetMinutes = " + getReqStartTimeGMTOffsetMinutes() );
      pageWriter.println( prefix + "reqEndTimeValue = " + getReqEndTimeValue() );
      pageWriter.println( prefix + "reqEndTimeFormat = " + getReqEndTimeFormat() );
      pageWriter.println( prefix + "reqEndTimeGMTOffsetMinutes = " + getReqEndTimeGMTOffsetMinutes() );
      pageWriter.println( prefix + "reqFreeFormText = " + getReqFreeFormText() );
      pageWriter.println( prefix + "reqAddressLine1 = " + getReqAddressLine1() );
      pageWriter.println( prefix + "reqAddressLine2 = " + getReqAddressLine2() );
      pageWriter.println( prefix + "reqAddressLine3 = " + getReqAddressLine3() );
      pageWriter.println( prefix + "reqCity = " + getReqCity() );
      pageWriter.println( prefix + "reqProvince = " + getReqProvince() );
      pageWriter.println( prefix + "reqStateCode = " + getReqStateCode() );
      pageWriter.println( prefix + "reqCountryCode = " + getReqCountryCode() );
      pageWriter.println( prefix + "reqZipCode = " + getReqZipCode() );
      pageWriter.println( prefix + "reqPostalCode = " + getReqPostalCode() );
      pageWriter.println( prefix + "responseCode = " + getResponseCode() );
      try
      {
         pageWriter.println( prefix + "responseCode_text = " + getResponseCode_text() );
      }
      catch (IfsException e)
      {
         pageWriter.println(prefix + "responseCode_text throws IfsException: " + e.getMessage());
      }

      pageWriter.println( prefix + "referenceNumber = " + getReferenceNumber() );
      pageWriter.println( prefix + "serviceProviderReference = " + getServiceProviderReference() );
      pageWriter.println( prefix + "fIProcInd = " + getFIProcInd() );
      try
      {
         pageWriter.println( prefix + "fIProcInd_text = " + getFIProcInd_text() );
      }
      catch (IfsException e)
      {
         pageWriter.println(prefix + "fIProcInd_text throws IfsException: " + e.getMessage());
      }
   }


//==============================================================================

   public void populateFromURL( Object txn, HttpServletRequest req, String prefix )
   {
      int repeatCounter;

      repeatCounter = getLongFromURL( req, prefix+"reqConsumerIDData_repeatCounter" );
      reqConsumerIDData = new GsCnsmrDataId[ repeatCounter ];
      for (int i = 0; i < repeatCounter; i++)
      {
         reqConsumerIDData[i] = new GsCnsmrDataId(getIfsSession());
         reqConsumerIDData[i].populateFromURL(txn, req, prefix+"reqConsumerIDData_"+i+"_");
      }
      reqWireTransferReferenceNumber = getCharFromURL(req, prefix+"reqWireTransferReferenceNumber" );
      reqWireTransferNickname = getCharFromURL(req, prefix+"reqWireTransferNickname" );

      reqWireTransferAmount=populate.populateGsCurrencyFromURL(req, prefix,(IfsTxn)txn,false,"reqWireTransferAmount","GtSmallBusWireTransferInit" );

      reqWireTransferEffectiveDate=populate.populateGsDateFromURL(req, prefix,(IfsTxn)txn,false,"reqWireTransferEffectiveDate","GtSmallBusWireTransferInit" );
      reqConsumerInstructionText = getCharFromURL(req, prefix+"reqConsumerInstructionText" );
      reqHomePhone = getCharFromURL(req, prefix+"reqHomePhone" );
      reqWorkPhone = getCharFromURL(req, prefix+"reqWorkPhone" );
      reqEmailAddress = getCharFromURL(req, prefix+"reqEmailAddress" );
      reqFaxNumber = getCharFromURL(req, prefix+"reqFaxNumber" );
      reqResponsePath = getLongFromURL(req, prefix+"reqResponsePath" );
      reqContactRequired = getLongFromURL(req, prefix+"reqContactRequired" );
      reqContactOptional = getCharFromURL(req, prefix+"reqContactOptional" );
      reqStartTimeValue = getCharFromURL(req, prefix+"reqStartTimeValue" );
      reqStartTimeFormat = getCharFromURL(req, prefix+"reqStartTimeFormat" );
      reqStartTimeGMTOffsetMinutes = getLongFromURL(req, prefix+"reqStartTimeGMTOffsetMinutes" );
      reqEndTimeValue = getCharFromURL(req, prefix+"reqEndTimeValue" );
      reqEndTimeFormat = getCharFromURL(req, prefix+"reqEndTimeFormat" );
      reqEndTimeGMTOffsetMinutes = getLongFromURL(req, prefix+"reqEndTimeGMTOffsetMinutes" );
      reqFreeFormText = getCharFromURL(req, prefix+"reqFreeFormText" );
      reqAddressLine1 = getCharFromURL(req, prefix+"reqAddressLine1" );
      reqAddressLine2 = getCharFromURL(req, prefix+"reqAddressLine2" );
      reqAddressLine3 = getCharFromURL(req, prefix+"reqAddressLine3" );
      reqCity = getCharFromURL(req, prefix+"reqCity" );
      reqProvince = getCharFromURL(req, prefix+"reqProvince" );
      reqStateCode = getCharFromURL(req, prefix+"reqStateCode" );
      reqCountryCode = getCharFromURL(req, prefix+"reqCountryCode" );
      reqZipCode = getCharFromURL(req, prefix+"reqZipCode" );
      reqPostalCode = getCharFromURL(req, prefix+"reqPostalCode" );
      responseCode = getCharFromURL(req, prefix+"responseCode" );
      referenceNumber = getCharFromURL(req, prefix+"referenceNumber" );
      serviceProviderReference = getCharFromURL(req, prefix+"serviceProviderReference" );
      fIProcInd = getCharFromURL(req, prefix+"fIProcInd" );
   }

}
