//***************************************************************************
//
// FILE NAME:   GtSmallBusWireTransferInquiry.java
//
// COPYRIGHT:
//
//   Licensed Materials - Property of IBM
//   5648-A06
//   (C) Copyright IBM Corp. 1996, 2000 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//
//  CHANGES:
//    05/03/2000  rhs   d6662   Readded _text fields to toString or showObject.
//    05/25/2000  rhs   d6837   Added use of constructor which takes an IfsSession
//                              when new Gs* objects are created.
//
//***************************************************************************

package com.ibm.ifs.gold;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.ifs.gold.auto.*;

public class GtSmallBusWireTransferInquiry extends IfsTxn
{
   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean debugOn=false;

   /**
   * Default constructor.
   */
   public GtSmallBusWireTransferInquiry(){}

   /**
   * Constructor with an associated IfsSession object.
   */
   public GtSmallBusWireTransferInquiry(IfsSession sess) {
      setIfsSession(sess);
   }
   private static final String GMRequestId             = "wtfrinqr_java";
   public  String getGMRequestId()                 {return(GMRequestId);}

   private static final String MTILevel                = "99.01.02";
   public  String getMTILevel()                    {return(MTILevel);}

   public static final int SD_OWNER = 0;
   public int getSD_OWNER() { return( SD_OWNER );}
   private static Hashtable registry = new Hashtable();
   public static Hashtable getRegistry() {return registry;}

   public static final int SDThis = 0;
   int i = 0;

// request message fields/setters/getters


   private int     reqMaxNbrRecVal = 0;
   public  void    setReqMaxNbrRecVal(int o) {reqMaxNbrRecVal = o;}
   public  int     getReqMaxNbrRecVal() {return(reqMaxNbrRecVal);}

   private int     reqLstNbrRecVal = 0;
   public  void    setReqLstNbrRecVal(int o) {reqLstNbrRecVal = o;}
   public  int     getReqLstNbrRecVal() {return(reqLstNbrRecVal);}

   private byte[]  reqCrsr = null;
   public  void    setReqCrsr(byte[] o) {reqCrsr = o;}
   public  byte[]  getReqCrsr() {return(reqCrsr);}

   private boolean encryptConsumerIDData = false;

   private String consumerIDDataEncryptionType = IfsSystem.RSA_ENCRYPTION;
   public String getConsumerIDDataEncryptionType() {return consumerIDDataEncryptionType;}
   public void setConsumerIDDataEncryptionType(String o) {consumerIDDataEncryptionType = o;}

   private GsCnsmrDataId[]      reqConsumerIDData = null;
   public  void      setReqConsumerIDData(GsCnsmrDataId[] o) {reqConsumerIDData = o;encryptConsumerIDData = false;}
   public  void      setReqConsumerIDData(String[] o) {
      reqConsumerIDData = new GsCnsmrDataId[o.length];
      for (i=0; i<o.length; i++)
      {
         reqConsumerIDData[i] = new GsCnsmrDataId(getIfsSession());
         try
         {
            reqConsumerIDData[i].setConsumerDataIDItem(o[i].getBytes(IfsSystem.cwsGMJCharSet));
         }
         catch (UnsupportedEncodingException e)
         {
            debug("Exception caught in the statement \"reqConsumerIDData[i].setConsumerDataIDItem(o[i].getBytes(IfsSystem.cwsGMJCharSet))\", exception is: "+
                  e.toString());
         }
         reqConsumerIDData[i].setConsumerDataEncryptionType(consumerIDDataEncryptionType);
      }
      encryptConsumerIDData = true;
   }
   public GsCnsmrDataId[]    getReqConsumerIDData() {return(reqConsumerIDData);}
   private GsCnsmrDataId[]   getReqConsumerIDData(boolean format) {
      if (true==format && null!=reqConsumerIDData)
      {
         if (encryptConsumerIDData && (IfsSystem.RSA_ENCRYPTION.equals(consumerIDDataEncryptionType)))
         {
            for (i=0; i<reqConsumerIDData.length; i++)
            {
               IfsSecurity security = IfsSecurity.encryptData(new String(reqConsumerIDData[i].getConsumerDataIDItem()), IfsSystem.RSA_ENCRYPTION);
               if (!security.isValid())
               {
                  message = "The RSA encryption routine failed, the return code is " + security.getReturnCode();
                  status = ENCRYPTION_FAILURE_ERROR;
                  return reqConsumerIDData;

               }
               reqConsumerIDData[i].setConsumerDataIDItem(security.getEncodedValue());
               reqConsumerIDData[i].setConsumerDataEncryptionType(IfsSystem.RSA_ENCRYPTION);
            }
            encryptConsumerIDData = false;
         }
      }
      return reqConsumerIDData;
   }

   private String        reqCnsmrIdData_packed = null;
   public  void          setReqCnsmrIdData_packed( String o ) {reqCnsmrIdData_packed = o;}
   public  String        getReqCnsmrIdData_packed() {return(reqCnsmrIdData_packed);}



// response message fields/setters/getters

   private String  responseCode = null;
   public  void    setResponseCode(String o) {responseCode = o;}
   public  String  getResponseCode() {return(responseCode);}
   public  String  getResponseCode_text() throws IfsException
   {
      return IfsNLS.getText("IfsRespCodeClResources",getIfsSession().getLocale(),getResponseCode());
   }

   private String  fIProcInd = null;
   public  void    setFIProcInd(String o) {fIProcInd = o;}
   public  String  getFIProcInd() {return(fIProcInd);}
   public  String  getFIProcInd_text() throws IfsException
   {
      return IfsNLS.getText("IfsFiProcIndResources",getIfsSession().getLocale(),getFIProcInd());
   }

   private String  moreFlag = null;
   public  void    setMoreFlag(String o) {moreFlag = o;}
   public  String  getMoreFlag() {return(moreFlag);}
   public  String  getMoreFlag_text() throws IfsException
   {
      return IfsNLS.getText("IfsMoreFResources",getIfsSession().getLocale(),getMoreFlag());
   }

   private int     totalTransactionsMatched = 0;
   public  void    setTotalTransactionsMatched(int o) {totalTransactionsMatched = o;}
   public  int     getTotalTransactionsMatched() {return(totalTransactionsMatched);}

   private int     totalTransactionsSent = 0;
   public  void    setTotalTransactionsSent(int o) {totalTransactionsSent = o;}
   public  int     getTotalTransactionsSent() {return(totalTransactionsSent);}

   private byte[]  cursor = null;
   public  void    setCursor(byte[] o) {cursor = o;}
   public  byte[]  getCursor() {return(cursor);}


   private GsWireTfrDtl[]    wireTfrIDtl = null;
   public  void      setWireTfrIDtl(GsWireTfrDtl[] o) {wireTfrIDtl = o;}
   public GsWireTfrDtl[]    getWireTfrIDtl() {return(wireTfrIDtl);}

   /**
   * Add GtSmallBusWireTransferInquiry gold message to Gold Message Java Input buffer.
   */
   public void addGoldToGMJI(int argFlags, String cacheTxnName )
   {
      addArgToGMJI( com.ibm.ifs.gold.auto.GMDGlobals.GtSmallBusWireTransferInquiry );
      addArgToGMJI( argFlags );
      addArgToGMJI( cacheTxnName );
      addArgToGMJI();

      addArgToGMJI( reqMaxNbrRecVal );
      addArgToGMJI( reqLstNbrRecVal );
      addArgToGMJI( reqCrsr );

      if ( reqConsumerIDData != null)
      {
         reqCnsmrIdData_packed = GsCnsmrDataId.pack( getReqConsumerIDData(IfsTxn.ENCRYPT));
         addArgToGMJI( reqCnsmrIdData_packed );
      }
      else
      {
         addArgToGMJI();
      }

      reqSelfDef_packed = GsGmSelfDefiningFieldDef.pack( reqSelfDef);
      addArgToGMJI( reqSelfDef_packed );

   }

   /**
   * Get GtSmallBusWireTransferInquiry gold message from Gold Message
   * Java Output buffer.
   */
   public void getGoldFromGMJO()
   {
      int repeatCounter;

      responseCode                       = getNextCharFromGMJO( 3,"responseCode" );
      fIProcInd                          = getNextCharFromGMJO( 1,"fIProcInd" );
      moreFlag                           = getNextCharFromGMJO( 1,"moreFlag" );
      totalTransactionsMatched           = getNextLongFromGMJO( "totalTransactionsMatched" );
      totalTransactionsSent              = getNextLongFromGMJO( "totalTransactionsSent" );
      cursor                             = getNextBinaryItemFromGMJO( "cursor" );

      repeatCounter = getNextLongFromGMJO( "repeatCounter" );
      wireTfrIDtl = new GsWireTfrDtl[repeatCounter];
      for (int i = 0; i < repeatCounter; i++)
      {
         wireTfrIDtl[i]                           = new GsWireTfrDtl(this);
      }

      addSelfDefProperties();

   }

   /**
   * showObject writes out the values of this GtSmallBusWireTransferInquiry
   * object with the supplied PrintWriter and prefix String
   * @param res -- unused
   * @param pageWriter -- the PrintWriter to use
   * @param prefix -- the String prefix to put in front of each value
   */
   public void showObject( HttpServletResponse res, PrintWriter pageWriter, String prefix )
   {
      final String newPrefix = prefix + "|  ";
      pageWriter.println( prefix + "reqMaxNbrRecVal = " + getReqMaxNbrRecVal() );
      pageWriter.println( prefix + "reqLstNbrRecVal = " + getReqLstNbrRecVal() );
      pageWriter.println( prefix + "reqCrsr = " + getReqCrsr() );
      pageWriter.println( prefix + "reqConsumerIDData =" );
      if (reqConsumerIDData == null)
      {
         pageWriter.println(" (null)");
      }
      else
      {
         for (int j = 0; j < reqConsumerIDData.length; j++)
         {
            reqConsumerIDData[j].showObject( res, pageWriter, newPrefix );
         }
         pageWriter.println("<br>");
      }
      pageWriter.println( prefix + "reqCnsmrIdData_packed = " + getReqCnsmrIdData_packed() );
      pageWriter.println( prefix + "responseCode = " + getResponseCode() );
      try
      {
         pageWriter.println( prefix + "responseCode_text = " + getResponseCode_text() );
      }
      catch (IfsException e)
      {
         pageWriter.println(prefix + "responseCode_text throws IfsException: " + e.getMessage());
      }
      pageWriter.println( prefix + "fIProcInd = " + getFIProcInd() );
      try
      {
         pageWriter.println( prefix + "fIProcInd_text = " + getFIProcInd_text() );
      }
      catch (IfsException e)
      {
         pageWriter.println(prefix + "fIProcInd_text throws IfsException: " + e.getMessage());
      }

      pageWriter.println( prefix + "moreFlag = " + getMoreFlag() );
      try
      {
         pageWriter.println( prefix + "moreFlag_text = " + getMoreFlag_text() );
      }
      catch (IfsException e)
      {
         pageWriter.println(prefix + "moreFlag_text throws IfsException: " + e.getMessage());
      }
      pageWriter.println( prefix + "totalTransactionsMatched = " + getTotalTransactionsMatched() );
      pageWriter.println( prefix + "totalTransactionsSent = " + getTotalTransactionsSent() );
      pageWriter.println( prefix + "cursor = " + getCursor() );
      pageWriter.println( prefix + "wireTfrIDtl =" );
      if (wireTfrIDtl == null)
      {
         pageWriter.println(" (null)");
      }
      else
      {
         for (int j = 0; j < wireTfrIDtl.length; j++)
         {
            wireTfrIDtl[j].showObject( res, pageWriter, newPrefix );
         }
         pageWriter.println("<br>");
      }
   }


//==============================================================================

   public void populateFromURL( Object txn, HttpServletRequest req, String prefix )
   {
      int repeatCounter;

      reqMaxNbrRecVal = getLongFromURL(req, prefix+"reqMaxNbrRecVal" );
      reqLstNbrRecVal = getLongFromURL(req, prefix+"reqLstNbrRecVal" );
      reqCrsr = getBinaryFromURL(req, prefix+"reqCrsr" );
      repeatCounter = getLongFromURL( req, prefix+"reqCnsmrIdData_repeatCounter" );
      reqConsumerIDData = new GsCnsmrDataId[ repeatCounter ];
      for (int i = 0; i < repeatCounter; i++)
      {
         reqConsumerIDData[i] = new GsCnsmrDataId(getIfsSession());
         reqConsumerIDData[i].populateFromURL(txn, req, prefix+"reqCnsmrIdData_"+i+"_");
      }
      responseCode = getCharFromURL(req, prefix+"responseCode" );
      fIProcInd = getCharFromURL(req, prefix+"fIProcInd" );
      moreFlag = getCharFromURL(req, prefix+"moreFlag" );
      totalTransactionsMatched = getLongFromURL(req, prefix+"totalTransactionsMatched" );
      totalTransactionsSent = getLongFromURL(req, prefix+"totalTransactionsSent" );
      cursor = getBinaryFromURL(req, prefix+"cursor" );
   }

}
