// ***************************************************************************
//  FILE NAME:   IfsCTWGoldTxn.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2001 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//  CMVC Info:
//
//    latest delta date and time........... 02/06/07-12:01:47
//    cmvc path, component, release........ src/cws/jcwsapi/com/ibm/ifs/gold/IfsCtwGoldTxn.java, cws.jcwsapi, cws.v1r6m1.mmp, v1r6m1_020828
//    sccs release.level.branch.sequence... 1.1
//    sccs filename........................ /cmvc/cmvccws/vc/0/0/0/4/s.64
//
//  CHANGES:
//    11/26/2001  nc/bb/db  dxxxx   Created
//    12/03/2001  nc/bb/db  dxxxx   Debug code
// ***************************************************************************

package com.ibm.ifs.gold;




/**
 *   This class sends a CTw Gold Request to<BR>
 *   the back end and receives a response.<BR>
 *
*/
public class IfsCtwGoldTxn {
  
/* ------------------------------- Class Variables ---------------------------- */

/**  */
protected static final boolean DEBUG = true;
  
/* ----------------------------- Instance Variables ------------------------- */
      
  /**  */
  protected String ipAddr    = null;

  /**  */
  protected String url       = null;
  
  /**  */
  protected String cookie   = null;
        
  /**  */
  protected byte[] goldResp  = null;
        
  /**  */
  protected byte[] goldReq   = null;
        
  /**  */
  protected String newCookie = null;

/* ------------------------------- Constructors ----------------------------- */

  /**
   * Constructor for setting instance variables.<BR>
   *
   * @param ipAddr   :  Client IP Address parameter <BR>
   * @param url      :  Client URL parameter <BR>
   * @param cookie   :  Client cookie parameter <BR>
   * @param goldReq  :  gold request array parameter <BR>
   * @exception      :  IllegalArgumentException If any parameter is null <BR>
   * @see com.ibm.ifs.servlets.IfsCtwHttpServlet <BR>
  */
  public IfsCtwGoldTxn(String ipAddr, String url, 
    String cookie, byte[] goldReq) throws IllegalArgumentException {

    if (DEBUG)
       System.out.println("IfsCtwGoldTxn cons");

    if (ipAddr == null || url == null || goldReq == null)
      throw new IllegalArgumentException("Parameters to constructor must not be null.");

    this.ipAddr = ipAddr;
    this.url = url;
    this.cookie = cookie;
    this.goldReq = goldReq;
    if (DEBUG)
      System.out.println("IfsCtwGoldTxn cons done");
  }
   
/* ------------------------------ Instance Methods  -------------------------- */
 
        
  /**
   * This method gets the gold response from the back end <BR>
   * and returns it to IfsCtwHttpServlet <BR>
   *
   * @return byte[] : byte array that will hold the gold response.<BR>
  **/ 
  public byte[] getGoldResp(){
    return goldResp;
  }
  
  /**
   * This method returns a newCookie to IfsCtwHttpServlet <BR>
   * when called. <BR>
   *
   * @return String: return cookie value as a String <BR>
  */
  public String getNewCookie(){
    return newCookie;
  }
        
              
  /**
   * This method makes a call to the native method <BR>
   *
   * @return void <BR>
  */      
  public void execCtwGoldTxn() throws IfsException {
     
    if (DEBUG)
      System.out.println("IfsCtwGoldTxn.execCtwGoldTxn");
     execCtwGoldTxn0(url,cookie,ipAddr,goldReq,goldResp);
     
     
    if (DEBUG)
      System.out.println("IfsCtwGoldTxn.execCtwGoldTxn done");
  }
    
  /**
   * native method that passes a URL, client cookie <BR>
   * client IP address, gold request and a gold response <BR> 
   * to the C code.<BR>
   * This method has 2 side effects:<BR>
   * <OL>
   * <LI>Changes the goldResp instance variable</LI>
   * <LI>Possibly changes the newCookie instance variable</LI>
   * </OL>
   * @return void <BR>
  */    
   private native void execCtwGoldTxn0( String clientURL, String clientCookie, String clientIPAddr,
        byte [] goldReq, byte [] goldResp ) throws IfsException;
               
} // End IfsCtwGoldTxn class
