package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsPendingTransactionInstance.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//    
// ****************************************************************************    
// Change History
// 
//  Date                         Name                Reason For Change
// ------                       ------              -------------------
//
//   --                          --                 Initial Creation
//
// 01/19/2001                  D.Deshpande          Fixed bugs and got code 
//                                                  working
//
// 04/19/2001                  A.Asokan             T0984067
// 05/02/2001                  A.Asokan             T0984067R
// 11/06/2001                  B.Brockway           Removed import statment
//                                                  that was not needed and
//                                                  causing compilation to fail
//                                                  with JDK1.3. Defect 7364
//
// ****************************************************************************

import java.sql.*;
import java.io.*;                                                              
import java.math.BigDecimal;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

/**
* The IfsPendingTransactionInstance class is a subclass of IfsJDBC
* that is used to perform queries/updates on the Pending Transaction Instance
* table.  It provides the following methods to manage the data in this table: <BR>
* create() <BR>
* read()  <BR>
* list() <BR>
* delete() <BR>
* update()
*/

public class IfsPendingTransactionInstance extends IfsJDBC {

   /**
   * Instance array of type IfsPendingTransactionInstanceDetail 
   * Used to return ResultSet data 
   */
   private IfsPendingTransactionInstanceDetail[] outputData = null;



   /**
   * A String that is used to hold transaction data
   */
   private String transactionData = null; 

   private byte[] transactionDataByteArray = null;  
   private String tempErrorCode = null;
   /**
   * Set this flag if you wish to reset all required input parameters
   */
   private  boolean resetFlag = false;

   public static final String statusCodePropertyFileName = "com.ibm.ifs.resource.IfsPendingTransactionStatus";

   /**
   * The name of the Pendining Instance DB2 table
   */
   //public static final String TABLE_NAME = IfsSystem.IFS_WDBC_SCHEMA + ".PNDING_TXN_INST";

   public static String Qualifier = IfsSystem.IFS_WDBC_SCHEMA;

   public static final String TABLE_NAME = (Qualifier != null && Qualifier.equals("") == false) ? (Qualifier + ".PNDING_TXN_INST") : (Qualifier + "PNDING_TXN_INST");

   /**
   * The columns within the Pendining Instance table: <P>
   *
   *
   * Financial Institution Identifier <BR>
   * Primary Consumer ID <BR>
   * Transaction Initiator Consumer ID <BR>
   * Service Request Name <BR>
   * Transaction  Initiation Timestamp <BR>
   * Service Provider Name <BR>
   * Transaction Debt Account Clearing House ID <BR>
   * Transaction Debt Account Type <BR> 
   * Transaction Debt Account  Sub Type <BR>
   * Transaction Debt Account Number <BR>
   * Transaction Debt Account Nickname <BR>
   * Transaction Amount <BR>
   * Transaction Amount Precision <BR>
   * Transaction Currency Type <BR>
   * Transaction Beneficiary <BR>
   * Transaction  Reference <BR> 
   * Transaction Value Date  <BR>
   * Transaction Status Code <BR>
   * Transaction Authorizer Consumer ID <BR>
   * Transaction  Authorization Timestamp <BR>
   * Transaction Data
   *
   */
   public static final String[] columnNames = {
      "FI_ID",
      "PRIM_CNSMR_ID",
      "TXN_INIT_CNSMR_ID",
      "SRVC_RQST_NM",
      "TXN_INIT_TMSTP",
      "SRVC_NM",
      "TXN_DB_ACCT_CLH_ID",
      "TXN_DB_ACCT_TP",
      "TXN_DB_ACCT_SUB_TP",
      "TXN_DB_ACCT_NBR",
      "TXN_DB_ACCT_NKNM",
      "TXN_AMT",
      "TXN_AMT_PRC",
      "TXN_CURR_TP",
      "TXN_BENF",
      "TXN_REF",
      "TXN_VAL_DT",
      "TXN_STAT_CD",
      "TXN_AUTH_CNSMR_ID",
      "TXN_AUTH_TMSTP ",
      "TXN_DATA"};

   public static final String  FI_ID                  = "FI_ID";
   public static final String  PRIM_CNSMR_ID          = "PRIM_CNSMR_ID";
   public static final String  TXN_INIT_CNSMR_ID      = "TXN_INIT_CNSMR_ID";
   public static final String  SRVC_RQST_NM           = "SRVC_RQST_NM";
   public static final String  TXN_INIT_TMSTP         = "TXN_INIT_TMSTP";
   public static final String  SRVC_NM                = "SRVC_NM";
   public static final String  TXN_DB_ACCT_CLH_ID     = "TXN_DB_ACCT_CLH_ID";
   public static final String  TXN_DB_ACCT_TP         = "TXN_DB_ACCT_TP";
   public static final String  TXN_DB_ACCT_SUB_TP     = "TXN_DB_ACCT_SUB_TP";
   public static final String  TXN_DB_ACCT_NBR        = "TXN_DB_ACCT_NBR";
   public static final String  TXN_DB_ACCT_NKNM       = "TXN_DB_ACCT_NKNM";
   public static final String  TXN_AMT                = "TXN_AMT";
   public static final String  TXN_AMT_PRC            = "TXN_AMT_PRC";
   public static final String  TXN_CURR_TP            = "TXN_CURR_TP";
   public static final String  TXN_BENF               = "TXN_BENF";
   public static final String  TXN_REF                = "TXN_REF";
   public static final String  TXN_VAL_DT             = "TXN_VAL_DT";
   public static final String  TXN_STAT_CD            = "TXN_STAT_CD";
   public static final String  TXN_AUTH_CNSMR_ID      = "TXN_AUTH_CNSMR_ID";
   public static final String  TXN_AUTH_TMSTP         = "TXN_AUTH_TMSTP";   
   public static final String  TXN_DATA               = "TXN_DATA";


   private String financialInstitutionIdentifier    = null;
   private String primaryConsumerID                 = null;
   private String transactionInitiatorConsumerID    = null;
   private String serviceRequestName                = null;

   /**
   * The transaction initiation timestamp property. This is ignored by 
   * the create method if it is set (system timestamp is used instead)
   * For the read method, this should be set from the output of the list method
   */
   private String transactionInitiationTimestamp = null;


   private int readArraySize = 1;
   private String serviceProviderName = null;
   /**
   * An instance of the GsAccount class 
   */
   private GsAccount transactionDebtAccount = null;
   /**
   * This method is used to set the transactionDebtAccount property.
   * This setter will extract the properties of the transactionDebtAccount
   * for use in various transaction methods. These folllowing properties will be set: <BR>
   * transactionDebtAccountType  <BR>
   * transactionDebtAccountNumber <BR>
   * transactionDebtAccountNickname <BR>
   * transactionDebtAccountSubType 
   * <P>
   * @param acct --  a com.ibm.ifs.gold.GsAccount object
   */
   public void setTransactionDebtAccount(GsAccount acct) {

      try {

         transactionDebtAccount = acct;
         if (acct != null) {

            //transactionDebtAccountSubTypeFlag = true; 
            //debug("IfsPendingTransactionInstance.setTransactionDebtAccount() - acct.getType() is " + acct.getType());
            transactionDebtAccountType =  acct.getType();
            transactionDebtAccountNumber = acct.getNumber ();
            transactionDebtAccountNickname = acct.getNickname();
            transactionDebtAccountSubType = (new Integer( acct.getAccountSubType() ) ).shortValue();
            transactionDebtAccountClearingHouseID = acct.getClearinghouseFIID();
         }
      }
      catch (Throwable e) {
         debug("IfsPendingTransactionInstance.setTransactionDebtAccount() - A throwable error has occurred " + e.toString());
      }

   }
   /**
   * This method returns the transactionDebtAccount property.
   * <P>
   * @return transactionDebtAccount --  a com.ibm.ifs.gold.GsAccount object
   */
   public GsAccount getTransactionDebtAccount() {
      return transactionDebtAccount;

   }
   private String transactionDebtAccountClearingHouseID = null;
   private String transactionDebtAccountType = null;
   private short transactionDebtAccountSubType = 0;

   private boolean transactionDebtAccountSubTypeFlag = false;
   private String transactionDebtAccountNumber = null;
   private String transactionDebtAccountNickname    = null;
   private short transactionAmountPrecision;

   private boolean transactionAmountPrecisionFlag = false;
   private String transactionCurrencyType = null;
   //private long transactionAmount         = 0; 

   private String transactionAmount         =  null; 
   private boolean transactionAmountFlag = false;
   private GsCurrency goldTransactionAmount = null; 
   private String goldTxnAmount = null;
   private String transactionBeneficiary = null;
   private String transactionReference = null;
   private String transactionAuthorizerConsumerID = null;
   private String transactionAuthorizationTimestamp = null;
   private String transactionValueDate = null;  

   /**
   * The transactionStatusCode property. Valid values include:
   * 0 = Pending, 1 = Reviewed, 2 = Authorized, 3 = Denied.
   */

   private short transactionStatusCode = 0;
   private boolean transactionStatusCodeFlag = false;
   private String txnStatusErrorCode = null;

   /**
   * debug flag
   */
   public static boolean  debugOn = false;

   /**
   * The registry defined for this class. This is used to register customized
   * methods .   
   */
   private static Hashtable registry = new Hashtable();

   /**
   * Default constructor.
   * <P> 
   * This can be used if one does not wish to utilize the session cache.
   * Usually this is done when locale specific information is not required. This
   * constructor should NOT be used if you wish to use the "handleErrors"   
   * servlet method defined in IfsHttpServlet, as it makes use of the session cache.
   */
   public IfsPendingTransactionInstance() {


   }   
   /**
   * Constructor that takes an IfsSession object
   * <P>   
   * @param ifsSession  An instance of IfsSession.
   * <P>
   * This constructor should be used if you wish to utilize the session 
   * cache. An example of this would be if you wish your currency formats
   * to be based on the locale stored in the current user session. This
   * constructor should also be used if you wish to use the "handleErrors"   
   * servlet method defined in IfsHttpServlet.

   */
   public IfsPendingTransactionInstance(IfsSession ifsSession) {

      setIfsSession(ifsSession);

   }   


   /**
   * The delete method is used to delete a row in the PNDING_TXN_INST table. To use 
   * this method you must first create an instance of the class and set
   * the all required fields. 
   * <P>
   *    IfsPendingTransactionInstance transAct = new IfsPendingTransactionInstance(ifsSession); <BR>
   *    transAct.setUrl("jdbc:db2:DBTIFAV"); <BR>
   *    transAct.setDriverName("COM.ibm.db2.jdbc.app.DB2Driver"); <BR>
   *
   *    //set required fields here. <BR> <BR>
   *    Financial Institution Identifier <BR>
   *    Primary Consumer ID  <BR>
   *    Transaction Initiator Consumer ID <BR>
   *    Service Request Name <BR>
   *    Transaction  Initiation Timestamp     
   * <P>
   *    A "validation" method will be called upon
   *    execution of this method to assure the required fields are set.<BR>
   *
   *    transAct.delete();
   *
   */

   public void delete() throws IfsException {

      StringBuffer strBuff = new StringBuffer();                        
      BigDecimal tempAmount = null;                                     
      String deleteStr = "DELETE FROM " + TABLE_NAME + " WHERE "; 
      resetTxnTypes();
     //build sql string
      strBuff.append(FI_ID + " = '" + getFinancialInstitutionIdentifier() + "' and ");
      strBuff.append(PRIM_CNSMR_ID + " = '" + getPrimaryConsumerID() + "' and ");
      strBuff.append(TXN_INIT_CNSMR_ID + " = '" + getTransactionInitiatorConsumerID() + "' and ");
      strBuff.append(SRVC_RQST_NM + " = '" + getServiceRequestName() + "' and ");
      //strBuff.append(TXN_INIT_TMSTP + " = '" + getTransactionInitiationTimestamp() + "' ");
   
      //check to see whether user has entered valid timestamp
      if(validTimestamp(getTransactionInitiationTimestamp()) == true)
      {
        transactionInitiationTimestamp = changeTimestamp(getTransactionInitiationTimestamp());
        strBuff.append(TXN_INIT_TMSTP + " = '" + transactionInitiationTimestamp + "' ");
      }

      setSqlString(deleteStr + strBuff.toString() );
      deleteTxn = true;
      try {
         execute();

      }

      catch(IfsException e) {
        String msg = e.toString();
        throw new IfsException(e,msg);
      }

      catch(Exception e) {
        debug("<BR>IfsPendingTransaction delete exception.<BR> " + e.toString());
        String msg = "IfsPendingTransactionInstance.delete() has thrown an exception. " + e.toString();
        throw new IfsException(e,msg);
      }

      catch (Throwable e) {

         debug("IfsPendingTransactionInstance.delete() - A throwable error has occurred " + e.toString());
         String msg = "IfsPendingTransactionInstance.delete() has thrown an exception  " +  e.toString();
         if (getMessage() == null) {
            message = msg;
         }

         throw new IfsException(e,  msg);
      }
   }  //end of delete(). 

   /**
   * Returns the financialInstitutionIdentifier property. This property is required
   * for all methods that perform a database transaction. It is a key
   * value within the Pending Transaction Instance table
   * <P>
   * @return financialInstitutionIdentifier -- The Financial Institution Identifier property.
   */
   public String getFinancialInstitutionIdentifier   () {return financialInstitutionIdentifier;}   

   /**
   * Returns the outputData property. This property is an array of type 
   * IfsPendingTransactionInstanceDetail that is populated with data obtained
   * from a database query. When a ResultSet is returned is from a query, the
   * data contained in it is put into this repeat structure.
   * <P>
   * @return outputData -- A repeat structure of type 
   *                       IfsPendingTransactionInstanceDetail.
   */
   public IfsPendingTransactionInstanceDetail[] getOutputData() {
      return outputData;
   }   

   /**
   * Returns the primaryConsumerID property. This property is required
   * for all methods that perform a database transaction. It is a key
   * value within the Pending Transaction Instance table
   * <P>
   * @return primaryConsumerID -- The Primary Consumer ID property.
   */

   public String getPrimaryConsumerID () {return primaryConsumerID;} 

    /**
   * This method returns the registry that stores registered methods.
   * A registered method can be used to override a default implementation. <BR>
   * Common registry methods include: <BR>
   * The validator method <BR>
   * The requestReadyAction method <BR>
   * The responseReadyAction method <BR>
   * These methods are invoked within the execute method.
   * <P>
   * @return registry - A hashtable used to access registered methods..
   */
   public static Hashtable getRegistry() {
      return registry;
   }   

   /**
   * Returns the serviceProviderName property. This property is required
   * for the create, read, update, and delete methods .    
   * <P>
   * @return serviceProviderName -- The Service Provider Name property.
   */
   public String getServiceProviderName() {return serviceProviderName;} 

   /**
   * Returns the serviceRequestName property. This property is required
   * for the create, read, update, and delete methods .    
   * <P>
   * @return serviceRequestName -- The Service Request Name property.
   */
   public String getServiceRequestName   () {return serviceRequestName;} 

   /**
   * Returns the transactionAmount property. This property is required
   * input for the create method.
   * <P>
   * @return transactionAmount -- The Transaction Amount property.
   */
   public String getTransactionAmount() {return transactionAmount;} 

   /**
   * This method gets the goldTransactionAmount object which was populated by 
   * the servlet. Values from this object will be used to populate the
   * "transactionAmount", "transactionAmountPrecision", and "transactionCurrencyType"
   * for the SQL query.  It is the servlet writer's responsibility to populate these
   * values before calling the "create" method.
   * <P>
   * @return goldTransactionAmount -- The Gold Transaction Amount property. 
   *                                  This object is of type GsCurrency.
   */
   //private GsCurrency getGoldTransactionAmount() {return goldTransactionAmount;}
   public GsCurrency getGoldTransactionAmount() {return goldTransactionAmount;}

   /**
   * Returns the transactionAmountPrecision property. This property is required
   * input for the create method.
   * <P>
   * @return transactionAmountPrecision -- The Transaction Amount Precision property.
   */
   public short getTransactionAmountPrecision() {

      return  transactionAmountPrecision;
   }   

   /**
   * Returns the transactionAuthorizationTimestamp property. This is an output property
   * returned from the read and list methods.
   * <P>
   * @return transactionAuthorizationTimestamp -- The Transaction Authorization Timestamp property.
   */
   public String getTransactionAuthorizationTimestamp() {
      return transactionAuthorizationTimestamp;

   } 

   /**
   * Returns the transactionAuthorizerConsumerID property. This is an output property
   * returned from the read and list methods.
   * <P>
   * @return transactionAuthorizerConsumerID -- The Transaction Authorizer Consumer ID property.
   */
   public String getTransactionAuthorizerConsumerID() {
      return transactionAuthorizerConsumerID;

   }   

   /**
   * Returns the transactionBeneficiary property. This is an optional input parameter
   * for the create and update methods. It also is returned from the read and list methods.
   * <P>
   * @return transactionBeneficiary -- The Transaction Beneficiary property.
   */
   public String getTransactionBeneficiary() {return transactionBeneficiary;}   

   /**
   * Returns the transactionCurrencyType property. This property is required
   * input for the create method. It contains the ISO currency code associated
   * with a given currency value.
   * <P>
   * @return transactionCurrencyType -- The Transaction Currency Type property.
   */
   public String getTransactionCurrencyType() {return transactionCurrencyType;}   

   /**
   * Returns the transactionData property. This property is required
   * input for the create method. It is an optional parameter for the update method.
   * <P>
   * @return transactionData -- The Transaction Data property.
   */   
   public String getTransactionData(){

      return transactionData;
   }   
   /**
   * Returns the transactionDebtAccountClearingHouseID property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and  list methods.
   * <P>
   * @return transactionDebtAccountClearingHouseID -- The Transaction Debt Account 
   *                                                   Clearing House ID property.
   */
   public String getTransactionDebtAccountClearingHouseID() {return transactionDebtAccountClearingHouseID;}   

   /**
   * Returns the transactionDebtAccountNickname property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and  list methods.
   * <P>
   * @return transactionDebtAccountNickname -- The Transaction Debt Account 
   *                                            Nickname property.
   */
   public String getTransactionDebtAccountNickname   () {return transactionDebtAccountNickname;}   

   /**
   * Returns the transactionDebtAccountNumber property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and  list methods.
   * <P>
   * @return transactionDebtAccountNumber -- The Transaction Debt Account 
   *                                          Number property.
   */
   public String getTransactionDebtAccountNumber() {return transactionDebtAccountNumber;}   

   /**
   * Returns the transactionDebtAccountSubType property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and list methods.
   * <P>
   * @return transactionDebtAccountSubType -- The Transaction Debt Account 
   *                                          Sub Type property.
   */
   public short getTransactionDebtAccountSubType () {return transactionDebtAccountSubType;}   

   /**
   * Returns the transactionDebtAccountType property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and list methods.
   * <P>
   * @return transactionDebtAccountType -- The Transaction Debt Account 
   *                                        Type property.
   */
   public String getTransactionDebtAccountType() {return transactionDebtAccountType;}   

   /**
   * Returns the transactionInitiatorConsumerID property. This property  
   * is a required input parameter for the create and update methods. It is also
   * returned from the read and list methods.
   * <P>
   * @return transactionInitiatorConsumerID -- The Transaction Initiator Consumer ID property.
   */
   public String getTransactionInitiatorConsumerID() {
      return transactionInitiatorConsumerID;
   } 

   /**
   * Returns the transactionInitiationTimestamp property. This property  
   * is a required input parameter for the  update method. 
   * For the create method, it assumes a default value in the database
   * when the update is made. This value returned from the read and list methods.
   * <P>
   * @return transactionInitiationTimestamp -- The Transaction Initiation Timestamp property.
   */
   public String getTransactionInitiationTimestamp() {return transactionInitiationTimestamp;}   

   /**
   * Returns the transactionReference property. This is an optional input parameter
   * for the create and update methods. It also is returned from the read and list methods.
   * <P>
   * @return transactionReference -- The Transaction Reference property.
   */
   public String getTransactionReference() {return transactionReference;} 

   /**
   * Returns the transactionStatusCode property. This is an optional input parameter
   * for the create and update methods. It also is returned from the read and list methods.
   * <P>
   * @return transactionStatusCode -- The Transaction Status Code property.
   */

   public String getTransactionStatusCode() {return (new Short(transactionStatusCode) ).toString();}   

   //public short getTransactionStatusCode() {return transactionStatusCode;} 

   /**
   * This method sets the transactionStatusCode property. This is an optional input parameter
   * for the update method. It is set to "0" for the create method by default. 
   * It also is returned from the read and list methods.
   * <P>
   * @param transactionStatusCode -- The Transaction Status Code property.
   */
   public void setTransactionStatusCode(short o) {
   

      transactionStatusCode = o;
      transactionStatusCodeFlag = true;

      //Check to see whether Transaction Status Code is between 0 to 3
      if (transactionStatusCode < 0 || transactionStatusCode >3)
      {
        txnStatusErrorCode = "Transaction Status Code should be between 0 to 3";
        transactionStatusCodeFlag = false;
      }

   } 

   /**
   * This method sets the transactionStatusCode property. This is an optional input parameter
   * for the update method. It is set to "0" for the create method by default. 
   * It also is returned from the read and list methods.
   * <P>
   * @param transactionStatusCode -- The Transaction Status Code property.
   */
   public void setTransactionStatusCode(String o) {

      if(o.equals("") != true)
      {

        transactionStatusCodeFlag = true;

        debug("entering  setTransactionStatusCode(String) " );
        
        //If transaction status code is not numeric then throw exception
        try {

           debug("in try of setTransactionStatusCode(String) " );
           transactionStatusCode = (new Short(o) ).shortValue();

           //Check to see whether Transaction Status Code is between 0 to 3
           if (transactionStatusCode < 0 || transactionStatusCode >3)
           {
             txnStatusErrorCode = "Transaction Status Code should be between 0 to 3";
             transactionStatusCodeFlag = false;
           }
        }
        catch (Throwable e) {
           txnStatusErrorCode = "Transaction Status Code should be a Numeric and between 0 to 3";
           transactionStatusCodeFlag = false;
           debug("setTransactionStatusCode(String o) has thrown an exception: " + e.toString() );
        }
      }
   } 


   /**
   * This method returns the transactionValueDate property. This is an optional input parameter
   * for the update method and a required parameter for the create method. 
   * It also is returned from the read and list methods.
   * <P>
   * @return transactionValueDate -- The Transaction Value Date property.
   */
   public String getTransactionValueDate() {return transactionValueDate;}  

   /**
   * The list method is used to display all row in the PNDING_TXN_INST table
   * that correspond to given search criteria. To use this method you must 
   * first create an instance of the class and set the all required fields. 
   * <P>
   *    IfsPendingTransactionInstance transAct = new IfsPendingTransactionInstance(ifsSession); <BR>
   *    transAct.setUrl("jdbc:db2:DBTIFAV"); <BR>
   *    transAct.setDriverName("COM.ibm.db2.jdbc.app.DB2Driver"); <BR>
   *    //set required fields here. <BR> <BR>
   * 
   *    Financial Institution Identifier <BR>
   *    Primary Consumer ID  <BR><BR>
   *
   *    A "validation" method will be called upon
   *    execution of this method to assure the required fields are set.
   * <BR>
   *    transAct.list();
   *
   */

   public void list   () throws IfsException{    

      StringBuffer strBuff = new StringBuffer();
      BigDecimal tempAmount = null;
      int scale = 0;
      boolean sessionFlag = false;
      IfsSession session =  null;
      resetTxnTypes();
      try {
         session = getIfsSession();

       //build sql string
         strBuff.append("SELECT FI_ID,PRIM_CNSMR_ID,TXN_INIT_CNSMR_ID,SRVC_RQST_NM,TXN_INIT_TMSTP,SRVC_NM,TXN_DB_ACCT_CLH_ID,TXN_DB_ACCT_TP,TXN_DB_ACCT_SUB_TP,TXN_DB_ACCT_NBR,TXN_DB_ACCT_NKNM,TXN_AMT_PRC,TXN_AMT,TXN_CURR_TP,TXN_BENF,TXN_REF,TXN_VAL_DT,TXN_STAT_CD,TXN_AUTH_CNSMR_ID,TXN_AUTH_TMSTP,TXN_DATA FROM "  + TABLE_NAME + " WHERE " );

         strBuff.append(FI_ID + " = '" + getFinancialInstitutionIdentifier() + "' and ");
         strBuff.append(PRIM_CNSMR_ID + " = '" + getPrimaryConsumerID() + "' ");

         setSqlString(strBuff.toString() );
         listTxn = true;
         execute();
         GsCurrency goldCurrency = null;
         ResultSet results = getResultSet();

         if (results == null)  {
            debug("IfsPendingTransactionInstance.list() - res set = null");
         }

         if (results != null) {

            debug("IfsPendingTransactionInstance.list() - res set != null");
            String statusCodeStr = null;
            short statusCode;
            IfsPendingTransactionInstanceDetail detail = null;
            Vector vector = new Vector();

            while (results.next()) {
               detail = new IfsPendingTransactionInstanceDetail();

               //get response values
               detail.setFinancialInstitutionIdentifier( results.getString(FI_ID) );
               detail.setPrimaryConsumerID( results.getString(PRIM_CNSMR_ID) );
               detail.setTransactionInitiatorConsumerID( results.getString(TXN_INIT_CNSMR_ID) );
               detail.setServiceRequestName(results.getString(SRVC_RQST_NM));
               
               //detail.setTransactionInitiationTimestamp( results.getString(TXN_INIT_TMSTP)); 

               String tempTimestamp = results.getString(TXN_INIT_TMSTP);
               
               if(tempTimestamp != null)
               {
                 //JDBC returns time stamp is escaped format. Format it same as data base format.
                 tempTimestamp = tempTimestamp.replace(' ','-');  //replace " " after date with "-"
                 tempTimestamp = tempTimestamp.replace(':','.');  //replace ":" after hh,mm,ss with "."
               }

               detail.setTransactionInitiationTimestamp( tempTimestamp); 

               
               detail.setServiceProviderName( results.getString(SRVC_NM) );
               
               try {

                  transactionDebtAccount = new GsAccount(getIfsSession());
                  if (transactionDebtAccount != null) {
                  
                      debug("In IfsPendingTransactionInstance.list() - before setting acct stuff " );
                      
                      transactionDebtAccount.setClearinghouseFIID(results.getString(TXN_DB_ACCT_CLH_ID));
                      
                      transactionDebtAccount.setType(results.getString(TXN_DB_ACCT_TP));
                      debug("In IfsPendingTransactionInstance.list() - after setting Type ");
                      
                      transactionDebtAccount.setAccountSubType(results.getShort(TXN_DB_ACCT_SUB_TP));
                      
                      transactionDebtAccount.setNumber(results.getString(TXN_DB_ACCT_NBR));
   
                      //transactionDebtAccount.setNickname(results.getString(TXN_DB_ACCT_NKNM));
     
                      //debug("Account Nick Name as bytes" + results.getBytes(TXN_DB_ACCT_NKNM));

                      //Account Nick Name column is defined as For Bit Data data type. Hence get it in bytes.

                      //Check to see whether account nick name is null
                      if (results.getBytes(TXN_DB_ACCT_NKNM) != null)
                      {
                         String tempAcctNickname = new String(results.getBytes(TXN_DB_ACCT_NKNM));


                         //String contains '@' which represent blanks in database. Replace them with spaces
                         if(tempAcctNickname != null) 
                         { 
                           tempAcctNickname = tempAcctNickname.replace('@',' ');
                        
                           transactionDebtAccount.setNickname( tempAcctNickname );

                         }
                      }

                      debug("In IfsPendingTransactionInstance.list() - after setting Nick ") ;
                  }
                  if (transactionDebtAccount != null) {

                     detail.setTransactionDebtAccount( transactionDebtAccount );            
                  }
                  else
                     debug("in IfsPendingTransactionInstanceoutputData[0].list, transactionDebtAccount is null");
                  transactionAmountPrecision = results.getShort(TXN_AMT_PRC);

                  debug("in IfsPendingTransactionInstanceoutputData[0].list, after transactionAmountPrecision");

                  tempAmount = results.getBigDecimal(TXN_AMT, scale);


                  debug("in IfsPendingTransactionInstanceoutputData[0].list, after tempAmount");
               }
               catch (Throwable e) {

                  debug("In IfsPendingTransactionInstance.list() - acct stuff threw an exception " +  e.toString() );
               }
               if (tempAmount != null) {
                  goldCurrency = GsCurrency.fromString(tempAmount.toString());
                  goldCurrency.setDigits(tempAmount.toString());
                  goldCurrency.setPrecision(transactionAmountPrecision);
                  goldCurrency.setIsoCode(results.getString(TXN_CURR_TP));            
                  detail.setGoldTransactionAmount(goldCurrency); 
                  tempAmount = null;
               }
               else
                  debug("in IfsPendingTransactionInstanceoutputData[0].list :tempAmount == null " );

               //detail.setTransactionBeneficiary( results.getString(TXN_BENF) );
  
               //Transaction Beneficiary column is defined as For Bit Data data type. Hence get it in bytes.

               //Check to see whether transaction beneficiary is null

               if(results.getBytes(TXN_BENF) != null)
               {
                 String tempTxnBenf = new String(results.getBytes(TXN_BENF));

                 //String contains '@' which represent blanks in database. Replace them with spaces

               
                 if(tempTxnBenf != null)
                 { 
                   tempTxnBenf = tempTxnBenf.replace('@',' ');

                   detail.setTransactionBeneficiary( tempTxnBenf);

                 }
               }
               detail.setTransactionReference( results.getString(TXN_REF) );
               detail.setTransactionValueDate( (results.getString(TXN_VAL_DT)) );
               statusCode = results.getShort(TXN_STAT_CD);
               detail.setTransactionStatusCode( statusCode );

           //set transactionStatusCode_text
               statusCodeStr = ( new Short(statusCode) ).toString();

               try {
                  detail.setTransactionStatusCode_text(IfsNLS.getString(statusCodePropertyFileName, getLocale(), statusCodeStr) );
               }
               catch (Throwable e) {

                  debug("In IfsPendingTransactionInstance.list() - in getString catch block - " +  e.toString() );
               }
               if (results.getString(TXN_AUTH_CNSMR_ID) != null) {

                  detail.setTransactionAuthorizerConsumerID( results.getString(TXN_AUTH_CNSMR_ID) );
               }

               String tempTS = results.getString(TXN_AUTH_TMSTP);

               if (tempTS != null) 
               {
                  //JDBC returns time stamp is escaped format. Format it same as data base format.
                  tempTS = tempTS.replace(' ','-');  //replace " " after date with "-" 
                  tempTS = tempTS.replace(':','.');  //replace ":" after hh,mm,ss with "."
               }
               
               detail.setTransactionAuthorizationTimestamp( tempTS );

         //set Transaction Data here
               try {

                  detail.setTransactionData((new String (results.getBytes(TXN_DATA) ) ));
               }
               catch (Throwable e) {

                  debug("In IfsPendingTransactionInstance.list() - in setTransactionData((new String (results.getBytes(TXN_DATA) ) catch block - " +  e.toString() );
               }
               vector.addElement(detail); //update vector
            }

            Integer vectorSize = new Integer (vector.size());

            outputData = new IfsPendingTransactionInstanceDetail[vector.size()];


            //check too see whether database returned rows
            
            //if (vector.size() != 0) {
            //copy results to data class
               for (int i = 0; i < vector.size();  i++) {
                 debug("IfsPendingTransactionInstance.list() - inside  for(int i = 0; i < vector.size();  i++) "  );
                 outputData[i] = (IfsPendingTransactionInstanceDetail) vector.elementAt(i); 
               }  
            //}
            
            //database didn't return rows
            /*else
            {
              String msg = "Given criteria did not match any record.";
              throw new IfsException("No Records Returned",msg);
            }*/
            results.close();
            debug("IfsPendingTransactionInstanceDetail list parsing done.");

         }

      }
       
      catch(IfsException e) {
        debug("in IfsException catch");
        String msg = e.toString();
        throw new IfsException(e,msg);
      }
  
      catch(Exception e) {
        debug("<BR>IfsPendingTransactionInstance list exception.<BR>" + e.toString());
        String msg = "IfsPendingTransactionInstance.list() has thrown an exception. " + e.toString();
        throw new IfsException(e,msg);
      }

      catch (Throwable e) {

         String msg = "In IfsPendingTransactionInstance.list() - an exception has been thrown - " +  e.toString();
         debug(msg);
         if (getMessage() == null) {
            message = msg;
         }

         throw new IfsException(e,  msg);

      }

   }  //end of list().  

   /**
   * The read method is used to display a single row of the PNDING_TXN_INST table.
   * corresponding to given search criteria. The input for this method should 
   * be derived from the output the list method. To use this method you must 
   * first create an instance of the class and set the all required fields. 
   * <P>
   *    IfsPendingTransactionInstance transAct = new IfsPendingTransactionInstance(ifsSession);<BR>
   *    transAct.setUrl("jdbc:db2:DBTIFAV"); <BR>
   *    transAct.setDriverName("COM.ibm.db2.jdbc.app.DB2Driver"); <BR>
   *
   *    //set required fields here. <BR><BR>
   *
   *    Financial Institution Identifier <BR>
   *    Primary Consumer ID  <BR>
   *    Transaction Initiator Consumer ID <BR>
   *    Service Request Name <BR>
   *    Transaction  Initiation Timestamp  <BR><BR>   
   *
   *    A "validation" method will be called upon
   *    execution of this method to assure the required fields are set.
   * <BR>
   *    transAct.read();
   *
   */

   public void read() throws IfsException{

      BigDecimal tempAmount = null;
      String[] s = null;
      ResultSetMetaData metaDataSet = null;
      String queryStr = null;
      StringBuffer query = new StringBuffer(); //null;
      IfsSession session =  null;
      resetTxnTypes();
      try {  
          try {
          
         session =  getIfsSession();

      //input comes from the list method
         query.append(FI_ID + "  =  '" + financialInstitutionIdentifier + "' and ");
         query.append(PRIM_CNSMR_ID + "  =  '" + primaryConsumerID + "' and ");
         query.append(TXN_INIT_CNSMR_ID + "  =  '" + transactionInitiatorConsumerID + "' and ");
         query.append(SRVC_RQST_NM + "  =  '" + serviceRequestName + "' and ");
         //query.append(TXN_INIT_TMSTP + " =  '" + transactionInitiationTimestamp + "' ");
     
         //Check to see whether the user has entered valid timestamp and if it is valid then change it to valid database format.
         if(validTimestamp(transactionInitiationTimestamp) == true)
         {
           transactionInitiationTimestamp = changeTimestamp(transactionInitiationTimestamp);
           query.append(TXN_INIT_TMSTP + " =  '" + transactionInitiationTimestamp + "' ");
         }

         setSqlString("SELECT FI_ID,PRIM_CNSMR_ID,TXN_INIT_CNSMR_ID,SRVC_RQST_NM,TXN_INIT_TMSTP,SRVC_NM,TXN_DB_ACCT_CLH_ID,TXN_DB_ACCT_TP,TXN_DB_ACCT_SUB_TP,TXN_DB_ACCT_NBR,TXN_DB_ACCT_NKNM,TXN_AMT_PRC,TXN_AMT,TXN_CURR_TP,TXN_BENF,TXN_REF,TXN_VAL_DT,TXN_STAT_CD,TXN_AUTH_CNSMR_ID,TXN_AUTH_TMSTP,TXN_DATA FROM " + TABLE_NAME + " WHERE " + query.toString() );

          }
         catch (Throwable e) {

                  debug("In IfsPendingTransactionInstance.read() - after setting req parms - " +  e.toString() );
         }
         readTxn = true;
         execute();
         ResultSet results = getResultSet();
         String statusCodeStr = null;
         short statusCode;
         outputData = null;
        
         debug("in IfsPendingTransactionInstance.read() - before checking for results ");

         if (results != null)  {
            int scale = 0;
            GsCurrency goldCurrency = null;
            int size = 1;
            
            debug("in IfsPendingTransactionInstanceout.read : in If results != null");

            while (results.next()) {

               debug("in IfsPendingTransactionInstanceout.read :in while");

               outputData = new IfsPendingTransactionInstanceDetail[readArraySize];

               outputData[0] = new IfsPendingTransactionInstanceDetail();

               debug("in IfsPendingTransactionInstanceoutputData[0].read :before setting outputData" );
               
               outputData[0].setFinancialInstitutionIdentifier( results.getString(FI_ID) );
               outputData[0].setPrimaryConsumerID( results.getString(PRIM_CNSMR_ID) );
               outputData[0].setTransactionInitiatorConsumerID( results.getString(TXN_INIT_CNSMR_ID) );
               
               String tempInitTimestamp = results.getString(TXN_INIT_TMSTP);
               
               if(tempInitTimestamp != null)
               {
                 //JDBC returns time stamp is escaped format. Format it same as data base format.
                 tempInitTimestamp = tempInitTimestamp.replace(' ','-');  //replace " " after date with "-"
                 tempInitTimestamp = tempInitTimestamp.replace(':','.');  //replace ":" after hh,mm,ss with "."
               }
               
               outputData[0].setTransactionInitiationTimestamp( tempInitTimestamp);
               
               outputData[0].setServiceRequestName( results.getString(SRVC_RQST_NM) );
           
               debug("in IfsPendingTransactionInstanceoutputData[0].read: after setting required fields");

               outputData[0].setServiceProviderName( results.getString(SRVC_NM) );           

               transactionDebtAccount = new GsAccount(getIfsSession());
               if (transactionDebtAccount != null) {
               
               transactionDebtAccount.setClearinghouseFIID(results.getString(TXN_DB_ACCT_CLH_ID));
               transactionDebtAccount.setType(results.getString(TXN_DB_ACCT_TP));
               transactionDebtAccount.setAccountSubType(results.getShort(TXN_DB_ACCT_SUB_TP));
               transactionDebtAccount.setNumber(results.getString(TXN_DB_ACCT_NBR));
               //transactionDebtAccount.setNickname(results.getString(TXN_DB_ACCT_NKNM));

               //Account Nick Name column is defined as For Bit Data data type. Hence get it in bytes.

               //Check to see whether account nick name is null

               debug("Before setting account nick name in pnding txn");

               if(results.getBytes(TXN_DB_ACCT_NKNM) != null)
               {
                 String tempAcctNickname = new String(results.getBytes(TXN_DB_ACCT_NKNM));

         
                 //String contains '@' which represent blanks in database. Replace them with spaces

                 if(tempAcctNickname != null)
                 {
                    tempAcctNickname = tempAcctNickname.replace('@',' ');

                   transactionDebtAccount.setNickname( tempAcctNickname ); 

                   debug("Using temp acct nick name again");
                 }
               }

               outputData[0].setTransactionDebtAccount( transactionDebtAccount );            
               }
               debug("in IfsPendingTransactionInstanceoutputData[0].read: after setting debtAccount");
               transactionAmountPrecision = results.getShort(TXN_AMT_PRC);
               tempAmount = results.getBigDecimal(TXN_AMT, scale);
               debug("in pnding txn , before temp amount is accessed");

               if (tempAmount != null) {

                  goldCurrency = GsCurrency.fromString(tempAmount.toString());
                  goldCurrency.setDigits(tempAmount.toString());
                  goldCurrency.setPrecision(transactionAmountPrecision);
                  goldCurrency.setIsoCode(results.getString(TXN_CURR_TP));
                  tempAmount = null;
               }
               else
                  debug("in IfsPendingTransactionInstanceoutputData[0].read :tempAmount == null " );

               outputData[0].setGoldTransactionAmount(goldCurrency);

               //outputData[0].setTransactionBeneficiary( results.getString(TXN_BENF) );
       
               //Transaction Beneficiary column is defined as For Bit Data data type. Hence get it in bytes.

               //Check to see whether transaction beneficiary is null

               if(results.getBytes(TXN_BENF) != null)
               {
                 String tempTxnBenf = new String(results.getBytes(TXN_BENF));
              
                 //String contains '@' which represent blanks in database. Replace them with spaces
     
                
                 if(tempTxnBenf != null)
                 {
              
                   tempTxnBenf = tempTxnBenf.replace('@',' ');

                   outputData[0].setTransactionBeneficiary( tempTxnBenf );
              
                 }
               }

               outputData[0].setTransactionReference( results.getString(TXN_REF) );
               outputData[0].setTransactionValueDate( (results.getString(TXN_VAL_DT)) );
             
               statusCode = results.getShort(TXN_STAT_CD);
               outputData[0].setTransactionStatusCode( statusCode );

               //set transactionStatusCode_text
               statusCodeStr = ( new Short(statusCode) ).toString();

               try {
                  outputData[0].setTransactionStatusCode_text(IfsNLS.getString(statusCodePropertyFileName, getLocale(), statusCodeStr) );
               }
               catch (Throwable e) {

                  debug("In IfsPendingTransactionInstance.read() - in getString catch block - " +  e.toString() );
               }
               if (results.getString(TXN_AUTH_CNSMR_ID) != null) {

                  outputData[0].setTransactionAuthorizerConsumerID( results.getString(TXN_AUTH_CNSMR_ID) );
               }

               //outputData[0].setTransactionAuthorizationTimestamp( results.getString(TXN_AUTH_TMSTP) );
               String tempAuthTimestamp = results.getString(TXN_AUTH_TMSTP);


               if (tempAuthTimestamp != null)  
               {
                  //JDBC returns time stamp is escaped format. Format it same as data base format.
                  tempAuthTimestamp = tempAuthTimestamp.replace(' ','-');  //replace " " after date with "-"
                  tempAuthTimestamp = tempAuthTimestamp.replace(':','.');  //replace ":" after hh,mm,ss with "."
                  
                  
                  outputData[0].setTransactionAuthorizationTimestamp( tempAuthTimestamp);   
               }

            //set Transaction Data here  results.getBytes(TXN_DATA

               //outputData[0].setTransactionData( results.getString(TXN_DATA) );
               

               outputData[0].setTransactionData((new String (results.getBytes(TXN_DATA) ) ));
               

               debug("in IfsPendingTransactionInstanceoutputData[0].read: All fields are set");
            }
         }  //end of if 

         closeResultSet();

         //result set doesn't contain data, hence outputData is null
         /*if(outputData == null) {
           String msg = "Given criteria did not match any record.";
           throw new IfsException("No Records Returned",msg);
         }*/ 
      }
      
      catch(IfsException e) {
        String msg = e.toString();
        throw new IfsException(e,msg);
      }
 
      catch(Exception e) {
        debug("<BR>IfsPendingTransactionInstance read exception.<BR> " + e.toString());
        String msg = "IfsPendingTransactionInstance.read() has thrown an exception. " + e.toString();
        throw new IfsException(e,msg);
      }

      catch (Throwable e) {
         String msg = "IfsPendingTransactionInstance.read() has thrown an exception  " +  e.toString();
         debug(msg);
         if (getMessage() == null) {
            message = msg;
         }

         throw new IfsException(e,  msg);    
      }
 
   }  //end of read().

   /**
   * This method is used to reset all required input parameters. It can be used
   * if you wish to issue a series of transactions in succession. 
   */

   public void resetInputParameters() {
      financialInstitutionIdentifier = null;
      primaryConsumerID = null;
      transactionInitiatorConsumerID = null;
      serviceRequestName = null;
      transactionInitiationTimestamp = null;
      transactionValueDate = null;
      transactionData = null;
   }   

   /**
   * This method is used to set the financialInstitutionIdentifier property. This property is required
   * for all methods that perform a database transaction. It is a key
   * value within the Pending Transaction Instance table
   * <P>
   * @param o -- The Financial Institution Identifier property.
   */
   public void setFinancialInstitutionIdentifier   (String o) {financialInstitutionIdentifier = o;}   
   /**
   * This method is used to set the primaryConsumerID property. This property is required
   * for all methods that perform a database transaction. It is a key
   * value within the Pending Transaction Instance table
   * <P>
   * @param o -- The Primary Consumer ID property.
   */
   public void setPrimaryConsumerID (String o) {primaryConsumerID = o;}   


   /**
   * This method is used to set the serviceProviderName property. This property is required
   * for the create, read, update, and delete methods .    
   * <P>
   * @param o  -- The Service Provider Name property.
   */

   public void setServiceProviderName(String o) {serviceProviderName = o;}   

   /**
   * This method is used to set the serviceRequestName property. This property is required
   * for the create, read, update, and delete methods .    
   * <P>
   * @param o  -- The Service Request Name property.
   */

   public void setServiceRequestName   (String o) {serviceRequestName = o;}   
   /**
   *  This method is used to set the goldTransactionAmount property.
   *  The input to this method is a GsCurrency object.
   *  When passed in, this object is used to obtain the
   *  transaction amount, precision, and currency code.
   */
   public void setGoldTransactionAmount(GsCurrency o) {

      long currency = 0;
      debug("in set Gold Transaction Amount");

      try {
         //if (o != null) {
            debug("in if of try");

            transactionCurrencyType = o.getIsoCode();

            transactionAmountPrecision = ( new Integer(o.getPrecision()) ).shortValue() ;
            transactionAmount = o.getDigits();
            transactionAmountPrecisionFlag = true;
            if (o.getDebitCredit() == 0) {
               transactionAmount = "-" + transactionAmount;
            }
            
            //Check whether transaction Amount is more than limit allowed.
            if (transactionAmount.length() > 15) 
            {
              debug("Transaction Amount is more than limit allowed. TxnAmount " + transactionAmount);
              goldTxnAmount = "Transaction Amount Value is out of Range";
              transactionAmount = null;
              transactionAmountPrecision = 0;
              transactionCurrencyType = null;
              transactionAmountPrecisionFlag = false;
            }
            
            debug("transactionCurrencyType = " + transactionCurrencyType);
            debug("transactionAmount = " + transactionAmount);
            debug("transactionAmountPrecision = " + transactionAmountPrecision);
         //}
         goldTransactionAmount = o;
         debug("goldTransactionAmount = " + goldTransactionAmount);
      }
      catch (Throwable e)  {

         debug("IfsPendingTransactionInstance.setTransactionAmount() - A throwable error has occurred " + e.toString());
         goldTransactionAmount = null;
         transactionAmount = null;
         transactionAmountPrecision = 0;
         transactionAmountPrecisionFlag = false; 
         transactionCurrencyType = null;
         goldTxnAmount = "Error in the format of Gold Transaction amount";
      }
      transactionAmountFlag = true;

   }   
   /**
   * This method is used to set the transactionAuthorizationTimestamp property. This is an output property
   * returned from the read and list methods.
   * <P>
   * @param o -- The Transaction Authorization Timestamp property.
   */
   public void setTransactionAuthorizationTimestamp(String o) {
      transactionAuthorizationTimestamp = o;

   }
   /**
   * This method is used to set the transactionAuthorizerConsumerID property. This is an output property
   * returned from the read and list methods.
   * <P>
   * @param o -- The Transaction Authorizer Consumer ID property.
   */
   public void setTransactionAuthorizerConsumerID(String o) {
      transactionAuthorizerConsumerID = o;

   }   
   /**
   * This method is used to set the transactionBeneficiary property. This is an optional input parameter
   * for the create and update methods. It also is returned from the read and list methods.
   * <P>
   * @param o -- The Transaction Beneficiary property.
   */

   public void setTransactionBeneficiary(String o) {transactionBeneficiary = o;}   

   /**
   * This method is used to set the transactionData property. This property is required
   * input for the create method. It is an optional parameter for the update method.
   * <P>
   * @param o -- The Transaction Data property.
   */ 
   public void setTransactionData(String data){

      transactionData = data;
   }   

   /**
   * This method is used to set the transactionDebtAccountClearingHouseID property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and  list methods.
   * <P>
   * @param o  -- The Transaction Debt Account Clearing House ID property.
   */

   public void setTransactionDebtAccountClearingHouseID(String o) {transactionDebtAccountClearingHouseID = o;}   
   /**
   * This method is used to set the transactionDebtAccountNickname property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and  list methods.
   * <P>
   * @param o  -- The Transaction Debt Account Nickname property.
   */
   public void setTransactionDebtAccountNickname   (String o) {transactionDebtAccountNickname = o;}   
   /**
   * This method is used to set the transactionDebtAccountNumber property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and  list methods.
   * <P>
   * @param o  -- The Transaction Debt Account Number property.
   */
   //public void setTransactionDebtAccountNumber(String o) {transactionDebtAccountNumber = o;}   
   public void setTransactionDebtAccountNumber(String o) {
     debug("in debt Account Number");
     transactionDebtAccountNumber = o;
   }   
   /**
   * This method is used to set the transactionDebtAccountSubType property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and  list methods.
   * <P>
   * @param o  -- The Transaction Debt Account SubType property.
   */
   public void setTransactionDebtAccountSubType (short o) {

      transactionDebtAccountSubType = o;
      transactionDebtAccountSubTypeFlag = true; 

   }

   /**
   * This method is used to set the transactionDebtAccountType property. This property  
   * is an optional input parameter for the create and update methods. It is also
   * returned from the read and  list methods.
   * <P>
   * @param o  -- The Transaction Debt Account Type property.
   */

   public void setTransactionDebtAccountType(String o) {transactionDebtAccountType = o;}   

   /**
   * This method is used to set the transactionInitiationTimestamp property. This property  
   * is a required input parameter for the  update method. 
   * For the create method, it assumes a default value in the database
   * when the update is made. This value returned from the read and list methods.
   * <P>
   * @param o -- Sets the Transaction Initiation Timestamp property.
   */
   
   public void setTransactionInitiationTimestamp(String o) {
      transactionInitiationTimestamp = o;
   }   
   /**
   * This method is used to set the transactionInitiatorConsumerID property. This property  
   * is a required input parameter for the  update method. 
   * For the create method, it assumes a default value in the database
   * when the update is made. This value returned from the read and list methods.
   * <P>
   * @param o -- Sets the Transaction Initiator Consumer ID property.
   */
   public void setTransactionInitiatorConsumerID(String o) {transactionInitiatorConsumerID = o;}   
   /**
   * This method is used to set the transactionReference property. This is an optional input parameter
   * for the create and update methods. It also is returned from the read and list methods.
   * <P>
   * @param o -- The Transaction Reference property.
   */
   public void setTransactionReference(String o) {transactionReference = o;}   

   /**
   * This method sets the transactionValueDate property. This is an optional input parameter
   * for the update method and a required parameter for the create method. 
   * It also is returned from the read and list methods.
   * <P>
   * @param o -- Used to set the Transaction Value Date property.
   */
   public void setTransactionValueDate(String o) {             

      transactionValueDate = o;

   }  

   /**
   * The update method is used to update rows of the PNDING_TXN_INST table.
   * corresponding to given search criteria. To use this method you must 
   * first create an instance of the class and set the all required fields. 
   * <P>
   *    IfsPendingTransactionInstance transAct = new IfsPendingTransactionInstance(); <BR>
   *    transAct.setUrl("jdbc:db2:DBTIFAV"); <BR>
   *    transAct.setDriverName("COM.ibm.db2.jdbc.app.DB2Driver"); <BR>
   *    //set required fields here. <BR> <BR>
   *    Financial Institution Identifier <BR>
   *    Primary Consumer ID  <BR>
   *    Transaction Initiator Consumer ID <BR>
   *    Service Request Name <BR>
   *    Transaction  Initiation Timestamp  <BR><BR>   
   *
   *    A "validation" method will be called upon
   *    execution of this method to assure the required fields are set.
   * <BR><BR>
   *    transAct.update();
   *
   */


   public void update() throws IfsException {

       String tempClearingHouseID = null;
       String tempType = null;
       short tempAccountSubType = 0;
       String tempAccountNumber = null;
       String tempAccountNickname = null;

      StringBuffer strBuf = null;

      //long tempTransactionAmount = 0;
 
      boolean accountClearingHouseIDFlag = false;
      boolean accountTypeFlag            = false;
      boolean accountSubTypeFlag         = false;
      boolean accountNumberFlag          = false;
      boolean accountNicknameFlag        = false;
 
      boolean transactionAmountFlag       =  false;
      boolean transactionCurrencyTypeFlag =  false;
      boolean precisionFlag               =  false;

      boolean statusCodeFlag        = false;
      boolean serviceProviderFlag   = false;
      boolean txnValueDateFlag      = false;
      boolean txnBeneficiaryFlag    = false;
      boolean txnReferenceFlag      = false;
      boolean txnAuthConsumerIDFlag = false;
      boolean txnAuthTimestampFlag  = false;
      boolean txnDataFlag           = false;

      String tempSqlString = "UPDATE " + TABLE_NAME + " SET "; 
      try {  
         debug("in IfsPendingTransactionInstance.update(): in update");

         resetTxnTypes();
         strBuf = new StringBuffer();

         //Check whether transaction status code was invalid
         if(txnStatusErrorCode != null)
         {
           debug("in IfsPendingTransactionInstance.update(): in txnStatusErrorCode check");
           String msg = txnStatusErrorCode;
           txnStatusErrorCode = null;
           throw new IfsException("Transaction Status Code Error",msg); 
         }

         //Check whether gold transaction amount was invalid
         if(goldTxnAmount != null)
         {
           String msg = goldTxnAmount;
           goldTxnAmount = null;
           throw new IfsException("Amount Format Exception", msg);
         }

         if ( (serviceProviderName != null) && (serviceProviderName.equals("") != true) )
         {
            strBuf.append(  SRVC_NM + " = ?,");
            serviceProviderFlag = true;
         }

            if ( ( (transactionDebtAccount != null) &&
                (transactionDebtAccount.getClearinghouseFIID() != null) ) &&
                (transactionDebtAccount.getClearinghouseFIID().equals("") != true ) ) {
               strBuf.append(  TXN_DB_ACCT_CLH_ID + " = ?, ");
               tempClearingHouseID = transactionDebtAccount.getClearinghouseFIID();
               accountClearingHouseIDFlag = true;

            }
            else if ( (transactionDebtAccountClearingHouseID != null) && (transactionDebtAccountClearingHouseID.equals("") != true ) )
            {
               strBuf.append(  TXN_DB_ACCT_CLH_ID + " = ?, ");
               tempClearingHouseID = transactionDebtAccountClearingHouseID;
               accountClearingHouseIDFlag = true;

            }
            if ( ( (transactionDebtAccount != null) && (transactionDebtAccount.getType() != null) ) &&
               (transactionDebtAccount.getType().equals("") != true ) )
            {
                strBuf.append(  TXN_DB_ACCT_TP + " = ?, ");
               tempType = transactionDebtAccount.getType();
               accountTypeFlag = true;

            }
            else if ( (transactionDebtAccountType != null) && (transactionDebtAccountType.equals("") != true ) ) 
            {
                strBuf.append(  TXN_DB_ACCT_TP + " = ?, ");
               tempType = transactionDebtAccountType;
               accountTypeFlag = true;

            }
            //if  ( (transactionDebtAccount != null) && (transactionDebtAccountSubTypeFlag ) )
            if (transactionDebtAccount != null)
            {
                strBuf.append(  TXN_DB_ACCT_SUB_TP + " = ?, ");
               //tempAccountSubType = transactionDebtAccountSubType;
               tempAccountSubType = (new Integer( transactionDebtAccount.getAccountSubType() ) ).shortValue();
               transactionDebtAccountSubTypeFlag = false;
               accountSubTypeFlag = true;


            }
           /* else if (transactionDebtAccountSubTypeFlag) {
                strBuf.append(  TXN_DB_ACCT_SUB_TP + " = ?, ");
               transactionDebtAccountSubTypeFlag = false;
               tempAccountSubType = transactionDebtAccountSubType;
               accountSubTypeFlag = true;

            } */

            if ( ( (transactionDebtAccount != null) && (transactionDebtAccount.getNumber() != null) ) &&
               (transactionDebtAccount.getNumber().equals("") != true) ) 
            {
                strBuf.append(  TXN_DB_ACCT_NBR + " = ?, ");
               tempAccountNumber = transactionDebtAccount.getNumber();
               accountNumberFlag = true;

            }
            else if ( (transactionDebtAccountNumber != null) && (transactionDebtAccountNumber.equals("") != true) )
            {
                strBuf.append(  TXN_DB_ACCT_NBR + " = ?, ");
               tempAccountNumber = transactionDebtAccountNumber;
               accountNumberFlag = true;

            }
            if ( ( (transactionDebtAccount != null) && (transactionDebtAccount.getNickname() != null) ) && (transactionDebtAccount.getNickname().equals("") != true) )
            {
                strBuf.append(  TXN_DB_ACCT_NKNM + " = ?, ");
               tempAccountNickname = transactionDebtAccount.getNickname();
               accountNicknameFlag = true;

            }
            else if ( (transactionDebtAccountNickname != null) && (transactionDebtAccountNickname.equals("") != true) )
            {
                strBuf.append(  TXN_DB_ACCT_NKNM + " = ?, ");
               tempAccountNickname = transactionDebtAccountNickname;
               accountNicknameFlag = true;

            }


         if ( (goldTransactionAmount != null) && (transactionAmount != null) && (transactionAmount.equals("") != true) )
         {
             strBuf.append(  TXN_AMT + " = ?, ");
             transactionAmountFlag = false;

             debug("in Transaction Amount. Transaction Amount is " + transactionAmount);
         }
         if (goldTransactionAmount != null && transactionAmountPrecisionFlag) {
            strBuf.append(  TXN_AMT_PRC + " = ?, ");
            transactionAmountPrecisionFlag = false;
            precisionFlag = true;

            debug("in Transaction Amount Precision. Transaction Amount Precision is " + transactionAmountPrecision);
         }
         if (goldTransactionAmount != null && transactionCurrencyType != null) {
            strBuf.append(  TXN_CURR_TP + " = ?, ");

            debug("in Transaction Amount Currency Type. Transaction currency Type is " + transactionCurrencyType);
         }

         if ( (transactionBeneficiary != null) && (transactionBeneficiary.equals("") != true) )
         {
            strBuf.append(  TXN_BENF + " = ?, ");
            txnBeneficiaryFlag = true;
         }
         if ( (transactionReference != null) && (transactionReference.equals("") != true) ) 
         {
            strBuf.append(  TXN_REF + " = ?, ");
            txnReferenceFlag = true;
         }
         if ( (transactionValueDate != null) && (transactionValueDate.equals("") != true ) ) 
         {
            strBuf.append(  TXN_VAL_DT + " = ?, ");
            txnValueDateFlag = true; 

            debug("value date is " + transactionValueDate);
         }
         if (transactionStatusCodeFlag) {
            strBuf.append(  TXN_STAT_CD + " = ?, ");
            transactionStatusCodeFlag = false;
            statusCodeFlag = true;
         }
         if ( (transactionAuthorizerConsumerID != null) && (transactionAuthorizerConsumerID.equals("") != true) ) 
         {
            strBuf.append(  TXN_AUTH_CNSMR_ID + " = ?, ");
            txnAuthConsumerIDFlag = true;
         }
         if ( (transactionAuthorizationTimestamp != null) && (transactionAuthorizationTimestamp.equals("") != true) )
         {
            strBuf.append(  TXN_AUTH_TMSTP + " = ?, ");
            txnAuthTimestampFlag = true;
         }
         if ( (transactionData != null) && (transactionData.equals("") != true) )
         {
            strBuf.append(  TXN_DATA + " = ? "); //no comma
            txnDataFlag = true;
         }
         debug("in IfsPendingTransactionInstance.update(): after setting updatable fields");



       //eliminate comma
         if( (strBuf != null) && (strBuf.length() != 0) )
         {
           debug("in strBuf != null and length !=0 ");
           if (strBuf.charAt(strBuf.length() - 1) == ',') 
           {
             try {
 
               strBuf.setLength(strBuf.length() - 1 );
               debug("in first set char at");
             }
             catch (Exception e) {

                debug("IfsPendingTransactionInstance.update() - strBuf.setCharAt has thrown an exception " + e.toString());
             }

           }
           else if (strBuf.charAt(strBuf.length() - 2) == ',')
           {
              try {

                strBuf.setLength(strBuf.length() - 2 );
                debug("in second set char at");
              } 
              catch (Exception e) {
                 debug("IfsPendingTransactionInstance.update() - strBuf.second setCharAt has thrown an exception " + e.toString());
              }
           }
         }
         else {
           debug("strBuf is empty");
           String msg = "At least One field should be updated";
           throw new IfsException("Update Not Successful",msg);
         }
         strBuf.append( " where " );
         strBuf.append(FI_ID + " = ? and ");        
         strBuf.append(PRIM_CNSMR_ID + " = ? and ");
         strBuf.append(TXN_INIT_CNSMR_ID +  " = ? and ");
         strBuf.append(SRVC_RQST_NM +  " = ? and ");
         strBuf.append(TXN_INIT_TMSTP +  " = ? ");

         setSqlString(tempSqlString + strBuf.toString());
         updateTxn = true;
         establishConnection();

         int i = 1;
         try {
             if (serviceProviderFlag) {  //5
                 preparedStatement.setString(i, serviceProviderName);
                 i++;
               debug("IfsPendingTransactionInstance.update() - serviceProviderName = " + serviceProviderName);
             }

            if (accountClearingHouseIDFlag) { //6

               preparedStatement.setString(i, tempClearingHouseID);
               i ++;


            }
            if (accountTypeFlag) { //7

               preparedStatement.setString(i, tempType);
               i ++;

            }
            if (accountSubTypeFlag) { //8

               preparedStatement.setShort(i, tempAccountSubType);
               i ++;
               transactionDebtAccount.setAccountSubType(0);

            }
            if (accountNumberFlag) {  //9

               preparedStatement.setString(i, tempAccountNumber);
               i ++;

            }
            if (accountNicknameFlag) {  //10

               //preparedStatement.setString(i, tempAccountNickname);
               
               //Account Nick Name column is defined as For Bit Data data type. Hence set it as bytes.

           //  include for space-@ fix
           //  String pTempAcctNickName = padding(tempAccountNickname, 30);
           //  byte[] tempAcctNicknameBytes = pTempAcctNickName.getBytes();

               byte[] tempAcctNicknameBytes = tempAccountNickname.getBytes(); 

               preparedStatement.setBytes(i, tempAcctNicknameBytes);
               i ++;

            }
         }
         catch (Throwable e) {

            debug("IfsPendingTransactionInstance.update(), after setting acct sql - A throwable error has occurred " + e.toString());
         }
         try {

             try {
             
            if (goldTransactionAmount != null && transactionAmount != null) { //11

               //preparedStatement.setLong(i, tempTransactionAmount);
               //Changes made here to set TXN_AMT as String as setLong is throwing exception
               try {
                 preparedStatement.setString(i, transactionAmount);
               } catch(SQLException e) {
                 preparedStatement.setString(i, "0");
               }
               i ++;

               debug("IfsPendingTransactionInstance.update(): transactionAmount = " + transactionAmount );
            }

            if (goldTransactionAmount != null && precisionFlag) {
                preparedStatement.setShort(i, transactionAmountPrecision);  //12
                i ++;
            }

            debug("IfsPendingTransactionInstance.update(): transactionAmountPrecision = " + transactionAmountPrecision );
            if (goldTransactionAmount != null && transactionCurrencyType != null) {  //13

               preparedStatement.setString(i, transactionCurrencyType);
               i ++;

               debug("IfsPendingTransactionInstance.update(): transactionCurrencyType = " + transactionCurrencyType );
            }
         
            //Initialize goldTransactionAmount to nulls.
            if (goldTransactionAmount != null) {
               goldTransactionAmount = null;
               transactionAmount = null;
               transactionAmountPrecision = 0;
               transactionAmountPrecisionFlag = false;  
               transactionCurrencyType = null;
               debug("after setting goldTransactionAmount fields. Setting goldTransaction Amount to null. Txn Amount = " + transactionAmount);
            }

             }
         catch (Throwable e) {

            debug("IfsPendingTransactionInstance.update(), currency setters - A throwable error has occurred " + e.toString());
         }
            //if (transactionBeneficiary != null) { //14
            if (txnBeneficiaryFlag) { //14

               //preparedStatement.setString(i, transactionBeneficiary);

               //Transaction Beneficiary column is defined as For Bit Data data type. Hence set it as bytes.

          //  include for space-@ fix
          //  String pTxnBenf = padding(transactionBeneficiary, 40);
          //  byte [] tempBenfBytes = pTxnBenf.getBytes();

              byte[] tempBenfBytes = transactionBeneficiary.getBytes();

              preparedStatement.setBytes(i, tempBenfBytes);
              i ++;

               debug("IfsPendingTransactionInstance.update(): transactionBeneficiary = " + transactionBeneficiary );
            } 
            if (txnReferenceFlag) {     //15

               preparedStatement.setString(i, transactionReference);
               i ++;

               debug("IfsPendingTransactionInstance.update(): transactionReference = " + transactionReference );
            }
            //if (transactionValueDate != null) {   //16
            if (txnValueDateFlag) {   //16

               preparedStatement.setString(i, transactionValueDate);
               i ++;

               debug("IfsPendingTransactionInstance.update(): transactionValueDate = " + transactionValueDate );
            }
         }
         catch (Throwable e) {

            debug("IfsPendingTransactionInstance.PNDING_TXN_INST(), after setting remaining props - A throwable error has occurred " + e.toString());
         }
         try {
            if (statusCodeFlag) {
            
                preparedStatement.setShort(i, transactionStatusCode);                                  
                i ++;
            }
            //if (transactionAuthorizerConsumerID != null) {
            if (txnAuthConsumerIDFlag) {

                preparedStatement.setString(i, transactionAuthorizerConsumerID);
                i ++;
            }

            //if (transactionAuthorizationTimestamp != null) {
            if (txnAuthTimestampFlag) {
                
                //Check to see whether the user has entered valid timestamp and if it is valid then change it to valid database format.
                if (validTimestamp(transactionAuthorizationTimestamp) == true)
                {
                  transactionAuthorizationTimestamp = changeTimestamp(transactionAuthorizationTimestamp);
                  preparedStatement.setString(i, transactionAuthorizationTimestamp);
                  i ++;
                }
            }
         }
         catch (Throwable e) {

            debug("IfsPendingTransactionInstance.PNDING_TXN_INST(), after setting remaining props - A throwable error has occurred " + e.toString());
         }
         //if (transactionData != null) {
         if (txnDataFlag) {

             //ByteArrayInputStream byteArray = new ByteArrayInputStream(transactionData.getBytes());
             debug("IfsPendingTransactionInstance.update(): transactionStatusCode = " + transactionStatusCode );
             //preparedStatement.setBinaryStream(i, byteArray, transactionData.getBytes().length); //18
             preparedStatement.setString(i,transactionData);
             i ++;

             debug("IfsPendingTransactionInstance.update(): transactionData length is = " + transactionData.length() );
         }
         //start where clause
           preparedStatement.setString(i, financialInstitutionIdentifier    );
            i++;

            preparedStatement.setString(i, primaryConsumerID);
            i++;
            preparedStatement.setString(i, transactionInitiatorConsumerID);
            i ++;
            preparedStatement.setString(i, serviceRequestName);                       
            i ++;
            
            //preparedStatement.setString(i, transactionInitiationTimestamp );
              //i ++;
            
            //Check to see whether the user has entered valid timestamp and if it is valid then change it to valid database format.
            if(validTimestamp(transactionInitiationTimestamp) == true)
            {
              transactionInitiationTimestamp = changeTimestamp(transactionInitiationTimestamp);
              preparedStatement.setString(i, transactionInitiationTimestamp );
            }

              //debug("IfsPendingTransactionInstance.update(): transactionData as string = " + transactionData );

              //debug("IfsPendingTransactionInstance.update(): transactionData as bit data = " + transactionData.getBytes() );
         execute();

       //resetInputParameters();
      }

      catch(IfsException e) {
        String msg = e.toString();
        throw new IfsException(e,msg);
      }

      catch(Exception e) { 
        debug("<BR>IfsPendingTransactionInstance update exception.<BR> " + e.toString());
        String msg = "IfsPendingTransactionInstance.update() has thrown an exception " + e.toString();
        throw new IfsException(e, msg);
      }

      catch (Throwable e) {

         String msg = "IfsPendingTransactionInstance.update() has thrown an exception  " +  e.toString();
         debug(msg);
         if (getMessage() == null) {
            message = msg;
         }

         throw new IfsException(e,  msg);         
      }



   }  //end of update().

   /**
   * Converts the IfsPendingTransactionInstance object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      if (hasRegisteredMethod("toString"))
         return super.toString();
      else
         return toString("\n");  // Return string from toString with
                                 // carriage return, line feed prefix.
   }

   /**
   * Converts the IfsPendingTransactionInstance object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix) {
      if (hasRegisteredMethod("toStringWithString"))
         return super.toString(prefix);

      //Cookie cookie = getIfsSession().getSessionKey();
      StringBuffer buff = new StringBuffer(250);
      buff.append(super.toString(prefix));
      debug("In IfsPendingTransactionInstance.toString(String prefix): getting request properties");

      buff.append(prefix + "| financialInstitutionIdentifier = " + financialInstitutionIdentifier );
      buff.append(prefix + "| primaryConsumerID = " + primaryConsumerID );
      buff.append(prefix + "| transactionInitiatorConsumerID = " + transactionInitiatorConsumerID );
      buff.append(prefix + "| serviceRequestName = " + serviceRequestName );
      buff.append(prefix + "| transactionInitiationTimestamp  = " + transactionInitiationTimestamp  );

      buff.append(prefix + "| serviceProviderName = " + serviceProviderName );

      debug("In IfsPendingTransactionInstance.toString(String prefix): getting GsAccount properties via toString");
      if (transactionDebtAccount != null) {

         debug("In IfsPendingTransactionInstance.toString(String prefix): transactionDebtAccount != null");
      }
      try {

         if (transactionDebtAccount != null) {
            buff.append(prefix + "| transactionDebtAccount  = " + transactionDebtAccount.toString(prefix)  );
         }

      }
      catch (Throwable e) {

         debug("In IfsPendingTransactionInstance.toString(String prefix): transactionDebtAccount.toString has thrown an exception" + e.toString() );
      }
      debug("In IfsPendingTransactionInstance.toString(String prefix): getting individual account properties");
      buff.append(prefix + "| transactionDebtAccountClearingHouseID  = " + transactionDebtAccountClearingHouseID  );
      buff.append(prefix + "| transactionDebtAccountType  = " + transactionDebtAccountType  );

      buff.append(prefix + "| transactionDebtAccountSubType = " + Short.toString(transactionDebtAccountSubType ) );
      buff.append(prefix + "| transactionDebtAccountNumber  = " + transactionDebtAccountNumber  );
      buff.append(prefix + "| transactionDebtAccountNickname     = " + transactionDebtAccountNickname);
      buff.append(prefix + "| transactionAmountPrecision; = " + Short.toString(transactionAmountPrecision) );
      buff.append(prefix + "| transactionCurrencyType  = " + transactionCurrencyType  );
      buff.append(prefix + "| transactionAmount          = " + transactionAmount );

      if (goldTransactionAmount != null) {
         buff.append(prefix + "| goldTransactionAmount  = " + goldTransactionAmount.toString() );
      }
      buff.append(prefix + "| transactionBeneficiary  = " + transactionBeneficiary  );
      buff.append(prefix + "| transactionReference   = " + transactionReference   );
      buff.append(prefix + "| transactionAuthorizerConsumerID  = " + transactionAuthorizerConsumerID  );

      buff.append(prefix + "| transactionAuthorizationTimestamp   = " + transactionAuthorizationTimestamp   );
      buff.append(prefix + "| transactionValueDate   = " + transactionValueDate   );
      buff.append(prefix + "| transactionStatusCode  = " + Short.toString(transactionStatusCode) );
      if (outputData != null) {      
         debug("In IfsPendingTransactionInstance.toString(String prefix): outputData.length = " + Integer.toString(outputData.length) );
         for (int i =0; i < outputData.length; i ++) {
            debug("In IfsPendingTransactionInstance.toString(String prefix): In for loop " );
            buff.append(prefix + "| outputData[" + Integer.toString(i) + "]   = " + outputData[i].toString(prefix)   );
         }

      }

      if (registry.isEmpty())
      {
         buff.append(prefix + "| Registry is empty");
      }
      else
      {
         Enumeration keys = registry.keys();
         while (keys.hasMoreElements())
         {
            String key = (String) keys.nextElement();
            buff.append(prefix + "| Registry contains: ");
            buff.append(prefix + "| " + key + " = ");
            buff.append(registry.get(key).toString());
         }
      }
      return buff.toString();

   }


   /**
   * The validate() method is a default method used to validate input to various 
   * methods (create, read, list, update, and delete). Validators are
   * called within the execute method of IfsJDBC. If a validator has been registered, 
   * it will be invoked instead.
   * This method also checks for valid dates and timestamps entered by the user.
   * And also verifies each field entered is of valid length.
   * <P> 
   * @return msg -- If the validation fails, a message from a properties file
   *                describing the error will be returned. Otherwise "OK" will
   *                be returned.
   */    

   public String validate() {

     //use locale provided by the getLocale() method of IfsObject
      ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",getLocale());    
      String str = super.validate();
      if (createTxn){
         debug("in validate: createTxn case");

         if (financialInstitutionIdentifier == null ||
             financialInstitutionIdentifier.equals("") ||
             primaryConsumerID  == null ||
             primaryConsumerID.equals("") ||
             transactionInitiatorConsumerID == null ||
             transactionInitiatorConsumerID.equals("") ||
             serviceRequestName == null ||
             serviceRequestName.equals("") ||
             transactionValueDate == null ||
             transactionValueDate.equals("") ||
             (validDate(transactionValueDate) == false) ||
             transactionData == null ||
             transactionData.equals("")  ||    //Validation of required fields ends here

             //validation of length starts here.
             ( (financialInstitutionIdentifier != null) && (financialInstitutionIdentifier.equals("") == false ) &&    
             (financialInstitutionIdentifier.length() > 8) ) ||
             ( (primaryConsumerID != null) && (primaryConsumerID.equals("") == false) &&
             (primaryConsumerID.length() > 30) ) ||
             ( (transactionInitiatorConsumerID != null) && (transactionInitiatorConsumerID.equals("") == false) &&
             (transactionInitiatorConsumerID.length() > 30) ) ||
             ( (serviceRequestName != null) && (serviceRequestName.equals("") == false) &&
             (serviceRequestName.length() > 8) ) ||
             ( (transactionData != null) && (transactionData.equals("") == false) &&
             (transactionData.length() > 31000) ) ||
             ( (transactionInitiationTimestamp != null) && (transactionInitiationTimestamp.equals("") == false) &&
             (validTimestamp(transactionInitiationTimestamp) == false) ) ||
             ( (serviceProviderName != null) && (serviceProviderName.equals("") == false) &&
             (serviceProviderName.length() > 8) ) ||
             ( (transactionDebtAccount.getClearinghouseFIID() != null) && (transactionDebtAccount.getClearinghouseFIID().equals("") == false) &&
             (transactionDebtAccount.getClearinghouseFIID().length() > 28) ) ||
             ( (transactionDebtAccount.getType() != null) && (transactionDebtAccount.getType().equals("") == false) &&
             (transactionDebtAccount.getType().length() > 3) ) ||
             ( (transactionDebtAccount.getNumber() != null) && (transactionDebtAccount.getNumber().equals("") == false) &&
             (transactionDebtAccount.getNumber().length() > 22) ) ||
             ( (transactionDebtAccount.getNickname() != null) && (transactionDebtAccount.getNickname().equals("") == false) &&
             (transactionDebtAccount.getNickname().length() > 30) ) ||
             ( (transactionBeneficiary != null) && (transactionBeneficiary.equals("") == false) &&
             (transactionBeneficiary.length() > 40) ) ||
             ( (transactionReference != null) && (transactionReference.equals("") == false) &&
             (transactionReference.length() > 40) ) )    //validation of length ends here          
         {
            status = VALIDATION_ERROR;
            debug("in validate: createTxn case: some field is null");
            str = bundle.getString("IfsPendingTransactionInstance.create.validation.invalidInput");
         }
      }
      else if (readTxn){
         debug("in validate: readTxn case");

         if (financialInstitutionIdentifier == null ||
             financialInstitutionIdentifier.equals("") ||
             primaryConsumerID  == null ||
             primaryConsumerID.equals("") ||
             transactionInitiatorConsumerID == null ||
             transactionInitiatorConsumerID.equals("") ||
             serviceRequestName == null ||
             serviceRequestName.equals("") ||
             transactionInitiationTimestamp == null ||
             transactionInitiationTimestamp.equals("") || 
             (validTimestamp(transactionInitiationTimestamp) == false) ||
              
             //validation of length starts here.
             ( (financialInstitutionIdentifier.equals("") == false ) &&
             (financialInstitutionIdentifier.length() > 8) ) ||
             ( (primaryConsumerID.equals("") == false) &&
             (primaryConsumerID.length() > 30) ) ||
             ( (transactionInitiatorConsumerID.equals("") == false) &&
             (transactionInitiatorConsumerID.length() > 30) ) ||
             ( (serviceRequestName.equals("") == false) &&
             (serviceRequestName.length() > 8) ) )       //validation of length ends here
         {
            status = VALIDATION_ERROR;
            str = bundle.getString("IfsPendingTransactionInstance.read.validation.invalidInput");
         }
      }
      else if (listTxn){
         debug("in validate: listTxn case");

         if (financialInstitutionIdentifier == null ||
             financialInstitutionIdentifier.equals("") ||
             primaryConsumerID  == null || 
             primaryConsumerID.equals("") ||

             //validation of length starts here.
             ( (financialInstitutionIdentifier.equals("") == false ) &&
             (financialInstitutionIdentifier.length() > 8) ) ||
             ( (primaryConsumerID.equals("") == false) &&
             (primaryConsumerID.length() > 30) ) )    //validation of length ends here
         {
            status = VALIDATION_ERROR;
            str = bundle.getString("IfsPendingTransactionInstance.list.validation.invalidInput");
         }
      }
      else if (updateTxn){
         debug("in validate: updateTxn case");

         if (financialInstitutionIdentifier == null ||
             financialInstitutionIdentifier.equals("") ||
             primaryConsumerID  == null ||
             primaryConsumerID.equals("") ||
             transactionInitiatorConsumerID == null ||
             transactionInitiatorConsumerID.equals("") ||
             serviceRequestName == null ||
             serviceRequestName.equals("") ||
             transactionInitiationTimestamp == null ||
             transactionInitiationTimestamp.equals("") || 
             (validTimestamp(transactionInitiationTimestamp) == false) ||
             ( (transactionValueDate.equals("") == false) && (validDate(transactionValueDate) == false) ) ||
             ( (transactionAuthorizationTimestamp.equals("") == false) && (validTimestamp(transactionAuthorizationTimestamp) == false) )  ||

             //validation of length starts here.
             ( (financialInstitutionIdentifier != null) && (financialInstitutionIdentifier.equals("") == false ) &&
             (financialInstitutionIdentifier.length() > 8) ) ||
             ( (primaryConsumerID != null) && (primaryConsumerID.equals("") == false) &&
             (primaryConsumerID.length() > 30) ) ||
             ( (transactionInitiatorConsumerID != null) && (transactionInitiatorConsumerID.equals("") == false) &&
             (transactionInitiatorConsumerID.length() > 30) ) ||
             ( (serviceRequestName != null) && (serviceRequestName.equals("") == false) &&
             (serviceRequestName.length() > 8) ) ||
             ( (transactionData != null) && (transactionData.equals("") == false) &&
             (transactionData.length() > 31000) ) ||
             ( (serviceProviderName != null) && (serviceProviderName.equals("") == false) &&
             (serviceProviderName.length() > 8) ) ||
             ( (transactionDebtAccount.getClearinghouseFIID() != null) && (transactionDebtAccount.getClearinghouseFIID().equals("") == false) &&
             (transactionDebtAccount.getClearinghouseFIID().length() > 28) ) ||
             ( (transactionDebtAccount.getType() != null) && (transactionDebtAccount.getType().equals("") == false) &&
             (transactionDebtAccount.getType().length() > 3) ) ||
             ( (transactionDebtAccount.getNumber() != null) && (transactionDebtAccount.getNumber().equals("") == false) &&
             (transactionDebtAccount.getNumber().length() > 22) ) ||
             ( (transactionDebtAccount.getNickname() != null) && (transactionDebtAccount.getNickname().equals("") == false) &&
             (transactionDebtAccount.getNickname().length() > 30) ) ||
             ( (transactionBeneficiary != null) && (transactionBeneficiary.equals("") == false) &&
             (transactionBeneficiary.length() > 40) ) ||
             ( (transactionReference != null) && (transactionReference.equals("") == false) &&
             (transactionReference.length() > 40) ) ||
             ( (transactionAuthorizerConsumerID != null) && (transactionAuthorizerConsumerID.equals("") == false) &&
             (transactionAuthorizerConsumerID.length() > 30) ) ) //validation of length ends here  
         {
            status = VALIDATION_ERROR;
            str = bundle.getString("IfsPendingTransactionInstance.update.validation.invalidInput");
         }
      }
      else if (deleteTxn){
         debug("in validate: deleteTxn case");

         if (financialInstitutionIdentifier == null ||
             financialInstitutionIdentifier.equals("") ||
             primaryConsumerID  == null ||
             primaryConsumerID.equals("") ||
             transactionInitiatorConsumerID == null ||
             transactionInitiatorConsumerID.equals("") ||
             serviceRequestName == null ||
             serviceRequestName.equals("") ||
             transactionInitiationTimestamp == null ||
             transactionInitiationTimestamp.equals("") || 
             (validTimestamp(transactionInitiationTimestamp) == false) ||
        
             //validation of length starts here.
             ( (financialInstitutionIdentifier.equals("") == false ) &&
             (financialInstitutionIdentifier.length() > 8) ) ||
             ( (primaryConsumerID.equals("") == false) &&
             (primaryConsumerID.length() > 30) ) ||
             ( (transactionInitiatorConsumerID.equals("") == false) &&
             (transactionInitiatorConsumerID.length() > 30) ) ||
             ( (serviceRequestName.equals("") == false) &&
             (serviceRequestName.length() > 8) ) )  //validation of length ends here
         {
            status = VALIDATION_ERROR;
            debug("in validate: deleteTxn case: in if case");
            str = bundle.getString("IfsPendingTransactionInstance.delete.validation.invalidInput");

         }
      }

      return str;
   }   

   /**
  *Funtion to pad spaces to the right of a string for the given length.
  */
  public String padding(String inputString, int width) {

     String outputString;

     if (inputString == null)        //allow null input string
         outputString = "";
     else
         outputString = inputString;

     while (outputString.length() < width)
     {
        outputString = outputString + " ";
     }

  return (outputString);

  }//end of padding(string, int)


   /**
   * Accepts a Date in String format as input parameter.
   * validates the date and returns boolean value.
   * @return flag -- true or false (Date is valid or invalid)
   **/
   public boolean validDate(String inDate) {

   try {

     inDate.trim();

     StringTokenizer strtkn = new StringTokenizer(inDate , "-");

     if ( strtkn.countTokens() != 3 ) return false;
   
     String yearstr  = strtkn.nextToken();
     String monthstr = strtkn.nextToken(); 
     String daystr   = strtkn.nextToken();

     int day   = 0;
     int month = 0;
     int year  = 0;
     int len   = 0;

     yearstr = IfsApi.chop(yearstr,'0');
     len = yearstr.length();
     if (len != 4)
        return false;

        day = (new Integer(daystr)).intValue();
      month = (new Integer(monthstr)).intValue();
       year = (new Integer(yearstr)).intValue();

     return IfsApi.isValidDate(month,day,year);

     } catch (Exception e) {
       debug ("<BR> Exception occured in validDate() function of IfsPendingTransactionInstance() " + e.toString());
       return false;
     }

   }

   /**
   *Accepts a Timestamp in String format as input parameter.
   *Validates the timestamp and returns a boolean value.
   *
   * return flag -- true of false (Date is valid or invalid)
   */
   public boolean validTimestamp(String inTimestamp) {

     try {
       inTimestamp = inTimestamp.trim();

       StringTokenizer strtkn = new StringTokenizer(inTimestamp , "-.");
    
       if ( strtkn.countTokens() != 7 ) return false;

       String year  = strtkn.nextToken();
       String month = strtkn.nextToken();
       String day   = strtkn.nextToken();
 
       String hours     = strtkn.nextToken();
       String minutes   = strtkn.nextToken();
       String seconds   = strtkn.nextToken(); 
       String micro_sec = strtkn.nextToken();

       String date = year + "-" + month + "-" + day;

       if (validDate(date) == false) return false;

       int hoursi     = (new Integer(hours)).intValue();
       int minutesi   = (new Integer(minutes)).intValue();
       int secondsi   = (new Integer(seconds)).intValue();
       int micro_seci = (new Integer(micro_sec)).intValue();


       if ( (hoursi<0 || hoursi>23)   || 
          (minutesi<0 || minutesi>59) || 
          (secondsi<0 || secondsi>59) || 
          (micro_seci<0 || micro_seci>999999) )
         return false;
       else
         return true;
     } catch(Exception e) {
         return false;
     }

   }  // end of validTimestamp(String)


 
   /**
   * Accepts a valid Timestamp in String format as input parameter.
   * Inserts a preceding '0' for hours, minutes and seconds fields if they are in single digits.
   *
   * @ return timestamp - appending 0 for hrs,mins or secs if they are single digits.
   */ 
   
   public String changeTimestamp(String inputTimestamp) {
   
     inputTimestamp = inputTimestamp.trim();
   
    
     StringTokenizer strtkn = new StringTokenizer(inputTimestamp , "-.");

     String year      = strtkn.nextToken();
     String month     = strtkn.nextToken();
     String day       = strtkn.nextToken();
     String hours     = strtkn.nextToken();
     String minutes   = strtkn.nextToken();
     String seconds   = strtkn.nextToken();
     String micro_sec = strtkn.nextToken();
 
     try {
       int hoursi     = (new Integer(hours)).intValue();
       int minutesi   = (new Integer(minutes)).intValue();
       int secondsi   = (new Integer(seconds)).intValue();
       int micro_seci = (new Integer(micro_sec)).intValue();

       if (hours.length() == 1 && hoursi < 10)
         hours = "0" + hours;
    
       if (minutes.length() == 1 && minutesi < 10)
         minutes = "0" + minutes;
   
       if (seconds.length() == 1 && secondsi < 10)
         seconds = "0" + seconds;

       String outputTimestamp = year + "-" + month + "-" + day + "-" + hours + "." + minutes + "." + seconds + "." + micro_sec;
    
    
       return outputTimestamp;
     } catch(Exception e) {
       String errTimestamp = "0000-00-00-00.00.00.000000"; 
       return errTimestamp; 
     }

   } // end of changeTimestamp(String)

   /**
   * The create method is used to create a row in the PNDING_TXN_INST table. To use 
   * this method you must first create an instance of the class and set
   * the all required fields. The transactionInitiationTimestamp is
   * not passed in the sql issued by this method. If one is set, it is ignored 
   * transactionStatusCode is always set to 0 and will be ignored if entered.
   * transactionAuthorizerConsumerID and transactionAuthorizationTimestamp columns are set in update 
   * and will be ignored while creating record.
   * <P>
   *    IfsPendingTransactionInstance transAct = new IfsPendingTransactionInstance(ifsSession); <BR>
   *    transAct.setUrl("jdbc:db2:DBTIFAV"); <BR>
   *    transAct.setDriverName("COM.ibm.db2.jdbc.app.DB2Driver"); <BR><BR>
   *    //set required fields here. <BR> <BR>
   *
   *    Financial Institution Identifier <BR>
   *    Primary Consumer ID  <BR>
   *    Transaction Initiator Consumer ID <BR>
   *    Service Request Name <BR>
   *    Transaction  Initiation Timestamp  <BR>
   *    Service Provider Name <BR>
   *    Transaction Value Date <BR>
   *    Transaction Status Code should be set to 0 (the default value)
   *    Transaction Data
   * <P>
   *    A "validation" method will be called upon 
   *    execution of this method to assure the required fields are set. <BR>
   *
   *    transAct.create();
   *
   */

   public void create () throws IfsException { 

      debug("<BR>Entry to prepStmtCreate.");

      String tempSqlStr = null;
      StringBuffer updateStr = new StringBuffer();
      StringBuffer colStr = new StringBuffer();
      String tempClearingHouseID = null;
      String tempType = null;
      short tempAccountSubType = 0;
      String tempAccountNumber = null;
      String tempAccountNickname = null;
      boolean accountClearingHouseIDFlag = false;
      boolean accountTypeFlag = false;
      boolean accountSubTypeFlag = false;
      boolean accountNumberFlag = false;

      boolean accountNicknameFlag = false;

      boolean transactionAmountFlag =  false;
      boolean transactionCurrencyTypeFlag =  false;

      boolean txnBenfFlag = false;
      boolean txnRefFlag  = false;

      resetTxnTypes();

      try {
         createTxn = true; 

         //Check whether gold transaction amount was invalid
         if(goldTxnAmount != null)
         {
           String msg = goldTxnAmount;
           goldTxnAmount = null; 
           throw new IfsException("Amount Format Exception",msg);
         }

         colStr.append(FI_ID + ", "  ); 
         colStr.append(PRIM_CNSMR_ID + ", "  ); 
         colStr.append(TXN_INIT_CNSMR_ID + ", "  ); 
         colStr.append(SRVC_RQST_NM + ", "  );        
         colStr.append(SRVC_NM + ", "  ); 
         updateStr.append( "?, ?, ?, ?, ?");
          // set optional fields


            if (getTransactionInitiationTimestamp().equals("") != true ) {
              updateStr.append(", ?");
              colStr.append(TXN_INIT_TMSTP + ","  );
              debug("IfsPendingTransactionInstance.prepStmtCreate(), inside if: transactionInitiationTimestamp = ");
            }


         
         debug("In IfsPendingTransactionInstance.create() - before getting accout info");

         if (transactionDebtAccount == null) {

            debug("In IfsPendingTransactionInstance.create() - transactionDebtAccount == null");
         }
         else

            debug("In IfsPendingTransactionInstance.create() - transactionDebtAccount != null: " );
         
         try {

            if ( ( (transactionDebtAccount != null) &&
                (transactionDebtAccount.getClearinghouseFIID() != null) ) &&
               (transactionDebtAccount.getClearinghouseFIID().equals("") != true ) )
            {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_CLH_ID + ", "  );
               tempClearingHouseID = transactionDebtAccount.getClearinghouseFIID();
               accountClearingHouseIDFlag = true;

            }
            else if ( (transactionDebtAccountClearingHouseID != null) && 
                    (transactionDebtAccountClearingHouseID.equals("") != true ) )
            {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_CLH_ID + ", "  ); 
               tempClearingHouseID = transactionDebtAccountClearingHouseID;
               accountClearingHouseIDFlag = true;

            }
             
            if ( ( (transactionDebtAccount != null) && (transactionDebtAccount.getType() != null) )             && (transactionDebtAccount.getType().equals("") != true ) )
            {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_TP + ", "  ); 
               tempType = transactionDebtAccount.getType();
               accountTypeFlag = true;
            }
            else if ( (transactionDebtAccountType != null) && (transactionDebtAccountType.equals("") != true ) )
            {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_TP + ", "  );
               tempType = transactionDebtAccountType;
               accountTypeFlag = true;
            }
            
            //if ( (transactionDebtAccount != null) && (transactionDebtAccountSubTypeFlag )) {
            if (transactionDebtAccount != null)  {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_SUB_TP + ","  );
               tempAccountSubType = (new Integer( transactionDebtAccount.getAccountSubType() ) ).shortValue();
               transactionDebtAccountSubTypeFlag = false;
               accountSubTypeFlag = true;
            }
            /* else if (transactionDebtAccountSubTypeFlag) {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_SUB_TP + ","  );
               transactionDebtAccountSubTypeFlag = false;
               tempAccountSubType = transactionDebtAccountSubType;
               accountSubTypeFlag = true;
            }  */ 

            if ( ( (transactionDebtAccount != null) && (transactionDebtAccount.getNumber() != null))             && (transactionDebtAccount.getNumber().equals("") != true) )
            {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_NBR + ", "  );
               tempAccountNumber = transactionDebtAccount.getNumber();
               accountNumberFlag = true;
            }
            else if ( (transactionDebtAccountNumber != null) && (transactionDebtAccountNumber.equals("") != true) )
            {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_NBR + ", "  );
               tempAccountNumber = transactionDebtAccountNumber;
               accountNumberFlag = true;
            }  

            if ( ( (transactionDebtAccount != null) && (transactionDebtAccount.getNickname() != null) ) && (transactionDebtAccount.getNickname().equals("") != true) )
            {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_NKNM + ","  ); 
               tempAccountNickname = transactionDebtAccount.getNickname();
               accountNicknameFlag = true;
            }
            else if ( (transactionDebtAccountNickname != null) && (transactionDebtAccountNickname.equals("") != true) )
            {
               updateStr.append(", ?"); 
               colStr.append(TXN_DB_ACCT_NKNM + ","  ); 
               tempAccountNickname = transactionDebtAccountNickname;
               accountNicknameFlag = true;
           }    

         }
         catch (Throwable e) {

            debug("IfsPendingTransactionInstance.tCreate(): acct stuff has thrown an Exception = " + e.toString( ) );
         }

          if (transactionAmount != null) {
            updateStr.append(", ?"); 
            colStr.append(TXN_AMT + ","  ); 
            debug("IfsPendingTransactionInstance.prepStmtCreate(), inside if: transactionAmount = " + transactionAmount);
          }
         
          if(transactionAmountPrecisionFlag) {
            updateStr.append(", ?"); 
            colStr.append(TXN_AMT_PRC + ","  ); 
          }
         
         if (transactionCurrencyType != null) {       
            updateStr.append(", ?"); 
            colStr.append(TXN_CURR_TP + ","  ); 
            debug("IfsPendingTransactionInstance.prepStmtCreate(), inside if: transactionCurrencyType = " + transactionCurrencyType);
         }
         else
         {
            debug("Couldn't set Currency Type. Currency Type Value is " +  transactionCurrencyType);
         }  

         //if (transactionBeneficiary != null) {
         if (transactionBeneficiary.equals("") != true) 
         {
            updateStr.append(", ?"); 
            colStr.append(TXN_BENF + ","  );
            txnBenfFlag = true;
            debug("IfsPendingTransactionInstance.prepStmtCreate(), inside if: transactionBeneficiary = " + transactionBeneficiary);

         }  
         if (transactionReference.equals("") != true) 
         {
            updateStr.append(", ?"); 
            colStr.append(TXN_REF + ","  ); 
            txnRefFlag = true;
            debug("IfsPendingTransactionInstance.prepStmtCreate(), inside if: transactionReference = " + transactionReference);
         }  
         if (transactionValueDate != null) {
            updateStr.append(", ?"); 
            colStr.append(TXN_VAL_DT + ","  ); 
            debug("IfsPendingTransactionInstance.prepStmtCreate(), inside if: transactionValueDate = " + transactionValueDate);
         } 
         transactionStatusCode = 0;  //ensure status code = 0
         updateStr.append(", ?"); 
         colStr.append(TXN_STAT_CD + ","  ); 
         transactionStatusCodeFlag = false;
         updateStr.append(", ?"); 
         colStr.append(TXN_DATA  ); 

         tempSqlStr = "insert into " + TABLE_NAME +  " ( " + colStr.toString() + " ) " + " values( " +  updateStr.toString() + " )";
         setSqlString(tempSqlStr);
          
       //int loopSize = colStr.toString().length();

         establishConnection();
         try {

            try {

               preparedStatement.setString(1, financialInstitutionIdentifier);
               preparedStatement.setString(2, primaryConsumerID);
               preparedStatement.setString(3, transactionInitiatorConsumerID);
               preparedStatement.setString(4, serviceRequestName);
               preparedStatement.setString(5, serviceProviderName);
            }
            catch (Throwable e) {

               debug("IfsPendingTransactionInstance.prepStmtCreate(), after setting required props - A throwable error has occurred " + e.toString());
            }
            int i = 6;
            try {

              if (getTransactionInitiationTimestamp().equals("") != true ) {

              if (validTimestamp(transactionInitiationTimestamp))
                transactionInitiationTimestamp = changeTimestamp(transactionInitiationTimestamp);

                  preparedStatement.setString(i, transactionInitiationTimestamp);  
                  i ++;
               debug("IfsPendingTransactionInstance.prepStmtCreate(): transactionInitiationTimestamp = " );
               }



               if (accountClearingHouseIDFlag) { //6

                  preparedStatement.setString(i, tempClearingHouseID);
                  i ++;


               }
                
               if (accountTypeFlag) { //7

                  preparedStatement.setString(i, tempType);
                  i ++;

               }
                
               if (accountSubTypeFlag) { //8

                 preparedStatement.setShort(i, tempAccountSubType);
                 i ++;


                 transactionDebtAccount.setAccountSubType(0);
               } 

               if (accountNumberFlag) {  //9

                  preparedStatement.setString(i, tempAccountNumber);
                  i ++;

               }  
               if (accountNicknameFlag) {  //10


                  //preparedStatement.setString(i, tempAccountNickname);
                  //Account Nick Name column is defined as For Bit Data data type. Hence set it as bytes.

              //  include for space-@ fix
              //  String pTempAcctNickName = padding(tempAccountNickname, 30);
              //  byte[] tempAcctNicknameBytes = pTempAcctNickName.getBytes();

                  byte[] tempAcctNicknameBytes = tempAccountNickname.getBytes();

                  preparedStatement.setBytes(i, tempAcctNicknameBytes);
                  i ++;

               }  
            }
            catch (Throwable e) {

               debug("IfsPendingTransactionInstance.create(), after setting acct sql - A throwable error has occurred " + e.toString());
            }
            
            try {

                if (transactionAmount != null) { //11

                  try {
                    debug("in transactionAmount and about to set");
                    //preparedStatement.setLong(i, tempTransactionAmount);
                    preparedStatement.setString(i, transactionAmount);
                  } catch (Exception e) {
                    preparedStatement.setString(i, "0");
                    debug("in catch of transactionAmount and about to set it 0");
                  }
                  i ++;

                  debug("IfsPendingTransactionInstance.prepStmtCreate(): transactionAmount = " + transactionAmount );
                 }      
              
               if (transactionAmountPrecisionFlag) {

                preparedStatement.setShort(i, transactionAmountPrecision);  //12
                transactionAmountPrecisionFlag = false;
                i ++;
               debug("IfsPendingTransactionInstance.prepStmtCreate(): transactionAmountPrecision = " + transactionAmountPrecision );
               }

               if (transactionCurrencyType != null) {  //13

                  preparedStatement.setString(i, transactionCurrencyType);
                  i ++;

                  debug("IfsPendingTransactionInstance.prepStmtCreate(): transactionCurrencyType = " + transactionCurrencyType );
               }  
            
               if(goldTransactionAmount != null)
               {
                 goldTransactionAmount = null;
                 transactionAmount = null;
                 transactionAmountPrecisionFlag = false;
                 transactionCurrencyType = null;
                 debug("after setting goldTransactionAmount fields. Setting goldTransaction Amount to null. Txn Amount = " + transactionAmount);
               }

               //if (transactionBeneficiary != null) { //14
               if (txnBenfFlag) { //14
              
                  //preparedStatement.setString(i, transactionBeneficiary);
                  //Transaction Beneficiary column is defined as For Bit Data data type. Hence set it as bytes.

               // include for space-@ fix
               // String pTxnBenf = padding(transactionBeneficiary, 40);
               // byte [] tempBenfBytes = pTxnBenf.getBytes();

                  byte[] tempBenfBytes = transactionBeneficiary.getBytes();

                  preparedStatement.setBytes(i, tempBenfBytes);
                  i ++;

                  debug("IfsPendingTransactionInstance.prepStmtCreate(): transactionBeneficiary = " + transactionBeneficiary );
               }  

               //if (transactionReference != null) {     //15
               if (txnRefFlag) {     //15

                  preparedStatement.setString(i, transactionReference);
                  i ++;

                  debug("IfsPendingTransactionInstance.prepStmtCreate(): transactionReference = " + transactionReference );
               } 
               if (transactionValueDate != null) {   //16

                  preparedStatement.setString(i, transactionValueDate);
                  i ++;

                  debug("IfsPendingTransactionInstance.prepStmtCreate(): transactionValueDate = " + transactionValueDate );
               }
             }
            catch (Throwable e) {

               debug("IfsPendingTransactionInstance.create(), after setting remaining props - A throwable error has occurred " + e.toString());
            } 
            try {

               transactionStatusCode = 0;  //ensure status code = 0  //17
               preparedStatement.setShort(i, transactionStatusCode);                                  
               i ++;
            }
            catch (Throwable e) {

               debug("IfsPendingTransactionInstance.create(), after setting remaining props - A throwable error has occurred " + e.toString());
            }
            //if (transactionData != null) {
            
            //ByteArrayInputStream byteArray = new ByteArrayInputStream(transactionData.getBytes());
            debug("IfsPendingTransactionInstance.prepStmtCreate(): transactionStatusCode = " + transactionStatusCode );
            //preparedStatement.setBinaryStream(i, byteArray, transactionData.getBytes().length); //18

                 debug("IfsPendingTransactionInstance.prepStmtCreate(): transaction data length is " + transactionData.length());

                 preparedStatement.setString(i, transactionData);
                 //i ++;



            execute();  //call execute to update table
         }

         catch (IfsException e) {
           String msg = e.toString();           
           debug("in IfsException catch of create " + e.toString());
           throw new IfsException(e,msg);
         }
 
         catch (Exception e) {
           debug("<BR>IfsPendingTransactionInstance create exception.<BR> " + e.toString());
           String msg = "IfsPendingTransactionInstance.create() has thrown an exception. " + e.toString();
           throw new IfsException(e,msg);
         }
         catch (Throwable e) {
            debug("IfsPendingTransactionInstance.prepStmtCreate() After setters- A throwable error has occurred " + e.toString());
            String msg = "IfsPendingTransactionInstance.create() has thrown an exception  " +  e.toString();           
            debug(msg);
            if (getMessage() == null) {
               message = msg;
            }
            throw new IfsException(e,  msg);
         }
      }

      catch (IfsException e) {
        String msg = e.toString();
        debug("in outer IfsException catch of create " + msg);
        throw new IfsException(e,msg);
      }
   
      catch (Exception e) {
         debug("<BR>IfsPendingTransactionInstance create exception.<BR> " + e.toString());
         String msg = "IfsPendingTransactionInstance.create() has thrown an exception. " + e.toString();
         throw new IfsException(e,msg);
      }
      
      catch (Throwable e) {
         if (status == MISSING_STRING_DELIMETER) {
                //call replace, rebuild sqlstring, then execute 
         }
         debug("IfsPendingTransactionInstance.prepStmtCreate() - A throwable error has occurred " + e.toString());
         //String msg = "IfsPendingTransactionInstance.prepStmtCreate() has thrown an exception  " + e.toString();
         String msg = "IfsPendingTransactionInstance.create() has thrown an exception " + e.toString();
         if (getMessage() == null) {
            message = msg;
         }
         throw new IfsException(e,  msg);
      }
   }  //end of create(). 

}
