package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsPendingTransactionInstanceDetail.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
// ***************************************************************************
// Change History
//
//  Date                         Name                Reason For Change
// ------                       ------              -------------------
//
//   --                          --                 Initial Creation
//
// 01/19/2001                  D.Deshpande          Fixed bugs and got code
//                                                  working
//
//
// ****************************************************************************

import java.util.Locale;
import java.sql.*;
import java.math.BigDecimal;
import java.util.Enumeration;

/**
* The IfsPendingTransactionInstanceDetail is a data class that contains 
* query data obtained from the list and read methods of the IfsPendingTransactionInstance class.
* An array of type IfsPendingTransactionInstanceDetail within the
* IfsPendingTransactionInstance class is populated whenever a query is successfully
* executed.
* This class is a subclass of IfsObject
*/

public class IfsPendingTransactionInstanceDetail extends IfsObject {

//Instance variables for this class

   private String       financialInstitutionIdentifier        = null;
   private String       primaryConsumerID                     = null;
   private String       transactionInitiatorConsumerID        = null;
   private String       transactionInitiationTimestamp        = null; 
   private String       serviceProviderName                   = null;
   private String       serviceRequestName                    = null;
   private String       transactionDebtAccountClearingHouseID = null;
   private String       transactionDebtAccountType            = null;
   private short        transactionDebtAccountSubType         = 0;
   private String       transactionDebtAccountNumber          = null;
   private String       transactionDebtAccountNickname        = null;
   private GsCurrency   goldTransactionAmount                 = null;          
   private String       transactionAmount                     = null;
   private short        transactionAmountPrecision            = 0;
   private String       transactionCurrencyType               = null;
   private String       transactionBeneficiary                = null;
   private String       transactionReference                  = null;
   private String       transactionValueDate                  = null;
   private short        transactionStatusCode                 = 0;
   private String       transactionStatusCode_text            = null;
   private String       transactionData                       = null;
   private String       transactionAuthorizerConsumerID       = null;
   private String       transactionAuthorizationTimestamp     = null;


   /**
   * Default constructor 
   */   
   public IfsPendingTransactionInstanceDetail() {

   }
   /**
   * An instance of the GsAccount class 
   */
   private GsAccount    transactionDebtAccount                = null;
   /**
   * This method is used to set the transactionDebtAccount property.
   * This setter will extract the properties of the transactionDebtAccount
   * for use in various transaction methods. These folllowing properties will be set: <BR>
   * transactionDebtAccountType <BR>
   * transactionDebtAccountNumber <BR>
   * transactionDebtAccountNickname <BR>
   * transactionDebtAccountSubType 
   * <P>
   * @param acct --  a com.ibm.ifs.gold.GsAccount object
   */
   public void setTransactionDebtAccount(GsAccount acct) {

       transactionDebtAccount = acct;
       if (acct != null) {
       
       transactionDebtAccount = acct;
       transactionDebtAccountType =  acct.getType();
       transactionDebtAccountNumber = acct.getNumber ();
       transactionDebtAccountNickname = acct.getNickname();
       transactionDebtAccountClearingHouseID = acct.getClearinghouseFIID();
       transactionDebtAccountSubType = (new Integer( acct.getAccountSubType() ) ).shortValue(); 
       }
      //transactionDebtAccountSubType = 

   }

   /**
   * This method returns the transactionDebtAccount property.
   * <P>
   * @return transactionDebtAccount --  a com.ibm.ifs.gold.GsAccount object
   */
   public GsAccount getTransactionDebtAccount() {
    return transactionDebtAccount;
   }

  /** 
   *This is defined as an abstract method in IfsObject, 
   * so we must provide a definition here.
   * <P>
   * @return  -1
   */
   public int getSD_OWNER() {return (-1);} 

   /**
   * This method returns the financialInstitutionIdentifier property. . 
   * <P>
   * @return financialInstitutionIdentifier -- The Financial Institution Identifier property.
   */
   public String getFinancialInstitutionIdentifier() {return financialInstitutionIdentifier;}
 
   /** 
   * This method returns the primaryConsumerID property. .
   * <P>
   * @return primaryConsumerID -- The Primary Consumer ID property.
   */ 
   public String getPrimaryConsumerID() {return primaryConsumerID;}

   /**
   * This method returns the serviceProviderName property. .    
   * <P>
   * @return serviceProviderName  -- The Service Provider Name property.
   */
   public String getServiceProviderName() {return serviceProviderName;} 


   /**
   * This method returns the serviceRequestName property. .    
   * <P>
   * @return serviceRequestName  -- The Service Request Name property.
   */
   public String getServiceRequestName() {return serviceRequestName;} 
   /**
   * This method returns the goldTransactionAmount. This property 
   * is an  instance of the GsCurrency class.  
   * <P>
   * @return goldTransactionAmount  -- The Gold Transaction Amount property.
   */
   public GsCurrency getGoldTransactionAmount() {return goldTransactionAmount;}   


   /**
   * This method returns the transactionAmount . This property 
   * is populated when the GsCurrency instance within this class is set.  
   * <P>
   * @return ransactionAmount  -- The Transaction Amount property.
   */
   public String getTransactionAmount() {return transactionAmount;}
   /**
   * This method returns the transactionAmountPrecision 
   * obtained from the GsCurrency instance within this class.  
   * <P>
   * @return transactionAmountPrecision   -- The Transaction Amount Precision property.
   */
   public short getTransactionAmountPrecision() {
      if (goldTransactionAmount != null) {
         try {  

            transactionAmountPrecision = (new Integer(goldTransactionAmount.getPrecision() ) ).shortValue();

         }
         catch (Throwable e) {
            
         }
      }
      return transactionAmountPrecision;

   } 
   /**
   * This method returns the transactionAuthorizationTimestamp property. 
   * <P>
   * @return transactionAuthorizationTimestamp  -- The transaction Authorization Timestamp property.
   */

   public String getTransactionAuthorizationTimestamp() {
      return transactionAuthorizationTimestamp;
   }

   /**
   * This method returns the transactionAuthorizerConsumerID property. 
   * <P>
   * @return transactionAuthorizerConsumerID  -- 
             The Transaction Authorizer Consumer ID property.
   */

   public String getTransactionAuthorizerConsumerID() {
      return transactionAuthorizerConsumerID;
   }   

   /**
   * This method returns the transactionBeneficiary property. 
   * <P>
   * @return transactionBeneficiary  -- 
             The Transaction Beneficiary property.
   */

   public String getTransactionBeneficiary() {return transactionBeneficiary;}   
 
   /**
   * This method returns the transactionCurrencyType 
   * obtained from the GsCurrency instance within this class. This is the ISO
   * code associated with the amount.
   * <P>
   * @return transactionCurrencyType  -- The Transaction Currency Type property.
   */
   public String getTransactionCurrencyType() {

      if (goldTransactionAmount != null) {
         try {  

            transactionCurrencyType = goldTransactionAmount.getIsoCode();

         }
         catch (Throwable e) {
            
         }
      }

      return transactionCurrencyType;
   }   
   /**
   * This method returns the transactionData property 
   * <P>
   * @return transactionData  -- The Transaction Data property.
   */

   public String getTransactionData() {return transactionData;}
   /**
   * This method returns the transactionDebtAccountClearingHouseID property 
   * <P>
   * @return transactionDebtAccountClearingHouseID  -- The Transaction Data property.
   */
   public String getTransactionDebtAccountClearingHouseID() {
     return transactionDebtAccountClearingHouseID;
   }   

   /**
   * This method returns the transactionDebtAccountNickname property 
   * <P>
   * @return transactionDebtAccountNickname  -- The Transaction Debt Account Nickname property.
   */
   public String getTransactionDebtAccountNickname   () {return transactionDebtAccountNickname;}   

   /**
   * This method returns the transactionDebtAccountNumber property 
   * <P>
   * @return transactionDebtAccountNumber  -- The Transaction Debt Account Number property.
   */
   public String getTransactionDebtAccountNumber() {return transactionDebtAccountNumber;}   

   /**
   * This method returns the transactionDebtAccountSubType property 
   * <P>
   * @return transactionDebtAccountSubType  -- The Transaction Debt Account SubType property.
   */
   public short getTransactionDebtAccountSubType () {return transactionDebtAccountSubType;}   

   /**
   * This method returns the transactionDebtAccountType property 
   * <P>
   * @return transactionDebtAccountType  -- The Transaction Debt Account Type property.
   */
   public String getTransactionDebtAccountType() {return transactionDebtAccountType;}   

   /**
   * This method returns the transactionInitiationTimestamp property 
   * <P>
   * @return transactionInitiationTimestamp  -- The Transaction Initiation Timestamp property.
   */
   public String getTransactionInitiationTimestamp() {return transactionInitiationTimestamp;}   

   /**
   * This method returns the transactionInitiatorConsumerID property 
   * <P>
   * @return transactionInitiatorConsumerID  -- The Transaction Initiator Consumer ID property.
   */
   public String getTransactionInitiatorConsumerID() {return transactionInitiatorConsumerID;}   

   /**
   * This method returns the transactionReference property 
   * <P>
   * @return transactionReference  -- The Transaction Reference property.
   */
   public String getTransactionReference() {return transactionReference;}  

   /**
   * This method returns the transactionStatusCode property as a String
   * <P>
   * @return transactionStatusCode  -- The Transaction Status Code property.
   */
   public String getTransactionStatusCode() {return (new Short(transactionStatusCode) ).toString();}   


  //public short getTransactionStatusCode() {return transactionStatusCode;}   

   /**
   * This method returns the transactionStatusCode_text property. This
   * a text translation of the statusCode property.
   * <P>
   * @return transactionStatusCode_text  -- The text translation for
                                            Transaction Status Code property.
   */
   public String getTransactionStatusCode_text() {return transactionStatusCode_text;}   

   /**
   * This method returns the transactionValueDate property 
   * <P>
   * @return transactionValueDate  -- The Transaction Value Date property.
   */
   public String getTransactionValueDate() {return transactionValueDate;}   

   /**
   * This method sets the financialInstitutionIdentifier property.
   * <P>
   * @param o -- The Financial Institution Identifier property.
   */
   public void setFinancialInstitutionIdentifier(String o) {financialInstitutionIdentifier = o;}
  
   /**
   * This method sets the primaryConsumerID property.
   * <P>
   * @param o -- The Primary Consumer ID property.
   */
   public void setPrimaryConsumerID(String o) {primaryConsumerID = o;}

   /**
   * This method sets the ServiceProviderName property 
   * <P>
   * @param o  -- The Service Provider Name property.
   */
   public void setServiceProviderName(String o) {serviceProviderName = o;}   

   /**
   * This method sets the ServiceRequestName property 
   * <P>
   * @param o  -- The Service Request Name property.
   */
   public void setServiceRequestName   (String o) {serviceRequestName = o;}   


   /**
   *  This method is used to set the goldTransactionAmount property.
   *  The input to this method is a GsCurrency object.
   *  When passed in, this object is used to obtain the
   *  transaction amount, precision, and currency code.
   */
   public void setGoldTransactionAmount(GsCurrency o) {

	  String debitCredit = "";

	  try {
	if (o != null) {
       
		  transactionCurrencyType = o.getIsoCode();
		  transactionAmountPrecision = ( new Integer(o.getPrecision()) ).shortValue();
		  goldTransactionAmount = o;
        if (o.getDebitCredit() == 0 ) {
            debitCredit = "-";
        }
        transactionAmount = debitCredit + goldTransactionAmount.getDigits();
       }
	  }
	  catch (Throwable e)  {

		 debug("IfsPendingTransactionInstanceDetail.setTransactionAmount() - A throwable error has occurred " + e.toString());

	  }

   }  

   /**
   * This method sets the transactionAuthorizationTimestamp property 
   * <P>
   * @param o  -- Used to set the Transaction Authorization Timestamp property.
   */
   public void setTransactionAuthorizationTimestamp(String o) {
      if (o != null) {
      
        transactionAuthorizationTimestamp = o;
        
      }
   }

   /**
   * This method sets the transactionAuthorizerConsumerID property 
   * <P>
   * @param o  -- Used to set the Transaction Authorizer Consumer ID property.
   */
   public void setTransactionAuthorizerConsumerID(String o) {

      if (o != null) {
        transactionAuthorizerConsumerID = o;

      }
   }   

   /**
   * This method sets the transactionBeneficiary property 
   * <P>
   * @param o  -- Used to set the Transaction Beneficiary property.
   */
   public void setTransactionBeneficiary(String o) {transactionBeneficiary = o;}   

   /**
   * This method sets the transactionData property 
   * <P>
   * @param o  -- Used to set the Transaction Data property.
   */
   public void setTransactionData(String o){
      transactionData = o;

   }   

   /**
   * This method sets the transactionDebtAccountClearingHouseID property 
   * <P>
   * @param o  -- Used to set the 
   *              Transaction Debt Account Clearing House ID property.
   */
   public void setTransactionDebtAccountClearingHouseID(String o) {
     transactionDebtAccountClearingHouseID = o;
   }   

   /**
   * This method sets the transactionInitiationTimestamp property 
   * <P>
   * @param o  -- Used to set the 
   *              Transaction Initiation Timestamp property.
   */
   public void setTransactionInitiationTimestamp(String o) {transactionInitiationTimestamp = o;}   

   /**
   * This method sets the transactionInitiatorConsumerID property 
   * <P>
   * @param o  -- Used to set the 
   *              Transaction Initiator Consumer ID property.
   */
   public void setTransactionInitiatorConsumerID(String o) {transactionInitiatorConsumerID = o;}   

   /**
   * This method sets the transactionReference property 
   * <P>
   * @param o  -- Used to set the 
   *              Transaction Reference property.
   */
   public void setTransactionReference(String o) {transactionReference = o;}   


   /**
   * This method sets the transactionStatusCode property as a short. 
   * <P>
   * @param o  -- Used to set the 
   *              Transaction Status Code property.
   */
   public void setTransactionStatusCode(short o) {transactionStatusCode = o;}   

   /**
   * This method sets the transactionStatusCode property as a String. 
   * <P>
   * @param transactionStatusCode -- The Transaction Status Code property.
   */
   public void setTransactionStatusCode(String o) {
      try {
      
        transactionStatusCode = (new Short(o) ).shortValue();
      }
      catch (Throwable e) {
          debug("setTransactionStatusCode(String o) has thrown an exception: " + e.toString() );
      }
   
   }

   /**
   * This method sets the transactionStatusCode_text property.
   * This property is a text translation of the Status Code. 
   * <P>
   * @param o  -- Used to set the 
   *              Transaction Status Code text property.
   */

   public void setTransactionStatusCode_text( String o) {transactionStatusCode_text = o;}   

   /**
   * This method sets the transactionValueDate property 
   * <P>
   * @param o  -- Used to set the 
   *              Transaction Value Date property.
   */
   public void setTransactionValueDate(String o) {transactionValueDate = o;}   




   /**
   * Converts the IfsPendingTransactionInstanceDetail object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      if (hasRegisteredMethod("toString"))
         return super.toString();
      else
         return toString("\n");  // Return string from toString with
                                 // carriage return, line feed prefix.
   }

   /**
   * Converts the IfsPendingTransactionInstanceDetail object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix) {
      if (hasRegisteredMethod("toStringWithString"))
         return super.toString(prefix);

      StringBuffer buff = new StringBuffer(250);
      buff.append(super.toString(prefix));
      debug("In IfsPendingTransactionInstanceDetail.toString(String prefix): getting request properties");

      buff.append(prefix + "| transactionInitiatorConsumerID = " + transactionInitiatorConsumerID );
      buff.append(prefix + "| serviceRequestName = " + serviceRequestName );
      buff.append(prefix + "| transactionInitiationTimestamp  = " + transactionInitiationTimestamp  );

      buff.append(prefix + "| serviceProviderName = " + serviceProviderName );

      debug("In IfsPendingTransactionInstanceDetail.toString(String prefix): getting GsAccount properties via toString");
      try {
      
          if (transactionDebtAccount != null) {
    
              buff.append(prefix + "| transactionDebtAccount  = " + transactionDebtAccount.toString()  );
          }
      }
      catch (Throwable e) {
          debug("In IfsPendingTransactionInstanceDetail.toString(String prefix): GsAccount.toString has caused an exception" + e.toString() );
      }
      debug("In IfsPendingTransactionInstance.toString(String prefix): getting individual account properties");
      buff.append(prefix + "| transactionDebtAccountClearingHouseID  = " + transactionDebtAccountClearingHouseID  );
      buff.append(prefix + "| transactionDebtAccountType  = " + transactionDebtAccountType  );

      buff.append(prefix + "| transactionDebtAccountSubType = " + Short.toString(transactionDebtAccountSubType ) );
      buff.append(prefix + "| transactionDebtAccountNumber  = " + transactionDebtAccountNumber  );
      buff.append(prefix + "| transactionDebtAccountNickname     = " + transactionDebtAccountNickname);
      buff.append(prefix + "| transactionAmountPrecision; = " + Short.toString(transactionAmountPrecision) );
      buff.append(prefix + "| transactionCurrencyType  = " + transactionCurrencyType  );
      buff.append(prefix + "| transactionAmount          = " + transactionAmount );
      try {
      
          if (goldTransactionAmount != null) {
    
              buff.append(prefix + "| goldTransactionAmount  = " + goldTransactionAmount.toString() );
          }
      }
      catch (Throwable e) {
          debug("In IfsPendingTransactionInstanceDetail.toString(String prefix): GsCurrency.toString has caused an exception" + e.toString() );
      }      
      
      buff.append(prefix + "| transactionBeneficiary  = " + transactionBeneficiary  );
      buff.append(prefix + "| transactionReference   = " + transactionReference   );
      buff.append(prefix + "| transactionAuthorizerConsumerID  = " + transactionAuthorizerConsumerID  );
      buff.append(prefix + "| transactionAuthorizationTimestamp   = " + transactionAuthorizationTimestamp   );
      buff.append(prefix + "| transactionValueDate   = " + transactionValueDate   );
      buff.append(prefix + "| transactionStatusCode  = " + Short.toString(transactionStatusCode) );

      buff.append(prefix + "| transactionStatusCode_text  = " + transactionStatusCode_text );
      return buff.toString();

   }


}

