package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsSecurity.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    03/06/2000  rhs   d6423   Made JavaDoc additions.
//    03/17/2000  rhs   d6545   Added NLS support
//
// ***************************************************************************

import java.io.UnsupportedEncodingException;

/**
* IfsSecurity class provides RSA encryption functions for the page servlet developer.
*/
public class IfsSecurity
{

   private  String returnCode = "0";
   private  byte[] encodedValue = null;

   /**
   * IfsSecurity default constructor.
   */
   public IfsSecurity() {
      super();
   }

   /**
   * Encrypts a byte array using the RSA algorithm.
   * The RSA public key contained in the file specified by the "wsp_encrypt_file"
   * parameter in the config file is used to perform the encryption.
   *
   * @param byteArray -- The byte array data which will be RSA encrypted.
   * @param encodeFlag -- Flag which controls the RSA encryption.
   *    This flag's value can be either IfsSystem.RSA_ENCRYPTION or
   *    IfsSystem.NO_ENCRYPTION
   * @return
   *    The encrypted data and return code is returned in an IfsSecurity object.
   */
   public static IfsSecurity encryptData(byte[] byteArray, String encodeFlag) //the value of the encodeFlag
   {                                                                    //should be set from constants in IfsSystem

      IfsSecurity obj = new IfsSecurity();
      obj.encodedValue = (byte[])encryptRSAJ(byteArray , byteArray.length, encodeFlag.charAt(0));

      //String returnCode = null;
      if (obj.encodedValue.length==4)
      { //4 is size of error byte array.
         try
         {
            obj.returnCode = new String(obj.encodedValue);
         }
         catch (Exception e)
         {
            obj.returnCode = "Exception occurred in IfsSecurity.encryptData(), value is " + e.toString();
         }
      }
      return obj;
   }

   /**
   * Encrypts a string of information using the RSA algorithm.
   * The RSA public key contained in the file specified by the "wsp_encrypt_file"
   * parameter in the config file is used to perform the encryption.
   *
   * @param str -- A String object which is converted to a byte array and then
   *    RSA encrypted.
   * @param encodeFlag -- Flag which controls the RSA encryption.
   *    This flag's value can be either IfsSystem.RSA_ENCRYPTION or
   *    IfsSystem.NO_ENCRYPTION
   * @return
   *    The encrypted data and return code is returned in an IfsSecurity object.
   */
   public static IfsSecurity encryptData(String str, String encodeFlag)
   {
      IfsSecurity obj = null;
      try
      {
         obj = encryptData(str.getBytes(IfsSystem.cwsGMJCharSet), encodeFlag);
      }
      catch (UnsupportedEncodingException e)
      {
         String exceptMsg = new String("Exception occurred in IfsSecurity.encryptData(), value is -- " + e.toString() );
         IfsSystem.debug(exceptMsg);

         obj = new IfsSecurity();
         obj.returnCode = exceptMsg;
      }
      return(obj);
   }

   /**
   * Returns in a byte array the encrypted data from the RSA encryption operation.
   */
   public byte[] getEncodedValue() {
      return encodedValue;
   }

   /**
   * Returns the return code from the RSA encryption operation.
   */
   public String getReturnCode() {
      return returnCode;
   }

   /**
   * Returns true if the encrypted data from the RSA encryption operation
   * is valid.  Returns false when the encryption data is invalid.
   */
   boolean isValid() {
      if (returnCode.equals("0"))
         return true;

      return false;
   }

   /**
   * A native interface function which performs the RSA encryption function.
   * The RSA public key contained in the file specified by the "wsp_encrypt_file"
   * parameter in the config file is used to perform the encryption.
   *
   * @param in -- The byte array data which will be RSA encrypted.
   * @param length -- The number of bytes to encrypt.
   * @param encodeFlag -- Flag which controls the RSA encryption.
   *    This flag's value can be either a '1' for RSA_ENCRYPTION or
   *    a '0' for NO_ENCRYPTION.
   * @return
   *    The encrypted data is returned in a byte array.
   */
   public static native byte[] encryptRSAJ(byte[]  in,int length, char encodeFlag); //native C funtion requires


}

