package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsWebTransactionActivity.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
// ***************************************************************************
// Change History
//
//  Date                         Name                Reason For Change
// ------                       ------              -------------------
//
//   --                          --                 Initial Creation
//
// 01/19/2001                  A.Asokan             Fixed bugs and validation
//                                                  so that exceptions are thrown for error messages.
//
// 04/19/2001                  A.Asokan             T0984067
// 05/02/2001                  A.Asokan             T0984067R
// ****************************************************************************


import java.sql.*;
import java.math.*;
import java.io.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Enumeration;
import java.util.StringTokenizer;

/**
* The IfsWebTransactionActivity class is responsible for manipulation of data
* contained in a database which is normally maintained on a mainframe. This
* class contains a set of setters and getters for manipulation of data, as well
* as methods for creating a new row in the database, reading one row from the
* database, listing a set of rows from the database, and deleting a row from 
* the database.<BR><BR>This class inherits from the IfsJDBC class which contains
* methods used by this class, most importantly an "execute" method which does
* the majority of the database connectivity maintenance and communication.
* <BR><BR>The servlet writer will normally create an instance of the 
* IfsWebTransactionActivity object, set some required fields, and then invoke
* one of the database row create, read, list, or delete methods.
*
*/

public class IfsWebTransactionActivity extends IfsJDBC {

   /**
   * debug flag
   */
 public static boolean  debugOn = false;

   private IfsWebTransactionActivityDetail[] transactionQueryData = null;

   /**
   * This method gets the IfsWebTransactionActivityDetail 
   * repeating structure which is populated by the read and list method. For the
   * read method there will only be (at most) one row element in the array.  For
   * the list method there could be 0 or several, depending on the results of the
   * SQL query.
   */
   public IfsWebTransactionActivityDetail[] getTransactionQueryData() { return transactionQueryData; }

   // The following are the column names for the DB2 database:
   String col_Fi_Id = "FI_ID";
   String col_Prim_Cnsmr_Id = "PRIM_CNSMR_ID";
   String col_Txn_Init_Cnsmr_Id = "TXN_INIT_CNSMR_ID";
   String col_Srvc_Rqst_Nm = "SRVC_RQST_NM";
   String col_Txn_Logged_Tmstp = "TXN_LOGGED_TMSTP";
   String col_Txn_Amt = "TXN_AMT";
   String col_Txn_Amt_Prc = "TXN_AMT_PRC";
   String col_Txn_Curr_Tp = "TXN_CURR_TP";
   String col_Txn_Benf = "TXN_BENF";
   String col_Txn_Ref = "TXN_REF";
   String col_Txn_Val_Dt = "TXN_VAL_DT";
   String col_Txn_Log_Reason_Cd = "TXN_LOG_REASON_CD";
   String col_Txn_Auth_Cnsmr_Id = "TXN_AUTH_CNSMR_ID";
   String col_Txn_Auth_Tmstp = "TXN_AUTH_TMSTP";
   String col_Txn_Data = "TXN_DATA";

   private String errorCode = null;


   private String financialInstitutionIdentifier      = null;
   private String primaryConsumerID                   = null;
   private String transactionInitiatorConsumerID      = null;
   private String serviceRequestName                  = null;
   private String transactionLoggedTimestamp          = null;
   private String transactionAmount                   = null;
   private short transactionAmountPrecision           =    0;
   private String transactionCurrencyType             = null;
   private GsCurrency goldTransactionAmount           = null; 
   private String transactionBeneficiary              = null;
   private String transactionReference                = null;
   private String transactionValueDate                = null;
   private String transactionLoggingReasonCode        = null;
   private Timestamp  logTimestamp;
   private String transactionAuthorizerConsumerID     = null;
   private String transactionAuthorizationTimestamp   = null;
   private String transactionData                     = null;

   /**
   * This method gets the financialInstitutionIdentifier field populated by the servlet.
   */
   public String getFinancialInstitutionIdentifier() {return financialInstitutionIdentifier;}

   /**
   * This method gets the getPrimaryConsumerID field populated by the servlet.
   */
   public String getPrimaryConsumerID() {return primaryConsumerID;}

   /**
   * This method gets the transactionInitiatorConsumerID field populated by the servlet.
   */
   public String getTransactionInitiatorConsumerID() {return transactionInitiatorConsumerID;}

   /**
   * This method gets the serviceRequestName field populated by the servlet.
   */
   public String getServiceRequestName() {return serviceRequestName;}

   /**
   * This method gets the transactionLoggedTimestamp field populated by the servlet.
   */
   public String getTransactionLoggedTimestamp() {return transactionLoggedTimestamp;}


   /**
   * This method gets the transactionAmount field populated by the servlet.
   */
   public String getTransactionAmount() {return transactionAmount;}

   /**
   * This method gets the goldTransactionAmount object which was populated by 
   * the servlet. Values from this object will be used to populate the
   * "transactionAmount", "transactionAmountPrecision", and "transactionCurrencyType"
   * for the SQL query.  It is the servlet writer's responsibility to populate these
   * values before calling the "create" method.
   */
   public GsCurrency getGoldTransactionAmount() {return goldTransactionAmount;}

   /**
   * This method gets the transactionAmountPrecision field populated by the servlet.
   */
   public short getTransactionAmountPrecision() {return transactionAmountPrecision;}

   /**
   * This method gets the transactionCurrencyType field populated by the servlet.
   */
   public String getTransactionCurrencyType() {return transactionCurrencyType;}

   /**
   * This method gets the transactionBeneficiary field populated by the servlet.
   */
   public String getTransactionBeneficiary() {return transactionBeneficiary;}

   /**
   * This method gets the transactionReference field populated by the servlet.
   */
   public String getTransactionReference() {return transactionReference;}

   /**
   * This method gets the transactionValueDate field populated by the servlet.
   */
   public String getTransactionValueDate() {return transactionValueDate;}

   /**
   * This method gets the transactionLoggingReasonCode field populated by the servlet.
   */
   public String getTransactionLoggingReasonCode() {return transactionLoggingReasonCode;}


   /**
   * This method gets the transactionAuthorizerConsumerID field populated by the servlet.
   */
   public String getTransactionAuthorizerConsumerID() {return transactionAuthorizerConsumerID;}

   /**
   * This method gets the transactionAuthorizationTimestamp field populated by the servlet.
   */
   public String getTransactionAuthorizationTimestamp() {return transactionAuthorizationTimestamp;}

   /**
   * This method gets the transactionData field populated by the servlet.
   */
   public String getTransactionData() {return transactionData;}


   /**
   * This method sets the financialInstitutionIdentifier field during the population of 
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setFinancialInstitutionIdentifier(String o) {financialInstitutionIdentifier = o;}

   /**
   * This method sets the primaryConsumerID field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setPrimaryConsumerID(String o) {primaryConsumerID = o;}

   /**
   * This method sets the transactionInitiatorConsumerID field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionInitiatorConsumerID(String o) {transactionInitiatorConsumerID = o;}

   /**
   * This method sets the serviceRequestName field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setServiceRequestName(String o) {serviceRequestName = o;}

   /**
   * This method sets the transactionLoggedTimestamp field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionLoggedTimestamp(String o) {transactionLoggedTimestamp = o;}

   /**
   * This method sets the goldTransactionAmount field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setGoldTransactionAmount(GsCurrency o) {
      
	  try {

		 transactionCurrencyType = o.getIsoCode();
		 transactionAmountPrecision = ( new Integer(o.getPrecision()) ).shortValue() ;
                 transactionAmount = o.getDigits();

                 if (o.getDebitCredit() == 0) 
                     transactionAmount = "-" + transactionAmount;
       
		 goldTransactionAmount = o;
                 //debug("goldTransactionAmount is " + goldTransactionAmount.toString());
                 //debug("transactionAmount is " + transactionAmount);
              }
          catch (Throwable e)  {

	         //debug("IfsWebTransactionActivity.setTransactionAmount() A throwable error has occurred " + e.toString());
                 errorCode = "Error in the format of Gold Transaction amount.";
                 goldTransactionAmount = null;

	      }   
   }

   /**
   * This method sets the transactionBeneficiary field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionBeneficiary(String o) {transactionBeneficiary = o;}

   /**
   * This method sets the transactionReference field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionReference(String o) {transactionReference = o;}

   /**
   * This method sets the transactionValueDate field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionValueDate(String o) {transactionValueDate = o;}

   /**
   * This method sets the transactionLoggingReasonCode field as a String during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionLoggingReasonCode(String o) {transactionLoggingReasonCode = o;}

   /**
   * This method sets the transactionLoggingReasonCode field as an integer during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionLoggingReasonCode(int o) {transactionLoggingReasonCode = (new Integer(o)).toString();}

   /**
   * This method sets the transactionAuthorizerConsumerID field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionAuthorizerConsumerID(String o) {transactionAuthorizerConsumerID = o;}

   /**
   * This method sets the transactionAuthorizationTimestamp field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionAuthorizationTimestamp(String o) {transactionAuthorizationTimestamp = o;}

   /**
   * This method sets the transactionData field during the population of
   * the IfsWebTransactionActivityDetail repeat structure.
   */
   public void setTransactionData(String o) {transactionData = o;}

   private static Hashtable registry = new Hashtable();

   /**
   * This method gets the registry defined for this class to register customized
   * methods applicable to all IfsObjects.
   */
   public static Hashtable getRegistry() {
          return registry;
   }

   /**
   * Generic constructor that takes no IfsSession object
   */
   public IfsWebTransactionActivity() {
   }

   /**
   * Constructor that takes an IfsSession object
   */
   public IfsWebTransactionActivity(IfsSession ifsSession) {
          setIfsSession(ifsSession);
   }

   /**
   * Name of IfsWebTransactionActivity DB2 table
   */

   public static String Qualifier = IfsSystem.IFS_WDBC_SCHEMA;

   public static final String TABLE_NAME = (Qualifier != null && Qualifier.equals("") == false) ? (Qualifier + ".REL_TXN_ACTY") : (Qualifier + "REL_TXN_ACTY");



   /**
   * This method creates a row in the database.  To use 
   * this method you must first create an instance of the class and set
   * the following fields with the appropriate setters:<BR><BR>
   *
   *    IfsWebTransactionActivity transAct = new IfsWebTransactionActivity();<BR>
   *    transAct.setUrl("jdbc:db2:DBTIFAV");<BR>
   *    transAct.setDriverName("COM.ibm.db2.jdbc.app.DB2Driver");<BR><BR>
   *    
   *    <UL>(An additional set of required data fields must be set for this transaction
   *    using the appropriate setters. The "validation" method will be called upon
   *    execution of this method, and either the registered validation method or the
   *    default validation method will be invoked to assure these required fields are 
   *    set. Those fields are:
   *
   *    <BR><BR>
   *    
   *      financialInstitutionIdentifier<BR>
   *      primaryConsumerID<BR>
   *      transactionInitiatorConsumerID<BR>
   *      serviceRequestName<BR>
   *      transactionLoggingReasonCode<BR>
   *      transactionData<BR><BR>
   *    If these fields are not provided an error will occur.)<BR><BR></UL>
   *
   *    transAct.create();<BR>
   *    transAct.close();   // See the following Note!<BR><BR>
   *
   *    Note: The closing of the connection is optional at this point.  If the 
   *    servlet writer does not invoke the close method at this point the connection
   *    to the database will stay open until either the servlet writer closes the 
   *    connection in another servlet, the session times out, or the user logs off.
   *    Not closing the connection allows the sharing of this connection between
   *    servlet pages, thus reducing the number of required connections and providing
   *    better performance.
   *
   */


   public void create () throws IfsException {  

      String tempSqlStr = null;
      StringBuffer createColumns = new StringBuffer();
      StringBuffer createValues  = new StringBuffer();
      resetTxnTypes();
      createTxn = true;

      try {
          
          if (errorCode != null) {
              String msg = errorCode;
              errorCode = null;
              throw new IfsException("Amount Format Exception",msg);
          }

            //  Construct column and data strings for concatenated column and values for SQL statement
            createColumns.append("(");
            createValues.append("(");

            createColumns.append(col_Fi_Id + ", ");
            //createValues.append("'" + getFinancialInstitutionIdentifier() + "', "); 
            createValues.append("?, "); 

            createColumns.append(col_Prim_Cnsmr_Id + ", ");
            createValues.append("?, "); 
            //createValues.append("'" + getPrimaryConsumerID() + "', ");

            createColumns.append(col_Txn_Init_Cnsmr_Id + ", ");
            createValues.append("?, "); 
            //createValues.append("'" + getTransactionInitiatorConsumerID() + "', ");

            createColumns.append(col_Srvc_Rqst_Nm + ", ");
            createValues.append("?, "); 
            //createValues.append("'" + getServiceRequestName() + "', ");

         if ( getTransactionLoggedTimestamp().equals("") != true ) {
            createColumns.append(col_Txn_Logged_Tmstp + ", ");
            //createValues.append("'" + getTransactionLoggedTimestamp() + "', ");
            createValues.append("?, ");
         }
 


            // Get the transactionAmount, transactionAmountPrecision, & transactionCurrencyType for goldTransactionAmount

            String tempTransactionAmount = null;
            short tempTransactionAmountPrecision = 0;
            String tempTransactionCurrencyType    =  null;

            if (goldTransactionAmount != null) {
            
                tempTransactionAmount = goldTransactionAmount.getDigits(); //returned a signed value

                if (tempTransactionAmount.length() >15) {
                      goldTransactionAmount = null;
                      throw new IfsException("Validation Error","Transaction Amount Value is out of Range");
                }

                tempTransactionAmountPrecision = (new Integer(goldTransactionAmount.getPrecision()) ).shortValue();

                tempTransactionCurrencyType    = goldTransactionAmount.getIsoCode();
            }


            if (tempTransactionAmount != null) {            
                createColumns.append(col_Txn_Amt + ", ");
                //createValues.append(tempTransactionAmount + ", ");
                createValues.append("?, "); 
            }

            if (tempTransactionAmount != null) {
                createColumns.append(col_Txn_Amt_Prc + ", ");
                //createValues.append(tempTransactionAmountPrecision + ", ");
                createValues.append("?, "); 
            }

            if (tempTransactionCurrencyType != null) {            
                createColumns.append(col_Txn_Curr_Tp + ", ");
                //createValues.append("'" + tempTransactionCurrencyType + "', ");
                createValues.append("?, "); 
            }

            if ( getTransactionBeneficiary().equals("") != true ) {
               createColumns.append(col_Txn_Benf + ", ");
               //createValues.append("'" + getTransactionBeneficiary() + "', ");
               createValues.append("?, "); 
            }

            if ( getTransactionReference().equals("") != true ) {
               createColumns.append(col_Txn_Ref + ", ");
               //createValues.append("'" + getTransactionReference() + "', ");
               createValues.append("?, "); 
            }

            if (getTransactionValueDate().equals("") != true) {
               createColumns.append(col_Txn_Val_Dt + ", ");
               //createValues.append("'" + getTransactionValueDate() + "', ");
               createValues.append("?, "); 
            }

            if (getTransactionLoggingReasonCode() != null) {
                createColumns.append(col_Txn_Log_Reason_Cd + ", ");            
                //createValues.append(getTransactionLoggingReasonCode() + ", ");
                createValues.append("?, "); 
             }

            if ( getTransactionAuthorizerConsumerID().equals("") != true) {
               createColumns.append(col_Txn_Auth_Cnsmr_Id + ", ");
               //createValues.append("'" + getTransactionAuthorizerConsumerID() + "', ");
               createValues.append("?, "); 
            }

            if (getTransactionAuthorizationTimestamp().equals("") != true ) {
                createColumns.append(col_Txn_Auth_Tmstp + ", ");
                //createValues.append("'" + getTransactionAuthorizationTimestamp() + "', ");
                createValues.append("?, "); 
            }
            
            createColumns.append(col_Txn_Data);
            //createValues.append("'" + getTransactionData() + "'");
            createValues.append("? "); 

            createColumns.append(")");
            createValues.append(") "); 

            tempSqlStr = "INSERT INTO " + TABLE_NAME + " " + createColumns.toString() + " VALUES " +  createValues.toString();
            setSqlString(tempSqlStr);

            createTxn = true;
            establishConnection();

            //set values here
            int  i = 1;

            preparedStatement.setString(i, financialInstitutionIdentifier    );
            i++;

            preparedStatement.setString(i, primaryConsumerID);
            i++;

            preparedStatement.setString(i, transactionInitiatorConsumerID);
            i ++;

            preparedStatement.setString(i, serviceRequestName);                       
            i ++;


           if (getTransactionLoggedTimestamp().equals("") != true) {

              if (validTimestamp(transactionLoggedTimestamp))
                transactionLoggedTimestamp = changeTimestamp(transactionLoggedTimestamp);

            preparedStatement.setString(i, transactionLoggedTimestamp    );
            i++;
            }


            if (tempTransactionAmount != null) {            
              goldTransactionAmount = null;
              preparedStatement.setString(i, tempTransactionAmount    );
              i++;            
            }

            if (tempTransactionAmount != null) {
              preparedStatement.setShort(i, tempTransactionAmountPrecision    );
              i++;
            }

            if (tempTransactionCurrencyType != null) {            
              preparedStatement.setString(i, tempTransactionCurrencyType    );
              i++;            
            }

            if (transactionBeneficiary.equals("") != true) {

          // include for space-@ fix.
          // String pTxnBenf = padding(transactionBeneficiary, 40);
          // byte [] tempTxnBenf = pTxnBenf.getBytes();

              byte [] tempTxnBenf = transactionBeneficiary.getBytes();

              preparedStatement.setBytes(i, tempTxnBenf);
              i++;
            }

            if (transactionReference.equals("") != true) {
              preparedStatement.setString(i, transactionReference    );
              i++; 
            }

            if (getTransactionValueDate().equals("") != true) {
                preparedStatement.setString(i, transactionValueDate    );
                i++;
             }


            try {
                    if (getTransactionLoggingReasonCode() != null) {
                       int tempTransactionLoggingReasonCode;
                       tempTransactionLoggingReasonCode = (new Integer(transactionLoggingReasonCode)).intValue();
                       preparedStatement.setInt(i, tempTransactionLoggingReasonCode    );
                       i++;
                     }
                 }
            catch (Exception e) {
                 //debug("<BR>TransactionLoggingReasonCode has thrown an exception.<BR>" + e.toString());
                 String msg = "Data Error : Logging Reason Code expects integer value.";
                 throw new IfsException(e,  msg);
                 }  

            if (transactionAuthorizerConsumerID.equals("") != true) {
              preparedStatement.setString(i, transactionAuthorizerConsumerID    );
              i++;
            }

            if (getTransactionAuthorizationTimestamp().equals("") != true) {

               if (validTimestamp(transactionAuthorizationTimestamp)) 
                   transactionAuthorizationTimestamp = changeTimestamp(transactionAuthorizationTimestamp);

               preparedStatement.setString(i, transactionAuthorizationTimestamp    );
              i++;
            }

            preparedStatement.setString(i, transactionData    );


// calling the execute() method for create().

            execute();


      }
      catch (IfsException e) { 
         String msg = e.toString();
         throw new IfsException(e,msg);
      }
      catch (Exception e) {
         //debug("<BR>IfsWebTransactionActivity create exception.<BR>" + e.toString());
         String msg = "IfsWebTransactionActivity.create() has thrown an exception. " + e.toString();
         if (message == null) 
             message = msg;
         throw new IfsException(e,  msg);
      }

   }// end of create().




   /**
   * This method reads one row from the database.  To use
   * this method you must first create an instance of the class and set
   * the following fields with the appropriate setters:<BR><BR>
   *
   *    IfsWebTransactionActivity transAct = new IfsWebTransactionActivity();<BR>
   *    transAct.setUrl("jdbc:db2:DBTIFAV");<BR>
   *    transAct.setDriverName("COM.ibm.db2.jdbc.app.DB2Driver");<BR><BR>
   *
   *    <UL>(An additional set of required data fields must be set for this transaction
   *    using the appropriate setters. A "validation" method will be called upon
   *    execution of this method, and either the registered validation method or the
   *    default validation method will be invoked to assure these required fields are
   *    set. Those fields are:
   *
   *    <BR><BR>
   *    
   *      financialInstitutionIdentifier<BR>
   *      primaryConsumerID<BR>
   *      transactionInitiatorConsumerID<BR>
   *      serviceRequestName<BR>
   *      transactionLoggedTimestamp<BR><BR>
   *    If these fields are not provided an error will occur.)<BR></UL>
   *
   *    transAct.read();<BR>
   *    transAct.close();   // See the following Note! <BR><BR>
   *
   *    Note: The closing of the connection is optional at this point.  If the 
   *    servlet writer does not invoke the close method at this point the connection
   *    to the database will stay open until either the servlet writer closes the 
   *    connection in another servlet, the session times out, or the user logs off.
   *    Not closing the connection allows the sharing of this connection between
   *    servlet pages, thus reducing the number of required connections and providing
   *    better performance.
   *
   */
   public void read () throws IfsException {
 
      StringBuffer readQuery = new StringBuffer();
      resetTxnTypes();
      readTxn = true;

      try {

            // Construct column and data strings for concatenated column and values for SQL statement
            readQuery.append(col_Fi_Id + "='" + getFinancialInstitutionIdentifier() + "' and ");
            readQuery.append(col_Prim_Cnsmr_Id + "='" + getPrimaryConsumerID() + "' and ");
            readQuery.append(col_Txn_Init_Cnsmr_Id + "='" + getTransactionInitiatorConsumerID() + "' and ");
            readQuery.append(col_Srvc_Rqst_Nm + "='" + getServiceRequestName() + "' and ");


            if (validTimestamp(transactionLoggedTimestamp)) 
                transactionLoggedTimestamp = changeTimestamp(transactionLoggedTimestamp);

            readQuery.append(col_Txn_Logged_Tmstp + "='" + getTransactionLoggedTimestamp() + "' ");

            String sqlString = "SELECT FI_ID,PRIM_CNSMR_ID,TXN_INIT_CNSMR_ID,SRVC_RQST_NM,TXN_LOGGED_TMSTP,TXN_AMT,TXN_AMT_PRC,TXN_CURR_TP,TXN_BENF,TXN_REF,TXN_VAL_DT,TXN_LOG_REASON_CD,TXN_AUTH_CNSMR_ID,TXN_AUTH_TMSTP,TXN_DATA FROM " + TABLE_NAME + " WHERE " + readQuery.toString();

            setSqlString(sqlString);

            execute();

            ResultSet queryResults = getResultSet();

            Vector results = new Vector();
            IfsWebTransactionActivityDetail element = null;

            GsCurrency goldCurrencyAmount = null;
                       goldCurrencyAmount = new GsCurrency();
            BigDecimal tempAmount = null;


              if (queryResults != null) {
            while (queryResults.next()) {
               element = new IfsWebTransactionActivityDetail();
               element.setFinancialInstitutionIdentifier(queryResults.getString("FI_ID"));
               element.setPrimaryConsumerID(queryResults.getString("PRIM_CNSMR_ID"));
               element.setTransactionInitiatorConsumerID(queryResults.getString("TXN_INIT_CNSMR_ID"));
               element.setServiceRequestName(queryResults.getString("SRVC_RQST_NM"));

                 String tempTmstp = queryResults.getString("TXN_LOGGED_TMSTP").replace(' ','-');
                 tempTmstp = tempTmstp.replace(':','.');
                 element.setTransactionLoggedTimestamp(tempTmstp);

               transactionAmountPrecision = queryResults.getShort("TXN_AMT_PRC");
               tempAmount = queryResults.getBigDecimal("TXN_AMT",0);
               element.setTransactionAmountPrecision(queryResults.getShort("TXN_AMT_PRC"));
               element.setTransactionCurrencyType(queryResults.getString("TXN_CURR_TP"));

               if (tempAmount != null) {
               
                   goldCurrencyAmount = GsCurrency.fromString(tempAmount.toString());
                   goldCurrencyAmount.setDigits(tempAmount.toString());
                   goldCurrencyAmount.setPrecision(transactionAmountPrecision);
                   goldCurrencyAmount.setIsoCode(queryResults.getString("TXN_CURR_TP"));
                   element.setTransactionAmount(tempAmount.toString());
                   element.setGoldTransactionAmount(goldCurrencyAmount);
               }
               else
                  debug("In IfsWebTransactionActivity.read,  tempAmount == null.");
               debug("IfsWebTransactionActivity Ending parse currency to gold.");

               //String contains '@' which represent blanks in database. Strip them out from string
    
               String tempTxnBenf = null;

               if ( queryResults.getBytes("TXN_BENF") != null) {
                   tempTxnBenf = new String(queryResults.getBytes("TXN_BENF"));

                 if (tempTxnBenf != null)   
                     tempTxnBenf = tempTxnBenf.replace('@',' ');
               }

    
               element.setTransactionBeneficiary(tempTxnBenf);
               element.setTransactionReference(queryResults.getString("TXN_REF"));
               element.setTransactionValueDate(queryResults.getString("TXN_VAL_DT"));
               element.setTransactionLoggingReasonCode(queryResults.getString("TXN_LOG_REASON_CD"));
               element.setTransactionAuthorizerConsumerID(queryResults.getString("TXN_AUTH_CNSMR_ID"));

                 tempTmstp = queryResults.getString("TXN_AUTH_TMSTP");

               if (tempTmstp != null) {
                 tempTmstp = tempTmstp.replace(' ','-');
                 tempTmstp = tempTmstp.replace(':','.');
               }
 
                 element.setTransactionAuthorizationTimestamp(tempTmstp);


               element.setTransactionData((new String (queryResults.getBytes("TXN_DATA") ) ));
               results.addElement(element);
            }

            debug("<BR><B>Size of ResultSet is " + results.size() + "</B>");
            /*if (results.size() == 0) {
                String msg = "Given criteria did not match any record.";
                throw new IfsException("No Records Returned",msg);
            }*/

            transactionQueryData = new IfsWebTransactionActivityDetail[results.size()];
            for(int i=0; i<results.size(); i++) {
               transactionQueryData[i] = (IfsWebTransactionActivityDetail) results.elementAt(i);
               debug("IfsWebTransactionActivityDetail.list() - transactionQueryData[" + i + "].getFinancialInstitutionIdentifier() = " + transactionQueryData[i].getFinancialInstitutionIdentifier());
            }
            queryResults.close();
            debug("IfsWebTransactionActivity read parsing done.");
              }
      }
      catch (IfsException e) {
           String msg = e.toString();
           throw new IfsException(e,msg);
      }

      catch (Exception e) {
         debug("<BR>IfsWebTransactionActivity read exception.<BR>" + e.toString());
         String msg = "IfsWebTransactionActivity.read() has thrown an exception." + e.toString();
         throw new IfsException(e,  msg);
      }

   } // end of read().


   /**
   * This method reads one or more rows from the database.  To use
   * this method you must first create an instance of the class and set
   * the following fields with the appropriate setters:<BR><BR>
   *
   *    IfsWebTransactionActivity transAct = new IfsWebTransactionActivity();<BR>
   *    transAct.setUrl("jdbc:db2:DBTIFAV");<BR>
   *    transAct.setDriverName("COM.ibm.db2.jdbc.app.DB2Driver");<BR><BR>
   *
   *    <UL>(An additional set of required data fields must be set for this transaction
   *    using the appropriate setters. A "validation" method will be called upon
   *    execution of this method, and either the registered validation method or the
   *    default validation method will be invoked to assure these required fields are
   *    set. Those fields are:
   *
   *    <BR><BR>
   *    
   *      financialInstitutionIdentifier<BR>
   *      primaryConsumerID<BR><BR>
   *    If these fields are not provided an error will occur.)<BR><BR></UL>
   *
   *    transAct.list();<BR>
   *    transAct.close();   // See the following Note! <BR><BR>
   *
   *    Note: The closing of the connection is optional at this point.  If the 
   *    servlet writer does not invoke the close method at this point the connection
   *    to the database will stay open until either the servlet writer closes the 
   *    connection in another servlet, the session times out, or the user logs off.
   *    Not closing the connection allows the sharing of this connection between
   *    servlet pages, thus reducing the number of required connections and providing
   *    better performance.
   *
   */
   public void list () throws IfsException {

      StringBuffer listQuery = new StringBuffer();

      resetTxnTypes();
      listTxn = true;

      try {

            // Construct column and data strings for concatenated column and values for SQL statement
            listQuery.append(col_Fi_Id + "='" + getFinancialInstitutionIdentifier() + "' and ");
            listQuery.append(col_Prim_Cnsmr_Id + "='" + primaryConsumerID + "'");

            String sqlString = "SELECT FI_ID,PRIM_CNSMR_ID,TXN_INIT_CNSMR_ID,SRVC_RQST_NM,TXN_LOGGED_TMSTP,TXN_AMT,TXN_AMT_PRC,TXN_CURR_TP,TXN_BENF,TXN_REF,TXN_VAL_DT,TXN_LOG_REASON_CD,TXN_AUTH_CNSMR_ID,TXN_AUTH_TMSTP,TXN_DATA FROM " + TABLE_NAME + " WHERE " + listQuery.toString();


            setSqlString(sqlString);

            execute();

            ResultSet queryResults = getResultSet();

            Vector results = new Vector();
            IfsWebTransactionActivityDetail element = null;

            GsCurrency goldCurrencyAmount = null;
            BigDecimal tempAmount = null;


            if (queryResults != null) {
            while (queryResults.next()) {
	       element = new IfsWebTransactionActivityDetail();	
               element.setFinancialInstitutionIdentifier(queryResults.getString("FI_ID"));
               element.setPrimaryConsumerID(queryResults.getString("PRIM_CNSMR_ID"));
               element.setTransactionInitiatorConsumerID(queryResults.getString("TXN_INIT_CNSMR_ID"));
               element.setServiceRequestName(queryResults.getString("SRVC_RQST_NM"));

               String tempTmstp = queryResults.getString("TXN_LOGGED_TMSTP").replace(' ','-');
               tempTmstp = tempTmstp.replace(':','.'); 
               element.setTransactionLoggedTimestamp(tempTmstp);

               transactionAmountPrecision = queryResults.getShort("TXN_AMT_PRC");
               debug("IfsWebTransactionActivity tempAmount equate.");
 
               tempAmount = queryResults.getBigDecimal("TXN_AMT",0);
               element.setTransactionAmountPrecision(queryResults.getShort("TXN_AMT_PRC"));
               element.setTransactionCurrencyType(queryResults.getString("TXN_CURR_TP"));

               if (tempAmount != null) {
               
                   goldCurrencyAmount = GsCurrency.fromString(tempAmount.toString());
                   goldCurrencyAmount.setDigits(tempAmount.toString());
                   goldCurrencyAmount.setPrecision(transactionAmountPrecision);
                   goldCurrencyAmount.setIsoCode(queryResults.getString("TXN_CURR_TP"));
                   element.setTransactionAmount(tempAmount.toString());
                   element.setGoldTransactionAmount(goldCurrencyAmount);
               }
               else                  
                  debug("IfsWebTransactionActivity.list, tempAmount == null.");
               debug("IfsWebTransactionActivity Endingparse currency to gold.");

               //String contains '@' which represent blanks in database.


               String tempTxnBenf = null;

               if ( queryResults.getBytes("TXN_BENF") != null) {
                   tempTxnBenf = new String(queryResults.getBytes("TXN_BENF"));

                 if (tempTxnBenf != null)  
                     tempTxnBenf = tempTxnBenf.replace('@',' ');
               }



               element.setTransactionBeneficiary(tempTxnBenf);
               element.setTransactionReference(queryResults.getString("TXN_REF"));
               element.setTransactionValueDate(queryResults.getString("TXN_VAL_DT"));
               element.setTransactionLoggingReasonCode(queryResults.getString("TXN_LOG_REASON_CD"));
               element.setTransactionAuthorizerConsumerID(queryResults.getString("TXN_AUTH_CNSMR_ID"));

                 tempTmstp = queryResults.getString("TXN_AUTH_TMSTP");

               if (tempTmstp != null) {
                 tempTmstp = tempTmstp.replace(' ','-');
                 tempTmstp = tempTmstp.replace(':','.');
               }
                 element.setTransactionAuthorizationTimestamp(tempTmstp);


               element.setTransactionData((new String (queryResults.getBytes("TXN_DATA") ) ));
	       results.addElement(element);
            }

            debug("<BR><B>Size of ResultSet is " + results.size() + "</B>");
            /*if (results.size() == 0) {
                String msg = "Given criteria did not match any record."; 
                throw new IfsException("No Records Returned",msg);
            }*/

            transactionQueryData = new IfsWebTransactionActivityDetail[results.size()];
            for(int i=0; i<results.size(); i++) {
	       transactionQueryData[i] = (IfsWebTransactionActivityDetail) results.elementAt(i); 
	       debug("IfsWebTransactionActivityDetail.list() - transactionQueryData[" + i + "].getFinancialInstitutionIdentifier() = " + transactionQueryData[i].getFinancialInstitutionIdentifier());
            }	
            queryResults.close();

            }
      }
      catch (IfsException e) {
           String msg = e.toString();
           throw new IfsException(e,msg);
      }

      catch (Exception e) {
         debug("<BR>IfsWebTransactionActivity list exception.<BR>" + e.toString());
         String msg = "IfsWebTransactionActivity.list() has thrown an exception." + e.toString();
         throw new IfsException(e,  msg);
      }
      
   } // end of list().


   /**
   * This method deletes a row from the database.  To use
   * this method you must first create an instance of the class and set
   * the following fields with the appropriate setters:<BR><BR>
   *
   *    IfsWebTransactionActivity transAct = new IfsWebTransactionActivity();<BR>
   *    transAct.setUrl("jdbc:db2:DBTIFAV");<BR>
   *    transAct.setDriverName("COM.ibm.db2.jdbc.app.DB2Driver");<BR><BR>
   *
   *    <UL>(An additional set of required data fields must be set for this transaction
   *    using the appropriate setters. A "validation" method will be called upon
   *    execution of this method, and either the registered validation method or the
   *    default validation method will be invoked to assure these required fields are
   *    set. Those fields are:
   *
   *    <BR><BR>
   *    
   *      financialInstitutionIdentifier<BR>
   *      primaryConsumerID<BR>
   *      transactionInitiatorConsumerID<BR>
   *      serviceRequestName<BR>
   *      transactionLoggedTimestamp<BR><BR>
   *    If these fields are not provided an error will occur.)<BR><BR></UL>
   *
   *    transAct.delete();<BR>
   *    transAct.close();   // See the following Note! <BR><BR>
   *
   *    Note: The closing of the connection is optional at this point.  If the 
   *    servlet writer does not invoke the close method at this point the connection
   *    to the database will stay open until either the servlet writer closes the 
   *    connection in another servlet, the session times out, or the user logs off.
   *    Not closing the connection allows the sharing of this connection between
   *    servlet pages, thus reducing the number of required connections and providing
   *    better performance.
   *
   */
   public void delete () throws IfsException {  

      StringBuffer deleteSqlString = new StringBuffer();
      resetTxnTypes();
      deleteTxn = true;

      try {

            // Construct column and data strings for concatenated column and values for SQL statement
            deleteSqlString.append(col_Fi_Id + "='" + getFinancialInstitutionIdentifier() + "' and ");
            deleteSqlString.append(col_Prim_Cnsmr_Id + "='" + getPrimaryConsumerID() + "' and ");
            deleteSqlString.append(col_Txn_Init_Cnsmr_Id + "='" + getTransactionInitiatorConsumerID() + "' and ");
            deleteSqlString.append(col_Srvc_Rqst_Nm + "='" + getServiceRequestName() + "' and ");

            if (validTimestamp(transactionLoggedTimestamp))
                  transactionLoggedTimestamp = changeTimestamp(transactionLoggedTimestamp);

            deleteSqlString.append(col_Txn_Logged_Tmstp + "='" + getTransactionLoggedTimestamp() + "'");

            String sqlString = "DELETE FROM " + TABLE_NAME + " WHERE " + deleteSqlString.toString();

            setSqlString(sqlString);
            execute();
      }
      catch (Exception e) {
         debug("<BR>IfsWebTransactionActivity delete exception.<BR>" + e.toString());
         String msg = "IfsWebTransactionActivity.delete() has thrown an exception." + e.toString();
         throw new IfsException(e,  msg);
      }
    
   } // end of delete().


   /**
   * Converts the IfsWebTransactionActivity object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      if (hasRegisteredMethod("toString"))
         return super.toString();
      else
         return toString("\n");  // Return string from toString with
                                 // carriage return, line feed prefix.
   }

   /**
   * Converts the IfsWebTransactionActivity object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix) {
      if (hasRegisteredMethod("toStringWithString"))
         return super.toString(prefix);

      //Cookie cookie = getIfsSession().getSessionKey();
      StringBuffer buff = new StringBuffer(250);
      try {
      
      buff.append(super.toString(prefix));
      debug("In IfsWebTransactionActivity.toString(String prefix): getting request properties");

      buff.append(prefix + "| financialInstitutionIdentifier = " + financialInstitutionIdentifier );
      buff.append(prefix + "| primaryConsumerID = " + primaryConsumerID );
      buff.append(prefix + "| transactionInitiatorConsumerID = " + transactionInitiatorConsumerID );
      buff.append(prefix + "| serviceRequestName = " + serviceRequestName );
      buff.append(prefix + "| transactionLoggedTimestamp  = " + transactionLoggedTimestamp  );

     // buff.append(prefix + "| serviceProviderName = " + serviceProviderName );

      debug("In IfsWebTransactionActivity.toString(String prefix): before getting currrency properties");
      buff.append(prefix + "| transactionAmountPrecision; = " + Short.toString(transactionAmountPrecision) );
      buff.append(prefix + "| transactionCurrencyType  = " + transactionCurrencyType  );
      buff.append(prefix + "| transactionAmount          = " + transactionAmount );

      debug("In IfsWebTransactionActivity.toString(String prefix): after getting currrency properties");
      try {
      
         if (goldTransactionAmount != null) {

            debug("In IfsWebTransactionActivity.toString(String prefix) try block: goldTransactionAmount ! null");
           buff.append(prefix + "| goldTransactionAmount  = " + goldTransactionAmount.toString() );
         }
      }
      catch(Throwable e) {

       String msg = "In IfsWebTransactionActivity.toString(String prefix):An exception was thrown by goldTransactionAmount " + e.toString() ;
       debug(msg);

      }

      try {
      
      buff.append(prefix + "| transactionBeneficiary  = " + transactionBeneficiary  );
      buff.append(prefix + "| transactionReference   = " + transactionReference   );
      buff.append(prefix + "| transactionAuthorizerConsumerID  = " + transactionAuthorizerConsumerID  );

      buff.append(prefix + "| transactionAuthorizationTimestamp   = " + transactionAuthorizationTimestamp   );
      buff.append(prefix + "| transactionValueDate   = " + transactionValueDate   );

      buff.append(prefix + "| transactionLoggingReasonCode  = " + transactionLoggingReasonCode );
      }

      catch(Throwable e) {

       String msg = "In IfsWebTransactionActivity.toString(String prefix):An exception was thrown by remaining props " + e.toString();
       debug(msg);
      }

      if (transactionQueryData != null) {
               
         debug("In IfsWebTransactionActivity.toString(String prefix): transactionQueryData.length = " + Integer.toString(transactionQueryData.length) );
         for (int i =0; i < transactionQueryData.length; i ++) {
         debug("In IfsWebTransactionActivity.toString(String prefix): In for loop " );
         buff.append(prefix + "| transactionQueryData[" + Integer.toString(i) + "]   = " + transactionQueryData[i].toString(prefix)   );
   
   
         }
      }
      if (registry.isEmpty())
      {
         buff.append(prefix + "| Registry is empty");
      }
      else
      {
         Enumeration keys = registry.keys();
         while (keys.hasMoreElements())
         {
            String key = (String) keys.nextElement();
            buff.append(prefix + "| Registry contains: ");
            buff.append(prefix + "| " + key + " = ");
            buff.append(registry.get(key).toString());
         }
      }

      }//end try
      catch(Throwable e) {

        String msg = "In IfsWebTransactionActivity.toString(String prefix) outer try/catch:An exception was thrown by remaining props " + e.toString();
        debug(msg);

      }

      finally {

      return buff.toString();

              }

   }

   /**
  *Funtion to pad spaces to the right of a string for the given length.
  */
  public String padding(String inputString, int width) {

     String outputString;

     if (inputString == null)        //allow null input string
         outputString = "";
     else
         outputString = inputString;

     while (outputString.length() < width)
     {
        outputString = outputString + " ";
     }

  return (outputString); 

  }//end of padding(string, int)



   /**
  *Accepts a Date in String format (year,month,day) as input parameter.
  *validates the date and returns boolean value.
  */
  public boolean validDate(String inDate) {

  try {

 inDate = inDate.trim();

 StringTokenizer strtkn = new StringTokenizer(inDate , "-");

 if ( strtkn.countTokens() != 3 ) return false;

 String yearstr = strtkn.nextToken();
 String monthstr = strtkn.nextToken();
 String daystr = strtkn.nextToken();

    int day = 0;
    int month = 0;
    int year = 0;
    int len = 0;

 yearstr = IfsApi.chop(yearstr,'0');
     len = yearstr.length();
 if (len != 4)
     return false;

        day = (new Integer(daystr)).intValue();
      month = (new Integer(monthstr)).intValue();
       year = (new Integer(yearstr)).intValue();


    return IfsApi.isValidDate(month,day,year);

        } catch (Exception e) { 
          debug ("<BR> Exception occured in validDate() function of IfsWebTransactionActivity() " + e.toString());
          return false;
        }

  } // end of validDate(String).



  /**
 *Accepts a Timestamp in String format as input parameter.
 *Validates the timestamp and returns a boolean value.
 */
  public boolean validTimestamp(String inTimestamp) {

try {

 inTimestamp = inTimestamp.trim();


 StringTokenizer strtkn = new StringTokenizer(inTimestamp , "-.");

 if ( strtkn.countTokens() != 7 ) return false;

 String year = strtkn.nextToken();
 String month = strtkn.nextToken();
 String day = strtkn.nextToken();
 String hours = strtkn.nextToken();
 String minutes = strtkn.nextToken();
 String seconds = strtkn.nextToken();
 String micro_sec = strtkn.nextToken();

 String date = year + "-" + month + "-" + day;
 if (validDate(date) == false ) return false;


 int hoursi = (new Integer(hours)).intValue();
 int minutesi = (new Integer(minutes)).intValue();
 int secondsi = (new Integer(seconds)).intValue();
 int micro_seci = (new Integer(micro_sec)).intValue();


 if (hoursi<0 || hoursi>23 ||
     minutesi<0 || minutesi>59 ||
     secondsi<0 || secondsi>59 ||
     micro_seci<0 || micro_seci>999999)
    return false;
 else 
    return true; 

 } catch (Exception e) {
   return false;
   }


 }// end of validTimestamp(String)


 /**
 *Accepts a valid Timestamp in String format as input parameter.
 *Inserts a preceding '0' for hours, minutes and seconds fields if they are in single digits.
 */
 public String changeTimestamp(String inputTimestamp) {

 inputTimestamp = inputTimestamp.trim();


 StringTokenizer strtkn = new StringTokenizer(inputTimestamp , "-.");

 String year = strtkn.nextToken();
 String month = strtkn.nextToken();
 String day = strtkn.nextToken();
 String hours = strtkn.nextToken();
 String minutes = strtkn.nextToken();
 String seconds = strtkn.nextToken();
 String micro_sec = strtkn.nextToken();


 int hoursi = (new Integer(hours)).intValue();
 int minutesi = (new Integer(minutes)).intValue();
 int secondsi = (new Integer(seconds)).intValue();
 int micro_seci = (new Integer(micro_sec)).intValue();

 if (hours.length() == 1 && hoursi < 10) 
     hours = "0" + hours;
 if (minutes.length() == 1 && minutesi < 10)
     minutes = "0" + minutes;
 if (seconds.length() == 1 && secondsi < 10)
     seconds = "0" + seconds;

 String outputTimestamp = year + "-" + month + "-" + day + "-" + hours + "." + minutes + "." + seconds + "." + micro_sec;


 return outputTimestamp;

 } // end of changeTimestamp(String)



   /**
   *
   * This method checks the input for the list, read, create, 
   * and delete methods to assure all required input fields have been provided
   * before invoking these methods.  Each of the methods must set an associated 
   * flag reflecting which method is being invoked for validation.
   *
   */
   public String validate(){

      ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",getLocale());

      String errorString = super.validate();

      if (createTxn) {
         //status = OK;
         debug("IfsWebTransactionActivity create validation.");
         if ((getFinancialInstitutionIdentifier() == null) || (getFinancialInstitutionIdentifier().equals("") == true) ||
            (getPrimaryConsumerID() == null) || (getPrimaryConsumerID().equals("") == true) ||
            (getTransactionInitiatorConsumerID() == null) || (getTransactionInitiatorConsumerID().equals("") == true) ||
            (getServiceRequestName() == null) || (getServiceRequestName().equals("") == true) ||
            (getTransactionLoggingReasonCode() == null) || (getTransactionLoggingReasonCode().equals("") == true) ||
            (getTransactionData() == null) || (getTransactionData().equals("") == true) ||

            ( (getFinancialInstitutionIdentifier() != null) && (getFinancialInstitutionIdentifier().equals("") == false) && (getFinancialInstitutionIdentifier().length() > 8) ) ||
            ( (getPrimaryConsumerID() != null) && (getPrimaryConsumerID().equals("") == false) && (getPrimaryConsumerID().length() > 30) ) ||
            ( (getServiceRequestName() != null) && (getServiceRequestName().equals("") == false) && (getServiceRequestName().length() > 8) ) ||
            ( (getTransactionAuthorizerConsumerID() != null) && (getTransactionAuthorizerConsumerID().equals("") == false) && (getTransactionAuthorizerConsumerID().length() > 30) ) ||
            ( (getTransactionBeneficiary() != null) && (getTransactionBeneficiary().equals("") == false) && (getTransactionBeneficiary().length() > 40) ) ||
            ( (getTransactionReference() != null) && (getTransactionReference().equals("") == false) && (getTransactionReference().length() > 40) ) ||
            ( (getTransactionInitiatorConsumerID() != null) && (getTransactionInitiatorConsumerID().equals("") == false) && (getTransactionInitiatorConsumerID().length() > 30) ) ||
            ( (getTransactionLoggingReasonCode() != null) && (getTransactionLoggingReasonCode().equals("") == false) && (getTransactionLoggingReasonCode().length() > 4) ) ||
            ( (getTransactionData() != null) && (getTransactionData().equals("") == false) && (getTransactionData().length() > 31000) ) ||


            ( (getTransactionValueDate() != null) && (getTransactionValueDate().equals("") == false) && (validDate(getTransactionValueDate()) == false) ) ||
            ( (getTransactionLoggedTimestamp() != null) && (getTransactionLoggedTimestamp().equals("") == false) && (validTimestamp(getTransactionLoggedTimestamp()) == false) )||
            ( (getTransactionAuthorizationTimestamp() != null) && (getTransactionAuthorizationTimestamp().equals("") == false) && (validTimestamp(getTransactionAuthorizationTimestamp()) == false) )

             ) {
               status = VALIDATION_ERROR;
               debug("IfsWebTransactionActivity create validation failed, missing required field(s).");
               //debug("IfsWebTransactionActivity create validation - status = " + getStatus());
               //debug("IfsWebTransactionActivity create validation - status FinancialInstitutionIdentifier  = " + getFinancialInstitutionIdentifier());
               errorString = bundle.getString("IfsWebTransactionActivity.create.validation.invalidInput");
         }
      }

      if (readTxn) {
         debug("IfsWebTransactionActivity read validation.");
         //status = OK;
         if ((getFinancialInstitutionIdentifier() == null) || (getFinancialInstitutionIdentifier().equals("") == true) ||
            (getPrimaryConsumerID() == null) || (getPrimaryConsumerID().equals("") == true) ||
            (getTransactionInitiatorConsumerID() == null) || (getTransactionInitiatorConsumerID().equals("") == true) ||
            (getServiceRequestName() == null) || (getServiceRequestName().equals("") == true) ||

            ( (getFinancialInstitutionIdentifier().equals("") == false) && (getFinancialInstitutionIdentifier().length() > 8) ) ||
            ( (getPrimaryConsumerID().equals("") == false) && (getPrimaryConsumerID().length() > 30) ) ||
            ( (getServiceRequestName().equals("") == false) && (getServiceRequestName().length() > 8) ) ||
            ( (getTransactionInitiatorConsumerID().equals("") == false) && (getTransactionInitiatorConsumerID().length() > 30) ) ||

            (getTransactionLoggedTimestamp() == null) || (getTransactionLoggedTimestamp().equals("") == true)   || (!validTimestamp(getTransactionLoggedTimestamp())  )   ) {

               status = VALIDATION_ERROR;
               debug("IfsWebTransactionActivity read validation failed, missing required field(s).");
               //debug("IfsWebTransactionActivity create validation - status = " + getStatus());
               //debug("IfsWebTransactionActivity list validation - status FinancialInstitutionIdentifier  = " + getFinancialInstitutionIdentifier());

               errorString = bundle.getString("IfsWebTransactionActivity.read.validation.invalidInput");
         }
      }

      if (listTxn) {
         //status = OK;
         debug("IfsWebTransactionActivity list validation.");
         if ((getFinancialInstitutionIdentifier() == null) || (getFinancialInstitutionIdentifier().equals("") == true) ||
            (getPrimaryConsumerID() == null) || (getPrimaryConsumerID().equals("") == true) ||

            ( (getFinancialInstitutionIdentifier().equals("") == false) && (getFinancialInstitutionIdentifier().length() > 8) ) ||
            ( (getPrimaryConsumerID().equals("") == false) && (getPrimaryConsumerID().length() > 30) ) 

            ) {
               status = VALIDATION_ERROR;
               debug("IfsWebTransactionActivity list validation failed, missing required field(s).");

               //debug("IfsWebTransactionActivity create validation - status = " + getStatus());
               //debug("IfsWebTransactionActivity list validation - status FinancialInstitutionIdentifier  = " + getFinancialInstitutionIdentifier());
               errorString = bundle.getString("IfsWebTransactionActivity.list.validation.invalidInput");
         }
      }

      if (deleteTxn) {
         debug("IfsWebTransactionActivity delete validation.");
         //status = OK;
         if ((getFinancialInstitutionIdentifier() == null) || (getFinancialInstitutionIdentifier().equals("") == true) ||
            (getPrimaryConsumerID() == null) || (getPrimaryConsumerID().equals("") == true) ||
            (getTransactionInitiatorConsumerID() == null) || (getTransactionInitiatorConsumerID().equals("") == true) ||
            (getServiceRequestName() == null) || (getServiceRequestName().equals("") == true) ||

            ( (getFinancialInstitutionIdentifier().equals("") == false) && (getFinancialInstitutionIdentifier().length() > 8) ) ||
            ( (getPrimaryConsumerID().equals("") == false) && (getPrimaryConsumerID().length() > 30) ) ||
            ( (getServiceRequestName().equals("") == false) && (getServiceRequestName().length() > 8) ) ||
            ( (getTransactionInitiatorConsumerID().equals("") == false) && (getTransactionInitiatorConsumerID().length() > 30) ) ||


            (getTransactionLoggedTimestamp() == null) || (getTransactionLoggedTimestamp().equals("") == true)  || (!validTimestamp(getTransactionLoggedTimestamp())  )  ) {
               status = VALIDATION_ERROR;
               debug("IfsWebTransactionActivity delete validation failed, missing required field(s).");
               //debug("IfsWebTransactionActivity delete validation - status FinancialInstitutionIdentifier  = "+ getFinancialInstitutionIdentifier());

               errorString = bundle.getString("IfsWebTransactionActivity.delete.validation.invalidInput");
         }
      }
      return errorString;
   }

}
