package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsWebTransactionActivityDetail.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
// ***************************************************************************
// Change History
//
//  Date                         Name                Reason For Change
// ------                       ------              -------------------
//
//   --                          --                 Initial Creation
//
// 01/19/2001                  A.Asokan             Fixed bugs
//
//
//
// ****************************************************************************



import java.sql.*;
import java.math.BigDecimal;


/**
* The IfsWebTransactionActivityDetail class is used to contain the details
* of the result set returned from a "read" or "list" method invocation.  This
* will be in a "repeat structure", similar to those returned on most Gold
* "inquiry" messages.
*/
public class IfsWebTransactionActivityDetail extends IfsObject {


/*   The IfsWebTransactionActivityDetail structure is made of the following
     data types 

    Data Field                            Type       # of Characters

Financial Institution Identifier       Character           8
Primary Consumer ID                    Character          30
Transaction Initiator Consumer ID      Character          30
Service Request Name                   Character           8
Transaction Logged Timestamp           Timestamp
Transaction Amount                     Decimal            15
Transaction Amount Precision           Short
Transaction Currency Type              Character           3
Transaction Beneficiary                Character          40
Transaction Reference                  Character          40
Transaction Value Date                 Date
Transaction Logging Reason Code        Integer
Transaction Authorizer Consumer ID     Character          30
Transaction Authorization Timestamp    Timestamp
Transaction Data                       Long Char       31000

FinancialInstitutionIdentifier
PrimaryConsumerID
TransactionInitiatorConsumerID
ServiceRequestName
TransactionLoggedTimestamp
TransactionAmount
TransactionAmountPrecision
TransactionCurrencyType
TransactionBeneficiary
TransactionReference
TransactionValueDate
TransactionLoggingReasonCode
TransactionAuthorizerConsumerID
TransactionAuthorizationTimestamp
TransactionData

*/

   /**
   * This is defined as an abstract method in IfsObject, so we must provide a definition here
   */
   public int getSD_OWNER() {return(-1);}

   private String financialInstitutionIdentifier      = null;
   private String primaryConsumerID                   = null;
   private String transactionInitiatorConsumerID      = null;
   private String serviceRequestName                  = null;
   private String transactionLoggedTimestamp          = null;
   private String transactionAmount                   = null;
   private GsCurrency goldTransactionAmount           = null; 
   private short transactionAmountPrecision           =    0;
   private String transactionCurrencyType             = null;
   private String transactionBeneficiary              = null;
   private String transactionReference                = null;
   private String transactionValueDate                = null;
   private String transactionLoggingReasonCode        = null;
   private String transactionAuthorizerConsumerID     = null;
   private String transactionAuthorizationTimestamp   = null;
   private String transactionData                     = null;


   /**
   * This method gets the financialInstitutionIdentifier field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getFinancialInstitutionIdentifier() {return financialInstitutionIdentifier;}

   /**
   * This method gets the primaryConsumerID field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getPrimaryConsumerID() {return primaryConsumerID;}

   /**
   * This method gets the transactionInitiatorConsumerID field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionInitiatorConsumerID() {return transactionInitiatorConsumerID;}

   /**
   * This method gets the serviceRequestName field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getServiceRequestName() {return serviceRequestName;}

   /**
   * This method gets the transactionLoggedTimestamp field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionLoggedTimestamp() {return transactionLoggedTimestamp;}

   /**
   * This method gets the transactionAmount field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionAmount() {return transactionAmount;}

   /**
   * This method gets the goldTransactionAmount field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.  The goldTransactionAmount
   * object contains the amount, precision, type, and debit/credit indicator.
   */
   public GsCurrency getGoldTransactionAmount() {return goldTransactionAmount;}

   /**
   * This method gets the transactionAmountPrecision field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set".  This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public short getTransactionAmountPrecision() {return transactionAmountPrecision;}

   /**
   * This method gets the transactionCurrencyType field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set".  This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionCurrencyType() {return transactionCurrencyType;}

   /**
   * This method gets the transactionBeneficiary field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set".  This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionBeneficiary() {return transactionBeneficiary;}

   /**
   * This method gets the transactionReference field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set".  This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionReference() {return transactionReference;}

   /**
   * This method gets the transactionValueDate field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionValueDate() {return transactionValueDate;}

   /**
   * This method gets the transactionLoggingReasonCode field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionLoggingReasonCode() {return transactionLoggingReasonCode;}

   /**
   * This method gets the transactionAuthorizerConsumerID field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionAuthorizerConsumerID() {return transactionAuthorizerConsumerID;}

   /**
   * This method gets the transactionAuthorizationTimestamp field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionAuthorizationTimestamp() {return transactionAuthorizationTimestamp;}

   /**
   * This method gets the transactionData field populated by 
   * the "IfsWebTransactionActivity" class which was returned in the SQL query's
   * "result set". This method can be used by the servlet writer to retrieve data
   * that could then be displayed or used in subsequent logic.
   */
   public String getTransactionData() {return transactionData;}


   /**
   * This method sets the financialInstitutionIdentifier field during population of
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setFinancialInstitutionIdentifier(String o) {financialInstitutionIdentifier = o;}

   /**
   * This method sets the primaryConsumerID field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setPrimaryConsumerID(String o) {primaryConsumerID = o;}

   /**
   * This method sets the transactionInitiatorConsumerID field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionInitiatorConsumerID(String o) {transactionInitiatorConsumerID = o;}

   /**
   * This method sets the serviceRequestName field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setServiceRequestName(String o) {serviceRequestName = o;}

   /**
   * This method sets the transactionLoggedTimestamp field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionLoggedTimestamp(String o) {transactionLoggedTimestamp = o;}

   /**
   * This method sets the transactionAmount field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionAmount(String o) {transactionAmount = o;}

   /**
   * This method sets the goldTransactionAmount field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setGoldTransactionAmount(GsCurrency o) {goldTransactionAmount = o;}

   /**
   * This method sets the transactionAmountPrecision field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionAmountPrecision(short o) {transactionAmountPrecision = o;}

   /**
   * This method sets the transactionCurrencyType field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionCurrencyType(String o) {transactionCurrencyType = o;}

   /**
   * This method sets the transactionBeneficiary field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionBeneficiary(String o) {transactionBeneficiary = o;}

   /**
   * This method sets the transactionReference field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionReference(String o) {transactionReference = o;}

   /**
   * This method sets the transactionValueDate field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionValueDate(String o) {transactionValueDate = o;}

   /**
   * This method sets the transactionLoggingReasonCode field as a String during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionLoggingReasonCode(String o) {transactionLoggingReasonCode = o;}


   /**
   * This method sets the transactionLoggingReasonCode field as an integer during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionLoggingReasonCode(int o) {transactionLoggingReasonCode = (new Integer(o)).toString();}



   /**
   * This method sets the transactionAuthorizerConsumerID field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionAuthorizerConsumerID(String o) {transactionAuthorizerConsumerID = o;}

   /**
   * This method sets the transactionAuthorizationTimestamp field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionAuthorizationTimestamp(String o) {transactionAuthorizationTimestamp = o;}

   /**
   * This method sets the transactionData field during population of 
   * the "IfsWebTransactionActivityDetail" object with data which was returned in the SQL query's
   * "result set". This method will be used by the "read" and "list" methods. After they are set,
   * these values may be retrieved for use by the servlet using the corresponding "getter".
   */
   public void setTransactionData(String o) {transactionData = o;}



   /**
   * Converts the IfsWebTransactionActivityDetail object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      if (hasRegisteredMethod("toString"))
         return super.toString();
      else
         return toString("\n");  // Return string from toString with
                                 // carriage return, line feed prefix.
   }

   /**
   * Converts the IfsWebTransactionActivityDetail object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix) {
      if (hasRegisteredMethod("toStringWithString"))
         return super.toString(prefix);

      //Cookie cookie = getIfsSession().getSessionKey();
      StringBuffer buff = new StringBuffer(250);
      buff.append(super.toString(prefix));
      debug("In IfsWebTransactionActivityDetail.toString(String prefix): getting request properties");

      buff.append(prefix + "| transactionInitiatorConsumerID = " + transactionInitiatorConsumerID );
      buff.append(prefix + "| serviceRequestName = " + serviceRequestName );
      buff.append(prefix + "| transactionLoggedTimestamp  = " + transactionLoggedTimestamp  );

     // buff.append(prefix + "| serviceProviderName = " + serviceProviderName );

      debug("In IfsWebTransactionActivityDetail.toString(String prefix): before getting currency properties");
      buff.append(prefix + "| transactionAmountPrecision; = " + Short.toString(transactionAmountPrecision) );
      buff.append(prefix + "| transactionCurrencyType  = " + transactionCurrencyType  );
      buff.append(prefix + "| transactionAmount          = " + transactionAmount );

      if (goldTransactionAmount != null) {
        buff.append(prefix + "| goldTransactionAmount  = " + goldTransactionAmount.toString() );
      }

      debug("In IfsWebTransactionActivityDetail.toString(String prefix): after getting currency properties");
      buff.append(prefix + "| transactionBeneficiary  = " + transactionBeneficiary  );
      buff.append(prefix + "| transactionReference   = " + transactionReference   );
      buff.append(prefix + "| transactionAuthorizerConsumerID  = " + transactionAuthorizerConsumerID  );

      buff.append(prefix + "| transactionAuthorizationTimestamp   = " + transactionAuthorizationTimestamp   );
      buff.append(prefix + "| transactionValueDate   = " + transactionValueDate   );
      buff.append(prefix + "| transactionLoggingReasonCode  = " + transactionLoggingReasonCode );

      

      return buff.toString();

   }


}
