package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   populate.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    03/15/2000  rhs   d6545   NLS changes.
//    02/05/2002  pdl   d7430   Fix calls to deprecated APIs.
//
// ***************************************************************************

import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.ifs.gold.*;
import com.ibm.ifs.gold.auto.*;
import com.ibm.ifs.servlets.IfsHttpServlet;

// helper class for DCR 503
// This class contains a number of helper methods to populate various
// containers from an URL request.
//
// Each method follows the following convention:
//
// + The name of the method is populateClassFromURL where Class is
//   replaced by the classname, ie GsAccount.
// + The method takes 4 parameters:
//    1. An instance of the container class
//    2. The request Object
//    3. A prefix String
//    4. A boolean flag which denotes whether an index is being passed,
//       or the parameters of the container.
//
// The methods return the instance if all went well, or null if unsucessful.

/**
* populate class contains a number of helper methods to populate various
* containers from an URL request.
*/
public class populate
{


   static int getLongFromURL(HttpServletRequest req, String name)
   {
      String s;
      Integer i;
      try
      {
         s = new String(IfsHttpServlet.getParameterValues(name,IfsSystem.cwsGMJCharSet,req)[0]);
      }
      catch (Exception e)
      {
         s = "0";
      }

      try
      {
         i = new Integer(s);   // convert string value to Integer object
      }
      catch (Exception e1)
      {
         i = new Integer(0);
      }
      return(i.intValue());
   }

   static String getFromURL(HttpServletRequest req, String name)
   {
      String s;
      try
      {
         s = new String(IfsHttpServlet.getParameterValues(name,IfsSystem.cwsGMJCharSet,req)[0]);
      }
      catch (Exception e)
      {
         s = "UnsupportedEncodingException";
      }

      return(s);
   }


  public static GsAccount populateGsAccountFromURL(HttpServletRequest req,
						   String prefix,
						   IfsTxn txn,
						   boolean flag,
						   String pattern,
						   String classname)
  {

    if ((req==null))
      {
	txn.debug("passed null values!!!!<br>Exiting populateGsAccountFromURL");
	return null;
      }
    String index=getFromURL(req,prefix+"reqIndexed");
    if(index.indexOf(pattern)!=-1)
      {
	// it's indexed 
	if (txn==null)
	  {
	    System.out.println("Txn is null!");
	  }
	//handle the index case as a special case
	Object bar=null;
	try
	  {
	    bar=txn.getIfsSession().getHttpSession().getAttribute("__UserProfile");
	  }
	catch (Exception Ex1)
	  {
	    txn.debug("Error Trying to get UserProfile from the session:"+Ex1);
	    bar=null;
	  }

	GsAccount acct=new GsAccount("","","");
	GtUserProfile foo=null;
	if (bar==null)
	  {
            try
	      {
		System.out.println("Session="+txn.getIfsSession());
		foo=new GtUserProfile(txn.getIfsSession());
		foo.execute();

	      }
            catch (Exception ex2)
	      {
		txn.debug("Exception encountered getting the User Profile:"+ex2);
	      }
            try
	      {
		if ((foo!=null)&&(foo.getStatus()==1))
		  {
		    txn.getIfsSession().getHttpSession().setAttribute("__UserProfile",foo);
            
		    bar=foo;
		  }
		else
		  {
		    txn.debug("Unable to get the User Profile");
		    txn.debug("Status=="+foo.getStatus());
		    return null;
		  }	      
	      }
	    catch (Exception ex3)
	      {
		txn.debug("Exception encountered caching the User Profile:"+ex3);
	      }
	  }
    // investment accounts go here
    
	if (classname.equals("GtInvestmentOrderStock")||
	    // the next is misspelled in the class
	    classname.equals("GtInvestmentOrderStatusInquiry")||
	    classname.equals("GtOrderCompletionAdvice")||
	    classname.equals("GtGetSecuritiesDescriptions")||
	    classname.equals("GtInvestmentAccountInformation")||
	    classname.equals("GtSettleToAccount")||
	    classname.equals("GtInvestmentAccountDetail")||
	    classname.equals("GtInvestmentOrderCancel")||
	    classname.equals("GtInvestmentOrderCancelChange")||
	    classname.equals("GtInvestmentAccountHistory"))
	  {
	    txn.debug("Got an investment Txn:"+classname);
	    txn.debug("Alias = "+pattern);
	    String s2=getFromURL(req,prefix+pattern);
	    txn.debug("value="+s2);
	    if (s2.equals("UnsupportedEncodingException"))
	      {
		txn.debug("Bailing Out");
		return null;
	      }
	    int acctnum=getLongFromURL(req,prefix+pattern);
	    txn.debug("Getting Account");
	    try 
	      {
		acct=((GtUserProfile)bar).getIfsInvAcctDtl()[acctnum].getAcctId();
		txn.debug("account="+acct.toString("|"));
		txn.debug("Exiting populateGsAccountFromURL");
		return acct;
	      }
	    catch (Exception invExcept)
	      {
		txn.debug("Exception encountered:"+invExcept);
		return null;
	      }
	  }
	else
	  {
	    try 
	      {
		int i=getLongFromURL(req,prefix+pattern);
		acct=((GtUserProfile)bar).getAccountDetail()[i].getAccount();
		return acct;
	      }
	    catch (Exception indexExcept)
	      {
		txn.debug("Exception encountered:"+indexExcept);
		return null;
	      }
	  }
      }
    else			// not indexed
      {
	GsAccount acct=new GsAccount("","","");
        acct.setIfsSession(txn.getIfsSession());
	
	String s1;
	s1=getFromURL(req,prefix+pattern+"_Number");
	if (s1.equals( "UnsupportedEncodingException"))
	  s1=null;
	acct.setNumber(s1);
	
	s1=getFromURL(req,prefix+pattern+"_Nickname");
	if (s1.equals( "UnsupportedEncodingException"))
	  s1=null;
	acct.setNickname(s1);
	
	s1=getFromURL(req,prefix+pattern+"_Type");
	if (s1.equals( "UnsupportedEncodingException"))
	  s1=null;
	acct.setType(s1);
	
	s1=getFromURL(req,prefix+pattern+"_ClearinghouseFIID");
	if (s1.equals( "UnsupportedEncodingException"))
	  s1=null;
	acct.setClearinghouseFIID(s1);
	
	int i=0;
	i=getLongFromURL(req,prefix+pattern+"_SubType");
	acct.setAccountSubType(i);
	
	return acct;
      }
  }



   public static GsDate populateGsDateFromURL(HttpServletRequest req,
                                              String prefix,
                                              IfsTxn txn, boolean flag ,
                                              String pattern,
                                              String classname)
   {
      txn.debug("Entering populateGsDateFromURL, flag=" + flag +
                ", pattern="+pattern);
      if ((req==null))
      {
         txn.debug("Called with nulls, exiting.");
         return null;
      }
      if (flag==true)
      {
         //handle the index case as a special case
      }
      else
      {
         GsDate g=null;
         try
         {
            txn.debug("\n  Passed "+ getFromURL(req,prefix+pattern));
            g=GsDate.fromString(getFromURL(req,prefix + pattern));
            txn.debug("step 1\n");
            GsDateFormat f=new GsDateFormat("yyyy-MM-dd");
            txn.debug("Step 2\n");
            String s=g.toString(f);
            txn.debug("Step 3 and s="+s);
            g=GsDate.fromString(s,f);
            txn.debug("\n returning "+ g.toString()+"\n date="+g.getDate()+
                      "\n  format="+g.getFormat());
         }
         catch (Exception e)
         {
            txn.debug("Exception creating date -- "+e);
            return null;
         }


         return g;
      }
      txn.debug("Exiting");
      return null;
   }


   public static GsCurrency populateGsCurrencyFromURL(HttpServletRequest req,
                                                      String prefix,
                                                      IfsTxn txn,
                                                      boolean flag,
                                                      String pattern,
                                                      String classname)
   {
      txn.debug("Entering populateGsCurrencyFromURL, flag="+
                flag + ", pattern="+pattern);
      if ((req==null))
      {
         txn.debug("<br><b>Called with nulls, exiting.");
         if (req==null)
            txn.debug("b==null!!!");
         txn.debug("</b><br>");
         return null;
      }
      if (flag==true)
      {
         //handle the index case as a special case
      }
      else
      {
         GsCurrency g=null;
         try
         {
            txn.debug("trying to get :"+getFromURL(req,prefix+pattern));
            String s=getFromURL(req,prefix+pattern);
            if (s.equals("UnsupportedEncodingException"))
               return null;
            g=GsCurrency.fromString("$"+getFromURL(req,prefix+pattern));
         }
         catch (Exception e)
         {
            txn.debug("Exception creating currency -- "+e);
            return null;
         }
         return g;
      }
      txn.debug("Exiting");
      return null;
   }



   public static GsDecimal populateGsDecimalFromURL(HttpServletRequest req,
                                                    String prefix,
                                                    IfsTxn txn,
                                                    boolean flag,
                                                    String pattern,
                                                    String classname)
   {
      if ((req==null))
         return null;
      if (flag==true)
      {
         //handle the index case as a special case
      }
      else
      {
         GsDecimal g=null;
         try
         {
            String s=getFromURL(req,prefix+pattern);
            if (s.equals("UnsupportedEncodingException"))
               return null;
            g=GsDecimal.fromString(s);
         }
         catch (Exception e)
         {
            txn.debug("Error creating GsDecimal from "+pattern+":"+e);
            return null;
         }
         return g;
      }
      return null;
   }


   public static GsTime populateGsTimeFromURL(HttpServletRequest req, String prefix, IfsTxn txn, boolean flag, String pattern, String classname)
   {
      if ((req==null))
         return null;
      if (flag==true)
      {
         //handle the index case as a special case
      }
      else
      {
         GsTime g=null;
         try
         {
            String s=getFromURL(req,prefix+pattern);
            if (s.equals("UnsupportedEncodingException"))
               return null;
            g=GsTime.fromString(s);
         }
         catch (Exception e)
         {
            txn.debug("Error creating GsTime from "+pattern+":"+e);
            return null;
         }
         return g;
      }
      return null;
   }

}
