// ***************************************************************************
//  FILE NAME:   IfsCtwHttpServlet.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2001 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//  CMVC Info:
//
//    latest delta date and time........... 02/06/07-12:02:23
//    cmvc path, component, release........ src/cws/jcwsapi/com/ibm/ifs/servlets/IfsCtwHttpServlet.java, cws.jcwsapi, cws.v1r6m1.mmp, v1r6m1_020828
//    sccs release.level.branch.sequence... 1.1
//    sccs filename........................ /cmvc/cmvccws/vc/0/0/0/4/s.86
//
//  CHANGES:
//    11/26/2001  bb/nc/db  dxxxx   Created
//    12/03/2001  bb/nc/db  dxxxx   Updated doGet added debug
//    03/25/2002  pdl       d7499   Get inputStream in service()         
//
// ***************************************************************************

package com.ibm.ifs.servlets;

import java.io.*; 
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsException;
import com.ibm.ifs.gold.IfsCtwGoldTxn;


/**
 * This servlet handles incoming Gold requests <BR>
 * from CTW clients. It looks at the method    <BR>
 * GET,POST or HEAD the client uses to reach it. <BR>
 * After the method the client used has been determined,<BR>
 * it vlidates the content type and length to verify <BR>
 * that the client is indeed valid then it reads the <BR>
 * GOLD request.<BR>
 * This servlet instantiates an IFSCtwGoldTxn and invoke<BR>
 * the native method execCtwGoldTxn0() through a call to <BR>
 * exexCtwGoldTxn().<BR>
 * IfsCtwGoldTxn returns control to this servlet to handle<BR>
 * any error conditions, update the client cookie if it is <BR>
 * needed, sets the content length and writes the Gold <BR>
 * response back to the client.<BR>   
 * 
 **/
public class IfsCtwHttpServlet extends IfsHttpServlet {

/* ------------------------------ Class Variables --------------------------- */

  /**  */
  public static final String HTTP_GET = "GET";

  /**  */
  public static final String HTTP_POST = "POST";

  /**  */
  public static final String HTTP_HEAD = "HEAD";

  /**  */
  public static final String CTW_CONTENT_TYPE ="application/x-ctw-bin";

  private byte[] goldRequest = null; 


  /**
  * Default constructor.
  */
  public IfsCtwHttpServlet() {
	  super();
	  if ( debugOn ) System.out.println( "IfsCtwHttpServlet.cons" );
	  if ( debugOn ) System.out.println( "IfsCtwHttpServlet.cons done" );
  }


/* ---------------------------- Instance Methods  --------------------------- */

  /**
   * This method gets the request's inputStream <BR>
   * before the superclass's service() method <BR>
   * is called. <BR>
   *
   * @param request: Object that contains the request <BR>
   *                 the client has made of the servlet.<BR>
   * @param response: Object that contains the response <BR>
   *                  the servlet sends to the client. <BR>
   * @return void
   * @exception ServletException:  Exception thrown by the servlet<BR>
   *                               when it encounter difficulty. <BR>
   * @exception IOException     :  Exception thrown by the servlet to <BR>
   *                               indicate that an IO Exception occurred.<BR> 
  */
  public void service
( HttpServletRequest request, HttpServletResponse response
 )
    throws ServletException, IOException {
    try {
       goldRequest = readGoldRequest(request);
    }                                                       
    catch (IfsException e) {
      // Exception handling and return without sending respone to client;
       log("Exception while sending/receiving gold message: " + e.getMessage());
      System.out.println( e );
       response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, e.getMessage());
       return;
    }
    super.service(request, response);
 }
  /**
   * This method gets the method Type used by  <BR>
   * the client to reach the IfsCtwHttpServlet. <BR>
   * it reads the content type and content length  <BR>
   * information then it write out the gold message <BR>
   *
   * @param request: Object that contains the request <BR>
   *                 the client has made of the servlet.<BR>
   * @param response: Object that contains the response <BR>
   *                  the servlet sends to the client. <BR>
   * @return void
   * @exception ServletException:  Exception thrown by the servlet<BR>
   *                               when it encounter difficulty. <BR>
   * @exception IOException     :  Exception thrown by the servlet to <BR>
   *                               indicate that an IO Exception occurred.<BR> 
  */
  public void doGet(HttpServletRequest request, HttpServletResponse response)
                     throws ServletException, IOException{
 
     if ( debugOn ) System.out.println( "IfsCtwHttpServlet doGet" );
    String httpMethod = request.getMethod();
     if ( debugOn ) System.out.println( "IfsCtwHttpServlet doGet httpMethod=" + httpMethod );
    String contentType = request.getContentType();
    if ( debugOn ) System.out.println( "IfsCtwHttpServlet doGet contentType=" + contentType );

    if (httpMethod.equals(HTTP_GET)) {
        if ((contentType == null ) || (!contentType.equals(CTW_CONTENT_TYPE))) {
        response.sendError(HttpServletResponse.SC_UNSUPPORTED_MEDIA_TYPE);
        return;
      }
    }
    else if (httpMethod.equals(HTTP_POST)) {
      // Do nothing but don't return, required validation is done outside of this if statement
    }
    else if (httpMethod.equals(HTTP_HEAD)) {
      response.setContentLength(0);
      response.setContentType(null);
      return;
    }
    else {
      response.sendError(HttpServletResponse.SC_NOT_IMPLEMENTED,
                         httpMethod + " not a supported HTTP method");
      return;
    }

    // Required for both GET and POST
    if (request.getContentLength() == -1) {
        response.sendError(HttpServletResponse.SC_LENGTH_REQUIRED);
        return;
    }
     if ( debugOn ) System.out.println( "IfsCtwHttpServlet doGet ready for gold action." );
    byte[] goldResponse = null;
    try {
       goldResponse = sendGoldRequest(goldRequest, request, response);
    }                                                       
    catch (IfsException e) {
      // Exception handling and return without sending respone to client;
       log("Exception while sending/receiving gold message: " + e.getMessage());
      System.out.println( e );
       response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, e.getMessage());
       return;
    }
      
   response.setContentLength(goldResponse.length);
   OutputStream out = response.getOutputStream();
   out.write(goldResponse);
 } //end of doGet method

  /**
   * This method gets the method Type used by <BR>
   * the client to reach the IfsCtwHttpServlet. <BR>
   * it reads content length information then <BR>
   * it sends out the gold message. <BR>
   *
   * @param request : Object that contains the request<BR>
   *                  the client has made of the servlet.<BR>
   * @param response: Object that contains the response <BR>
   *                  the servlet sends to the client. <BR>
   * @return void
   * @exception ServletException:  Exception thrown by the servlet<BR>
   *                               when it encounter difficulty.  <BR>
   * @exception IOException     :  Exception thrown by the servlet to <BR>
   *                               indicate that an IO Exception occurred.<BR> 
  */
  public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException{
    if ( debugOn ) System.out.println( "IfsCtwHttpServlet doPost" );
    doGet(request, response);
  }

  /**
   * This method reads the gold request from the input stream. <BR>
   *
   * @param request : Object that contains the request <BR>
   *                  the client has made of the servlet.  <BR>
   * @return byte[] : Array that will hold the gold request. <BR>
   * @exception IOException     :  Exception thrown by the servlet to <BR>
   *                               indicate that an IO Exception occurred.<BR>
   * @exception IfsException    : Exception thrown by the servlet which is <BR>
   *                             generated by IFS objects and is unique to <BR>
   *                             to the IFS objects. <BR>
  */
  protected byte[] readGoldRequest(HttpServletRequest request) throws IOException, IfsException {
      if ( debugOn ) System.out.println( "IfsCtwHttpServlet.readGoldRequest" );
      byte[] goldRequest = new byte[request.getContentLength()];
      ServletInputStream in = request.getInputStream();
      int bytesRead = in.read(goldRequest, 0, goldRequest.length);
      in.close(); 
      if (bytesRead != goldRequest.length) {
        throw new IfsException("com.ibm.ifs.gold.IfsException", " invalid gold request length");
      }
       if ( debugOn ) System.out.println( "IfsCtwHttpServlet.readGoldRequest done" );
      return goldRequest;
  }
      
  /**
   * This method gets the client cookie value, the Client IP address,<BR>
   * the client URL. It set the client cookie and sends the * gold transaction.<BR>
   * @param goldRequest:  gold request <BR>
   * @param request : Object that contains the request<BR>
   *                  the client has made of the servlet.<BR>
   * @return byte[]    : Array that will hold the gold response.<BR> 
   * @exception IfsException    : Exception thrown by the servlet which is<BR>
   *                             generated by IFS objects and is unique to <BR>
   *                             to the IFS objects.<BR>
  */
  public byte[] sendGoldRequest(byte[] goldRequest, HttpServletRequest request, HttpServletResponse response)
    throws IfsException {
    if ( debugOn ) System.out.println( "IfsCtwHttpServlet.sendGoldRequest" );

    // Get client cookie
    String cookieValue = null;
    Cookie ifsCookie = getCookieByName(request.getCookies(), IfsSystem.cwsCookieName);
    if (ifsCookie != null) {
      cookieValue = ifsCookie.getValue();
      if ( debugOn ) 
        System.out.println("IfsCtwHttpServlet.sendGoldRequest: ifsCookie.getValue cookieValue=" + cookieValue);
    }
         
    // Get client IP address
    String clientIPAddr = request.getRemoteAddr(); 
    if ( debugOn ) System.out.println( "IfsCtwHttpServlet.doGet clientIPAddr=" + clientIPAddr );
         
    // Get requested URL
    String  clientURL  = HttpUtils.getRequestURL(request).toString();
    if ( debugOn ) System.out.println( "IfsCtwHttpServlet.doGet clientURL=" + clientURL );
         
    // Send gold transaction
    IfsCtwGoldTxn ctwGoldTxn = new IfsCtwGoldTxn(clientIPAddr, clientURL, cookieValue, goldRequest);
     String newCookie = ctwGoldTxn.getNewCookie();
     if ( debugOn ) System.out.println( "IfsCtwHttpServlet.sendGoldRequest before execCtwGoldTxn newCookie=" + newCookie );
     byte [] goldResp = ctwGoldTxn.getGoldResp();
     if ( debugOn ) System.out.println( "IfsCtwHttpServlet.sendGoldRequest before execCtwGoldTxn goldResp=" + goldResp );
     if ( goldResp != null ) {
        if ( debugOn ) System.out.println( "IfsCtwHttpServlet.sendGoldRequest before execCtwGoldTxn goldResp length=" + goldResp.length );
     }
    ctwGoldTxn.execCtwGoldTxn();
    
     newCookie = ctwGoldTxn.getNewCookie();
     if ( debugOn ) System.out.println( "IfsCtwHttpServlet.sendGoldRequest after execCtwGoldTxn newCookie=" + newCookie );
     goldResp = ctwGoldTxn.getGoldResp();
     if ( debugOn ) System.out.println( "IfsCtwHttpServlet.sendGoldRequest after execCtwGoldTxn goldResp=" + goldResp );
     if ( debugOn && (goldResp != null))
        System.out.println( "IfsCtwHttpServlet.sendGoldRequest after execCtwGoldTxn goldResp length=" + goldResp.length );
    // set client cookie
    if (ctwGoldTxn.getNewCookie() != null) {
        response.addCookie(new Cookie(IfsSystem.cwsCookieName, ctwGoldTxn.getNewCookie()));
    }

     if ( debugOn ) System.out.println( "IfsCtwHttpServlet.sendGoldRequest done" );
    return ctwGoldTxn.getGoldResp();
  }
        
} // End com.ibm.ifs.servlets.IfsCtwHttpServlet class


  
