package com.ibm.ifs.servlets;

// ***************************************************************************
//
//  FILE NAME:   IfsHttpServlet.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//  CMVC Info:
//    latest delta date and time........... 02/07/17-17:26:18
//    cmvc path, component, release........ src/cws/jcwsapi/com/ibm/ifs/servlets/IfsHttpServlet.java, cws.jcwsapi, cws.v1r6m1.mmp, v1r6m1_020828
//    sccs release.level.branch.sequence... 1.2
//    sccs filename........................ /cmvc/cmvccws/vc/0/0/0/4/s.87
//
//  CHANGES:
//    02/16/2000  rhs   d6341   Made JavaDoc correction/additions.
//    03/03/2000  rhs   d6423   Made JavaDoc additions.
//    03/13/2000  rhs   d6545   Added NLS support
//    05/04/2000  rhs   d6707   Updated checkBrowserLocale to use
//                              IfsNLS.getDefaultCountryCode.
//    01/30/2001  tlt   d7233   Ported sla to v1r6
//    07/10/2001  mbc   T1000035 Fixed navigation locking for packages
//    07/10/2001  tlt   T1000038 Corrected unicode conversion
//    10/26/2001  nc/dob  d7359 Instantiated IfsConnection.
//    11/07/2001  dob     d7365 Improve exception handling. Implement initConnection.
//    11/09/2001  nc/dob  dxxxx Invoke procVerify from service method.
//    11/12/2001  nc/dob  dxxxx added code for getting the client URL,cookie,IP Address
//                               and send s redirection URL for a failure
//    11/26/2001  pdl   d7386 Comment out procVerify.
//    01/15/2002  nc		d7396 Fixed redirection problem
//    01/22/2002  nc/bb d7400 Fixed Copyright problem. added code to improve display 
//                            copyright text
//    02/05/2002  pdl   d7430 Fix calls to deprecated APIs.
//    03/05/2002  pdl   d7478 Fix character encoding problem in toUnicode()
//    07/15/2002  dob   T2000147 Save/retrieve client address to session.
//										
// ***************************************************************************

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.lang.reflect.*;
import java.net.URLEncoder;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Locale;
import java.util.StringTokenizer;
import java.text.Collator;
import java.text.CollationKey;
import java.text.DateFormat;
import java.lang.Double;
import java.lang.Character;
import com.ibm.ifs.gold.GsCurrency;
import com.ibm.ifs.gold.GtLogon;
import com.ibm.ifs.gold.GtReauthentication;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsObject;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsTxn;
import com.ibm.ifs.gold.IComparator;
import com.ibm.ifs.gold.IfsException;
import com.ibm.ifs.gold.IfsNLS;
import com.ibm.ifs.gold.GtSLAEvent;

/**
* IfsHttpServlet class provides additional functions to the page servlet developer
* which make it easier to create and initialize objects, store and retrieve objects
* in the http session, share objects between multiple web pages, sort objects,
* control page navigation, process IFS gold transactions errors, and debug page
* servlet code.
*/
public abstract class IfsHttpServlet extends javax.servlet.http.HttpServlet
{
   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static final boolean  debugOn = false;
   protected static IfsException singleException = null;

   /**
   * Default constructor.
   * instantiate IfsConnection by invoking
   * IfsConnection.getConnection()
   */
   public IfsHttpServlet()
   {
      super();
      if ( debugOn ) System.out.println( "IfsHttpServlet.cons" );
      if ( debugOn ) System.out.println( "IfsHttpServlet.cons done" );
   }

   /**
    *  Called by the servlet container to indicate to a servlet that 
    *  the servlet is being placed into service.
    *
    *  This implementation stores the ServletConfig object it receives 
    *  from the servlet container for later use. When overriding this 
    *  form of the method, call super.init(config).
    */
   public void init( ServletConfig config ) throws ServletException {
      super.init( config );
      if ( debugOn ) System.out.println( "IfsHttpServlet.init" );
      try {
         // Open an IFS connection. Do not repeat if previous errors.
         if ( singleException == null ) 
            IfsConnection.getConnection( this );
      } catch ( IfsException e ) {
         // Warning. Other exceptions such as bad args, out of memory are not handled here.
         // Only errors exclusively dealing with IFS connections are recorded here.
         e.printStackTrace( System.err );
         log( e.toString() );
         singleException = e;
         // Rethrow as a more horrible error so the servlet is not used.
         throw new IllegalStateException( e.toString() );
      }
      if ( debugOn ) System.out.println( "IfsHttpServlet.init done" );
   } // init

	// d7400
   /**
   * Creates the standard copyright section that must
   * appear on every page/servlet.
   * @param response -- An HttpServletResponse object which is being used to
   *    send MIME-encoded data back to the client.
   * @deprecated This method obtains and writes to the passed in HttpServletResponse
   *             PrintWriter Object. In WebSphere 2 you could write to the PrintWriter
   *             and then forward to another resource (Servlet, JSP, etc.), however, in
   *             WebSphere 4 this will cause an Error: 500. This method should only be used
   *             in a resource that does not forward to another.
   */
   public void addCopyright(HttpServletResponse response) throws IOException {
      response.getWriter().println(IfsSystem.getIfsSystemProperty(IfsSystem.COPYRIGHT_PROP_NAME));
   }

   /**
    *
    * @return String Copyright text defined by in the IfsSystem.properties or null
    *         if IfsSystem.properties could not be found or the copyright property
    *         was not defined.
   */
   public static String getCopyright() {
     return IfsSystem.getIfsSystemProperty(IfsSystem.COPYRIGHT_PROP_NAME);
   }

   /**
    *	 This method read the copyright text store in the properties file
    *	 IfsSystem.properties and stores it in the request.
	 *  @param req: An HttpServletRequest object which contains the user's
    *              request.
    */
  public static void addCopyright(HttpServletRequest request) {
    request.setAttribute(IfsSystem.COPYRIGHT_PROP_NAME,
                         IfsSystem.getIfsSystemProperty(IfsSystem.COPYRIGHT_PROP_NAME));
  } // d7400


   /**
   * Redirects the page flow to the page specified by the url.
   * @param response -- An HttpServletResponse object which is being used to
   *    send MIME-encoded data back to the client.
   * @param url -- The uniform resource locator (URL) to which the page flow is
   *    being re-directed too.
   */
   public void callPage(HttpServletResponse response, String url) throws IOException
   {
      response.sendRedirect(url);
   }

   /**
   * Redirects the page flow to the page specified by the url and passes
   * additionalParms string to it.
   * @param response -- An HttpServletResponse object which is being used to
   *    send MIME-encoded data back to the client.
   * @param url -- The uniform resource locator (URL) to which the page flow is
   *    being re-directed too.
   * @param additionalParms -- The additional parameters which are to be appended
   *    to the end of the URL.
   */
   public void callPage(HttpServletResponse response, String url, String additionalParms) throws IOException
   {
      String newUrl;
      if (url.indexOf('?') < 0)
         newUrl = url + "?" + additionalParms;
      else
         newUrl = url + "&" + additionalParms;
      callPage(response, newUrl);
   }

   /**
   * When the session's locale is to be set from the browser's accept-language
   * attribute, this method checks for potential update of the session's locale.
   * When the session locale has not been set yet and the page header does not
   * contain a valid accept-language attribute, the session Locale is set to the
   * IfsSystem.SERVER_LOCALE .
   * <P>
   * When the browser's accept-language (language and country) are different from
   * the session's (language and country) or when the browser's accept-language
   * only specifies a language and it is different from the session's language,
   * the session's locale is changed if the new locale is available on the server.
   * The session's locale is not changed when the browser's accept-language
   * attribute specifies a locale which is not available on the server.
   * <P>
   * The search for an available locale is done in the order of the following three
   * steps.
   * First, the language and country code from the browser's accept-language
   * attribute is used.
   * Second, When browser's accept-language attribute does not specify country code,
   * the default server Locale's (IfsSystem.SERVER_LOCALE) country code is used.
   * Third, When browser's accept-language attribute does not specify country code,
   * the default country code (IfsNLS.getDefaultCountryCode) for the language
   * from the browser's accept-language attribute is used.
   *
   * @param req -- An HttpServletRequest object which contains the user's
   *    request. The accept-language attribute is obtained from this
   *    request's header.
   */
   public static void checkBrowserLocale(HttpServletRequest req)
   {
      int i, indexTemp = 0;
      String language = null;
      String country = null;
      Locale sessionLocale = null;
      String browserAcceptLang = req.getHeader("accept-language");
      HttpSession session = req.getSession(false);

      if ( (IfsSystem.browserLocaleUsage != IfsSystem.USE_SERVER_LOCALE) &&
           (browserAcceptLang != null) && (browserAcceptLang.length() > 1) )
      {
         // The session Locale is to be updated from the page header and the
         // accept-language attribute has been specified in the page header.
         //
         // Use the first language-country code since they are in order of preference.
         indexTemp = browserAcceptLang.indexOf(",");
         if (indexTemp > 0)
            browserAcceptLang = browserAcceptLang.substring(0,indexTemp);

         // Remove any q-value that comes after the semi-colon.
         indexTemp = browserAcceptLang.indexOf(";");
         if (indexTemp > 0)
            browserAcceptLang = browserAcceptLang.substring(0,indexTemp);

         // Isolate the country and language code values.
         indexTemp = browserAcceptLang.indexOf("-");
         if (indexTemp > 0)
         {
            language = browserAcceptLang.substring(0,indexTemp).trim();
            country = browserAcceptLang.substring(indexTemp + 1).trim();
         }
         else
         {
            language = browserAcceptLang.trim();
            country = "";
         }

         // Update the __Locale object in the cache when the browser language or
         // country code is different and the new Locale is available on the server.
         //
         sessionLocale = (Locale)session.getAttribute("__Locale");
         if ( language.equals("") == false )
         {
            Locale newSessionLocale = new Locale(language,country);

            Locale[] availLocales = null;
            int localeFoundState = 0;

            // The search for available locale is done in three tries.
            //   1. The language and country code from the browser is used.
            //   2. When browser did not specify country code use the default server
            //      Locale country with the language from the browser.
            //   3. When browser did not specify country code use the default country
            //      for the language from the browser.
            //
            while( localeFoundState < 3 )
            {
               if ( (sessionLocale == null ) ||
                    (newSessionLocale.getLanguage().equals(sessionLocale.getLanguage()) == false ) ||
                    (newSessionLocale.getCountry().equals(sessionLocale.getCountry()) == false ) )
               {
                  if ( (sessionLocale != null ) &&
                       (country == "" ) &&
                       (language.equals(sessionLocale.getLanguage()) == true ))
                  {
                     // New browser Locale has same language as sessionLocale and country is empty
                     // string.  Don't change the session locale.
                     break;
                  }
               }
               else
               {
                  // New browser Locale has same language and country as sessionLocale or the
                  // sessionLocale is null. Don't change the session locale.
                  break;
               }

               if (availLocales == null)
                  availLocales = DateFormat.getAvailableLocales();

               // Assure the new locale is available on the server.
               for( i = 0; i < availLocales.length; i++)
               {
                  if ( newSessionLocale.equals(availLocales[i]) == true)
                  {
                     // The new Locale is a valid Locale for this server.
                     session.setAttribute("__Locale", newSessionLocale);
                     localeFoundState = 99;
                     break;
                  }
               }

               if (country.equals("") == true)
               {
                  // Browser did not specify country, try using the default country code values.
                  String newCountry = (localeFoundState == 1)
                                      ? IfsNLS.getDefaultCountryCode(language)
                                      : IfsSystem.SERVER_LOCALE.getCountry();

                  newSessionLocale = new Locale(language, newCountry);
               }
               else
               {
                  // The Locale for the specified language and country code is not available.
                  // Don't change the session locale.
                  break;
               }

               ++localeFoundState;  // Change to next state.

            } // end-while, search for available locales.
         } // end-if, browser language is missing.
         else
         {
            // Set the session locale to the server locale when the session locale
            // has not been set for the session yet.
            //
            if (sessionLocale == null)
               session.setAttribute("__Locale", IfsSystem.SERVER_LOCALE);
         }

      } // end-if, set locale from browser accept-language.

   } // end-of, checkBrowserLocale()

   /**
   * Returns an instance of a transaction or data object that is being used on the
   * current web page.
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param className -- Name of the class which is associated with the varName
   *    parameter.
   * @param varName -- Name of the HTTP request parameter whose value is obtained.
   * @return
   *    If "varName" parameter value is present in the HTTP request parameters,
   *    it is parsed to obtain the object's identifier (oid).  This method then tries
   *    to retrieve the object from the session cache using the oid.
   *    If successful, the object is returned.
   *    <P>
   *    When the "varName" parameter value is an oid but there is no object bound to
   *    the oid in the session cache,
   *    a new object of the class is created, cached under the oid, and returned.
   *    When the "varName" parameter is not present in the HTTP request parameters
   *    or it is present but the parameter value is a composite, a new instance of
   *    the class is created and returned.
   *    In all other cases, a null is returned.
   */
   public Object createInstance(IfsSession ifsSession,
                                String     className,
                                String     varName)
   {
      Object obj = null;

      if (ifsSession != null && className != null && varName!= null)
      {
         try
         {
            Class theClass = Class.forName(className);
            HttpServletRequest request = ifsSession.getHttpServletRequest();
            if (varName.equals("")) return null;
            String[] values = IfsHttpServlet.getParameterValues(varName,
                                                                IfsSystem.cwsGMJCharSet,
                                                                request);

            if (values == null || values[0].startsWith("??"))
            {
               // The parameter for setting the object does not exist in the http request, So create the object.
               obj = theClass.newInstance();
               if (obj instanceof IfsObject)
                  ((IfsObject)obj).setIfsSession(ifsSession);
            }
            else if (values[0].startsWith("__"))
            {
               // The parameter value of verName is an OID.
               obj = getObject(request.getSession(false), values[0]);
               if (obj==null && values[0].indexOf('.')<0)
               {
                  // the parameter value in the http request is a base OID but
                  // there is no object cached under that OID. So create the object
                  // and cache it using the given oid.
                  obj = theClass.newInstance();
                  if (obj instanceof IfsObject)
                     ((IfsObject)obj).setIfsSession(ifsSession);
                  putObject(request.getSession(false), (String)values[0], obj);
               }
            }
         }
         catch (Exception e)
         {
            obj = null;
         }
      }
      return obj;
   }

   /**
   * This method is not complete and should not be used.
   */
   public void createSession(HttpServletRequest request, HttpServletResponse response, IfsTxn txn)
   {
      try
      {
         GtLogon logon = (GtLogon)txn;
         HttpSession session = request.getSession(true);
         session.setAttribute("__SessionKey", logon.getSessionKey());
         session.setAttribute("__UserProfile", ""); // logon.getUserProfile());
         session.setAttribute("__Locale", IfsSystem.SERVER_LOCALE );
         session.setAttribute("__OidTable", new Hashtable());
      }
      catch (Exception e)
      {
         handleException(response, e);
      }
   }

   /**
   * When debugging is enabled the debug information is sent to the standard
   * output, a browser, a log file, or a combination of the previous 3
   * output vehicles. The browser output is in HTML format.
   * IfsSystem debug attributes control the enabling and disabling of debug
   * output and to where the debug information is outputted.
   *
   * @param ifsSession -- IfsSession object which contains the
   *    HttpServletResponse object for this client's session.
   * @param str -- The string of debug information.
   * @see IfsSystem
   */
   public void debug(IfsSession ifsSession, String str)
   {
      debug(ifsSession.getHttpServletResponse(), str);
   }

   /**
   * When debugging is enabled the debug information is sent to the standard
   * output, a log file, or a combination of the previous 2
   * output vehicles.
   * IfsSystem debug attributes control the enabling and disabling of debug
   * output and to where the debug information is outputted.
   *
   * @param str -- The string of debug information.
   * @see IfsSystem
   */
   public void debug(String str)
   {
      HttpServletResponse rsp = null;
      debug(rsp, str);
   }

   /**
   * When debugging is enabled the debug information is sent to the standard
   * output, a browser, a log file, or a combination of the previous 3
   * output vehicles. The browser output is in HTML format.
   * IfsSystem debug attributes control the enabling and disabling of debug
   * output and to where the debug information is outputted.
   *
   * @param response -- HttpServletResponse object which contains the
   *    response writer for this client's session.
   * @param str -- The string of debug information.
   * @see IfsSystem
   */
   public void debug(HttpServletResponse response, String str)
   {
      if (!isDebugEnabled()) return;

      if (IfsSystem.debugInfoToStdOut)
         System.out.println(str);

      if (IfsSystem.debugInfoToHtml)
      {
         try
         {
            response.getWriter().println(str +"<BR>");
         }
         catch (Exception ex)
         { /*....*/

         }
      }

      if (IfsSystem.debugInfoToFile)
      {
         try
         {
            if (IfsSystem.debugLogFile == null)
            {
               // true is for auto flush
               IfsSystem.debugLogFile =
               new PrintWriter(new FileOutputStream(IfsSystem.getDebugLogFileName()), true);
            }
            IfsSystem.debugLogFile.println((new java.util.Date()).toString() + " " + str);

         }
         catch (Exception ex)
         { /*....*/

         }
      }
   }

   /**
   * When debugging is enabled the debug information is sent to the standard
   * output, a browser, a log file, or a combination of the previous 3
   * output vehicles. The browser output is in HTML format.
   * IfsSystem debug attributes control the enabling and disabling of debug
   * output and to where the debug information is outputted.
   *
   * @param session -- HttpSession object which contains the
   *    cached HttpServletResponse object for this client's session.
   * @param str -- The string of debug information.
   * @see IfsSystem
   */
   public void debug(HttpSession session, String str)
   {
      debug((HttpServletResponse)(session.getAttribute("__HttpServletResponse")), str);
   }

   /**
   * Returns the URL of the page stored in the session cache under
   * the "__LockedPage" oid.
   * The "__LockedPage" oid is created by the lockNavigation method.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session (the HttpSession object plus other attributes).
   * @return
   *    When the "__LockedPage" oid exists in the session cache, the URL
   *    associated with it is returned. When no URL is associated with the
   *    "__LockedPage" oid, a null is returned.
   */
   public String getLockedPage(IfsSession ifsSession)
   {
      if (ifsSession != null)
         return getLockedPage(ifsSession.getHttpSession());
      else
         return null;
   }

   /**
   * Returns the URL of the page stored in the session cache under
   * the "__LockedPage" oid.
   * The "__LockedPage" oid is created by the lockNavigation method.
   *
   * @param session -- HttpSession object which contains the session cache for
   *    this client's session.
   * @return
   *    When the "__LockedPage" oid exists in the session cache, the URL
   *    associated with it is returned. When no URL is associated with the
   *    "__LockedPage" oid, a null is returned.
   */
   public String getLockedPage(HttpSession session)
   {
      if (session != null)
         return(String)session.getAttribute("__LockedPage");
      else
         return null;
   }

   /**
   * Returns the getter method for fieldName parameter.
   *
   * @param fieldName -- Name of the field (class attribute) whose getter
   *    method is to be returned.
   * @return
   *    When the fieldName parameter value starts with "get", the fieldName value
   *    is returned.  Otherwise, the first letter of fieldName is capitalized
   *    and "get" is appended to front of fieldname.  This value is then returned.
   */
   private String getMethodName(String fieldName)
   {
      String objMethodName = fieldName;

      if (!(fieldName.toLowerCase().startsWith("get")))
         objMethodName = "get"+fieldName.substring(0,1).toUpperCase()+ fieldName.substring(1);

      return objMethodName;
   }

   /**
   * Returns a unique system generated object identifier (oid) for the session.
   *
   * The oid is a string used as the key to store or retrieve an object to or from
   * the session cache.  An oid must start with two underscores.
   * The format of the returned oid is __className_# (where className is the
   * class name for the object excluding the package name, and # is a unique number).
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session (the HttpSession object plus other attributes).
   * @param className -- Name of the class for which an oid is to be generated.
   * @return
   *    A unique system generated object identifier (oid) is created for the session
   *    and returned.  If either of the input parameters is equal to null,
   *    then a null value is returned.
   */
   public String getNewOid(IfsSession ifsSession, String className)
   {
      if (ifsSession != null)
         return getNewOid(ifsSession.getHttpSession(), className);
      else
         return null;
   }

   /**
   * Returns a unique system generated object identifier (oid) for the session.
   *
   * The oid is a string used as the key to store or retrieve an object to or from
   * the session cache.  An oid must start with two underscores.
   * The format of the oid is __className_# (where className is the class name for
   * the object excluding the package name, and # is a unique number).
   *
   * @param session -- HttpSession object which contains the information about
   *    this client's session.
   * @param className -- Name of the class for which an oid is to be generated.
   * @return
   *    A unique system generated object identifier (oid) is created for the session
   *    and returned.  If either of the input parameters is equal to null,
   *    then a null value is returned.
   */
   public String getNewOid(HttpSession session, String className)
   {
      if (session != null && className != null)
      {
         int index = className.lastIndexOf('.');
         if (index > 0)
         {
            try
            {
               Class  theClass = Class.forName(className);
               if (theClass==null) return null;
            }
            catch (Exception e)
            {
               return null;
            }
            className = className.substring(index + 1);
         }
         String  countKey = "____ObjectCount";
         String oid = null;
         synchronized(session)
         {
            Integer count = (Integer) getObject(session, countKey);
            if (count == null) count = new Integer(0);
            int c = count.intValue();
            oid = "__" + className + "_" + c++;
            putObject(session, countKey, new Integer(c));
         }
         return oid;
      }
      else return null;
   }

   /**
   * Using an oid, it returns the associated cached object or its designated field
   * from the session cache.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param oid -- Object identifier of the object to be returned from the cache.
   * @return
   *    The object or its designated field which is associated with the oid
   *    in the session cache is returned.
   *    If the oid is invalid, a null is returned.
   */
   public Object getObject(IfsSession ifsSession, String oid)
   {
      if (ifsSession != null)
         return getObject(ifsSession.getHttpSession(), oid);
      else
         return null;
   }

   /**
   * Using an oid, it returns the associated cached object or its designated field
   * from the session cache.
   *
   * @param httpSessionn -- HttpSession object which contains the session cache for
   *    this client's session.
   * @param oid -- Object identifier of the object to be returned from the cache.
   * @return
   *    The object or its designated field which is associated with the oid
   *    in the session cache is returned.
   *    If the oid is invalid, a null is returned.
   */
   public Object getObject(HttpSession httpSession, String oid)
   {
      Object obj = null;

      if (oid != null && oid.startsWith("__"))
      {
         try
         {
            int index = oid.indexOf('.');
            if (index<0)
            {
               // oid is just a base oid.
               obj = httpSession.getAttribute(oid);
            }
            else
            {
               // oid is a compound oid.
               StringTokenizer st = new StringTokenizer(oid, ".");
               String baseOid = st.nextToken();
               obj= httpSession.getAttribute(baseOid);

               if (obj != null)
               {
                  Class    theClass;
                  String   fieldName;
                  Method   method;
                  Class[]  classParms = new Class[]{};
                  Object[] objectParms = new Object[]{};
                  int      i;

                  while (st.hasMoreTokens())
                  {
                     theClass = obj.getClass();
                     fieldName = st.nextToken();
                     try
                     {
                        // fieldName is actually an array index
                        i = Integer.parseInt(fieldName);
                        obj = ((Object[])obj)[i];
                     }
                     catch (NumberFormatException e)
                     {
                        // fieldName is indeed a field name
                        method = theClass.getMethod("get"+
                                                    String.valueOf(Character.toUpperCase(fieldName.charAt(0))) +
                                                    fieldName.substring(1),
                                                    classParms);
                        obj = method.invoke(obj, objectParms);
                     }
                  }
               }
            }
         }
         catch (Exception e)
         {
            obj = null;
         }
      }

      return obj;
   }

   /**
   * Returns the oid which is associated with the object in the session cache.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param object -- Object in the cache whose oid is to be returned.
   * @return
   *    The object's object identifier in the session cache is
   *    returned. If the object is not cached or either of the inputs is null,
   *    a null is returned.
   */
   public String getOid(IfsSession ifsSession, Object object)
   {
      if (ifsSession != null)
         return getOid(ifsSession.getHttpSession(), object);
      else
         return null;
   }

   /**
   * Returns the oid which is associated with the object in the session cache.
   *
   * @param session -- HttpSession object which contains the session cache for
   *    this client's session.
   * @param object -- Object in the cache whose oid is to be returned.
   * @return
   *    The object's object identifier in the session cache is
   *    returned. If the object is not cached or either of the inputs is null,
   *    a null is returned.
   */
   public String getOid(HttpSession session, Object object)
   {
      String oid = null;
      if (object != null && session != null)
      {
         Hashtable oidTable = (Hashtable)session.getAttribute("__OidTable");
         oid = (String)oidTable.get(object);
      }
      return oid;
   }

   /**
   * Using the specified character set, this method obtains one or more values
   * associated with "name" parameter from the HttpServletRequest.
   *
   * @param name -- The name of the HttpServletRequest parameter.
   * @param charSet -- The character set which is used to encode the byte string value
   *    from the request to a unicode String object.
   * @param req -- HttpServletRequest object which contains the request parameters
   *    for this client's session.
   * @return
   *    Returns the values associated with "name" as an array of String objects.
   */
   public static String[] getParameterValues( String name, String charSet,
                                              HttpServletRequest req)
   {
      String[] newValues = null;
      String[] values = req.getParameterValues(name);

      if (values != null)
      {
         newValues = new String[values.length];

         // For each value, convert the byte values to valid unicode characters.
         for( int i = 0; i < values.length; i++)
            newValues[i] = toUnicodeString( values[i], charSet);
      }

      return( newValues);
   }

   /**
   * Returns the object specified by the name in the HttpServletRequest parameter.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param parmName -- The name of the HttpServletRequest parameter.
   * @return
   *    When the value, from HttpServletRequest parameter string,
   *    associated with the name specified in "parmName" is a valid oid,
   *    the corresponding cached object or its designated field is returned.
   *    <P>
   *    If the parameter value is not an oid, the string itself is returned.
   *    If the parameter does not exist in the HttpServletRequest, or the
   *    parameter value is an oid but there is no object bound to the oid,
   *    a null is returned.
   */
   public Object getRequestParameter(IfsSession ifsSession, String parmName)
   {
      if (ifsSession != null)
         return getRequestParameter(ifsSession.getHttpServletRequest(), parmName);
      else
         return null;
   }

   /**
   * Returns the object specified by the name in the HttpServletRequest parameter.
   *
   * @param request -- HttpServletRequest object which contains the request
   *    parameters for this client's session.
   * @param parmName -- The name of the HttpServletRequest parameter.
   * @return
   *    When the value, from HttpServletRequest parameter string,
   *    associated with the name specified in "parmName" is a valid oid,
   *    the corresponding cached object or its designated field is returned.
   *    <P>
   *    If the parameter value is not an oid, the string itself is returned.
   *    If the parameter does not exist in the HttpServletRequest, or the
   *    parameter value is an oid but there is no object bound to the oid,
   *    a null is returned.
   */
   public Object getRequestParameter(HttpServletRequest request, String parmName)
   {
      Object   obj   = null;
      try
      {
         String[] values = IfsHttpServlet.getParameterValues(parmName,
                                                             IfsSystem.cwsGMJCharSet,
                                                             request);

         if (values!=null)
         {
            if (values[0].startsWith("__"))
            {
               obj = getObject(request.getSession(false), values[0]);
            }
            else
            {
               obj = values[0];
            }
         }
      }
      catch (Exception e)
      {
         obj = null;
      }

      return obj;
   }


	// d7396

   /**
   * Redirects the page flow to the proper page based on the transaction object's
   * execution status.
   * <P>
   * In addition to the three error conditions (REDO_ERROR, INFO_ERROR, FATAL_ERROR),
   * this method handles reroutes to a Reauthorization or Logon page.
   * Conditions causing a reroute of this type are CWS
   * completion codes returned from the Gold Manager.  This method compares the codes returned
   * for aTxn with the IfsSystem.ReauthBRC and IfsSystem.ReLogonBRC to determine if the browser
   * should be rerouted to the Reauthorization or Logon page.  If the transaction code values match the
   * values in the IfsSystem object, rerouting will occur.  The Reauthorization and Logon pages
   * to reroute to are also set within the IfsSystem object as ReauthenticationPage and LogonPage.
   * <P>
   * If a redirection does occur, the current transaction object is cached and
   * passed to the redirect page. When no redirection occurs, control remains
   * with the current servlet page.
   *
   *
   * This method returns a true or false boolean. true is return if they are no errors.
   * 
   * @param aTxn -- IfsTxn transaction object which contains the transaction status
   *     and error codes.
   * @param errorPage -- If the status is "FATAL_ERROR" or any other types of errors,
   *     and errorPage is not null, page flow is redirected to the errorPage URL.
   * @param infoPage -- If the status is "INFO_ERROR" and infoPage is not null,
   *     page flow is redirected to the infoPage URL.
   * @param redoPage -- If the status is "REDO_ERROR" and redoPage is not null,
   *     page flow is redirected to the redoPage URL.
   * @see IfsSystem
   */
   public boolean handleErrors(IfsTxn aTxn,
                            String errorPage,
                            String infoPage,
                            String redoPage) throws IOException
   {
      boolean ret_Val = false;
      int txnStatus = aTxn.getStatus();
      int txnErrorCode = aTxn.getErrorCode();
      HttpSession session = aTxn.getIfsSession().getHttpSession();

      if (txnStatus == IfsTxn.OK) { // 1 is OK
		ret_Val = true;
		return ret_Val;
      }

      if (txnStatus == IfsTxn.INFO_ERROR && infoPage != null)
      {
         session.setAttribute("__ErrorPage", infoPage);
         callPage(aTxn.getIfsSession().getHttpServletResponse(), infoPage,
                  "txn="+ putObject(aTxn.getIfsSession(), aTxn));
      }
      else if (txnStatus == IfsTxn.REDO_ERROR || txnStatus == IfsTxn.VALIDATION_ERROR && redoPage != null)
      {
         session.setAttribute("__ErrorPage", redoPage);
         callPage(aTxn.getIfsSession().getHttpServletResponse(), redoPage,
                  "txn="+ putObject(aTxn.getIfsSession(), aTxn));

      } else if (isSpecialReroute(aTxn)) {
        handleSpecialReroute(aTxn);

      } else if (errorPage != null)
      {
         session.setAttribute("__ErrorPage", errorPage);
         // IfsTxn.FATAL_ERROR or other types of errors
         callPage(aTxn.getIfsSession().getHttpServletResponse(), errorPage,
                  "txn="+ putObject(aTxn.getIfsSession(), aTxn));
      }
     return ret_Val; 
   }	// d7396

   /**
   * Determines if special rerouting is required.  Special rerouting is required for user
   * reauthentication and for re-logging onto the system.  The transaction passed to this
   * method is to determine if rerouting is required.  The transaction's completion code
   * is compared to customizable values in the IfsSystem object.  The reauthentication
   * customized variable is ReauthBRC and the logon customized variable is ReLogonBRC.
   *
   * @param aTxn -- IfsTxn transaction object which contains the completion code.
   * @return
   *     Returns true when the transaction completion code equals
   *     IfsSystem.ReauthBRC or IfsSystem.ReLogonBRC, false otherwise.
   */
   private boolean isSpecialReroute(IfsTxn aTxn) {
        String compCode = aTxn.getCompCode();
        return (compCode != null && (compCode.equals(IfsSystem.ReauthBRC) || compCode.equals(IfsSystem.ReLogonBRC)));
   }

   /**
   * Handles rerouting for reauthentication and relogon conditions.  Reauthorization and Logon pages
   * are specified within the IfsSystem object via the ReauthenticationPage and LogonPage parameters.
   * If rerouting to the Reauthentication page, the current servlet page and any request parameters are
   * formated and sent to the reauthentication page as referer.  After successful reauthentication, the
   * reauthentication page can use the referer request parameter to reroute the browser back to the
   * servlet page that was being executed before the need to reauthenticate.
   *
   * @param aTxn -- IfsTxn transaction object which contains the completion code.
   */
   private void handleSpecialReroute(IfsTxn aTxn) throws IOException {
        String compCode = aTxn.getCompCode();
        if(compCode != null) {
                if(compCode.equals(IfsSystem.ReauthBRC)) {
                        debug("IfsHttpServlet.handleSpecialReroute() rerouting to Reauth Page");
                        StringBuffer buff = new StringBuffer();
                        buff.append(aTxn.getIfsSession().getHttpServletRequest().getRequestURI());
                        buff.append(getRequestParameterPairs(aTxn));
                        buff.append("&method=" + aTxn.getIfsSession().getHttpServletRequest().getMethod());
                        debug("IfsHttpServlet.handleSpecialReroute() - method value is " + aTxn.getIfsSession().getHttpServletRequest().getMethod());
                        aTxn.getIfsSession().getHttpSession().setAttribute("referer", buff.toString());
                        callPage(aTxn.getIfsSession().getHttpServletResponse(), IfsSystem.ReauthenticationPage);

                } else if(compCode.equals(IfsSystem.ReLogonBRC))
                        callPage(aTxn.getIfsSession().getHttpServletResponse(), IfsSystem.LogonPage);
        }
        return;
   }

   /**
   * This method constructs all of the http request parameters into one URL Encoded string.
   * The resulting string is used in conjunction with the current servlet page name as the
   * value for the referer http request parameter.
   *
   * @param aTxn -- IfsTxn transaction object which contains the HttpServletRequest.
   * @return
   *     The http request parameters encoded into one URL string.
   */
   protected String getRequestParameterPairs(IfsTxn aTxn) {
        HttpServletRequest req = aTxn.getIfsSession().getHttpServletRequest();
        Enumeration names = req.getParameterNames();
        String[] values = null;
        StringBuffer buff = new StringBuffer();
        buff.append("?");
        String name = null;

        while(names.hasMoreElements()) {
                name = (String) names.nextElement();
                buff.append(name);
                buff.append("=");
                buff.append(IfsHttpServlet.getParameterValues(name,IfsSystem.cwsGMJCharSet,req)[0]);
                if(names.hasMoreElements())
                        buff.append("&");

        }
        return buff.toString();
   }
   /**
   * This method determines if the previous page method was a POST.  If the method was POST, a
   * boolean value of true will be returned, otherwise a boolean value of false will be returned.
   * This method uses objects from the HTTP session cache to make the determination of the
   * method value.  This helper method is used when a reauthentication event occured and the
   * browser must be rerouted back to the previous page.
   */
   public static boolean isPreviousPageMethodPost(HttpServletRequest request) {
        GsCurrency curr = new GsCurrency();
        try {
                String referer = (String)request.getSession(false).getAttribute("referer");
                curr.debug("IfsHttpServlet.isPreviousPageMethodPost() - Size of referer is " + referer.length());
                StringTokenizer uriString = new StringTokenizer(referer, "?", false);
                String page = (String)uriString.nextElement();
                curr.debug("IfsHttpServlet.isPreviousPageMethodPost() - page value is " + page);
                curr.debug("IfsHttpServlet.isPreviousPageMethodPost() getting request Parameters");
                String queryString = (String)uriString.nextElement();
                Hashtable requestParms = HttpUtils.parseQueryString(queryString);
                String method = ((String[])requestParms.get("method"))[0];
                if(null != method && method.equalsIgnoreCase("post"))
                        return true;

                return false;
        }
        catch(Exception e) {
                curr.debug("IfsHttpServlet.isPreviousPageMethodPost() - exception occurred, valueis " + e.toString());
                return false;
        }
   }
   /**
   * This method returns the name of the previous page. This method uses objects from the
   * HTTP session cache to make the determination of the previous page value.  This helper
   * method is used when a reauthentication event occurs and the browser must be rerouted
   * back to the previous page. If an exception occurs or the page name value is not found
   * in the Http Session cache a null value is returned.
   */
   public static String getPreviousPage(HttpServletRequest request) {
        GsCurrency curr = new GsCurrency();
        try {
                String referer = (String)request.getSession(false).getAttribute("referer");
                curr.debug("IfsHttpServlet.getPreviousPage() - Size of referer is " + referer.length());
                StringTokenizer uriString = new StringTokenizer(referer, "?", false);
                return (String)uriString.nextElement();
        }
        catch(Exception e) {
                curr.debug("IfsHttpServlet.getPreviousPage() - exception occurred, value is " + e.toString());
                return null;
        }
   }
   /**
   * This method adds all of the previous page request parameters to the form as hidden fields.
   * The HTML form tag must have already been set in the Print Writer for the Http Servlet
   * Response.  This method uses objects previously set into the HTTP session cache for
   * determining the name/value request parameter pairs from the previous page.  This method
   * is used when a reauthentication event occurs and the browser must be rerouted back to the
   * previous page.
   */
   public static void addPreviousPageRequestParameters(HttpServletRequest request,PrintWriter pw) {
        GsCurrency curr = new GsCurrency();
        Object key = null;
        try {
                String referer = (String)request.getSession(false).getAttribute("referer");
                StringTokenizer uriString = new StringTokenizer(referer, "?", false);
                uriString.nextElement();
                curr.debug("IfsHttpServlet.addPreviousPageRequestParameters() getting request Parameters");
                String queryString = (String)uriString.nextElement();
                Hashtable requestParms = HttpUtils.parseQueryString(queryString);
                curr.debug("IfsHttpServlet.addPreviousPageRequestParameters() getting keys");
                Enumeration keys = requestParms.keys();
                while(keys.hasMoreElements()) {
                        curr.debug("IfsHttpServlet.addPreviousPageRequestParameters() getting specific key");
                        key = keys.nextElement();
                        curr.debug("IfsHttpServlet.addPreviousPageRequestParameters() - Key is " + key + ", value is " + ((String[])requestParms.get(key))[0]);
                        pw.println("<INPUT TYPE=hidden NAME='" + key + "' VALUE='" + ((String[])requestParms.get(key))[0] + "'>");

                }
        }
        catch(Exception e) {
                curr.debug("IfsHttpServlet.addPreviousPageRequestParameters() - Exception occurred, value is " + e.toString());
        }
   }

   /**
   * When enabled the exception stack trace is sent to the standard
   * output, a log file, or a combination of the previous 2
   * output vehicles.
   * IfsSystem exception stack trace attribute (IfsSystem.exceptionLogFileName)
   * controls the enabling and disabling
   * of stack trace output and to where it is outputted.
   *
   * @param exception -- The exception whose stack trace is to be outputted.
   * @see IfsSystem
   */
   public void handleException(Throwable exception)
   {
      HttpServletResponse response = null;
      handleException(response, exception);
   }

   /**
   * When enabled the exception stack trace is sent to the standard
   * output, a browser, a log file, or a combination of the previous 3
   * output vehicles.
   * IfsSystem exception stack trace attributes (IfsSystem.exceptionInfoToHtml,
   * IfsSystem.exceptionLogFileName) control the enabling and disabling
   * of stack trace output and to where it is outputted.
   *
   * @param response -- HttpServletResponse object which contains the response object
   *     for this client's session.
   * @param exception -- The exception whose stack trace is to be outputted.
   * @see IfsSystem
   */
   public void handleException(HttpServletResponse response, Throwable exception)
   {
      if (response != null && IfsSystem.exceptionInfoToHtml)
      {
         try
         {
            exception.printStackTrace(response.getWriter());
         }
         catch (IOException e)
         {
         }
      }

      if (IfsSystem.exceptionLogFileName != null)
      {
         try
         {
            // true is for auto flush
            PrintWriter logFile = new PrintWriter(new FileOutputStream(IfsSystem.exceptionLogFileName, true), true);
            exception.printStackTrace(logFile);
         }
         catch (IOException e)
         {
         }
      }

      exception.printStackTrace(System.out);
   }

   /**
   * When enabled the exception stack trace is sent to the standard
   * output, a browser, a log file, or a combination of the previous 3
   * output vehicles.
   * IfsSystem exception stack trace attributes (IfsSystem.exceptionInfoToHtml,
   * IfsSystem.exceptionLogFileName) control the enabling and disabling
   * of stack trace output and to where it is outputted.
   *
   * @param session -- HttpSession object which contains the response object
   *     for this client's session.
   * @param exception -- The exception whose stack trace is to be outputted.
   * @see IfsSystem
   */
   public void handleException(HttpSession session, Throwable exception)
   {
      if (session != null)
         handleException((HttpServletResponse)session.getAttribute("__HttpServletResponse"), exception);
   }

   /**
   * Indicates whether debugging is enabled for this class.
   * IfsSystem debug attributes control the enabling and disabling of debug
   * output and to where the debug information is outputted.
   *
   * @return
   *     Returns true when debugging is enabled, false otherwise.
   * @see IfsSystem
   */
   public boolean isDebugEnabled()
   {
      if (IfsSystem.debugMode == IfsSystem.DEBUG_OFF) return false;
      if (IfsSystem.debugMode == IfsSystem.DEBUG_ON ) return true;
      try
      {
         Class theClass = this.getClass();
         Field field = theClass.getField("debugOn");
         if (field != null) return field.getBoolean(null);
      }
      catch (Exception e)
      {

      }
      return false;
   }

   /**
   * Indicates whether page flow navigation has been locked for this session.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @return
   *     Returns true when the __LockedPage oid is associated with a value in the
   *     current session cache, false otherwise.
   */
   public boolean isNavigationLocked(IfsSession ifsSession)
   {
      return isNavigationLocked(ifsSession.getHttpSession());
   }

   /**
   * Indicates whether page flow navigation has been locked for this session.
   *
   * @param session -- HttpSession object which contains the session cache for
   *    this client's session.
   * @return
   *    Returns true when the __LockedPage oid is associated with a value in the
   *    current session cache, false otherwise.
   */
   public boolean isNavigationLocked(HttpSession session)
   {
      return((String)session.getAttribute("__LockedPage") != null);
   }

   /**
   * Puts the URL in the session cache under the __LockedPage oid
   * and there by locks page navigation flow to this URL for the session.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param url -- The URL to which page flow navigation is to be locked too.
   */
   public void lockNavigation(IfsSession ifsSession, String url) throws IOException
   {
      ifsSession.getHttpSession().setAttribute("__LockedPage", url);
   }

   /**
   * Puts the URL in the session cache under the __LockedPage oid
   * and there by locks page navigation flow to this URL for the session.
   *
   * @param request -- HttpServletRequest object which has access to the session
   *    cache for this client's session.
   * @param url -- The URL to which page flow navigation is to be locked too.
   */
   public void lockNavigation(HttpServletRequest request, String url) throws IOException
   {
      request.getSession(false).setAttribute("__LockedPage", url);
   }

   /**
   * Searches an array of strings for a match with the target string.
   *
   * @param target -- The string which is used to be searched for a match in an
   *    array of strings.
   * @param list -- An array of strings which is searched for a matching entry.
   * @return
   *    The index of the first array entry in the list which equals the target
   *    string is returned.
   *    When no array entry in the list equals the target string, a -1 is returned.
   */
   public int Match(String target, String[] list)
   {
      for (int i=0;i<list.length;i++)
      {
         if (target.equals(list[i])) return i;
      }
      return -1;
   }

   /**
   * Puts the object into the session cache under a system generated unique oid
   * and returns the oid.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param object -- The object which is to be cached in the session cache.
   * @return
   *    When the object is already cached, the existing oid associated with
   *    the object in the session cache is returned.  If either ifsSession or
   *    object is null, a null is returned.
   */
   public String putObject(IfsSession ifsSession, Object object)
   {
      if (ifsSession != null)
         return putObject(ifsSession.getHttpSession(), null, object);
      else
         return null;
   }

   /**
   * Puts the object into the session cache under the specified oid and returns the
   * oid. If the oid is already used for a cached
   * object, the existing cached object is removed from the cache and the object
   * is then cached under the specified oid.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param oid -- The oid which is to be assigned to the object in the session cache.
   * @param object -- The object which is to be cached in the session cache.
   * @return
   *    If the object is already cached, the existing oid associated with the object
   *    in the session cache is returned. If either ifsSession or object is null or
   *    the oid is invalid or a compound value, a null is returned.
   */
   public String putObject(IfsSession ifsSession, String oid, Object object)
   {
      if (ifsSession != null)
         return putObject(ifsSession.getHttpSession(), oid, object);
      else
         return null;
   }

   /**
   * Puts the object in the session cache under a system generated unique oid
   * and returns the oid.
   *
   * @param session -- HttpSession object which contains the session cache for
   *    this client's session.
   * @param object -- The object which is to be cached in the session cache.
   * @return
   *    If the object is already cached, the existing oid bound to the object in
   *    the session is returned. If either session or object is null, a null is returned.
   */
   public String putObject(HttpSession session, Object object)
   {
      return putObject(session, null, object);
   }

   /**
   * Puts the object in the session cache under the specified oid and returns the
   * oid. If the oid is already used for a cached object, the existing cached
   * object is removed from the cache and the object is then cached under the oid.
   *
   * @param session -- HttpSession object which contains the session cache for
   *    this client's session.
   * @param oid -- The oid which is to be assigned to the object in the session cache.
   * @param object -- The object which is to be cached in the session cache.
   * @return
   *    If the object is already cached, the existing oid associated with the object
   *    in the session cache is returned. If either session or object is null or
   *    the oid is invalid or a compound value, a null is returned.
   */
   public String putObject(HttpSession session, String oid, Object object)
   {
      String theOid = null;

      if (session != null && object != null)
      {
         //  oid is a base oid.
         Hashtable oidTable = (Hashtable)session.getAttribute("__OidTable");
         theOid = (String)oidTable.get(object);
         if (theOid == null)
         {
            if (oid == null || oid.equals(""))
               theOid = getNewOid(session, object.getClass().getName());
            else if (oid.indexOf('.')<0 && oid.startsWith("__"))
            {
               theOid = oid;
               Object oldObject = session.getAttribute(theOid);
               if (oldObject != null)
                  oidTable.remove(oldObject);
            }
            else
               return null;
            session.setAttribute(theOid,object);
            oidTable.put(object,theOid);
         }
      }
      return theOid;
   }

   /**
   * Removes the object from the session cache and returns the object.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param object -- The object which is to be removed from the session cache.
   * @return
   *    The removed object is returned. If either ifsSession or object is null,
   *    a null is returned.
   */
   public Object removeObject(IfsSession ifsSession, Object object)
   {
      if (ifsSession != null)
         return removeObject(ifsSession.getHttpSession(),object);
      else
         return null;
   }

   /**
   * Removes the object cached under the specified oid from the session cache
   * and returns the object.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param oid -- The oid which is assigned to the object that is to be removed
   *    from the session cache.
   * @return
   *    The removed object is returned.  A null is returned
   *    when no object is associated with the oid in the session cache
   *    or the ifsSession is null.
   *    Also when the oid is invalid or is a compound value, a null is returned.
   */
   public Object removeObject(IfsSession ifsSession, String oid)
   {
      if (ifsSession != null)
         return removeObject(ifsSession.getHttpSession(), oid);
      else
         return null;
   }

   /**
   * Removes the object from the session cache and returns the object.
   *
   * @param session -- HttpsSession object which contains the session cache for
   *    this client's session.
   * @param object -- The object which is to be removed from the session cache.
   * @return
   *    The removed object is returned. If either session or object is null,
   *    a null is returned.
   */
   public Object removeObject(HttpSession session, Object object)
   {
      if (session != null && object != null)
      {
         Hashtable oidTable = (Hashtable)session.getAttribute("__OidTable");
         String oid = (String)oidTable.get(object);
         if (oid!=null)
         {
            session.removeAttribute(oid);
            oidTable.remove(object);
            return object;
         }
         else
            return null;
      }
      else
         return null;
   }

   /**
   * Removes the object associated with the specified oid from the session cache
   * and returns the object.
   *
   * @param session -- HttpsSession object which contains the session cache for
   *    this client's session.
   * @param oid -- The oid which is assigned to the object that is to be removed
   *    from the session cache.
   * @return
   *    The removed object is returned.  A null is returned
   *    when no object is associated with the oid in the session cache
   *    or the session is null.
   *    Also when the oid is invalid or is a compound value, a null is returned.
   */
   public Object removeObject(HttpSession session, String oid)
   {
      Object object = null;

      if (session !=null && oid!=null && oid.startsWith("__") && oid.indexOf('.')<0)
      {
         //  oid is a base oid.
         object = session.getAttribute(oid);
         if (object != null)
         {
            Hashtable oidTable = (Hashtable)session.getAttribute("__OidTable");
            session.removeAttribute(oid);
            oidTable.remove(object);
         }
      }
      return object;
   }

   /**
   * The service method is invoked by the server at runtime when a concrete servlet
   * (derived from IfsHttpServlet) is requested and this method also controls the
   * HttpSession creation, oid table creation, and navigation lock check.
   * <P>
   * If no HttpSession has been created for this request, one is created.
   * When no oidTable has been saved in the session cache, one is created and saved in
   * the session cache.
   * If the navigation is not locked, the service method in the super class is invoked
   * to dispatch the request.  Otherwise, it redirects to the locked page.
   *
   * @param request -- HttpServletRequest object which contains the client's request.
   * @param response -- HttpServletResponse object which is used to send information back
   *    to the client.
   * @exception ServletException      if an exception that interfered with the servlet's
   *                                  normal operation occurred
   * @exception IOException           if an input or output exception occurred
   */
   public void service( HttpServletRequest request, HttpServletResponse response ) 
      throws ServletException, IOException
   {
      GtSLAEvent sla = new GtSLAEvent(new IfsSession(request, response));
      sla.setState(GtSLAEvent.BEGIN_PAGE);
      sla.execute();
      
      try {

         HttpSession session = request.getSession();
         if (session == null)
             throw new ServletException( "could not create HttpSession" );

         // Gather information for procVerify call.
         String clientIPAddr = request.getRemoteAddr();
         if ( debugOn ) System.out.println( "IfsHttpServlet.service request.getRemoteAddr IP Address=" + clientIPAddr );
         // PTR 2000147 - save the client address in the current session.
         session.setAttribute( IfsSystem.CLIENT_ADDR, clientIPAddr );
          
         String clientURL  = HttpUtils.getRequestURL( request ).toString();
         if ( debugOn ) System.out.println( "IfsHttpServlet.service request.getRequestURL  URL=" + clientURL );

         Cookie [] clientCookies = request.getCookies();
         Cookie ifsCookie = getCookieByName( clientCookies, IfsSystem.cwsCookieName );
         String cookieValue = "NoCookie";
         if ( ifsCookie != null ) 
             cookieValue = ifsCookie.getValue();
         if ( debugOn ) System.out.println( "IfsHttpServlet.service ifsCookie.getValue cookieValue=" + clientCookies );

         // For now, use the old style cookie value of name=value. In the future, simply pass value.
//d7386         String returnString = procVerify( clientURL, IfsSystem.cwsCookieName + "=" + cookieValue, clientIPAddr );
         String returnString = null;   //d7386
         if ( debugOn ) System.out.println( "IfsHttpServlet.service procVerify returned=" + returnString );

         if (returnString != null) {
            response.sendRedirect( returnString );
            return;
         } 

         Hashtable oidTable = (Hashtable)session.getAttribute("__OidTable");
         if (oidTable == null) session.setAttribute("__OidTable", new Hashtable());
         session.setAttribute("__HttpServletResponse", response);

         // Check for potential session locale updates when the session locale
         // is supposed to reflect the current browser page.
         if (IfsSystem.browserLocaleUsage == IfsSystem.USE_BROWSER_LANG_ALWAYS)
            checkBrowserLocale(request);

         // Check if the navigation page flow is locked.
         String lockedPage = getLockedPage(request.getSession(false));
         if (lockedPage == null)
         {
            // Navigation page flow is NOT locked.
            // If reauthorization required, then do so NOW
            // before invoking servlet.
            if (getRequestParameter(request, "reauth") != null &&
                getRequestParameter(request, "method") != null)
                  handleReauthentication(request, response);
                
            super.service(request, response);
         } else {
            // Strip request parameters from locked page.
            // That is, remove from first "?" to end of string.
            int i = lockedPage.indexOf('?');
            if (i > 0)
               lockedPage = lockedPage.substring(0, i - 1);

            // Get error page from session cache.
            String errorPage = (String)session.getAttribute("__ErrorPage");

            // Determine name of current servlet, both as a package
            // and just servlet name (T1000035).
            String thisPackageName = getClass().getName();
            String thisServletName = getClass().getName();
            i = thisServletName.lastIndexOf('.');
            if (i >= 0)
               thisServletName = thisServletName.substring(i + 1);

            if (errorPage != null &&
                (thisPackageName.equals(errorPage) ||
                 thisServletName.equals(errorPage)))
            {
               super.service(request, response);
            }
            else if (thisPackageName.equals(lockedPage) ||
                     thisServletName.equals(lockedPage))
            {
               super.service(request, response);
            }
            else
            {
               sla.setState(GtSLAEvent.END_PAGE);
               sla.execute();   
               response.sendRedirect(response.encodeRedirectURL(lockedPage));
            }
         }
         sla.setIfsSession(new IfsSession(request, response));
         sla.setState(GtSLAEvent.END_PAGE);
         sla.execute();

      }
      catch(Throwable t)
      {
         sla.setIfsSession(new IfsSession(request, response));
         sla.execute();
         throw new ServletException(t.toString());
      }
        
}

   /**
    * Given an array of cookies, searches by name for a given cookie.
    * Returns null if the cookie name does not exist or cookies is null.
    */
   public static Cookie getCookieByName( Cookie [] cookies, String name ) {
       if (cookies != null) {
           for ( int i = 0; i < cookies.length; i++ ) {
               Cookie cookie = cookies[ i ];
               if ( cookie.getName().equals( name ) ) 
                   return cookie;
           }
       } 
       return null;
   } // getCookieByName


   /**
   * Processes the GtReauthentication transaction.
   * <P>
   * Method gets "pin" and "password" request Parameters from request and sets them into
   * GtReauthentication transaction and then executes the transaction.  Any errors are 
   * reported in the IfsSystem.ErrorPage, any information is reported in the IfsSystem.InfoPage,
   * and any redo conditions are sent back to the IfsSystem.ReauthenticationPage.  Exceptions
   * are caught internally and written to the debugLog.
   *
   * @param request -- HttpServletRequest object which contains the client's request.
   * @param response -- HttpServletResponse object which is used to send information
   *    back to the client.
   */
   public void handleReauthentication(HttpServletRequest request, HttpServletResponse response) {
			boolean ret_Status;
			IfsSession sess = new IfsSession(request,response);
			debug("IfsHttpServlet.handleReauthentication()");

        try {
                GtReauthentication reauth = (GtReauthentication) createInstance(sess, "com.ibm.ifs.gold.GtReauthentication", "reauth");
                reauth.setPin((String)getRequestParameter(request, "pin"));
                reauth.setPassword((String)getRequestParameter(request, "password"));
                reauth.setValidateActionRequired(false);
                reauth.setExecutionMode(IfsTxn.EXECUTE);
                reauth.execute();
                ret_Status = handleErrors(reauth, IfsSystem.ErrorPage, IfsSystem.InfoPage, IfsSystem.ReauthenticationPage);
					//Finished with the referer object, free from HTTP Session Cache.
					sess.getHttpSession().removeAttribute("referer");

                debug("IfsHttpServlet.handleReauthentication() - method value is " + ((String) request.getParameterValues("method")[0]));
                if(isDebugEnabled()) {
                        Enumeration keys = request.getParameterNames();
                        String key = null;
                        String value = null;
                        while(keys.hasMoreElements()) {
                                key = (String)keys.nextElement();
                                value = (String)request.getParameterValues(key)[0];
                                debug("IfsHttpServlet.handleReauthentication() - Key is " + key + ", Value is " + value);
                        }
                }
        }
        catch(Exception e) {
                debug("IfsHttpServlet.handleReauthentication() - Exception caught, value is " + e.toString());
        }
   }

   /**
   * Set default values in the header of the Http response.
   * <P>
   * It sets the content type to
   * <EM>("text/html; charset=" + IfsSystem.cwsBrowserCharSet)</EM>
   * and the header to indicate that the page contains
   * dynamic data and should not be cached by the browser
   * (Pragma = No-cache, Cache-Control = no-cache, and expiration time = 0).
   *
   * @param response -- HttpServletResponse object which is used to send information
   *    back to the client.
   */
   public void setDefaultHeader(HttpServletResponse response)
   {
      response.setContentType("text/html; charset=" + IfsSystem.cwsBrowserCharSet);

      // indicate page contains dynamic data and should not be cached by browser.
      response.setHeader("Pragma","No-cache");
      response.setHeader("Cache-Control", "no-cache");
      response.setDateHeader("Expires",0);
   }

   /**
   * Sets the fields of the object defined by the parmName and the parmValue.
   * First, the setter of the designated field defined by the parmName is obtained.
   * During this process, if the parmName is hierarchical, each element of the parmName
   * (a field name or an array index) is resolved using Java reflection methods.
   * <P>
   * If parmName is a direct field name and it is not found to be a field of the object,
   * the exception is ignored and the control returns to the caller.
   * If parmName is hierarchical and any part of the element cannot be resolved,
   * an exception of NoSuchFieldException, NoSuchMethodException, or
   * ArrayIndexOutOfBoundsException is thrown.
   * <P>
   * Secondly, the parameter value is resolved to the ultimate value it represents
   * based on whether it is a regular string constant, a compound oid, or a composite
   * parameter value.
   * <P>
   * Then, the field of the object is set to the ultimate value.
   *
   * It throws NoSuchFieldException, NoSuchMethodException,
   * ArrayIndexOutOfBoundsException, or NullPointerException.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param object -- The object whose fields are to be set.
   * @param parmName -- Specifies the names of the fields to be set.
   * @param parmValue-- Specifies the value for the fields..
   */
   private void setField(IfsSession  ifsSession,
                         Object      object,
                         String      parmName,
                         String      parmValue)
   throws IfsException
   {
      StringTokenizer st = new StringTokenizer(parmName, ".");
      Class    theClass = null;
      String   fieldName  = null;
      String   fieldClassName = null;
      Object   destObject = object;
      Object   srcObject  = null;
      Object   value = null;
      Method   method = null;
      String   methodName = null;
      Class[]  zeroClassParms = new Class[]{};
      Class[]  classParms = new Class[1];
      Object[] zeroObjectParms = new Object[]{};
      Object[] objectParms = new Object[1];
      int      i;

      try
      {
         while (st.hasMoreTokens())
         {
            theClass = destObject.getClass();
            fieldName = st.nextToken();
            i = -1;
            if (st.hasMoreTokens() || parmValue.startsWith("??"))
            {
               // There is more lower-level field(s) than the current field or
               // the current field name is the last one in the compound parm name but
               // its parm value is a composite value with leading "??".  Either case
               // needs to get the object defined by the current field name.
               try
               {
                  // fieldName is actually an array index
                  i = Integer.parseInt(fieldName);
                  value = ((Object[])destObject)[i];
               }
               catch (NumberFormatException e)
               {
                  // fieldName is indeed a field name

                  methodName = String.valueOf(Character.toUpperCase(fieldName.charAt(0))) +
                               fieldName.substring(1);
                  method = theClass.getMethod("get"+ methodName, zeroClassParms);
                  value = method.invoke(destObject, zeroObjectParms);
               }

               if (value == null)
               {
                  // The object defined by the current field name is null.
                  // So create and set it.
                  if (i>=0)
                  {
                     // The current field name is an array index.
                     fieldClassName = destObject.getClass().getName();
                     fieldClassName = fieldClassName.substring(2, fieldClassName.length()-1);
                     value = Class.forName(fieldClassName).newInstance();
                     ((Object[])destObject)[i] = value;
                  }
                  else
                  {
                     // The current field name is indeed a field name.
                     classParms[0] = method.getReturnType();
                     fieldClassName = classParms[0].getName();

                     if (fieldClassName.startsWith("[L"))
                        throw new NullPointerException("An array field is not intialized in the HTTP parameter name.");

                     method = theClass.getMethod("set"+ methodName, classParms);
                     value = classParms[0].newInstance();
                     objectParms[0] = value;
                     method.invoke(destObject, objectParms);
                  }
                  if (value instanceof IfsObject)
                     ((IfsObject)value).setIfsSession(ifsSession);
               }

               destObject = value;
            }
            else
            {
               // The current field name is the last one in the compound parm name and
               // its parm value is not composite.  So get the setter for the field.
               methodName = String.valueOf(Character.toUpperCase(fieldName.charAt(0))) +
                            fieldName.substring(1);
               method = theClass.getMethod("get"+ methodName, zeroClassParms);
               classParms[0] = method.getReturnType();
               method = theClass.getMethod("set"+ methodName, classParms);
            }
         }

         if (parmValue.startsWith("??"))
         {
            setFieldsUsingCompParmValue(ifsSession, destObject, parmValue);
            return;
         }

         // parse parmValue
         if (parmValue.startsWith("__"))
            // parameterValue is an OID
            srcObject = getObject(ifsSession, parmValue);
         else
            // parameterValue is a string value
            srcObject = parmValue;

         fieldClassName = classParms[0].getName();
         if (srcObject instanceof String)
         {
            if (fieldClassName.endsWith("String"))
               value = (String)srcObject;
            else if (fieldClassName.endsWith("Integer") || fieldClassName.endsWith("int"))
               value = new Integer((String)srcObject);
            else if (fieldClassName.endsWith("Long") || fieldClassName.endsWith("long"))
               value = new Long((String)srcObject);
            else if (fieldClassName.endsWith("Character") || fieldClassName.endsWith("char"))
               value = new Character(((String)srcObject).charAt(0));
            else if (fieldClassName.endsWith("Boolean") || fieldClassName.endsWith("boolean"))
               value = new Boolean((String)srcObject);
            else if (fieldClassName.endsWith("Float") || fieldClassName.endsWith("float"))
               value = new Float((String)srcObject);
            else if (fieldClassName.endsWith("Double") || fieldClassName.endsWith("double"))
               value = new Double((String)srcObject);
            else
            {
               Method fromStringMethod =  classParms[0].getMethod("fromString", new Class[]{String.class});
               value = fromStringMethod.invoke(null, new Object[]{(String)srcObject});
               if (value instanceof IfsObject)
                  ((IfsObject)value).setIfsSession(ifsSession);
            }
            objectParms[0]= value;
            method.invoke(destObject, objectParms);
         }
         else
         {
            if (srcObject.getClass() == classParms[0])
            {
               objectParms[0]= srcObject;
               method.invoke(destObject, objectParms);
            }
            else
               throw new Exception("Illegal request parameter!");
         }
	 if(isDebugEnabled())
	 {
           debug(ifsSession, "Set One Field:  field name = " + fieldName + ";   field value = " + srcObject);
	 }
      }
      catch (Throwable e)
      {
         String message = (e.getMessage() != null) ? (e.getMessage() + ": ") : "";
         message = message + "parmName: " + parmName + "; parmValue: " + parmValue +
                   "; failed at the field name: " + fieldName;
         throw new IfsException(e, fieldName, message);
      }
   }

   /**
   * Sets the fields of the object specified by objectName based on the values of the
   * corresponding parameters in the HTTP request.
   * In order for this method to automatically match the HTTP request parameters and the
   * fields of the object in the requested page, the request parameter names and
   * values must be in either the basic or compound formats.
   * <P> <UL>
   * <LI>basic format --
   * It has a single field name or single value.  The following name=value
   * examples use the basic format.
   * <PRE>  type=CDA    nickName=roadRunner   value=account64
   * </PRE>
   * <LI>compound format --
   * It is a gold transaction or object which is followed by a
   * period and then one of the fields of the object or transaction. If the field of
   * the object or transaction is also an object, it can also be followed by a period
   * and one of its fields. Any of the fields within this compound format may
   * represent an array of the fields followed by a period and a positive integer
   * value representing the element of the array. There are no limits to the number
   * of periods and fields that can be listed within a compound format.
   * The following are examples of compound formats.
   * <PRE>
   *    object.reqInvoiceDate.date
   *    object.reqInvoiceDiscountPercent.debitCredit_text
   *    object.reqPaidInvoiceDetail.3.invoiceMemo
   * </PRE>
   * </UL>
   * Either one or both the name and value of an http request parameter can be in the
   * compound format. In addition to the compound format, an http request parameter
   * can contain a composite value.
   * <P> <UL>
   * <LI>composite value --
   * It's value begins with ?? and is followed by two or more name=value
   * pairs which are separated by the an ampersand .
   * The following example is a compund name with a composite value.
   * <PRE>
   * userProfile.accountDetail.0.account=??type=CDA&number=1234&nickname=MyAccount
   * </PRE>
   * </UL>
   * This method searches through all parameter names in the request for
   * the following 2 kinds of matches.
   * Whenever a match is found, the defined field is set in the object.
   * <P> <UL>
   * <LI>A parameter name starting with objectName.  Such a name may be a hierarchical name
   * containing one or more periods.  If it is just the objectName, the parameter value
   * must be a composite parameter value.
   * <LI>A non-hierarchical name that is a direct field of the object.
   * </UL>
   * <P>
   * The NoSuchFieldException, NoSuchMethodException,
   * or ArrayIndexOutOfBoundsException exceptions are outputted to the designated
   * media (console, file, and/or HTML page) when the field setter, or the array
   * index defined in either the parameter names or parameter values causes
   * such an exception.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param objectName -- The HttpServletRequest parameter name whose value specifies
   *    the oid of the object.
   * @param object -- The object whose fields are to be set
   *    from the HttpServletRequest parameter values.
   */
   public void setFields(IfsSession  ifsSession,
                         String      objectName,
                         Object      object)
   {
      try
      {
         HttpServletRequest request = ifsSession.getHttpServletRequest();
         HttpSession session = request.getSession(true);
         Enumeration parmNames = request.getParameterNames();
         String      parmName = null;
         String      parmValue = null;
         String      objectNameAndDot = objectName + ".";
         int         index;
         boolean     hasObjectNamePrefix;

         while (parmNames.hasMoreElements())
         {
            parmName = (String)parmNames.nextElement();
            index = parmName.indexOf('.');
            parmValue = IfsHttpServlet.getParameterValues(parmName,IfsSystem.cwsGMJCharSet,request)[0];
            hasObjectNamePrefix = parmName.equals(objectName) || parmName.startsWith(objectNameAndDot);
            if ( hasObjectNamePrefix && index>0 ||
                 (!hasObjectNamePrefix) && index <0)
            {
               // parmName is a fully qualified name starts with the objectName
               // or parmName contains no '.' and may be a field of the object

               if (hasObjectNamePrefix)
               {
                  if (index <0)
                     parmName = "";
                  else
                     parmName = parmName.substring(index+1);
               }
               try
               {
                  setField(ifsSession, object, parmName, parmValue);
               }
               // If the parmName is a single field name, capture the exception
               // since it simply means the field name is not intended for this object.
               // Otherwise rethrow the exception
               catch (IfsException e)
               {
                  if ((e.getType().equals("java.lang.NoSuchMethodException") ||
                       e.getType().equals("java.lang.ArrayIndexOutOfBoundsException")) &&
                      (hasObjectNamePrefix ||
                       (!(parmName.equals(e.getValue()))) && parmValue.startsWith("??")))
                     throw e;
               }
            }
         }
      }
      catch (Throwable e)
      {
         if (!(e instanceof IfsException))
            handleException(new IfsException(e, ""));
         else
            handleException(e);
      }
   }

   /**
   * Decomposes the sub-parameters and uses setField() method to set the field defined
   * by each sub-parameter.
   *
   * The NoSuchFieldException, NoSuchMethodException,
   * or ArrayIndexOutOfBoundsException exceptions are outputted to the designated
   * media (console, file, and/or HTML page) when the field setter, or the array
   * index defined in either the parameter names or parameter values causes
   * such an exception.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   * @param object -- The object whose fields are to be set
   *    from the HttpServletRequest parameter values.
   * @param compositeParmValue -- The string which contains the composite value
   *    from the request.
   */
   private void setFieldsUsingCompParmValue(IfsSession  ifsSession,
                                            Object      object,
                                            String      compositeParmValue) throws IfsException
   {
      String   parmName = null;
      String   parmValue = null;
      String   parms = compositeParmValue.substring(2);
      StringTokenizer st = new StringTokenizer(parms, "&");
      StringTokenizer ft;

      while (st.hasMoreTokens())
      {
         ft = new StringTokenizer(st.nextToken(), "=");
         parmName = ft.nextToken();
         parmValue = ft.nextToken();

         setField(ifsSession, object, parmName, parmValue);
      }
   }

   /**
   * Sorts the elements of an array, which consists of Gold Primitives, in
   * an ascending or descending order.
   *
   * @param target -- The array of Gold Primitives (field return type is Gold Primitive)
   *    which is to be sorted.
   * @param fieldName -- A field name, or a getter method name, or sub-level field names
   *    which are separated by a period, or
   *    sub-level getter method names which are separated by a period.
   * @param cmp -- Reference to a Gold Primitive object that implements IComparator interface.
   * @param ascendingOrder -- Sort in ascending order when true, descending order when false.
   * @exception Exception If the <code>target</code> is not an array, throws an IllegalArgumentException.
   */
   public void sort(Object target, String fieldName, IComparator cmp, boolean ascendingOrder) throws Exception
   {
      int i=0,j=0;
      int size = 0;
      try
      {
         size = Array.getLength(target);
      }
      catch (Exception e)
      {
         throw e;
      }
      int leftPtr = 0;
      int rightPtr = size - 1;
      Object tempObj,tempObjGetterValue = null;
      Object[] objGetterValue = new Object[size];
      String objMethodName = getMethodName(fieldName);
      int ind = fieldName.indexOf('.');
      int count = 0;
      String[] fieldSubstrings;
      StringTokenizer parser;
      Class[] classes = new Class[]{};
      Object[] objects = new Object[]{};

      for (i=0;i<fieldName.length();i++)
      {
         if (fieldName.charAt(i) == '.') count++;
      }
      fieldSubstrings = new String[count+1];
      parser = new StringTokenizer(fieldName,".");

      try
      {
         i=0;
         while (parser.hasMoreTokens())
         {
            fieldSubstrings[i] = parser.nextToken();
            i++;
         }
      }
      catch (Exception e)
      {
         throw e;
      }

      try
      {
         for (i = 0; i < size; i++)
         {
            tempObj = Array.get(target,i);
            if (tempObj == null) continue;
            if (ind > -1)
            {
               for (j=0;j<count;j++)
               {
                  objMethodName = getMethodName(fieldSubstrings[j]);
                  tempObj = tempObj.getClass().getMethod(objMethodName, classes).invoke(tempObj,objects);
               }
            }
            objMethodName = getMethodName(fieldSubstrings[j]);
            objGetterValue[i] = tempObj.getClass().getMethod(objMethodName, classes).invoke(tempObj,objects);
         }

         // In an O(n) operation, swap all of the null values up to the
         // top of the array.  When the loop is done, leftPtr will hold
         // the index of the first non-null element in the array, if any.
         // If there are no non-null elements in the array, leftPtr will
         // point at the final element in the array (a null), which will
         // result in no executions of the sorting loop below (so it's okay).
         for (;;)
         {
            for (int k=0;k<size+1;k++)
            {
               if (leftPtr < size)
                  if (objGetterValue[leftPtr] == null) leftPtr++;

               if (rightPtr >= 0)
                  if (objGetterValue[rightPtr] != null) rightPtr--;
            }

            if (leftPtr >= rightPtr)
            {
               break;
            }

            tempObjGetterValue = objGetterValue[leftPtr];
            objGetterValue[leftPtr] = objGetterValue[rightPtr];
            objGetterValue[rightPtr] = tempObjGetterValue;

            tempObj = Array.get(target,leftPtr);
            Array.set(target,leftPtr,Array.get(target,rightPtr));
            Array.set(target,rightPtr,tempObj);
         }

         for (i=leftPtr; i < size-1; i++)
         {
            for (j=i+1; j<size ;j++)
            {
               if ((ascendingOrder && cmp.compare(objGetterValue[i],objGetterValue[j]) > 0) ||
                   (!ascendingOrder && cmp.compare(objGetterValue[i],objGetterValue[j]) < 0))
               {
                  // Swap the values in the getterValue array.
                  tempObjGetterValue = objGetterValue[i];
                  objGetterValue[i] = objGetterValue[j];
                  objGetterValue[j] = tempObjGetterValue;

                  // Swap values in the target array.
                  tempObj = Array.get(target,i);
                  Array.set(target,i,Array.get(target,j));
                  Array.set(target,j,tempObj);
               }
            }
         }
      }
      catch (Exception e)
      {
         throw e;
      }
   }

   /**
   * Sorts the elements of an array, which consists of Java primitives,
   * in an ascending or descending order.
   *
   * @param target -- The array of Java primitives which is to be sorted.
   * @param fieldName -- A field name, or a getter method name, or sub-level field names
   *    which are separated by a period, or
   *    sub-level getter method names which are separated by a period.
   * @param ascendingOrder -- Sort in ascending order when true, descending order when false.
   * @exception Exception If the <code>target</code> is not an array, throws an IllegalArgumentException.
   */
   public void sort(Object target, String fieldName, boolean ascendingOrder) throws Exception
   {
      double xd=0.0d;
      double yd=0.0d;
      int xi=0;
      int yi=0;
      boolean xb=false;
      boolean yb=false;
      int i=0,j=0;
      int size = 0;
      try
      {
         size = Array.getLength(target);
      }
      catch (Exception e)
      {
         throw e;
      }
      Object[] objGetterValue = new Object[size];
      String objMethodName = getMethodName(fieldName);
      Object tempObj = null;
      Object o1 = null,o2 = null;
      String cn1 = null,cn2 = null;
      int ind = fieldName.indexOf('.');
      int count = 0;
      String[] fieldSubstrings;
      StringTokenizer parser;
      Class[] classes = new Class[]{};
      Object[] objects = new Object[]{};
      String[] list = {"java.lang.Byte",
         "java.lang.Double",
         "java.lang.Float",
         "java.lang.Integer",
         "java.lang.Long",
         "java.lang.Short",
      };

      for (i=0;i<fieldName.length();i++)
      {
         if (fieldName.charAt(i) == '.') count++;
      }
      fieldSubstrings = new String[count+1];
      parser = new StringTokenizer(fieldName,".");

      try
      {
         i=0;
         while (parser.hasMoreTokens())
         {
            fieldSubstrings[i] = parser.nextToken();
            i++;
         }
      }
      catch (Exception e)
      {
         throw e;
      }

      try
      {
         for (i = 0; i < size; i++)
         {
            tempObj = Array.get(target,i);
            if (tempObj == null) continue;
            if (ind > -1)
            {
               for (j=0;j<count;j++)
               {
                  objMethodName = getMethodName(fieldSubstrings[j]);
                  tempObj = tempObj.getClass().getMethod(objMethodName, classes).invoke(tempObj,objects);
               }
            }
            objMethodName = getMethodName(fieldSubstrings[j]);
            objGetterValue[i] = tempObj.getClass().getMethod(objMethodName, classes).invoke(tempObj,objects);
         }

         for (i=0; i < size; i++)
         {
            for (j=i+1; j<size ;j++)
            {
               o1 = objGetterValue[i];
               o2 = objGetterValue[j];
               cn1 = o1.getClass().getName();
               cn2 = o2.getClass().getName();
               if (Match(cn1,list) > -1 && Match(cn2,list) > -1)
               {
                  xd = ((Number)o1).doubleValue();
                  yd = ((Number)o2).doubleValue();
               }
               else if (cn1.endsWith("Character"))
               {
                  xi = (int)(((Character)o1).charValue());
                  yi = (int)(((Character)o2).charValue());
               }
               else if (cn1.endsWith("Boolean"))
               {
                  xb = ((Boolean)o1).booleanValue();
                  yb = ((Boolean)o2).booleanValue();

                  if (xb)
                     xd = 1.0d;
                  else
                     xd = 0.0d;

                  if (yb)
                     yd = 1.0d;
                  else
                     yd =0.0d;
               }
               if ((ascendingOrder) && (xd > yd || xi > yi) ||
                   (!ascendingOrder) && (xd < yd || xi < yi))
               {
                  // Swap the values in the getterValue array.
                  objGetterValue[i] = o2;
                  objGetterValue[j] = o1;

                  // Swap values in the target array.
                  tempObj = Array.get(target,i);
                  Array.set(target,i,Array.get(target,j));
                  Array.set(target,j,tempObj);
               }
            }
         }
      }
      catch (Exception e)
      {
         throw e;
      }
   }

   /**
   * Sorts the elements of an array, which consists of String objects,
   * in an ascending or descending order using a Collator object.
   * <P>
   * This sorting method uses compareTo() with CollationKey objects for comparing
   * strings due to its faster computation than the compare() method of Collator.
   * The Collator object is used to provide local-sensitive string comparison.
   *
   * @param collator -- The Collator object used in the sorting.
   * @param target -- The array which is to be sorted.
   * @param fieldName -- A field name, or a getter method name, or sub-level field names
   *    which are separated by a period, or
   *    sub-level getter method names which are separated by a period.
   * @param ascendingOrder -- Sort in ascending order when true, descending order when false.
   * @exception Exception If the <code>target</code> is not an array, throws an IllegalArgumentException.
   */
   public void sort(Collator collator, Object target, String fieldName, boolean ascendingOrder) throws Exception
   {
      int i=0,j=0;
      int size = 0;
      try
      {
         size = Array.getLength(target);
      }
      catch (Exception e)
      {
         throw e;
      }
      int leftPtr = 0;
      int rightPtr = size - 1;
      Object[] objGetterValue = new Object[size];
      CollationKey[] strKeys = new CollationKey[size];
      CollationKey tempStrKey;
      Object tempObj;
      String objMethodName = getMethodName(fieldName);
      int ind = fieldName.indexOf('.');
      int count = 0;
      String[] fieldSubstrings;
      StringTokenizer parser;
      Class[] classes = new Class[]{};
      Object[] objects = new Object[]{};

      for (i=0;i<fieldName.length();i++)
      {
         if (fieldName.charAt(i) == '.') count++;
      }
      fieldSubstrings = new String[count+1];
      parser = new StringTokenizer(fieldName,".");

      try
      {
         i=0;
         while (parser.hasMoreTokens())
         {
            fieldSubstrings[i] = parser.nextToken();
            i++;
         }
      }
      catch (Exception e)
      {
         throw e;
      }

      try
      {
         for (i = 0; i < size; i++)
         {
            tempObj = Array.get(target,i);
            if (tempObj == null) continue;
            if (ind > -1)
            {
               for (j=0;j<count;j++)
               {
                  objMethodName = getMethodName(fieldSubstrings[j]);
                  tempObj = tempObj.getClass().getMethod(objMethodName, classes).invoke(tempObj,objects);
               }
            }
            objMethodName = getMethodName(fieldSubstrings[j]);
            objGetterValue[i] = tempObj.getClass().getMethod(objMethodName, classes).invoke(tempObj,objects);
            strKeys[i] = collator.getCollationKey((String)objGetterValue[i]);
         }

         // In an O(n) operation, swap all of the null values up to the
         // top of the array.  When the loop is done, leftPtr will hold
         // the index of the first non-null element in the array, if any.
         // If there are no non-null elements in the array, leftPtr will
         // point at the final element in the array (a null), which will
         // result in no executions of the sorting loop below (so it's okay).
         for (;;)
         {
            for (int k=0;k<size+1;k++)
            {
               if (leftPtr < size)
                  if (objGetterValue[leftPtr] == null) leftPtr++;

               if (rightPtr >= 0)
                  if (objGetterValue[rightPtr] != null) rightPtr--;
            }

            if (leftPtr >= rightPtr)
            {
               break;
            }

            tempStrKey = strKeys[leftPtr];
            strKeys[leftPtr] = strKeys[rightPtr];
            strKeys[rightPtr] = tempStrKey;

            tempObj = Array.get(target,leftPtr);
            Array.set(target,leftPtr,Array.get(target,rightPtr));
            Array.set(target,rightPtr,tempObj);
         }

         for (i=leftPtr; i < size-1; i++)
         {
            for (j=i+1; j<size ;j++)
            {
               if ((ascendingOrder) && (strKeys[i].compareTo(strKeys[j]) > 0) ||
                   (!ascendingOrder) && (strKeys[i].compareTo(strKeys[j]) < 0))
               {
                  // Swap values in the target array.
                  tempObj = Array.get(target,i);
                  Array.set(target,i,Array.get(target,j));
                  Array.set(target,j,tempObj);

                  // Swap values in the strKeys array.
                  tempStrKey = strKeys[i];
                  strKeys[i] = strKeys[j];
                  strKeys[j] = tempStrKey;
               }
            }
         }
      }
      catch (Exception e)
      {
         throw e;
      }
   }

   /**
   * Encodes a String object into a valid unicode String object.
   * For each character in "inString", a byte is created in a byte array and then
   * the byte array is encoded into a String object via the IfsSystem.cwsGMJCharSet
   * character set.
   *
   * @param inString -- The String object which is encoded using
   *    IfsSystem.cwsGMJCharSet.
   * @return
   *    The encoded String object.
   *    A null is returned when input String object is null
   *    or when an UnsupportedEncodingException exception occurs..
   */
   public static String toUnicodeString( String inString )
   {
      return( toUnicodeString(inString, IfsSystem.cwsGMJCharSet));
   }

   /**
   * Encodes a String object into a valid unicode String object.
   * For each character in "inString", a byte is created in a byte array and then
   * the byte array is encoded into a String object via the "charSet"
   * character set.
   *
   * @param inString -- The String object which is encoded using "charSet".
   * @param charSet -- The character set which is used to encode "inString".
   * @return
   *    The encoded String object.
   *    A null is returned when either one of the input parameters is null
   *    or when an UnsupportedEncodingException exception occurs..
   */
   public static String toUnicodeString(String inString, String charSet)
   {
      String unicodeString = null;

      if (inString != null && charSet != null)
      {
         /* T1000038 - Start unicode conversion problem */
         try
         {
            unicodeString = new String(inString.getBytes("ISO-8859-1"), charSet); //d7478
         }
         catch (UnsupportedEncodingException e)
         {
            unicodeString = null;
         }
      }

      return unicodeString;
   }

   /**
   * Removes the __LockedPage value from the session cache and
   * there by unlocks the page flow navigation for the session.
   *
   * @param ifsSession -- IfsSession object which contains the information about
   *    this client's session.
   */
   public void unlockNavigation(IfsSession ifsSession)
   {
      ifsSession.getHttpSession().removeAttribute("__LockedPage");
   }

   /**
   * Removes the __LockedPage value from the session cache and
   * there by unlocks the page flow navigation for the session.
   *
   * @param session -- HttpSession object which contains the session cache for
   *    this client's session.
   */
   public void unlockNavigation(HttpSession session)
   {
      session.removeAttribute("__LockedPage");
   }

   /**
   * Sorts the elements of an array (whose types can be String, Java primitive and/or Gold primitive)
   * in an ascending or descending order. The method sorts the array first by the primary sort field which is
   * specified by the first array element in <code>fieldName</code> in the order given by the first element
   * of <code>ascendingOrder</code>. During sorting, if the primary field values are equal, the method sorts by
   * the secondary field specified by the second element of <code>fieldName</code>, sorting in the order given
   * by the second element in <code>ascendingOrder</code>. If the secondary field values are equal, then the method sorts
   * by its tertiary field specified by the third element of <code>fieldName</code>, sorting in the order given
   * by the third element in <code>ascendingOrder</code> array.
   * <P>
   * When sorting String types, the method uses compareTo() with CollationKey objects for comparing
   * strings due to its faster computation than the compare() method of Collator.
   * The method uses default Collator instance for locale-sensitive string comparison.
   * <P>
   * Although the length of <code>fieldName</code> and <code>ascendingOrder</code> arrays were expected to be 3, it could contain 
   * null values. In this situation the outcome of the sort is handled by the following table.
   * 
   * <pre>
   * 
   * +--------------------------------------------------------------------------------+
   * | case no. | primary sort |  secondary sort | tertiary sort | resulting order    |
   * |          | field name   |  field name     | field name    |                    |
   * +----------+--------------+-----------------+---------------+--------------------+
   * | 1        | NULL         | NULL            | NULL          | No Change          |
   * |----------|--------------|-----------------|---------------|--------------------|
   * | 2        | NULL         | NULL            | NON-NULL      | No Change          |
   * |----------|--------------|-----------------|---------------|--------------------|
   * | 3        | NULL         | NON-NULL        | NON-NULL      | No Change          |
   * |----------|--------------|-----------------|---------------|--------------------|
   * | 4        | NON-NULL     | NULL            | NULL          | Primary            |
   * |----------|--------------|-----------------|---------------|--------------------|
   * | 5        | NON-NULL     | NULL            | NON-NULL      | Primary            |
   * |----------|--------------|-----------------|---------------|--------------------|
   * | 6        | NON-NULL     | NON-NULL        | NULL          | Primary/Secondary  |
   * |----------|--------------|-----------------|---------------|--------------------|
   * | 7        | NON-NULL     | NON-NULL        | NON-NULL      | Primary/Secondary/ |
   * |          |              |                 |               | Tertiary           |
   * +----------+--------------+-----------------+---------------+--------------------+
   * 
   * </pre>
   * @param target -- The array which is to be sorted.
   * @param fieldName -- An array of primary, secondary and tertiary field names, or getter method names, or
   *                     sub-level field names, or sub-level getter method names. If sub-level names are provided, then each level name
   *                     should be separated by a period. The maximum length of this array is 3. If the length exceeds this value,
   *                     its contents are ignored.
   * @param ascendingOrder -- Sort in ascending order when true, descending order when false. The maximum
   *                     length of this array is 3. If the length exceeds this value, its contents are ignored.
   * @exception Exception If the <code>target</code> is not an array, throws an IllegalArgumentException.
   * <ul>
   * The following are few examples of the use of the sort method in a servlet.
   * <blockquote><pre>
   *  IfsSession ifsSession     = new IfsSession(httpServletRequest,httpServletResponse);
   *  GtUserProfile up          = new GtUserProfile(ifsSession);
   * <li>Use of field names
   *  sort(up.getAccountDetail(),new String[]{"informationDate","availableBalanceAmount","accountStatusType"},{true,false,false});
   * <li>Use of sub-level field names
   *  sort(up.getAccountDetail(),new String[]{"account.type","account.number","account.nickname"},{true,false,false});
   * <li>Use of getter method names
   *  sort(up.getAccountDetail(),new String[]{"getInformationDate","getAvailableBalanceAmount","getAccountStatusType"},{true,false,false});
   * <li>Use of sub-level getter method names
   *  sort(up.getAccountDetail(),new String[]{"getAccount.getType","getAccount.getNumber","getAccount.getNickname"},{true,false,false});
   * </pre></blockquote>
   * </ul>
   */
   public void sort(Object target, String[] fieldName, boolean[] ascendingOrder) throws Exception
   {
     if(fieldName.length != 3 || ascendingOrder.length != 3) return; // No point in going further if primary, secondary and tertiary
								     // fields are not passed. It could contain null values though,
								     // in which case the outcome is described by the table described
								     // above.
      // The user might have passed invalid arguments, 
      // so handle errors described by the table described above.
      if(fieldName[0] == null) return; // handles cases 1-3
      if(fieldName[0] != null && fieldName[1] == null && fieldName[2] == null) // handles case 4
      {
	try{
	  handleSort(target,fieldName[0],ascendingOrder[0]);
	  return;
	}
	catch(Exception e){
	  IfsSystem.debug("DEBUG:SORT - Exception caught when sorting by primary field using method, explanation:"+e.toString());
	}
      }
      if(fieldName[0] != null && fieldName[1] == null && fieldName[2] != null) // handles case 5
      {
	try{
	  handleSort(target,fieldName[0],ascendingOrder[0]);
	  return;
	}
	catch(Exception e){
	  IfsSystem.debug("DEBUG:SORT - Exception caught when sorting by primary field using handleSort method, explanation:"+e.toString());
	}
      }
      if(fieldName[0] != null && fieldName[1] != null && fieldName[2] == null) // handles case 6
      {
	try{
	  handleSort(target,new String[]{fieldName[0],fieldName[1]},new boolean[]{ascendingOrder[0],ascendingOrder[1]});
	  return;
	}
	catch(Exception e){
	  IfsSystem.debug("DEBUG:SORT - Exception caught when sorting by primary/secondary field using handleSort method, explanation:"+e.toString());
	}
      }

     // handles case 7 of the above table.
     /*
      * Initialize the variables.
      */
      int i=0,j=0,k=0;
      int size = 0;
      try
      {
         size = Array.getLength(target); // get the size of the array
	 IfsSystem.debug("DEBUG:SORT - Size of the array is: "+size);
      }
      catch (Exception e)
      {
         throw e;
      }
      int leftPtr = 0;
      int rightPtr = size - 1;
      int[] ind = new int[3];
      int[] count = new int[]{0,0,0};
      Object tempObj,tempObjGetterValue = null;
      Object[][] objGetterValue = new Object[3][size];
      String[] objMethodName = new String[3];
      Collator collator = Collator.getInstance();
      String[][] fieldSubstrings = new String[3][];
      StringTokenizer[] parser = new StringTokenizer[3];
      Class[] classes = new Class[]{};
      Object[] objects = new Object[]{};
      String[] className = new String[3];
      IComparator cmp = null;
      boolean[] x= {false,false,false},y={false,false,false};
      double xd=0.0d;
      double yd=0.0d;
      int xi=0;
      int yi=0;
      boolean xb=false;
      boolean yb=false;
      Object o1 = null,o2 = null;
      String cn1 = null,cn2 = null;
      String[] list = {"java.lang.Byte",
         "java.lang.Double",
         "java.lang.Float",
         "java.lang.Integer",
         "java.lang.Long",
         "java.lang.Short",
      };
      CollationKey strKey1, strKey2;

     try
     {
     /*
      * Extract each field values.
      */
      for(i=0;i<3;i++)
      {
        objMethodName[i] = getMethodName(fieldName[i]);
        ind[i] = fieldName[i].indexOf('.');
        for (j=0;j<fieldName[i].length();j++)
        {
         if (fieldName[i].charAt(j) == '.') count[i]++;
        }
        fieldSubstrings[i] = new String[count[i]+1];
        parser[i] = new StringTokenizer(fieldName[i],".");

         j=0;
         while (parser[i].hasMoreTokens())
         {
            fieldSubstrings[i][j] = parser[i].nextToken();
            j++;
         }
         j=0;
         for (k = 0; k < size; k++)
         {
            tempObj = Array.get(target,k);
            if (tempObj == null) continue;
            if (ind[i] > -1)
            {
               for (j=0;j<count[i];j++)
               {
                  objMethodName[i] = getMethodName(fieldSubstrings[i][j]);
                  tempObj = tempObj.getClass().getMethod(objMethodName[i], classes).invoke(tempObj,objects);
               }
            }
            objMethodName[i] = getMethodName(fieldSubstrings[i][j]);
            objGetterValue[i][k] = tempObj.getClass().getMethod(objMethodName[i], classes).invoke(tempObj,objects);
         }
         className[i] = objGetterValue[i][0].getClass().getName();
      }

     /*
      * Sort the "target" array by comparing field values.
      */
      for (i=leftPtr; i < size-1; i++)
      {
         for (j=i+1; j<size ;j++)
         {
	  for(k=0;k<3;k++)
	  {
           if(className[k].endsWith("String")) // the field is a string, so use the Collator for string comparison.
           {
	     IfsSystem.debug("DEBUG:SORT - Entered String Comparation Case");
	     strKey1 = collator.getCollationKey((String)objGetterValue[k][i]);
	     strKey2 = collator.getCollationKey((String)objGetterValue[k][j]);
	     x[k] = ((ascendingOrder[k] && strKey1.compareTo(strKey2) > 0)||(!ascendingOrder[k] && strKey1.compareTo(strKey2) < 0));
             y[k] = (strKey1.compareTo(strKey2) == 0); // string's are equal
           }
           else if(className[k].startsWith("com.ibm.ifs.gold.Gs")) // the field is a gold primitive, so use its compare() method.
           {
	     IfsSystem.debug("DEBUG:SORT - Entered Gold Primitives Comparation Case");
	     cmp = (IComparator)objGetterValue[k][i];
	     x[k] = ((ascendingOrder[k] && cmp.compare(objGetterValue[k][i],objGetterValue[k][j]) > 0) ||
	             (!ascendingOrder[k] && cmp.compare(objGetterValue[k][i],objGetterValue[k][j]) < 0));
             y[k] = (cmp.compare(objGetterValue[k][i],objGetterValue[k][j]) == 0);
           }
           else // the field is a java primitive
           {
	     IfsSystem.debug("DEBUG:SORT - Entered Java Primitives Comparation Case");
               o1 = objGetterValue[k][i];
               o2 = objGetterValue[k][j];
               cn1 = o1.getClass().getName();
               cn2 = o2.getClass().getName();
               if (Match(cn1,list) > -1 && Match(cn2,list) > -1)
               {
                  xd = ((Number)o1).doubleValue();
                  yd = ((Number)o2).doubleValue();
		  x[k] = ((ascendingOrder[k] && (xd > yd)) ||(!ascendingOrder[k] && (xd < yd)));
                  y[k] = (xd == yd); 
		  if(x[k] == false)
		  {
		    if(xd < yd) 
		       y[k] = false;
		  }
               }
               else if (cn1.endsWith("Character"))
               {
                  xi = (int)(((Character)o1).charValue());
                  yi = (int)(((Character)o2).charValue());
		  x[k] = ((ascendingOrder[k] && (xi > yi)) ||(!ascendingOrder[k] && (xi < yi)));
                  y[k] = (xd == yd); 
		  if(x[k] == false)
		  {
		    if(xi < yi) 
		       y[k] = false;
		  }
               }
               else if (cn1.endsWith("Boolean"))
               {
                  xb = ((Boolean)o1).booleanValue();
                  yb = ((Boolean)o2).booleanValue();

                  if (xb)
                     xd = 1.0d;
                  else
                     xd = 0.0d;

                  if (yb)
                     yd = 1.0d;
                  else
                     yd =0.0d;

		  x[k] = ((ascendingOrder[k] && (xd > yd)) ||(!ascendingOrder[k] && (xd < yd)));
                  y[k] = (xd == yd); 
		  if(x[k] == false)
		  {
		    if(xd < yd) 
		       y[k] = false;
		  }
               }
           }
	   xd=0.0d;yd=0.0d;xi=0;yi=0; // reset the variables.
	  }

           if ((x[0] == true)||
	       ((y[0] == true) && (x[1] == true))||
	       ((y[0] == true) && (y[1] == true) && (x[2] == true))
	      ) 
           {
	     IfsSystem.debug("DEBUG:SORT - swapping the values"); 
	      for(k=0;k<3;k++)
	      {
                  // Swap the values in the getterValue array.
                  tempObjGetterValue = objGetterValue[k][i];
                  objGetterValue[k][i] = objGetterValue[k][j];
                  objGetterValue[k][j] = tempObjGetterValue;
	      }

               // Swap values in the target array.
               tempObj = Array.get(target,i);
               Array.set(target,i,Array.get(target,j));
               Array.set(target,j,tempObj);
           }
         }
       }
     }
     catch(Exception e)
     {
	IfsSystem.debug("DEBUG:SORT - Exception caught in the sort method, explanation:"+e.toString());
     }
     IfsSystem.debug("DEBUG:SORT - End of sort method");
   }

  /*
   * Sorts the elements of an array (whose types can be String, Java primitive and/or Gold primitive)
   * in an ascending or descending order. The method sorts the array by the field <code>fieldName</code>
   * in the order specified by the <code>ascendingOrder</code>.
   */
   private void handleSort(Object target, String fieldName, boolean ascendingOrder) throws Exception
   {
     IfsSystem.debug("DEBUG:SORT - Entered handleSort(Object,String,boolean) method");
     int i=0,j=0,k=0;
     int count = 0;
     int size = 0;
     String className = "";
     String objMethodName = getMethodName(fieldName);
     int index = fieldName.indexOf('.');
     Object tempObj = null, objGetterValue = null;
     Class[] classes = new Class[]{};
     Object[] objects = new Object[]{};

     try
     {
       size = Array.getLength(target);
     }
     catch (Exception e) {}

     for (j=0;j<fieldName.length();j++)
     {
       if (fieldName.charAt(j) == '.') count++;
     }
     String[] fieldSubstrings = new String[count+1];
     StringTokenizer parser = new StringTokenizer(fieldName,".");

     while (parser.hasMoreTokens())
     {
       fieldSubstrings[i] = parser.nextToken();
       i++;
     }
     j=0;
     for(k=0;k<size;k++)
     {
       tempObj = Array.get(target,k);
       if (tempObj == null) continue;
       if (index > -1)
       {
         for (j=0;j<count;j++)
         {
           objMethodName = getMethodName(fieldSubstrings[j]);
           tempObj = tempObj.getClass().getMethod(objMethodName, classes).invoke(tempObj,objects);
         }
       }
       objMethodName  = getMethodName(fieldSubstrings[j]);
       objGetterValue = tempObj.getClass().getMethod(objMethodName, classes).invoke(tempObj,objects);

       if( objGetterValue != null ) break;
     }
     className = objGetterValue.getClass().getName();
     if(className.endsWith("String"))
     {
       sort(Collator.getInstance(), target, fieldName, ascendingOrder);
     }
     else if(className.startsWith("com.ibm.ifs.gold.Gs"))
     {
       sort(target, fieldName, (IComparator)objGetterValue, ascendingOrder);
     }
     else
     {
       sort(target, fieldName, ascendingOrder);
     }
   }

  /*
   * Sorts the elements of an array (whose types can be String, Java primitive and/or Gold primitive)
   * in an ascending or descending order. The method sorts the array by the primary field. The method
   * sorts by secondary field if any of the primary field values are same. The order of the sort is 
   * given by <code>ascendingOrder[0] and ascendingOrder[1]</code> for primary and secondary fields
   * respectively.
   */
   private void handleSort(Object target, String[] fieldName, boolean[] ascendingOrder) throws Exception
   {
     IfsSystem.debug("DEBUG:SORT - Entered handleSort(Object,String[],boolean[]) method");
     /*
      * Initialize the variables.
      */
      int i=0,j=0,k=0;
      int size = 0;
      try
      {
         size = Array.getLength(target);
	 IfsSystem.debug("DEBUG:SORT - Size of the array is: "+size);
      }
      catch (Exception e)
      {
         throw e;
      }
      int leftPtr = 0;
      int[] ind = new int[2];
      int[] count = new int[]{0,0};
      Object tempObj,tempObjGetterValue = null;
      Object[][] objGetterValue = new Object[2][size];
      String[] objMethodName = new String[2];
      Collator collator = Collator.getInstance();
      String[][] fieldSubstrings = new String[2][];
      StringTokenizer[] parser = new StringTokenizer[2];
      Class[] classes = new Class[]{};
      Object[] objects = new Object[]{};
      String[] className = new String[2];
      IComparator cmp = null;
      boolean[] x= {false,false},y={false,false};
      double xd=0.0d;
      double yd=0.0d;
      int xi=0;
      int yi=0;
      boolean xb=false;
      boolean yb=false;
      Object o1 = null,o2 = null;
      String cn1 = null,cn2 = null;
      String[] list = {"java.lang.Byte",
         "java.lang.Double",
         "java.lang.Float",
         "java.lang.Integer",
         "java.lang.Long",
         "java.lang.Short",
      };
      CollationKey strKey1, strKey2;

     try
     {
     /*
      * Extract each field values.
      */
      for(i=0;i<2;i++)
      {
        objMethodName[i] = getMethodName(fieldName[i]);
        ind[i] = fieldName[i].indexOf('.');
        for (j=0;j<fieldName[i].length();j++)
        {
         if (fieldName[i].charAt(j) == '.') count[i]++;
        }
        fieldSubstrings[i] = new String[count[i]+1];
        parser[i] = new StringTokenizer(fieldName[i],".");

         j=0;
         while (parser[i].hasMoreTokens())
         {
            fieldSubstrings[i][j] = parser[i].nextToken();
            j++;
         }
         j=0;
         for (k = 0; k < size; k++)
         {
            tempObj = Array.get(target,k);
            if (tempObj == null) continue;
            if (ind[i] > -1)
            {
               for (j=0;j<count[i];j++)
               {
                  objMethodName[i] = getMethodName(fieldSubstrings[i][j]);
                  tempObj = tempObj.getClass().getMethod(objMethodName[i], classes).invoke(tempObj,objects);
               }
            }
            objMethodName[i] = getMethodName(fieldSubstrings[i][j]);
            objGetterValue[i][k] = tempObj.getClass().getMethod(objMethodName[i], classes).invoke(tempObj,objects);
         }
         className[i] = objGetterValue[i][0].getClass().getName();
	 IfsSystem.debug("DEBUG:SORT - className for index "+i+" ="+className[i]);
      }

     /*
      * Sort the "target" array by comparing field values.
      */
      for (i=leftPtr; i < size-1; i++)
      {
         for (j=i+1; j<size ;j++)
         {
	  for(k=0;k<2;k++)
	  {
           if(className[k].endsWith("String")) // the field is a string, so use the Collator for string comparison.
           {
	     IfsSystem.debug("DEBUG:SORT - Entered String Comparation Case");
	     strKey1 = collator.getCollationKey((String)objGetterValue[k][i]);
	     strKey2 = collator.getCollationKey((String)objGetterValue[k][j]);
	     x[k] = ((ascendingOrder[k] && strKey1.compareTo(strKey2) > 0)||(!ascendingOrder[k] && strKey1.compareTo(strKey2) < 0));
             y[k] = (strKey1.compareTo(strKey2) == 0); // string's are equal
           }
           else if(className[k].startsWith("com.ibm.ifs.gold.Gs")) // the field is a gold primitive, so use its compare() method.
           {
	     IfsSystem.debug("DEBUG:SORT - Entered Gold Primitives Comparation Case");
	     cmp = (IComparator)objGetterValue[k][i];
	     x[k] = ((ascendingOrder[k] && cmp.compare(objGetterValue[k][i],objGetterValue[k][j]) > 0) ||
	             (!ascendingOrder[k] && cmp.compare(objGetterValue[k][i],objGetterValue[k][j]) < 0));
             y[k] = (cmp.compare(objGetterValue[k][i],objGetterValue[k][j]) == 0);
           }
           else // the field is a java primitive
           {
	     IfsSystem.debug("DEBUG:SORT - Entered Java Primitives Comparation Case");
               o1 = objGetterValue[k][i];
               o2 = objGetterValue[k][j];
               cn1 = o1.getClass().getName();
               cn2 = o2.getClass().getName();
               if (Match(cn1,list) > -1 && Match(cn2,list) > -1)
               {
                  xd = ((Number)o1).doubleValue();
                  yd = ((Number)o2).doubleValue();
		  x[k] = ((ascendingOrder[k] && (xd > yd)) ||(!ascendingOrder[k] && (xd < yd)));
                  y[k] = (xd == yd); 
		  if(x[k] == false)
		  {
		    if(xd < yd) 
		       y[k] = false;
		  }
               }
               else if (cn1.endsWith("Character"))
               {
                  xi = (int)(((Character)o1).charValue());
                  yi = (int)(((Character)o2).charValue());
		  x[k] = ((ascendingOrder[k] && (xi > yi)) ||(!ascendingOrder[k] && (xi < yi)));
                  y[k] = (xd == yd); 
		  if(x[k] == false)
		  {
		    if(xi < yi) 
		       y[k] = false;
		  }
               }
               else if (cn1.endsWith("Boolean"))
               {
                  xb = ((Boolean)o1).booleanValue();
                  yb = ((Boolean)o2).booleanValue();

                  if (xb)
                     xd = 1.0d;
                  else
                     xd = 0.0d;

                  if (yb)
                     yd = 1.0d;
                  else
                     yd =0.0d;

		  x[k] = ((ascendingOrder[k] && (xd > yd)) ||(!ascendingOrder[k] && (xd < yd)));
                  y[k] = (xd == yd); 
		  if(x[k] == false)
		  {
		    if(xd < yd) 
		       y[k] = false;
		  }
               }
           }
	   xd=0.0d;yd=0.0d;xi=0;yi=0; // reset the variables.
	  }

           if ((x[0] == true)||
	       ((y[0] == true) && (x[1] == true))
	      ) 
           {
	     IfsSystem.debug("DEBUG:SORT - swapping the values"); 
	      for(k=0;k<2;k++)
	      {
                  // Swap the values in the getterValue array.
                  tempObjGetterValue = objGetterValue[k][i];
                  objGetterValue[k][i] = objGetterValue[k][j];
                  objGetterValue[k][j] = tempObjGetterValue;
	      }

               // Swap values in the target array.
               tempObj = Array.get(target,i);
               Array.set(target,i,Array.get(target,j));
               Array.set(target,j,tempObj);
           }
         }
       }
     }
     catch(Exception e)
     {
	    IfsSystem.debug("DEBUG:SORT - Exception caught when sorting by primary/secondary field, explanation:"+e.toString());
     }
     IfsSystem.debug("DEBUG:SORT - End of handleSort method");
   }

   // Native calls
   public native String procVerify( String clientURL, String clientCookie, String clientIPAddr );
} // class IfsHttpServlet

